(* ::Package:: *)

(************************************************************************)
(* This file was generated automatically by the Mathematica front end.  *)
(* It contains Initialization cells from a Notebook file, which         *)
(* typically will have the same name as this file except ending in      *)
(* ".nb" instead of ".m".                                               *)
(*                                                                      *)
(* This file is intended to be loaded into the Mathematica kernel using *)
(* the package loading commands Get or Needs.  Doing so is equivalent   *)
(* to using the Evaluate Initialization Cells menu command in the front *)
(* end.                                                                 *)
(*                                                                      *)
(* DO NOT EDIT THIS FILE.  This entire file is regenerated              *)
(* automatically each time the parent Notebook file is saved in the     *)
(* Mathematica front end.  Any changes you make to this file will be    *)
(* overwritten.                                                         *)
(************************************************************************)



(* ::Input::Initialization:: *)
BeginPackage["Rotations`Euler`",
	{	"Global`",
		"SpinDynamica`",
		"General`UserLevel`",
		(*"General`ModifyBuiltIn`",*)
		"General`General3D`",
		"General`Miscellaneous`"
	}
];


(* ::Input::Initialization:: *)
AxesToEuler::usage = 
	"AxesToEuler[system2,system1,opts] gives the Euler angles \[CapitalOmega]21={\[Alpha]21,\[Beta]21,\[Gamma]21} by which each axis of system2 may be rotated around the system1 axes in order to be brought into coincide with the corresponding axis of system1:
		e\[Micro]1=Rz1(\[Alpha]21)Ry1(\[Beta]21)Rz1(\[Gamma]21).e\[Micro]2, where \[Mu]\[Element]{x,y,z}.
The same result is generated by rotating around the system2 axes:
		e\[Micro]1=Rz2(\[Alpha]21)Ry2(\[Beta]21)Rz2(\[Gamma]21).e\[Micro]2, where \[Mu]\[Element]{x,y,z}.
The same result is also generated by using \"moving axes\", where the axes follow the Euler rotations, but in this case the angles are applied in reverse order:
		e\[Micro]1=Rz2'(\[Gamma]21)Ry2'(\[Beta]21)Rz2(\[Alpha]21).e\[Micro]2, 
where the primed axes of system1 follow the rotation.  

In a typical application, system2 is the principal axis system P of an anisotropic interaction, and system1 is a reference axis system such as a molecular frame M, or the laboratory frame L. The routine AxesToEuler[P,M] then generates the Euler angles \[CapitalOmega]PM which would be used in spherical tensor realisations of the spin Hamiltonian. 

Each of system1 and system2 may have the form {Z} or {X,Z} or {X,Y,Z}. Here Z,X and Y are 3-vectors. If only Z is specified, the two remaining axes are constructed. If X and Z is specified, the z-axis of the constructed system is equal to Z and the x-axis is in the XZ plane. If all three axes are specified, they must be a right-handed orthogonal set.
If system1 is missing, the system {ex,ey,ez} is assumed.

AxesToEuler also accepts axis definitions containing symbolic angles; in this case the symbolic result assumes that all angles are in the first quadrant (i.e. greater than 0 and less than \[Pi]/2).
";


(* ::Input::Initialization:: *)
RotateEuler::usage =
	"RotateEuler[obj,{\[Alpha],\[Beta],\[Gamma]},axes] rotates the vector according to the Euler angles {\[Alpha],\[Beta],\[Gamma]} around the specified axes.

axes may have the form {z}, {x,z} or {x,y,z} (see routine AxisSystem).

If axes are missing, the system {ex,ey,ez} is assumed.
		
For example, RotateEuler[obj,{\[Alpha],\[Beta],\[Gamma]},{vx,vy,vz}] rotates obj by the angle \[Gamma] about the axis vz, followed by a rotation by the angle \[Beta] about vy, followed by a rotation through the angle \[Alpha] about vz.

The same rotations may be performed in the order \[Alpha]->\[Beta]->\[Gamma] about the axes vz->vy'->vz'' where the axes {vx,vy',vz''} are simultaneously rotated together with the object.

If the Euler angles relating two axis systems sys1 and sys2 are deduced using {\[Alpha]12,\[Beta]12,\[Gamma]12}=AxesToEuler[sys1,sys2], then the axis system sys1 may be superimposed on the axis system sys2 using the rotation RotateEuler[sys1,{\[Alpha]12,\[Beta]12,\[Gamma]12},sys1].

The object obj may be a 3-vector, an axis system, or contain lines, polygons, and text.
";


(* ::Input::Initialization:: *)
EulerRotationMatrix::usage =
	"EulerRotationMatrix[{\[Alpha],\[Beta],\[Gamma]}] gives the 3D rotation matrix for the rotation product Rz(\[Alpha])Ry(\[Beta])Rz(\[Gamma]). See VMK p30.";
	


(* ::Input::Initialization:: *)
Begin["`Private`"];


(* ::Input::Initialization:: *)
AxesToEuler[opts___Rule]:=AxesToEuler[{ex,ey,ez},opts];


(* ::Input::Initialization:: *)
AxesToEuler[system2:{{_,_,_}..},opts___Rule]:=
	AxesToEuler[system2,{ex,ey,ez},opts];


(* ::Input::Initialization:: *)
AxesToEuler[system2:{{_,_,_}..},system1:{{_,_,_}..},opts___Rule]/;And@@(NumericQ/@Flatten[{system1,system2}]):=
Module[{ex1,ey1,ez1,ex2,ey2,ez2,Rzz},(
{ex1,ey1,ez1}=AxisSystem@@system1;
{ex2,ey2,ez2}=AxisSystem@@system2;
Rzz=ez2.ez1;
RoundAngles/@
Which[
Chop@N@(Rzz-1)==0,
	{ArcTan[ex2.ex1,-ex2.ey1],0,0},
Chop@N@(Rzz+1)==0,
	{ArcTan[-ex2.ex1,-ex2.ey1],\[Pi],0},
True,
	{ArcTan[ex2.ez1,ey2.ez1],
	ArcCos[Rzz],
	ArcTan[-ez2.ex1,ez2.ey1]
	}
]
)];


(* ::Input::Initialization:: *)
AxesToEuler[system2:{{_,_,_}..},system1:{{_,_,_}..},opts___Rule]:=
Module[{ex1,ey1,ez1,ex2,ey2,ez2,Rzz,symbols,assumptions},(
{ex1,ey1,ez1}=AxisSystem@@system1;
{ex2,ey2,ez2}=AxisSystem@@system2;
symbols=Union@Cases[Flatten[{{ex1,ey1,ez1},{ex2,ey2,ez2}}],_Symbol,Infinity];
assumptions=And@@((0<#<\[Pi]/2)&/@symbols);
Rzz=ez2.ez1;
Refine@Simplify[
Which[
TrueQ[Chop@N@(Rzz-1)==0],
	{ArcTan[ex2.ex1,-ex2.ey1],0,0},
TrueQ[Chop@N@(Rzz+1)==0],
	{ArcTan[-ex2.ex1,-ex2.ey1],\[Pi],0},
True,
	{ArcTan[ex2.ez1,ey2.ez1],
	ArcCos[Rzz],
	ArcTan[-ez2.ex1,ez2.ey1]
	}
],
assumptions
]
)];


(* ::Input::Initialization:: *)
AxesToEuler[args___]:=(Message[AxesToEuler::input];$Failed);


(* ::Input::Initialization:: *)
AxesToEuler::input=
	"input did not consist of a proper axis system specification.";


(* ::Input::Initialization:: *)
RoundAngles[ang_?NumericQ]:= N[Mod[ang+Pi,2Pi]-Pi];
RoundAngles[ang_]:= ang;


(* ::Input::Initialization:: *)
EulerRotationMatrix[{a_,b_,g_}]:=
	{{Cos[a]Cos[b]Cos[g]-Sin[a]Sin[g],
			-Cos[a]Cos[b]Sin[g]-Sin[a]Cos[g],
			Cos[a]Sin[b]},
		{Sin[a]Cos[b]Cos[g]+Cos[a]Sin[g],
			-Sin[a]Cos[b]Sin[g]+Cos[a]Cos[g],
			Sin[a]Sin[b]},
		{-Sin[b]Cos[g], Sin[b]Sin[g], Cos[b]}};


RotateEuler[vec:{Except[_List],Except[_List],Except[_List]},
	{\[Alpha]_, \[Beta]_, \[Gamma]_},opts___Rule ]:=
	EulerRotationMatrix[{\[Alpha],\[Beta],\[Gamma]}].vec


RotateEuler[vec:{Except[_List],Except[_List],Except[_List]},\[CapitalOmega]:{\[Alpha]_,\[Beta]_,\[Gamma]_},axes_,opts___Rule ]:=
	Module[{\[CapitalOmega]FE},(
		\[CapitalOmega]FE = AxesToEuler[AxisSystem@@axes];
	Fold[RotateEuler[#1,#2]&,vec,{-Reverse[\[CapitalOmega]FE],\[CapitalOmega],\[CapitalOmega]FE}]
	)]


(* ::Input::Initialization:: *)
RotateEuler[obj_List,ang:{_, _, _},opts___]:=
		Map[ 
			RotateEuler[#,ang,opts]&, 
			obj 
		] 


RotateEuler[Graphics3D[obj___],\[CapitalOmega]:{\[Alpha]_,\[Beta]_,\[Gamma]_}]:=Graphics3D@GeometricTransformation[Normal@obj,{EulerRotationMatrix[\[CapitalOmega]],{0,0,0}}]


(* ::Input::Initialization:: *)
RotateEuler[obj_,ang:{_, _, _},opts___ ]:=
	obj/; !TrueQ[ Head[obj]==List ];


(* ::Input::Initialization:: *)
End[];
EndPackage[];
