(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    988827,      23365]
NotebookOptionsPosition[    856557,      20751]
NotebookOutlinePosition[    858163,      20798]
CellTagsIndexPosition[    858024,      20792]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Events`Events`", "Title",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.4526855795508213`*^9, 3.452685580972721*^9}, 
   3.682593449496829*^9, {3.683108288909317*^9, 3.683108292274687*^9}, 
   3.691833142720273*^9},
 AspectRatioFixed->True,
 Background->RGBColor[
  1, 0.9, 0.8],ExpressionUUID->"48318cb7-511a-408b-9448-ce078472d7aa"],

Cell[CellGroupData[{

Cell["notes", "Section",
 CellChangeTimes->{{3.690546924699486*^9, 
  3.690546925452395*^9}},ExpressionUUID->"6a54542e-86b7-49cc-9af2-\
7a878fc8402d"],

Cell["\<\
The interpretation of event sequences in SpinDynamica currently consists of 2 \
stages: 
EventSpecifications converts the sequence into a standardized list of event \
specifications, checking validity, and standardizing chronology and other \
syntax forms. \
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.68329033033468*^9, 3.683290392267776*^9}, 
   3.683957680998136*^9},ExpressionUUID->"b0a9a083-7a31-4160-bfde-\
14ef45ccf5ce"],

Cell["\<\
Applying EventList to the result converts the EventSpecifications into an \
EventList object containing a list of Event objects, each of which include:
\t*\tevent timing interval
\t*\tstandardized event time interval 
\t\t\t(taking into account the periodicity, or time-translation invariance of \
the event)
\t*\tpropagator calculation procedure
\t*\tHilbert space compatibility
\t*\tevolution background
\t*\tflag showing whether the evolution background has been incorporated into \
the event

The EventList has its own specifications which depend on the specifications \
of the component events. \
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.6832904049464207`*^9, 3.683290625015061*^9}, {
   3.683291376000222*^9, 3.683291469796132*^9}, {3.683368731988813*^9, 
   3.683368742157116*^9}, {3.683369501592431*^9, 3.683369502670076*^9}, {
   3.683369555828307*^9, 3.683369715478136*^9}, {3.683369748487104*^9, 
   3.683369807830604*^9}, 3.683957681012969*^9, {3.68503353212426*^9, 
   3.6850335552268877`*^9}, {3.685033585701749*^9, 3.685033585736528*^9}, 
   3.685035810535602*^9, 
   3.689007202885425*^9},ExpressionUUID->"d28ad154-feec-4b44-86d1-\
3b399d96329a"]
}, Closed]],

Cell[CellGroupData[{

Cell["BeginPackage", "Subsubtitle",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{3.605545641519261*^9, 3.605546445799432*^9, 
  3.629385531775414*^9, 3.683109016913093*^9},
 AspectRatioFixed->True,
 Background->RGBColor[
  1, 0.9, 0.8],ExpressionUUID->"6e44c6d6-3fbc-4265-9751-fef4e485686e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"BeginPackage", "[", 
   RowBox[{"\"\<Events`Events`\>\"", ",", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<Global`\>\"", ",", "\[IndentingNewLine]", "\"\<SpinDynamica`\>\"", 
      ",", "\[IndentingNewLine]", "\"\<General`UserLevel`\>\"", ",", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{"\"\<General`ModifyBuiltIn`\>\"", ","}], "*)"}], 
      "\[IndentingNewLine]", "\"\<General`Matrices`\>\"", ",", 
      "\[IndentingNewLine]", "\"\<General`Miscellaneous`\>\"", ",", 
      "\[IndentingNewLine]", "\"\<QM`Hilbert`\>\"", ",", 
      "\[IndentingNewLine]", "\"\<QM`SpinOperators`\>\"", ",", 
      "\[IndentingNewLine]", "\"\<QM`Liouville`\>\"", ",", 
      "\[IndentingNewLine]", "\"\<SD`Generators`\>\"", ",", 
      "\[IndentingNewLine]", "\"\<Rotations`Wigner`\>\"", ",", 
      "\[IndentingNewLine]", "\"\<Units`\>\"", ",", "\[IndentingNewLine]", 
      "\"\<General`PhysicalConstants`\>\""}], "}"}]}], "]"}], ";"}]], "Input",\

 InitializationCell->True,
 CellChangeTimes->{{3.682597329576949*^9, 3.682597339617495*^9}, {
   3.682611681453899*^9, 3.682611690204586*^9}, 3.682611819113118*^9, {
   3.6826128177214537`*^9, 3.682612818096237*^9}, {3.683108298853897*^9, 
   3.6831083075309563`*^9}, {3.6831084813023567`*^9, 3.6831084822051497`*^9}, 
   3.683110284092412*^9, 3.691833148456338*^9, 3.701151373284646*^9, 
   3.745397208731578*^9},ExpressionUUID->"2bfa497a-f373-405e-a723-\
bfef431dc644"]
}, Open  ]],

Cell[CellGroupData[{

Cell["exports", "Subsubtitle",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{3.605545641519261*^9, 3.605546445799432*^9, 
  3.629385531775414*^9},
 AspectRatioFixed->
  True,ExpressionUUID->"c3e96f5f-7777-4da0-89db-ac69d48ce737"],

Cell[CellGroupData[{

Cell["EventSpecifications and associated routines", "Section",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.4526856290175543`*^9, {3.4526857703029137`*^9, 3.452685773062087*^9}, {
   3.454560053522246*^9, 3.454560058216958*^9}, {3.454560100594059*^9, 
   3.4545601017292137`*^9}, {3.454571512615376*^9, 3.454571533798867*^9}, {
   3.455118358989929*^9, 3.455118360699583*^9}, 3.455118777072443*^9, {
   3.455258350884461*^9, 3.455258351588242*^9}, {3.455259235163052*^9, 
   3.455259235841276*^9}, {3.464689355678136*^9, 3.464689360101637*^9}, {
   3.683110909408251*^9, 3.683110911917656*^9}, {3.6831120140304947`*^9, 
   3.6831120158213253`*^9}, {3.683804939896698*^9, 3.683804945091913*^9}, 
   3.6839576810318527`*^9},
 AspectRatioFixed->
  True,ExpressionUUID->"e445d6e0-5503-40e6-b249-bafeb479c090"],

Cell[CellGroupData[{

Cell["EventSpecifications", "Subsubsection",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.4526856290175543`*^9, {3.4526857703029137`*^9, 3.452685773062087*^9}, {
   3.456220934933506*^9, 3.456220936481489*^9}, {3.464257411710397*^9, 
   3.46425742625841*^9}, {3.464689352215589*^9, 3.464689353069914*^9}, 
   3.6831109017853622`*^9, 3.683957681047388*^9},
 AspectRatioFixed->
  True,ExpressionUUID->"03f89ecf-f14d-437b-b70c-d97a5299587c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "::", "usage"}], "=", 
   "\"\<EventSpecifications[events] converts events into standard form. The \
list of events has the syntax {event1,event2...} where each event may either \
have the form {HamiltonianOperator,duration}  or be a single superoperator, \
for example a RotationSuperoperator or a \
CoherenceOrderFiltrationSuperoperator.  EventSpecifications converts the list \
of events into standard form, including the imposition of right-to-left \
chronology. \nEach event may also carry its internal options. Two examples \
are as follows: \n {opI[\\\"x\\\"],1,Basis\[Rule]ZeemanBasis[]}, and \
{RotationSuperoperator[{\[Pi]/2,\\\"x\\\"},OperatorBasis\[Rule]\
CartesianProductOperatorBasis[]}. These local options are carried into the \
subsequent spin dynamical computations. \nOptions may also be imposed on \
entire blocks of events, using syntax of the form {event1,{event2,event3,c\
\[Rule]d},a\[Rule]b}. In this case the option a\[Rule]b applies to event1, \
while both a\[Rule]b and c\[Rule]d apply to event2 and event3. Such \
constructions may be nested indefinitely. \n.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683110930825906*^9, 3.683110960429503*^9}, {
  3.683957681054821*^9, 3.683957681056888*^9}, {3.68473148634813*^9, 
  3.6847319047868156`*^9}, {3.684732038249943*^9, 
  3.68473207538481*^9}},ExpressionUUID->"6aca0684-d5bc-4463-ad5a-\
84fe60cce813"]
}, Closed]],

Cell[CellGroupData[{

Cell["ValidEventSpecificationQ and StandardizedEventSpecificationQ", \
"Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{
  3.512388651596961*^9, {3.700485006583397*^9, 
   3.700485035326064*^9}},ExpressionUUID->"68cb664e-c697-42ca-a77a-\
6d47306ccebd"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ValidEventSpecificationQ", "::", "usage"}], "=", 
    "\"\<ValidEventSpecificationQ[evspec] returns True if evspec is a valid \
specification for a spin dynamical event, and False otherwise.\>\""}], ";"}], 
  " "}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.512388654924242*^9, 3.512388698890791*^9}, {
  3.683111377663907*^9, 3.683111378173328*^9}, {3.700485008690384*^9, 
  3.700485008695964*^9}, {3.700485041989764*^9, 
  3.700485048256084*^9}},ExpressionUUID->"8bd1d0fb-705e-4801-bc5f-\
9858bcdf40e8"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"StandardizedEventSpecificationQ", "::", "usage"}], "=", 
    "\"\<StandardizedEventSpecificationQ[evspec] returns True if evspec is a \
standardized specification for a spin dynamical event, and False otherwise.\>\
\""}], ";"}], " "}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.512388654924242*^9, 3.512388698890791*^9}, {
  3.683111377663907*^9, 3.683111378173328*^9}, {3.700485008690384*^9, 
  3.700485008695964*^9}, {3.700485052896515*^9, 
  3.700485055549872*^9}},ExpressionUUID->"d55af982-7217-4a53-94d5-\
92babb0c412c"]
}, Closed]],

Cell[CellGroupData[{

Cell["StandardizeEventSpecification", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{3.512388651596961*^9, 
  3.691948270787545*^9},ExpressionUUID->"1c8bc702-3c1f-4171-911c-\
6efe6a83bc80"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"StandardizeEventSpecification", "::", "usage"}], "=", 
    "\"\<StandardizeEventSpecification[evspec] converts an EventSpecification \
into standard form.\>\""}], ";"}], " "}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.512388654924242*^9, 3.512388698890791*^9}, {
  3.683111377663907*^9, 3.683111378173328*^9}, {3.6919482761811533`*^9, 
  3.691948291112012*^9}},ExpressionUUID->"1fd19918-44aa-42f5-85c6-\
7e028d914af0"]
}, Closed]],

Cell[CellGroupData[{

Cell["Event specification patterns", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.455454136981319*^9, 3.455454190582034*^9}, {
   3.455454472943882*^9, 3.455454474174431*^9}, {3.455455572039624*^9, 
   3.4554555804898577`*^9}, 3.455456514657077*^9, 3.455456988966947*^9, 
   3.455457877088749*^9, {3.455458472404565*^9, 3.45545847594317*^9}, {
   3.455634306583088*^9, 3.455634315476242*^9}, {3.456228041618371*^9, 
   3.456228065560835*^9}, {3.464332395796246*^9, 3.4643323961969957`*^9}, {
   3.4646057352918577`*^9, 3.464605771018136*^9}, {3.464605819725277*^9, 
   3.464605820219603*^9}, {3.465791712651339*^9, 3.4657917176480503`*^9}, 
   3.497572434410894*^9, {3.499557315376687*^9, 3.499557317863377*^9}, {
   3.514351750904572*^9, 3.514351753119151*^9}, 3.6850335552497473`*^9, 
   3.6850358105547953`*^9, 
   3.689007204425733*^9},ExpressionUUID->"6e74398f-ff04-426f-a898-\
091cebb410ec"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SingleTimeIndependentEventPattern", "::", "usage"}], "=", 
   "\"\<SingleTimeIndependentEventPattern is the pattern for a single \
time-independent event. It is used internally by SpinDynamica.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6831110850576897`*^9, 
  3.683111114640679*^9}},ExpressionUUID->"cacf5db1-b9ce-4850-8a7f-\
b9ffb27078b9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RepeatedTimeIndependentEventSeqPattern", "::", "usage"}], "=", 
   "\"\<RepeatedTimeIndependentEventSeqPattern is the pattern for a repeated \
sequence of time-independent events. It is used internally by SpinDynamica.\>\
\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6831110850576897`*^9, 
  3.683111150885728*^9}},ExpressionUUID->"743078f6-9f64-42c8-8ab3-\
a08ca0d05c0e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PiecewiseTimeIndependentEventSeqPattern", "::", "usage"}], "=", 
   "\"\<PiecewiseTimeIndependentEventSeqPattern is the pattern for a sequence \
of piecewise time-independent events. It is used internally by \
SpinDynamica.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6831110850576897`*^9, 3.683111114640679*^9}, {
  3.6831111628324738`*^9, 
  3.683111182428277*^9}},ExpressionUUID->"cc2f3737-acbd-4c0f-90cd-\
9683b47b64f9"]
}, Closed]],

Cell[CellGroupData[{

Cell["DivideEventSpecifications", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.455454136981319*^9, 3.455454190582034*^9}, {
   3.455454472943882*^9, 3.455454474174431*^9}, {3.455455572039624*^9, 
   3.4554555804898577`*^9}, 3.455456514657077*^9, 3.455456988966947*^9, 
   3.455457877088749*^9, {3.455458472404565*^9, 3.45545847594317*^9}, {
   3.455634306583088*^9, 3.455634315476242*^9}, {3.456228041618371*^9, 
   3.456228065560835*^9}, {3.464332395796246*^9, 3.4643323961969957`*^9}, {
   3.4646057352918577`*^9, 3.464605771018136*^9}, {3.464605819725277*^9, 
   3.464605820219603*^9}, {3.465791712651339*^9, 3.4657917176480503`*^9}, 
   3.497572434410894*^9, {3.499557315376687*^9, 3.499557317863377*^9}, {
   3.514351750904572*^9, 3.514351753119151*^9}, 3.683111226294403*^9, 
   3.684732307032296*^9},ExpressionUUID->"67c7b615-5fbc-45cb-9664-\
ab9879c37a4b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DivideEventSpecifications", "::", "usage"}], "=", 
   "\"\<DivideEventSpecifications[evspecs,{..,\[Tau]2,\[Tau]1}] divides up a \
list of event specifications using intervals {..,\[Tau]2,\[Tau]1} (reading \
from right to left). DivideEventSpecifications[evspecs,n] divides the list of \
event specifications into n sets of equal duration. If the option \
IncludeRepetitionNumbers\[Rule]True is used, the output contains information \
on the number of repetitions of identical event sequences.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6831110850576897`*^9, 3.683111114640679*^9}, {
   3.68311122763102*^9, 3.68311124673007*^9}, {3.683804963404469*^9, 
   3.683805081787279*^9}, {3.683806601666019*^9, 3.6838066489897327`*^9}, {
   3.684732307040989*^9, 3.684732307047571*^9}, 
   3.684946126265486*^9},ExpressionUUID->"569cb565-ba3a-4db0-98d1-\
5bbe5b16f430"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"IncludeRepetitionNumbers", "::", "usage"}], "=", 
   "\"\<IncludeRepetitionNumbers is an option for DivideEventSpecifications. \
It is used internally by SpinDynamica.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.68380665938026*^9, 3.683806679643272*^9}, 
   3.684732307053232*^9},ExpressionUUID->"541593df-e626-4dc3-a2c0-\
fcdbbab71986"]
}, Open  ]],

Cell[CellGroupData[{

Cell["FurtherEventSpecificationPatterns", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.455454136981319*^9, 3.455454190582034*^9}, {
   3.455454472943882*^9, 3.455454474174431*^9}, {3.455455572039624*^9, 
   3.4554555804898577`*^9}, 3.455456514657077*^9, 3.455456988966947*^9, 
   3.455457877088749*^9, {3.455458472404565*^9, 3.45545847594317*^9}, {
   3.455634306583088*^9, 3.455634315476242*^9}, {3.456228041618371*^9, 
   3.456228065560835*^9}, {3.464332395796246*^9, 3.4643323961969957`*^9}, {
   3.4646057352918577`*^9, 3.464605771018136*^9}, {3.464605819725277*^9, 
   3.464605820219603*^9}, {3.465791712651339*^9, 3.4657917176480503`*^9}, 
   3.497572434410894*^9, {3.499557315376687*^9, 3.499557317863377*^9}, {
   3.514351750904572*^9, 3.514351753119151*^9}, 3.683111226294403*^9, 
   3.684732307032296*^9, 3.691941920505395*^9, {3.691948514995245*^9, 
   3.691948537759405*^9}, 
   3.691948602344239*^9},ExpressionUUID->"3ea83b6a-8e4b-4d25-b6de-\
579fde0da160"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FurtherEventSpecificationPatterns", "::", "usage"}], "=", 
   "\"\<FurtherEventSpecificationPatterns is a set of patterns for valid \
event specifications, which are passed from other packages to the Events \
package. This routine is used internally by SpinDynamica.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691941911430025*^9, 3.691941955526766*^9}, {
  3.6919485163367443`*^9, 
  3.691948620685315*^9}},ExpressionUUID->"a881c8d3-a9bb-4a06-ac7a-\
b3d3592383a4"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Event and associated routines", "Section",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.4526856290175543`*^9, {3.4526857703029137`*^9, 3.452685773062087*^9}, {
   3.454560053522246*^9, 3.454560058216958*^9}, {3.454560100594059*^9, 
   3.4545601017292137`*^9}, {3.454571512615376*^9, 3.454571533798867*^9}, {
   3.455118358989929*^9, 3.455118360699583*^9}, 3.455118777072443*^9, {
   3.455258350884461*^9, 3.455258351588242*^9}, {3.455259235163052*^9, 
   3.455259235841276*^9}, {3.464689355678136*^9, 3.464689360101637*^9}, 
   3.683804948239987*^9, 3.685033555270837*^9, 3.68503581057368*^9, 
   3.689007176453301*^9},
 AspectRatioFixed->
  True,ExpressionUUID->"7fbc7ad7-d9e9-441d-b4ea-569f15f739e4"],

Cell[CellGroupData[{

Cell["Event", "Subsubsection",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.4526856290175543`*^9, {3.4526857703029137`*^9, 3.452685773062087*^9}, {
   3.456220934933506*^9, 3.456220936481489*^9}, {3.464257411710397*^9, 
   3.46425742625841*^9}, {3.464689352215589*^9, 3.464689353069914*^9}, 
   3.6839584924128647`*^9, 3.6850335696595163`*^9},
 AspectRatioFixed->
  True,ExpressionUUID->"10ab5f22-2a2e-4d36-87d6-3be4871f8707"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "::", "usage"}], "=", 
   "\"\<Event is the internal format for spin-dynamical events. It is used \
internally by SpinDynamica.\>\""}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.452685645562502*^9, 3.45268596259144*^9}, {
   3.452686007045341*^9, 3.452686024085301*^9}, {3.45268620091995*^9, 
   3.452686229304071*^9}, {3.4526890536560707`*^9, 3.4526890683727083`*^9}, {
   3.45269039577855*^9, 3.4526904110664663`*^9}, {3.4527118279826117`*^9, 
   3.452712227130121*^9}, {3.452712277449806*^9, 3.452712280755077*^9}, {
   3.452712380593004*^9, 3.452712381700644*^9}, {3.452712440922771*^9, 
   3.452712454114936*^9}, {3.4527126208999443`*^9, 3.452712629385104*^9}, {
   3.452712715419836*^9, 3.452712759967291*^9}, {3.452712897486701*^9, 
   3.452712995788553*^9}, {3.452714346495099*^9, 3.452714552284289*^9}, {
   3.452791964028174*^9, 3.452792398306984*^9}, 3.4527924459630947`*^9, {
   3.452792644226605*^9, 3.4527926849281473`*^9}, {3.4527927209681797`*^9, 
   3.452792741061995*^9}, {3.452792783081061*^9, 3.45279281940061*^9}, {
   3.4527928670506573`*^9, 3.452792894220006*^9}, {3.45279292667695*^9, 
   3.452793088020947*^9}, {3.45279312594724*^9, 3.4527931360714693`*^9}, {
   3.454560061350251*^9, 3.4545602516118813`*^9}, {3.454571523067834*^9, 
   3.454571580274415*^9}, {3.454571615796256*^9, 3.454571731313817*^9}, {
   3.454571797247858*^9, 3.454571869797944*^9}, {3.454578908211237*^9, 
   3.454578924543745*^9}, {3.454587299797113*^9, 3.454587443427005*^9}, {
   3.455118364222556*^9, 3.45511865827855*^9}, {3.455118689418354*^9, 
   3.4551187064071817`*^9}, {3.4551187874827213`*^9, 
   3.4551188019210043`*^9}, {3.455258355837818*^9, 3.455258370966402*^9}, {
   3.455259238906035*^9, 3.4552593753760843`*^9}, 3.455268612922326*^9, {
   3.455346721466736*^9, 3.4553467219827414`*^9}, {3.455377365093207*^9, 
   3.455377376240534*^9}, {3.6831110161649837`*^9, 3.683111044881998*^9}, {
   3.683958493806487*^9, 3.683958509794202*^9}, {3.6850335696671343`*^9, 
   3.685033569669672*^9}},ExpressionUUID->"06fbb407-9ad2-4789-b9a9-\
7e6bdc8090d6"]
}, Closed]],

Cell[CellGroupData[{

Cell["PropagatorCalculationProcedure", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683186944493711*^9, 3.683186950334387*^9}, 
   3.690555542339767*^9},ExpressionUUID->"09603298-2ea4-4233-a5ed-\
39e60a7c2e83"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PropagatorCalculationProcedure", "::", "usage"}], "=", 
   "\"\<PropagatorCalculationProcedure is an event property used internally \
by SpinDynamica.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683114017331316*^9, 3.683114046289069*^9}, {
  3.6831147280993*^9, 
  3.6831147291141863`*^9}},ExpressionUUID->"ab3e33e1-7e6e-47d9-bbf7-\
65977f03f0a1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PropagatorCalculationProceduresDetermined", "::", "usage"}], "=", 
   "\"\<PropagatorCalculationProceduresDetermined is an event property used \
internally by SpinDynamica.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683114017331316*^9, 3.683114046289069*^9}, {
  3.6831147280993*^9, 3.6831147291141863`*^9}, {3.6831174538817596`*^9, 
  3.683117454994279*^9}},ExpressionUUID->"0a42c1ad-3176-4426-bd09-\
a3d6e2a7675f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"InstantaneousTransformation", "::", "usage"}], "=", 
   "\"\<InstantaneousTransformation is a value for \
PropagatorCalculationProcedure used internally by SpinDynamica.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683114017331316*^9, 3.6831140880807943`*^9}, {
  3.683114150767873*^9, 
  3.683114151767647*^9}},ExpressionUUID->"85df9acf-d875-452a-8986-\
b4cf23b8d616"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"IntegrateEvolutionOverTime", "::", "usage"}], "=", 
   "\"\<IntegrateEvolutionOverTime is a value for \
PropagatorCalculationProcedure used internally by SpinDynamica.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683114017331316*^9, 
  3.6831140880807943`*^9}},ExpressionUUID->"a42c8a19-f580-47e3-9fb6-\
91815ca1a2fb"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RepeatPropagator", "::", "usage"}], "=", 
   "\"\<RepeatPropagator is a value for PropagatorCalculationProcedure used \
internally by SpinDynamica.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683114017331316*^9, 3.6831140880807943`*^9}, {
  3.683114150767873*^9, 3.683114151767647*^9}, {3.68318696973728*^9, 
  3.683186970936982*^9}},ExpressionUUID->"4add872f-c24a-484c-bdd6-\
80b4eccaace0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"UsePrecalculatedPropagator", "::", "usage"}], "=", 
   "\"\<UsePrecalculatedPropagator is a value for \
PropagatorCalculationProcedure used internally by SpinDynamica.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691437750993745*^9, 
  3.691437756579714*^9}},ExpressionUUID->"41ab7692-f980-44f2-b952-\
698c5b283ede"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PropagationMethod", "::", "usage"}], "=", 
   "\"\<PropagationMethod is an option for individual events. Possible \
PropagationMethods are Automatic and Adiabatic. Automatic employs standard \
numerical methods to solve the Schroedinger equation. Adiabatic on the other \
hand assumes adiabaticity along the whole event to solve the Schrodinger \
equation leading to speed improvements in most cases. The default option for \
PropagationMethod is: ProapgationMethod \[Rule] Automatic.\>\""}], 
  ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.464694550960771*^9, 3.4646946750267963`*^9}, {
  3.534398274061007*^9, 3.5343982862769833`*^9}, {3.719834367855938*^9, 
  3.7198343701362467`*^9}, {3.7198348916658974`*^9, 3.71983489771011*^9}, {
  3.7280391153480434`*^9, 3.728039246721529*^9}, {3.7280393269141645`*^9, 
  3.7280393553295193`*^9}, {3.728039413889021*^9, 3.728039447624425*^9}, {
  3.7280398611584682`*^9, 
  3.7280398931260805`*^9}},ExpressionUUID->"3d291e98-fded-4198-9a7a-\
8ee914c2f6e6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Adiabatic", "::", "usage"}], "=", 
   "\"\<Adiabatic is a possible PropagationMethod for a single event. \
SpinDynamica then assumes adiabaticity for the specified event. A set of \
correlated adiabatic eigenstates may then be calculated along the \
time-trajectory of the event, which may then be used to determine the final \
state of the system.\nUsing the Adiabatic PropagationMethod requires the \
additional input: EventSampling.\>\""}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.464694550960771*^9, 3.4646946750267963`*^9}, {
  3.534398274061007*^9, 3.5343982862769833`*^9}, {3.719834367855938*^9, 
  3.7198343701362467`*^9}, {3.7198348916658974`*^9, 3.71983489771011*^9}, {
  3.719834989309648*^9, 3.7198350220795364`*^9}, {3.7280392530255985`*^9, 
  3.728039258314784*^9}, {3.728039299690678*^9, 3.728039308786084*^9}, {
  3.728039372450774*^9, 3.7280393964808836`*^9}, {3.72803946753621*^9, 
  3.728039477391576*^9}, {3.728039703903517*^9, 
  3.7280398069437237`*^9}},ExpressionUUID->"ef655e92-7d05-442d-94b7-\
6d8905ac8acb"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSampling", "::", "usage"}], "=", 
   "\"\<EventSampling may be used to specify the time-sampling of an \
adiabatic event. EventSampling may be of the following forms:\nUsing an \
integer, N:\n- EventSampling \[Rule] {N}\nSpinDynamica will then uniformly \
distribute the overall duration of the event into N pieces.\nUsing a specific \
timegrid:\n- EventSampling \[Rule] {\!\(\*SubscriptBox[\(t\), \(1\)]\), \
\!\(\*SubscriptBox[\(t\), \(2\)]\), ..., T}\nSpinDynamica will make use of \
the specified timegrid to calculate the adiabatic eigenstates.\n\>\""}], 
  ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.464694550960771*^9, 3.4646946750267963`*^9}, {
  3.534398274061007*^9, 3.5343982862769833`*^9}, {3.719834367855938*^9, 
  3.7198343701362467`*^9}, {3.7198348916658974`*^9, 3.71983489771011*^9}, {
  3.719834989309648*^9, 3.7198350220795364`*^9}, {3.719835800477024*^9, 
  3.719835824487556*^9}, {3.728039483758486*^9, 3.7280396606706448`*^9}, {
  3.7280402814179773`*^9, 3.728040282937043*^9}, {3.728042208723099*^9, 
  3.7280422097167377`*^9}},ExpressionUUID->"7879936f-cbf6-4ff0-9b80-\
1eeb573da296"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DeleteCoherences", "::", "usage"}], "=", 
   "\"\<DeleteCoherences may be used to specify whether SpinDynamica should \
also propagate coherences during an adiabatic event or not. By default \
SpinDynamica assumes DeleteCoherences \[Rule] True. Coherences will then not \
be propagated and neglected and removed from the initial density operator. If \
DeleteCoherences \[Rule] False is chosen SpinDynamica will also propagate \
coherences. During an adiabatic event coherences will aquire an additional \
phase, \[Phi]. Generally \[Phi] may have two contributions, a dynamic phase \
\!\(\*SubscriptBox[\(\[Phi]\), \(dyn\)]\) and a geometric phase \
\!\(\*SubscriptBox[\(\[Phi]\), \(geo\)]\). SpinDynamica will attempt to \
calculate both of these contributions.\>\""}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.464694550960771*^9, 3.4646946750267963`*^9}, {
  3.534398274061007*^9, 3.5343982862769833`*^9}, {3.719834367855938*^9, 
  3.7198343701362467`*^9}, {3.7198348916658974`*^9, 3.71983489771011*^9}, {
  3.719834989309648*^9, 3.7198350220795364`*^9}, {3.719835800477024*^9, 
  3.719835824487556*^9}, {3.719850696470831*^9, 3.7198507210299826`*^9}, {
  3.7280398228882027`*^9, 3.728039853798623*^9}, {3.7280399067835784`*^9, 
  3.7280399373821354`*^9}, {3.728039976885955*^9, 3.72804012718071*^9}, {
  3.7280402078287306`*^9, 
  3.7280402170511937`*^9}},ExpressionUUID->"5d791b3a-cad8-486d-b329-\
04f00dbddc87"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["EventList and associated routines", "Section",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.4526856290175543`*^9, {3.4526857703029137`*^9, 3.452685773062087*^9}, {
   3.454560053522246*^9, 3.454560058216958*^9}, {3.454560100594059*^9, 
   3.4545601017292137`*^9}, {3.454571512615376*^9, 3.454571533798867*^9}, {
   3.455118358989929*^9, 3.455118360699583*^9}, 3.455118777072443*^9, {
   3.455258350884461*^9, 3.455258351588242*^9}, {3.455259235163052*^9, 
   3.455259235841276*^9}, {3.464689355678136*^9, 3.464689360101637*^9}, {
   3.683117648865467*^9, 3.6831176494934893`*^9}, 3.683804949071197*^9, 
   3.685033532184546*^9, 3.685033584793983*^9},
 AspectRatioFixed->
  True,ExpressionUUID->"030a298a-2990-463c-9492-f393613422e0"],

Cell[CellGroupData[{

Cell["EventList", "Subsubsection",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.4526856290175543`*^9, {3.4526857703029137`*^9, 3.452685773062087*^9}, {
   3.456220934933506*^9, 3.456220936481489*^9}, {3.464257411710397*^9, 
   3.46425742625841*^9}, {3.464689352215589*^9, 3.464689353069914*^9}, {
   3.6831112871941223`*^9, 3.6831112877204323`*^9}, 3.685033532203491*^9, 
   3.685033584806823*^9},
 AspectRatioFixed->
  True,ExpressionUUID->"bfad492b-a589-4c49-b139-395b976c9780"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "::", "usage"}], "=", 
   "\"\<EventList is the internal format for a list of time-ordered \
spin-dynamical events. It is used internally by SpinDynamica.\>\""}], 
  ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.452685645562502*^9, 3.45268596259144*^9}, {
   3.452686007045341*^9, 3.452686024085301*^9}, {3.45268620091995*^9, 
   3.452686229304071*^9}, {3.4526890536560707`*^9, 3.4526890683727083`*^9}, {
   3.45269039577855*^9, 3.4526904110664663`*^9}, {3.4527118279826117`*^9, 
   3.452712227130121*^9}, {3.452712277449806*^9, 3.452712280755077*^9}, {
   3.452712380593004*^9, 3.452712381700644*^9}, {3.452712440922771*^9, 
   3.452712454114936*^9}, {3.4527126208999443`*^9, 3.452712629385104*^9}, {
   3.452712715419836*^9, 3.452712759967291*^9}, {3.452712897486701*^9, 
   3.452712995788553*^9}, {3.452714346495099*^9, 3.452714552284289*^9}, {
   3.452791964028174*^9, 3.452792398306984*^9}, 3.4527924459630947`*^9, {
   3.452792644226605*^9, 3.4527926849281473`*^9}, {3.4527927209681797`*^9, 
   3.452792741061995*^9}, {3.452792783081061*^9, 3.45279281940061*^9}, {
   3.4527928670506573`*^9, 3.452792894220006*^9}, {3.45279292667695*^9, 
   3.452793088020947*^9}, {3.45279312594724*^9, 3.4527931360714693`*^9}, {
   3.454560061350251*^9, 3.4545602516118813`*^9}, {3.454571523067834*^9, 
   3.454571580274415*^9}, {3.454571615796256*^9, 3.454571731313817*^9}, {
   3.454571797247858*^9, 3.454571869797944*^9}, {3.454578908211237*^9, 
   3.454578924543745*^9}, {3.454587299797113*^9, 3.454587443427005*^9}, {
   3.455118364222556*^9, 3.45511865827855*^9}, {3.455118689418354*^9, 
   3.4551187064071817`*^9}, {3.4551187874827213`*^9, 
   3.4551188019210043`*^9}, {3.455258355837818*^9, 3.455258370966402*^9}, {
   3.455259238906035*^9, 3.4552593753760843`*^9}, 3.455268612922326*^9, {
   3.455346721466736*^9, 3.4553467219827414`*^9}, {3.455377365093207*^9, 
   3.455377376240534*^9}, {3.6831110161649837`*^9, 3.683111044881998*^9}, {
   3.683111290924274*^9, 3.683111302280889*^9}, {3.6831283700630713`*^9, 
   3.68312837216317*^9}, {3.685033532212092*^9, 3.6850335322146797`*^9}, {
   3.685033584811097*^9, 
   3.685033584813237*^9}},ExpressionUUID->"2bce2125-94bc-423b-8e48-\
4fa7fa59d091"]
}, Closed]],

Cell[CellGroupData[{

Cell["MixedHilbertSpaceCompatibility", "Subsubsection",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.4526856290175543`*^9, {3.4526857703029137`*^9, 3.452685773062087*^9}, {
   3.456220934933506*^9, 3.456220936481489*^9}, {3.464257411710397*^9, 
   3.46425742625841*^9}, {3.464689352215589*^9, 3.464689353069914*^9}, {
   3.6831112871941223`*^9, 3.6831112877204323`*^9}, 3.685033532203491*^9, 
   3.685033584806823*^9, 3.69055537255033*^9},
 AspectRatioFixed->
  True,ExpressionUUID->"0ef4b960-4c9d-4f1d-9b88-df93e4e56422"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MixedHilbertSpaceCompatibility", "::", "usage"}], "=", 
   "\"\<MixedHilbertSpaceCompatibility is an event property used internally \
by SpinDynamica. The HilbertSpaceCompatibility of an EventList takes the \
value MixedHilbertSpaceCompatibility if some, but not all, of the contained \
events are Hilbert-space compatible.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683114017331316*^9, 3.683114046289069*^9}, {
   3.683114103995716*^9, 3.683114105161358*^9}, {3.6831147231477623`*^9, 
   3.6831147524527817`*^9}, {3.6831176104129763`*^9, 3.683117611652811*^9}, {
   3.690555382134417*^9, 3.690555459256078*^9}, 
   3.690555524158374*^9},ExpressionUUID->"3eb9acf3-b789-44e7-a33d-\
e63cf7e6ba2d"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["timing routines", "Section",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.4526856290175543`*^9, {3.4526857703029137`*^9, 3.452685773062087*^9}, {
   3.454560053522246*^9, 3.454560058216958*^9}, {3.454560100594059*^9, 
   3.4545601017292137`*^9}, {3.454571512615376*^9, 3.454571533798867*^9}, {
   3.455118358989929*^9, 3.455118360699583*^9}, 3.455118777072443*^9, {
   3.455258350884461*^9, 3.455258351588242*^9}, {3.455259235163052*^9, 
   3.455259235841276*^9}, {3.464689355678136*^9, 3.464689360101637*^9}, 
   3.683804948239987*^9, 3.685033555270837*^9, 3.68503581057368*^9, 
   3.689007176453301*^9, {3.690555028741444*^9, 3.690555029808777*^9}, {
   3.690555304406231*^9, 3.6905553052362747`*^9}, {3.69055556803207*^9, 
   3.690555571179397*^9}},
 AspectRatioFixed->
  True,ExpressionUUID->"5d954f93-85ce-49d4-8580-0a3b289e5626"],

Cell[CellGroupData[{

Cell["EventDuration", "Subsubsection",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.4526856290175543`*^9, {3.4526857703029137`*^9, 3.452685773062087*^9}, {
   3.454560053522246*^9, 3.454560058216958*^9}, {3.454560100594059*^9, 
   3.4545601017292137`*^9}, {3.454571512615376*^9, 3.454571533798867*^9}, {
   3.455118358989929*^9, 3.455118360699583*^9}, 3.455118777072443*^9, {
   3.455258350884461*^9, 3.455258351588242*^9}, {3.455259235163052*^9, 
   3.455259235841276*^9}, {3.464689355678136*^9, 3.464689360101637*^9}, {
   3.506862449312269*^9, 3.506862458071252*^9}, {3.507986955194007*^9, 
   3.507986956344584*^9}, 3.6802814725533867`*^9, 3.711716424527893*^9},
 AspectRatioFixed->
  True,ExpressionUUID->"cf18bcb9-0e15-424e-b107-95796338b06f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventDuration", "::", "usage"}], "=", 
   "\"\<EventDuration[event] returns the time duration of an event or list of \
events.\>\""}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.456331105185563*^9, 3.456331185418571*^9}, {
  3.4628565413874063`*^9, 3.4628565486136217`*^9}, {3.507986975614111*^9, 
  3.5079870080120707`*^9}, {3.680281472562668*^9, 3.6802814725661488`*^9}, {
  3.7117164245614305`*^9, 
  3.7117164245649414`*^9}},ExpressionUUID->"af1ccd96-b7a1-4a6d-b23e-\
7a95a1a42787"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$VersionNumber", "<", "11"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Duration", "::", "usage"}], "=", 
     "\"\<The SpinDynamica routine Duration has been superseded by \
EventDuration.\>\""}]}], "\[IndentingNewLine]", "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.680285001628216*^9, 3.6802850519321404`*^9}, 
   3.711716424568948*^9},ExpressionUUID->"8e160aa2-ca5a-4c59-9908-\
4033b92207c4"]
}, Closed]],

Cell[CellGroupData[{

Cell["InitialTimePoint, FinalTimePoint", "Subsubsection",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.4526856290175543`*^9, {3.4526857703029137`*^9, 3.452685773062087*^9}, {
   3.456220934933506*^9, 3.456220936481489*^9}, {3.464257411710397*^9, 
   3.46425742625841*^9}},
 AspectRatioFixed->
  True,ExpressionUUID->"60590028-251a-4fc0-8fb4-7cf747ca95d3"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"InitialTimePoint", "::", "usage"}], "=", 
   "\"\<InitialTimePoint is an option for various SpinDynamics functions, \
specifying the time coordinate for the start of a sequence of events.\>\""}], 
  ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.4642574311602983`*^9, 
  3.46425746906033*^9}},ExpressionUUID->"934133a6-961f-40fd-b756-\
50aef022b537"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FinalTimePoint", "::", "usage"}], "=", 
   "\"\<FinalTimePoint is an option for various SpinDynamics functions, \
specifying the time coordinate for the end of a sequence of events.\>\""}], 
  ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.4642574311602983`*^9, 
  3.464257483300243*^9}},ExpressionUUID->"8f6d1f67-0038-42a9-af82-\
c436af20b6c9"]
}, Closed]],

Cell[CellGroupData[{

Cell["TotalEventTimeInterval, EventTimeIntervals", "Subsubsection",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.4526856290175543`*^9, {3.4526857703029137`*^9, 3.452685773062087*^9}, {
   3.454560053522246*^9, 3.454560058216958*^9}, {3.454560100594059*^9, 
   3.4545601017292137`*^9}, {3.454571512615376*^9, 3.454571533798867*^9}, {
   3.455118358989929*^9, 3.455118360699583*^9}, 3.455118777072443*^9, {
   3.455258350884461*^9, 3.455258351588242*^9}, {3.455259235163052*^9, 
   3.455259235841276*^9}, {3.456331097882916*^9, 3.4563311002463293`*^9}, {
   3.683127943103052*^9, 3.683127945735544*^9}, {3.683128107178812*^9, 
   3.683128107903926*^9}, 3.6831282239881773`*^9},
 AspectRatioFixed->
  True,ExpressionUUID->"125f548f-0951-467a-b22b-183cbf02b032"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TotalEventTimeInterval", "::", "usage"}], "=", 
   "\"\<TotalEventTimeInterval[event] returns the {end,start} time points of \
an Event or EventList.\>\""}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.456331105185563*^9, 3.456331185418571*^9}, 
   3.507987137580472*^9, {3.683128193623003*^9, 3.683128214071809*^9}, {
   3.683128296793333*^9, 3.68312833311755*^9}, {3.685033532223845*^9, 
   3.685033554183495*^9}, 3.685033584816407*^9, 3.685035810616865*^9, 
   3.6890071946664743`*^9},ExpressionUUID->"ef2adbc7-4afb-49a4-b3b0-\
525ff89fbfa7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventTimeIntervals", "::", "usage"}], "=", 
   "\"\<EventTimeIntervals[eventlist] returns the set of time pairs \
{{end,start}..} defining the elements of an EventList.\>\""}], ";"}]], "Input",\

 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.456331105185563*^9, 3.456331185418571*^9}, 
   3.507987137580472*^9, {3.683127955292849*^9, 3.683127986038204*^9}, {
   3.6831280280463867`*^9, 3.683128069170538*^9}, {3.6831281254136143`*^9, 
   3.6831281264707537`*^9}, {3.683128256099133*^9, 3.683128290231277*^9}, 
   3.68503353223164*^9, 
   3.685033584819521*^9},ExpressionUUID->"871dbf5c-8027-4a94-88b9-\
97477a3dea87"]
}, Closed]],

Cell[CellGroupData[{

Cell["PermittedTimeShift, StandardizedInterval", "Subsubsection",
 CellChangeTimes->{{3.683982441282588*^9, 3.683982468788306*^9}, 
   3.689662897670429*^9, {3.690555232279879*^9, 
   3.690555232741066*^9}},ExpressionUUID->"e6ac0f0c-a803-4459-8f8e-\
355106282c0a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "::", "usage"}], "=", 
   "\"\<PermittedTimeShift is an event property used internally by \
SpinDynamica. It specifies the time shift through which an event may be \
translated while still retaining the same propagator. It may have the values \
None, All, or take a numerical value. \>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683114017331316*^9, 3.683114046289069*^9}, {
  3.683114103995716*^9, 3.683114105161358*^9}, {3.6831147231477623`*^9, 
  3.6831147524527817`*^9}, {3.690555082429057*^9, 
  3.6905551353925123`*^9}},ExpressionUUID->"7939091f-ae20-4d0a-806c-\
ac0e5e5c940d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizedInterval", "::", "usage"}], "=", 
   "\"\<StandardizedInterval is an event property used internally by \
SpinDynamica.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683114017331316*^9, 3.683114046289069*^9}, {
  3.683114103995716*^9, 3.683114105161358*^9}, {3.6831147231477623`*^9, 
  3.683114724402379*^9}},ExpressionUUID->"04fdc510-047d-4968-9bff-\
0dcb87ef2865"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Repeat and options", "Section",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.4526856290175543`*^9, {3.4526857703029137`*^9, 3.452685773062087*^9}, {
   3.454560053522246*^9, 3.454560058216958*^9}, {3.454560100594059*^9, 
   3.4545601017292137`*^9}, {3.454571512615376*^9, 3.454571533798867*^9}, {
   3.455118358989929*^9, 3.455118360699583*^9}, 3.455118777072443*^9, {
   3.455258350884461*^9, 3.455258351588242*^9}, {3.455259235163052*^9, 
   3.455259235841276*^9}, {3.4556317366262007`*^9, 3.455631739792364*^9}, {
   3.455635072682274*^9, 3.455635076244506*^9}, {3.45563543338888*^9, 
   3.455635434064197*^9}, 3.459343594526519*^9, 3.464355592030745*^9, {
   3.4657938594682913`*^9, 3.465793863040501*^9}, {3.4862762746096687`*^9, 
   3.486276276798992*^9}, {3.506862563071916*^9, 3.506862565053204*^9}, 
   3.684774710035553*^9, 3.6848634359500637`*^9, 3.684946401152316*^9, 
   3.692848274696947*^9},
 AspectRatioFixed->
  True,ExpressionUUID->"fa172ec7-49b1-44c0-aed5-ac2de417cd85"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Repeat", "::", "usage"}], "=", 
   "\"\<Repeat[events,n,options] specifies the n-fold repetition of an event \
or set of events. The output of Repeat is presented in condensed form, unless \
Expand is applied.\>\""}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.452685645562502*^9, 3.45268596259144*^9}, {
   3.452686007045341*^9, 3.452686024085301*^9}, {3.45268620091995*^9, 
   3.452686229304071*^9}, {3.4526890536560707`*^9, 3.4526890683727083`*^9}, {
   3.45269039577855*^9, 3.4526904110664663`*^9}, {3.4527118279826117`*^9, 
   3.452712227130121*^9}, {3.452712277449806*^9, 3.452712280755077*^9}, {
   3.452712380593004*^9, 3.452712381700644*^9}, {3.452712440922771*^9, 
   3.452712454114936*^9}, {3.4527126208999443`*^9, 3.452712629385104*^9}, {
   3.452712715419836*^9, 3.452712759967291*^9}, {3.452712897486701*^9, 
   3.452712995788553*^9}, {3.452714346495099*^9, 3.452714552284289*^9}, {
   3.452791964028174*^9, 3.452792398306984*^9}, 3.4527924459630947`*^9, {
   3.452792644226605*^9, 3.4527926849281473`*^9}, {3.4527927209681797`*^9, 
   3.452792741061995*^9}, {3.452792783081061*^9, 3.45279281940061*^9}, {
   3.4527928670506573`*^9, 3.452792894220006*^9}, {3.45279292667695*^9, 
   3.452793088020947*^9}, {3.45279312594724*^9, 3.4527931360714693`*^9}, {
   3.454560061350251*^9, 3.4545602516118813`*^9}, {3.454571523067834*^9, 
   3.454571580274415*^9}, {3.454571615796256*^9, 3.454571731313817*^9}, {
   3.454571797247858*^9, 3.454571869797944*^9}, {3.454578908211237*^9, 
   3.454578924543745*^9}, {3.454587299797113*^9, 3.454587443427005*^9}, {
   3.455118364222556*^9, 3.45511865827855*^9}, {3.455118689418354*^9, 
   3.4551187064071817`*^9}, {3.4551187874827213`*^9, 
   3.4551188019210043`*^9}, {3.455258355837818*^9, 3.455258370966402*^9}, {
   3.455259238906035*^9, 3.4552593753760843`*^9}, 3.455268612922326*^9, {
   3.455346721466736*^9, 3.4553467219827414`*^9}, {3.455377365093207*^9, 
   3.455377376240534*^9}, {3.455631743707788*^9, 3.455631802467959*^9}, {
   3.4556354392890043`*^9, 3.455635443992669*^9}, {3.464355600866314*^9, 
   3.464355636120579*^9}, 3.4644483879205227`*^9, {3.4644486678901577`*^9, 
   3.4644486765381327`*^9}, {3.4657938823545322`*^9, 3.465793909630354*^9}, {
   3.504268443715642*^9, 3.504268550449213*^9}, {3.508338888517846*^9, 
   3.508338911117831*^9}, {3.6848635007557163`*^9, 3.6848635890770493`*^9}, {
   3.684946401158679*^9, 3.684946401164785*^9}, {3.692848276784699*^9, 
   3.692848280800576*^9}},ExpressionUUID->"24ee6785-5bca-4824-958b-\
ed322f3f878c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ExpandRepeatedEvents", "::", "usage"}], "=", 
   "\"\<ExpandRepeatedEvents\[Rule]True is an option for EventSpecifications \
which forces the expansion of any Repeat objects into a list of consecutive \
events. The option ExpandRepeatedEvents\[Rule]Automatic induces an expansion \
depending on the context of the repeated event (including the \
BackgroundGenerator). \n\>\""}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.452685645562502*^9, 3.45268596259144*^9}, {
   3.452686007045341*^9, 3.452686024085301*^9}, {3.45268620091995*^9, 
   3.452686229304071*^9}, {3.4526890536560707`*^9, 3.4526890683727083`*^9}, {
   3.45269039577855*^9, 3.4526904110664663`*^9}, {3.4527118279826117`*^9, 
   3.452712227130121*^9}, {3.452712277449806*^9, 3.452712280755077*^9}, {
   3.452712380593004*^9, 3.452712381700644*^9}, {3.452712440922771*^9, 
   3.452712454114936*^9}, {3.4527126208999443`*^9, 3.452712629385104*^9}, {
   3.452712715419836*^9, 3.452712759967291*^9}, {3.452712897486701*^9, 
   3.452712995788553*^9}, {3.452714346495099*^9, 3.452714552284289*^9}, {
   3.452791964028174*^9, 3.452792398306984*^9}, 3.4527924459630947`*^9, {
   3.452792644226605*^9, 3.4527926849281473`*^9}, {3.4527927209681797`*^9, 
   3.452792741061995*^9}, {3.452792783081061*^9, 3.45279281940061*^9}, {
   3.4527928670506573`*^9, 3.452792894220006*^9}, {3.45279292667695*^9, 
   3.452793088020947*^9}, {3.45279312594724*^9, 3.4527931360714693`*^9}, {
   3.454560061350251*^9, 3.4545602516118813`*^9}, {3.454571523067834*^9, 
   3.454571580274415*^9}, {3.454571615796256*^9, 3.454571731313817*^9}, {
   3.454571797247858*^9, 3.454571869797944*^9}, {3.454578908211237*^9, 
   3.454578924543745*^9}, {3.454587299797113*^9, 3.454587443427005*^9}, {
   3.455118364222556*^9, 3.45511865827855*^9}, {3.455118689418354*^9, 
   3.4551187064071817`*^9}, {3.4551187874827213`*^9, 
   3.4551188019210043`*^9}, {3.455258355837818*^9, 3.455258370966402*^9}, {
   3.455259238906035*^9, 3.4552593753760843`*^9}, 3.455268612922326*^9, {
   3.455346721466736*^9, 3.4553467219827414`*^9}, {3.455377365093207*^9, 
   3.455377376240534*^9}, {3.455631743707788*^9, 3.455631802467959*^9}, {
   3.4556354392890043`*^9, 3.455635443992669*^9}, {3.464355600866314*^9, 
   3.464355636120579*^9}, 3.4644483879205227`*^9, {3.4644486678901577`*^9, 
   3.4644486765381327`*^9}, {3.4657938823545322`*^9, 3.465793909630354*^9}, {
   3.504268443715642*^9, 3.504268550449213*^9}, {3.508338888517846*^9, 
   3.508338911117831*^9}, {3.68477472003666*^9, 3.6847747897300653`*^9}, {
   3.68486361147587*^9, 3.684863620595846*^9}, 3.6849464011709003`*^9, {
   3.6894104996545477`*^9, 3.689410557950017*^9}, 3.692848284878603*^9, 
   3.709374827516967*^9},ExpressionUUID->"f37de882-975d-4636-a103-\
43a7c20782ad"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RepeatEvent", "::", "usage"}], "=", 
   "\"\<RepeatEvent has been replaced by Repeat\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6849465657427998`*^9, 3.6849465784776793`*^9}, 
   3.6928482889720373`*^9},ExpressionUUID->"3e63b788-842e-4e12-a2b8-\
f86cd3a29f7f"]
}, Closed]],

Cell[CellGroupData[{

Cell["BackgroundGenerator and associated routines", "Section",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.4526856290175543`*^9, {3.4526857703029137`*^9, 3.452685773062087*^9}, {
   3.454560053522246*^9, 3.454560058216958*^9}, {3.454560100594059*^9, 
   3.4545601017292137`*^9}, {3.454571512615376*^9, 3.454571533798867*^9}, {
   3.455118358989929*^9, 3.455118360699583*^9}, 3.455118777072443*^9, {
   3.455258350884461*^9, 3.455258351588242*^9}, {3.455259235163052*^9, 
   3.455259235841276*^9}, {3.464689355678136*^9, 3.464689360101637*^9}, 
   3.683115180788289*^9, {3.683115351906549*^9, 3.68311535633784*^9}, 
   3.683115455863945*^9, {3.690555294358346*^9, 3.6905552947859583`*^9}, 
   3.70937483173384*^9},
 AspectRatioFixed->
  True,ExpressionUUID->"c6817669-6518-4a17-9cf5-eb83a84c33b0"],

Cell[CellGroupData[{

Cell["BackgroundGenerator", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683186944493711*^9, 3.683186950334387*^9}, {
   3.6831870082461443`*^9, 3.68318701531157*^9}, 3.690555289160674*^9, 
   3.709374834961645*^9},ExpressionUUID->"3dfaa3fc-98f4-4163-8335-\
363fc8db2868"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"BackgroundGenerator", "::", "usage"}], "=", 
   "\"\<BackgroundGenerator is an option for Event, NPropagate, Trajectory, \
TransformationAmplitude and other spin dynamical routines. \
BackgroundGenerator may be an operator, a superoperator, a time-dependent \
operator function, or a time-dependent superoperator function. The value of \
BackgroundGenerator is combined with the specified sequence of events to \
create the complete Liouvillian for the dynamic evolution of the system. \
Often BackgroundGenerator is used specify the internal Hamiltonian and/or \
relaxation superoperator, while the sequence of events is used to specify the \
applied pulse sequence, or similar.\>\""}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.497572057528099*^9, 3.497572304079651*^9}, {
   3.502839488141056*^9, 3.50283951865994*^9}, {3.6831152164021387`*^9, 
   3.683115249190535*^9}, 3.683115563174448*^9, 3.685033554196919*^9, 
   3.685035810623814*^9, 3.689007195589016*^9, {3.70937483599688*^9, 
   3.709374840170874*^9}},ExpressionUUID->"920b749e-5813-4523-a5de-\
295d891e8308"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EvolutionBackground", "::", "usage"}], "=", 
   "\"\<EvolutionBackground has been replaced by BackgroundGenerator.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.709372957034082*^9, 3.7093729929549017`*^9}, {
  3.7093746660953207`*^9, 
  3.7093746726518793`*^9}},ExpressionUUID->"e1578a28-487b-46ed-a2c5-\
24054f1a6555"]
}, Closed]],

Cell[CellGroupData[{

Cell["implementation of BackgroundGenerator", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683186944493711*^9, 3.683186950334387*^9}, {
   3.6831870082461443`*^9, 3.68318701531157*^9}, 
   3.7093748471547422`*^9},ExpressionUUID->"2d67b4f4-b862-49e3-b499-\
31dabf85a5fe"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"BackgroundGeneratorImplemented", "::", "usage"}], "=", 
   "\"\<BackgroundGeneratorImplemented is a property of an Event, used \
internally by SpinDynamica.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683115360507173*^9, 3.68311538370627*^9}, 
   3.6850335552950172`*^9, 3.68503581060363*^9, 3.689007191760003*^9, {
   3.7093748708609257`*^9, 
   3.709374872401969*^9}},ExpressionUUID->"79640fce-7560-48c2-aad8-\
baf35eaa025a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ImplementBackgroundGenerator", "::", "usage"}], "=", 
   "\"\<ImplementBackgroundGenerator is an instruction for Event, used \
internally by SpinDynamica.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683115360507173*^9, 3.68311538370627*^9}, {
   3.683115466409007*^9, 3.683115477518096*^9}, 3.685033555301489*^9, 
   3.685035810610338*^9, 3.689007193370409*^9, {3.70937487331359*^9, 
   3.709374874314856*^9}},ExpressionUUID->"82f47344-a58b-4e53-89ca-\
98e1566a9c31"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["computational status", "Section",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.4526856290175543`*^9, {3.4526857703029137`*^9, 3.452685773062087*^9}, {
   3.454560053522246*^9, 3.454560058216958*^9}, {3.454560100594059*^9, 
   3.4545601017292137`*^9}, {3.454571512615376*^9, 3.454571533798867*^9}, {
   3.455118358989929*^9, 3.455118360699583*^9}, 3.455118777072443*^9, {
   3.455258350884461*^9, 3.455258351588242*^9}, {3.455259235163052*^9, 
   3.455259235841276*^9}, {3.464689355678136*^9, 3.464689360101637*^9}, 
   3.683115180788289*^9, {3.683115351906549*^9, 3.68311535633784*^9}, 
   3.683115455863945*^9, {3.683963540534446*^9, 3.6839635432111053`*^9}, 
   3.683963636204051*^9, {3.683982444601425*^9, 3.683982446770685*^9}, {
   3.6890072389671383`*^9, 3.689007239772148*^9}, {3.690555179956868*^9, 
   3.690555191409527*^9}},
 AspectRatioFixed->
  True,ExpressionUUID->"8f78acff-3c5e-4b00-96c3-3b209aa9e9db"],

Cell[CellGroupData[{

Cell["UnassignedSymbols", "Subsubsection",
 CellChangeTimes->{
  3.683982441282588*^9},ExpressionUUID->"998ccb7b-858e-4ae2-ab85-\
8646f212a022"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"UnassignedSymbols", "::", "usage"}], "=", 
   "\"\<UnassignedSymbols[expr,options] detects any symbols in expr which do \
not have assigned values. The options may contain a PermittedSymbols->symbols \
directive.\>\""}], ";"}]], "Code",
 CellChangeTimes->{{3.683963546769224*^9, 3.683963639870132*^9}, {
  3.683963776207348*^9, 
  3.6839637954225597`*^9}},ExpressionUUID->"3955ab39-ed9c-4d9e-9619-\
361c1611edbd"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"PermittedSymbols", "::", "usage"}], "=", 
    "\"\<PermittedSymbols is an option for UnassignedSymbols specifying a set \
of permitted symbols that are exempted from the test.\>\""}], ";"}], 
  " "}]], "Code",
 CellChangeTimes->{{3.6839636673768797`*^9, 
  3.683963763369013*^9}},ExpressionUUID->"6b44094e-03fb-4b54-8805-\
a21e6a960e03"]
}, Closed]],

Cell[CellGroupData[{

Cell["NumericalComputationReadyQ", "Subsubsection",
 CellChangeTimes->{{3.683982441282588*^9, 
  3.683982468788306*^9}},ExpressionUUID->"61b91b07-b58e-4e9f-90a0-\
abe4509a4dc4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NumericalComputationReadyQ", "::", "usage"}], "=", 
   "\"\<NumericalComputationReadyQ[expr] determines whether events is ready \
for numerical computation.\>\""}], ";"}]], "Code",
 CellChangeTimes->{{3.683963546769224*^9, 3.683963639870132*^9}, {
  3.683963776207348*^9, 3.6839637954225597`*^9}, {3.6839824724547167`*^9, 
  3.683982503429968*^9}},ExpressionUUID->"7d7079dc-f068-4ca7-81fc-\
d909da5a1228"]
}, Closed]],

Cell[CellGroupData[{

Cell["ReportSuspectSymbols", "Subsubsection",
 CellChangeTimes->{
  3.683982441282588*^9, {3.6909726002950583`*^9, 
   3.690972604651525*^9}},ExpressionUUID->"79708f8f-f7f0-47c4-a548-\
206c75be708b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ReportSuspectSymbols", "::", "usage"}], "=", 
   "\"\<A reporting function used internally by SpinDynamica.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.69097263366296*^9, 
  3.6909726567715693`*^9}},ExpressionUUID->"6396e5b2-c2a5-476a-b1d9-\
c9f6dd213bb9"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PrecalculatedEvent", "Section",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.4526856290175543`*^9, {3.4526857703029137`*^9, 3.452685773062087*^9}, {
   3.454560053522246*^9, 3.454560058216958*^9}, {3.454560100594059*^9, 
   3.4545601017292137`*^9}, {3.454571512615376*^9, 3.454571533798867*^9}, {
   3.455118358989929*^9, 3.455118360699583*^9}, 3.455118777072443*^9, {
   3.455258350884461*^9, 3.455258351588242*^9}, {3.455259235163052*^9, 
   3.455259235841276*^9}, {3.4556317366262007`*^9, 3.455631739792364*^9}, {
   3.455635072682274*^9, 3.455635076244506*^9}, {3.45563543338888*^9, 
   3.455635434064197*^9}, 3.459343594526519*^9, 3.464355592030745*^9, {
   3.4657938594682913`*^9, 3.465793863040501*^9}, {3.4862762746096687`*^9, 
   3.486276276798992*^9}, {3.506862563071916*^9, 3.506862565053204*^9}, 
   3.684774710035553*^9, 3.6848634359500637`*^9, 3.684946401152316*^9, {
   3.691431315417358*^9, 3.691431320641889*^9}, 3.691431405654745*^9, 
   3.691434697740055*^9, 3.69143487558951*^9, 3.691436746693194*^9},
 AspectRatioFixed->
  True,ExpressionUUID->"ab364176-e77a-401a-8f1a-a06ec8d0b739"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PrecalculatedEvent", "::", "usage"}], "=", 
   "\"\<PrecalculatedEvent is an object containing a precalculated propagator \
that may be used as part of a sequence of events. It is generated by applying \
Precalculate to a single event, or list of events.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691431542971724*^9, 3.691431581659822*^9}, {
  3.69143486866503*^9, 3.691434870686027*^9}, {3.691436746710747*^9, 
  3.691436746715165*^9}, {3.719746802192575*^9, 
  3.719746857405493*^9}},ExpressionUUID->"f9073b9e-2041-4d79-84ba-\
6f4193c36546"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["start", "Subtitle",
 InitializationCell->True,
 CellChangeTimes->{
  3.454575871518672*^9, {3.455454799573387*^9, 3.455454807249361*^9}, {
   3.4639354155697937`*^9, 3.463935420440484*^9}, 3.4639358563258047`*^9, {
   3.4642544189569263`*^9, 3.464254422443118*^9}, {3.464327897371964*^9, 
   3.464327899208976*^9}, 3.464350113189783*^9, {3.5414885543485613`*^9, 
   3.541488570856545*^9}, 3.569514863691033*^9, {3.683367738777005*^9, 
   3.683367739383636*^9}},ExpressionUUID->"f6a4639d-21b9-47cf-80c2-\
6ab689b66ada"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Chronology", "=", "\"\<LeftToRight\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.545050393596538*^9, 3.5450503995208893`*^9}, {
  3.545050732985859*^9, 
  3.545050735366548*^9}},ExpressionUUID->"386fe2da-14a9-4ee0-81fc-\
8eece48e28b6"],

Cell["Begin[\"`Private`\"];", "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->
  True,ExpressionUUID->"b663f77e-9118-45ef-856b-5369c5612cdc"]
}, Closed]],

Cell[CellGroupData[{

Cell["EventSpecifications and associated routines", "Subtitle",
 InitializationCell->True,
 CellChangeTimes->{
  3.454575871518672*^9, {3.455454799573387*^9, 3.455454807249361*^9}, {
   3.4639354155697937`*^9, 3.463935420440484*^9}, 3.4639358563258047`*^9, {
   3.4642544189569263`*^9, 3.464254422443118*^9}, {3.464327897371964*^9, 
   3.464327899208976*^9}, 3.464350113189783*^9, {3.5414885543485613`*^9, 
   3.541488570856545*^9}, 3.569514863691033*^9, 
   3.6839576810887012`*^9},ExpressionUUID->"7d849820-cf44-4104-a28e-\
adcd3126e45a"],

Cell["\<\
EventSpecifications converts a raw set of event specifications to a standard \
form which may be processed by Event and EventList\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.541488400078479*^9, 3.541488423332264*^9}, {
   3.614934392418447*^9, 3.6149344246074657`*^9}, {3.6831292692691193`*^9, 
   3.683129272473508*^9}, 3.6839576811028433`*^9, {3.6850335322588587`*^9, 
   3.685033554246243*^9}, 3.6850335848348293`*^9, 3.685035810640245*^9, 
   3.689007197278126*^9},ExpressionUUID->"0e610048-9225-46f6-b7ef-\
b6331670b097"],

Cell[CellGroupData[{

Cell["EventSpecification patterns", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455454136981319*^9, 3.455454190582034*^9}, {
   3.455454472943882*^9, 3.455454474174431*^9}, {3.455455572039624*^9, 
   3.4554555804898577`*^9}, 3.455456514657077*^9, 3.455456988966947*^9, 
   3.455457877088749*^9, {3.455458472404565*^9, 3.45545847594317*^9}, {
   3.455634306583088*^9, 3.455634315476242*^9}, {3.456228041618371*^9, 
   3.456228065560835*^9}, {3.464332395796246*^9, 3.4643323961969957`*^9}, {
   3.4646057352918577`*^9, 3.464605771018136*^9}, {3.464605819725277*^9, 
   3.464605820219603*^9}, {3.465791712651339*^9, 3.4657917176480503`*^9}, 
   3.497572434410894*^9, {3.499557315376687*^9, 3.499557317863377*^9}, {
   3.514351750904572*^9, 3.514351753119151*^9}, 3.685033554264123*^9, 
   3.685035810659214*^9, 
   3.685167647427272*^9},ExpressionUUID->"63a7b40a-a51f-4805-bd10-\
b9328fae2fc0"],

Cell[CellGroupData[{

Cell["TimePointPattern,  DurationPattern, and IntervalPattern", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.683442156334942*^9, 3.683442162240394*^9}, {
  3.683443497372756*^9, 3.683443500419524*^9}, {3.683444627397482*^9, 
  3.683444634694757*^9}},ExpressionUUID->"3aaf33e7-1cd7-4380-a8df-\
7bbbd604f0b5"],

Cell["\<\
allow timing to take a numerical or symbolic value: or anything that is not a \
list and is free from operators or superoperators.\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.683442284406344*^9, 3.683442332623271*^9}, {
  3.683444639500681*^9, 
  3.683444640443445*^9}},ExpressionUUID->"c087b7ff-50f2-4778-8e9c-\
7204b11a8b90"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TimePointPattern", "=", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"_", "?", "NumericQ"}], "|", "_Symbol", "|", 
    RowBox[{"_", "?", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"MatchQ", "[", 
         RowBox[{"#", ",", 
          RowBox[{"Except", "[", "_List", "]"}]}], "]"}], "&&", 
        RowBox[{"FreeQ", "[", 
         RowBox[{"#", ",", 
          RowBox[{
           RowBox[{"_", "?", "OperatorQ"}], "|", 
           RowBox[{"_", "?", "SuperoperatorQ"}]}]}], "]"}]}], "&"}], 
      ")"}]}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683442166051342*^9, 3.683442278670116*^9}, {
   3.6834427204527617`*^9, 3.683442728109325*^9}, 3.6834435063091717`*^9, {
   3.683443734424686*^9, 3.683443735547799*^9}, {3.683443985696916*^9, 
   3.6834440498662653`*^9}, {3.6834446482987423`*^9, 
   3.6834446524703503`*^9}},ExpressionUUID->"6125cabb-d8c1-4147-b945-\
dec5292c9d2a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"DurationPattern", "=", "TimePointPattern"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683443742320965*^9, 
  3.68344375127731*^9}},ExpressionUUID->"a7787159-f280-4cd7-b1a5-\
4ad8e3dec3a4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"IntervalPattern", "=", 
   RowBox[{"DurationPattern", "|", 
    RowBox[{"{", 
     RowBox[{"TimePointPattern", ",", "TimePointPattern"}], "}"}]}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6834427369973793`*^9, 3.683442749608192*^9}, {
  3.683443508725836*^9, 3.683443511639868*^9}, {3.6834437884830437`*^9, 
  3.683443793566761*^9}},ExpressionUUID->"df8a71e7-e105-456d-a7a5-\
e03a7681a616"]
}, Closed]],

Cell[CellGroupData[{

Cell["RepeatNumberPattern", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.683442156334942*^9, 3.683442162240394*^9}, {
  3.683443497372756*^9, 3.683443500419524*^9}, {3.683443610930723*^9, 
  3.683443630988546*^9}, {3.68344466321564*^9, 
  3.683444664136242*^9}},ExpressionUUID->"d2619672-7b04-48d9-b73e-\
75ffc3560767"],

Cell["\<\
allow a repeat number to take an integer or symbolic value: or anything that \
is not a list and is free from operators or superoperators.\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.683442284406344*^9, 3.683442332623271*^9}, {
  3.683443649776587*^9, 3.683443656627434*^9}, {3.68344466719847*^9, 
  3.683444668062463*^9}},ExpressionUUID->"dceda209-3c06-4c5d-99af-\
083eda26b69f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RepeatNumberPattern", "=", 
   RowBox[{"_Integer", "|", "_Symbol", "|", 
    RowBox[{"_", "?", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"MatchQ", "[", 
         RowBox[{"#", ",", 
          RowBox[{"Except", "[", "_List", "]"}]}], "]"}], "&&", 
        RowBox[{"FreeQ", "[", 
         RowBox[{"#", ",", 
          RowBox[{
           RowBox[{"_", "?", "OperatorQ"}], "|", 
           RowBox[{"_", "?", "SuperoperatorQ"}]}]}], "]"}]}], "&"}], 
      ")"}]}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683442166051342*^9, 3.683442278670116*^9}, {
   3.6834427204527617`*^9, 3.683442728109325*^9}, 3.6834435063091717`*^9, {
   3.683443636090006*^9, 3.683443642981366*^9}, {3.683444103442369*^9, 
   3.683444131081184*^9}, {3.683444671802828*^9, 
   3.683444679571731*^9}},ExpressionUUID->"d6011324-76e4-40d6-b47e-\
a79f6f8b0d15"]
}, Closed]],

Cell[CellGroupData[{

Cell["EventSpecification Patterns", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.504259747153373*^9, 3.5042597539203157`*^9}, {
   3.683368138913183*^9, 3.683368161209241*^9}, 3.684233874810207*^9, {
   3.6842341876092176`*^9, 3.684234190526638*^9}, {3.684235941328562*^9, 
   3.6842359440943317`*^9}, 3.684501784432807*^9, 3.68503355428093*^9, 
   3.685035810678938*^9, 
   3.685167664175256*^9},ExpressionUUID->"0353233e-e4fc-4e8a-9777-\
249f5f7974c9"],

Cell["\<\
 a standardized event specification has the form {op, \[Tau], options} or \
{superop, options} or {Repeat[..], options} or {PrecalculatedEvent[..], \
options}\
\>", "Text",
 CellChangeTimes->{{3.684234194591098*^9, 3.684234238856298*^9}, {
   3.684497631991108*^9, 3.684497691727927*^9}, 3.68494640119372*^9, 
   3.691435122320981*^9, 3.691436746795484*^9, 
   3.692848291470284*^9},ExpressionUUID->"d1858c31-fd1d-4982-aa1e-\
e752fff32312"],

Cell[CellGroupData[{

Cell["StandardEventSpecificationPattern", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.504259747153373*^9, 3.5042597539203157`*^9}, {
   3.683368138913183*^9, 3.683368161209241*^9}, {3.6842338791533957`*^9, 
   3.684233911295724*^9}, {3.684234003810217*^9, 3.684234004705817*^9}, 
   3.684235977034493*^9, 
   3.684501832972299*^9},ExpressionUUID->"5ff53a0e-a196-416e-ab94-\
a89579ed3b81"],

Cell[BoxData[
 RowBox[{"StandardEventSpecificationPattern", ":=", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"_", "?", "SuperoperatorQ"}], ",", "___Rule"}], "}"}], "|", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"_", "?", "ValidGeneratorQ"}], ",", "DurationPattern", ",", 
     "___Rule"}], "}"}], "|", 
   RowBox[{"{", 
    RowBox[{"_Repeat", ",", "___Rule"}], "}"}], "|", 
   RowBox[{"{", 
    RowBox[{"_PrecalculatedEvent", ",", "___Rule"}], "}"}], "|", 
   "StandardNullEventSpecificationPattern"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.684233915568609*^9, 3.684234026641556*^9}, {
   3.684235963807577*^9, 3.684235977044784*^9}, 3.6844976203054733`*^9, {
   3.684498325178371*^9, 3.6844983475980797`*^9}, 3.684501832982543*^9, 
   3.684946401200336*^9, {3.690992765661949*^9, 3.690992786254394*^9}, 
   3.691435063860828*^9, 3.691436746811767*^9, 3.692848293123004*^9, 
   3.7093737167734203`*^9},ExpressionUUID->"aff8381b-b213-4d5b-b3a7-\
0ec547716bb4"]
}, Closed]],

Cell[CellGroupData[{

Cell["NullEventSpecifications and patterns", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.504259747153373*^9, 3.5042597539203157`*^9}, {
   3.683368138913183*^9, 3.683368161209241*^9}, {3.6842338791533957`*^9, 
   3.684233911295724*^9}, {3.684234003810217*^9, 3.684234004705817*^9}, 
   3.684235977034493*^9, {3.6845017954011583`*^9, 3.684501811082267*^9}, {
   3.6845018464661694`*^9, 3.684501847409246*^9}, 3.684502630174471*^9, {
   3.690990262738124*^9, 3.690990263529917*^9}, {3.690990720738617*^9, 
   3.690990723253902*^9}, {3.690992603194869*^9, 
   3.690992611632896*^9}},ExpressionUUID->"b781b4f1-2a9e-459f-b352-\
9656a54bf14b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NonStandardNullEventSpecificationPattern", ":=", 
   RowBox[{"None", "|", 
    RowBox[{"{", 
     RowBox[{"None", ",", "___Rule"}], "}"}], "|", 
    RowBox[{"UnitySuperoperator", "[", "___", "]"}], "|", 
    RowBox[{"{", "___Rule", "}"}], "|", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"None", "|", 
       RowBox[{"_", "?", "OperatorQ"}], "|", 
       RowBox[{"_", "?", "SuperoperatorQ"}], "|", "_Function", "|", 
       "_PeriodicFunction", "|", "_ShapeFunction"}], ",", "0", ",", 
      "___Rule"}], "}"}], "|", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"None", "|", 
       RowBox[{"_", "?", "OperatorQ"}], "|", 
       RowBox[{"_", "?", "SuperoperatorQ"}], "|", "_Function", "|", 
       "_PeriodicFunction", "|", "_ShapeFunction"}], ",", 
      RowBox[{"\[Tau]_", "?", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Chop", "@", 
            RowBox[{"N", "[", "#", "]"}]}], "\[Equal]", "0"}], ")"}], "&"}], 
        ")"}]}], ",", "___Rule"}], "}"}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.690990268362631*^9, 3.690990338773497*^9}, {
   3.6909905140496187`*^9, 3.690990521111864*^9}, {3.69099072753907*^9, 
   3.690990733464458*^9}, {3.690990817007818*^9, 3.69099081730095*^9}, {
   3.690992014266572*^9, 3.69099202667767*^9}, {3.6909920927630043`*^9, 
   3.690992144033474*^9}, 3.690992405911728*^9, 3.690992793741671*^9, {
   3.690994627702339*^9, 3.69099468554424*^9}, {3.690995481810608*^9, 
   3.690995488720625*^9}, {3.691056231616729*^9, 3.691056304276931*^9}, {
   3.691056407522126*^9, 
   3.691056438104863*^9}},ExpressionUUID->"574d6238-21ba-4567-b107-\
f2464046372d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"StandardNullEventSpecificationPattern", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"UnitySuperoperator", "[", "___Rule", "]"}], ",", "___Rule"}], 
    "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.690990268362631*^9, 3.690990338773497*^9}, {
   3.6909905140496187`*^9, 3.690990521111864*^9}, {3.69099072753907*^9, 
   3.690990733464458*^9}, {3.690990817007818*^9, 3.69099081730095*^9}, {
   3.690992014266572*^9, 3.69099202667767*^9}, {3.6909920927630043`*^9, 
   3.690992144033474*^9}, 3.690992405911728*^9, {3.6909926222559566`*^9, 
   3.6909926378316813`*^9}},ExpressionUUID->"4ef724b0-808d-4ead-8dd0-\
22cf4c6c78d1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NullEventSpecificationPattern", "=", 
   RowBox[{
   "NonStandardNullEventSpecificationPattern", "|", 
    "StandardNullEventSpecificationPattern"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6909926400557623`*^9, 
  3.6909926573199368`*^9}},ExpressionUUID->"3a3fbe7f-5351-4817-987a-\
e6ea08cddeb1"]
}, Closed]],

Cell[CellGroupData[{

Cell["NonStandardEventSpecificationPattern", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.504259747153373*^9, 3.5042597539203157`*^9}, {
   3.683368138913183*^9, 3.683368161209241*^9}, {3.6842338791533957`*^9, 
   3.684233911295724*^9}, {3.684234003810217*^9, 3.684234004705817*^9}, 
   3.684235977034493*^9, {3.6845017954011583`*^9, 3.684501811082267*^9}, {
   3.6845018464661694`*^9, 3.684501847409246*^9}, 
   3.684502630174471*^9},ExpressionUUID->"66610759-a816-4914-887b-\
9d5575fe512d"],

Cell[BoxData[
 RowBox[{"NonStandardEventSpecificationPattern", ":=", 
  RowBox[{
   RowBox[{"_", "?", "SuperoperatorQ"}], "|", "_Repeat", "|", 
   "_PrecalculatedEvent", "|", 
   "NonStandardNullEventSpecificationPattern"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.684233915568609*^9, 3.684234026641556*^9}, {
   3.684235963807577*^9, 3.684235977044784*^9}, 3.6844976203054733`*^9, {
   3.684498325178371*^9, 3.6844983475980797`*^9}, {3.684501832931096*^9, 
   3.6845018813177967`*^9}, {3.684502630145563*^9, 3.6845026405706377`*^9}, {
   3.684519283629546*^9, 3.684519284692327*^9}, 3.684946401206081*^9, {
   3.690992037272596*^9, 3.6909920430891657`*^9}, 3.6909927902217913`*^9, 
   3.691435085687386*^9, 3.691436746825136*^9, 
   3.692848294471904*^9},ExpressionUUID->"c090f14c-eb9b-4c56-9692-\
16db01f0c87f"]
}, Closed]],

Cell[CellGroupData[{

Cell["EventSpecificationPattern", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.504259747153373*^9, 3.5042597539203157`*^9}, {
   3.683368138913183*^9, 3.683368161209241*^9}, {3.6842338791533957`*^9, 
   3.684233911295724*^9}, {3.684234003810217*^9, 3.684234004705817*^9}, 
   3.684235977034493*^9, {3.6845017954011583`*^9, 3.684501811082267*^9}, {
   3.6845018464661694`*^9, 3.684501847409246*^9}, 3.684502630174471*^9, 
   3.690992700294066*^9},ExpressionUUID->"b47969f6-d612-4149-b402-\
24cd4ef087af"],

Cell[BoxData[
 RowBox[{"EventSpecificationPattern", ":=", 
  RowBox[{
  "StandardEventSpecificationPattern", "|", 
   "NonStandardEventSpecificationPattern"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.69099270436789*^9, 3.690992705244927*^9}, {
   3.690992753957408*^9, 3.690992760850636*^9}, 
   3.690992831107154*^9},ExpressionUUID->"97aebe76-d776-49c3-82e6-\
eb2dbfa5e15e"]
}, Closed]],

Cell[CellGroupData[{

Cell["special Patterns for time-independent event specifications", \
"Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.504259747153373*^9, 3.5042597539203157`*^9}, {
  3.683368138913183*^9, 3.683368161209241*^9}, {3.6842338791533957`*^9, 
  3.684233881327683*^9}, {3.7004853128064413`*^9, 
  3.7004853139164743`*^9}},ExpressionUUID->"e3db0bf6-9fda-42a2-9ecf-\
ddf5119cdd54"],

Cell["I don\[CloseCurlyQuote]t currently know when these are used", "Text",
 CellChangeTimes->{{3.684233885897521*^9, 3.684233894735424*^9}, {
  3.6909928523469152`*^9, 
  3.690992854905513*^9}},ExpressionUUID->"c07ee9d2-9cf5-4676-a1d6-\
5e2ee6b2a686"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SingleTimeIndependentEventPattern", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"NullGeneratorPattern", "|", 
       RowBox[{"_", "?", "OperatorQ"}], "|", 
       RowBox[{"_", "?", "SuperoperatorQ"}]}], ",", "DurationPattern"}], 
     "}"}], "|", 
    RowBox[{"{", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"NullGeneratorPattern", "|", 
        RowBox[{"_", "?", "OperatorQ"}], "|", 
        RowBox[{"_", "?", "SuperoperatorQ"}]}], ",", "DurationPattern"}], 
      "}"}], "}"}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.4993704202188997`*^9, 3.4993704745867662`*^9}, 
   3.4993706371955757`*^9, {3.499370720475182*^9, 3.499370721368759*^9}, {
   3.499371504443967*^9, 3.499371506024867*^9}, {3.4996255346029797`*^9, 
   3.499625550942233*^9}, {3.514351758876719*^9, 3.514351772620372*^9}, {
   3.5143519242137337`*^9, 3.514351926971146*^9}, 3.5143521037835093`*^9, {
   3.614934720396343*^9, 3.6149347240505857`*^9}, {3.683443560442552*^9, 
   3.683443569520398*^9}, {3.683443707514365*^9, 3.683443712228112*^9}, {
   3.70939968818799*^9, 
   3.709399691641872*^9}},ExpressionUUID->"eedf6f9f-2920-483f-82ec-\
882144e54fe5"],

Cell[BoxData[
 RowBox[{
  RowBox[{
  "RepeatedTimeIndependentEventSeqPattern", "=", "\[IndentingNewLine]", 
   RowBox[{"RepeatedEvent", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"EventSpecifications", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"NullGeneratorPattern", "|", 
            RowBox[{"_", "?", "OperatorQ"}], "|", 
            RowBox[{"_", "?", "SuperoperatorQ"}]}], ",", "DurationPattern"}], 
          "}"}], ".."}], "}"}], ",", "___Rule"}], "]"}], ",", 
     "\[IndentingNewLine]", "RepeatNumberPattern"}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.499371200027638*^9, 3.4993712589948673`*^9}, {
   3.514351847517358*^9, 3.514351921332333*^9}, 3.51435211036547*^9, 
   3.6149347302154093`*^9, {3.6834319823791533`*^9, 3.6834319949833107`*^9}, 
   3.6834341371516438`*^9, 3.6834435871042547`*^9, 3.68344367519368*^9, 
   3.6834437224883842`*^9, 3.683444692527287*^9, 3.683957681113488*^9, 
   3.709399689802396*^9},ExpressionUUID->"12a9ed6f-81c8-4422-97ec-\
262082987ba1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
  "PiecewiseTimeIndependentEventSeqPattern", "=", "\[IndentingNewLine]", 
   RowBox[{"SingleTimeIndependentEventPattern", "|", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
       "SingleTimeIndependentEventPattern", "|", "\[IndentingNewLine]", 
        "RepeatedTimeIndependentEventSeqPattern"}], ")"}], "..."}], "}"}]}]}],
   ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.499370838541771*^9, 3.499370902269443*^9}, {
   3.499371130206971*^9, 3.499371156836811*^9}, 3.499371275010803*^9, {
   3.499371307234784*^9, 3.499371315570369*^9}, {3.499371360068842*^9, 
   3.499371372572167*^9}, {3.5143519686387997`*^9, 3.51435200350959*^9}, {
   3.51435211473143*^9, 
   3.514352115681308*^9}},ExpressionUUID->"9bf0c0f4-27e0-4fd5-9180-\
2a020700ef14"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ValidEventSpecificationQ and StandardizedEventSpecificationQ", \
"Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455265644416114*^9, 3.455265648200944*^9}, {
   3.464256156522676*^9, 3.4642561730892*^9}, {3.700484857957287*^9, 
   3.700484861867897*^9}, 3.700485008749867*^9, {3.700485097365365*^9, 
   3.700485106726488*^9}},ExpressionUUID->"22b8ed96-6222-4b25-b829-\
a388785f63b4"],

Cell[CellGroupData[{

Cell["ValidEventSpecificationQ", "Subsection",
 CellChangeTimes->{{3.700485115371357*^9, 
  3.700485123603608*^9}},ExpressionUUID->"47748f9d-b4cd-4b50-a8d3-\
edd0c9374add"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ValidEventSpecificationQ", "[", 
    RowBox[{
     RowBox[{"evspec_", "?", "StandardizedEventSpecificationQ"}], ",", 
     "___Rule"}], "]"}], ":=", "True"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.7004851380357323`*^9, 3.7004851535516043`*^9}, 
   3.700485247535571*^9},ExpressionUUID->"409b65e6-8cee-4368-9f81-\
c4a690516fbb"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ValidEventSpecificationQ", "[", 
    RowBox[{
     RowBox[{"evspec", ":", "EventSpecificationPattern"}], ",", "___Rule"}], 
    "]"}], ":=", "True"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.700485207374167*^9, 
  3.700485249047999*^9}},ExpressionUUID->"e8344ead-015b-4fbc-9d7d-\
7b549bde50a2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ValidEventSpecificationQ", "[", 
    RowBox[{"evspec_", ",", "___Rule"}], "]"}], ":=", "False"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.700485227708897*^9, 
  3.7004852509504147`*^9}},ExpressionUUID->"5323e98c-8d56-4062-9d16-\
07af069252a8"]
}, Closed]],

Cell[CellGroupData[{

Cell["StandardizedEventSpecificationQ", "Subsection",
 CellChangeTimes->{
  3.700485115371357*^9},ExpressionUUID->"e0a1b0ed-e9b4-4681-b226-\
4a2c192b634f"],

Cell["\<\
StandardizedEventSpecificationQ permits non-numeric duration specifications, \
but traps out some obvious problems where the duration contains an operator \
or a superoperator.\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.6833928826117287`*^9, 3.683392910120216*^9}, {
   3.683442786510786*^9, 3.68344280243859*^9}, {3.684233175655485*^9, 
   3.684233192316928*^9}, 
   3.7004850088163013`*^9},ExpressionUUID->"db23e459-df29-422c-a36f-\
d581ea5520e9"],

Cell["\<\
Another valid specification is a list containing specific options for the \
event\
\>", "Text",
 CellChangeTimes->{{3.684233198013433*^9, 3.684233205956262*^9}, {
  3.684233236532921*^9, 
  3.684233244611684*^9}},ExpressionUUID->"9410e384-6f37-4b31-8fc6-\
ebfa221a0ffc"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "StandardizedEventSpecificationQ", "]"}], "=", 
   RowBox[{"{", "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.508306070915915*^9, 3.508306108654612*^9}, 
   3.5343983556865*^9, 
   3.700485008826127*^9},ExpressionUUID->"c1ca660e-662b-4a15-a5c6-\
8b8be3acffb6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizedEventSpecificationQ", "[", 
    RowBox[{"StandardEventSpecificationPattern", ",", "___Rule"}], "]"}], "=",
    "True"}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.455331184765587*^9, 3.455331198410469*^9}, {
   3.4642548112065277`*^9, 3.464254816272388*^9}, 3.464256326524571*^9, 
   3.5083072679541397`*^9, {3.684234055824237*^9, 3.684234078247473*^9}, 
   3.684234165154557*^9, 3.68423597705396*^9, 3.684501832940888*^9, 
   3.700485008836402*^9},ExpressionUUID->"82328dc2-a3f4-4177-b38e-\
a6de810c1c03"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizedEventSpecificationQ", "[", 
    RowBox[{
     RowBox[{"Except", "[", "StandardEventSpecificationPattern", "]"}], ",", 
     "___Rule"}], "]"}], "=", "False"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.684234156634842*^9, 3.684234163311685*^9}, 
   3.6842359770625153`*^9, 3.6845018329472227`*^9, 
   3.700485008843811*^9},ExpressionUUID->"682d521d-e10b-4ded-937b-\
51c5b108d8d9"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["StandardizeEventSpecification", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455265644416114*^9, 3.455265648200944*^9}, {
  3.464256156522676*^9, 3.4642561730892*^9}, {3.6837299399643993`*^9, 
  3.683729955457025*^9}},ExpressionUUID->"911fa07b-7c21-4b76-a897-\
7ce41702d784"],

Cell["\<\
converts a single non-standard event specification into standard form. Also \
incorporates options into the event specification.\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.683729968953651*^9, 3.683729979927641*^9}, {
  3.684235128035798*^9, 3.6842351289209642`*^9}, {3.684516904947171*^9, 
  3.684516923726293*^9}},ExpressionUUID->"080ce6fd-4dc8-4190-8095-\
3f4c1f0de93e"],

Cell[CellGroupData[{

Cell["continue if valid", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683730152042799*^9, 3.683730176539816*^9}, {
  3.683730487760866*^9, 3.6837304892112513`*^9}, {3.6837836190618887`*^9, 
  3.6837836309599867`*^9}, {3.6837837756319017`*^9, 3.683783790309249*^9}, {
  3.690993822759798*^9, 
  3.690993826305231*^9}},ExpressionUUID->"7b8318fd-8632-4e3a-b367-\
ed3275616988"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeEventSpecification", "[", 
    RowBox[{
     RowBox[{"ev_", "?", "StandardizedEventSpecificationQ"}], ",", 
     "___Rule"}], "]"}], ":=", "ev"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.684517068696649*^9, 3.684517098189928*^9}, {
   3.690993847428919*^9, 3.6909938493694468`*^9}, 
   3.7004850088571672`*^9},ExpressionUUID->"000fd58c-126a-4815-a71f-\
592ebe2c5715"]
}, Closed]],

Cell[CellGroupData[{

Cell["encapsulate in List if needed", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683730152042799*^9, 3.683730157092507*^9}, {
  3.684233829930523*^9, 3.68423383719345*^9}, {3.684497770457748*^9, 
  3.6844977713556967`*^9}},ExpressionUUID->"e986dd95-e9f6-4e65-9942-\
84f53d334880"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeEventSpecification", "[", 
    RowBox[{
     RowBox[{"ev", ":", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"_", "?", "SuperoperatorQ"}], "|", "_Repeat", "|", 
        "_PrecalculatedEvent"}], ")"}]}], ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"StandardizeEventSpecification", "[", 
    RowBox[{
     RowBox[{"{", "ev", "}"}], ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683729964064437*^9, 3.6837300066481647`*^9}, {
   3.6842340928251657`*^9, 3.684234107615261*^9}, 3.684235963826593*^9, 
   3.684497737577211*^9, {3.684497776970068*^9, 3.6844977955886383`*^9}, {
   3.68451719515452*^9, 3.6845172038245487`*^9}, 3.684946401236361*^9, 
   3.691435093487237*^9, 3.69143674683786*^9, 
   3.692848295364603*^9},ExpressionUUID->"ea71c5a8-455a-410c-8e7e-\
adc4e207c305"]
}, Closed]],

Cell[CellGroupData[{

Cell["incorporate any external options into the List", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683730152042799*^9, 3.683730157092507*^9}, {
  3.684516981660862*^9, 3.684516999539733*^9}, {3.6909941049949617`*^9, 
  3.690994110924879*^9}, {3.690994143983363*^9, 
  3.690994146382578*^9}},ExpressionUUID->"c58d0339-022c-4bf7-a39e-\
ba4056b6e9b1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeEventSpecification", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"sop_", "?", "SuperoperatorQ"}], ",", "evopts___Rule"}], "}"}],
      ",", "opts__Rule"}], "]"}], ":=", 
   RowBox[{"StandardizeEventSpecification", "[", 
    RowBox[{"{", 
     RowBox[{"sop", ",", 
      RowBox[{"SimplifyRules", "[", 
       RowBox[{"evopts", ",", "opts"}], "]"}]}], "}"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683729964064437*^9, 3.6837300066481647`*^9}, {
  3.684517008878005*^9, 3.68451704820859*^9}, {3.69099407187926*^9, 
  3.690994079246008*^9}},ExpressionUUID->"02bd5712-b208-48f2-a1c8-\
ff042fa58299"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeEventSpecification", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"repev_Repeat", ",", "evopts___Rule"}], "}"}], ",", 
     "opts__Rule"}], "]"}], ":=", 
   RowBox[{"StandardizeEventSpecification", "[", 
    RowBox[{"{", 
     RowBox[{"repev", ",", 
      RowBox[{"SimplifyRules", "[", 
       RowBox[{"evopts", ",", "opts"}], "]"}]}], "}"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683729964064437*^9, 3.6837300066481647`*^9}, {
   3.684517008878005*^9, 3.68451704820859*^9}, {3.684517156570578*^9, 
   3.684517176042232*^9}, 3.684946401229281*^9, {3.690994083807832*^9, 
   3.69099408907025*^9}, 
   3.6928482963318853`*^9},ExpressionUUID->"7588e4e8-9494-4407-b3eb-\
1d3091a26074"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeEventSpecification", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"precalev_PrecalculatedEvent", ",", "evopts___Rule"}], "}"}], 
     ",", "opts__Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"StandardizeEventSpecification", "[", 
    RowBox[{"{", 
     RowBox[{"precalev", ",", 
      RowBox[{"SimplifyRules", "[", 
       RowBox[{"evopts", ",", "opts"}], "]"}]}], "}"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683729964064437*^9, 3.6837300066481647`*^9}, {
   3.684517008878005*^9, 3.68451704820859*^9}, {3.684517156570578*^9, 
   3.6845171857231607`*^9}, {3.6909940969508753`*^9, 3.690994101881879*^9}, {
   3.690994409503231*^9, 3.690994411449974*^9}, 3.691435100438445*^9, 
   3.691436746851869*^9},ExpressionUUID->"7d5e20d3-8d1e-4dd4-84dc-\
21847c5e90a0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeEventSpecification", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"evgen_", "?", "ValidGeneratorQ"}], ",", 
       RowBox[{"dur", ":", "DurationPattern"}], ",", " ", "evopts___Rule"}], 
      "}"}], ",", "opts__Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"StandardizeEventSpecification", "[", 
    RowBox[{"{", 
     RowBox[{"evgen", ",", "dur", ",", 
      RowBox[{"SimplifyRules", "[", 
       RowBox[{"evopts", ",", "opts"}], "]"}]}], "}"}], "\[IndentingNewLine]",
     "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683729964064437*^9, 3.6837300066481647`*^9}, {
   3.684517008878005*^9, 3.68451704820859*^9}, {3.684517119205172*^9, 
   3.684517144587081*^9}, {3.690993535799841*^9, 3.690993538972856*^9}, 
   3.690993731299098*^9, {3.690993997412113*^9, 3.69099400984696*^9}, 
   3.690994050055582*^9, 
   3.709373716790804*^9},ExpressionUUID->"11f12166-e8ce-4421-8351-\
86a87575da9c"]
}, Closed]],

Cell[CellGroupData[{

Cell["standardize any evolution generators", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683730152042799*^9, 3.683730157092507*^9}, {
  3.684516981660862*^9, 3.684516999539733*^9}, {3.6909941049949617`*^9, 
  3.690994110924879*^9}, {3.690994143983363*^9, 
  3.6909941640143538`*^9}},ExpressionUUID->"ab86cb5a-0955-4f9d-aefd-\
433e7f07e705"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeEventSpecification", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"evgen_", "?", "ValidGeneratorQ"}], ",", 
      RowBox[{"dur", ":", "DurationPattern"}], ",", " ", "evopts___Rule"}], 
     "}"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"test", ",", "stdevg"}], "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{"StandardizeEventSpecification", "[", 
        RowBox[{"{", 
         RowBox[{"stdevg", ",", "dur", ",", "evopts"}], "}"}], "]"}], 
       "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"stdevg", "=", 
         RowBox[{"StandardizeGenerator", "[", "evgen", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"!", 
         RowBox[{"MatchQ", "[", 
          RowBox[{"stdevg", ",", "evgen"}], "]"}]}]}], "\[IndentingNewLine]", 
       ")"}]}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.690994269284101*^9, 3.690994384094325*^9}, 
   3.709373716799655*^9, 3.709374087048538*^9, {3.7114527126795883`*^9, 
   3.7114527160481043`*^9}, {3.7114527836768165`*^9, 
   3.7114527838454466`*^9}, {3.7114529743729725`*^9, 3.711452985979124*^9}, {
   3.7114530435872583`*^9, 3.7114530685566826`*^9}, {3.711453257471542*^9, 
   3.7114532576320534`*^9}, {3.7114533245196652`*^9, 3.71145333959134*^9}, {
   3.7114534158194594`*^9, 3.7114534210933523`*^9}, {3.7114538170116205`*^9, 
   3.7114538219952183`*^9}},ExpressionUUID->"5f2b2eda-f769-49c3-848a-\
dd24ee5a94c0"]
}, Closed]],

Cell[CellGroupData[{

Cell["standardize null events", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683730152042799*^9, 3.683730157092507*^9}, {
  3.683730254323736*^9, 3.683730264695097*^9}, {3.684497983287274*^9, 
  3.684497985512292*^9}},ExpressionUUID->"da22ab44-083b-44fd-9671-\
58d18c88b3bb"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeEventSpecification", "[", 
    RowBox[{"NonStandardNullEventSpecificationPattern", ",", "opts___Rule"}], 
    "]"}], ":=", "StandardNullEventSpecification"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.684497858915963*^9, 3.684497887119195*^9}, {
  3.684497937159707*^9, 3.6844979685474043`*^9}, {3.6844982037696657`*^9, 
  3.684498241669441*^9}, {3.684517216120817*^9, 3.684517228294837*^9}, {
  3.690990869340474*^9, 
  3.690990907511011*^9}},ExpressionUUID->"b61b344f-30e9-457f-959f-\
11af106f7235"]
}, Closed]],

Cell[CellGroupData[{

Cell["standardize EventSpecifications if not already done", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683730152042799*^9, 3.683730176539816*^9}, {
   3.683730487760866*^9, 3.6837304892112513`*^9}, 3.683957681141574*^9, {
   3.684498095259145*^9, 
   3.684498095729856*^9}},ExpressionUUID->"3b989af0-03d4-46ae-8aac-\
59baf2fa2b7c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeEventSpecification", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"evsp", ":", 
      RowBox[{"EventSpecifications", "[", 
       RowBox[{
        RowBox[{"status", ":", 
         RowBox[{"True", "|", 
          RowBox[{"{", 
           RowBox[{"chronstatus_", ",", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"_", ",", "True"}], "}"}], "|", "True"}], ",", 
            "morestatus___"}], "}"}]}]}], ",", "evspecs_", ",", 
        "evspecopts___Rule"}], "\[IndentingNewLine]", "]"}]}], ",", 
     "\[IndentingNewLine]", "opts___Rule"}], "\[IndentingNewLine]", "]"}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{"status", ",", "evspecs", ",", 
     RowBox[{"SimplifyRules", "[", 
      RowBox[{"evspecopts", ",", "opts"}], "]"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.68373019762939*^9, 3.683730221323333*^9}, 
   3.683957681149428*^9, {3.6842272380118017`*^9, 3.684227259521674*^9}, {
   3.684517325698957*^9, 3.684517373051866*^9}, 
   3.684517525807366*^9},ExpressionUUID->"3d3c9733-029d-4d7f-8fd7-\
d059dc8fa91e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeEventSpecification", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"EventSpecifications", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"chronstatus_", ",", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"_", ",", "False"}], "}"}], "|", "False"}], ",", 
         "morestatus___"}], "}"}], ",", "evspecs_", ",", 
       "evspecopts___Rule"}], "\[IndentingNewLine]", "]"}], ",", 
     "\[IndentingNewLine]", "opts___Rule"}], "\[IndentingNewLine]", "]"}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"chronstatus", ",", 
       RowBox[{"{", 
        RowBox[{"True", ",", "False"}], "}"}], ",", "morestatus"}], "}"}], 
     ",", "evspecs", ",", 
     RowBox[{"SimplifyRules", "[", 
      RowBox[{"evspecopts", ",", "opts"}], "]"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6837300272426443`*^9, 3.683730112911686*^9}, {
  3.683730183949093*^9, 3.683730186387388*^9}, {3.683957681155521*^9, 
  3.683957681157836*^9}, {3.684517384359024*^9, 
  3.684517412753068*^9}},ExpressionUUID->"576509f0-7605-4e01-9a69-\
93e1b063ecd7"]
}, Closed]],

Cell[CellGroupData[{

Cell["event specification cannot be interpreted", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683730152042799*^9, 3.683730176539816*^9}, {
  3.683730487760866*^9, 3.6837304892112513`*^9}, {3.6837836190618887`*^9, 
  3.6837836309599867`*^9}, {3.6837837756319017`*^9, 
  3.683783790309249*^9}},ExpressionUUID->"9c07ff43-fc9b-4b05-b0aa-\
536e69c3cfbe"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"StandardizeEventSpecification", "[", 
     RowBox[{"evspec_", ",", "opts___Rule"}], "]"}], "/;", 
    RowBox[{
     RowBox[{"!", 
      RowBox[{"TrueQ", "[", 
       RowBox[{"StandardizedEventSpecificationQ", "[", "evspec", "]"}], 
       "]"}]}], "&&", 
     RowBox[{"!", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"evspec", ",", 
        RowBox[{"Alternatives", "@@", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"FurtherEventSpecificationPatterns", "/.", 
            RowBox[{"{", "opts", "}"}]}], "/.", 
           RowBox[{"Options", "[", "EventSpecifications", "]"}]}], ")"}]}]}], 
       "]"}]}]}]}], ":=", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Message", "[", 
      RowBox[{
       RowBox[{"StandardizeEventSpecification", "::", "badevspec"}], ",", 
       "evspec"}], "]"}], ";", "$Failed"}], ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683783638058876*^9, 3.683783724760419*^9}, {
   3.6837839199061317`*^9, 3.683783923752118*^9}, {3.684517421540112*^9, 
   3.684517423532536*^9}, {3.691948788312018*^9, 3.691948823236978*^9}, 
   3.700485008864896*^9},ExpressionUUID->"c4a5d5f9-61f8-4684-a76d-\
28af8988f0f5"]
}, Closed]],

Cell[CellGroupData[{

Cell["messages", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683730152042799*^9, 3.683730176539816*^9}, {
   3.683730487760866*^9, 3.6837304892112513`*^9}, {3.6837836190618887`*^9, 
   3.6837836309599867`*^9}, {3.6837837303693457`*^9, 3.683783731920476*^9}, 
   3.683783793280416*^9},ExpressionUUID->"07f4fe60-fd5e-467d-815d-\
152e819671b6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeEventSpecification", "::", "badevspec"}], "=", 
   "\"\<The event specification <`1`> cannot be interpreted.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683783737770002*^9, 3.6837837529206333`*^9}, {
  3.683807762072991*^9, 
  3.68380776397987*^9}},ExpressionUUID->"38b86f92-c12a-43cb-9b4d-\
105bbf78349c"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["standard event specifications", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455454136981319*^9, 3.455454190582034*^9}, {
   3.455454472943882*^9, 3.455454474174431*^9}, {3.455455572039624*^9, 
   3.4554555804898577`*^9}, 3.455456514657077*^9, 3.455456988966947*^9, 
   3.455457877088749*^9, {3.455458472404565*^9, 3.45545847594317*^9}, {
   3.455634306583088*^9, 3.455634315476242*^9}, {3.456228041618371*^9, 
   3.456228065560835*^9}, {3.464332395796246*^9, 3.4643323961969957`*^9}, {
   3.4646057352918577`*^9, 3.464605771018136*^9}, {3.464605819725277*^9, 
   3.464605820219603*^9}, {3.465791712651339*^9, 3.4657917176480503`*^9}, 
   3.497572434410894*^9, {3.499557315376687*^9, 3.499557317863377*^9}, {
   3.514351750904572*^9, 3.514351753119151*^9}, 3.685033554264123*^9, 
   3.685035810659214*^9, 3.685167647427272*^9, 
   3.6910565137178802`*^9},ExpressionUUID->"3f39e788-032a-401a-84be-\
b2232fcef1e0"],

Cell[CellGroupData[{

Cell["StandardNullEventSpecification", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.504259747153373*^9, 3.5042597539203157`*^9}, {
   3.683368138913183*^9, 3.683368161209241*^9}, 3.684233874810207*^9, {
   3.6842341876092176`*^9, 3.684234190526638*^9}, {3.684235941328562*^9, 
   3.6842359440943317`*^9}, 3.684501784432807*^9, 3.68503355428093*^9, 
   3.685035810678938*^9, 3.685167664175256*^9, 
   3.690992730961186*^9},ExpressionUUID->"fb6143f8-1eba-485b-9431-\
790712021bc9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"StandardNullEventSpecification", ":=", 
   RowBox[{"{", 
    RowBox[{"UnitySuperoperator", "[", "]"}], "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.690990769993844*^9, 3.6909907990297318`*^9}, {
  3.6909920832519712`*^9, 
  3.690992088615155*^9}},ExpressionUUID->"11021d44-8264-48ac-8ae3-\
3e206905bf31"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["EventSpecifications", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455265644416114*^9, 3.455265648200944*^9}, {
   3.464256156522676*^9, 3.4642561730892*^9}, {3.507988005506776*^9, 
   3.507988026103808*^9}, {3.507988217579123*^9, 3.507988218121315*^9}, {
   3.5141085496721973`*^9, 3.514108561951315*^9}, 3.6833690953699417`*^9, 
   3.683957681171587*^9},
 CellTags->{
  "PrecalculationRules - working", 
   "EventSpecificationList"},ExpressionUUID->"dbb7611f-0ccc-457f-8ba5-\
770b92c44939"],

Cell["\<\
includes implementation of Chronology and optional expansion of Repeat.\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.514108987689993*^9, 3.514109006044366*^9}, {
   3.690554789710857*^9, 3.690554791047488*^9}, 
   3.692848297139656*^9},ExpressionUUID->"f96203a3-1df3-40b1-a8da-\
9b58da91dd29"],

Cell["\<\
The output of EventSpecifications always has RightToLeft Chronology\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.54505093316109*^9, 3.545050950992406*^9}, {
   3.5450523401113873`*^9, 3.545052344948779*^9}, {3.545052490408968*^9, 
   3.545052491677512*^9}, {3.683785821301919*^9, 3.683785821630674*^9}, 
   3.683957681186902*^9},ExpressionUUID->"070d1636-9898-45f1-a9c4-\
411ddcbe41ce"],

Cell[CellGroupData[{

Cell["syntax", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.513611021384453*^9, 3.513611111096521*^9}, {
  3.513611362301363*^9, 3.513611398542715*^9}, {3.513612019100143*^9, 
  3.513612029359522*^9}, {3.514109278829135*^9, 3.514109279514531*^9}, {
  3.683728405769937*^9, 
  3.683728411792903*^9}},ExpressionUUID->"f3febb1c-5cc4-4b05-b3e5-\
462296ba04d5"],

Cell["\<\
EventSpecifications[status, events, options] with
status = {checkstatus,chronstatus,nestedliststatus, \
standstatus,expintegerrepstatus}.
Each of the statusflags = {implement, done}
implement = True signals implementation to proceed.
done = True signals implementation has been done.
* checkstatus indicates that all event specifications have been checked and \
are valid.
* chronstatus refers to standardization of RtoL chronology.
* nestedliststatus refers to implementation of any nested lists.
* standstatus refers to the standardization of all events. 
* expintegerrepstatus refers to expansion of Repeat with integer loops\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.683728307124783*^9, 3.683728570664878*^9}, {
   3.6837296450378647`*^9, 3.683729675555797*^9}, {3.683784071922155*^9, 
   3.683784085760625*^9}, {3.683785829736596*^9, 3.683785859507976*^9}, 
   3.6839576812018003`*^9, {3.684500363738378*^9, 3.684500426074274*^9}, 
   3.690554802447773*^9, 
   3.6928482981961527`*^9},ExpressionUUID->"762b9976-b349-489d-ab70-\
7aa340952d8e"]
}, Closed]],

Cell[CellGroupData[{

Cell["default options", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.513611021384453*^9, 3.513611111096521*^9}, {
  3.513611362301363*^9, 3.513611398542715*^9}, {3.513612019100143*^9, 
  3.513612029359522*^9}, {3.514109278829135*^9, 3.514109279514531*^9}, {
  3.6845004427725143`*^9, 
  3.684500444479751*^9}},ExpressionUUID->"be139c20-8f97-4d97-9667-\
d60d7469852b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "EventSpecifications", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Chronology", "\[RuleDelayed]", "$Chronology"}], ",", 
     RowBox[{"ExpandRepeatedEvents", "\[Rule]", "False"}], ",", 
     RowBox[{"Verbose", "\[Rule]", "False"}], ",", 
     RowBox[{"FurtherEventSpecificationPatterns", "\[Rule]", 
      RowBox[{"{", "}"}]}]}], "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5136120313750753`*^9, 3.513612053002006*^9}, 
   3.514108566653123*^9, {3.514108974668661*^9, 3.514108975039687*^9}, {
   3.5147924513172817`*^9, 3.5147924516305304`*^9}, {3.534398372223238*^9, 
   3.5343983725629787`*^9}, 3.545050462851315*^9, {3.5450508970912952`*^9, 
   3.545050898139974*^9}, {3.5450510116261473`*^9, 3.545051066489931*^9}, {
   3.545052478520299*^9, 3.545052483903796*^9}, {3.587816022890677*^9, 
   3.58781603391149*^9}, {3.587816073929941*^9, 3.5878160849356403`*^9}, 
   3.680344801962186*^9, 3.683728580198654*^9, 3.6839576812100477`*^9, {
   3.6847755204915257`*^9, 3.684775523587432*^9}, {3.691043823990176*^9, 
   3.691043828988109*^9}, {3.691947830638752*^9, 3.691947870920369*^9}, {
   3.6919484798619432`*^9, 3.691948502256534*^9}, 
   3.691948645848484*^9},ExpressionUUID->"5da2111a-512d-46a4-bb47-\
80ae5116612d"]
}, Closed]],

Cell[CellGroupData[{

Cell["$Failed", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, {3.683367853191883*^9, 3.683367853786949*^9}, 
   3.68336810348781*^9},ExpressionUUID->"8e152ec3-5602-4c9c-8342-\
b745d44a7839"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventSpecifications", "[", "args___", "]"}], "/;", 
    RowBox[{"!", 
     RowBox[{"FreeQ", "[", 
      RowBox[{
       RowBox[{"{", "args", "}"}], ",", "$Failed"}], "]"}]}]}], ":=", 
   "$Failed"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5144539068637877`*^9, 3.514453917622925*^9}, 
   3.5144562170712967`*^9, {3.683367787008257*^9, 3.683367806267187*^9}, 
   3.6839576812160807`*^9},ExpressionUUID->"84b95d0f-3a13-407b-8a4f-\
bab4a2471a51"]
}, Closed]],

Cell[CellGroupData[{

Cell["status", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.514738781326264*^9, 3.514738794407814*^9}, {
   3.545052507651114*^9, 3.545052523135462*^9}, 3.545052645598901*^9, {
   3.683728597721991*^9, 3.683728601679985*^9}, 3.683824888026587*^9, 
   3.683825236493738*^9, {3.6845007268874474`*^9, 
   3.6845007275152473`*^9}},ExpressionUUID->"721d808a-977a-4851-bfb1-\
24e7d2aa1ae5"],

Cell["\<\
{<checked>,<chronology>, <nestedliststatus>,<standardization of event \
specifications>,<expansion of integer loop repeated events>}\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.6837336954886312`*^9, 3.683733728044663*^9}, {
  3.683781382106575*^9, 3.683781398865197*^9}, {3.683824895200515*^9, 
  3.683824906533574*^9}, {3.684500467826048*^9, 
  3.684500475114438*^9}},ExpressionUUID->"56f6bd58-803c-4596-915c-\
68cea3616c22"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StatusDimension", "[", "EventSpecifications", "]"}], "=", "5"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683825197880946*^9, 3.683825264681492*^9}, 
   3.683957681222283*^9, 
   3.6845004539106617`*^9},ExpressionUUID->"28b41ab0-e0c4-466a-b344-\
abac20e72fb7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", 
    RowBox[{"evspecs_", ",", "evspecsopts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{"False", ",", "evspecs", ",", "evspecsopts"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683728608339706*^9, 3.683728662000853*^9}, {
  3.683733669872313*^9, 3.683733674053347*^9}, {3.683739962260284*^9, 
  3.683739966025916*^9}, {3.683785873791065*^9, 3.6837858748680353`*^9}, {
  3.683809669729992*^9, 3.68380967038408*^9}, {3.6839576812283897`*^9, 
  3.68395768123037*^9}},ExpressionUUID->"9939a4af-9086-4a00-891b-\
e756f2b9643b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", 
    RowBox[{"False", ",", "evspecs_", ",", "evspecsopts___Rule"}], "]"}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{"False", ",", 
       RowBox[{"{", 
        RowBox[{"StatusDimension", "[", "EventSpecifications", "]"}], "}"}]}],
       "]"}], ",", "evspecs", ",", "evspecsopts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683809653952795*^9, 3.6838096657855864`*^9}, {
   3.683825252435833*^9, 3.6838252583778057`*^9}, {3.6839576812365227`*^9, 
   3.683957681240633*^9}, 3.691044459217677*^9, {3.691044916560384*^9, 
   3.691044916943965*^9}},ExpressionUUID->"8188266a-a7d6-4499-8052-\
a95b60fa72ea"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"True", ".."}], "}"}], ",", "evspecs_", ",", "opts___Rule"}], 
    "]"}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{"True", ",", "evspecs", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683824849504134*^9, 3.6838248657125683`*^9}, {
  3.683957681246735*^9, 
  3.6839576812488537`*^9}},ExpressionUUID->"5a086a83-3041-40a6-a133-\
fd26effab7d2"]
}, Closed]],

Cell[CellGroupData[{

Cell["non-list argument", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.514738781326264*^9, 3.514738794407814*^9}, {
   3.545052507651114*^9, 3.545052523135462*^9}, 3.545052645598901*^9, {
   3.683728597721991*^9, 3.683728601679985*^9}, 3.683824888026587*^9, 
   3.683825236493738*^9, {3.6845007268874474`*^9, 3.6845007275152473`*^9}, {
   3.68469204776381*^9, 
   3.684692052603023*^9}},ExpressionUUID->"59cfcc90-ebc0-4315-9a13-\
219a37fac72a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"arg", ":", 
      RowBox[{"Except", "[", "_List", "]"}]}], ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", "arg", "}"}], ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5144539068637877`*^9, 3.514453917622925*^9}, 
   3.5144562170712967`*^9, {3.683367787008257*^9, 3.683367806267187*^9}, 
   3.6839576812160807`*^9, {3.6846920070224867`*^9, 
   3.684692031666548*^9}},ExpressionUUID->"723cc71d-4ef3-47a1-8262-\
759681c41642"]
}, Closed]],

Cell[CellGroupData[{

Cell["event standardization", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.454567394439436*^9, 3.45456739938997*^9}, {
   3.454576414690864*^9, 3.4545764315130777`*^9}, {3.454576579278468*^9, 
   3.454576580716839*^9}, {3.455336663472224*^9, 3.4553366732854643`*^9}, {
   3.455452365031452*^9, 3.455452384820578*^9}, {3.464256930105167*^9, 
   3.464256938583194*^9}, {3.464257382327674*^9, 3.464257387747345*^9}, {
   3.464272043354971*^9, 3.464272045792259*^9}, 3.464275608036623*^9, {
   3.464331333001854*^9, 3.464331334992557*^9}, 3.5343880033241243`*^9, {
   3.6837294633814993`*^9, 3.683729467171728*^9}, {3.683729698252934*^9, 
   3.683729700912363*^9}, {3.6837298176331873`*^9, 3.683729821495585*^9}, {
   3.684500789022656*^9, 
   3.684500792901903*^9}},ExpressionUUID->"847bc678-d3f7-4742-9a43-\
d3d882091872"],

Cell[CellGroupData[{

Cell["standardization status flags", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683729040586273*^9, 3.6837290508666983`*^9}, {
  3.683731076062901*^9, 3.6837310800779457`*^9}, {3.684499560409563*^9, 
  3.684499561251005*^9}},ExpressionUUID->"ef0e3636-2b30-4d3c-af2b-\
4681bb506c36"],

Cell["\<\
standardization status has form 
1. <implement>, 2. <empty events removed>, 3.<options incorporated>,
4. <no-option-events removed>, 5.<events standardized>\
\>", "Text",
 CellChangeTimes->{{3.684688383077387*^9, 3.684688517368384*^9}, {
  3.684690395426276*^9, 
  3.684690398724072*^9}},ExpressionUUID->"b54a845c-9f58-450b-a2a6-\
0e9867f2f861"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "checkstatus_", ",", "chronstatus_", ",", "nestliststatus_", ",", 
       "False", ",", "morestatus___"}], "}"}], ",", "evspecs_", ",", 
     "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"checkstatus", ",", "chronstatus", ",", "nestliststatus", ",", 
       RowBox[{"Table", "[", 
        RowBox[{"False", ",", 
         RowBox[{"{", "5", "}"}]}], "]"}], ",", "morestatus"}], "}"}], ",", 
     "evspecs", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.68372974910354*^9, 3.6837298122967987`*^9}, {
  3.683730957983409*^9, 3.683730976165638*^9}, {3.6837862249136744`*^9, 
  3.6837862372310467`*^9}, {3.6839576814318*^9, 3.683957681433938*^9}, {
  3.684500492712467*^9, 3.684500503567472*^9}, {3.684688523562709*^9, 
  3.68468852871082*^9}},ExpressionUUID->"00a469f0-35cb-47fe-813e-\
38688d73d2a8"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "checkstatus_", ",", "chronstatus_", ",", "nestliststatus_", ",", 
       RowBox[{"{", 
        RowBox[{"True", ".."}], "}"}], ",", "morestatus___"}], "}"}], ",", 
     "evspecs_", ",", "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "checkstatus", ",", "chronstatus", ",", "nestliststatus", ",", "True", 
       ",", "morestatus"}], "}"}], ",", "evspecs", ",", "opts"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.68372974910354*^9, 3.6837298122967987`*^9}, {
  3.683730957983409*^9, 3.683730976165638*^9}, {3.683731114649835*^9, 
  3.683731118572782*^9}, {3.683786241936226*^9, 3.6837862477577677`*^9}, {
  3.683957681441441*^9, 3.683957681443549*^9}, {3.684500511164007*^9, 
  3.684500521140523*^9}, {3.68468853601725*^9, 
  3.684688537359475*^9}},ExpressionUUID->"2eb07680-cf8e-481e-8bf3-\
ba253a1476ab"]
}, Closed]],

Cell[CellGroupData[{

Cell["standardize event specifications", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683729040586273*^9, 3.6837290508666983`*^9}, {
   3.6837310203114443`*^9, 3.6837310221665573`*^9}, {3.683731133390791*^9, 
   3.683731136308968*^9}, 
   3.691045139130485*^9},ExpressionUUID->"68e2fb64-f5ac-428e-900d-\
67404a713477"],

Cell[CellGroupData[{

Cell["initialize status ", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683731796279813*^9, 3.683731808833954*^9}, {
  3.684499589310541*^9, 3.6844995934721403`*^9}, {3.6845022381072493`*^9, 
  3.684502239500084*^9}},ExpressionUUID->"24f65e0e-b1da-457f-96e1-\
d0e34067d5ba"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "checkstatus_", ",", "chronstatus_", ",", "nestliststatus_", ",", 
       RowBox[{"{", 
        RowBox[{"False", ",", "___"}], "}"}], ",", "morestatus___"}], "}"}], 
     ",", "evspecs_", ",", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "checkstatus", ",", "chronstatus", ",", "nestliststatus", ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
        "True", ",", "False", ",", "False", ",", "False", ",", "False"}], 
        "}"}], ",", "morestatus"}], "}"}], ",", "evspecs", ",", "opts"}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.68372974910354*^9, 3.6837298122967987`*^9}, 
   3.683731364566168*^9, {3.6837866498436213`*^9, 3.683786654643095*^9}, {
   3.68395768145128*^9, 3.683957681453465*^9}, {3.684500534868494*^9, 
   3.684500542426063*^9}, {3.68468856162414*^9, 3.684688569445827*^9}, {
   3.684689457447435*^9, 
   3.6846894579972343`*^9}},ExpressionUUID->"e1d42cbe-3546-4d25-8ce3-\
cb008303ba2b"]
}, Closed]],

Cell[CellGroupData[{

Cell["set Standardization status if all done", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683731796279813*^9, 3.6837318185361633`*^9}, {
   3.683786383040831*^9, 3.683786387901951*^9}, 3.684236524413398*^9, {
   3.6844993000857162`*^9, 3.684499311334092*^9}, {3.684501574808803*^9, 
   3.6845016023012457`*^9}, {3.684501893615694*^9, 3.684501894571189*^9}, {
   3.6845026864505987`*^9, 3.6845026886198874`*^9}, {3.684503215984846*^9, 
   3.684503220427644*^9}, {3.68451883025848*^9, 3.68451883254591*^9}, {
   3.6910533260334463`*^9, 3.691053341808277*^9}, {3.691053544675734*^9, 
   3.691053545353375*^9}},ExpressionUUID->"eea80239-f3a9-4064-aa1d-\
185def147793"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"status", ":", 
       RowBox[{"{", 
        RowBox[{
        "checkstatus_", ",", "chronstatus_", ",", "nestliststatus_", ",", 
         RowBox[{"{", 
          RowBox[{
          "True", ",", "True", ",", "True", ",", "True", ",", "False"}], 
          "}"}], ",", "morestatus___"}], "}"}]}], ",", "\[IndentingNewLine]", 
      
      RowBox[{"evspecs", ":", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"_Except", "[", "Rule", "]"}], ".."}], "}"}]}], ",", 
      "opts___Rule"}], "\[IndentingNewLine]", "]"}], "/;", 
    RowBox[{"And", "@@", 
     RowBox[{"(", 
      RowBox[{"StandardizedEventSpecificationQ", "/@", "evspecs"}], ")"}]}]}],
    ":=", "\[IndentingNewLine]", 
   RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"checkstatus", ",", "chronstatus", ",", "nestliststatus", ",", 
       RowBox[{"{", 
        RowBox[{"True", ",", "True", ",", "True", ",", "True", ",", "True"}], 
        "}"}], ",", "morestatus"}], "}"}], ",", "\[IndentingNewLine]", 
     "evspecs", ",", "opts"}], "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",\

 InitializationCell->True,
 CellChangeTimes->{{3.6910534131670732`*^9, 3.691053540044817*^9}, {
   3.691053675928211*^9, 3.69105368800469*^9}, 
   3.700485008872347*^9},ExpressionUUID->"0ba4d5fd-331d-438f-95f8-\
7c722afed365"]
}, Closed]],

Cell[CellGroupData[{

Cell["case of single event", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683731796279813*^9, 3.683731808833954*^9}, 
   3.684499706629848*^9},ExpressionUUID->"7ee47225-10e5-41ca-aed6-\
0ef8a882af84"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"status", ":", 
      RowBox[{"{", 
       RowBox[{
       "checkstatus_", ",", "chronstatus_", ",", "nestliststatus_", ",", 
        RowBox[{"{", 
         RowBox[{"True", ",", "__"}], "}"}], ",", "morestatus___"}], "}"}]}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"evspec", ":", "EventSpecificationPattern"}], ",", 
     "opts___Rule"}], "\[IndentingNewLine]", "]"}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{"status", ",", 
     RowBox[{"{", "evspec", "}"}], ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6837317177710323`*^9, 3.6837317802412367`*^9}, {
   3.683786263193097*^9, 3.683786317896369*^9}, {3.6839576814612217`*^9, 
   3.683957681463418*^9}, {3.684236000841113*^9, 3.684236003141459*^9}, 
   3.684236075727487*^9, {3.684500549156082*^9, 3.684500549795033*^9}, 
   3.684501832953163*^9, {3.68451917976676*^9, 3.684519180391522*^9}, {
   3.684688584688122*^9, 3.6846886037490263`*^9}, 
   3.690992881299995*^9},ExpressionUUID->"3ac83fcf-5896-4a75-b094-\
b745cd50a84a"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
case of a single, already standardized EventSpecifications object\
\>", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683731796279813*^9, 3.683731808833954*^9}, 
   3.684499706629848*^9, {3.692172660087749*^9, 3.6921726641270514`*^9}, {
   3.692172751991521*^9, 3.692172754171262*^9}, {3.692172808229141*^9, 
   3.6921728093705*^9}, {3.6922592776901093`*^9, 
   3.6922592779125137`*^9}},ExpressionUUID->"7b8cbd62-f49a-496e-9cb1-\
bfdf532af150"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
    RowBox[{"status_", ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"evspec", ":", 
        RowBox[{"EventSpecifications", "[", 
         RowBox[{"True", ",", 
          RowBox[{"evspecargs", ":", 
           RowBox[{
            RowBox[{"Except", "[", "_Rule", "]"}], "..."}]}], ",", 
          "evspecopts___Rule"}], "]"}]}], ",", "moreevspecsopts___Rule"}], 
      "}"}], ",", "\[IndentingNewLine]", "opts___Rule"}], 
    "\[IndentingNewLine]", "]"}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{"True", ",", "evspecargs", ",", "\[IndentingNewLine]", 
     RowBox[{"SimplifyRules", "[", 
      RowBox[{"evspecopts", ",", "moreevspecsopts", ",", "opts"}], "]"}]}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.692172287303646*^9, 3.692172306646195*^9}, {
   3.6921723641648912`*^9, 3.6921724305718803`*^9}, {3.692172465297291*^9, 
   3.692172466024129*^9}, {3.692172498505097*^9, 3.692172541717333*^9}, 
   3.6921726104289637`*^9, {3.692172686829265*^9, 3.692172793562302*^9}, {
   3.692257681276566*^9, 
   3.692257812574782*^9}},ExpressionUUID->"115d1bfc-5a9d-4c54-a3a0-\
75c28a81be2d"]
}, Closed]],

Cell[CellGroupData[{

Cell["remove any null events", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683731796279813*^9, 3.683731808833954*^9}, {
  3.6837925303963957`*^9, 3.68379253526569*^9}, {3.6910515158611813`*^9, 
  3.691051516633635*^9}},ExpressionUUID->"dc6da514-2585-4627-9bd9-\
e4ade4fdb216"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"status", ":", 
      RowBox[{"{", 
       RowBox[{
       "checkstatus_", ",", "chronstatus_", ",", "nestliststatus_", ",", 
        RowBox[{"{", 
         RowBox[{"True", ",", "False", ",", "morestandstatus___"}], "}"}], 
        ",", "morestatus___"}], "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"evspecs", ":", 
      RowBox[{"{", 
       RowBox[{"rawevspecs___", ",", "evspecsopts___Rule"}], "}"}]}], ",", 
     "\[IndentingNewLine]", "opts___Rule"}], "\[IndentingNewLine]", "]"}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"If", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Count", "[", 
         RowBox[{
          RowBox[{"{", "rawevspecs", "}"}], ",", 
          "NullEventSpecificationPattern", ",", "1"}], "]"}], 
        "\[GreaterEqual]", "1"}], ")"}], "&&", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Count", "[", 
         RowBox[{
          RowBox[{"{", "rawevspecs", "}"}], ",", 
          RowBox[{"Except", "[", "NullEventSpecificationPattern", "]"}], ",", 
          "1"}], "]"}], "\[GreaterEqual]", "1"}], ")"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Verbose", "/.", 
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "EventSpecifications", "]"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
         "Print", "[", 
          "\"\<EventSpecifications: stripping out null events.\>\"", "]"}]}], 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"EventSpecifications", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
          "checkstatus", ",", "chronstatus", ",", "nestliststatus", ",", 
           RowBox[{"{", 
            RowBox[{"True", ",", "True", ",", "morestandstatus"}], "}"}], ",",
            "morestatus"}], "}"}], ",", "\[IndentingNewLine]", 
         RowBox[{"Replace", "[", "\[IndentingNewLine]", 
          RowBox[{"evspecs", ",", "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"nullevspec", ":", "NullEventSpecificationPattern"}], 
             "\[RuleDelayed]", 
             RowBox[{"Sequence", "[", "]"}]}], "}"}], ",", 
           "\[IndentingNewLine]", "1"}], "\[IndentingNewLine]", "]"}], ",", 
         "\[IndentingNewLine]", "opts"}], "\[IndentingNewLine]", "]"}]}], 
      "\[IndentingNewLine]", ")"}], ",", "\[IndentingNewLine]", 
     RowBox[{"EventSpecifications", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"checkstatus", ",", "chronstatus", ",", "nestliststatus", ",", 
         RowBox[{"{", 
          RowBox[{"True", ",", "True", ",", "morestandstatus"}], "}"}], ",", 
         "morestatus"}], "}"}], ",", "\[IndentingNewLine]", "evspecs", ",", 
       "opts"}], "]"}]}], "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6837317177710323`*^9, 3.6837317802412367`*^9}, {
   3.683786263193097*^9, 3.683786317896369*^9}, {3.683792552668041*^9, 
   3.6837925832891607`*^9}, {3.683957681471335*^9, 3.683957681473515*^9}, {
   3.684499643921213*^9, 3.684499660513595*^9}, {3.684500573532851*^9, 
   3.6845005740057783`*^9}, {3.684688616519433*^9, 3.6846886738287487`*^9}, {
   3.684688853581386*^9, 3.6846888695546703`*^9}, {3.6846889196111393`*^9, 
   3.684689040356763*^9}, 3.6846901971913633`*^9, {3.684776075804206*^9, 
   3.684776145731701*^9}, 3.690992572982088*^9, {3.690992971798154*^9, 
   3.690992986593717*^9}, {3.690995322376397*^9, 3.6909953253607388`*^9}, {
   3.6909959507426767`*^9, 3.6909959639758387`*^9}, {3.690995998089115*^9, 
   3.690996075117853*^9}, 3.690997292074362*^9, {3.690997489703287*^9, 
   3.690997528192293*^9}, {3.690997602440131*^9, 3.6909976410748997`*^9}, {
   3.6910439593065643`*^9, 3.691043972734151*^9}, {3.691045187417583*^9, 
   3.691045205474483*^9}, {3.691045451840418*^9, 3.6910455529314003`*^9}, {
   3.691045855649362*^9, 3.6910458595953217`*^9}, 3.6910460044468*^9, {
   3.69105158005197*^9, 3.691051638767563*^9}, {3.6910517462724648`*^9, 
   3.691051780045295*^9}, {3.691051812131419*^9, 3.691051818109708*^9}, {
   3.6910519855495167`*^9, 3.691052037781986*^9}, {3.691054160609387*^9, 
   3.691054240687866*^9}},ExpressionUUID->"3f2c46ef-5de3-40fd-ae10-\
37db61902700"]
}, Closed]],

Cell[CellGroupData[{

Cell["incorporate options into events", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683731796279813*^9, 3.683731808833954*^9}, {
  3.6837925303963957`*^9, 3.68379253526569*^9}, {3.684517657717452*^9, 
  3.684517662298555*^9}},ExpressionUUID->"e776e1b7-ba64-49cf-a647-\
895b7325e9b0"],

Cell["incorporate options into events only when standardized", "Text",
 CellChangeTimes->{{3.6845196244858847`*^9, 3.68451963510826*^9}, {
  3.684690338286475*^9, 3.684690345861343*^9}, {3.684690417533588*^9, 
  3.6846904210503883`*^9}, {3.684690618898128*^9, 
  3.684690628279954*^9}},ExpressionUUID->"d4f14bd9-761d-4b26-bc57-\
c80bfdb03ef4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"status", ":", 
      RowBox[{"{", 
       RowBox[{
       "checkstatus_", ",", "chronstatus_", ",", "nestliststatus_", ",", 
        RowBox[{"{", 
         RowBox[{
         "True", ",", "True", ",", "False", ",", "stnoopevs_", ",", "True"}], 
         "}"}], ",", "morestatus___"}], "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"evspecs", ":", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"evspecseq", ":", 
         RowBox[{
          RowBox[{"Except", "[", "_Rule", "]"}], ".."}]}], ",", 
        "evspecopts___Rule"}], "}"}]}], ",", "opts___Rule"}], 
    "\[IndentingNewLine]", "]"}], ":=", 
   RowBox[{"(", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Verbose", "/.", 
         RowBox[{"{", "opts", "}"}]}], "/.", 
        RowBox[{"Options", "[", "EventSpecifications", "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Print", "[", 
        RowBox[{
        "\"\<EventSpecifications: incorporating options: \>\"", ",", " ", 
         RowBox[{
          RowBox[{"Expand", "[", "evspecs", "]"}], "/.", 
          RowBox[{"s_ShapeFunction", "\[RuleDelayed]", 
           RowBox[{"s", "\[LeftDoubleBracket]", 
            RowBox[{"{", "4", "}"}], "\[RightDoubleBracket]"}]}]}]}], "]"}]}],
       "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"checkstatus", ",", "chronstatus", ",", "nestliststatus", ",", 
         RowBox[{"{", 
          RowBox[{
          "True", ",", "True", ",", "True", ",", "stnoopevs", ",", "True"}], 
          "}"}], ",", "morestatus"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Replace", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"{", "evspecseq", "}"}], ",", "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"sop_", "?", "SuperoperatorQ"}], ",", "evopts___Rule"}],
              "}"}], "\[RuleDelayed]", "\[IndentingNewLine]", 
            RowBox[{"{", 
             RowBox[{"sop", ",", 
              RowBox[{"allowedeventopts", "[", 
               RowBox[{"evopts", ",", "evspecopts", ",", "opts"}], "]"}]}], 
             "}"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"evgen_", "?", "ValidGeneratorQ"}], ",", 
              RowBox[{"dur", ":", "DurationPattern"}], ",", " ", 
              "evopts___Rule"}], "}"}], ":>", 
            RowBox[{"{", 
             RowBox[{"evgen", ",", "dur", ",", 
              RowBox[{"allowedeventopts", "[", 
               RowBox[{"evopts", ",", "evspecopts", ",", "opts"}], "]"}]}], 
             "}"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"repev_Repeat", ",", "evopts___Rule"}], "}"}], ":>", 
            RowBox[{"{", 
             RowBox[{"repev", ",", 
              RowBox[{"allowedeventopts", "[", 
               RowBox[{
               "Repeat", ",", "evopts", ",", "evspecopts", ",", "opts"}], 
               "]"}]}], "}"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"precalev_PrecalculatedEvent", ",", "evopts___Rule"}], 
             "}"}], ":>", 
            RowBox[{"{", 
             RowBox[{"precalev", ",", 
              RowBox[{"allowedeventopts", "[", 
               RowBox[{"evopts", ",", "evspecopts", ",", "opts"}], "]"}]}], 
             "}"}]}]}], "\[IndentingNewLine]", "}"}], ",", 
         "\[IndentingNewLine]", "1"}], "\[IndentingNewLine]", "]"}], ",", 
       "\[IndentingNewLine]", "opts"}], "\[IndentingNewLine]", "]"}]}], 
    ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.684517681955415*^9, 3.68451776728899*^9}, {
   3.684517856213764*^9, 3.684518073053934*^9}, {3.684518104123602*^9, 
   3.684518110907692*^9}, {3.684688729050946*^9, 3.684688770597949*^9}, {
   3.6846890741107187`*^9, 3.684689110034608*^9}, 3.6846895947077093`*^9, {
   3.684690370167158*^9, 3.6846903875879583`*^9}, {3.684690431363453*^9, 
   3.684690466114399*^9}, {3.684690738380527*^9, 3.684690825466757*^9}, {
   3.684945241617899*^9, 3.684945276912692*^9}, {3.684945375930277*^9, 
   3.6849453940959187`*^9}, 3.684945432129637*^9, 3.684946401247311*^9, {
   3.684990565738267*^9, 3.684990665737722*^9}, {3.684990912474309*^9, 
   3.684990933649926*^9}, {3.684991020461918*^9, 3.6849910211400146`*^9}, {
   3.691043986529599*^9, 3.691044025543948*^9}, {3.691050519172268*^9, 
   3.691050520258068*^9}, {3.691055411774796*^9, 3.691055428531576*^9}, {
   3.6912324874587*^9, 3.691232490954829*^9}, 3.6914351080941668`*^9, 
   3.69143674686467*^9, {3.692848299354212*^9, 3.692848301401145*^9}, {
   3.699204733276744*^9, 3.699204758179076*^9}, 
   3.7093737168243933`*^9},ExpressionUUID->"164ff284-a8bc-4870-adde-\
73f64a38a096"],

Cell["\<\
only allow incorporation of options that are known to be appropriate to \
future calculations\
\>", "Text",
 CellChangeTimes->{{3.684945193276149*^9, 3.684945208786931*^9}, {
  3.6849459168066397`*^9, 3.684945926915021*^9}, {3.684990946951445*^9, 
  3.684990971123149*^9}},ExpressionUUID->"95f71a18-b3e3-4676-a23e-\
46e2d43bb360"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"allowedeventopts", "[", "opts___Rule", "]"}], ":=", 
   RowBox[{"Sequence", "@@", 
    RowBox[{"SimplifyRules", "[", 
     RowBox[{"FilterRules", "[", 
      RowBox[{
       RowBox[{"{", "opts", "}"}], ",", 
       RowBox[{"Join", "[", 
        RowBox[{
         RowBox[{"Options", "[", "Event", "]"}], ",", 
         RowBox[{"Options", "[", "EventList", "]"}], ",", 
         RowBox[{"Options", "[", "NDSolve", "]"}]}], "]"}]}], "]"}], 
     "]"}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.684945290013015*^9, 3.684945360966422*^9}, {
  3.684945686642542*^9, 3.684945689411599*^9}, {3.6849458617357492`*^9, 
  3.684945904979638*^9}, {3.684990669784911*^9, 3.6849907637576036`*^9}, {
  3.684990828915156*^9, 3.684990859008788*^9}, {3.684990978687194*^9, 
  3.684990978924062*^9}, {3.6849919985557833`*^9, 3.684992001779004*^9}, {
  3.685033539434896*^9, 
  3.685033569305461*^9}},ExpressionUUID->"b9c1ccce-18be-40c3-a8bb-\
a4b4d5cbcb16"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"allowedeventopts", "[", 
    RowBox[{"Repeat", ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"Sequence", "@@", 
    RowBox[{"SimplifyRules", "[", 
     RowBox[{"FilterRules", "[", 
      RowBox[{
       RowBox[{"{", "opts", "}"}], ",", 
       RowBox[{"Join", "[", 
        RowBox[{
         RowBox[{"{", "ExpandRepeatedEvents", "}"}], ",", 
         RowBox[{"Options", "[", "Event", "]"}], ",", 
         RowBox[{"Options", "[", "EventList", "]"}], ",", 
         RowBox[{"Options", "[", "NDSolve", "]"}]}], "]"}]}], "]"}], 
     "]"}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.684945290013015*^9, 3.684945360966422*^9}, {
   3.684945686642542*^9, 3.684945689411599*^9}, {3.6849458617357492`*^9, 
   3.684945904979638*^9}, {3.684990669784911*^9, 3.6849907637576036`*^9}, {
   3.684990828915156*^9, 3.684990890841537*^9}, {3.684991170656568*^9, 
   3.6849911755025806`*^9}, {3.684991977102807*^9, 3.6849919807809362`*^9}, {
   3.685033539441852*^9, 3.685033569308361*^9}, 
   3.692848302903637*^9},ExpressionUUID->"341e46ac-faf6-4fbc-8986-\
403e6614d165"]
}, Closed]],

Cell[CellGroupData[{

Cell["case of no EventSpecifications arguments", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683731796279813*^9, 3.683731808833954*^9}, {
   3.6837925303963957`*^9, 3.68379253526569*^9}, {3.684517657717452*^9, 
   3.684517662298555*^9}, {3.6845182006962147`*^9, 3.684518230682577*^9}, {
   3.684518478105418*^9, 3.6845184798408203`*^9}, {3.691044988238505*^9, 
   3.691044989939796*^9}, 3.691050697365509*^9, {3.691051022229241*^9, 
   3.691051029676602*^9}, {3.692172324383422*^9, 3.692172325236281*^9}, {
   3.6921728700910997`*^9, 
   3.692172871576309*^9}},ExpressionUUID->"8949da97-649c-4d76-b714-\
07026c6e6774"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"status", ":", 
       RowBox[{"{", 
        RowBox[{
        "checkstatus_", ",", "chronstatus_", ",", "nestliststatus_", ",", 
         RowBox[{"{", 
          RowBox[{
          "True", ",", "stemp_", ",", "stop_", ",", "False", ",", 
           "morestandstatus___"}], "}"}], ",", "morestatus___"}], "}"}]}], 
      ",", "\[IndentingNewLine]", "evspecs_List", ",", "\[IndentingNewLine]", 
      "opts___Rule"}], "\[IndentingNewLine]", "]"}], "/;", 
    RowBox[{"FreeQ", "[", 
     RowBox[{"evspecs", ",", "_EventSpecifications"}], "]"}]}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "checkstatus", ",", "chronstatus", ",", "nestliststatus", ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
        "True", ",", "stemp", ",", "stop", ",", "True", ",", 
         "morestandstatus"}], "}"}], ",", "morestatus"}], "}"}], ",", 
     "evspecs", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.692172287303646*^9, 
  3.692172306646195*^9}},ExpressionUUID->"e54b1fb6-d545-4871-bdb8-\
e81fd52c5ca3"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
case of more than one already standardized EventSpecifications argument. \
\>", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683731796279813*^9, 3.683731808833954*^9}, {
   3.6837925303963957`*^9, 3.68379253526569*^9}, {3.684517657717452*^9, 
   3.684517662298555*^9}, {3.6845182006962147`*^9, 3.684518230682577*^9}, {
   3.684518478105418*^9, 3.6845184798408203`*^9}, {3.691044988238505*^9, 
   3.691044989939796*^9}, 3.691050697365509*^9, {3.691051022229241*^9, 
   3.691051029676602*^9}, {3.692172324383422*^9, 3.692172325236281*^9}, {
   3.692172879459988*^9, 3.692172895841728*^9}, {3.692172962642089*^9, 
   3.692172974541439*^9}},ExpressionUUID->"36b7f580-d8fa-4684-97b6-\
011bf9c27d08"],

Cell["\<\
If the Chronology of the containing EventSpecifications object and all of the \
contained ones are already standardized, just absorb the arguments\
\>", "Text",
 CellChangeTimes->{{3.69217330306962*^9, 3.692173327507534*^9}, {
  3.692174186566293*^9, 3.692174192043898*^9}, {3.692260124092766*^9, 
  3.6922601256979713`*^9}},ExpressionUUID->"ea62fddc-817a-432e-90d0-\
139c07b2baa1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"status", ":", 
       RowBox[{"{", 
        RowBox[{"checkstatus_", ",", "True", ",", "nestliststatus_", ",", 
         RowBox[{"{", 
          RowBox[{
          "True", ",", "stemp_", ",", "stop_", ",", "False", ",", 
           "morestandstatus___"}], "}"}], ",", "morestatus___"}], "}"}]}], 
      ",", "\[IndentingNewLine]", "evspecs_List", ",", "\[IndentingNewLine]", 
      "opts___Rule"}], "\[IndentingNewLine]", "]"}], "/;", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Count", "[", 
        RowBox[{"evspecs", ",", 
         RowBox[{"EventSpecifications", "[", 
          RowBox[{"True", ",", "___"}], "]"}], ",", 
         RowBox[{"{", "1", "}"}]}], "]"}], "\[GreaterEqual]", "1"}], ")"}], "&&", 
     RowBox[{"FreeQ", "[", 
      RowBox[{"evspecs", ",", 
       RowBox[{"EventSpecifications", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"_", ",", "False", ",", "___"}], "}"}], ",", "___"}], 
        "]"}]}], "]"}]}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "newstatus", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"newstatus", "=", 
        RowBox[{"{", 
         RowBox[{
         "checkstatus", ",", "True", ",", "nestliststatus", ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
           "True", ",", "stemp", ",", "stop", ",", "True", ",", 
            "morestandstatus"}], "}"}], ",", "morestatus"}], "}"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"(", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Verbose", "/.", 
             RowBox[{"{", "opts", "}"}]}], "/.", 
            RowBox[{"Options", "[", "EventSpecifications", "]"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"Print", "[", 
            RowBox[{
            "\"\<EventSpecifications: absorbing EventSpecifications arguments \
with no change in Chronology: \>\"", ",", 
             RowBox[{
              RowBox[{"Expand", "[", "evspecs", "]"}], "/.", 
              RowBox[{"s_ShapeFunction", "\[RuleDelayed]", 
               RowBox[{"s", "\[LeftDoubleBracket]", 
                RowBox[{"{", "4", "}"}], "\[RightDoubleBracket]"}]}]}]}], 
            "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
          RowBox[{"newstatus", ",", "\[IndentingNewLine]", 
           RowBox[{"Replace", "[", "\[IndentingNewLine]", 
            RowBox[{"evspecs", ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"EventSpecifications", "[", 
               RowBox[{"True", ",", 
                RowBox[{"args", ":", 
                 RowBox[{
                  RowBox[{"Except", "[", "_Rule", "]"}], "..."}]}], ",", 
                "___Rule"}], "]"}], "\[RuleDelayed]", 
              RowBox[{"Sequence", "@@", "args"}]}], ",", 
             "\[IndentingNewLine]", "1"}], "\[IndentingNewLine]", "]"}], ",", 
           "\[IndentingNewLine]", "opts"}], "\[IndentingNewLine]", "]"}]}], 
        "\[IndentingNewLine]", ")"}]}], "\[IndentingNewLine]", ")"}]}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6845182481006193`*^9, 3.68451828843286*^9}, {
   3.684518328848041*^9, 3.684518355640546*^9}, {3.684518396687233*^9, 
   3.6845184558904057`*^9}, {3.684518494771653*^9, 3.684518518210699*^9}, {
   3.68468913279709*^9, 3.6846891957901087`*^9}, {3.684689817116561*^9, 
   3.684689819849725*^9}, {3.6910440149044743`*^9, 3.6910440641706133`*^9}, {
   3.691044993632266*^9, 3.6910449980293217`*^9}, {3.691046427392967*^9, 
   3.69104658974467*^9}, 3.691047387265213*^9, {3.691050787475651*^9, 
   3.691050852990609*^9}, {3.691232756192234*^9, 3.6912327714454107`*^9}, {
   3.691232804119555*^9, 3.69123281632303*^9}, {3.692173016161251*^9, 
   3.692173031750238*^9}, {3.6921731981317787`*^9, 3.692173272527672*^9}, 
   3.692257849800712*^9, {3.692260085653174*^9, 3.692260115164401*^9}, {
   3.692263759716744*^9, 3.692263764937585*^9}, 
   3.69920476514196*^9},ExpressionUUID->"373b4955-7a29-4842-b64e-\
a3791ffb1899"],

Cell["\<\
If the Chronology is inconsistent between all of the contained and enclosing \
EventSpecifications objects, the chronological ordering of the contained \
objects must be reversed before absorption. \
\>", "Text",
 CellChangeTimes->{{3.692173334996591*^9, 3.6921733924574223`*^9}, {
  3.692174021811273*^9, 3.692174056638322*^9}, {3.692174285274632*^9, 
  3.6921742893843803`*^9}, {3.692259348779264*^9, 3.6922594338309317`*^9}, {
  3.692259470149781*^9, 3.692259492742064*^9}, {3.692260138476424*^9, 
  3.6922601454740458`*^9}, {3.6922602307634783`*^9, 
  3.692260231087494*^9}},ExpressionUUID->"3c5dd2b0-cd24-471b-a97c-\
a55a294d0e90"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"status", ":", 
       RowBox[{"{", 
        RowBox[{"checkstatus_", ",", "False", ",", "nestliststatus_", ",", 
         RowBox[{"{", 
          RowBox[{"True", ",", "morestandstatus___"}], "}"}], ",", 
         "morestatus___"}], "}"}]}], ",", "\[IndentingNewLine]", 
      "evspecs_List", ",", "\[IndentingNewLine]", "opts___Rule"}], 
     "\[IndentingNewLine]", "]"}], "/;", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Count", "[", 
       RowBox[{"evspecs", ",", 
        RowBox[{"EventSpecifications", "[", 
         RowBox[{
          RowBox[{"True", "|", 
           RowBox[{"{", 
            RowBox[{"_", ",", "True", ",", "___"}], "}"}]}], ",", "___"}], 
         "]"}], ",", 
        RowBox[{"{", "1", "}"}]}], "]"}], "\[GreaterEqual]", "1"}], ")"}]}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "newstatus", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"newstatus", "=", 
        RowBox[{"{", 
         RowBox[{
         "checkstatus", ",", "False", ",", "nestliststatus", ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"True", ",", "morestandstatus"}], "}"}], ",", 
          "morestatus"}], "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"(", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Verbose", "/.", 
             RowBox[{"{", "opts", "}"}]}], "/.", 
            RowBox[{"Options", "[", "EventSpecifications", "]"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"Print", "[", 
            RowBox[{
            "\"\<EventSpecifications: absorbing EventSpecifications arguments \
with reversed chronology (case 1): \>\"", ",", 
             RowBox[{
              RowBox[{"Expand", "[", "evspecs", "]"}], "/.", 
              RowBox[{"s_ShapeFunction", "\[RuleDelayed]", 
               RowBox[{"s", "\[LeftDoubleBracket]", 
                RowBox[{"{", "4", "}"}], "\[RightDoubleBracket]"}]}]}]}], 
            "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
          RowBox[{"status", ",", "\[IndentingNewLine]", 
           RowBox[{"Replace", "[", "\[IndentingNewLine]", 
            RowBox[{"evspecs", ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"EventSpecifications", "[", 
               RowBox[{
                RowBox[{"True", "|", 
                 RowBox[{"{", 
                  RowBox[{"_", ",", "True", ",", "___"}], "}"}]}], ",", 
                RowBox[{"args", ":", 
                 RowBox[{
                  RowBox[{"Except", "[", "_Rule", "]"}], "..."}]}], ",", 
                "___Rule"}], "]"}], "\[RuleDelayed]", 
              RowBox[{"Sequence", "@@", 
               RowBox[{"Reverse", "[", "args", "]"}]}]}], ",", 
             "\[IndentingNewLine]", "1"}], "\[IndentingNewLine]", "]"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"SimplifyRules", "[", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"Chronology", "\[Rule]", "\"\<LeftToRight\>\""}], ",", 
             "\[IndentingNewLine]", "opts"}], "\[IndentingNewLine]", "]"}]}], 
          "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", ")"}]}], 
      "\[IndentingNewLine]", ")"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6845182481006193`*^9, 3.68451828843286*^9}, {
   3.684518328848041*^9, 3.684518355640546*^9}, {3.684518396687233*^9, 
   3.6845184558904057`*^9}, {3.684518494771653*^9, 3.684518518210699*^9}, {
   3.68468913279709*^9, 3.6846891957901087`*^9}, {3.684689817116561*^9, 
   3.684689819849725*^9}, {3.6910440149044743`*^9, 3.6910440641706133`*^9}, {
   3.691044993632266*^9, 3.6910449980293217`*^9}, {3.691046427392967*^9, 
   3.69104658974467*^9}, 3.691047387265213*^9, {3.691050787475651*^9, 
   3.691050852990609*^9}, {3.691232756192234*^9, 3.6912327714454107`*^9}, {
   3.691232804119555*^9, 3.69123281632303*^9}, {3.692173016161251*^9, 
   3.692173031750238*^9}, {3.6921731981317787`*^9, 3.692173272527672*^9}, 
   3.692257849800712*^9, {3.692259508878325*^9, 3.692259623715167*^9}, {
   3.692259797780181*^9, 3.6922598353998203`*^9}, {3.692259897269882*^9, 
   3.6922599196055727`*^9}, {3.6922600087282248`*^9, 
   3.6922600094113007`*^9}, {3.6922601573939123`*^9, 
   3.6922601655413113`*^9}, {3.692260221443603*^9, 3.69226025651598*^9}, {
   3.692263726548678*^9, 3.692263755041842*^9}, {3.692264286214054*^9, 
   3.692264303975366*^9}, {3.692264372851573*^9, 3.692264374127887*^9}, 
   3.699204773012381*^9},ExpressionUUID->"c10af689-4121-453c-8095-\
061cf5221930"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"status", ":", 
       RowBox[{"{", 
        RowBox[{"checkstatus_", ",", "True", ",", "nestliststatus_", ",", 
         RowBox[{"{", 
          RowBox[{"True", ",", "morestandstatus___"}], "}"}], ",", 
         "morestatus___"}], "}"}]}], ",", "\[IndentingNewLine]", 
      "evspecs_List", ",", "\[IndentingNewLine]", "opts___Rule"}], 
     "\[IndentingNewLine]", "]"}], "/;", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Count", "[", 
       RowBox[{"evspecs", ",", 
        RowBox[{"EventSpecifications", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"_", ",", "False", ",", "___"}], "}"}], ",", "___"}], 
         "]"}], ",", 
        RowBox[{"{", "1", "}"}]}], "]"}], "\[GreaterEqual]", "1"}], ")"}]}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "newstatus", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"newstatus", "=", 
        RowBox[{"{", 
         RowBox[{
         "checkstatus", ",", "True", ",", "nestliststatus", ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"True", ",", "morestandstatus"}], "}"}], ",", 
          "morestatus"}], "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"(", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Verbose", "/.", 
             RowBox[{"{", "opts", "}"}]}], "/.", 
            RowBox[{"Options", "[", "EventSpecifications", "]"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"Print", "[", 
            RowBox[{
            "\"\<EventSpecifications: absorbing EventSpecifications arguments \
with reversed chronology (case 2): \>\"", ",", 
             RowBox[{
              RowBox[{"Expand", "[", "evspecs", "]"}], "/.", 
              RowBox[{"s_ShapeFunction", "\[RuleDelayed]", 
               RowBox[{"s", "\[LeftDoubleBracket]", 
                RowBox[{"{", "4", "}"}], "\[RightDoubleBracket]"}]}]}]}], 
            "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
          RowBox[{"status", ",", "\[IndentingNewLine]", 
           RowBox[{"Replace", "[", "\[IndentingNewLine]", 
            RowBox[{"evspecs", ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"EventSpecifications", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"_", ",", "False", ",", "___"}], "}"}], ",", 
                RowBox[{"args", ":", 
                 RowBox[{
                  RowBox[{"Except", "[", "_Rule", "]"}], "..."}]}], ",", 
                "___Rule"}], "]"}], "\[RuleDelayed]", 
              RowBox[{"Sequence", "@@", 
               RowBox[{"Reverse", "[", "args", "]"}]}]}], ",", 
             "\[IndentingNewLine]", "1"}], "\[IndentingNewLine]", "]"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"SimplifyRules", "[", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"Chronology", "\[Rule]", "\"\<RightToLeft\>\""}], ",", 
             "\[IndentingNewLine]", "opts"}], "\[IndentingNewLine]", "]"}]}], 
          "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", ")"}]}], 
      "\[IndentingNewLine]", ")"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6845182481006193`*^9, 3.68451828843286*^9}, {
   3.684518328848041*^9, 3.684518355640546*^9}, {3.684518396687233*^9, 
   3.6845184558904057`*^9}, {3.684518494771653*^9, 3.684518518210699*^9}, {
   3.68468913279709*^9, 3.6846891957901087`*^9}, {3.684689817116561*^9, 
   3.684689819849725*^9}, {3.6910440149044743`*^9, 3.6910440641706133`*^9}, {
   3.691044993632266*^9, 3.6910449980293217`*^9}, {3.691046427392967*^9, 
   3.69104658974467*^9}, 3.691047387265213*^9, {3.691050787475651*^9, 
   3.691050852990609*^9}, {3.691232756192234*^9, 3.6912327714454107`*^9}, {
   3.691232804119555*^9, 3.69123281632303*^9}, {3.692173016161251*^9, 
   3.692173031750238*^9}, {3.6921731981317787`*^9, 3.692173272527672*^9}, 
   3.692257849800712*^9, {3.692259508878325*^9, 3.692259623715167*^9}, {
   3.692259797780181*^9, 3.6922598353998203`*^9}, {3.692259897269882*^9, 
   3.6922599196055727`*^9}, {3.6922599731984043`*^9, 3.692259995600931*^9}, {
   3.692260174796476*^9, 3.692260193656986*^9}, 3.692260225619813*^9, {
   3.692260262611643*^9, 3.692260267284006*^9}, {3.692263746220923*^9, 
   3.6922637502641277`*^9}, {3.69226431936677*^9, 3.692264328012444*^9}, 
   3.692264378818413*^9, 
   3.699204780355987*^9},ExpressionUUID->"f69a9378-830c-47d0-8472-\
7043344d3d20"]
}, Closed]],

Cell[CellGroupData[{

Cell["standardize all events", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683731796279813*^9, 3.6837318185361633`*^9}, {
   3.683786383040831*^9, 3.683786387901951*^9}, 3.684236524413398*^9, {
   3.6844993000857162`*^9, 3.684499311334092*^9}, {3.684501574808803*^9, 
   3.6845016023012457`*^9}, {3.684501893615694*^9, 3.684501894571189*^9}, {
   3.6845026864505987`*^9, 3.6845026886198874`*^9}, {3.684503215984846*^9, 
   3.684503220427644*^9}, {3.68451883025848*^9, 3.68451883254591*^9}, {
   3.691054596473194*^9, 3.691054602631547*^9}, 
   3.691054879290168*^9},ExpressionUUID->"64be09ce-ae9d-44bf-a330-\
a50e319133e8"],

Cell["\<\
set status to show that all events have been standardized (except in the case \
where an embedded EventSpecifications object is created). \
\>", "Text",
 CellChangeTimes->{{3.691054615127749*^9, 3.691054624399317*^9}, {
  3.6910548810734673`*^9, 
  3.691054903782198*^9}},ExpressionUUID->"613f7e19-cf83-461b-8ba2-\
beb31339a254"],

Cell["\<\
the FurtherEventSpecificationPatterns option of EventSpecifications may be \
set outside this package in order to implement new EventSpecifications \
definitions. This is done in the package Events`PulseSequence\
\>", "Text",
 CellChangeTimes->{{3.6919418208418293`*^9, 3.691941894025577*^9}, {
  3.692171053567739*^9, 
  3.6921710634123774`*^9}},ExpressionUUID->"337f35ca-1ba6-4b7d-bd86-\
2775315f00de"],

Cell["\<\
when an EventList is the argument, the chronology must be handled carefully. \
\
\>", "Text",
 CellChangeTimes->{{3.6922630586040916`*^9, 
  3.692263078861133*^9}},ExpressionUUID->"7e2e4c63-5446-4282-9b66-\
452e74b8cd31"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"status", ":", 
      RowBox[{"{", 
       RowBox[{
       "checkstatus_", ",", "chronstatus_", ",", "nestliststatus_", ",", 
        RowBox[{"{", 
         RowBox[{
         "True", ",", "stemp_", ",", "stop_", ",", "ststr_", ",", "False"}], 
         "}"}], ",", "morestatus___"}], "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"evspecs__", ",", "evspecsopts___Rule"}], "}"}], ",", 
     "opts___Rule"}], "\[IndentingNewLine]", "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"stdevspecs", ",", "newstandstatus", ",", "fevspecpatts"}], 
      "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Verbose", "/.", 
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "EventSpecifications", "]"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<EventSpecifications: standardizing event specifications: \>\"",
            ",", 
           RowBox[{
            RowBox[{"Expand", "@", 
             RowBox[{"{", 
              RowBox[{"evspecs", ",", "evspecsopts"}], "}"}]}], "/.", 
            RowBox[{"s_ShapeFunction", "\[RuleDelayed]", 
             RowBox[{"s", "\[LeftDoubleBracket]", 
              RowBox[{"{", "4", "}"}], "\[RightDoubleBracket]"}]}]}]}], 
          "]"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"fevspecpatts", "=", 
        RowBox[{"Alternatives", "@@", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"FurtherEventSpecificationPatterns", "/.", 
            RowBox[{"{", "opts", "}"}]}], "/.", 
           RowBox[{"Options", "[", "EventSpecifications", "]"}]}], ")"}]}]}], 
       ";", "\[IndentingNewLine]", 
       RowBox[{"stdevspecs", "=", 
        RowBox[{"Sequence", "@@", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Function", "[", 
            RowBox[{"evspec", ",", "\[IndentingNewLine]", 
             RowBox[{"Switch", "[", 
              RowBox[{"evspec", ",", "\[IndentingNewLine]", 
               RowBox[{"_", "?", "StandardizedEventSpecificationQ"}], ",", 
               "evspec", ",", "\[IndentingNewLine]", "fevspecpatts", ",", 
               "\[IndentingNewLine]", "\t", 
               RowBox[{"StandardizeEventSpecification", "[", 
                RowBox[{"evspec", ",", "opts"}], "]"}], ",", 
               "\[IndentingNewLine]", "NonStandardEventSpecificationPattern", 
               ",", "\[IndentingNewLine]", "\t", 
               RowBox[{"StandardizeEventSpecification", "[", 
                RowBox[{"evspec", ",", "opts"}], "]"}], ",", 
               "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"EventSpecifications", "[", 
                 RowBox[{"True", ",", "___"}], "]"}], "|", 
                RowBox[{"EventSpecifications", "[", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"_", ",", "_", ",", "_", ",", "True", ",", "___"}],
                    "}"}], ",", "___"}], "]"}]}], ",", "\[IndentingNewLine]", 
               "\t", "evspec", ",", "\[IndentingNewLine]", "_List", ",", 
               RowBox[{"EventSpecifications", "[", 
                RowBox[{"False", ",", "evspec", ",", 
                 RowBox[{"SimplifyRules", "[", 
                  RowBox[{"evspecsopts", ",", "opts"}], "]"}]}], "]"}], ",", 
               "\[IndentingNewLine]", "_Event", ",", 
               RowBox[{"evspec", "/.", 
                RowBox[{
                 RowBox[{"Event", "[", 
                  RowBox[{"_", ",", "_", ",", 
                   RowBox[{"spec", ":", 
                    RowBox[{"Except", "[", "_Rule", "]"}]}], ",", "___Rule"}],
                   "]"}], "\[RuleDelayed]", "spec"}]}], ",", 
               "\[IndentingNewLine]", "_EventList", ",", 
               RowBox[{"evspec", "/.", 
                RowBox[{
                 RowBox[{"EventList", "[", 
                  RowBox[{"_", ",", "_", ",", "specs_List", ",", "___Rule"}], 
                  "]"}], "\[RuleDelayed]", "\[IndentingNewLine]", 
                 RowBox[{"If", "[", 
                  RowBox[{"chronstatus", ",", 
                   RowBox[{"EventSpecifications", "/@", "specs"}], ",", 
                   RowBox[{"EventSpecifications", "/@", 
                    RowBox[{"Reverse", "[", "specs", "]"}]}]}], "]"}]}]}], 
               ",", "\[IndentingNewLine]", "_", ",", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"Message", "[", 
                  RowBox[{
                   RowBox[{"EventSpecifications", "::", "inval"}], ",", 
                   "evspec"}], "]"}], ";", "$Failed"}], ")"}]}], 
              "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}], "/@", 
           RowBox[{"{", "evspecs", "}"}]}], ")"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"newstandstatus", "=", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"MemberQ", "[", 
           RowBox[{
            RowBox[{"{", "stdevspecs", "}"}], ",", 
            RowBox[{"EventSpecifications", "[", 
             RowBox[{
              RowBox[{"Except", "[", "True", "]"}], ",", "___"}], "]"}]}], 
           "]"}], ",", "False", ",", "True"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
          "checkstatus", ",", "chronstatus", ",", "nestliststatus", ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{
            "True", ",", "False", ",", "False", ",", "False", ",", 
             "newstandstatus"}], "}"}], ",", "\[IndentingNewLine]", 
           "morestatus"}], "}"}], ",", "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"stdevspecs", ",", 
           RowBox[{"SimplifyRules", "[", 
            RowBox[{"evspecsopts", ",", "opts"}], "]"}]}], "}"}], ",", 
         "\[IndentingNewLine]", "opts"}], "\[IndentingNewLine]", "]"}]}], 
      "\[IndentingNewLine]", ")"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6910546688015823`*^9, 3.691054711842752*^9}, {
   3.691054777532371*^9, 3.691054783858348*^9}, {3.6910548739604397`*^9, 
   3.691054874160668*^9}, {3.691054907969832*^9, 3.691054973542884*^9}, {
   3.6910550392294064`*^9, 3.6910551086174*^9}, {3.691055167911872*^9, 
   3.691055264011476*^9}, {3.691063142235083*^9, 3.691063168871542*^9}, {
   3.691063201450345*^9, 3.691063202526889*^9}, {3.6912325973393908`*^9, 
   3.6912325986645327`*^9}, {3.691941763325441*^9, 3.691941774103088*^9}, {
   3.6919418118593187`*^9, 3.6919418158663597`*^9}, 3.6919418816035767`*^9, {
   3.691947883674865*^9, 3.691947886537409*^9}, {3.691948664859962*^9, 
   3.6919487330282707`*^9}, 3.692171030630838*^9, {3.692257208892096*^9, 
   3.6922572173871307`*^9}, {3.6922631391924973`*^9, 3.692263164948689*^9}, 
   3.69920479057183*^9, 
   3.7004850088800287`*^9},ExpressionUUID->"fb32791d-6168-4cea-b5e3-\
756296b39812"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["implement RtoL chronology", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.513611021384453*^9, 3.513611111096521*^9}, {
  3.513611362301363*^9, 3.513611398542715*^9}, {3.5141093248467007`*^9, 
  3.514109331669417*^9}, {3.683729356423153*^9, 3.683729365332357*^9}, {
  3.6845043569716578`*^9, 
  3.68450435970543*^9}},ExpressionUUID->"228fb6c7-4440-4481-bb8d-\
911a9cdf2603"],

Cell["\<\
invert the Chronology if required. 
The intended value of Chronology is specified as an input to the routine.
Indicate that standard chronology has been implemented (if necessary by \
inverting the order of elements) by using the flag \
StandardizedChronologyImplemented.\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.545052388798958*^9, 3.545052419393093*^9}, {
  3.6803444810387297`*^9, 3.680344509139575*^9}, {3.680344685072041*^9, 
  3.680344753347446*^9}},ExpressionUUID->"51f8096e-94b3-44de-af43-\
2e5bbc17c133"],

Cell[CellGroupData[{

Cell["standardize chronology status", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683729040586273*^9, 3.6837290508666983`*^9}, {
  3.683731076062901*^9, 3.6837310800779457`*^9}, {3.683785969833952*^9, 
  3.683785975417405*^9}},ExpressionUUID->"9c0a021e-55c2-4be3-81ac-\
320a7a144a24"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"check_", ",", "False", ",", "otherstatus___"}], "}"}], ",", 
     "evspecs_", ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"check", ",", 
       RowBox[{"{", 
        RowBox[{"False", ",", "False"}], "}"}], ",", "otherstatus"}], "}"}], 
     ",", "evspecs", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6837308882288647`*^9, 3.683730923889195*^9}, {
  3.683786165666356*^9, 3.683786171266078*^9}, {3.6839576813838177`*^9, 
  3.683957681385903*^9}},ExpressionUUID->"ed20430b-9728-41f6-8a2e-\
abdd54d005ee"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"check_", ",", 
       RowBox[{"{", 
        RowBox[{"True", ",", "True"}], "}"}], ",", "otherstatus___"}], "}"}], 
     ",", "evspecs_", ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"check", ",", "True", ",", "otherstatus"}], "}"}], ",", 
     "evspecs", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6837308882288647`*^9, 3.683730923889195*^9}, {
  3.683731055079211*^9, 3.683731063405858*^9}, {3.683786177440505*^9, 
  3.683786183511106*^9}, {3.683957681393269*^9, 
  3.683957681395379*^9}},ExpressionUUID->"4b0836b0-0892-4045-94a7-\
4ff92aecba20"]
}, Closed]],

Cell[CellGroupData[{

Cell["implement RtoL chronology if not done", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683729040586273*^9, 
  3.6837290508666983`*^9}},ExpressionUUID->"8287af4e-1776-40b9-8883-\
8a3082a6d238"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"check_", ",", 
       RowBox[{"chron", ":", 
        RowBox[{"{", 
         RowBox[{"False", ",", "chrondone_"}], "}"}]}], ",", 
       "otherstatus___"}], "}"}], ",", "evspecs_", ",", "opts___Rule"}], 
    "]"}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"check", ",", 
       RowBox[{"{", 
        RowBox[{"True", ",", "False"}], "}"}], ",", "otherstatus"}], "}"}], 
     ",", "evspecs", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6837289248499727`*^9, 3.683729002157889*^9}, {
  3.683729108394034*^9, 3.683729109152411*^9}, {3.683730987286901*^9, 
  3.6837309875434227`*^9}, {3.683786192331918*^9, 3.683786196247691*^9}, {
  3.683957681402843*^9, 
  3.683957681405039*^9}},ExpressionUUID->"08fdc227-fc89-4f7a-883a-\
4315185ee65b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"check_", ",", 
       RowBox[{"chron", ":", 
        RowBox[{"{", 
         RowBox[{"True", ",", "False"}], "}"}]}], ",", "otherstatus___"}], 
      "}"}], ",", 
     RowBox[{"evspecs", ":", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"_", "?", "StandardizedEventSpecificationQ"}], "|", 
          "_EventSpecifications"}], ")"}], "..."}], "}"}]}], ",", 
     "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Verbose", "/.", 
         RowBox[{"{", "opts", "}"}]}], "/.", 
        RowBox[{"Options", "[", "EventSpecifications", "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Print", "[", 
        RowBox[{
        "\"\<EventSpecifications: implementing RtoL Chronology:\>\"", ",", 
         " ", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"chron", ",", "evspecs"}], "}"}], "/.", 
          RowBox[{"s_ShapeFunction", "\[RuleDelayed]", 
           RowBox[{"s", "\[LeftDoubleBracket]", 
            RowBox[{"{", "4", "}"}], "\[RightDoubleBracket]"}]}]}]}], "]"}]}],
       "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"check", ",", "True", ",", "otherstatus"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Switch", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Chronology", "/.", 
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "EventSpecifications", "]"}]}], ",", 
         "\[IndentingNewLine]", "\"\<RightToLeft\>\"", ",", "evspecs", ",", 
         "\[IndentingNewLine]", "\"\<LeftToRight\>\"", ",", 
         RowBox[{"Reverse", "[", "evspecs", "]"}], ",", "\[IndentingNewLine]",
          "_", ",", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Message", "[", 
            RowBox[{"EventSpecifications", "::", "invchron"}], "]"}], ";", 
           "$Failed"}], ")"}]}], "\[IndentingNewLine]", "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"SimplifyRules", "[", 
        RowBox[{
         RowBox[{"Chronology", "\[Rule]", "\"\<RightToLeft\>\""}], ",", 
         "opts"}], "]"}]}], "\[IndentingNewLine]", "]"}]}], ")"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5141093437376013`*^9, 3.514109492229361*^9}, {
   3.514109707642164*^9, 3.514109826340163*^9}, 3.514109894776071*^9, 
   3.5141100726990128`*^9, {3.514110738042583*^9, 3.514110804428212*^9}, {
   3.514111244625819*^9, 3.514111247231327*^9}, 3.514111561834921*^9, {
   3.514907346422619*^9, 3.5149074307012053`*^9}, {3.5149074674487762`*^9, 
   3.514907537430416*^9}, {3.514952270508398*^9, 3.514952274833396*^9}, {
   3.5450510859781103`*^9, 3.545051097728586*^9}, 3.545051400629643*^9, {
   3.545052700194055*^9, 3.545052753970426*^9}, {3.545052817969309*^9, 
   3.545052818686075*^9}, {3.5878162721704893`*^9, 3.587816508550103*^9}, {
   3.6803447749395657`*^9, 3.6803447765709352`*^9}, {3.683707737317865*^9, 
   3.6837077790031843`*^9}, {3.683707975940206*^9, 3.6837079786821623`*^9}, {
   3.683708587693303*^9, 3.6837085991727543`*^9}, {3.683728906660502*^9, 
   3.683728912416422*^9}, {3.683729092053643*^9, 3.683729093963409*^9}, {
   3.683729125457597*^9, 3.683729159797277*^9}, {3.683729215115305*^9, 
   3.6837292172112427`*^9}, {3.6837293050280733`*^9, 3.683729342577808*^9}, {
   3.683729404613503*^9, 3.683729405369857*^9}, {3.683786200627432*^9, 
   3.683786211007935*^9}, {3.683957681413913*^9, 3.683957681423921*^9}, {
   3.691044307661799*^9, 3.6910443270034018`*^9}, {3.691868100358436*^9, 
   3.691868107275984*^9}, {3.691868182970077*^9, 3.691868301247732*^9}, {
   3.692263928816752*^9, 3.692263941652747*^9}, 3.699204800412578*^9, 
   3.700485008886949*^9},ExpressionUUID->"233afebb-c3a5-4e72-89bf-\
bcdc28078f85"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["nested lists", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.454567394439436*^9, 3.45456739938997*^9}, {
   3.454576414690864*^9, 3.4545764315130777`*^9}, {3.454576579278468*^9, 
   3.454576580716839*^9}, {3.455336663472224*^9, 3.4553366732854643`*^9}, {
   3.455452365031452*^9, 3.455452384820578*^9}, {3.464256930105167*^9, 
   3.464256938583194*^9}, {3.464257382327674*^9, 3.464257387747345*^9}, {
   3.464272043354971*^9, 3.464272045792259*^9}, 3.464275608036623*^9, {
   3.464331333001854*^9, 3.464331334992557*^9}, 3.5343880033241243`*^9, {
   3.6837294633814993`*^9, 3.683729467171728*^9}, {3.683729698252934*^9, 
   3.683729700912363*^9}, {3.6837298176331873`*^9, 3.683729821495585*^9}, {
   3.684500789022656*^9, 3.684500792901903*^9}, {3.684500926792489*^9, 
   3.6845009335979156`*^9}},ExpressionUUID->"02de2996-4bcc-4328-8e02-\
434a063f67ae"],

Cell[CellGroupData[{

Cell["nest list status flags", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683729040586273*^9, 3.6837290508666983`*^9}, {
  3.683731076062901*^9, 3.6837310800779457`*^9}, {3.684499560409563*^9, 
  3.684499561251005*^9}, {3.684500941327606*^9, 
  3.684500942890552*^9}},ExpressionUUID->"fda99d7e-eb4d-4716-833f-\
481cee6b7eba"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "checkstatus_", ",", "chronstatus_", ",", "False", ",", 
       "morestatus___"}], "}"}], ",", "evspecs_", ",", "opts___Rule"}], "]"}],
    ":=", "\[IndentingNewLine]", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"checkstatus", ",", "chronstatus", ",", 
       RowBox[{"{", 
        RowBox[{"False", ",", "False"}], "}"}], ",", "morestatus"}], "}"}], 
     ",", "evspecs", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.68372974910354*^9, 3.6837298122967987`*^9}, {
  3.683730957983409*^9, 3.683730976165638*^9}, {3.6837862249136744`*^9, 
  3.6837862372310467`*^9}, {3.6839576814318*^9, 3.683957681433938*^9}, {
  3.684500492712467*^9, 3.684500503567472*^9}, {3.684500957585704*^9, 
  3.684500968386181*^9}},ExpressionUUID->"74451bd8-f5e4-4db4-ad4b-\
ec25eec8fc12"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"checkstatus_", ",", "chronstatus_", ",", 
       RowBox[{"{", 
        RowBox[{"True", ",", "True"}], "}"}], ",", "morestatus___"}], "}"}], 
     ",", "evspecs_", ",", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "checkstatus", ",", "chronstatus", ",", "True", ",", "morestatus"}], 
      "}"}], ",", "evspecs", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.68372974910354*^9, 3.6837298122967987`*^9}, {
  3.683730957983409*^9, 3.683730976165638*^9}, {3.683731114649835*^9, 
  3.683731118572782*^9}, {3.683786241936226*^9, 3.6837862477577677`*^9}, {
  3.683957681441441*^9, 3.683957681443549*^9}, {3.684500511164007*^9, 
  3.684500521140523*^9}, {3.684500974337401*^9, 
  3.684500977867872*^9}},ExpressionUUID->"2a537976-e29a-4484-9582-\
9c1a5140c38c"]
}, Closed]],

Cell[CellGroupData[{

Cell["handle nested list", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683729040586273*^9, 3.6837290508666983`*^9}, {
  3.683731076062901*^9, 3.6837310800779457`*^9}, {3.684499560409563*^9, 
  3.684499561251005*^9}, {3.684500941327606*^9, 3.684500942890552*^9}, {
  3.6845009982089157`*^9, 
  3.684501002111836*^9}},ExpressionUUID->"06e49fc0-f7c9-431c-9d48-\
b6a4aa2bfccc"],

Cell[CellGroupData[{

Cell["initialize status ", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683731796279813*^9, 3.683731808833954*^9}, {
  3.684499589310541*^9, 3.6844995934721403`*^9}, {3.684503995377922*^9, 
  3.684503996882471*^9}},ExpressionUUID->"de92505d-d136-414d-a764-\
d97828becc98"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"checkstatus_", ",", "chronstatus_", ",", 
       RowBox[{"{", 
        RowBox[{"False", ",", "_"}], "}"}], ",", "morestatus___"}], "}"}], 
     ",", "evspecs_", ",", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"checkstatus", ",", "chronstatus", ",", 
       RowBox[{"{", 
        RowBox[{"True", ",", "False"}], "}"}], ",", "morestatus"}], "}"}], 
     ",", "evspecs", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.68372974910354*^9, 3.6837298122967987`*^9}, 
   3.683731364566168*^9, {3.6837866498436213`*^9, 3.683786654643095*^9}, {
   3.68395768145128*^9, 3.683957681453465*^9}, {3.684500534868494*^9, 
   3.684500542426063*^9}, {3.684501022442316*^9, 
   3.684501025670109*^9}},ExpressionUUID->"b2f41a8d-3cc5-4e46-abf7-\
ca2b48c62e5a"]
}, Closed]],

Cell[CellGroupData[{

Cell["case of no internal list", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683731796279813*^9, 3.683731808833954*^9}, 
   3.684499706629848*^9, {3.6845010445372458`*^9, 
   3.6845010516353188`*^9}},ExpressionUUID->"6b928a63-8bd0-40b3-8643-\
482957d9398b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"status", ":", 
      RowBox[{"{", 
       RowBox[{"checkstatus_", ",", "chronstatus_", ",", 
        RowBox[{"{", 
         RowBox[{"True", ",", "False"}], "}"}], ",", "morestatus___"}], 
       "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"evspecs", ":", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"_", "?", "StandardizedEventSpecificationQ"}], ".."}], 
       "}"}]}], ",", "opts___Rule"}], "\[IndentingNewLine]", "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Verbose", "/.", 
         RowBox[{"{", "opts", "}"}]}], "/.", 
        RowBox[{"Options", "[", "EventSpecifications", "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
         "Print", "[", 
          "\"\<EventSpecifications: no internal list detected.\>\"", "]"}], 
         ";", "\[IndentingNewLine]", 
         RowBox[{"Print", "[", 
          RowBox[{"status", ",", 
           RowBox[{
            RowBox[{"Expand", "@", "evspecs"}], "/.", 
            RowBox[{"s_ShapeFunction", "\[RuleDelayed]", 
             RowBox[{"s", "\[LeftDoubleBracket]", 
              RowBox[{"{", "4", "}"}], "\[RightDoubleBracket]"}]}]}]}], 
          "]"}]}], ")"}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"EventSpecifications", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
        "checkstatus", ",", "chronstatus", ",", "True", ",", "morestatus"}], 
        "}"}], ",", "evspecs", ",", "opts"}], "]"}]}], "\[IndentingNewLine]", 
    ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6837317177710323`*^9, 3.6837317802412367`*^9}, {
   3.683786263193097*^9, 3.683786317896369*^9}, {3.6839576814612217`*^9, 
   3.683957681463418*^9}, {3.684236000841113*^9, 3.684236003141459*^9}, 
   3.684236075727487*^9, {3.684500549156082*^9, 3.684500549795033*^9}, {
   3.684501066399737*^9, 3.684501130651496*^9}, {3.691045933408235*^9, 
   3.691045949002883*^9}, {3.691046116922306*^9, 3.6910461282964697`*^9}, {
   3.691232511939397*^9, 3.6912325131285477`*^9}, 3.699204808554385*^9, {
   3.6992049910690107`*^9, 3.6992049963066053`*^9}, {3.699205050164021*^9, 
   3.6992050615041237`*^9}, 
   3.700485008893573*^9},ExpressionUUID->"a7de85dc-3056-465c-bb97-\
6a397741ff0a"]
}, Closed]],

Cell[CellGroupData[{

Cell["set status ", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683731796279813*^9, 3.683731808833954*^9}, {
  3.684499589310541*^9, 3.6844995934721403`*^9}, {3.684503995377922*^9, 
  3.68450401150795*^9}},ExpressionUUID->"4f893a2b-2fe0-4423-aa96-\
72d6b734e01c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"checkstatus_", ",", "chronstatus_", ",", 
        RowBox[{"{", 
         RowBox[{"True", ",", "False"}], "}"}], ",", "morestatus___"}], "}"}],
       ",", "evspecs_", ",", "opts___Rule"}], "]"}], "/;", 
    RowBox[{"FreeQ", "[", 
     RowBox[{
      RowBox[{"{", "evspecs", "}"}], ",", "_EventSpecifications", ",", "1"}], 
     "]"}]}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"checkstatus", ",", "chronstatus", ",", 
       RowBox[{"{", 
        RowBox[{"True", ",", "True"}], "}"}], ",", "morestatus"}], "}"}], ",",
      "evspecs", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.68372974910354*^9, 3.6837298122967987`*^9}, 
   3.683731364566168*^9, {3.6837866498436213`*^9, 3.683786654643095*^9}, {
   3.68395768145128*^9, 3.683957681453465*^9}, {3.684500534868494*^9, 
   3.684500542426063*^9}, {3.684501022442316*^9, 3.684501025670109*^9}, {
   3.68450401661812*^9, 
   3.684504139434766*^9}},ExpressionUUID->"68ddfc19-6db0-4f64-87d7-\
348fa925f08e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"checkstatus_", ",", "chronstatus_", ",", 
        RowBox[{"{", 
         RowBox[{"True", ",", "False"}], "}"}], ",", "morestatus___"}], "}"}],
       ",", "evspecs_", ",", "opts___Rule"}], "]"}], "/;", 
    RowBox[{"FreeQ", "[", 
     RowBox[{
      RowBox[{"{", "evspecs", "}"}], ",", 
      RowBox[{"EventSpecifications", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"_", ",", "_", ",", 
          RowBox[{"False", "|", 
           RowBox[{"{", 
            RowBox[{"_", ",", "False"}], "}"}]}], ",", "___"}], "}"}], ",", 
        "___"}], "]"}], ",", "1"}], "]"}]}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"checkstatus", ",", "chronstatus", ",", 
       RowBox[{"{", 
        RowBox[{"True", ",", "True"}], "}"}], ",", "morestatus"}], "}"}], ",",
      "evspecs", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.68372974910354*^9, 3.6837298122967987`*^9}, 
   3.683731364566168*^9, {3.6837866498436213`*^9, 3.683786654643095*^9}, {
   3.68395768145128*^9, 3.683957681453465*^9}, {3.684500534868494*^9, 
   3.684500542426063*^9}, {3.684501022442316*^9, 3.684501025670109*^9}, {
   3.68450401661812*^9, 3.684504139434766*^9}, {3.684504179419127*^9, 
   3.684504230595785*^9}},ExpressionUUID->"a9912b59-ed3e-484b-9fcc-\
776e0f5c2609"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["handle Repeat", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.454567394439436*^9, 3.45456739938997*^9}, {
   3.454576414690864*^9, 3.4545764315130777`*^9}, {3.454576579278468*^9, 
   3.454576580716839*^9}, {3.455336663472224*^9, 3.4553366732854643`*^9}, {
   3.455452365031452*^9, 3.455452384820578*^9}, {3.464256930105167*^9, 
   3.464256938583194*^9}, {3.464257382327674*^9, 3.464257387747345*^9}, {
   3.464272043354971*^9, 3.464272045792259*^9}, 3.464275608036623*^9, {
   3.464331333001854*^9, 3.464331334992557*^9}, {3.464954372674386*^9, 
   3.464954383209238*^9}, {3.465623295389319*^9, 3.4656232981701717`*^9}, 
   3.465623514429843*^9, {3.4671424571852293`*^9, 3.467142459225038*^9}, {
   3.5079696721581697`*^9, 3.507969674651121*^9}, {3.514110882622114*^9, 
   3.514110883418701*^9}, {3.5141125829785137`*^9, 3.5141125917454367`*^9}, {
   3.514118975379033*^9, 3.5141189832335587`*^9}, {3.6833680028864803`*^9, 
   3.68336800532656*^9}, {3.683732549913217*^9, 3.6837325521019583`*^9}, {
   3.683780495142355*^9, 3.683780506708066*^9}, {3.6844999019030113`*^9, 
   3.684499905343403*^9}, {3.684774959471526*^9, 3.684774961091578*^9}, 
   3.6848632302726603`*^9, 3.684946401267552*^9, 
   3.692848307749258*^9},ExpressionUUID->"fe7a76f9-4979-44b5-8fb7-\
fa719f7e1b55"],

Cell[CellGroupData[{

Cell["omit zero repeats", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.454567394439436*^9, 3.45456739938997*^9}, {
   3.454576414690864*^9, 3.4545764315130777`*^9}, {3.454576579278468*^9, 
   3.454576580716839*^9}, {3.455336663472224*^9, 3.4553366732854643`*^9}, {
   3.455452365031452*^9, 3.455452384820578*^9}, {3.464256930105167*^9, 
   3.464256938583194*^9}, {3.464257382327674*^9, 3.464257387747345*^9}, {
   3.464272043354971*^9, 3.464272045792259*^9}, 3.464275608036623*^9, {
   3.464331333001854*^9, 3.464331334992557*^9}, {3.464954372674386*^9, 
   3.464954383209238*^9}, {3.465623295389319*^9, 3.4656232981701717`*^9}, 
   3.465623514429843*^9, {3.4671424571852293`*^9, 3.467142459225038*^9}, {
   3.5079696721581697`*^9, 3.507969674651121*^9}, {3.514110882622114*^9, 
   3.514110883418701*^9}, {3.5141125829785137`*^9, 3.5141125917454367`*^9}, {
   3.514118975379033*^9, 
   3.5141189832335587`*^9}},ExpressionUUID->"e6b2e57a-6bea-4955-8f83-\
5dbe434f41ac"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"EventSpecifications", "[", 
     RowBox[{"status_List", ",", 
      RowBox[{"{", 
       RowBox[{"a___", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Repeat", "[", 
           RowBox[{"repevs_", ",", "0", ",", "___Rule"}], "]"}], ",", 
          "repevopts___Rule"}], "}"}], ",", "b___"}], "}"}], ",", 
      "opts___Rule"}], "]"}]}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{"False", ",", 
     RowBox[{"{", 
      RowBox[{"a", ",", "b"}], "}"}], ",", "opts"}], "]"}]}], ";"}]], "Input",\

 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.464331350651689*^9, 3.464331390691472*^9}, {
   3.464954395919595*^9, 3.4649544471102324`*^9}, 3.4649718854934*^9, {
   3.465623322957295*^9, 3.4656233561896477`*^9}, {3.465623393027063*^9, 
   3.4656233940097322`*^9}, {3.465623427629793*^9, 3.465623439109494*^9}, {
   3.4656235759227543`*^9, 3.465623661269508*^9}, {3.465623870360927*^9, 
   3.465623975094227*^9}, 3.465624121564168*^9, {3.465624200691575*^9, 
   3.4656242011032333`*^9}, 3.4657315349193993`*^9, 3.465736861682335*^9, {
   3.465737664331317*^9, 3.465737672143135*^9}, {3.467142492777875*^9, 
   3.4671424959987087`*^9}, {3.507969779293845*^9, 3.507969789442181*^9}, {
   3.507969846851676*^9, 3.5079699288773823`*^9}, {3.507986485834978*^9, 
   3.507986564426889*^9}, {3.507988157847456*^9, 3.50798818181115*^9}, {
   3.507988347851799*^9, 3.507988353731371*^9}, {3.507988582918777*^9, 
   3.507988638447109*^9}, {3.5079891121864653`*^9, 3.5079891204599733`*^9}, {
   3.508306852463457*^9, 3.5083068977603827`*^9}, {3.508308003721632*^9, 
   3.50830800565412*^9}, {3.514108795460669*^9, 3.5141088629193563`*^9}, {
   3.514110399594993*^9, 3.514110402308083*^9}, 3.514111268777712*^9, 
   3.514111545072719*^9, {3.5141126000881367`*^9, 3.514112698406157*^9}, {
   3.5141127338734493`*^9, 3.5141127369261*^9}, {3.514113042771422*^9, 
   3.514113043232748*^9}, {3.5141189873920717`*^9, 3.5141190170225697`*^9}, {
   3.68373256107705*^9, 3.683732580029711*^9}, {3.683957681500987*^9, 
   3.6839576815031652`*^9}, {3.684498533907523*^9, 3.68449854192704*^9}, 
   3.684498623584684*^9, {3.6847762329711103`*^9, 3.684776234046583*^9}, 
   3.6849464012787952`*^9, {3.691676691963439*^9, 3.6916766950323133`*^9}, 
   3.69284830941024*^9},ExpressionUUID->"cbbece38-b9bc-4a95-b6c2-\
18251d93e804"]
}, Closed]],

Cell[CellGroupData[{

Cell["implement one repeat explicitly", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.454567394439436*^9, 3.45456739938997*^9}, {
   3.454576414690864*^9, 3.4545764315130777`*^9}, {3.454576579278468*^9, 
   3.454576580716839*^9}, {3.455336663472224*^9, 3.4553366732854643`*^9}, {
   3.455452365031452*^9, 3.455452384820578*^9}, {3.464256930105167*^9, 
   3.464256938583194*^9}, {3.464257382327674*^9, 3.464257387747345*^9}, {
   3.464272043354971*^9, 3.464272045792259*^9}, 3.464275608036623*^9, {
   3.464331333001854*^9, 3.464331334992557*^9}, {3.464954372674386*^9, 
   3.464954383209238*^9}, {3.465623295389319*^9, 3.4656232981701717`*^9}, 
   3.465623514429843*^9, {3.4671424571852293`*^9, 3.467142459225038*^9}, {
   3.5079696721581697`*^9, 3.507969674651121*^9}, {3.514110882622114*^9, 
   3.514110883418701*^9}, {3.5141125829785137`*^9, 3.5141125917454367`*^9}, {
   3.514118975379033*^9, 3.5141189832335587`*^9}, {3.684498555461791*^9, 
   3.6844985626612787`*^9}, {3.684775891586214*^9, 
   3.6847758949051933`*^9}},ExpressionUUID->"5de6a9c0-24b4-4a4c-8ea2-\
1e6a6425951c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"EventSpecifications", "[", 
     RowBox[{"status_List", ",", 
      RowBox[{"{", 
       RowBox[{"a___", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Repeat", "[", 
           RowBox[{"repevs_", ",", "1", ",", "___Rule"}], "]"}], ",", 
          "repevopts___Rule"}], "}"}], ",", "b___"}], "}"}], ",", 
      "opts___Rule"}], "]"}]}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{"False", ",", 
     RowBox[{"{", 
      RowBox[{"a", ",", 
       RowBox[{"{", 
        RowBox[{"repevs", ",", "repevopts"}], "}"}], ",", "b"}], "}"}], ",", 
     "opts"}], "]"}]}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.464331350651689*^9, 3.464331390691472*^9}, {
   3.464954395919595*^9, 3.4649544471102324`*^9}, 3.4649718854934*^9, {
   3.465623322957295*^9, 3.4656233561896477`*^9}, {3.465623393027063*^9, 
   3.4656233940097322`*^9}, {3.465623427629793*^9, 3.465623439109494*^9}, {
   3.4656235759227543`*^9, 3.465623661269508*^9}, {3.465623870360927*^9, 
   3.465623975094227*^9}, 3.465624121564168*^9, {3.465624200691575*^9, 
   3.4656242011032333`*^9}, 3.4657315349193993`*^9, 3.465736861682335*^9, {
   3.465737664331317*^9, 3.465737672143135*^9}, {3.467142492777875*^9, 
   3.4671424959987087`*^9}, {3.507969779293845*^9, 3.507969789442181*^9}, {
   3.507969846851676*^9, 3.5079699288773823`*^9}, {3.507986485834978*^9, 
   3.507986564426889*^9}, {3.507988157847456*^9, 3.50798818181115*^9}, {
   3.507988347851799*^9, 3.507988353731371*^9}, {3.507988582918777*^9, 
   3.507988638447109*^9}, {3.5079891121864653`*^9, 3.5079891204599733`*^9}, {
   3.508306852463457*^9, 3.5083068977603827`*^9}, {3.508308003721632*^9, 
   3.50830800565412*^9}, {3.514108795460669*^9, 3.5141088629193563`*^9}, {
   3.514110399594993*^9, 3.514110402308083*^9}, 3.514111268777712*^9, 
   3.514111545072719*^9, {3.5141126000881367`*^9, 3.514112698406157*^9}, {
   3.5141127338734493`*^9, 3.5141127369261*^9}, {3.514113042771422*^9, 
   3.514113043232748*^9}, {3.5141189873920717`*^9, 3.5141190170225697`*^9}, {
   3.68373256107705*^9, 3.683732580029711*^9}, {3.683957681500987*^9, 
   3.6839576815031652`*^9}, {3.684498533907523*^9, 3.6844986379160833`*^9}, {
   3.684775797647594*^9, 3.684775838693502*^9}, {3.684775872452839*^9, 
   3.68477587795243*^9}, 3.684946401285142*^9, 3.6916767053579283`*^9, 
   3.6928483104392233`*^9},ExpressionUUID->"e8c4d971-8480-4e6b-b322-\
08a64019337a"]
}, Closed]],

Cell[CellGroupData[{

Cell["ExpandRepeatedEvents", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.454567394439436*^9, 3.45456739938997*^9}, {
   3.454576414690864*^9, 3.4545764315130777`*^9}, {3.454576579278468*^9, 
   3.454576580716839*^9}, {3.455336663472224*^9, 3.4553366732854643`*^9}, {
   3.455452365031452*^9, 3.455452384820578*^9}, {3.464256930105167*^9, 
   3.464256938583194*^9}, {3.464257382327674*^9, 3.464257387747345*^9}, {
   3.464272043354971*^9, 3.464272045792259*^9}, 3.464275608036623*^9, {
   3.464331333001854*^9, 3.464331334992557*^9}, {3.464954372674386*^9, 
   3.464954383209238*^9}, {3.465623295389319*^9, 3.4656232981701717`*^9}, 
   3.465623514429843*^9, {3.4671424571852293`*^9, 3.467142459225038*^9}, {
   3.5079696721581697`*^9, 3.507969674651121*^9}, {3.514110882622114*^9, 
   3.514110883418701*^9}, {3.5141125829785137`*^9, 3.5141125917454367`*^9}, {
   3.683733760497828*^9, 
   3.683733764403296*^9}},ExpressionUUID->"8a7c02e4-c1e8-489c-8bab-\
858a37612f29"],

Cell[CellGroupData[{

Cell["standardize status flags", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683729040586273*^9, 3.6837290508666983`*^9}, {
  3.683731076062901*^9, 3.6837310800779457`*^9}, {3.6912325226747293`*^9, 
  3.691232525764634*^9}},ExpressionUUID->"49e1425d-bd92-4fe5-802e-\
ad2479805247"],

Cell["\<\
the status flags for ExpandRepeatedEvents are: {<implement>, <completed for \
numeric n>}. \
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.6837399882221823`*^9, 3.683740048952195*^9}, {
  3.68378052447642*^9, 3.6837805409145813`*^9}, {3.683780698539259*^9, 
  3.683780713391652*^9}},ExpressionUUID->"6d291b02-1714-4dfc-aa84-\
ea9f063a38c4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "checkstatus_", ",", "chronstatus_", ",", "nestliststatus_", ",", 
       "standstatus_", ",", "False", ",", "otherstatus___"}], "}"}], ",", 
     "evspecs_", ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "checkstatus", ",", "chronstatus", ",", "nestliststatus", ",", 
       "standstatus", ",", 
       RowBox[{"{", 
        RowBox[{"False", ",", "False"}], "}"}], ",", "otherstatus"}], "}"}], 
     ",", "evspecs", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6837308882288647`*^9, 3.683730923889195*^9}, {
  3.6837337971350107`*^9, 3.683733835857036*^9}, {3.683739981415185*^9, 
  3.6837399828369923`*^9}, {3.683780519402964*^9, 3.6837805197951527`*^9}, {
  3.6837866849389133`*^9, 3.683786689331113*^9}, {3.6839576815113773`*^9, 
  3.683957681513542*^9}, {3.6845008025471573`*^9, 
  3.684500808394864*^9}},ExpressionUUID->"a4cb3036-4d46-4ebf-8036-\
9d4478250d05"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "checkstatus_", ",", "chronstatus_", ",", "nestliststatus_", ",", 
       "standstatus_", ",", 
       RowBox[{"{", 
        RowBox[{"True", ",", "True"}], "}"}], ",", "otherstatus___"}], "}"}], 
     ",", "evspecs_", ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "checkstatus", ",", "chronstatus", ",", "nestliststatus", ",", 
       "standstatus", ",", "True", ",", "otherstatus"}], "}"}], ",", 
     "evspecs", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6837308882288647`*^9, 3.683730923889195*^9}, {
  3.683731055079211*^9, 3.683731063405858*^9}, {3.6837338495635242`*^9, 
  3.683733865631022*^9}, {3.683740055089244*^9, 3.683740056134583*^9}, {
  3.68378054664366*^9, 3.683780546906335*^9}, {3.6837866949981537`*^9, 
  3.68378670024218*^9}, {3.6839576815211153`*^9, 3.6839576815232773`*^9}, {
  3.684500813415258*^9, 
  3.68450082143408*^9}},ExpressionUUID->"3fd54889-51a0-49c8-9953-\
2bb282c8cef4"]
}, Closed]],

Cell[CellGroupData[{

Cell["expand Repeat, where possible", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683729040586273*^9, 3.6837290508666983`*^9}, {
   3.683731076062901*^9, 3.6837310800779457`*^9}, {3.683733879752816*^9, 
   3.6837338961104937`*^9}, {3.6837344639954147`*^9, 3.683734465274839*^9}, {
   3.683735230715642*^9, 3.68373524350346*^9}, {3.683735523006228*^9, 
   3.683735526078505*^9}, 3.690554814575576*^9, 
   3.692848311460709*^9},ExpressionUUID->"591d0724-128b-41bf-a15a-\
5ae9b4b7aa89"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "checkstatus_", ",", "chronstatus_", ",", "nestliststatus_", ",", 
       "standstatus_", ",", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"False", ",", "False"}], "}"}], "|", "False"}], ",", 
       "otherstatus___"}], "}"}], ",", "evspecs_", ",", "opts___Rule"}], 
    "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "checkstatus", ",", "chronstatus", ",", "nestliststatus", ",", 
       "standstatus", ",", 
       RowBox[{"{", 
        RowBox[{"True", ",", "False"}], "}"}], ",", "otherstatus"}], "}"}], 
     ",", "\[IndentingNewLine]", "evspecs", ",", "opts"}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683733948765671*^9, 3.683734051248674*^9}, {
   3.683740085098158*^9, 3.683740094176468*^9}, {3.6837805726079407`*^9, 
   3.683780572830449*^9}, {3.6837814254395113`*^9, 3.683781439060135*^9}, {
   3.683786707460331*^9, 3.683786713083033*^9}, {3.6839576815314713`*^9, 
   3.683957681535738*^9}, {3.684500831584468*^9, 3.684500839130877*^9}, 
   3.684775490831417*^9, {3.6847772886028223`*^9, 3.684777308213643*^9}, {
   3.684777828912703*^9, 3.684777830300064*^9}, 
   3.68486121913371*^9},ExpressionUUID->"6f53eea6-3c23-4b90-9ced-\
ad884d8ac1c9"],

Cell["\<\
The chronological ordering of the events must be handled very carefully upon \
expansion. \
\>", "Text",
 CellChangeTimes->{{3.692261214542804*^9, 3.692261231397791*^9}, {
  3.692261784174653*^9, 3.6922618252039127`*^9}, {3.692261889107654*^9, 
  3.6922618895387897`*^9}},ExpressionUUID->"1cb96aef-8fac-4e16-8eaa-\
5e094e7fdcbe"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"evspecst", ":", 
      RowBox[{"{", 
       RowBox[{
       "checkstatus_", ",", "chronstatus_", ",", "nestliststatus_", ",", 
        "standstatus_", ",", 
        RowBox[{"{", 
         RowBox[{"True", ",", "False"}], "}"}], ",", "otherstatus___"}], 
       "}"}]}], ",", "\[IndentingNewLine]", "evspecs_List", ",", 
     "\[IndentingNewLine]", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"(", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Verbose", "/.", 
         RowBox[{"{", "opts", "}"}]}], "/.", 
        RowBox[{"Options", "[", "EventSpecifications", "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<EventSpecifications: handling Repeat: \>\"", ",", 
         RowBox[{
          RowBox[{"Expand", "@", 
           RowBox[{"{", 
            RowBox[{"evspecst", ",", "evspecs"}], "}"}]}], "/.", 
          RowBox[{"s_ShapeFunction", "\[RuleDelayed]", 
           RowBox[{"s", "\[LeftDoubleBracket]", 
            RowBox[{"{", "4", "}"}], "\[RightDoubleBracket]"}]}]}]}], "]"}]}],
       "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"EventSpecifications", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
        "False", ",", "chronstatus", ",", "False", ",", "False", ",", "True", 
         ",", "otherstatus"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"evspecs", "/.", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{"evsa___", ",", 
            RowBox[{"Repeat", "[", 
             RowBox[{
              RowBox[{"EventList", "[", 
               RowBox[{"_", ",", "_", ",", 
                RowBox[{"events", ":", 
                 RowBox[{"{", 
                  RowBox[{"_Event", ".."}], "}"}]}], ",", 
                "evlistopts___Rule"}], "]"}], ",", "n_Integer", ",", 
              "repevopts___Rule"}], "]"}], ",", "evsb___", ",", 
            "evsopts___Rule"}], "}"}], "/;", 
          RowBox[{"TrueQ", "[", 
           RowBox[{
            RowBox[{"ExpandRepeatedEvents", "/.", 
             RowBox[{"{", 
              RowBox[{"repevopts", ",", "evsopts"}], "}"}]}], "/.", 
            RowBox[{"Options", "[", "EventSpecifications", "]"}]}], "]"}]}], ":>",
          "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"evsa", ",", "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Sequence", "@@", 
              RowBox[{"If", "[", 
               RowBox[{"chronstatus", ",", 
                RowBox[{"Table", "[", 
                 RowBox[{"events", ",", 
                  RowBox[{"{", "n", "}"}]}], "]"}], ",", 
                RowBox[{"Table", "[", 
                 RowBox[{
                  RowBox[{"Reverse", "@", "events"}], ",", 
                  RowBox[{"{", "n", "}"}]}], "]"}]}], "]"}]}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"Sequence", "@@", 
              RowBox[{"Cases", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"evlistopts", ",", "repevopts"}], "}"}], ",", 
                RowBox[{"Except", "@", 
                 RowBox[{"HoldPattern", "[", 
                  RowBox[{"ExpandRepeatedEvents", "\[Rule]", "_"}], "]"}]}]}],
                "]"}]}]}], "\[IndentingNewLine]", "}"}], ",", 
           "\[IndentingNewLine]", "evsb", ",", "\[IndentingNewLine]", 
           RowBox[{"Sequence", "@@", 
            RowBox[{"Cases", "[", 
             RowBox[{
              RowBox[{"{", "evsopts", "}"}], ",", 
              RowBox[{"Except", "@", 
               RowBox[{"HoldPattern", "[", 
                RowBox[{"ExpandRepeatedEvents", "\[Rule]", "_"}], "]"}]}]}], 
             "]"}]}]}], "\[IndentingNewLine]", "}"}]}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Sequence", "@@", 
        RowBox[{"Cases", "[", 
         RowBox[{
          RowBox[{"{", "opts", "}"}], ",", 
          RowBox[{"Except", "@", 
           RowBox[{"HoldPattern", "[", 
            RowBox[{"ExpandRepeatedEvents", "\[Rule]", "_"}], "]"}]}]}], 
         "]"}]}]}], "\[IndentingNewLine]", "]"}]}], ")"}]}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.464331350651689*^9, 3.464331390691472*^9}, {
   3.464954395919595*^9, 3.4649544471102324`*^9}, 3.4649718854934*^9, {
   3.465623322957295*^9, 3.4656233561896477`*^9}, {3.465623393027063*^9, 
   3.4656233940097322`*^9}, {3.465623427629793*^9, 3.465623439109494*^9}, {
   3.4656235759227543`*^9, 3.465623661269508*^9}, {3.465623870360927*^9, 
   3.465623975094227*^9}, 3.465624121564168*^9, {3.465624200691575*^9, 
   3.4656242011032333`*^9}, 3.4657315349193993`*^9, 3.465736861682335*^9, {
   3.465737664331317*^9, 3.465737672143135*^9}, {3.467142492777875*^9, 
   3.4671424959987087`*^9}, {3.507969779293845*^9, 3.507969789442181*^9}, {
   3.507969846851676*^9, 3.5079699288773823`*^9}, {3.507986485834978*^9, 
   3.507986564426889*^9}, {3.507988157847456*^9, 3.50798818181115*^9}, {
   3.507988347851799*^9, 3.507988353731371*^9}, {3.507988582918777*^9, 
   3.507988638447109*^9}, {3.5079891121864653`*^9, 3.5079891204599733`*^9}, {
   3.508306852463457*^9, 3.5083068977603827`*^9}, {3.508308003721632*^9, 
   3.50830800565412*^9}, {3.514108795460669*^9, 3.5141088629193563`*^9}, {
   3.514110399594993*^9, 3.514110402308083*^9}, 3.514111268777712*^9, 
   3.514111545072719*^9, {3.5141126000881367`*^9, 3.514112698406157*^9}, {
   3.5141127338734493`*^9, 3.5141127369261*^9}, {3.514113042771422*^9, 
   3.514113043232748*^9}, {3.514720952062441*^9, 3.514720968916376*^9}, {
   3.683732624807906*^9, 3.683732794350946*^9}, {3.683732839441291*^9, 
   3.6837329589713573`*^9}, {3.6837334071208153`*^9, 3.683733414999709*^9}, 
   3.683733579623356*^9, 3.683733904213375*^9, {3.683734066550164*^9, 
   3.683734176606429*^9}, {3.683734539651016*^9, 3.68373456242211*^9}, {
   3.683734767616281*^9, 3.683734769655095*^9}, {3.6837352529452057`*^9, 
   3.683735300490799*^9}, {3.683735554930455*^9, 3.6837355602323227`*^9}, {
   3.6837360503916197`*^9, 3.683736145688335*^9}, {3.6837371147761517`*^9, 
   3.6837371208591413`*^9}, {3.683737214756555*^9, 3.68373721534553*^9}, {
   3.683737245988504*^9, 3.683737248063332*^9}, {3.6837401595956507`*^9, 
   3.6837401909260817`*^9}, {3.683780625778605*^9, 3.683780626820674*^9}, {
   3.683780680179459*^9, 3.683780680457473*^9}, {3.683780732817223*^9, 
   3.68378075005914*^9}, {3.683780900351728*^9, 3.6837809146643248`*^9}, {
   3.68378127270151*^9, 3.683781276895348*^9}, {3.683786728461651*^9, 
   3.683786733392218*^9}, 3.683788494627544*^9, {3.683957681554191*^9, 
   3.683957681556403*^9}, {3.684500867608219*^9, 3.684500882044211*^9}, {
   3.684776709625655*^9, 3.6847767701840477`*^9}, {3.6847768156780252`*^9, 
   3.684776883788517*^9}, {3.684777440308989*^9, 3.684777626131138*^9}, 
   3.684861688716671*^9, {3.684861748808196*^9, 3.6848617561014967`*^9}, {
   3.684861955050424*^9, 3.684861956353427*^9}, 3.684946401291222*^9, {
   3.6904577921739817`*^9, 3.690457902657148*^9}, {3.691044228089447*^9, 
   3.691044246400804*^9}, {3.6912350086866198`*^9, 3.691235064271204*^9}, {
   3.692261307072125*^9, 3.692261362477861*^9}, {3.692261764801565*^9, 
   3.6922617777373037`*^9}, {3.6928483130210333`*^9, 3.6928483138817377`*^9}, 
   3.699204826179512*^9},ExpressionUUID->"a6c87318-0a6b-4468-b7ca-\
3e23b366e259"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["implement overall check status", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.513611021384453*^9, 3.513611111096521*^9}, {
  3.513611362301363*^9, 3.513611398542715*^9}, {3.5141093248467007`*^9, 
  3.514109331669417*^9}, {3.683729356423153*^9, 3.683729365332357*^9}, {
  3.68378599082388*^9, 3.683785994309032*^9}, {3.683786157607451*^9, 
  3.6837861589209213`*^9}},ExpressionUUID->"5011e495-cd25-4819-8ee2-\
ed923e1fbaa2"],

Cell[CellGroupData[{

Cell["standardize check status", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683729040586273*^9, 3.6837290508666983`*^9}, {
  3.683731076062901*^9, 3.6837310800779457`*^9}, {3.683785969833952*^9, 
  3.683785975417405*^9}, {3.683786032167142*^9, 
  3.683786032808161*^9}},ExpressionUUID->"c6b436d8-5917-48b6-b299-\
c13449903b93"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "otherstatus___"}], "}"}], ",", "evspecs_", ",", 
     "opts___Rule"}], "]"}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"False", ",", "False"}], "}"}], ",", "otherstatus"}], "}"}], 
     ",", "evspecs", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6837308882288647`*^9, 3.683730923889195*^9}, {
  3.683957681345313*^9, 
  3.683957681347443*^9}},ExpressionUUID->"e8588fd7-3537-4d82-9a67-\
dc324e9a4803"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"True", ",", "True"}], "}"}], ",", "otherstatus___"}], "}"}], 
     ",", "evspecs_", ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"True", ",", "otherstatus"}], "}"}], ",", "evspecs", ",", 
     "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6837308882288647`*^9, 3.683730923889195*^9}, {
  3.683731055079211*^9, 3.683731063405858*^9}, {3.6839576813547287`*^9, 
  3.6839576813568373`*^9}},ExpressionUUID->"26d274cb-33c2-40bb-b3c6-\
15e9c4fb2033"]
}, Closed]],

Cell[CellGroupData[{

Cell["implement check if not done", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683729040586273*^9, 3.6837290508666983`*^9}, {
  3.683731076062901*^9, 3.6837310800779457`*^9}, {3.683785969833952*^9, 
  3.683785975417405*^9}, {3.683786036057438*^9, 
  3.683786047959963*^9}},ExpressionUUID->"cd7ab4f7-e7a9-49e6-aeb6-\
839412530044"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"False", ",", "False"}], "}"}], ",", "otherstatus___"}], 
      "}"}], ",", "evspecs_", ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"True", ",", "False"}], "}"}], ",", "otherstatus"}], "}"}], 
     ",", "evspecs", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6837308882288647`*^9, 3.683730923889195*^9}, {
  3.683786052800995*^9, 3.6837860594551*^9}, {3.683957681364263*^9, 
  3.683957681366395*^9}},ExpressionUUID->"8e0325f8-d08a-4b6a-840b-\
ff9e2c5d33e7"]
}, Closed]],

Cell[CellGroupData[{

Cell["set to True if all checks complete", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683729040586273*^9, 3.6837290508666983`*^9}, {
  3.683731076062901*^9, 3.6837310800779457`*^9}, {3.683785969833952*^9, 
  3.683785975417405*^9}, {3.683786036057438*^9, 3.683786047959963*^9}, {
  3.6845037383984413`*^9, 
  3.684503744956023*^9}},ExpressionUUID->"058bf942-bb10-483f-b1bd-\
e4ba51b98161"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"True", ",", "False"}], "}"}], ",", 
       RowBox[{"otherstatus", ":", 
        RowBox[{"True", "..."}]}]}], "}"}], ",", "evspecs_", ",", 
     "opts___Rule"}], "]"}], ":=", 
   RowBox[{"EventSpecifications", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"True", ",", "True"}], "}"}], ",", "otherstatus"}], "}"}], 
     ",", "evspecs", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6837308882288647`*^9, 3.683730923889195*^9}, {
  3.683786052800995*^9, 3.6837860594551*^9}, {3.683957681364263*^9, 
  3.683957681366395*^9}, {3.684503750145482*^9, 
  3.684503765095335*^9}},ExpressionUUID->"f07c5966-e353-479c-a096-\
9572905f6f7a"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["messages", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5079889997116957`*^9, 
  3.507989000926252*^9}},ExpressionUUID->"6e86f818-eff8-439c-9a1a-\
64d240c24f61"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "::", "inval"}], "=", 
   "\"\<The event specification <`1`> is invalid.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.534388740131811*^9, 3.534388755106008*^9}, {
   3.5652653987385883`*^9, 3.565265404254051*^9}, {3.683807774725223*^9, 
   3.683807779035404*^9}, 3.683957681575632*^9, {3.6847323793205967`*^9, 
   3.6847323966102457`*^9}, {3.6915954175271587`*^9, 
   3.691595419748551*^9}},ExpressionUUID->"e387baec-8106-4e86-86a2-\
960b9fc387e6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "::", "invchron"}], "=", 
   "\"\<Invalid value for Chronology.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5079891776898746`*^9, 3.507989188886891*^9}, {
   3.5083436347551603`*^9, 3.50834363865984*^9}, {3.509963761258123*^9, 
   3.5099637628638563`*^9}, 3.514108874662147*^9, {3.514109997585967*^9, 
   3.514110015392241*^9}, 
   3.6839576815826807`*^9},ExpressionUUID->"6f2bda63-d220-4815-b248-\
b7179997ba6d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventSpecifications", "::", "chronconflict"}], "=", 
   "\"\<An inconsistency has arisen between the chronological ordering of two \
or more EventSpecifications objects contained within another \
EventSpecifications object.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.69217309022932*^9, 
  3.69217314416982*^9}},ExpressionUUID->"a4ffcff0-e53d-4975-9706-\
ee25c205d721"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DivideEventSpecifications", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.625294213407867*^9, 3.625294266936805*^9}, {
   3.625296478152717*^9, 3.625296563948032*^9}, {3.625297057398859*^9, 
   3.625297133406013*^9}, {3.625306909740706*^9, 3.625306918234168*^9}, {
   3.6253850889642963`*^9, 3.625385097994274*^9}, 3.625397189623513*^9, 
   3.6255656811317177`*^9, {3.625566966384778*^9, 3.6255669875008917`*^9}, {
   3.6255670567962723`*^9, 3.625567062074286*^9}, {3.625572414691482*^9, 
   3.625572431010044*^9}, 3.625941563348687*^9, 3.6260027343865643`*^9, 
   3.684732307069537*^9},ExpressionUUID->"964bc46f-cc50-42d0-a1fc-\
78c5c0e42b3b"],

Cell["\<\
DivideEventSpecifications[evspecs,intervals] takes an EventSpecifications (in \
standardized chronology) and divides it according to a list of arbitrary time \
intervals. If the option IncludeRepetitionNumbers\[Rule]True is used, the \
divided list has the form {...{nrep2,EventSpecifications[..]},{nrep1, \
EventSpecifications[..]}}, where {nrep1,nrep2..} are integers for the number \
of repetitions of the following event specs. This is used by (for example) \
the COMPUTE algorithm to calculate the propagator sets for a periodic system. \
In some cases the repetitions allows the division of the event specs to be \
carried out efficiently\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.625941358778493*^9, 3.625941472061922*^9}, {
   3.625941514899457*^9, 3.625941529408704*^9}, {3.625941687662161*^9, 
   3.625941694164027*^9}, {3.6260026486767187`*^9, 3.626002690564611*^9}, {
   3.626004120035371*^9, 3.6260041585439262`*^9}, {3.634048788843594*^9, 
   3.634048812018289*^9}, {3.6838064466507463`*^9, 3.6838065398742456`*^9}, {
   3.683807715168396*^9, 3.683807717525901*^9}, {3.6839576816913033`*^9, 
   3.6839576818021927`*^9}, 
   3.6847323071475687`*^9},ExpressionUUID->"970e0f98-ce33-44cc-89a7-\
0bc84bf6ead3"],

Cell[CellGroupData[{

Cell["DivideEventSpecifications", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.625294213407867*^9, 3.625294266936805*^9}, {
   3.625296478152717*^9, 3.625296563948032*^9}, {3.625297057398859*^9, 
   3.625297133406013*^9}, {3.625306909740706*^9, 3.625306918234168*^9}, {
   3.6253850889642963`*^9, 3.625385097994274*^9}, 3.625397189623513*^9, 
   3.6255656811317177`*^9, {3.625566966384778*^9, 3.6255669875008917`*^9}, {
   3.6255670567962723`*^9, 3.625567062074286*^9}, {3.625572414691482*^9, 
   3.625572431010044*^9}, 3.625941563348687*^9, 3.6260027343865643`*^9, {
   3.683290722838217*^9, 3.6832907267072067`*^9}, 3.6833691304721518`*^9, 
   3.6847323071640787`*^9},ExpressionUUID->"3a12f4b1-4796-446d-ba34-\
56bacbc4f761"],

Cell[CellGroupData[{

Cell["defaults", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683790528478595*^9, 3.6837905417999496`*^9}, {
  3.683790573119254*^9, 3.6837905783744593`*^9}, {3.683794230112472*^9, 
  3.683794233999213*^9}, {3.6837943261409187`*^9, 3.683794329836049*^9}, {
  3.683806551338529*^9, 
  3.683806552433531*^9}},ExpressionUUID->"990c6fcd-41de-499e-af64-\
7e14ed62448a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "DivideEventSpecifications", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"IncludeRepetitionNumbers", "\[Rule]", "False"}], "}"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683806555797768*^9, 3.683806574681102*^9}, 
   3.6847323071737127`*^9},ExpressionUUID->"282d42cf-2a2e-4774-aaef-\
661c0b92d58c"]
}, Open  ]],

Cell[CellGroupData[{

Cell["apply EventSpecifications", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683790528478595*^9, 3.6837905417999496`*^9}, {
   3.683790573119254*^9, 3.6837905783744593`*^9}, {3.683794230112472*^9, 
   3.683794233999213*^9}, {3.6837943261409187`*^9, 3.683794329836049*^9}, 
   3.683957681822329*^9},ExpressionUUID->"e3a06712-80af-4ad8-a169-\
9698ce3465f6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DivideEventSpecifications", "[", 
    RowBox[{
     RowBox[{"source", ":", 
      RowBox[{"Except", "[", "_EventSpecifications", "]"}]}], ",", 
     RowBox[{"more", ":", 
      RowBox[{
       RowBox[{"Except", "[", "_Rule", "]"}], "..."}]}], ",", "opts___Rule"}],
     "]"}], ":=", 
   RowBox[{"DivideEventSpecifications", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"EventSpecifications", "[", 
      RowBox[{"source", ",", 
       RowBox[{"SimplifyRules", "[", 
        RowBox[{
         RowBox[{"ExpandRepeatedEvents", "\[Rule]", "True"}], ",", "opts"}], 
        "]"}]}], "]"}], ",", "\[IndentingNewLine]", "more", ",", 
     "\[IndentingNewLine]", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.626002762736123*^9, 3.626002807637622*^9}, {
  3.6260030328685083`*^9, 3.626003050180716*^9}, {3.6260031494879637`*^9, 
  3.626003176579619*^9}, {3.62600418905107*^9, 3.626004194630075*^9}, {
  3.6839576818322*^9, 3.6839576818349133`*^9}, {3.6847323071817513`*^9, 
  3.68473230718427*^9}, {3.6849449938028*^9, 
  3.684945022273704*^9}},ExpressionUUID->"4c98df68-0fd1-4f21-affe-\
dd830d65e57e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DivideEventSpecifications", "[", 
    RowBox[{"evspecs_EventSpecifications", ",", 
     RowBox[{"more", ":", 
      RowBox[{
       RowBox[{"Except", "[", "_Rule", "]"}], "..."}]}], ",", "opts___Rule"}],
     "]"}], "/;", 
   RowBox[{"!", 
    RowBox[{"FreeQ", "[", 
     RowBox[{"evspecs", ",", "_Repeat"}], "]"}]}]}], ":=", 
  "\[IndentingNewLine]", 
  RowBox[{"DivideEventSpecifications", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"EventSpecifications", "[", 
     RowBox[{"evspecs", ",", 
      RowBox[{"SimplifyRules", "[", 
       RowBox[{
        RowBox[{"ExpandRepeatedEvents", "\[Rule]", "True"}], ",", "opts"}], 
       "]"}]}], "]"}], ",", "\[IndentingNewLine]", "more", ",", 
    "\[IndentingNewLine]", "opts"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.684945039327302*^9, 3.684945089442133*^9}, 
   3.684946401297173*^9, 
   3.6928483145643053`*^9},ExpressionUUID->"3bade84b-ba43-4231-93f1-\
3a0db18192e5"]
}, Open  ]],

Cell[CellGroupData[{

Cell["validity checks", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683790528478595*^9, 3.6837905417999496`*^9}, {
  3.683790573119254*^9, 3.6837905783744593`*^9}, {3.683794230112472*^9, 
  3.683794233999213*^9}},ExpressionUUID->"8798c4ea-4a16-4661-ba58-\
84c250af199f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DivideEventSpecifications", "[", 
    RowBox[{
     RowBox[{"EventSpecifications", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Except", "[", "True", "]"}], ",", "morestatus___"}], "}"}], 
       ",", "evspecs_List", ",", "evspeclistopts___Rule"}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"more", ":", 
      RowBox[{
       RowBox[{"Except", "[", "_Rule", "]"}], "..."}]}], ",", "opts___Rule"}],
     "]"}], ":=", 
   RowBox[{"DivideEventSpecifications", "[", 
    RowBox[{
     RowBox[{"EventSpecifications", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"True", ",", "False"}], "}"}], ",", "morestatus"}], "}"}], 
       ",", "evspecs", ",", "evspeclistopts"}], "]"}], ",", 
     "\[IndentingNewLine]", "more", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.68379044789176*^9, 3.68379052291299*^9}, {
  3.6839576818429413`*^9, 3.6839576818451777`*^9}, {3.684732307192082*^9, 
  3.6847323071943502`*^9}},ExpressionUUID->"008d9a65-d056-4e63-bcf9-\
4c4878549412"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DivideEventSpecifications", "[", 
     RowBox[{"evspeclist_EventSpecifications", ",", "\[IndentingNewLine]", 
      RowBox[{"more", ":", 
       RowBox[{
        RowBox[{"Except", "[", "_Rule", "]"}], "..."}]}], ",", 
      "opts___Rule"}], "]"}], "/;", 
    RowBox[{"!", 
     RowBox[{"NumericQ", "[", 
      RowBox[{"EventDuration", "[", "evspeclist", "]"}], "]"}]}]}], ":=", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Message", "[", 
      RowBox[{"DivideEventSpecifications", "::", "notnumdur"}], "]"}], ";", 
     "$Failed"}], ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683790628686512*^9, 3.683790683022483*^9}, 
   3.683790766310575*^9, {3.683808632090085*^9, 3.683808634877508*^9}, 
   3.6839576818527727`*^9, {3.684732307202104*^9, 3.6847323072042923`*^9}, 
   3.711716424573436*^9},ExpressionUUID->"fe4ee9f1-2312-467d-92da-\
9bf5a2ae58c0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DivideEventSpecifications", "[", 
     RowBox[{
      RowBox[{
      "evspeclist_EventSpecifications", "\[IndentingNewLine]", 
       "\[Tau]vals_List"}], ",", "opts___Rule"}], "]"}], "/;", 
    RowBox[{
     RowBox[{"Chop", "[", 
      RowBox[{
       RowBox[{"Plus", "@@", "\[Tau]vals"}], "-", 
       RowBox[{"EventDuration", "[", 
        RowBox[{"evspeclist", ",", 
         RowBox[{"Check", "\[Rule]", "False"}]}], "]"}]}], "]"}], 
     "\[NotEqual]", "0"}]}], ":=", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Message", "[", 
      RowBox[{"DivideEventSpecifications", "::", "invaltimes"}], "]"}], ";", 
     "$Failed"}], ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683805177182313*^9, 3.6838052687030163`*^9}, 
   3.683957681861554*^9, {3.6847323072124033`*^9, 3.684732307214855*^9}, 
   3.71171642457747*^9},ExpressionUUID->"35a8131a-15e0-4dc2-b5a3-\
8a1a964d3716"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DivideEventSpecifications", "[", 
    RowBox[{
     RowBox[{"_EventSpecifications", "\[IndentingNewLine]", "n_Integer"}], 
     ",", "opts___Rule"}], "]"}], "/;", 
   RowBox[{"n", "\[LessEqual]", "0"}]}], ":=", "\[IndentingNewLine]", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"Message", "[", 
     RowBox[{"DivideEventSpecifications", "::", "baddiv"}], "]"}], ";", 
    "$Failed"}], ")"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683805467635069*^9, 3.683805477729805*^9}, 
   3.683957681869052*^9, {3.68473230722278*^9, 
   3.6847323072251577`*^9}},ExpressionUUID->"c544bf46-0e19-436c-a9ed-\
c1dd94e2a5e5"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DivideEventSpecifications", "[", 
    RowBox[{
     RowBox[{"_EventSpecifications", "\[IndentingNewLine]", 
      RowBox[{"n", ":", 
       RowBox[{"Except", "[", 
        RowBox[{"_Integer", "|", "_List"}], "]"}]}]}], ",", "opts___Rule"}], 
    "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Message", "[", 
      RowBox[{"DivideEventSpecifications", "::", "baddiv"}], "]"}], ";", 
     "$Failed"}], ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.626003209904697*^9, 3.6260032720978394`*^9}, {
   3.626003452102154*^9, 3.6260035084282713`*^9}, {3.6260035914128523`*^9, 
   3.626003665251165*^9}, {3.626004203532*^9, 3.626004208107716*^9}, 
   3.626004337555956*^9, 3.683784232414151*^9, {3.683784319354116*^9, 
   3.683784331920197*^9}, {3.683790559199967*^9, 3.683790566878725*^9}, {
   3.6837941330564327`*^9, 3.683794222398044*^9}, {3.6837942742454844`*^9, 
   3.6837942797801247`*^9}, {3.683805149726081*^9, 3.683805151667095*^9}, {
   3.68380533753583*^9, 3.683805337847378*^9}, 3.683957681876802*^9, {
   3.6847323072331257`*^9, 
   3.684732307235471*^9}},ExpressionUUID->"f702515d-bc37-44c9-aafc-\
42a1b41d69ad"]
}, Open  ]],

Cell[CellGroupData[{

Cell["divide into equal intervals", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683790528478595*^9, 3.6837905417999496`*^9}, {
  3.683790573119254*^9, 3.683790615543734*^9}, {3.683805503718506*^9, 
  3.683805508100086*^9}},ExpressionUUID->"65cdf9b7-6d03-4960-882a-\
17a6199288b4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DivideEventSpecifications", "[", 
    RowBox[{
    "evspeclist_EventSpecifications", ",", "\[IndentingNewLine]", "n_Integer",
      ",", "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"DivideEventSpecifications", "[", 
    RowBox[{"evspeclist", ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"EventDuration", "[", 
         RowBox[{"evspeclist", ",", 
          RowBox[{"Check", "\[Rule]", "False"}]}], "]"}], "/", "n"}], ",", 
       RowBox[{"{", "n", "}"}]}], "]"}], ",", "\[IndentingNewLine]", "opts"}],
     "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6838055560800867`*^9, 3.6838056380452843`*^9}, {
   3.683806949654188*^9, 3.6838069527487717`*^9}, 3.683957681884714*^9, {
   3.6847323072433357`*^9, 3.684732307245699*^9}, {3.711438429312857*^9, 
   3.711438437872013*^9}, {3.711447054679145*^9, 3.711447055911484*^9}, {
   3.7117158825362434`*^9, 3.7117158875198255`*^9}, {3.711715957935263*^9, 
   3.711715959354731*^9}, {3.7117160015665083`*^9, 3.711716002595746*^9}, {
   3.71171614111625*^9, 3.7117161427556567`*^9}, 3.711716424577985*^9, {
   3.7117165437310624`*^9, 3.7117165438593955`*^9}, {3.711716986909975*^9, 
   3.711716991301097*^9}, {3.7117170684062696`*^9, 3.7117170692840548`*^9}, {
   3.711717108436229*^9, 3.711717108485858*^9}, {3.7117171998734007`*^9, 
   3.7117172036303177`*^9}, {3.7119617337438807`*^9, 3.711961735637726*^9}, {
   3.711961769271415*^9, 3.711961776382142*^9}, {3.7284683008060446`*^9, 
   3.728468312053748*^9}, {3.7284683605162334`*^9, 
   3.7284683620847793`*^9}},ExpressionUUID->"35f8ac68-0967-472e-9c45-\
01dce1e98010"]
}, Open  ]],

Cell[CellGroupData[{

Cell["main", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683790528478595*^9, 3.6837905417999496`*^9}, {
  3.683790573119254*^9, 3.683790615543734*^9}, {3.683805503718506*^9, 
  3.683805508100086*^9}, {3.6838055454994392`*^9, 
  3.683805545994834*^9}},ExpressionUUID->"88a2ac90-f4b2-4dfb-90d1-\
55fa0d832488"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DivideEventSpecifications", "[", "\[IndentingNewLine]", 
     RowBox[{"evspeclist_EventSpecifications", ",", "\[IndentingNewLine]", 
      RowBox[{"\[Tau]vals", ":", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"_", "?", "NumericQ"}], ".."}], "}"}]}], ",", 
      "opts___Rule"}], "]"}], "/;", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"IncludeRepetitionNumbers", "/.", 
       RowBox[{"{", "opts", "}"}]}], "/.", 
      RowBox[{"Options", "[", "DivideEventSpecifications", "]"}]}], ")"}]}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"AddRepetitionIndices", "@", 
    RowBox[{"DivideEventSpecifications", "[", 
     RowBox[{"evspeclist", ",", "\[Tau]vals", ",", "\[IndentingNewLine]", 
      RowBox[{"SimplifyRules", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"IncludeRepetitionNumbers", "\[Rule]", "False"}], ",", 
        "opts"}], "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", 
     "]"}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.626003209904697*^9, 3.6260032720978394`*^9}, {
   3.626003452102154*^9, 3.6260035084282713`*^9}, {3.6260035914128523`*^9, 
   3.626003665251165*^9}, {3.626004203532*^9, 3.626004208107716*^9}, 
   3.626004337555956*^9, 3.683784232414151*^9, {3.683784319354116*^9, 
   3.683784331920197*^9}, {3.683790559199967*^9, 3.683790566878725*^9}, {
   3.6837941330564327`*^9, 3.683794154233601*^9}, {3.6838056701997757`*^9, 
   3.683805687409685*^9}, 3.6838058639767313`*^9, {3.683806398474963*^9, 
   3.6838063990580673`*^9}, {3.68380675156421*^9, 3.683806810382923*^9}, 
   3.683957681892888*^9, {3.684732307253479*^9, 3.684732307258256*^9}, {
   3.7114384424719906`*^9, 3.7114384506386724`*^9}, {3.7114456536419234`*^9, 
   3.711445659621562*^9}, {3.711445695112608*^9, 3.7114456952604637`*^9}, {
   3.711447060582576*^9, 3.711447062002581*^9}, 
   3.711716738021652*^9},ExpressionUUID->"023025f0-7347-47d5-be10-\
874a7e4db287"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DivideEventSpecifications", "[", 
     RowBox[{
      RowBox[{"EventSpecifications", "[", 
       RowBox[{
        RowBox[{"status", ":", 
         RowBox[{"True", "|", 
          RowBox[{"{", 
           RowBox[{"True", ",", "___"}], "}"}]}]}], ",", "evspecs_List", ",", 
        "evspeclistopts___Rule"}], "]"}], ",", "\[IndentingNewLine]", 
      RowBox[{"\[Tau]vals", ":", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"_", "?", "NumericQ"}], ".."}], "}"}]}], ",", 
      "opts___Rule"}], "]"}], "/;", 
    RowBox[{"!", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"IncludeRepetitionNumbers", "/.", 
        RowBox[{"{", "opts", "}"}]}], "/.", 
       RowBox[{"Options", "[", "DivideEventSpecifications", "]"}]}], 
      ")"}]}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"EventSpecifications", "[", 
       RowBox[{"status", ",", "#", ",", "evspeclistopts"}], "]"}], "&"}], "/@", 
     RowBox[{"DivideRawEventList", "[", 
      RowBox[{"evspecs", ",", "\[Tau]vals"}], "]"}]}], ")"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.626003209904697*^9, 3.6260032720978394`*^9}, {
   3.626003452102154*^9, 3.6260035084282713`*^9}, {3.6260035914128523`*^9, 
   3.626003665251165*^9}, {3.626004203532*^9, 3.626004208107716*^9}, 
   3.626004337555956*^9, 3.683784232414151*^9, {3.683784319354116*^9, 
   3.683784331920197*^9}, {3.683790559199967*^9, 3.683790566878725*^9}, {
   3.6837941330564327`*^9, 3.683794154233601*^9}, {3.6838056701997757`*^9, 
   3.683805687409685*^9}, 3.6838058639767313`*^9, {3.683806398474963*^9, 
   3.6838063990580673`*^9}, {3.683806713414757*^9, 3.683806744753389*^9}, 
   3.683957681900663*^9, {3.6847323072659473`*^9, 3.684732307268381*^9}, {
   3.6849426111457777`*^9, 3.684942613241497*^9}, {3.684942722190813*^9, 
   3.6849427487803698`*^9}, {3.68494281293017*^9, 3.684942816617836*^9}, {
   3.7114384763557796`*^9, 3.711438484378291*^9}, {3.711438611135359*^9, 
   3.711438612998011*^9}, {3.7114387188256936`*^9, 3.7114387425581236`*^9}, {
   3.7114387794427395`*^9, 3.7114387809829016`*^9}, {3.711438812375513*^9, 
   3.711438812526164*^9}, {3.7114390844286623`*^9, 3.7114390875278816`*^9}, {
   3.711445752138176*^9, 3.7114457548380065`*^9}, {3.7114458302872276`*^9, 
   3.7114458305662556`*^9}, {3.71144706470667*^9, 3.7114470664002657`*^9}, {
   3.7117160337818637`*^9, 3.711716062303874*^9}, {3.7119670051525636`*^9, 
   3.7119670052775755`*^9}},ExpressionUUID->"dd709984-4bec-40af-bf8d-\
9800bb44c1a2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["messages", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683790528478595*^9, 3.6837905417999496`*^9}, {
  3.683790573119254*^9, 3.683790615543734*^9}, {3.683790703269456*^9, 
  3.683790704780662*^9}},ExpressionUUID->"a67e0d2d-c7ea-4eb0-ac71-\
123a2f713aa6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DivideEventSpecifications", "::", "notnumdur"}], "=", 
   "\"\<The duration of the events to be divided must be numeric.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683790713111073*^9, 3.683790738701071*^9}, 
   3.684732307275984*^9},ExpressionUUID->"e2dd347c-383d-433d-aca6-\
c96bb4748ab5"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DivideEventSpecifications", "::", "baddiv"}], "=", 
   "\"\<The division specification must be a positive definite integer or a \
list of timing intervals.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683790713111073*^9, 3.683790738701071*^9}, {
   3.6837942851996403`*^9, 3.683794298243472*^9}, {3.683805341529017*^9, 
   3.683805383885947*^9}, 
   3.684732307283935*^9},ExpressionUUID->"8f8b2339-999a-488b-af3f-\
6ec7823143a2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DivideEventSpecifications", "::", "invaltimes"}], "=", 
   "\"\<The division intervals must add up to the total duration of the event \
sequence.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683805279077436*^9, 3.68380530766203*^9}, 
   3.6847323072918243`*^9},ExpressionUUID->"9dcca2d3-0d07-4c8e-bf4d-\
e2ee82a920b2"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["local subroutines", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.625294213407867*^9, 3.625294266936805*^9}, {
   3.625296478152717*^9, 3.625296563948032*^9}, {3.625297057398859*^9, 
   3.625297133406013*^9}, {3.625306909740706*^9, 3.625306918234168*^9}, {
   3.6253850889642963`*^9, 3.625385097994274*^9}, 3.625397189623513*^9, 
   3.6255656811317177`*^9, {3.625566966384778*^9, 3.6255669875008917`*^9}, {
   3.6255670567962723`*^9, 3.625567062074286*^9}, {3.625572414691482*^9, 
   3.625572431010044*^9}, 3.625941563348687*^9, 3.6260027343865643`*^9, {
   3.683290722838217*^9, 
   3.6832907267072067`*^9}},ExpressionUUID->"9b23ef1f-2800-4790-8c2f-\
e4d97d210764"],

Cell[CellGroupData[{

Cell["AddRepetitionIndices", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.625294213407867*^9, 3.625294266936805*^9}, {
   3.625296478152717*^9, 3.625296563948032*^9}, {3.625297057398859*^9, 
   3.625297133406013*^9}, {3.625306909740706*^9, 3.625306918234168*^9}, {
   3.6253850889642963`*^9, 3.625385097994274*^9}, 3.625397189623513*^9, 
   3.6255656811317177`*^9, {3.625566966384778*^9, 3.6255669875008917`*^9}, {
   3.6255670567962723`*^9, 3.625567062074286*^9}, {3.625572414691482*^9, 
   3.625572431010044*^9}, 3.625941563348687*^9, {3.626004258946567*^9, 
   3.626004265312113*^9}, {3.683805970473262*^9, 
   3.6838060067278223`*^9}},ExpressionUUID->"b0711809-9960-4d3f-9bbf-\
b6ef35884f3e"],

Cell["\<\
This detects repetitions emerging from DivideRawEventList and outputs the \
appropriate repetition index\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.683805976287184*^9, 
  3.683806016654727*^9}},ExpressionUUID->"9cf13029-e13c-44c4-94bc-\
1b94fa7c8a6d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"AddRepetitionIndices", "[", 
    RowBox[{"eventlists", ":", 
     RowBox[{"{", 
      RowBox[{"_EventSpecifications", ".."}], "}"}]}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "groupedlists", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"groupedlists", "=", 
        RowBox[{"SplitBy", "[", 
         RowBox[{
          RowBox[{"MapAt", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"(", "\[IndentingNewLine]", 
              RowBox[{"Replace", "[", 
               RowBox[{
                RowBox[{"Rationalize", "[", "#", "]"}], ",", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"notdur_", ",", "dur_"}], "}"}], "\[RuleDelayed]", 
                 RowBox[{"{", 
                  RowBox[{"notdur", ",", " ", 
                   RowBox[{"N", "[", "dur", "]"}]}], "}"}]}]}], "]"}], ")"}], 
             "&"}], ",", "eventlists", ",", 
            RowBox[{"{", 
             RowBox[{"All", ",", "2"}], "}"}]}], "]"}], "/.", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{"None", ",", "0"}], "}"}], "\[Rule]", " ", "Nothing"}], 
            ",", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"None", ",", "0."}], "}"}], "\[Rule]", " ", 
             "Nothing"}]}], "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"MapThread", "[", 
        RowBox[{"List", ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Length", "/@", "groupedlists"}], ",", 
           RowBox[{"First", "/@", "groupedlists"}]}], "}"}], ",", "1"}], 
        "]"}]}], "\[IndentingNewLine]", ")"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683806300304955*^9, 3.683806390680806*^9}, {
   3.683807169961537*^9, 3.683807191293738*^9}, 3.684942906160171*^9, {
   3.71144571060655*^9, 3.7114457398983035`*^9}, {3.713073977100566*^9, 
   3.7130739803331227`*^9}, {3.7130741267288465`*^9, 
   3.7130741278489246`*^9}, {3.7130789738576407`*^9, 3.7130789819952726`*^9}, 
   3.7130792357450705`*^9, {3.7131029362567*^9, 3.7131029436730547`*^9}, {
   3.713103015273511*^9, 3.7131030154885035`*^9}, {3.7284684067560816`*^9, 
   3.7284684127242346`*^9}, 3.7284684510245833`*^9, {3.7284684822628527`*^9, 
   3.7284684823754416`*^9}, {3.728468532383862*^9, 3.728468625470275*^9}, 
   3.728468824568286*^9, {3.72846886767548*^9, 3.728468867948954*^9}, 
   3.728468907340575*^9, {3.7284691203437433`*^9, 
   3.7284691394936943`*^9}},ExpressionUUID->"f2e4de2b-6453-48cd-8953-\
2a5ae917a9e9"]
}, Open  ]],

Cell[CellGroupData[{

Cell["DivideRawEventList", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.625294213407867*^9, 3.625294266936805*^9}, {
   3.625296478152717*^9, 3.625296563948032*^9}, {3.625297057398859*^9, 
   3.625297133406013*^9}, {3.625306909740706*^9, 3.625306918234168*^9}, {
   3.6253850889642963`*^9, 3.625385097994274*^9}, 3.625397189623513*^9, 
   3.6255656811317177`*^9, {3.625566966384778*^9, 3.6255669875008917`*^9}, {
   3.6255670567962723`*^9, 3.625567062074286*^9}, {3.625572414691482*^9, 
   3.625572431010044*^9}, 3.625941563348687*^9, {3.626004258946567*^9, 
   3.626004265312113*^9}},ExpressionUUID->"d80351dc-2d20-450e-9832-\
f4ba500ab416"],

Cell["\<\
this divides up a raw list of event specifications (after chronological \
ordering and standardization by EventSpecifications). It functions as a \
subroutine for DivideEventSpecifications\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.6260042239781523`*^9, 3.626004307853553*^9}, {
   3.6263707926408243`*^9, 3.6263708143576927`*^9}, 3.683957681925581*^9, 
   3.684732307312149*^9},ExpressionUUID->"cb3a2dbb-28ba-4d20-9ee1-\
97db550f21c3"],

Cell[CellGroupData[{

Cell["initialize empty target list", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.6837927495970583`*^9, 3.683792781355824*^9}, {
  3.683792825838216*^9, 3.683792826499248*^9}, {3.683805715115362*^9, 
  3.683805737400676*^9}},ExpressionUUID->"ff6d49f6-7f8f-4484-ab65-\
c8512cb75cf9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DivideRawEventList", "[", 
     RowBox[{"source_List", ",", "\[Delta]tvals_List"}], "]"}], "/;", 
    RowBox[{"!", 
     RowBox[{"MatchQ", "[", 
      RowBox[{
       RowBox[{"Last", "@", "source"}], ",", 
       RowBox[{
        RowBox[{"{", "}"}], "|", 
        RowBox[{"{", 
         RowBox[{"{", "}"}], "}"}]}]}], "]"}]}]}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"DivideRawEventList", "[", 
    RowBox[{"source", ",", 
     RowBox[{"{", "}"}], ",", "\[Delta]tvals"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6252976898594637`*^9, 3.625297792431519*^9}, {
   3.6252983780576344`*^9, 3.625298379968367*^9}, {3.625306929524453*^9, 
   3.625306931275217*^9}, {3.625385106065833*^9, 3.6253851077612467`*^9}, {
   3.62539720477033*^9, 3.625397205416442*^9}, {3.625397354164668*^9, 
   3.625397361294731*^9}, {3.625565743814251*^9, 3.625565744480936*^9}, {
   3.6259415824436913`*^9, 3.625941583147439*^9}, {3.6259417440104227`*^9, 
   3.625941764009548*^9}, 3.6260008218037786`*^9, {3.626004325495479*^9, 
   3.6260043261275587`*^9}, 3.68379081583879*^9, {3.683793455760768*^9, 
   3.683793474112166*^9}, {3.6837935157323647`*^9, 3.683793518603078*^9}, 
   3.683793634167238*^9, 3.68379406374201*^9, {3.683798820810618*^9, 
   3.683798821914728*^9}, {3.683798864080221*^9, 3.683798871901705*^9}, {
   3.6837994484119883`*^9, 3.683799451257639*^9}, {3.683804768675728*^9, 
   3.683804770640409*^9}, {3.711445842814725*^9, 
   3.711445875829194*^9}},ExpressionUUID->"a923afc9-3fb0-4cac-8d4f-\
1f24c15cf596"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DivideRawEventList", "[", 
    RowBox[{"source_List", ",", 
     RowBox[{"{", "}"}], ",", "\[Delta]tvals_List"}], "]"}], ":=", 
   RowBox[{"DivideRawEventList", "[", 
    RowBox[{"source", ",", 
     RowBox[{"{", 
      RowBox[{"{", "}"}], "}"}], ",", "\[Delta]tvals"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6252976898594637`*^9, 3.625297788274021*^9}, {
   3.625298382295884*^9, 3.625298384166419*^9}, {3.625306932678144*^9, 
   3.625306934067114*^9}, {3.625385109770445*^9, 3.625385111272997*^9}, {
   3.62539720588317*^9, 3.6253972063634243`*^9}, {3.625397366166834*^9, 
   3.62539737151797*^9}, {3.625565745071288*^9, 3.625565745608457*^9}, {
   3.625941583997621*^9, 3.625941584623508*^9}, {3.625941769461446*^9, 
   3.6259417776661367`*^9}, 3.626000824311639*^9, {3.626004326766575*^9, 
   3.6260043276910067`*^9}, 3.683790817946344*^9, 3.683791703644053*^9, 
   3.683791990625663*^9, {3.683793500885962*^9, 3.683793512763031*^9}, 
   3.6837940692604837`*^9, {3.683798823428084*^9, 3.683798824858675*^9}, {
   3.683798876103581*^9, 3.683798879647346*^9}, {3.683799455345512*^9, 
   3.683799461344152*^9}, {3.683804775200316*^9, 
   3.683804777120377*^9}},ExpressionUUID->"7a10f97d-bf72-4b5a-875b-\
a2b78502cae8"]
}, Open  ]],

Cell[CellGroupData[{

Cell["terminate when nothing left in the source:", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.625992963069727*^9, 3.625992973273164*^9}, {
  3.683792951477322*^9, 
  3.683792953026699*^9}},ExpressionUUID->"92a4180c-8e0c-4dad-90fe-\
309118a4de81"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DivideRawEventList", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"divlists", ":", 
      RowBox[{"{", 
       RowBox[{"_List", "..."}], "}"}]}], ",", "\[Delta]tvals_List"}], "]"}], 
   ":=", 
   RowBox[{"divlists", "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", "}"}], "\[Rule]", 
      RowBox[{"Sequence", "[", "]"}]}], "}"}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.625298297693948*^9, 3.625298339694497*^9}, 
   3.625298387568598*^9, {3.625305450426157*^9, 3.625305457155551*^9}, 
   3.625306937245048*^9, 3.6253825172074013`*^9, {3.625384836094645*^9, 
   3.6253848470603228`*^9}, 3.625385112993536*^9, 3.625397206908518*^9, {
   3.6253973770465183`*^9, 3.625397377459504*^9}, {3.625399767873966*^9, 
   3.6253997879753304`*^9}, 3.625565746202178*^9, 3.625941585296588*^9, {
   3.6259417852117367`*^9, 3.625941789642682*^9}, 3.62594206310079*^9, {
   3.625994444924993*^9, 3.625994448129197*^9}, {3.626000865217804*^9, 
   3.62600086760178*^9}, 3.626004328555417*^9, 3.683794073542612*^9, 
   3.683798837282238*^9, {3.683798900352064*^9, 3.683798906110323*^9}, {
   3.683799464728698*^9, 3.6837994685048323`*^9}, 
   3.6838046780432262`*^9},ExpressionUUID->"68aa72fe-93cd-4278-acf1-\
9b207a342273"]
}, Open  ]],

Cell[CellGroupData[{

Cell["pass elements from source to target", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.6253855315993233`*^9, 3.6253855571547003`*^9}, {
   3.625385658126376*^9, 3.6253856608103437`*^9}, {3.683792807653013*^9, 
   3.683792817227399*^9}, {3.683799017407104*^9, 3.683799018398157*^9}, 
   3.6838047858735523`*^9},ExpressionUUID->"371aa48e-d8d1-4666-bc63-\
d371889077f2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DivideRawEventList", "[", 
    RowBox[{"source_", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "}"}], ",", "prevlists___List"}], "}"}], ",", 
     "\[Delta]tvals_List"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"newsource", ",", "newlist"}], "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"newsource", ",", "newlist"}], "}"}], "=", 
         RowBox[{"TransferElement", "[", 
          RowBox[{"source", ",", 
           RowBox[{"{", "}"}], ",", 
           RowBox[{"Last", "[", "\[Delta]tvals", "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"DivideRawEventList", "[", 
         RowBox[{"newsource", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", "}"}], ",", "newlist", ",", "prevlists"}], "}"}], 
          ",", 
          RowBox[{"Most", "[", "\[Delta]tvals", "]"}]}], "]"}]}], 
       "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"EventDuration", "[", 
         RowBox[{"source", ",", 
          RowBox[{"Check", "\[Rule]", "False"}]}], "]"}], "\[GreaterEqual]", 
        RowBox[{"Last", "[", "\[Delta]tvals", "]"}]}], "\[IndentingNewLine]", 
       ")"}]}]}], "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6252980026595497`*^9, 3.6252982604533052`*^9}, {
   3.625298357768244*^9, 3.625298359446639*^9}, {3.625298390919633*^9, 
   3.6252983928495607`*^9}, {3.62530694020716*^9, 3.625306945832945*^9}, 
   3.625383391273451*^9, {3.625383563044932*^9, 3.6253835888309526`*^9}, {
   3.625383768620365*^9, 3.625383775424842*^9}, {3.62538387963516*^9, 
   3.6253838820470867`*^9}, 3.625384770996626*^9, {3.62538511481005*^9, 
   3.625385116753553*^9}, {3.625385219128742*^9, 3.625385240828023*^9}, {
   3.625385274244854*^9, 3.625385277360791*^9}, {3.625385325182749*^9, 
   3.62538538015069*^9}, {3.625385433052043*^9, 3.62538551906087*^9}, {
   3.625385667605523*^9, 3.625385673489025*^9}, {3.625385819444435*^9, 
   3.625385825446795*^9}, {3.6253859611028337`*^9, 3.625386079333906*^9}, {
   3.625386236403636*^9, 3.625386360739758*^9}, {3.625386486759493*^9, 
   3.625386487688863*^9}, {3.6253867382576933`*^9, 3.625386747562689*^9}, {
   3.62539720765508*^9, 3.625397208460661*^9}, {3.625397385017231*^9, 
   3.625397412677084*^9}, {3.625398107162826*^9, 3.625398118858919*^9}, {
   3.625398172834176*^9, 3.6253981751464252`*^9}, {3.625398258574745*^9, 
   3.6253983081662827`*^9}, {3.625398652578396*^9, 3.6253986619659843`*^9}, {
   3.625398693894423*^9, 3.625398696810349*^9}, {3.625399620970945*^9, 
   3.625399623130018*^9}, {3.62556574693684*^9, 3.625565747560074*^9}, {
   3.625941586090744*^9, 3.625941587101344*^9}, 3.625941806593156*^9, {
   3.625941856931085*^9, 3.625941874359565*^9}, {3.625942007304867*^9, 
   3.625942015709968*^9}, 3.6259420500693417`*^9, 3.625942297887892*^9, {
   3.625942509352318*^9, 3.625942523899831*^9}, 3.6259425793537073`*^9, 
   3.625993091530848*^9, 3.625993135715186*^9, {3.6259938301804857`*^9, 
   3.6259938517007217`*^9}, {3.6259942253133698`*^9, 3.625994271205269*^9}, 
   3.625994394151679*^9, {3.625994485640744*^9, 3.6259945393399963`*^9}, {
   3.625994626465664*^9, 3.625994626633327*^9}, 3.6259948864228563`*^9, {
   3.625995096899252*^9, 3.625995106775771*^9}, {3.62599516210779*^9, 
   3.625995182436078*^9}, {3.6260005711730947`*^9, 3.62600058976928*^9}, {
   3.626000661943062*^9, 3.6260006780748453`*^9}, {3.626000893910758*^9, 
   3.6260009090966387`*^9}, {3.6260009999815273`*^9, 3.62600100303763*^9}, {
   3.626004329535042*^9, 3.626004330286083*^9}, {3.680955460527204*^9, 
   3.6809554616060658`*^9}, {3.683790830759776*^9, 3.683790847319231*^9}, {
   3.683791613944706*^9, 3.683791616719802*^9}, {3.68379224733549*^9, 
   3.68379225056448*^9}, 3.683793257481619*^9, {3.6837933557571287`*^9, 
   3.6837933768911877`*^9}, 3.683793916171275*^9, 3.683794079542266*^9, {
   3.683794727566514*^9, 3.683794748434362*^9}, {3.683794990028533*^9, 
   3.683794997194511*^9}, {3.683795269190465*^9, 3.683795270756647*^9}, {
   3.683795537884173*^9, 3.683795540394244*^9}, {3.6837956271878443`*^9, 
   3.68379562814537*^9}, {3.683795850614991*^9, 3.68379587050916*^9}, {
   3.683795934867433*^9, 3.683795936159706*^9}, {3.683799023153351*^9, 
   3.6837990633168907`*^9}, {3.683799100886847*^9, 3.683799105586104*^9}, {
   3.683799140737883*^9, 3.683799159736977*^9}, {3.683799335191687*^9, 
   3.683799362814733*^9}, {3.683799487624648*^9, 3.683799521360297*^9}, {
   3.683799769606221*^9, 3.6837997723322363`*^9}, 3.6837998308544407`*^9, {
   3.683799883996188*^9, 3.683799885715638*^9}, {3.683800719892222*^9, 
   3.683800759916843*^9}, {3.683800834338583*^9, 3.683800886397829*^9}, {
   3.683800967845792*^9, 3.683800977005278*^9}, {3.683804608061008*^9, 
   3.683804610285288*^9}, {3.6838046809712563`*^9, 3.683804683051485*^9}, {
   3.711445887586115*^9, 3.711446041202645*^9}, 3.7114461572021976`*^9, {
   3.711446195919162*^9, 3.7114461983809032`*^9}, {3.7114463477890215`*^9, 
   3.711446352383356*^9}, {3.711446424847356*^9, 3.711446425018702*^9}, {
   3.711716321440054*^9, 3.7117163281001296`*^9}, {3.7117164232276125`*^9, 
   3.711716433417942*^9}, {3.7117165128259234`*^9, 3.711716512956156*^9}, 
   3.7117165486442966`*^9, {3.7119663266536336`*^9, 3.7119663268745475`*^9}, {
   3.7119664411016903`*^9, 3.711966467822528*^9}, {3.711967215588518*^9, 
   3.711967218831713*^9}, {3.711967293454054*^9, 3.711967298431772*^9}, {
   3.7119674626574717`*^9, 3.711967462782483*^9}, {3.713073619868298*^9, 
   3.7130736229970617`*^9}, {3.713073816724405*^9, 3.713073820181361*^9}, {
   3.713073896321107*^9, 3.713073925990956*^9}, {3.713074076383069*^9, 
   3.713074082888337*^9}, {3.713078992913558*^9, 
   3.713078993053562*^9}},ExpressionUUID->"88da2a19-22f7-4431-9ffb-\
c54ad7612103"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TransferElement", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.625294213407867*^9, 3.625294266936805*^9}, {
   3.625296478152717*^9, 3.6252964986642*^9}, 
   3.626004518777576*^9},ExpressionUUID->"18f54331-80d4-4a90-99cc-\
53a94e2fd46e"],

Cell["\<\
this is used when compiling a divided list by transferring an element from \
one list of events to another until events corresponding to the desired \
duration \[Delta]t are fully transferred.\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.626004474473237*^9, 3.626004532312069*^9}, {
  3.626370839106924*^9, 
  3.626370844774974*^9}},ExpressionUUID->"08028e46-1fd3-44c4-950e-\
80dd9c2d6581"],

Cell[CellGroupData[{

Cell["finish when nothing left to transfer", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.625295089897601*^9, 3.625295100262987*^9}, 
   3.6838002409961348`*^9},ExpressionUUID->"ee52a645-8bec-48a9-b09d-\
50db3ab5bd06"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TransferElement", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "target_List", ",", "\[Delta]t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "target"}], "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.62529406999446*^9, 3.6252942049133453`*^9}, {
  3.6837995526428337`*^9, 3.6837995738194933`*^9}, {3.6838001488881617`*^9, 
  3.6838001524938107`*^9}},ExpressionUUID->"9f497d1b-ee12-4d96-beb7-\
4825dd79951d"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
finish when the transfer is infinitesimal, but there is a finite source \
element left\
\>", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.625295089897601*^9, 3.625295100262987*^9}, 
   3.6838002409961348`*^9, {3.683804231215666*^9, 3.683804235925302*^9}, {
   3.683804817816401*^9, 
   3.6838048347094803`*^9}},ExpressionUUID->"157d0ce4-7210-43e5-88d7-\
d0506adc0320"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TransferElement", "[", 
    RowBox[{"source_List", ",", "target_List", ",", "\[Delta]t_"}], "]"}], "/;", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Chop", "[", "\[Delta]t", "]"}], "\[Equal]", "0"}], ")"}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"EventDuration", "[", 
       RowBox[{"source", ",", 
        RowBox[{"Check", "\[Rule]", "False"}]}], "]"}], ">", "0"}], ")"}]}]}],
   ":=", 
  RowBox[{"{", 
   RowBox[{"source", ",", "target"}], "}"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.62529406999446*^9, 3.6252942049133453`*^9}, {
   3.6837995526428337`*^9, 3.6837995738194933`*^9}, {3.6838001488881617`*^9, 
   3.6838001524938107`*^9}, {3.683804258362317*^9, 3.683804314659575*^9}, {
   3.683804493442801*^9, 3.6838045058097277`*^9}, {3.7114461071217103`*^9, 
   3.711446142913046*^9}, {3.711716423236636*^9, 3.7117164378280225`*^9}, 
   3.711716551628149*^9, {3.713591452908086*^9, 
   3.7135914588763533`*^9}},ExpressionUUID->"ff7f2d11-7e6b-4238-80f1-\
7aa92c772f7e"]
}, Open  ]],

Cell[CellGroupData[{

Cell["always transfer any element of infinitesimal duration", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.625295103478917*^9, 3.62529511148678*^9}, {
  3.6253847291159*^9, 3.625384736187374*^9}, {3.683800069200034*^9, 
  3.6838000737258263`*^9}, {3.683801312886938*^9, 3.683801313406934*^9}, {
  3.68380174716847*^9, 
  3.68380174822964*^9}},ExpressionUUID->"cc88bf87-bc59-40ba-a257-\
87ab84ab7b35"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TransferElement", "[", 
    RowBox[{"source_List", ",", "target_List", ",", "\[Delta]t_"}], "]"}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"firstel", ",", "first\[Tau]"}], "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{"TransferElement", "[", 
        RowBox[{
         RowBox[{"Most", "@", "source"}], ",", 
         RowBox[{"Join", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"Last", "@", "source"}], "}"}], ",", "target"}], "]"}], 
         ",", "\[Delta]t"}], "]"}], "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", " ", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Length", "[", "source", "]"}], "\[GreaterEqual]", "1"}], 
         ")"}], "&&", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Chop", "[", 
           RowBox[{"EventDuration", "[", 
            RowBox[{
             RowBox[{"Last", "@", "source"}], ",", 
             RowBox[{"Check", "\[Rule]", "False"}]}], "]"}], "]"}], 
          "\[Equal]", "0"}], ")"}]}], "\[IndentingNewLine]", ")"}]}]}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.62529406999446*^9, 3.6252942049133453`*^9}, {
   3.625294278593479*^9, 3.6252943644489822`*^9}, {3.6252947404758177`*^9, 
   3.6252947593697233`*^9}, {3.6252947917075357`*^9, 3.625294794377369*^9}, {
   3.625295184199264*^9, 3.625295365639678*^9}, {3.6253847042935257`*^9, 
   3.625384726217465*^9}, 3.680955464103218*^9, {3.683790863839161*^9, 
   3.683790864085161*^9}, 3.683795282650817*^9, {3.6837955085978127`*^9, 
   3.68379550878817*^9}, {3.683799615633719*^9, 3.683799623823886*^9}, {
   3.6838001980282097`*^9, 3.683800202968754*^9}, {3.683801324097295*^9, 
   3.683801347992571*^9}, {3.683802886704911*^9, 3.683802889708447*^9}, {
   3.683803415925303*^9, 3.6838034897028923`*^9}, 3.683804599448388*^9, {
   3.7117164232436543`*^9, 3.711716440207354*^9}, 3.7117164909667234`*^9, {
   3.7130736613473554`*^9, 
   3.7130736832468204`*^9}},ExpressionUUID->"cff19ebd-5777-46be-b0bb-\
3a1c2b840a6a"]
}, Open  ]],

Cell[CellGroupData[{

Cell["transfer any element that is shorter or equal to \[Delta]t", \
"Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.625295103478917*^9, 3.62529511148678*^9}, {
   3.6253847291159*^9, 3.625384736187374*^9}, {3.683800069200034*^9, 
   3.6838000737258263`*^9}, 
   3.683801394561037*^9},ExpressionUUID->"1e5996d2-0234-4839-a20b-\
fcc439b4e5d9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TransferElement", "[", 
    RowBox[{"source_List", ",", "target_List", ",", "\[Delta]t_"}], "]"}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"firstel", ",", "first\[Tau]"}], "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{"TransferElement", "[", 
        RowBox[{
         RowBox[{"Most", "@", "source"}], ",", 
         RowBox[{"Join", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"Last", "@", "source"}], "}"}], ",", "target"}], "]"}], 
         ",", 
         RowBox[{"\[Delta]t", "-", "first\[Tau]"}]}], "]"}], 
       "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", " ", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Length", "[", "source", "]"}], "\[GreaterEqual]", "1"}], 
         ")"}], "&&", 
        RowBox[{"(", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"firstel", "=", 
           RowBox[{"Last", "@", "source"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"first\[Tau]", "=", 
           RowBox[{"EventDuration", "[", 
            RowBox[{"firstel", ",", 
             RowBox[{"Check", "\[Rule]", "False"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"first\[Tau]", "\[LessEqual]", "\[Delta]t"}], ")"}], "&&", 
           RowBox[{
            RowBox[{"Chop", "[", "first\[Tau]", "]"}], "\[NotEqual]", 
            "0"}]}]}], "\[IndentingNewLine]", ")"}]}], "\[IndentingNewLine]", 
       ")"}]}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.62529406999446*^9, 3.6252942049133453`*^9}, {
   3.625294278593479*^9, 3.6252943644489822`*^9}, {3.6252947404758177`*^9, 
   3.6252947593697233`*^9}, {3.6252947917075357`*^9, 3.625294794377369*^9}, {
   3.625295184199264*^9, 3.625295365639678*^9}, {3.6253847042935257`*^9, 
   3.625384726217465*^9}, 3.680955464103218*^9, {3.683790863839161*^9, 
   3.683790864085161*^9}, 3.683795282650817*^9, {3.6837955085978127`*^9, 
   3.68379550878817*^9}, {3.683799615633719*^9, 3.683799623823886*^9}, {
   3.6838001980282097`*^9, 3.683800202968754*^9}, 3.683801409841263*^9, {
   3.683802974065906*^9, 3.683802978728796*^9}, {3.683803028159494*^9, 
   3.683803061444805*^9}, {3.683803541691674*^9, 3.6838035496336*^9}, 
   3.683804602889073*^9, {3.711716423250199*^9, 3.7117164440379286`*^9}, 
   3.7117165554159327`*^9, {3.7119667271458893`*^9, 3.711966730278401*^9}, {
   3.7119667667948112`*^9, 3.71196676818839*^9}, {3.7130736936413856`*^9, 
   3.713073694605525*^9}},ExpressionUUID->"2deea053-9e2d-4c5c-8e4c-\
17b5ecdb5a17"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
split and transfer any element that is longer than \[Delta]t, as long as \
\[Delta]t is finite\
\>", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.625295116328683*^9, 3.625295126094715*^9}, {
  3.683804067593207*^9, 
  3.683804080263968*^9}},ExpressionUUID->"289c026b-5efa-472f-9cd7-\
fe1da20f95c0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TransferElement", "[", 
    RowBox[{"source_List", ",", "target_List", ",", "\[Delta]t_"}], "]"}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "firstel", ",", "evgen", ",", "\[Tau]", ",", "splitel1", ",", 
       "splitel2"}], "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"splitel2", ",", "splitel1"}], "}"}], "=", 
         RowBox[{"SplitElement", "[", 
          RowBox[{"firstel", ",", "\[Delta]t"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"TransferElement", "[", 
         RowBox[{
          RowBox[{"Join", "[", 
           RowBox[{
            RowBox[{"Most", "[", "source", "]"}], ",", 
            RowBox[{"{", "splitel2", "}"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"Join", "[", 
           RowBox[{
            RowBox[{"{", "splitel1", "}"}], ",", "target"}], "]"}], ",", 
          "0"}], "]"}]}], "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"\[Delta]t", ">", "0"}], ")"}], "&&", 
        RowBox[{
         RowBox[{"Length", "[", "source", "]"}], ">", "0"}], "&&", 
        RowBox[{"(", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"firstel", "=", 
           RowBox[{"Last", "@", "source"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"MatchQ", "[", 
            RowBox[{"firstel", ",", 
             RowBox[{"{", 
              RowBox[{"evgen_", ",", "\[Tau]_", ",", "___Rule"}], "}"}]}], 
            "]"}], "&&", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"EventDuration", "[", 
             RowBox[{"firstel", ",", 
              RowBox[{"Check", "\[Rule]", "False"}]}], "]"}], ">", 
            "\[Delta]t"}]}]}], ")"}]}], "\[IndentingNewLine]", ")"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.62529406999446*^9, 3.6252942049133453`*^9}, {
   3.625294278593479*^9, 3.625294656192975*^9}, {3.625295209592702*^9, 
   3.62529521374704*^9}, {3.625295383223165*^9, 3.6252954158975687`*^9}, {
   3.625295671970326*^9, 3.625295682135391*^9}, {3.625295781749772*^9, 
   3.625295791591707*^9}, {3.625295919306574*^9, 3.625295955157494*^9}, {
   3.625296167946844*^9, 3.625296168073906*^9}, {3.625296213059782*^9, 
   3.625296225515079*^9}, {3.626369845291613*^9, 3.6263699329563847`*^9}, 
   3.680955465138501*^9, {3.683790869260518*^9, 3.683790869532987*^9}, 
   3.683795286541636*^9, {3.683795511957527*^9, 3.683795512181493*^9}, {
   3.6837996292312202`*^9, 3.683799647260734*^9}, {3.683800208161728*^9, 
   3.683800213247912*^9}, 3.683803792199988*^9, {3.683804084297398*^9, 
   3.683804111880172*^9}, {3.6838044018870907`*^9, 3.6838044022605743`*^9}, {
   3.6849431319133*^9, 3.684943134922374*^9}, {3.7114455709944077`*^9, 
   3.7114455749847393`*^9}, {3.7114457678379445`*^9, 3.711445768015159*^9}, {
   3.711446047662709*^9, 3.711446049833846*^9}, {3.7114460936490912`*^9, 
   3.7114460952020345`*^9}, {3.7117164232531796`*^9, 3.711716446257713*^9}, 
   3.711716478079514*^9, 3.711716558345783*^9, {3.7119675229633045`*^9, 
   3.711967550258494*^9}},ExpressionUUID->"b05268a8-fffe-4ef5-968d-\
c1cf0af70a0e"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SplitElement", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.625294213407867*^9, 3.625294266936805*^9}, {
   3.625296478152717*^9, 3.6252964986642*^9}, 3.626004518777576*^9, {
   3.6263697600201063`*^9, 3.626369762616696*^9}, {3.626369838009841*^9, 
   3.6263698387658253`*^9}},ExpressionUUID->"30f8b561-1f11-4c72-8574-\
d668f4933010"],

Cell["\<\
splits an event of finite time into two parts. For time-independent evolution \
generators this is trivial. However handling of the local time variables in \
ShapeFunction requires care. \
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.626369767265826*^9, 
  3.626369830092209*^9}},ExpressionUUID->"307e2706-387d-4317-9bf7-\
02998b01d5af"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SplitElement", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"evgen", ":", 
        RowBox[{"(", 
         RowBox[{"None", "|", 
          RowBox[{"_", "?", "OperatorQ"}], "|", 
          RowBox[{"_", "?", "SuperoperatorQ"}]}], ")"}]}], ",", "\[Tau]_", 
       ",", "opts___Rule"}], "}"}], ",", "\[Delta]t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"evgen", ",", 
       RowBox[{"Chop", "[", 
        RowBox[{"\[Tau]", "-", "\[Delta]t"}], "]"}], ",", "opts"}], "}"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"evgen", ",", 
       RowBox[{"Chop", "[", "\[Delta]t", "]"}], ",", "opts"}], "}"}]}], 
    "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.626369949579814*^9, 3.6263699892645893`*^9}, {
   3.626370037534227*^9, 3.62637005974739*^9}, {3.6263700899654217`*^9, 
   3.626370098449054*^9}, 3.626370645017105*^9, {3.684943145468321*^9, 
   3.684943162379386*^9}, {3.7119674024780025`*^9, 
   3.7119674326657505`*^9}},ExpressionUUID->"a8caa22d-4ec8-4e44-9436-\
e4dff0ef1966"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SplitElement", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"evgen", ":", 
        RowBox[{"ShapeFunction", "[", 
         RowBox[{"t_", ",", 
          RowBox[{"\[Tau]xpairs_", "?", "MatrixQ"}], ",", "T_", ",", 
          "func_"}], "]"}]}], ",", "\[Tau]_", ",", "opts___Rule"}], "}"}], 
     ",", "\[Delta]t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "\[Tau]vals", ",", "xvals", ",", "xvals1", ",", "xvals2", ",", 
       "\[Tau]xpairs1", ",", "\[Tau]xpairs2"}], "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"\[Tau]vals", ",", "xvals"}], "}"}], "=", 
        RowBox[{"Transpose", "[", "\[Tau]xpairs", "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"xvals1", "=", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"#", " ", 
            RowBox[{"\[Tau]", "/", "\[Delta]t"}]}], ")"}], "&"}], "/@", 
         "xvals"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"xvals2", "=", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"#", " ", "\[Tau]"}], "-", "\[Delta]t"}], ")"}], "/", 
            RowBox[{"(", 
             RowBox[{"\[Tau]", "-", "\[Delta]t"}], ")"}]}], ")"}], "&"}], "/@",
          "xvals"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"\[Tau]xpairs1", "=", 
        RowBox[{"Transpose", "[", 
         RowBox[{"{", 
          RowBox[{"\[Tau]vals", ",", "xvals1"}], "}"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"\[Tau]xpairs2", "=", 
        RowBox[{"Transpose", "[", 
         RowBox[{"{", 
          RowBox[{"\[Tau]vals", ",", "xvals2"}], "}"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"ShapeFunction", "[", 
            RowBox[{"t", ",", 
             RowBox[{"Evaluate", "@", "\[Tau]xpairs2"}], ",", "T", ",", 
             "func"}], "]"}], ",", 
           RowBox[{"Chop", "[", 
            RowBox[{"\[Tau]", "-", "\[Delta]t"}], "]"}], ",", "opts"}], "}"}],
          ",", "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"ShapeFunction", "[", 
            RowBox[{"t", ",", 
             RowBox[{"Evaluate", "@", "\[Tau]xpairs1"}], ",", "T", ",", 
             "func"}], "]"}], ",", 
           RowBox[{"Chop", "[", "\[Delta]t", "]"}], ",", "opts"}], "}"}]}], 
        "\[IndentingNewLine]", "}"}]}], "\[IndentingNewLine]", ")"}]}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.626369949579814*^9, 3.6263699892645893`*^9}, {
  3.626370037534227*^9, 3.62637005974739*^9}, {3.6263700899654217`*^9, 
  3.62637016093239*^9}, {3.626370198120289*^9, 3.626370202638897*^9}, {
  3.6263704008239822`*^9, 3.626370488601664*^9}, {3.626370597390666*^9, 
  3.6263706420515747`*^9}, {3.684943172530011*^9, 3.6849431946420794`*^9}, {
  3.711967369119956*^9, 3.7119673918220434`*^9}, {3.711967438797776*^9, 
  3.7119674545715046`*^9}, {3.7130737535924215`*^9, 
  3.7130737641631517`*^9}},ExpressionUUID->"f07724aa-bca3-43b4-a546-\
a985a70d1cf9"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Event", "Subtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455265644416114*^9, 3.455265648200944*^9}, {
   3.464256156522676*^9, 3.4642561730892*^9}, {3.464282095283453*^9, 
   3.4642821290032167`*^9}, {3.503094263066369*^9, 3.503094264574778*^9}, {
   3.503095573470295*^9, 3.503095585674507*^9}, {3.504260287465301*^9, 
   3.504260304480891*^9}, 3.504261075915663*^9, 3.514538950441782*^9, {
   3.541442633712829*^9, 3.541442660772046*^9}, {3.541485412813717*^9, 
   3.541485415659666*^9}, 3.54149325313505*^9, {3.683128937510503*^9, 
   3.683128938174704*^9}, {3.683958413924868*^9, 3.683958431756687*^9}, {
   3.683981590446064*^9, 3.6839815964179707`*^9}, 3.685033569322124*^9, 
   3.6850339229117403`*^9, 
   3.6904491051766663`*^9},ExpressionUUID->"54e0fb25-289c-4485-8feb-\
1f22e137e576"],

Cell[CellGroupData[{

Cell["defaults", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, {
   3.683973033466785*^9, 3.683973044514587*^9}, {3.6842152838892107`*^9, 
   3.684215286048623*^9}},ExpressionUUID->"4a9cf47d-8688-4a3c-8e72-\
15e3abba5a96"],

Cell[CellGroupData[{

Cell["default options", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683958534614377*^9, 
   3.683958557622012*^9}},ExpressionUUID->"8b04d86b-bb63-4155-9eba-\
e6652af56ad3"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "Event", "]"}], "=", "\[IndentingNewLine]", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"InitialTimePoint", "\[Rule]", "Automatic"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FinalTimePoint", "\[Rule]", "Automatic"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"BackgroundGenerator", "\[Rule]", "Undefined"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"PropagatorCalculationProcedure", "\[Rule]", "Automatic"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"PropagationMethod", "\[Rule]", " ", "Automatic"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"EventSampling", "\[Rule]", "None"}], ",", "\[IndentingNewLine]", 
     RowBox[{"DeleteCoherences", "\[Rule]", " ", "True"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"OperatorBasis", "\[Rule]", "Automatic"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Basis", "\[Rule]", "Automatic"}], ",", "\[IndentingNewLine]", 
     RowBox[{"ExpandRepeatedEvents", "\[Rule]", "Automatic"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Verbose", "\[Rule]", "False"}]}], "\[IndentingNewLine]", 
    "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.683958565949041*^9, {3.6839731149650908`*^9, 3.683973130952525*^9}, {
   3.683973897643121*^9, 3.683973898836083*^9}, {3.683977863168428*^9, 
   3.683977866811652*^9}, 3.683981650615322*^9, {3.683999571453845*^9, 
   3.683999572076592*^9}, {3.684036715181535*^9, 3.684036717076641*^9}, {
   3.684991203302208*^9, 3.684991254019801*^9}, 3.685033569383675*^9, 
   3.685035184902175*^9, {3.689410573686285*^9, 3.689410582200512*^9}, {
   3.690448106311458*^9, 3.690448110325068*^9}, 3.709374877014697*^9, {
   3.7250899656776237`*^9, 3.725089967619697*^9}, {3.7282165711299076`*^9, 
   3.7282165714875727`*^9}},ExpressionUUID->"3c8dfdde-594c-478d-b0b1-\
c41faff9f258"]
}, Open  ]],

Cell[CellGroupData[{

Cell["$Failed", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683958534614377*^9, 3.683958551028832*^9}, {
   3.68395858577393*^9, 3.6839585874144278`*^9}, 3.683959406338385*^9, {
   3.683963242982894*^9, 3.6839632443419228`*^9}, {3.683964445776389*^9, 
   3.683964455294318*^9}, {3.683983901941537*^9, 
   3.6839839032008667`*^9}},ExpressionUUID->"10ea614f-cedf-4eab-ad86-\
decea377b265"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Event", "[", "args___", "]"}], "/;", 
    RowBox[{"!", 
     RowBox[{"FreeQ", "[", 
      RowBox[{
       RowBox[{"{", "args", "}"}], ",", "$Failed"}], "]"}]}]}], ":=", 
   "$Failed"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683983914149064*^9, 3.6839839305516777`*^9}, 
   3.6850335693888206`*^9},ExpressionUUID->"04026fe6-ef9b-4b4a-9bb3-\
e01843376fb0"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["status & characteristics", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, {
   3.683973033466785*^9, 3.683973044514587*^9}, 
   3.6840002435345*^9},ExpressionUUID->"99a900ab-6db8-4294-b4e0-1cb6dec9b4b8"],

Cell[CellGroupData[{

Cell["Event status & char syntax", "Section",
 CellChangeTimes->{{3.6839769345684147`*^9, 3.683976958869174*^9}, {
   3.6851721013267317`*^9, 3.6851721044519997`*^9}, {3.685194518004075*^9, 
   3.6851945584473248`*^9}, 3.685196786924673*^9, {3.689410187697937*^9, 
   3.689410190149434*^9}},ExpressionUUID->"5cdb659b-0fba-4510-ac99-\
c904f4a6082c"],

Cell[CellGroupData[{

Cell["status:{<ready>,<numeric>}", "Subsection",
 CellChangeTimes->{{3.689409839714465*^9, 
  3.689409859202746*^9}},ExpressionUUID->"81321411-4ad4-4bca-a707-\
ba693aa2f13f"],

Cell["\<\
<ready>=True if ready for computation.
<numeric>=True if the event descriptors are purely numeric.\
\>", "Text",
 CellChangeTimes->{{3.689409869650942*^9, 
  3.6894098789918537`*^9}},ExpressionUUID->"e33b998e-f34e-45b3-893a-\
66267fbe4a1a"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
characteristics:{timing,evbackg,hilbertcompat,propcalcproc,exprep,precalchk}\
\>", "Subsection",
 CellChangeTimes->{{3.689409839714465*^9, 3.689409859202746*^9}, 
   3.689409906443359*^9, {3.691483113458617*^9, 3.691483114809393*^9}, 
   3.691483162098085*^9},ExpressionUUID->"f5411a0c-7fbc-4027-8b4e-\
d5bfaaee2334"],

Cell[CellGroupData[{

Cell["<timing> = {duration, {tb,ta}, pts, {tb0,ta0}}", "Subsubsection",
 CellChangeTimes->{{3.689409839714465*^9, 3.6894098405450172`*^9}, {
  3.689409926890026*^9, 
  3.689409928752881*^9}},ExpressionUUID->"37e0e147-143e-47e2-a0c6-\
06876ba000e7"],

Cell["\<\
{tb,ta} are the final and initial times in the sequence of events;
pts is the permitted time shift (pts) according to the type of event and the \
evolution background;
{tb0,ta0} are the standardized final and initial times, taking into account \
the pts. \
\>", "Text",
 CellChangeTimes->{{3.689409937366763*^9, 
  3.689409954352346*^9}},ExpressionUUID->"a4e1119a-487f-4acf-bb4c-\
0f8e3710cfc4"]
}, Closed]],

Cell[CellGroupData[{

Cell["<evbackg> = {evbackg,<reqbackgimp>,<backgimp>}\t\t", "Subsubsection",
 CellChangeTimes->{{3.689409839714465*^9, 3.6894098405450172`*^9}, {
  3.689409973737048*^9, 
  3.689409983579653*^9}},ExpressionUUID->"42d3edda-fc5a-4919-8914-\
e5de2c866cb1"],

Cell["\<\
where evbackg is the evolution background
<reqbackgimp>=True requests implementation of the background
<backgimp> = True when the backg has been incorporated into the event.\
\>", "Text",
 CellChangeTimes->{{3.6894099872283*^9, 
  3.689409995597994*^9}},ExpressionUUID->"09a97470-0a10-45ae-83c3-\
845383b538eb"]
}, Closed]],

Cell[CellGroupData[{

Cell["<hilbert>", "Subsubsection",
 CellChangeTimes->{{3.689409839714465*^9, 3.6894098405450172`*^9}, 
   3.68941000604853*^9},ExpressionUUID->"e5688cc5-e6f2-411d-8123-\
0d4ae798d203"],

Cell["<hilbert> = True if the event is Hilbert compatible", "Text",
 CellChangeTimes->{
  3.689410013197605*^9},ExpressionUUID->"0884613f-2fc3-4187-95c7-\
477e449726f9"]
}, Closed]],

Cell[CellGroupData[{

Cell["<propcalcproc>", "Subsubsection",
 CellChangeTimes->{{3.689409839714465*^9, 3.6894098405450172`*^9}, 
   3.689410028477584*^9},ExpressionUUID->"f7227176-75bc-47d4-a39d-\
0803059a3dfc"],

Cell["\<\
contains the preferred propagator calculation procedure for the event.\
\>", "Text",
 CellChangeTimes->{{3.689410039077362*^9, 
  3.689410040803338*^9}},ExpressionUUID->"8781db84-dbf7-46dc-a3a4-\
70af809d6268"]
}, Closed]],

Cell[CellGroupData[{

Cell["<exprep>={type,request,implemented}", "Subsubsection",
 CellChangeTimes->{{3.689409839714465*^9, 3.6894098405450172`*^9}, {
  3.689410048942927*^9, 3.689410051061969*^9}, {3.689410603637149*^9, 
  3.689410608688835*^9}},ExpressionUUID->"758d80c1-4292-4874-aaa0-\
9c1e64278a1e"],

Cell["\<\
type = True|All|Automatic
exprequest = True (requests expansion)
expimp=True (indicates implementation)\
\>", "Text",
 CellChangeTimes->{{3.689410614538101*^9, 
  3.689410668509727*^9}},ExpressionUUID->"190b9038-8e9f-4664-8061-\
bb211196bc6a"]
}, Closed]],

Cell[CellGroupData[{

Cell["<precalchk>={precalchkreq,precalchk}", "Subsubsection",
 CellChangeTimes->{{3.689409839714465*^9, 3.6894098405450172`*^9}, {
  3.689410048942927*^9, 3.689410051061969*^9}, {3.689410603637149*^9, 
  3.689410608688835*^9}, {3.691483126602542*^9, 
  3.691483156264538*^9}},ExpressionUUID->"1ac074c2-1b2f-4de3-be09-\
ef9885fa4e49"],

Cell["\<\
precalchkreq = True (requests check)
precalchk=True (indicates implementation)\
\>", "Text",
 CellChangeTimes->{{3.689410614538101*^9, 3.689410668509727*^9}, {
  3.691483132779667*^9, 
  3.691483170802287*^9}},ExpressionUUID->"cd17452c-41df-4bda-a3c5-\
56393d971b53"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["status", "Section",
 CellChangeTimes->{{3.6839769345684147`*^9, 3.683976958869174*^9}, {
   3.6851721013267317`*^9, 3.6851721044519997`*^9}, 
   3.691578231177886*^9},ExpressionUUID->"a8fb6b7a-c9a4-4052-9a6e-\
e1a8a3c829fd"],

Cell[CellGroupData[{

Cell["fill in False status if omitted", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683958727456455*^9, 3.6839587370191107`*^9}, {
  3.6839587793697453`*^9, 
  3.683958781370214*^9}},ExpressionUUID->"69cc3e14-733a-446f-97d2-\
a3f8e24a1609"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"evspec", ":", 
      RowBox[{"Except", "[", "_Rule", "]"}]}], ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"Event", "[", 
    RowBox[{"False", ",", "evspec", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683958754729197*^9, 3.683958774518546*^9}, {
  3.683958838874489*^9, 3.683958841777224*^9}, {3.683959513848783*^9, 
  3.68395951454422*^9}, {3.685033569392807*^9, 
  3.6850335693949623`*^9}},ExpressionUUID->"c97a54ec-c2a7-4a45-b828-\
1e07a9660938"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{"False", ",", 
     RowBox[{"more", ":", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Except", "[", "_Rule", "]"}], "..."}], ")"}]}], ",", 
     "opts___Rule"}], "]"}], ":=", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "False"}], "}"}], ",", "more", ",", "opts"}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6839586027648153`*^9, 3.6839586045846977`*^9}, {
   3.683958667580065*^9, 3.683958711018367*^9}, 3.683959521095401*^9, {
   3.68503356939885*^9, 3.685033569401066*^9}, {3.685171661615162*^9, 
   3.68517166592238*^9}, {3.685171757735458*^9, 
   3.685171761052166*^9}},ExpressionUUID->"0f093802-91b2-4b7e-b4fc-\
5c0649ae1938"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["characteristics", "Section",
 CellChangeTimes->{{3.6839769345684147`*^9, 3.683976958869174*^9}, {
   3.6851721013267317`*^9, 3.685172117851037*^9}, 
   3.691578234392618*^9},ExpressionUUID->"ffdb3ea2-f96a-4c04-8138-\
43f15eda8bb7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CharacteristicsDimension", "[", "Event", "]"}], "=", "6"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.685171637785768*^9, 3.6851716406862917`*^9}, {
   3.685171771673007*^9, 3.685171784823592*^9}, 3.685194584977775*^9, 
   3.685196441634557*^9, 
   3.6914832033775682`*^9},ExpressionUUID->"0a989bf7-73c3-4e45-a7d5-\
9c98e69980f1"],

Cell[CellGroupData[{

Cell["set up undefined characteristics", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683958727456455*^9, 3.6839587370191107`*^9}, {
  3.6839587793697453`*^9, 3.683958818498616*^9}, {3.683959106962901*^9, 
  3.683959108074684*^9}},ExpressionUUID->"f698207f-f2ab-4ffa-a82b-\
1ec90e80846b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "False"}], "}"}], ",", 
     RowBox[{"evspec", ":", 
      RowBox[{"Except", "[", "_Rule", "]"}]}], ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "False"}], "}"}], ",", "Undefined", ",", "evspec",
      ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.685193405712586*^9, 
  3.685193431801092*^9}},ExpressionUUID->"d1a27a5c-479a-40a5-be59-\
a4cca9153de1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "False"}], "}"}], ",", "Undefined", ",", 
     RowBox[{"evspec", ":", 
      RowBox[{"Except", "[", "_Rule", "]"}]}], ",", "opts___Rule"}], "]"}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "False"}], "}"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{"Undefined", ",", 
       RowBox[{"{", 
        RowBox[{"CharacteristicsDimension", "[", "Event", "]"}], "}"}]}], 
      "]"}], ",", "evspec", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683958923673643*^9, 3.683958957650661*^9}, 
   3.68395953006183*^9, {3.685033569404561*^9, 3.6850335694085703`*^9}, {
   3.685171799677648*^9, 3.685171806617219*^9}, {3.6851721444486856`*^9, 
   3.6851721456730633`*^9}, {3.685193435638172*^9, 
   3.685193437212512*^9}},ExpressionUUID->"da4c5cdb-14bc-489a-bea7-\
2b9c987f92d2"]
}, Closed]],

Cell[CellGroupData[{

Cell["expand undefined timing", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683959026765203*^9, 3.683959030893498*^9}, {
  3.683964505178977*^9, 
  3.683964506044327*^9}},ExpressionUUID->"2e982b5e-3140-44fb-ba99-\
92705fafdd22"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{"status_", ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Undefined", ",", "charsb___"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"evspec", ":", 
      RowBox[{"Except", "[", "_Rule", "]"}]}], ",", "opts___Rule"}], "]"}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{"status", ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
        "Undefined", ",", "Undefined", ",", "Undefined", ",", "Undefined"}], 
        "}"}], ",", "charsb"}], "}"}], ",", "\[IndentingNewLine]", "evspec", 
     ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683958754729197*^9, 3.683958774518546*^9}, {
   3.683958829583066*^9, 3.683958907629072*^9}, {3.683958964954804*^9, 
   3.683958973570923*^9}, {3.6839590069879436`*^9, 3.68395902262541*^9}, {
   3.68395907781216*^9, 3.683959098863797*^9}, {3.683959173590906*^9, 
   3.683959178632263*^9}, 3.683959535236364*^9, {3.685033569413889*^9, 
   3.685033569416368*^9}},ExpressionUUID->"a77aae09-bfc2-4753-afdd-\
3c7a23ef98cb"]
}, Closed]],

Cell[CellGroupData[{

Cell["expand Undefined BackgroundGenerator", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683959026765203*^9, 3.683959030893498*^9}, {
   3.683959139611781*^9, 3.683959149162888*^9}, {3.683964513177393*^9, 
   3.683964514887501*^9}, 3.6912374833924713`*^9, 
   3.709374878891251*^9},ExpressionUUID->"4d93bce9-fcf3-473a-ac6b-\
54b7f276c83c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{"status_", ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"charsa_", ",", "Undefined", ",", "charsb___"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"evspec", ":", 
      RowBox[{"Except", "[", "_Rule", "]"}]}], ",", "opts___Rule"}], "]"}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{"status", ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"charsa", ",", 
       RowBox[{"{", 
        RowBox[{"Undefined", ",", "Undefined", ",", " ", "Undefined"}], "}"}],
        ",", "charsb"}], "}"}], ",", "\[IndentingNewLine]", "evspec", ",", 
     "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683959160009802*^9, 3.683959170708824*^9}, {
   3.6839592323540983`*^9, 3.6839592401844482`*^9}, 3.683959539589954*^9, {
   3.685033569420289*^9, 3.685033569423012*^9}, {3.688839100697789*^9, 
   3.688839102966845*^9}, 
   3.691578339184248*^9},ExpressionUUID->"da67b12c-f884-458d-b318-\
77b330244fab"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ready status", "Section",
 CellChangeTimes->{{3.6839769345684147`*^9, 
  3.6839769370653973`*^9}},ExpressionUUID->"d611554f-3770-4589-905e-\
38b19b1cfd03"],

Cell[CellGroupData[{

Cell["set ready status to True", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683958727456455*^9, 3.6839587370191107`*^9}, {
   3.6839587793697453`*^9, 3.683958781370214*^9}, {3.683963257225411*^9, 
   3.683963281087583*^9}, 3.6909855399040318`*^9, 3.690986348924293*^9, {
   3.691483260888874*^9, 3.691483275373251*^9}, {3.691483308805921*^9, 
   3.691483309187549*^9}},ExpressionUUID->"1ef4b491-adae-448a-81f5-\
d6ac1cf05e52"],

Cell["\<\
if no Undefined characteristics left, the BackgroundGenerator has been \
implemented, all ExpandRepeatedEvents instructions have been carried out, and \
precal checks have been done\
\>", "Text",
 CellChangeTimes->{3.691483305286028*^9, 
  3.7093748810650797`*^9},ExpressionUUID->"7a985d63-07cd-4c27-a08e-\
a09e1a43db4d"],

Cell["\<\
also strip out any implemented BackgroundGenerator and ExpandRepeatedEvents \
rules which have already been implemented\
\>", "Text",
 CellChangeTimes->{{3.69098618783429*^9, 3.690986211241982*^9}, {
   3.690986288924268*^9, 3.6909862951934967`*^9}, {3.6909863536896343`*^9, 
   3.69098635884477*^9}, 
   3.709374882377054*^9},ExpressionUUID->"2705352d-bbd1-4937-94cb-\
52204361a98c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "False"}], "}"}], ",", "chars_List", ",", 
      RowBox[{"more", ":", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Except", "[", "_Rule", "]"}], ".."}], ")"}]}], ",", 
      "opts___Rule"}], "]"}], "/;", 
    RowBox[{
     RowBox[{"FreeQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"chars", ",", "more"}], "}"}], ",", "Undefined"}], "]"}], "&&", 
     RowBox[{"MatchQ", "[", 
      RowBox[{"chars", ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", 
         RowBox[{"{", 
          RowBox[{"__", ",", "True"}], "}"}], ",", "_", ",", "_", ",", 
         RowBox[{"True", "|", 
          RowBox[{"{", 
           RowBox[{"__", ",", "True"}], "}"}]}], ",", "___", ",", 
         RowBox[{"True", "|", 
          RowBox[{"{", 
           RowBox[{"__", ",", "True"}], "}"}]}]}], "}"}]}], "]"}]}]}], ":=", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"True", ",", "False"}], "}"}], ",", "chars", ",", "more", ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Sequence", "@@", 
      RowBox[{"FilterRules", "[", 
       RowBox[{
        RowBox[{"{", "opts", "}"}], ",", 
        RowBox[{"Except", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"BackgroundGenerator", "|", "ExpandRepeatedEvents"}], 
           ")"}], "\[Rule]", "_"}], "]"}]}], "\[IndentingNewLine]", "]"}]}]}],
     "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6839586027648153`*^9, 3.6839586045846977`*^9}, {
   3.683958667580065*^9, 3.683958711018367*^9}, 3.683959521095401*^9, {
   3.68396328628866*^9, 3.68396333632965*^9}, {3.685033569432683*^9, 
   3.685033569434887*^9}, {3.685281226312868*^9, 3.6852812290334377`*^9}, {
   3.6904027590724297`*^9, 3.690402769746436*^9}, 3.690438803842902*^9, {
   3.6904389126244783`*^9, 3.69043891345464*^9}, {3.690450128811356*^9, 
   3.690450136424799*^9}, {3.690985545328806*^9, 3.690985561906356*^9}, {
   3.6909856468934927`*^9, 3.690985648224573*^9}, 3.690985920548771*^9, {
   3.6909862149024467`*^9, 3.6909862789295053`*^9}, {3.691483289545179*^9, 
   3.6914832899676113`*^9}, 
   3.709374884417576*^9},ExpressionUUID->"ed2379ec-fd89-4aa3-aae9-\
2c0989bd643b"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["NumericalComputationReadyQ", "Section",
 CellChangeTimes->{{3.6839769345684147`*^9, 3.6839769370653973`*^9}, {
   3.6839769827104883`*^9, 3.6839769835869827`*^9}, 
   3.6839825304588413`*^9},ExpressionUUID->"dc9258ee-f71e-4c2b-ab84-\
cfb33b4da93c"],

Cell[CellGroupData[{

Cell["insert", "Subsubsection",
 CellChangeTimes->{{3.683982593244872*^9, 
  3.683982597125876*^9}},ExpressionUUID->"1cb946b5-0262-4d82-a777-\
9281e2e42ead"],

Cell["\<\
set numeric status to True if no unassigned symbols are detected\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.683958727456455*^9, 3.6839587370191107`*^9}, {
  3.6839587793697453`*^9, 3.683958781370214*^9}, {3.683963257225411*^9, 
  3.683963281087583*^9}, {3.683963372862555*^9, 3.683963411440543*^9}, {
  3.683964593734514*^9, 
  3.683964602800449*^9}},ExpressionUUID->"4bef32a1-bb6d-43ca-ac46-\
d3a6e28be19a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"True", ",", "False"}], "}"}], ",", "chars_List", ",", 
      RowBox[{"more", ":", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Except", "[", "_Rule", "]"}], ".."}], ")"}]}], ",", 
      "opts___Rule"}], "]"}], "/;", 
    RowBox[{"MatchQ", "[", 
     RowBox[{
      RowBox[{"UnassignedSymbols", "[", 
       RowBox[{
        RowBox[{"{", "more", "}"}], ",", "opts"}], "]"}], ",", 
      RowBox[{"{", "}"}]}], "]"}]}], ":=", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"True", ",", "True"}], "}"}], ",", "chars", ",", "more", ",", 
     "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.68396418471384*^9, 3.683964279298118*^9}, {
  3.6839649380405807`*^9, 3.683964939726199*^9}, {3.685033569438465*^9, 
  3.685033569441143*^9}, {3.690438825768672*^9, 
  3.690438851271624*^9}},ExpressionUUID->"00e7f44c-d898-4508-837d-\
25e8848e55a4"]
}, Open  ]],

Cell[CellGroupData[{

Cell["extract", "Subsubsection",
 CellChangeTimes->{{3.683982593244872*^9, 
  3.683982610806408*^9}},ExpressionUUID->"ab9a2857-0b8c-42f2-b237-\
9b3810caaa8d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NumericalComputationReadyQ", "[", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"False", ",", "num_"}], "}"}], ",", 
       RowBox[{"more", ":", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Except", "[", "_Rule", "]"}], "..."}], ")"}]}], ",", 
       "opts___Rule"}], "]"}], ",", "___Rule"}], "]"}], ":=", "False"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683982538783044*^9, 3.6839825780899067`*^9}, {
   3.6839826160631943`*^9, 3.683982620853909*^9}, 
   3.6850335694456244`*^9},ExpressionUUID->"68d66e55-9195-4e6c-a2b3-\
e1a0fdbdd7f7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NumericalComputationReadyQ", "[", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"True", ",", "num_"}], "}"}], ",", 
       RowBox[{"more", ":", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Except", "[", "_Rule", "]"}], "..."}], ")"}]}], ",", 
       "opts___Rule"}], "]"}], ",", "___Rule"}], "]"}], ":=", "num"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683982538783044*^9, 3.6839825780899067`*^9}, 
   3.68503356944973*^9},ExpressionUUID->"eaedc20c-720c-4b32-8098-\
3082d109bde4"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["RepeatEvent expansion", "Section",
 CellChangeTimes->{{3.6839769345684147`*^9, 3.6839769370653973`*^9}, {
   3.6839769827104883`*^9, 3.6839769835869827`*^9}, 3.6839825304588413`*^9, {
   3.689011375913102*^9, 
   3.689011382207841*^9}},ExpressionUUID->"f49a92a7-6e3b-4c06-af02-\
cfdcd8c80aa3"],

Cell[CellGroupData[{

Cell["set expansion type", "Subsection",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, {
   3.6851720063392677`*^9, 3.6851720124143248`*^9}, 3.68517252822554*^9, {
   3.685178235188466*^9, 3.685178240863802*^9}, {3.6851907278022537`*^9, 
   3.685190728734845*^9}, {3.685194472059217*^9, 3.685194480990243*^9}, {
   3.685196703313509*^9, 3.685196711827663*^9}, {3.685196993849419*^9, 
   3.685196995354904*^9}, {3.685197301349476*^9, 3.685197311742599*^9}, {
   3.685208325749523*^9, 3.6852083299331837`*^9}, {3.68884056596734*^9, 
   3.6888405678457747`*^9}, {3.689410742776469*^9, 3.689410743549037*^9}, {
   3.6894108991693363`*^9, 3.689410904057406*^9}, 
   3.68941126622158*^9},ExpressionUUID->"2c465202-b08f-4dc7-b80a-\
5b8a7299201a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"chars", ":", 
        RowBox[{"Repeated", "[", 
         RowBox[{"_", ",", 
          RowBox[{"{", "4", "}"}]}], "]"}]}], ",", "Undefined", ",", 
       "morech___"}], "}"}], ",", "evspec_", ",", "opts___Rule"}], "]"}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"chars", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"ExpandRepeatedEvents", "/.", 
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "Event", "]"}]}], ",", "True", ",", 
         "False"}], "}"}], ",", "morech"}], "}"}], ",", "evspec", ",", 
     "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683976480502823*^9, 
   3.683976634737287*^9}, {3.6839766756355143`*^9, 3.683976688875939*^9}, {
   3.683978094400653*^9, 3.68397814152866*^9}, {3.685033569573359*^9, 
   3.685033569575466*^9}, {3.6851719103554573`*^9, 3.685171975503281*^9}, {
   3.685172339369125*^9, 3.685172341020883*^9}, {3.685172489973385*^9, 
   3.685172520225918*^9}, {3.685196491944193*^9, 3.685196589021195*^9}, {
   3.685196691557366*^9, 3.685196692151346*^9}, {3.685197084908311*^9, 
   3.6851970882027617`*^9}, 3.685197417676444*^9, {3.68941077915304*^9, 
   3.689410791639366*^9}, {3.689410907634926*^9, 
   3.689410935191087*^9}},ExpressionUUID->"02f91f8e-c33b-45e8-824e-\
2743aced3c16"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
set expansion request flag upon application of ExpandRepeatedEvents\
\>", "Subsection",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, {
   3.6851720063392677`*^9, 3.6851720124143248`*^9}, 3.68517252822554*^9, {
   3.685178235188466*^9, 3.685178240863802*^9}, {3.6851907278022537`*^9, 
   3.685190728734845*^9}, {3.685194472059217*^9, 3.685194480990243*^9}, {
   3.685196703313509*^9, 3.685196711827663*^9}, {3.685196993849419*^9, 
   3.685196995354904*^9}, {3.685197301349476*^9, 3.685197311742599*^9}, {
   3.685208325749523*^9, 3.6852083299331837`*^9}, {3.68884056596734*^9, 
   3.6888405678457747`*^9}, {3.689411059447863*^9, 3.6894110631172523`*^9}, {
   3.689419326253831*^9, 3.6894193276615763`*^9}, {3.689421126538595*^9, 
   3.689421149275325*^9}, {3.689661256835268*^9, 
   3.689661273257234*^9}},ExpressionUUID->"99cb97de-a0cb-4be1-a36d-\
a3360f50ac98"],

Cell["\<\
Forced specifications supersede any previously set expansion type\
\>", "Text",
 CellChangeTimes->{{3.689411080782607*^9, 3.689411098002344*^9}, {
  3.689411752367354*^9, 
  3.689411755933936*^9}},ExpressionUUID->"be78e486-b752-46c8-b493-\
47dbe179e671"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ExpandRepeatedEvents", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"False", ",", "numst_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"chars", ":", 
          RowBox[{"Repeated", "[", 
           RowBox[{"_", ",", 
            RowBox[{"{", "4", "}"}]}], "]"}]}], ",", 
         RowBox[{"Undefined", "|", 
          RowBox[{"{", 
           RowBox[{"Undefined", ",", "__"}], "}"}]}], ",", "morech___"}], 
        "}"}], ",", "evspec_", ",", "opts___Rule"}], "]"}], ",", 
     "\[IndentingNewLine]", "exprepevopts___Rule"}], "\[IndentingNewLine]", 
    "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"chars", ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"ExpandRepeatedEvents", "/.", 
           RowBox[{"{", 
            RowBox[{"exprepevopts", ",", "opts"}], "}"}]}], "/.", 
          RowBox[{"Options", "[", "Event", "]"}]}], ",", "True", ",", 
         "False"}], "}"}], ",", "morech"}], "}"}], ",", "evspec", ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Sequence", "@@", 
      RowBox[{"FilterRules", "[", 
       RowBox[{
        RowBox[{"{", "opts", "}"}], ",", 
        RowBox[{"Except", "[", "ExpandRepeatedEvents", "]"}]}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683976480502823*^9, 
   3.683976634737287*^9}, {3.6839766756355143`*^9, 3.683976688875939*^9}, {
   3.683978094400653*^9, 3.68397814152866*^9}, {3.685033569573359*^9, 
   3.685033569575466*^9}, {3.6851719103554573`*^9, 3.685171975503281*^9}, {
   3.685172339369125*^9, 3.685172341020883*^9}, {3.685172489973385*^9, 
   3.685172520225918*^9}, {3.685196491944193*^9, 3.685196589021195*^9}, {
   3.685196691557366*^9, 3.685196692151346*^9}, {3.685197084908311*^9, 
   3.6851970882027617`*^9}, 3.685197417676444*^9, {3.689411067867564*^9, 
   3.6894110770160418`*^9}, {3.689411108371003*^9, 3.6894111806565323`*^9}, {
   3.6894112139978113`*^9, 3.689411253044654*^9}, {3.68941172256238*^9, 
   3.689411735062286*^9}, {3.689412201578143*^9, 3.689412206549265*^9}, {
   3.691506358423452*^9, 
   3.691506382918251*^9}},ExpressionUUID->"bd9f4ef1-9259-4da8-bcf8-\
28f9e6bec5bb"],

Cell[BoxData[
 TemplateBox[{
  "Event","badcharlen",
   "\"The Event characteristics do not have the correct length.\"",2,567,2,
   18076369408218967604,"Local"},
  "MessageTemplate"]], "Message", "MSG",
 CellChangeTimes->{
  3.691676908255803*^9},ExpressionUUID->"b0e1c270-5d5f-4f6b-981e-\
1d335ab8af09"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ExpandRepeatedEvents", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"False", ",", "numst_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"chars", ":", 
          RowBox[{"Repeated", "[", 
           RowBox[{"_", ",", 
            RowBox[{"{", "4", "}"}]}], "]"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"expreptype_", ",", "__"}], "}"}], ",", "morech___"}], 
        "}"}], ",", "evspec_", ",", "opts___Rule"}], "]"}], ",", 
     "\[IndentingNewLine]", "exprepevopts___Rule"}], "\[IndentingNewLine]", 
    "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"chars", ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"ExpandRepeatedEvents", "/.", 
           RowBox[{"{", 
            RowBox[{"exprepevopts", ",", "opts", ",", 
             RowBox[{"ExpandRepeatedEvents", "\[Rule]", "expreptype"}]}], 
            "}"}]}], "/.", 
          RowBox[{"Options", "[", "Event", "]"}]}], ",", "True", ",", 
         "False"}], "}"}], ",", "morech"}], "}"}], ",", "evspec", ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Sequence", "@@", 
      RowBox[{"FilterRules", "[", 
       RowBox[{
        RowBox[{"{", "opts", "}"}], ",", 
        RowBox[{"Except", "[", "ExpandRepeatedEvents", "]"}]}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683976480502823*^9, 
   3.683976634737287*^9}, {3.6839766756355143`*^9, 3.683976688875939*^9}, {
   3.683978094400653*^9, 3.68397814152866*^9}, {3.685033569573359*^9, 
   3.685033569575466*^9}, {3.6851719103554573`*^9, 3.685171975503281*^9}, {
   3.685172339369125*^9, 3.685172341020883*^9}, {3.685172489973385*^9, 
   3.685172520225918*^9}, {3.685196491944193*^9, 3.685196589021195*^9}, {
   3.685196691557366*^9, 3.685196692151346*^9}, {3.685197084908311*^9, 
   3.6851970882027617`*^9}, 3.685197417676444*^9, {3.689411067867564*^9, 
   3.6894110770160418`*^9}, {3.689411108371003*^9, 3.6894111806565323`*^9}, {
   3.6894112139978113`*^9, 3.689411253044654*^9}, {3.68941172256238*^9, 
   3.689411735062286*^9}, {3.689412201578143*^9, 
   3.689412206549265*^9}},ExpressionUUID->"de84e5d2-062e-432d-8700-\
ce18cffdb89a"],

Cell[BoxData[
 TemplateBox[{
  "Event","badcharlen",
   "\"The Event characteristics do not have the correct length.\"",2,568,3,
   18076369408218967604,"Local"},
  "MessageTemplate"]], "Message", "MSG",
 CellChangeTimes->{
  3.691676908309222*^9},ExpressionUUID->"f670e2e0-4ca8-45e9-9b1a-\
2788c2c60a36"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["no action if expansion already done", "Subsection",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, {
   3.6851720063392677`*^9, 3.6851720124143248`*^9}, 3.68517252822554*^9, {
   3.685178235188466*^9, 3.685178240863802*^9}, {3.6851907278022537`*^9, 
   3.685190728734845*^9}, {3.685194472059217*^9, 3.685194480990243*^9}, {
   3.685196703313509*^9, 3.685196711827663*^9}, {3.685196993849419*^9, 
   3.685196995354904*^9}, {3.685197301349476*^9, 3.685197311742599*^9}, {
   3.685208325749523*^9, 3.6852083299331837`*^9}, {3.68884056596734*^9, 
   3.6888405678457747`*^9}, {3.689411059447863*^9, 3.6894110631172523`*^9}, {
   3.689419326253831*^9, 3.6894193276615763`*^9}, {3.689421126538595*^9, 
   3.689421149275325*^9}, {3.689661256835268*^9, 3.689661273257234*^9}, {
   3.691596655138269*^9, 
   3.691596665137157*^9}},ExpressionUUID->"48a053e0-fd14-42e9-9c1f-\
9eb745b13f33"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ExpandRepeatedEvents", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ev", ":", 
      RowBox[{"Event", "[", 
       RowBox[{"st_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"chars", ":", 
           RowBox[{"Repeated", "[", 
            RowBox[{"_", ",", 
             RowBox[{"{", "4", "}"}]}], "]"}]}], ",", 
          RowBox[{"True", "|", 
           RowBox[{"{", 
            RowBox[{"True", ".."}], "}"}], "|", 
           RowBox[{"{", 
            RowBox[{"__", ",", "True"}], "}"}]}], ",", "morech___"}], "}"}], 
        ",", "evspec_", ",", "opts___Rule"}], "]"}]}], ",", 
     "\[IndentingNewLine]", "exprepevopts___Rule"}], "\[IndentingNewLine]", 
    "]"}], ":=", "ev"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683976480502823*^9, 
   3.683976634737287*^9}, {3.6839766756355143`*^9, 3.683976688875939*^9}, {
   3.683978094400653*^9, 3.68397814152866*^9}, {3.685033569573359*^9, 
   3.685033569575466*^9}, {3.6851719103554573`*^9, 3.685171975503281*^9}, {
   3.685172339369125*^9, 3.685172341020883*^9}, {3.685172489973385*^9, 
   3.685172520225918*^9}, {3.685196491944193*^9, 3.685196589021195*^9}, {
   3.685196691557366*^9, 3.685196692151346*^9}, {3.685197084908311*^9, 
   3.6851970882027617`*^9}, 3.685197417676444*^9, {3.689411067867564*^9, 
   3.6894110770160418`*^9}, {3.689411108371003*^9, 3.6894111806565323`*^9}, {
   3.6894112139978113`*^9, 3.689411253044654*^9}, {3.68941172256238*^9, 
   3.689411735062286*^9}, {3.689412201578143*^9, 3.689412206549265*^9}, {
   3.691506571439746*^9, 3.691506590158*^9}, {3.691507511319437*^9, 
   3.691507512285007*^9}, {3.691596508817916*^9, 3.691596512392317*^9}, 
   3.691596761554636*^9, {3.6928581889961233`*^9, 
   3.6928581972806263`*^9}},ExpressionUUID->"ea1630e2-9ca5-4187-b4c5-\
aeea64e72dd7"]
}, Closed]],

Cell[CellGroupData[{

Cell["set expansion implemented flag", "Subsection",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, {
   3.6851720063392677`*^9, 3.6851720124143248`*^9}, 3.68517252822554*^9, {
   3.685178235188466*^9, 3.685178240863802*^9}, {3.6851907278022537`*^9, 
   3.685190728734845*^9}, {3.685194472059217*^9, 3.685194480990243*^9}, {
   3.685196703313509*^9, 3.685196711827663*^9}, {3.685196993849419*^9, 
   3.685196995354904*^9}, {3.685197301349476*^9, 3.6851973245676126`*^9}, {
   3.685208332797408*^9, 3.685208338349524*^9}, {3.68900787195094*^9, 
   3.68900787253239*^9}},ExpressionUUID->"7f03823b-efef-4df3-87f1-\
69690226bbcf"],

Cell[CellGroupData[{

Cell["set True if no contained Repeat", "Subsubsection",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, {
   3.6851720063392677`*^9, 3.6851720124143248`*^9}, 3.68517252822554*^9, {
   3.685178235188466*^9, 3.685178240863802*^9}, {3.685190983158773*^9, 
   3.685190986892653*^9}, {3.6851964589256287`*^9, 3.685196460273987*^9}, {
   3.685196608654578*^9, 3.685196647352812*^9}, {3.685197037117754*^9, 
   3.685197047195401*^9}, {3.6851974721378613`*^9, 3.685197473429678*^9}, {
   3.689410723136648*^9, 3.689410725334772*^9}, 
   3.692848315318467*^9},ExpressionUUID->"ad408c22-087c-4808-92cb-\
d661048dd80c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"chars", ":", 
         RowBox[{"Repeated", "[", 
          RowBox[{"_", ",", 
           RowBox[{"{", "4", "}"}]}], "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"type_", ",", "True", ",", "False"}], "}"}], ",", 
        "morech___"}], "}"}], ",", "evspec_", ",", "opts___Rule"}], "]"}], "/;", 
    RowBox[{"FreeQ", "[", 
     RowBox[{"evspec", ",", "_Repeat"}], "]"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"chars", ",", 
       RowBox[{"{", 
        RowBox[{"type", ",", "True", ",", "True"}], "}"}], ",", "morech"}], 
      "}"}], ",", "evspec", ",", "\[IndentingNewLine]", 
     RowBox[{"Sequence", "@@", 
      RowBox[{"FilterRules", "[", 
       RowBox[{
        RowBox[{"{", "opts", "}"}], ",", 
        RowBox[{"Except", "[", "ExpandRepeatedEvents", "]"}]}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683976480502823*^9, 
   3.683976634737287*^9}, {3.6839766756355143`*^9, 3.683976688875939*^9}, {
   3.683978094400653*^9, 3.68397814152866*^9}, {3.685033569573359*^9, 
   3.685033569575466*^9}, {3.6851719103554573`*^9, 3.685171975503281*^9}, {
   3.685172339369125*^9, 3.685172341020883*^9}, {3.685172489973385*^9, 
   3.685172520225918*^9}, {3.685196491944193*^9, 3.685196589021195*^9}, {
   3.685197055635633*^9, 3.6851970803274117`*^9}, 3.685197488958256*^9, {
   3.685209841575141*^9, 3.685209953684054*^9}, {3.689411288030654*^9, 
   3.689411298107143*^9}, {3.68941879293351*^9, 3.689418800101569*^9}, {
   3.689702039551745*^9, 3.689702083081235*^9}, {3.689702120037437*^9, 
   3.689702150811906*^9}, 
   3.692848316026335*^9},ExpressionUUID->"50a3278a-fba0-4219-9ed6-\
6187f2f11cd7"]
}, Closed]],

Cell[CellGroupData[{

Cell["consolidate True flag", "Subsubsection",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, {
   3.6851720063392677`*^9, 3.6851720124143248`*^9}, 3.68517252822554*^9, {
   3.685178235188466*^9, 3.685178240863802*^9}, {3.685190983158773*^9, 
   3.685190986892653*^9}, {3.6851964589256287`*^9, 3.685196460273987*^9}, {
   3.685196608654578*^9, 3.685196647352812*^9}, {3.685197037117754*^9, 
   3.685197047195401*^9}, {3.6851973804595222`*^9, 3.6851973904260387`*^9}, {
   3.69150716393079*^9, 
   3.691507181606998*^9}},ExpressionUUID->"5dab55db-ebc2-43b9-bc93-\
cce62a4d8c76"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"chars", ":", 
        RowBox[{"Repeated", "[", 
         RowBox[{"_", ",", 
          RowBox[{"{", "4", "}"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"type_", ",", "True", ",", "True"}], "}"}], ",", 
       "morech___"}], "}"}], ",", "evspec_", ",", "opts___Rule"}], "]"}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"chars", ",", "True", ",", "morech"}], "}"}], ",", "evspec", 
     ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683976480502823*^9, 
   3.683976634737287*^9}, {3.6839766756355143`*^9, 3.683976688875939*^9}, {
   3.683978094400653*^9, 3.68397814152866*^9}, {3.685033569573359*^9, 
   3.685033569575466*^9}, {3.6851719103554573`*^9, 3.685171975503281*^9}, {
   3.685172339369125*^9, 3.685172341020883*^9}, {3.685172489973385*^9, 
   3.685172520225918*^9}, {3.685196491944193*^9, 3.685196589021195*^9}, {
   3.685196691557366*^9, 3.685196692151346*^9}, {3.685197084908311*^9, 
   3.6851970882027617`*^9}, {3.6851973316230507`*^9, 3.685197345662799*^9}, {
   3.6894113118856564`*^9, 
   3.68941131343468*^9}},ExpressionUUID->"1216421f-6d55-42ec-87dc-\
020208c2fdb8"]
}, Closed]]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["apply", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, 
   3.683973033466785*^9, {3.684215352256366*^9, 3.684215360934772*^9}, {
   3.6842155164523277`*^9, 3.684215526375304*^9}, {3.6850340141985607`*^9, 
   3.68503401550872*^9}, {3.685193191103589*^9, 3.685193197257063*^9}, {
   3.6894160664839973`*^9, 3.6894160701293507`*^9}, {3.689744845556263*^9, 
   3.689744848161083*^9}},ExpressionUUID->"f4277938-1f08-4c32-b0cd-\
995689f85af3"],

Cell[CellGroupData[{

Cell["apply to event specifications", "Section",
 CellChangeTimes->{
  3.605545643975979*^9, {3.689744852067357*^9, 3.689744857448547*^9}, {
   3.690397321089953*^9, 3.690397324206882*^9}, 
   3.690990156840599*^9},ExpressionUUID->"64d68854-25eb-40e9-9af5-\
c9d4d78df540"],

Cell[CellGroupData[{

Cell["standard event specifications", "Subsection",
 CellChangeTimes->{{3.684215534200747*^9, 3.6842155428798513`*^9}, {
   3.685193161133429*^9, 3.6851931875164557`*^9}, {3.689415538429841*^9, 
   3.689415546522295*^9}, {3.689415977435814*^9, 3.68941601872615*^9}, 
   3.689416081358457*^9},ExpressionUUID->"4b3a67fd-1ccd-4237-ab45-\
b3b6958f213f"],

Cell[CellGroupData[{

Cell["{evgen,duration} event specifications, containing options", \
"Subsubsection",
 CellChangeTimes->{{3.684215534200747*^9, 3.6842155428798513`*^9}, {
  3.685193161133429*^9, 3.68519317395903*^9}, {3.685193205335305*^9, 
  3.6851932094182873`*^9}, {3.689416029297145*^9, 
  3.689416032880047*^9}},ExpressionUUID->"a49e27fe-f213-4a39-aa9b-\
49cd7f4cab47"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", "chars_List", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"evgen_", "?", "ValidGeneratorQ"}], ",", 
       RowBox[{"\[Tau]", ":", "DurationPattern"}], ",", "evspecopts__Rule"}], 
      "}"}], ",", "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", "chars", ",", 
     RowBox[{"{", 
      RowBox[{"evgen", ",", "\[Tau]"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"SimplifyRules", "[", 
      RowBox[{"evspecopts", ",", "opts"}], "]"}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6842153784170113`*^9, 3.684215387489438*^9}, {
   3.684215432702135*^9, 3.6842155022871113`*^9}, 3.68421555382022*^9, {
   3.685033569453355*^9, 3.685033569455604*^9}, 
   3.709373716920126*^9},ExpressionUUID->"d0c6a572-2e0e-4a8c-b550-\
c03ae72111e3"]
}, Closed]],

Cell[CellGroupData[{

Cell["superoperator event specifications, containing options", "Subsubsection",
 CellChangeTimes->{{3.684215534200747*^9, 3.6842155428798513`*^9}, {
  3.685193161133429*^9, 3.68519317395903*^9}, {3.685193213999316*^9, 
  3.6851932233462973`*^9}, {3.68941604086163*^9, 
  3.689416041402294*^9}},ExpressionUUID->"a5b2d11c-f0fe-4f3e-8aae-\
9c4899a03017"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", "chars_List", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"sop_", "?", "SuperoperatorQ"}], ",", "evspecopts__Rule"}], 
      "}"}], ",", "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", "chars", ",", 
     RowBox[{"{", "sop", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"SimplifyRules", "[", 
      RowBox[{"evspecopts", ",", "opts"}], "]"}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6842153784170113`*^9, 3.684215387489438*^9}, {
   3.684215432702135*^9, 3.6842155022871113`*^9}, 3.68421555382022*^9, {
   3.685033569453355*^9, 3.685033569455604*^9}, {3.685193239919093*^9, 
   3.6851932601684732`*^9}},ExpressionUUID->"f395fdb9-d164-495d-bac2-\
64f898bccf80"]
}, Closed]],

Cell[CellGroupData[{

Cell["null event specifications", "Subsubsection",
 CellChangeTimes->{{3.684215534200747*^9, 3.6842155428798513`*^9}, {
  3.685193161133429*^9, 3.68519317395903*^9}, {3.685193213999316*^9, 
  3.6851932233462973`*^9}, {3.68941604086163*^9, 3.689416041402294*^9}, {
  3.690990438539627*^9, 
  3.690990442301284*^9}},ExpressionUUID->"43a47e7e-1b85-429a-ad32-\
7e50ab9ad471"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", "chars_List", ",", 
     RowBox[{"ev", ":", "NonStandardNullEventSpecificationPattern"}], ",", 
     "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", "chars", ",", 
     "StandardNullEventSpecification", ",", "opts"}], "]"}]}], ";"}]], "Input",\

 InitializationCell->True,
 CellChangeTimes->{{3.6842153784170113`*^9, 3.684215387489438*^9}, {
   3.684215432702135*^9, 3.6842155022871113`*^9}, 3.68421555382022*^9, {
   3.685033569453355*^9, 3.685033569455604*^9}, {3.685193239919093*^9, 
   3.6851932601684732`*^9}, 3.6909904510138903`*^9, {3.6909904845636787`*^9, 
   3.6909904947039537`*^9}, {3.690990533874337*^9, 3.6909905347667828`*^9}, {
   3.690990617758419*^9, 3.690990628333179*^9}, {3.690990677435944*^9, 
   3.6909907100077477`*^9}, {3.690990823755624*^9, 
   3.690990833740554*^9}},ExpressionUUID->"3c273add-aad8-4fa1-be95-\
ace77cc6bac2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["apply to EventSpecifications", "Subsection",
 CellChangeTimes->{{3.684215534200747*^9, 3.6842155428798513`*^9}, {
  3.685193161133429*^9, 3.6851931875164557`*^9}, {3.6852519484637747`*^9, 
  3.685251955043714*^9}, {3.6894160773077593`*^9, 3.689416087736155*^9}, {
  3.690990188484152*^9, 
  3.6909901932750463`*^9}},ExpressionUUID->"5b9fdac1-8adf-46d0-8b93-\
01d3ea4352e2"],

Cell["\<\
the EventSpecifications object may only contain a single event specification \
and must have True status\
\>", "Text",
 CellChangeTimes->{{3.685251958542508*^9, 
  3.685252010360044*^9}},ExpressionUUID->"0b594fbf-027f-400c-9b06-\
9c0f2f46982e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", "chars_List", ",", 
     RowBox[{"EventSpecifications", "[", 
      RowBox[{"True", ",", 
       RowBox[{"{", 
        RowBox[{"evspec_", "?", "StandardizedEventSpecificationQ"}], "}"}], 
       ",", "evspecopts___Rule"}], "]"}], ",", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", "chars", ",", "evspec", 
     ",", "\[IndentingNewLine]", 
     RowBox[{"SimplifyRules", "[", 
      RowBox[{"evspecopts", ",", "opts"}], "]"}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6842153784170113`*^9, 3.684215387489438*^9}, {
   3.684215432702135*^9, 3.6842155022871113`*^9}, 3.68421555382022*^9, {
   3.685033569453355*^9, 3.685033569455604*^9}, {3.685252018292478*^9, 
   3.685252060726592*^9}, 
   3.700485008922988*^9},ExpressionUUID->"b288d4a3-b08c-4f1f-af87-\
36a55f54bce5"]
}, Closed]],

Cell[CellGroupData[{

Cell["apply to Repeat", "Subsection",
 CellChangeTimes->{{3.690439779394033*^9, 3.690439790135735*^9}, {
   3.690456197325227*^9, 3.690456213052266*^9}, 3.690456413915045*^9, {
   3.6909901737628508`*^9, 3.690990200089157*^9}, 
   3.6928483168654327`*^9},ExpressionUUID->"348bb81e-e17b-4b72-9fd6-\
7d8a5b1ae7d5"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{"st_", ",", "chars_", ",", 
     RowBox[{"{", 
      RowBox[{"repevs_Repeat", ",", "opts1___Rule"}], "}"}], ",", 
     "opts2___Rule"}], "]"}], ":=", 
   RowBox[{"Event", "[", 
    RowBox[{"st", ",", "chars", ",", "repevs", ",", 
     RowBox[{"SimplifyRules", "[", 
      RowBox[{"opts1", ",", "opts2"}], "]"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6904562160604877`*^9, 3.6904562844662867`*^9}, {
   3.69045648799436*^9, 3.690456501385482*^9}, 
   3.692848317601534*^9},ExpressionUUID->"f7dc1fba-b2b0-4641-b744-\
bc486f2f49e5"]
}, Closed]],

Cell[CellGroupData[{

Cell["apply to PrecalculatedEvent", "Subsection",
 CellChangeTimes->{{3.690439779394033*^9, 3.690439790135735*^9}, {
   3.690456197325227*^9, 3.690456213052266*^9}, 3.690456413915045*^9, {
   3.6909901737628508`*^9, 3.690990200089157*^9}, {3.691492901608882*^9, 
   3.691492908706547*^9}},ExpressionUUID->"0f308417-516f-47c4-9251-\
442dda40b82c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{"st_", ",", "chars_", ",", 
     RowBox[{"{", 
      RowBox[{"precalev_PrecalculatedEvent", ",", "opts1___Rule"}], "}"}], 
     ",", "opts2___Rule"}], "]"}], ":=", 
   RowBox[{"Event", "[", 
    RowBox[{"st", ",", "chars", ",", "precalev", ",", 
     RowBox[{"SimplifyRules", "[", 
      RowBox[{"opts1", ",", "opts2"}], "]"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6904562160604877`*^9, 3.6904562844662867`*^9}, {
  3.69045648799436*^9, 3.690456501385482*^9}, {3.691492911026187*^9, 
  3.69149292156254*^9}},ExpressionUUID->"caf2ab4e-2c2d-4aec-91ce-\
66bc75a29be0"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["apply to Event argument", "Section",
 CellChangeTimes->{
  3.605545643975979*^9, {3.689744852067357*^9, 3.689744868481299*^9}, {
   3.690397312042626*^9, 
   3.69039731516735*^9}},ExpressionUUID->"8bb77fe6-641b-4d17-9186-\
adb037f0e056"],

Cell["merge any BackgroundGenerator options", "Text",
 CellChangeTimes->{{3.6897453750692463`*^9, 3.689745382164629*^9}, 
   3.709374887451087*^9},ExpressionUUID->"2fe800e6-c223-4508-90c7-\
499606fd4366"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"tch2_", ",", 
       RowBox[{"evbackgch2", ":", 
        RowBox[{"{", 
         RowBox[{"evbackg2_", ",", "evbackgimplreq2_", ",", "evbackgimpl2_"}],
          "}"}]}], ",", "morech2__"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"ev", ":", 
      RowBox[{"Event", "[", 
       RowBox[{"stat1_", ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"tch1_", ",", 
          RowBox[{"evbackgch1", ":", 
           RowBox[{"{", 
            RowBox[{
            "evbackg1_", ",", "evbackgimplreq1_", ",", "evbackgimpl1_"}], 
            "}"}]}], ",", "morech1__"}], "}"}], ",", "evspec_", ",", 
        "evopts1___Rule"}], "]"}]}], ",", "\[IndentingNewLine]", 
     "evopts2___Rule"}], "\[IndentingNewLine]", "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "evbackg", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Verbose", "/.", 
           RowBox[{"{", 
            RowBox[{"evopts2", ",", "evopts1"}], "}"}]}], "/.", 
          RowBox[{"Options", "[", "Event", "]"}]}], ",", 
         RowBox[{
         "Print", "[", "\"\<applying Event to another Event\>\"", "]"}]}], 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"Event", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"False", ",", "numst"}], "}"}], ",", "\[IndentingNewLine]", 
         
         RowBox[{"{", 
          RowBox[{"tch2", ",", "\[IndentingNewLine]", 
           RowBox[{"Switch", "[", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"evbackgch2", ",", "evbackgch1"}], "}"}], ",", 
             "\[IndentingNewLine]", "\t", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"Undefined", ",", "__"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"Undefined", ",", "__"}], "}"}]}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"Undefined", ",", "Undefined", ",", "Undefined"}], 
              "}"}], ",", "\[IndentingNewLine]", "\t", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"Undefined", ",", "__"}], "}"}], ",", 
               RowBox[{"Except", "[", 
                RowBox[{"{", 
                 RowBox[{"Undefined", ",", "__"}], "}"}], "]"}]}], "}"}], ",",
              "evbackgch1", ",", "\[IndentingNewLine]", "\t", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"Except", "[", 
                RowBox[{"{", 
                 RowBox[{"Undefined", ",", "__"}], "}"}], "]"}], ",", 
               RowBox[{"{", 
                RowBox[{"Undefined", ",", "__"}], "}"}]}], "}"}], ",", 
             "evbackgch2", ",", "\[IndentingNewLine]", "\t", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"evbackg_", ",", "___"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"evbackg_", ",", "___"}], "}"}]}], "}"}], ",", 
             "evbackgch1", ",", "\[IndentingNewLine]", "\t", "_", ",", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Message", "[", 
                RowBox[{"Event", "::", "backgconf"}], "]"}], ";", "$Failed"}],
               ")"}]}], "\[IndentingNewLine]", "]"}], ",", 
           "\[IndentingNewLine]", "Undefined", ",", "Undefined", ",", 
           "Undefined", ",", "Undefined"}], "\[IndentingNewLine]", "}"}], ",",
          "\[IndentingNewLine]", "evspec", ",", "\[IndentingNewLine]", 
         RowBox[{"SimplifyRules", "[", "\[IndentingNewLine]", 
          RowBox[{"evopts1", ",", "evopts2"}], "\[IndentingNewLine]", "]"}]}],
         "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", ")"}]}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.689744903826355*^9, 3.689744919538123*^9}, {
  3.689745393117938*^9, 3.689745938166157*^9}, {3.6912375785111427`*^9, 
  3.691237604452383*^9}, {3.691237647812539*^9, 3.6912379657851973`*^9}, {
  3.691238015322486*^9, 3.69123805937537*^9}, {3.691579535228169*^9, 
  3.691579536880968*^9}},ExpressionUUID->"26ccd987-e6d3-482a-8461-\
880975604ded"]
}, Closed]],

Cell[CellGroupData[{

Cell["apply to EventList argument", "Section",
 CellChangeTimes->{
  3.605545643975979*^9, {3.689744852067357*^9, 3.689744868481299*^9}, {
   3.690397312042626*^9, 3.69039731516735*^9}, {3.690445339832961*^9, 
   3.690445340520508*^9}},ExpressionUUID->"d7565882-126a-47c9-a447-\
3eb185cf9a23"],

Cell["return EventList argument", "Text",
 CellChangeTimes->{{3.6897453750692463`*^9, 3.689745382164629*^9}, {
  3.690445478751423*^9, 
  3.690445483734311*^9}},ExpressionUUID->"56971b9e-27ed-4371-a5ef-\
6dd7ddc8322d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
    "evst_", ",", "evchs_", ",", "evlist_EventList", ",", "evopts___Rule"}], 
    "]"}], ":=", "evlist"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.689744903826355*^9, 3.689744919538123*^9}, {
  3.689745393117938*^9, 3.689745938166157*^9}, {3.690445353588708*^9, 
  3.690445460918993*^9}, {3.69044549528841*^9, 3.6904455267112017`*^9}, {
  3.6904456404788113`*^9, 3.690445643484922*^9}, {3.6904457407242002`*^9, 
  3.690445743328697*^9}},ExpressionUUID->"add70586-8f2e-44d0-9130-\
bfd8293f7623"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["timing", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, 
   3.683973033466785*^9},ExpressionUUID->"120563db-c9e5-41ee-b8bb-\
bd6373bf0abb"],

Cell[CellGroupData[{

Cell["duration", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 
  3.683972657724471*^9}},ExpressionUUID->"030f5f76-13c1-49bd-89c1-\
0e78faa382be"],

Cell[CellGroupData[{

Cell["insert", "Subsubsection",
 CellChangeTimes->{{3.683979726739832*^9, 
  3.6839797551918573`*^9}},ExpressionUUID->"85b7b8e8-cc5f-44b0-8d3d-\
5e9a19fd223a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Undefined", ",", "moretimingch___"}], "}"}], ",", 
       "morech___"}], "}"}], ",", "evspec_", ",", "opts___Rule"}], "]"}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "dur", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"dur", "=", 
        RowBox[{"EventDuration", "[", "evspec", "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Verbose", "/.", 
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "Event", "]"}]}], ",", 
         RowBox[{"Print", "[", "\"\<inserting duration\>\"", "]"}]}], "]"}], 
       ";", "\[IndentingNewLine]", 
       RowBox[{"If", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"TrueQ", "[", 
          RowBox[{
           RowBox[{"Chop", "@", 
            RowBox[{"N", "@", "dur"}]}], "<", "0"}], "]"}], ",", 
         "\[IndentingNewLine]", "\t", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Message", "[", 
            RowBox[{"Event", "::", "negdur"}], "]"}], ";", "$Failed"}], ")"}],
          ",", "\[IndentingNewLine]", 
         RowBox[{"Event", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"False", ",", "numst"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"dur", ",", "moretimingch"}], "}"}], ",", "morech"}], 
            "}"}], ",", "evspec", ",", "opts"}], "]"}]}], 
        "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", ")"}]}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.683980767477813*^9, 3.683980892077633*^9}, {3.68503356945933*^9, 
   3.685033569463925*^9}, 3.689848495905349*^9, 3.689848536110839*^9, {
   3.689848647575288*^9, 3.6898486547550163`*^9}, {3.690448171474093*^9, 
   3.690448179695253*^9}, {3.6912372871092653`*^9, 3.6912373423926287`*^9}, {
   3.69123738737071*^9, 3.691237389471738*^9}, {3.691238177538726*^9, 
   3.6912381816874733`*^9}, 3.691491660770323*^9, 
   3.7117164232586946`*^9},ExpressionUUID->"c517dd5c-1b65-4dca-9d53-\
d826cc34eade"]
}, Closed]],

Cell[CellGroupData[{

Cell["extract", "Subsubsection",
 CellChangeTimes->{{3.683979726739832*^9, 
  3.6839797455615473`*^9}},ExpressionUUID->"30002d7b-00f4-4bdb-b46c-\
3803a3f57cb0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventDuration", "[", 
    RowBox[{"Event", "[", 
     RowBox[{"status_", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"dur_", ",", "moretimingch___"}], "}"}], ",", "morech___"}], 
       "}"}], ",", "evspec_", ",", "opts___Rule"}], "]"}], "]"}], ":=", 
   "dur"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683979669007251*^9, 3.683979709514778*^9}, {
   3.6839800680331697`*^9, 3.6839800796436453`*^9}, 3.6850335694674177`*^9, 
   3.7117164232682204`*^9},ExpressionUUID->"0b3a77c8-f6a5-48e6-b405-\
a5783882faff"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["initial and final time points", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
  3.683972979124955*^9, 
  3.683972983330543*^9}},ExpressionUUID->"82c5d2f9-f04a-4a1f-aadb-\
845f1a4506d1"],

Cell[CellGroupData[{

Cell["trap conflict", "Subsubsection",
 CellChangeTimes->{{3.683979764654413*^9, 3.683979767388132*^9}, {
  3.684000030541182*^9, 
  3.6840000326924334`*^9}},ExpressionUUID->"9cc803c5-6806-49a7-bff9-\
ad90e3949695"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"dur_", ",", 
         RowBox[{"{", 
          RowBox[{"tb_", ",", "ta_"}], "}"}], ",", "moretimingch___"}], "}"}],
        ",", "morech___"}], "}"}], ",", "evspec_", ",", "opts___Rule"}], 
    "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"newtb", ",", "newta", ",", "newsttb", ",", "newstta"}], "}"}], 
     ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Message", "[", 
         RowBox[{"Event", "::", "tconflict"}], "]"}], ";", "$Failed"}], 
       "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"newtb", ",", "newta"}], "}"}], "=", 
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{"FinalTimePoint", ",", "InitialTimePoint"}], "}"}], "/.", 
           
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "Event", "]"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"TrueQ", "[", 
          RowBox[{
           RowBox[{"Chop", "[", 
            RowBox[{"newtb", "-", "tb"}], "]"}], "\[NotEqual]", "0"}], "]"}], 
         "||", 
         RowBox[{"TrueQ", "[", 
          RowBox[{
           RowBox[{"Chop", "[", 
            RowBox[{"newta", "-", "ta"}], "]"}], "\[NotEqual]", "0"}], 
          "]"}]}]}], "\[IndentingNewLine]", ")"}]}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6839805406968193`*^9, 3.683980682667598*^9}, {
  3.685033569471175*^9, 3.685033569475665*^9}, {3.691582851775693*^9, 
  3.691582899581706*^9}},ExpressionUUID->"17cea919-6fc7-4934-b54b-\
4aa1d4f023a4"]
}, Closed]],

Cell[CellGroupData[{

Cell["insert", "Subsubsection",
 CellChangeTimes->{{3.683979764654413*^9, 
  3.683979767388132*^9}},ExpressionUUID->"97bb7cff-cdf1-44dd-9ec9-\
cb8f98b543b6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ready_", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"dur", ":", 
          RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "Undefined", ",", 
         "moretimingch___"}], "}"}], ",", "morech___"}], "}"}], ",", 
     "evspec_", ",", "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ta", ",", "tb"}], "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Verbose", "/.", 
            RowBox[{"{", "opts", "}"}]}], "/.", 
           RowBox[{"Options", "[", "Event", "]"}]}], ",", 
          RowBox[{"Print", "[", "\"\<inserting {tb,ta}\>\"", "]"}]}], "]"}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"Event", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"ready", ",", "numst"}], "}"}], ",", 
          RowBox[{"{", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"dur", ",", "\[IndentingNewLine]", 
              RowBox[{"Switch", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"tb", ",", "ta"}], "}"}], ",", "\[IndentingNewLine]", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Except", "[", 
                   RowBox[{"Automatic", "|", "Undefined"}], "]"}], ",", 
                  RowBox[{"Automatic", "|", "Undefined"}]}], "}"}], ",", 
                "\[IndentingNewLine]", "\t", 
                RowBox[{"{", 
                 RowBox[{"tb", ",", 
                  RowBox[{"tb", "-", "dur"}]}], "}"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Automatic", "|", "Undefined"}], ",", 
                  RowBox[{"Except", "[", 
                   RowBox[{"Automatic", "|", "Undefined"}], "]"}]}], "}"}], 
                ",", "\[IndentingNewLine]", "\t", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"ta", "+", "dur"}], ",", "ta"}], "}"}], ",", 
                "\[IndentingNewLine]", "_", ",", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"Message", "[", 
                   RowBox[{"Event", "::", "badtimepts"}], "]"}], ";", 
                  "$Failed"}], ")"}]}], "\[IndentingNewLine]", "]"}], ",", 
              "\[IndentingNewLine]", "moretimingch"}], "}"}], ",", "morech"}],
            "}"}], ",", "\[IndentingNewLine]", "evspec", ",", 
          RowBox[{"Sequence", "@@", 
           RowBox[{"Cases", "[", 
            RowBox[{
             RowBox[{"{", "opts", "}"}], ",", 
             RowBox[{"Except", "[", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"FinalTimePoint", "|", "InitialTimePoint"}], ")"}], 
               "\[Rule]", "_"}], "]"}]}], "]"}]}]}], "]"}]}], 
       "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"tb", ",", "ta"}], "}"}], "=", 
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{"FinalTimePoint", ",", "InitialTimePoint"}], "}"}], "/.", 
           
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "Event", "]"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"!", 
         RowBox[{"MatchQ", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"tb", ",", "ta"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Automatic", "|", "Undefined"}], ",", 
             RowBox[{"Automatic", "|", "Undefined"}]}], "}"}]}], "]"}]}]}], 
       "\[IndentingNewLine]", ")"}]}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839730673310833`*^9, 3.683973101981674*^9}, {3.683973149371945*^9, 
   3.6839734156761*^9}, {3.683973451960537*^9, 3.683973475698845*^9}, {
   3.6839736462618427`*^9, 3.683973679564999*^9}, {3.68397413593371*^9, 
   3.683974154427085*^9}, {3.6839804964580173`*^9, 3.683980530614625*^9}, {
   3.683999991296249*^9, 3.6839999989798326`*^9}, {3.6850335694797573`*^9, 
   3.685033569491074*^9}, {3.689848217062913*^9, 3.689848249164035*^9}, 
   3.689848290940682*^9, {3.689848671894956*^9, 3.68984867582812*^9}, {
   3.690448194067181*^9, 
   3.690448200683037*^9}},ExpressionUUID->"0fb825d9-1dec-4c4d-a7b5-\
7d73a052de44"]
}, Closed]],

Cell[CellGroupData[{

Cell["Repeat", "Subsubsection",
 CellChangeTimes->{{3.683979764654413*^9, 3.683979767388132*^9}, {
   3.689705530562594*^9, 3.689705542799768*^9}, {3.6897056043849983`*^9, 
   3.689705606551056*^9}, 
   3.6928483184100733`*^9},ExpressionUUID->"7b437d82-78cc-47bf-819d-\
e20264f71606"],

Cell["\<\
pass down initial/final time point information to Repeat, providing that all \
expansions have been done and that initial/final time points exist\
\>", "Text",
 CellChangeTimes->{{3.689705609407236*^9, 3.689705654923925*^9}, {
   3.689706272950098*^9, 3.6897062759959793`*^9}, {3.689848465249506*^9, 
   3.689848478174222*^9}, 
   3.692848319115965*^9},ExpressionUUID->"c39e1f14-1d61-4366-aa04-\
661579391a26"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"dur_", ",", 
          RowBox[{"{", 
           RowBox[{"tb_", ",", "ta_"}], "}"}], ",", "moretimingch___"}], 
         "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"backg", ":", 
           RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "False", ",", 
          "False"}], "}"}], ",", "hilbch_", ",", "pcpch_", ",", 
        RowBox[{"exprepch", ":", 
         RowBox[{"True", "|", 
          RowBox[{"{", 
           RowBox[{"exptype_", ",", "expreq_", ",", "True"}], "}"}]}]}]}], 
       "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"Repeat", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"EventList", "[", 
         RowBox[{"evliststatus_", ",", 
          RowBox[{"evlistch", ":", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"evlisttimech", ":", 
              RowBox[{"{", 
               RowBox[{
               "evlistdur_", ",", "Undefined", ",", "moreevlisttimech__"}], 
               "}"}]}], ",", "evlistbackgch_", ",", "moreevlistch___"}], 
            "}"}]}], ",", "events_List", ",", "\[IndentingNewLine]", 
          "evlistopts___Rule"}], "\[IndentingNewLine]", "]"}], ",", 
        "\[IndentingNewLine]", "nrep_", ",", "\[IndentingNewLine]", 
        "repevopts___Rule"}], "\[IndentingNewLine]", "]"}], ",", 
      "\[IndentingNewLine]", "opts___Rule"}], "]"}]}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"dur", ",", 
         RowBox[{"{", 
          RowBox[{"tb", ",", "ta"}], "}"}], ",", "moretimingch"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"backg", ",", "True", ",", "False"}], "}"}], ",", "hilbch", 
       ",", "pcpch", ",", "exprepch"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Repeat", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"EventList", "[", 
        RowBox[{
        "evliststatus", ",", "evlistch", ",", "\[IndentingNewLine]", "events",
          ",", "\[IndentingNewLine]", 
         RowBox[{"SimplifyRules", "[", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"InitialTimePoint", "\[Rule]", "ta"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"FinalTimePoint", "\[Rule]", "Automatic"}], ",", 
           "\[IndentingNewLine]", "evlistopts"}], "\[IndentingNewLine]", 
          "]"}]}], "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
       "nrep", ",", "\[IndentingNewLine]", "repevopts"}], 
      "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", "opts"}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683974333334736*^9, 
   3.683974359243943*^9}, {3.68397440341503*^9, 3.683974453895319*^9}, {
   3.683974485206881*^9, 3.683974500280587*^9}, {3.683974637224945*^9, 
   3.683974642623952*^9}, {3.683974679977281*^9, 3.683974740133045*^9}, {
   3.6839747909097548`*^9, 3.6839748224301777`*^9}, {3.68397496707168*^9, 
   3.683974981930236*^9}, {3.685033569546874*^9, 3.6850335695495977`*^9}, {
   3.688839430247795*^9, 3.688839449996852*^9}, {3.68883989841578*^9, 
   3.688839899158692*^9}, 3.689416890355986*^9, 3.689417028829543*^9, {
   3.689417386291594*^9, 3.689417589758437*^9}, {3.6894176372531357`*^9, 
   3.689417666094516*^9}, {3.6894179356197157`*^9, 3.6894179409054823`*^9}, {
   3.6894179904474*^9, 3.689418088544161*^9}, 3.6894184207895727`*^9, {
   3.689418514026092*^9, 3.689418516215681*^9}, 3.6894185895608463`*^9, {
   3.689705668063191*^9, 3.689705674580228*^9}, {3.689705706062847*^9, 
   3.689705736858597*^9}, {3.689706039161312*^9, 3.689706258733636*^9}, 
   3.6916767125410223`*^9, {3.692848319941*^9, 
   3.692848320786384*^9}},ExpressionUUID->"16bd16fd-e2d4-4955-bee3-\
f7be4afbc8ad"]
}, Closed]],

Cell[CellGroupData[{

Cell["extract", "Subsubsection",
 CellChangeTimes->{{3.683979764654413*^9, 
  3.683979774036868*^9}},ExpressionUUID->"8a898fe8-63ee-4b70-bf4e-\
7301e545bad8"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"InitialTimePoint", "[", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"dur_", ",", "Undefined", ",", "moretimingch___"}], "}"}], 
         ",", "morech___"}], "}"}], ",", "evspec_", ",", "opts___Rule"}], 
      "]"}], ",", "___Rule"}], "]"}], ":=", "Undefined"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6894247868684397`*^9, 
  3.68942479585989*^9}},ExpressionUUID->"31459d09-ca3b-488c-9165-\
63af4ef4b69e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"InitialTimePoint", "[", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"dur_", ",", 
           RowBox[{"{", 
            RowBox[{"tb_", ",", "ta_"}], "}"}], ",", "moretimingch___"}], 
          "}"}], ",", "morech___"}], "}"}], ",", "evspec_", ",", 
       "opts___Rule"}], "]"}], ",", "___Rule"}], "]"}], ":=", "ta"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683979781879389*^9, 3.683979821255773*^9}, 
   3.685033569494997*^9},ExpressionUUID->"20be9f17-df00-4a2f-8ea4-\
cc06b8f4799a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FinalTimePoint", "[", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"dur_", ",", "Undefined", ",", "moretimingch___"}], "}"}], 
         ",", "morech___"}], "}"}], ",", "evspec_", ",", "opts___Rule"}], 
      "]"}], ",", "___Rule"}], "]"}], ":=", "Undefined"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6894247868684397`*^9, 
  3.6894248081237593`*^9}},ExpressionUUID->"0c6bc07a-b229-4392-9e00-\
faaaa8a6d6ce"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FinalTimePoint", "[", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"dur_", ",", 
           RowBox[{"{", 
            RowBox[{"tb_", ",", "ta_"}], "}"}], ",", "moretimingch___"}], 
          "}"}], ",", "morech___"}], "}"}], ",", "evspec_", ",", 
       "opts___Rule"}], "]"}], ",", "___Rule"}], "]"}], ":=", "tb"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683979781879389*^9, 3.683979835364011*^9}, 
   3.68503356949868*^9},ExpressionUUID->"e750d540-dafe-402d-bd0e-\
17a905f461d3"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["permitted time shift", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
  3.683972979124955*^9, 3.683972983330543*^9}, {3.683975121606881*^9, 
  3.683975124294643*^9}},ExpressionUUID->"dcb44ddc-951c-47ba-a225-\
b7fac540df71"],

Cell[CellGroupData[{

Cell["insert", "Subsubsection",
 CellChangeTimes->{{3.683979764654413*^9, 
  3.683979767388132*^9}},ExpressionUUID->"2c1f9e90-4e02-4cde-b306-\
5c90dc682bb8"],

Cell["\<\
Implement if BackgroundGenerator has been defined (even if not yet \
implemented)\
\>", "Text",
 CellChangeTimes->{{3.683975126804194*^9, 3.6839751371992807`*^9}, {
   3.6851908422061663`*^9, 3.68519084610798*^9}, {3.6851908853171577`*^9, 
   3.685190897263502*^9}, {3.689416593676862*^9, 3.689416595489444*^9}, {
   3.689847818282947*^9, 3.6898478688604727`*^9}, 
   3.709374888724209*^9},ExpressionUUID->"6906c4ac-5c09-4d43-a124-\
a93817927e80"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
         "dur_", ",", "timepts_", ",", "Undefined", ",", "moretimingch___"}], 
         "}"}], ",", 
        RowBox[{"backgch", ":", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"backg", ":", 
            RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "_", ",", "_"}],
           "}"}]}], ",", "morech___"}], "}"}], ",", "evspec_", ",", 
      "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", "pts", "}"}], ",", 
      RowBox[{
       RowBox[{"(", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Verbose", "/.", 
             RowBox[{"{", "opts", "}"}]}], "/.", 
            RowBox[{"Options", "[", "Event", "]"}]}], ",", 
           RowBox[{
           "Print", "[", "\"\<inserting PermittedTimeShift\>\"", "]"}]}], 
          "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"Event", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"False", ",", "numst"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
              "dur", ",", "timepts", ",", "pts", ",", "moretimingch"}], "}"}],
              ",", "backgch", ",", "morech"}], "}"}], ",", "evspec", ",", 
           "opts"}], "]"}]}], "\[IndentingNewLine]", ")"}], "/;", 
       RowBox[{"(", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"pts", "=", 
          RowBox[{"PermittedTimeShift", "[", 
           RowBox[{"evspec", ",", 
            RowBox[{"BackgroundGenerator", "->", "backg"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"!", 
          RowBox[{"MatchQ", "[", 
           RowBox[{"pts", ",", "Undefined"}], "]"}]}]}], 
        "\[IndentingNewLine]", ")"}]}]}], "]"}]}], ";"}], " "}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839730673310833`*^9, 3.683973101981674*^9}, {3.683973149371945*^9, 
   3.6839734156761*^9}, {3.683973451960537*^9, 3.683973475698845*^9}, {
   3.6839736462618427`*^9, 3.683973679564999*^9}, {3.68397413593371*^9, 
   3.683974154427085*^9}, {3.683975165827835*^9, 3.683975303283432*^9}, {
   3.683975518769175*^9, 3.683975550989229*^9}, {3.683975611081381*^9, 
   3.683975614758295*^9}, {3.683975718424601*^9, 3.6839757265040693`*^9}, {
   3.685033569502289*^9, 3.6850335695046453`*^9}, {3.685190849270446*^9, 
   3.685190880421232*^9}, {3.6894166031655827`*^9, 3.6894166300603743`*^9}, 
   3.689847821873576*^9, {3.6898487128086643`*^9, 3.689848721254016*^9}, {
   3.6898493433769913`*^9, 3.6898493701924677`*^9}, {3.6904482134217443`*^9, 
   3.690448238655051*^9}, 
   3.709374890312801*^9},ExpressionUUID->"ab69c563-b95a-47a6-a420-\
d519b026ff6e"]
}, Closed]],

Cell[CellGroupData[{

Cell["extract", "Subsubsection",
 CellChangeTimes->{{3.683979764654413*^9, 
  3.683979774036868*^9}},ExpressionUUID->"0f7a2118-4bbb-43f1-954e-\
0e9831b62920"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
          "dur_", ",", "timepts_", ",", "pts_", ",", "moretimingch___"}], 
          "}"}], ",", "morech___"}], "}"}], ",", "evspec_", ",", 
       "opts___Rule"}], "]"}], ",", "___Rule"}], "]"}], ":=", "pts"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683979781879389*^9, 3.683979821255773*^9}, {
   3.683979901994973*^9, 3.683979916587043*^9}, {3.68398010211156*^9, 
   3.683980110356081*^9}, 3.68398017786053*^9, 
   3.685033569510149*^9},ExpressionUUID->"eeb9a307-90c5-4f7c-a18d-\
2b66d7a5a08d"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["StandardizedInterval", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683972979124955*^9, 3.683972983330543*^9}, {3.683975121606881*^9, 
   3.683975124294643*^9}, {3.68397585606467*^9, 3.683975860766287*^9}, 
   3.683979956583666*^9},ExpressionUUID->"64855f77-290a-4be0-b77f-\
0768adbdbced"],

Cell["\<\
implement only if time points and permitted time shift have been defined\
\>", "Text",
 CellChangeTimes->{{3.683975126804194*^9, 3.6839751371992807`*^9}, {
  3.68397587060152*^9, 
  3.6839758797351437`*^9}},ExpressionUUID->"64ce4b43-76b2-423c-9e3a-\
2538bb52e979"],

Cell[CellGroupData[{

Cell["insert", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.54306105604321*^9, 3.5430610664577312`*^9}, {
   3.543062494985586*^9, 3.5430625010874157`*^9}, 3.683976182937875*^9, {
   3.6839799398099117`*^9, 
   3.68397997971033*^9}},ExpressionUUID->"b4a13f02-c554-4fb4-b3a0-\
c62da0202a86"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"dur_", ",", 
         RowBox[{"{", 
          RowBox[{"tb_", ",", "ta_"}], "}"}], ",", 
         RowBox[{"pts", ":", 
          RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "Undefined"}], 
        "}"}], ",", "morech___"}], "}"}], ",", "evspec_", ",", 
     "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Verbose", "/.", 
         RowBox[{"{", "opts", "}"}]}], "/.", 
        RowBox[{"Options", "[", "Event", "]"}]}], ",", 
       RowBox[{
       "Print", "[", "\"\<inserting StandardizedInterval\>\"", "]"}]}], "]"}],
      ";", "\[IndentingNewLine]", 
     RowBox[{"Event", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"False", ",", "numst"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"dur", ",", 
           RowBox[{"{", 
            RowBox[{"tb", ",", "ta"}], "}"}], ",", "pts", ",", 
           RowBox[{"StandardizedInterval", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"tb", ",", "ta"}], "}"}], ",", "pts"}], "]"}]}], "}"}], 
         ",", "morech"}], "}"}], ",", "\[IndentingNewLine]", "evspec", ",", 
       "opts"}], "]"}]}], ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839730673310833`*^9, 3.683973101981674*^9}, {3.683973149371945*^9, 
   3.6839734156761*^9}, {3.683973451960537*^9, 3.683973475698845*^9}, {
   3.6839736462618427`*^9, 3.683973679564999*^9}, {3.68397413593371*^9, 
   3.683974154427085*^9}, {3.683975165827835*^9, 3.683975303283432*^9}, {
   3.683975518769175*^9, 3.683975550989229*^9}, {3.683975611081381*^9, 
   3.683975614758295*^9}, {3.683975718424601*^9, 3.6839757265040693`*^9}, {
   3.683975889885955*^9, 3.683975944990078*^9}, {3.683976054866651*^9, 
   3.683976077613914*^9}, {3.68397614157769*^9, 3.683976179005468*^9}, {
   3.683976227116561*^9, 3.683976238578314*^9}, 3.683979983856182*^9, 
   3.683980125738283*^9, {3.684000198958784*^9, 3.684000207057146*^9}, {
   3.684000284089324*^9, 3.68400030054813*^9}, {3.6850335695143833`*^9, 
   3.6850335695167017`*^9}, {3.689848732911871*^9, 3.689848742405058*^9}, {
   3.690448220888996*^9, 
   3.6904482317830143`*^9}},ExpressionUUID->"507ceb13-7c16-4c10-b0cd-\
3cf220965689"]
}, Closed]],

Cell[CellGroupData[{

Cell["extract", "Subsubsection",
 CellChangeTimes->{{3.683979764654413*^9, 
  3.683979774036868*^9}},ExpressionUUID->"81ceceae-d0e4-4ed8-81ab-\
f85f010e14ea"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizedInterval", "[", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
          "dur_", ",", "timepts_", ",", "pts_", ",", "standintv_", ",", 
           "moretimingch___"}], "}"}], ",", "morech___"}], "}"}], ",", 
       "evspec_", ",", "opts___Rule"}], "]"}], ",", "___Rule"}], "]"}], ":=", 
   "standintv"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683979781879389*^9, 3.683979821255773*^9}, {
   3.683979901994973*^9, 3.683979916587043*^9}, 3.683980041030056*^9, {
   3.683980132621498*^9, 3.683980192916423*^9}, 
   3.68503356952022*^9},ExpressionUUID->"d9c69c5c-e330-48df-b247-\
fff98b2fa088"]
}, Closed]],

Cell[CellGroupData[{

Cell["StandardizedInterval routine", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.54306105604321*^9, 3.5430610664577312`*^9}, {
   3.543062494985586*^9, 3.5430625010874157`*^9}, 3.683976182937875*^9, {
   3.6839799398099117`*^9, 
   3.683979968481955*^9}},ExpressionUUID->"9c51666b-9365-4db8-b2bb-\
fbc54b188ae0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizedInterval", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"tb_", ",", "ta_"}], "}"}], ",", "None"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{"tb", ",", "ta"}], "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.543061071317506*^9, 3.543061134256523*^9}, 
   3.683976188000587*^9, {3.683979934732354*^9, 
   3.683979936102715*^9}},ExpressionUUID->"ad037959-4cbc-4894-911f-\
b0cc05ac1533"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizedInterval", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"tb_", ",", "ta_"}], "}"}], ",", "All"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"ta", "-", "tb"}]}], "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.543061071317506*^9, 3.543061120356283*^9}, 
   3.683976189257948*^9, 
   3.683979941843032*^9},ExpressionUUID->"767ce42b-c83e-4b18-9c20-\
d276ec41a71e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"StandardizedInterval", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"tb_", ",", "ta_"}], "}"}], ",", 
      RowBox[{"T_", "?", "NumericQ"}]}], "]"}], "/;", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Chop", "[", "T", "]"}], "\[NotEqual]", "0"}], ")"}]}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "\[CapitalDelta]", "}"}], ",", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[CapitalDelta]", "=", 
        RowBox[{"T", "\[Times]", 
         RowBox[{"Floor", "[", 
          RowBox[{"tb", "/", "T"}], "]"}]}]}], ";", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"tb", "-", "\[CapitalDelta]"}], ",", 
         RowBox[{"ta", "-", "\[CapitalDelta]"}]}], "}"}]}], ")"}]}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.543061071317506*^9, 3.543061120356283*^9}, {
   3.543061174816175*^9, 3.543061180727231*^9}, {3.543061241219453*^9, 
   3.5430613232894373`*^9}, {3.543061418406011*^9, 3.543061423220615*^9}, 
   3.683976190686952*^9, 
   3.683979943057859*^9},ExpressionUUID->"cbd5c911-ce59-457c-96a4-\
ab05c0bedade"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"StandardizedInterval", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"tb_", ",", "ta_"}], "}"}], ",", 
      RowBox[{"T_", "?", "NumericQ"}]}], "]"}], "/;", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Chop", "[", "T", "]"}], "==", "0"}], ")"}]}], ":=", 
   RowBox[{"StandardizedInterval", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"tb", ",", "ta"}], "}"}], ",", "All"}], "]"}]}], ";"}]], "Input",\

 InitializationCell->True,
 CellChangeTimes->{{3.543061071317506*^9, 3.543061120356283*^9}, {
  3.543061174816175*^9, 3.543061180727231*^9}, {3.543061241219453*^9, 
  3.5430613232894373`*^9}, {3.543061418406011*^9, 3.543061473043766*^9}, {
  3.683976192239615*^9, 3.683976193589382*^9}, {3.683979944588315*^9, 
  3.6839799455893593`*^9}},ExpressionUUID->"946abef3-f939-4a22-bc3f-\
e1ce34892f97"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizedInterval", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"tb_", ",", "ta_"}], "}"}], ",", 
     RowBox[{"T", ":", "DurationPattern"}]}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "\[CapitalDelta]", "}"}], ",", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[CapitalDelta]", "=", 
        RowBox[{"T", "\[Times]", 
         RowBox[{"Floor", "[", 
          RowBox[{"tb", "/", "T"}], "]"}]}]}], ";", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"tb", "-", "\[CapitalDelta]"}], ",", 
         RowBox[{"ta", "-", "\[CapitalDelta]"}]}], "}"}]}], ")"}]}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.543061071317506*^9, 3.543061120356283*^9}, {
   3.543061174816175*^9, 3.543061180727231*^9}, {3.543061241219453*^9, 
   3.5430613232894373`*^9}, {3.543061418406011*^9, 3.543061423220615*^9}, {
   3.683435058940647*^9, 3.683435064995463*^9}, {3.6834522212415037`*^9, 
   3.683452222308874*^9}, 3.6839761953108788`*^9, 
   3.683979946939542*^9},ExpressionUUID->"c66a516a-416c-47ee-88da-\
1c255efbdc3b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizedInterval", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"tb_", ",", "ta_"}], "}"}], ",", "_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{"tb", ",", "ta"}], "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.543061518442151*^9, 3.543061524328844*^9}, 
   3.6839761968633823`*^9, 
   3.683979948615869*^9},ExpressionUUID->"ac603640-ffe7-4b5b-bda9-\
42d7ec28f2eb"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Repeat", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683972979124955*^9, 3.683972983330543*^9}, {3.683975121606881*^9, 
   3.683975124294643*^9}, {3.68397585606467*^9, 3.683975860766287*^9}, 
   3.683979956583666*^9, {3.690456332841338*^9, 3.690456335344699*^9}, 
   3.692848321602676*^9},ExpressionUUID->"1dae09ac-723e-4040-87fc-\
496135ef2641"],

Cell[CellGroupData[{

Cell["pass down timing to EventList inside Repeat", "Subsection",
 CellChangeTimes->{{3.690439779394033*^9, 3.690439790135735*^9}, 
   3.692848322293298*^9},ExpressionUUID->"a286657e-f168-4dda-997e-\
08ffe7bfa022"],

Cell["\<\
special checks sometimes need to be done after RepeatEvent expansion\
\>", "Text",
 CellChangeTimes->{{3.6904399137451153`*^9, 
  3.69043992689646*^9}},ExpressionUUID->"553b7770-3246-4337-9b78-\
feb25a3de1aa"],

Cell["\<\
If the RepeatEvent expansion has been done, ensure that the EventList inside \
Repeat inherits any implemented timing information from the enclosing Event\
\>", "Text",
 CellChangeTimes->{{3.69039753912848*^9, 3.690397558821666*^9}, {
   3.6904035400501823`*^9, 3.690403575037136*^9}, {3.690403651282372*^9, 
   3.690403657109928*^9}, 
   3.692848323056748*^9},ExpressionUUID->"8e16a0a4-251a-4b26-b3d0-\
05971b0863ad"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"evstatus", ":", 
       RowBox[{"{", 
        RowBox[{"False", ",", "numst_"}], "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"evch", ":", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"evT_", ",", 
           RowBox[{"{", 
            RowBox[{"tb_", ",", 
             RowBox[{"ta", ":", 
              RowBox[{"Except", "[", "Undefined", "]"}]}]}], "}"}], ",", 
           "moreevtch__"}], "}"}], ",", "moreevch__", ",", 
         RowBox[{"True", "|", 
          RowBox[{"{", 
           RowBox[{"__", ",", "True"}], "}"}]}]}], "}"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"Repeat", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"evlist", ":", 
         RowBox[{"EventList", "[", "\[IndentingNewLine]", 
          RowBox[{"evliststatus_", ",", "\[IndentingNewLine]", 
           RowBox[{"evlistchars", ":", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"evlistT_", ",", "Undefined", ",", "moreevlisttch__"}],
                "}"}], ",", "moreevlistch__"}], "}"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"events", ":", 
            RowBox[{"{", 
             RowBox[{"_Event", ".."}], "}"}]}], ",", "\[IndentingNewLine]", 
           "evlistopts___Rule"}], "]"}]}], ",", "\[IndentingNewLine]", 
        "nrep_", ",", "\[IndentingNewLine]", "repevopts___Rule"}], 
       "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
      "evopts___Rule"}], "\[IndentingNewLine]", "]"}]}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Verbose", "/.", 
         RowBox[{"{", "evopts", "}"}]}], "/.", 
        RowBox[{"Options", "[", "Event", "]"}]}], ",", 
       RowBox[{
       "Print", "[", "\"\<passing down timing to EventList inside Repeat\>\"",
         "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Event", "[", 
      RowBox[{"evstatus", ",", "evch", ",", "\[IndentingNewLine]", 
       RowBox[{"Repeat", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"EventList", "[", "\[IndentingNewLine]", 
          RowBox[{
          "evliststatus", ",", "evlistchars", ",", "\[IndentingNewLine]", 
           RowBox[{"events", "/.", 
            RowBox[{
             RowBox[{"Event", "[", 
              RowBox[{"ievst_", ",", 
               RowBox[{"{", 
                RowBox[{"ievttch_", ",", "moreievch__"}], "}"}], ",", 
               "ievspecs_", ",", "ievopts___Rule"}], "]"}], "\[RuleDelayed]", 
             
             RowBox[{"Event", "[", 
              RowBox[{"ievst", ",", 
               RowBox[{"{", 
                RowBox[{"Undefined", ",", "moreievch"}], "}"}], ",", 
               "ievspecs", ",", "ievopts"}], "]"}]}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"SimplifyRules", "[", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"InitialTimePoint", "\[Rule]", "ta"}], ",", 
             "\[IndentingNewLine]", "evlistopts"}], "\[IndentingNewLine]", 
            "]"}]}], "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]",
          "nrep", ",", "\[IndentingNewLine]", "repevopts"}], 
        "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", "evopts"}], 
      "\[IndentingNewLine]", "]"}]}], ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.69039760330293*^9, 3.690397964453435*^9}, {
   3.690398030642524*^9, 3.690398054392424*^9}, {3.690403273953143*^9, 
   3.690403298614609*^9}, {3.6904033734809637`*^9, 3.690403376100613*^9}, {
   3.690403454412055*^9, 3.6904034679545717`*^9}, {3.690403588245069*^9, 
   3.690403635104392*^9}, {3.690403816724823*^9, 3.690403823617791*^9}, {
   3.69043964287814*^9, 3.690439654250663*^9}, {3.690443981913184*^9, 
   3.690444097825885*^9}, {3.690448126183292*^9, 3.69044814715125*^9}, 
   3.691676719067388*^9, {3.692848323728354*^9, 
   3.6928483253943443`*^9}},ExpressionUUID->"a27549e3-4fcb-426b-b604-\
683f9de25aaf"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["BackgroundGenerator", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, 
   3.683973033466785*^9, {3.683973750698831*^9, 3.683973753893053*^9}, {
   3.683978343050639*^9, 3.683978346277608*^9}, 3.683979658068478*^9, 
   3.683980204513444*^9, 
   3.709374892168702*^9},ExpressionUUID->"de734165-1451-4b23-a41f-\
c50d41f83a5f"],

Cell[CellGroupData[{

Cell["insert in chars", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, {
   3.689747516070005*^9, 
   3.6897475187955647`*^9}},ExpressionUUID->"1bf014b3-da8a-421c-9de3-\
03b31936147d"],

Cell[CellGroupData[{

Cell["general", "Subsection",
 CellChangeTimes->{{3.6840401696780767`*^9, 3.684040177324834*^9}, 
   3.684213498054487*^9, {3.685170203003892*^9, 
   3.685170204240177*^9}},ExpressionUUID->"c10b2b01-9a13-4b86-a398-\
0310eb0bead8"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"timingch_", ",", 
       RowBox[{"{", 
        RowBox[{"Undefined", ",", "backgimp__"}], "}"}], ",", "morech___"}], 
      "}"}], ",", "evspec_", ",", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "evbackg", "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Verbose", "/.", 
            RowBox[{"{", "opts", "}"}]}], "/.", 
           RowBox[{"Options", "[", "Event", "]"}]}], ",", 
          RowBox[{"Print", "[", 
           RowBox[{"\"\<inserting BackgroundGenerator: \>\"", ",", " ", 
            RowBox[{"Expand", "[", "evbackg", "]"}]}], "]"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Event", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"False", ",", "numst"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"timingch", ",", 
            RowBox[{"{", 
             RowBox[{"evbackg", ",", "backgimp"}], "}"}], ",", "morech"}], 
           "}"}], ",", "evspec", ",", "opts"}], "]"}]}], 
       "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"evbackg", "=", 
         RowBox[{
          RowBox[{"BackgroundGenerator", "/.", 
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "Event", "]"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"!", 
         RowBox[{"MatchQ", "[", 
          RowBox[{"evbackg", ",", "Undefined"}], "]"}]}]}], 
       "\[IndentingNewLine]", ")"}]}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.684040721469942*^9, 
   3.68404072922604*^9}, {3.685033569524284*^9, 3.6850335695288467`*^9}, {
   3.685170184971273*^9, 3.6851701857342863`*^9}, 3.688839381656958*^9, 
   3.68970457452805*^9, {3.689848761803308*^9, 3.6898487685989122`*^9}, {
   3.6904482475767736`*^9, 3.690448258095343*^9}, {3.6912364277523613`*^9, 
   3.691236438887586*^9}, {3.691237131867841*^9, 3.691237141327362*^9}, 
   3.6914916806811733`*^9, {3.709374893688089*^9, 
   3.709374895229308*^9}},ExpressionUUID->"afa1450d-d61b-474d-a7ad-\
bff8d54702bd"]
}, Closed]],

Cell[CellGroupData[{

Cell["trap conflicts", "Subsection",
 CellChangeTimes->{{3.6840401696780767`*^9, 3.684040177324834*^9}, 
   3.684213498054487*^9},ExpressionUUID->"cc2443ad-2b30-44b8-b137-\
fcee8ea06032"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"timingch_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"evbackg1", ":", 
          RowBox[{"Except", "[", 
           RowBox[{"Undefined", "|", "False"}], "]"}]}], ",", "backgimp__"}], 
        "}"}], ",", "morech___"}], "}"}], ",", "evspec_", ",", 
     "opts___Rule"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "evbackg2", "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Message", "[", 
          RowBox[{"Event", "::", "backgconflict"}], "]"}], ";", "$Failed"}], 
        ")"}], "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"evbackg2", "=", 
         RowBox[{
          RowBox[{"BackgroundGenerator", "/.", 
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "Event", "]"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"!", 
          RowBox[{"MatchQ", "[", 
           RowBox[{"evbackg2", ",", "Undefined"}], "]"}]}], "&&", 
         "\[IndentingNewLine]", 
         RowBox[{"!", 
          RowBox[{"GeneratorMatchQ", "[", 
           RowBox[{"evbackg1", ",", "evbackg2"}], "]"}]}]}]}], 
       "\[IndentingNewLine]", ")"}]}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.68403977771763*^9, 
   3.684039791234996*^9}, {3.684039833627499*^9, 3.684039921648159*^9}, {
   3.684039966990039*^9, 3.6840400339178257`*^9}, 3.68404011039635*^9, {
   3.684040413478554*^9, 3.684040414429345*^9}, {3.684041012098938*^9, 
   3.6840410337227983`*^9}, {3.684213192254325*^9, 3.684213199510613*^9}, {
   3.684213519468956*^9, 3.684213535372663*^9}, {3.6850335695326233`*^9, 
   3.6850335695372*^9}, 3.688839483060275*^9, 3.7093743009813643`*^9, 
   3.709374896579142*^9},ExpressionUUID->"937c9353-1dc4-4b3d-9bd8-\
2a0c5e3daea8"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["implement: simple events", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, {3.6839742986026897`*^9, 
   3.683974310684813*^9}, 3.683980216123659*^9, {3.6904648238548813`*^9, 
   3.690464834357132*^9}},ExpressionUUID->"0e8c40d5-15a3-4985-a778-\
ae61b23793c8"],

Cell[CellGroupData[{

Cell["simple events", "Subsection",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, {3.6839742986026897`*^9, 
   3.683974310684813*^9}, 3.683980216123659*^9, {3.6904648238548813`*^9, 
   3.690464834357132*^9}, 
   3.690464878183707*^9},ExpressionUUID->"aa71010e-7f39-4e89-b0f8-\
d7b9a16a6dce"],

Cell[CellGroupData[{

Cell["zero duration event", "Subsubsection",
 CellChangeTimes->{{3.6839744581245117`*^9, 3.683974461273919*^9}, {
  3.683974579789905*^9, 3.683974581239148*^9}, {3.6839748499844303`*^9, 
  3.6839748507250147`*^9}},ExpressionUUID->"8eb321da-76c4-4629-909b-\
1c133b7358b6"],

Cell["If the background is specified, request implementation", "Text",
 CellChangeTimes->{{3.688839777384162*^9, 3.6888398187443857`*^9}, {
  3.688839857121035*^9, 3.688839863414445*^9}, {3.689417051858543*^9, 
  3.689417052790783*^9}},ExpressionUUID->"015c49db-6bf1-4f97-91f5-\
529de04544e8"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"dur_", ",", "moretimingch___"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"backg", ":", 
           RowBox[{"Except", "[", "Undefined", "]"}]}], ",", 
          RowBox[{"False", "|", "Undefined"}], ",", 
          RowBox[{"False", "|", "Undefined"}]}], "}"}], ",", "morech___"}], 
       "}"}], ",", "evspec_", ",", "opts___Rule"}], "]"}], "/;", 
    RowBox[{"TrueQ", "[", 
     RowBox[{
      RowBox[{"Chop", "[", "dur", "]"}], "\[Equal]", "0"}], "]"}]}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"dur", ",", "moretimingch"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"backg", ",", "True", ",", "False"}], "}"}], ",", "morech"}], 
      "}"}], ",", "evspec", ",", "\[IndentingNewLine]", 
     RowBox[{"Sequence", "@@", 
      RowBox[{"FilterRules", "[", 
       RowBox[{
        RowBox[{"{", "opts", "}"}], ",", 
        RowBox[{"Except", "[", "BackgroundGenerator", "]"}]}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683974333334736*^9, 
   3.683974359243943*^9}, {3.68397440341503*^9, 3.683974453895319*^9}, {
   3.683974485206881*^9, 3.683974500280587*^9}, {3.683974650645075*^9, 
   3.683974654403214*^9}, {3.683974928626976*^9, 3.683974947921425*^9}, {
   3.685033569541053*^9, 3.685033569543223*^9}, {3.688839400103409*^9, 
   3.688839411068014*^9}, {3.6888397424985323`*^9, 3.688839743529241*^9}, {
   3.6894169765340433`*^9, 3.6894170084200974`*^9}, 
   3.709374897681664*^9},ExpressionUUID->"f22e3173-545e-49d4-9885-\
1fa79e02e385"],

Cell["and flag it as implemented", "Text",
 CellChangeTimes->{{3.688839777384162*^9, 3.6888398187443857`*^9}, {
  3.688839857121035*^9, 3.688839863414445*^9}, {3.689417051858543*^9, 
  3.689417067677598*^9}, {3.6898488230712*^9, 
  3.6898488265085363`*^9}},ExpressionUUID->"9a521704-4bd4-4e4e-87ae-\
74e7c68ea5cb"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"dur_", ",", "moretimingch___"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"backg", ":", 
           RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "True", ",", 
          RowBox[{"False", "|", "Undefined"}]}], "}"}], ",", "morech___"}], 
       "}"}], ",", "evspec_", ",", "opts___Rule"}], "]"}], "/;", 
    RowBox[{"TrueQ", "[", 
     RowBox[{
      RowBox[{"Chop", "[", "dur", "]"}], "\[Equal]", "0"}], "]"}]}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"dur", ",", "moretimingch"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"backg", ",", "True", ",", "True"}], "}"}], ",", "morech"}], 
      "}"}], ",", "evspec", ",", "opts"}], "\[IndentingNewLine]", "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683974333334736*^9, 
   3.683974359243943*^9}, {3.68397440341503*^9, 3.683974453895319*^9}, {
   3.683974485206881*^9, 3.683974500280587*^9}, {3.683974650645075*^9, 
   3.683974654403214*^9}, {3.683974928626976*^9, 3.683974947921425*^9}, {
   3.685033569541053*^9, 3.685033569543223*^9}, {3.688839400103409*^9, 
   3.688839411068014*^9}, {3.688839750709038*^9, 
   3.68883975169109*^9}},ExpressionUUID->"d00d55bd-65a8-4fa2-a9c9-\
ce71ade8c6a3"]
}, Closed]],

Cell[CellGroupData[{

Cell["finite {generator,\[Tau]} event", "Subsubsection",
 CellChangeTimes->{{3.6839744581245117`*^9, 3.683974461273919*^9}, {
  3.683974579789905*^9, 3.683974581239148*^9}, {3.683974628143805*^9, 
  3.683974629110386*^9}, {3.683974668748519*^9, 3.683974674146552*^9}, {
  3.683974858287311*^9, 
  3.6839748592529097`*^9}},ExpressionUUID->"119e5f4f-f410-4041-8e38-\
af170eefdb15"],

Cell["If the background is specified, request implementation", "Text",
 CellChangeTimes->{{3.688839777384162*^9, 3.6888398187443857`*^9}, {
  3.688839857121035*^9, 3.688839863414445*^9}, {3.689417051858543*^9, 
  3.689417052790783*^9}},ExpressionUUID->"77de03cc-fcde-46d1-979b-\
cb7553f8feb2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"dur_", ",", "moretimingch___"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"backg", ":", 
           RowBox[{"Except", "[", "Undefined", "]"}]}], ",", 
          RowBox[{"False", "|", "Undefined"}], ",", 
          RowBox[{"False", "|", "Undefined"}]}], "}"}], ",", "morech___"}], 
       "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"evgen_", ",", "\[Tau]_"}], "}"}], ",", "opts___Rule"}], "]"}],
     "/;", 
    RowBox[{"!", 
     RowBox[{"TrueQ", "[", 
      RowBox[{
       RowBox[{"Chop", "[", "dur", "]"}], "\[Equal]", "0"}], "]"}]}]}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"dur", ",", "moretimingch"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"backg", ",", "True", ",", "False"}], "}"}], ",", "morech"}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"evgen", ",", "\[Tau]"}], "}"}], ",", 
     RowBox[{"Sequence", "@@", 
      RowBox[{"FilterRules", "[", 
       RowBox[{
        RowBox[{"{", "opts", "}"}], ",", 
        RowBox[{"Except", "[", "BackgroundGenerator", "]"}]}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683974333334736*^9, 
   3.683974359243943*^9}, {3.68397440341503*^9, 3.683974453895319*^9}, {
   3.683974485206881*^9, 3.683974500280587*^9}, {3.683974637224945*^9, 
   3.683974642623952*^9}, {3.683974679977281*^9, 3.683974740133045*^9}, {
   3.6839747909097548`*^9, 3.6839748224301777`*^9}, {3.68397496707168*^9, 
   3.683974981930236*^9}, {3.685033569546874*^9, 3.6850335695495977`*^9}, {
   3.688839430247795*^9, 3.688839449996852*^9}, {3.68883989841578*^9, 
   3.688839899158692*^9}, 3.689416890355986*^9, 3.689417028829543*^9, 
   3.709374898547904*^9},ExpressionUUID->"102d11b2-e672-4fdc-9c92-\
8fa590dd751a"],

Cell["and implement it", "Text",
 CellChangeTimes->{{3.688839777384162*^9, 3.6888398187443857`*^9}, {
  3.688839857121035*^9, 3.688839863414445*^9}, {3.689417051858543*^9, 
  3.689417067677598*^9}},ExpressionUUID->"90d5d7e1-7e7d-4621-bb46-\
4fbcc94be68e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"dur_", ",", "moretimingch___"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"backg", ":", 
           RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "True", ",", 
          RowBox[{"False", "|", "Undefined"}]}], "}"}], ",", "morech___"}], 
       "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"evgen_", ",", "\[Tau]_"}], "}"}], ",", "opts___Rule"}], "]"}],
     "/;", 
    RowBox[{"!", 
     RowBox[{"TrueQ", "[", 
      RowBox[{
       RowBox[{"Chop", "[", "dur", "]"}], "\[Equal]", "0"}], "]"}]}]}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Verbose", "/.", 
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "Event", "]"}]}], ",", 
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<implementing BackgroundGenerator in finite-duration Event: \
\>\"", ",", " ", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"backg", ",", "evgen"}], "}"}], "/.", 
            RowBox[{"s_ShapeFunction", "\[RuleDelayed]", 
             RowBox[{"s", "\[LeftDoubleBracket]", 
              RowBox[{"{", "4", "}"}], "\[RightDoubleBracket]"}]}]}]}], 
          "]"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"Event", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"False", ",", "numst"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"dur", ",", "moretimingch"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"backg", ",", "True", ",", "True"}], "}"}], ",", 
           "morech"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"CombineGenerators", "[", 
            RowBox[{"evgen", ",", "backg"}], "]"}], ",", "\[Tau]"}], "}"}], 
         ",", "opts"}], "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]",
       ")"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683974333334736*^9, 
   3.683974359243943*^9}, {3.68397440341503*^9, 3.683974453895319*^9}, {
   3.683974485206881*^9, 3.683974500280587*^9}, {3.683974637224945*^9, 
   3.683974642623952*^9}, {3.683974679977281*^9, 3.683974740133045*^9}, {
   3.6839747909097548`*^9, 3.6839748224301777`*^9}, {3.68397496707168*^9, 
   3.683974981930236*^9}, {3.685033569546874*^9, 3.6850335695495977`*^9}, {
   3.688839430247795*^9, 3.688839449996852*^9}, {3.688839911055963*^9, 
   3.688839911980206*^9}, {3.699203099343348*^9, 3.699203142643264*^9}, {
   3.699203179654674*^9, 3.699203239302826*^9}, {3.6992033090323963`*^9, 
   3.699203311677561*^9}, {3.699203429877062*^9, 3.699203442448455*^9}, {
   3.699203621319363*^9, 3.699203632301071*^9}, {3.699203728113778*^9, 
   3.6992037317447042`*^9}, 3.709373444190856*^9, 
   3.709374899436941*^9},ExpressionUUID->"4a4ab5e7-3a7d-42f4-bbd6-\
91bce5e904d6"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Repeat", "Subsection",
 CellChangeTimes->{{3.6839744581245117`*^9, 3.683974461273919*^9}, {
   3.683974579789905*^9, 3.683974581239148*^9}, {3.683974628143805*^9, 
   3.683974629110386*^9}, {3.683974668748519*^9, 3.683974674146552*^9}, {
   3.683974858287311*^9, 3.6839748592529097`*^9}, {3.685170310724441*^9, 
   3.685170316855648*^9}, {3.6894142131125917`*^9, 3.68941421654068*^9}, {
   3.689417259809701*^9, 3.689417264129222*^9}, 3.689417778115696*^9, 
   3.690464839445745*^9, 3.690464885108502*^9, 
   3.692848326277223*^9},ExpressionUUID->"abb66f6d-7fe9-4440-9ce2-\
016ece8a70b3"],

Cell[CellGroupData[{

Cell["pass BackgroundGenerator into Repeat", "Subsubsection",
 CellChangeTimes->{{3.690464862519751*^9, 3.6904649114862223`*^9}, 
   3.6928483272599373`*^9, 
   3.709374900472394*^9},ExpressionUUID->"9a274f8d-8820-43ea-b188-\
4744149690a4"],

Cell["\<\
If the background is specified, but is Undefined inside the Repeat/EventList, \
insert the background into the Repeat/EventList but do not immediately \
request implementation.\
\>", "Text",
 CellChangeTimes->{{3.688839777384162*^9, 3.6888398187443857`*^9}, {
  3.688839857121035*^9, 3.688839863414445*^9}, {3.689417051858543*^9, 
  3.689417052790783*^9}, {3.6894173014018784`*^9, 3.689417373000588*^9}, {
  3.689417691045731*^9, 3.689417715730329*^9}, {3.689848853721257*^9, 
  3.689848883525401*^9}, {3.6912359954861526`*^9, 3.6912360497539387`*^9}, {
  3.6928483279807673`*^9, 
  3.6928483292171717`*^9}},ExpressionUUID->"57977b8e-4af3-4f24-b542-\
d38ec03acd99"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"dur_", ",", "moretimingch___"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"backg", ":", 
           RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "_", ",", "_"}], 
         "}"}], ",", "morech___"}], "}"}], ",", 
      RowBox[{"repevs", ":", 
       RowBox[{"Repeat", "[", 
        RowBox[{
         RowBox[{"EventList", "[", 
          RowBox[{"evliststatus_", ",", 
           RowBox[{"{", 
            RowBox[{"evlisttiming_", ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"evlistbackg", ":", "Undefined"}], ",", 
               "evlistbackgreqimpl_", ",", "evlistbackgimpl_"}], "}"}], ",", 
             "moreevlistch___"}], "}"}], ",", "events_List", ",", 
           "evlistopts___Rule"}], "]"}], ",", "nrep_", ",", 
         "repevopts___Rule"}], "]"}]}], ",", "\[IndentingNewLine]", 
      "opts___Rule"}], "]"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Verbose", "/.", 
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "Event", "]"}]}], ",", 
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<inserting BackgroundGenerator into Repeat: \>\"", ",", " ", 
           RowBox[{"Expand", "[", "backg", "]"}]}], "]"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Event", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"False", ",", "numst"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"dur", ",", "moretimingch"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"backg", ",", "False", ",", "False"}], "}"}], ",", 
           "morech"}], "}"}], ",", 
         RowBox[{"Repeat", "[", 
          RowBox[{
           RowBox[{"EventList", "[", 
            RowBox[{"evliststatus", ",", 
             RowBox[{"{", 
              RowBox[{"evlisttiming", ",", 
               RowBox[{"{", 
                RowBox[{"backg", ",", "False", ",", "False"}], "}"}], ",", 
               "moreevlistch"}], "}"}], ",", "events", ",", "evlistopts"}], 
            "]"}], ",", "nrep", ",", "repevopts"}], "]"}], ",", 
         RowBox[{"Sequence", "@@", 
          RowBox[{"FilterRules", "[", 
           RowBox[{
            RowBox[{"{", "opts", "}"}], ",", 
            RowBox[{"Except", "[", "BackgroundGenerator", "]"}]}], "]"}]}]}], 
        "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", ")"}]}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683974333334736*^9, 
   3.683974359243943*^9}, {3.68397440341503*^9, 3.683974453895319*^9}, {
   3.683974485206881*^9, 3.683974500280587*^9}, {3.683974637224945*^9, 
   3.683974642623952*^9}, {3.683974679977281*^9, 3.683974740133045*^9}, {
   3.6839747909097548`*^9, 3.6839748224301777`*^9}, {3.68397496707168*^9, 
   3.683974981930236*^9}, {3.685033569546874*^9, 3.6850335695495977`*^9}, {
   3.688839430247795*^9, 3.688839449996852*^9}, {3.68883989841578*^9, 
   3.688839899158692*^9}, 3.689416890355986*^9, 3.689417028829543*^9, {
   3.689417386291594*^9, 3.689417589758437*^9}, {3.6894176372531357`*^9, 
   3.689417666094516*^9}, 3.6894179155765343`*^9, 3.689704971722123*^9, {
   3.689705028020874*^9, 3.6897050721735487`*^9}, {3.689848886216588*^9, 
   3.689848916660735*^9}, {3.689849162348979*^9, 3.689849166322287*^9}, {
   3.690448266343643*^9, 3.69044828280937*^9}, {3.690455851475438*^9, 
   3.6904558546253967`*^9}, {3.6912364547195253`*^9, 3.691236464774665*^9}, 
   3.691676730170948*^9, {3.69284833124378*^9, 3.692848335142826*^9}, {
   3.7093749013159246`*^9, 
   3.7093749021819963`*^9}},ExpressionUUID->"c637d09a-434b-4d2c-b858-\
a2bbf6b0d6b9"]
}, Closed]],

Cell[CellGroupData[{

Cell["request BackgroundGenerator implementation", "Subsubsection",
 CellChangeTimes->{{3.690464862519751*^9, 3.690464930237608*^9}, {
   3.6904649754087477`*^9, 3.6904649815711203`*^9}, {3.690465035516286*^9, 
   3.6904650407100077`*^9}, 
   3.709374902990074*^9},ExpressionUUID->"add1664d-2443-4340-9999-\
c37d31cadd74"],

Cell["\<\
request the implementation of the background inside the Repeat if all \
expansions of Repeat have been done. \
\>", "Text",
 CellChangeTimes->{{3.688839777384162*^9, 3.6888398187443857`*^9}, {
   3.688839857121035*^9, 3.688839863414445*^9}, {3.689417051858543*^9, 
   3.689417052790783*^9}, {3.6894173014018784`*^9, 3.689417373000588*^9}, {
   3.689417691045731*^9, 3.689417820139112*^9}, 3.689705104062916*^9, 
   3.690554821134547*^9, {3.692848336176777*^9, 
   3.6928483369897633`*^9}},ExpressionUUID->"b682c1cf-26c7-44f8-bb67-\
4d72a3f23a4d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"dur_", ",", "moretimingch___"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"backg", ":", 
          RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "False", ",", 
         "False"}], "}"}], ",", "hilbch_", ",", "pcpch_", ",", 
       RowBox[{"exprepch", ":", 
        RowBox[{"True", "|", 
         RowBox[{"{", 
          RowBox[{"exptype_", ",", "expreq_", ",", "True"}], "}"}]}]}], ",", 
       "morech___"}], "}"}], ",", "\[IndentingNewLine]", "repevs_Repeat", ",",
      "\[IndentingNewLine]", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Verbose", "/.", 
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "Event", "]"}]}], ",", 
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<requesting BackgroundGenerator implementation for Repeat: \
\>\"", ",", " ", 
           RowBox[{"Expand", "[", "backg", "]"}]}], "]"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Event", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"False", ",", "numst"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"dur", ",", "moretimingch"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"backg", ",", "True", ",", "False"}], "}"}], ",", 
           "hilbch", ",", "pcpch", ",", "exprepch", ",", "morech"}], "}"}], 
         ",", "\[IndentingNewLine]", "repevs", ",", "\[IndentingNewLine]", 
         RowBox[{"Sequence", "@@", 
          RowBox[{"FilterRules", "[", 
           RowBox[{
            RowBox[{"{", "opts", "}"}], ",", 
            RowBox[{"Except", "[", "BackgroundGenerator", "]"}]}], "]"}]}]}], 
        "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", ")"}]}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683974333334736*^9, 
   3.683974359243943*^9}, {3.68397440341503*^9, 3.683974453895319*^9}, {
   3.683974485206881*^9, 3.683974500280587*^9}, {3.683974637224945*^9, 
   3.683974642623952*^9}, {3.683974679977281*^9, 3.683974740133045*^9}, {
   3.6839747909097548`*^9, 3.6839748224301777`*^9}, {3.68397496707168*^9, 
   3.683974981930236*^9}, {3.685033569546874*^9, 3.6850335695495977`*^9}, {
   3.688839430247795*^9, 3.688839449996852*^9}, {3.68883989841578*^9, 
   3.688839899158692*^9}, 3.689416890355986*^9, 3.689417028829543*^9, {
   3.689417386291594*^9, 3.689417589758437*^9}, {3.6894176372531357`*^9, 
   3.689417666094516*^9}, {3.6894179356197157`*^9, 3.6894179409054823`*^9}, {
   3.6894179904474*^9, 3.689418088544161*^9}, 3.6894184207895727`*^9, {
   3.689418514026092*^9, 3.689418516215681*^9}, 3.6894185895608463`*^9, {
   3.689848929543783*^9, 3.689848960757049*^9}, {3.690448291359351*^9, 
   3.690448309107152*^9}, {3.691236082641074*^9, 3.691236089130445*^9}, {
   3.691236471280661*^9, 3.691236488596105*^9}, {3.69149647146719*^9, 
   3.6914964851765423`*^9}, {3.692848337782749*^9, 3.6928483385707207`*^9}, {
   3.7093749039481277`*^9, 
   3.7093749050619993`*^9}},ExpressionUUID->"7bf78a7a-4c78-440b-8891-\
ffecbaf16753"]
}, Closed]],

Cell[CellGroupData[{

Cell["pass BackgroundGenerator implementation request into Repeat", \
"Subsubsection",
 CellChangeTimes->{{3.690464862519751*^9, 3.690464930237608*^9}, {
   3.690465045628284*^9, 3.690465051992174*^9}, 3.692848339605702*^9, 
   3.709374905825157*^9},ExpressionUUID->"a6aabd53-ac4c-4705-8d9c-\
e8252026de53"],

Cell["\<\
pass a backg implementation request to the EventList inside Repeat\
\>", "Text",
 CellChangeTimes->{{3.688839777384162*^9, 3.6888398187443857`*^9}, {
   3.688839857121035*^9, 3.688839863414445*^9}, {3.689417051858543*^9, 
   3.689417052790783*^9}, {3.6894173014018784`*^9, 3.689417373000588*^9}, {
   3.689417691045731*^9, 3.689417820139112*^9}, {3.6894181347948236`*^9, 
   3.689418156952094*^9}, {3.689848990380116*^9, 3.689849014472595*^9}, 
   3.692848340231007*^9},ExpressionUUID->"9390a7b3-a4f4-4116-96b1-\
c9e7b2e9d1ad"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"dur_", ",", "moretimingch___"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"backg", ":", 
           RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "True", ",", 
          "False"}], "}"}], ",", "morech___"}], "}"}], ",", 
      RowBox[{"repevs", ":", 
       RowBox[{"Repeat", "[", 
        RowBox[{
         RowBox[{"EventList", "[", 
          RowBox[{"evliststatus_", ",", 
           RowBox[{"{", 
            RowBox[{"evlisttiming_", ",", 
             RowBox[{"{", 
              RowBox[{"backg_", ",", 
               RowBox[{"False", "|", "Undefined"}], ",", 
               RowBox[{"False", "|", "Undefined"}]}], "}"}], ",", 
             "moreevlistch___"}], "}"}], ",", "events_List", ",", 
           "evlistopts___Rule"}], "]"}], ",", "nrep_", ",", 
         "repevopts___Rule"}], "]"}]}], ",", "\[IndentingNewLine]", 
      "opts___Rule"}], "]"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Verbose", "/.", 
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "Event", "]"}]}], ",", 
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<passing BackgroundGenerator implementation request into \
Repeat: \>\"", ",", 
           RowBox[{"Expand", "[", "backg", "]"}]}], "]"}]}], 
        "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"Event", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"False", ",", "numst"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"dur", ",", "moretimingch"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"backg", ",", "True", ",", "False"}], "}"}], ",", 
           "morech"}], "}"}], ",", "\[IndentingNewLine]", 
         RowBox[{"Repeat", "[", 
          RowBox[{
           RowBox[{"EventList", "[", 
            RowBox[{"evliststatus", ",", 
             RowBox[{"{", 
              RowBox[{"evlisttiming", ",", 
               RowBox[{"{", 
                RowBox[{"backg", ",", "True", ",", "False"}], "}"}], ",", 
               "moreevlistch"}], "}"}], ",", "events", ",", "evlistopts"}], 
            "]"}], ",", "nrep", ",", "repevopts"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Sequence", "@@", 
          RowBox[{"FilterRules", "[", 
           RowBox[{
            RowBox[{"{", "opts", "}"}], ",", 
            RowBox[{"Except", "[", "BackgroundGenerator", "]"}]}], "]"}]}]}], 
        "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", ")"}]}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683974333334736*^9, 
   3.683974359243943*^9}, {3.68397440341503*^9, 3.683974453895319*^9}, {
   3.683974485206881*^9, 3.683974500280587*^9}, {3.683974637224945*^9, 
   3.683974642623952*^9}, {3.683974679977281*^9, 3.683974740133045*^9}, {
   3.6839747909097548`*^9, 3.6839748224301777`*^9}, {3.68397496707168*^9, 
   3.683974981930236*^9}, {3.685033569546874*^9, 3.6850335695495977`*^9}, {
   3.688839430247795*^9, 3.688839449996852*^9}, {3.68883989841578*^9, 
   3.688839899158692*^9}, 3.689416890355986*^9, 3.689417028829543*^9, {
   3.689417386291594*^9, 3.689417589758437*^9}, {3.6894176372531357`*^9, 
   3.689417666094516*^9}, 3.6894179155765343`*^9, {3.68941817367529*^9, 
   3.689418338608553*^9}, {3.68970578787713*^9, 3.689705868446164*^9}, {
   3.6897059158427067`*^9, 3.689705949949997*^9}, {3.689849016747768*^9, 
   3.6898490659361677`*^9}, 3.6903927631915607`*^9, {3.69044832054987*^9, 
   3.690448332754415*^9}, {3.69055491665829*^9, 3.6905549168130703`*^9}, {
   3.691236506425568*^9, 3.691236516322884*^9}, 3.691676734882567*^9, {
   3.6928483410726433`*^9, 3.692848342583694*^9}, {3.709374906603382*^9, 
   3.709374907480811*^9}},ExpressionUUID->"4d18b33e-208b-4a9f-8277-\
285e7df305fd"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
pass BackgroundGenerator implementation status upwards into enclosing Event\
\>", "Subsubsection",
 CellChangeTimes->{{3.690464862519751*^9, 3.690464930237608*^9}, {
   3.690465045628284*^9, 3.6904650881806707`*^9}, 
   3.709374908760847*^9},ExpressionUUID->"df700886-8f6a-4443-aabc-\
b1403ac6c1b3"],

Cell["\<\
set backg implementation flag if the EventList inside Repeat has the backg \
implementation flag set\
\>", "Text",
 CellChangeTimes->{{3.688839777384162*^9, 3.6888398187443857`*^9}, {
   3.688839857121035*^9, 3.688839863414445*^9}, {3.689417051858543*^9, 
   3.689417052790783*^9}, {3.6894173014018784`*^9, 3.689417373000588*^9}, {
   3.689417691045731*^9, 3.689417820139112*^9}, {3.6894181347948236`*^9, 
   3.689418156952094*^9}, {3.689848990380116*^9, 3.689849014472595*^9}, {
   3.689850777797792*^9, 3.689850796891492*^9}, 
   3.69284834334124*^9},ExpressionUUID->"bd11ba15-2d06-4261-a529-\
319bde2c4ae9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"dur_", ",", "moretimingch___"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"backg", ":", 
           RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "True", ",", 
          "False"}], "}"}], ",", "morech___"}], "}"}], ",", 
      RowBox[{"repevs", ":", 
       RowBox[{"Repeat", "[", 
        RowBox[{
         RowBox[{"EventList", "[", 
          RowBox[{"evliststatus_", ",", 
           RowBox[{"{", 
            RowBox[{"evlisttiming_", ",", 
             RowBox[{"{", 
              RowBox[{"backg_", ",", "_", ",", "True"}], "}"}], ",", 
             "moreevlistch___"}], "}"}], ",", "events_List", ",", 
           "evlistopts___Rule"}], "]"}], ",", "nrep_", ",", 
         "repevopts___Rule"}], "]"}]}], ",", "\[IndentingNewLine]", 
      "opts___Rule"}], "]"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Verbose", "/.", 
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "Event", "]"}]}], ",", 
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<passing BackgroundGenerator implementation status from Repeat \
upwards to enclosing Event: \>\"", ",", 
           RowBox[{"Expand", "@", "backg"}]}], " ", "]"}]}], 
        "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"Event", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"False", ",", "numst"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"dur", ",", "moretimingch"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"backg", ",", "True", ",", "True"}], "}"}], ",", 
           "morech"}], "}"}], ",", "\[IndentingNewLine]", 
         RowBox[{"Repeat", "[", 
          RowBox[{
           RowBox[{"EventList", "[", 
            RowBox[{"evliststatus", ",", 
             RowBox[{"{", 
              RowBox[{"evlisttiming", ",", 
               RowBox[{"{", 
                RowBox[{"backg", ",", "True", ",", "True"}], "}"}], ",", 
               "moreevlistch"}], "}"}], ",", "events", ",", "evlistopts"}], 
            "]"}], ",", "nrep", ",", "repevopts"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Sequence", "@@", 
          RowBox[{"FilterRules", "[", 
           RowBox[{
            RowBox[{"{", "opts", "}"}], ",", 
            RowBox[{"Except", "[", "BackgroundGenerator", "]"}]}], "]"}]}]}], 
        "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", ")"}]}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683974333334736*^9, 
   3.683974359243943*^9}, {3.68397440341503*^9, 3.683974453895319*^9}, {
   3.683974485206881*^9, 3.683974500280587*^9}, {3.683974637224945*^9, 
   3.683974642623952*^9}, {3.683974679977281*^9, 3.683974740133045*^9}, {
   3.6839747909097548`*^9, 3.6839748224301777`*^9}, {3.68397496707168*^9, 
   3.683974981930236*^9}, {3.685033569546874*^9, 3.6850335695495977`*^9}, {
   3.688839430247795*^9, 3.688839449996852*^9}, {3.68883989841578*^9, 
   3.688839899158692*^9}, 3.689416890355986*^9, 3.689417028829543*^9, {
   3.689417386291594*^9, 3.689417589758437*^9}, {3.6894176372531357`*^9, 
   3.689417666094516*^9}, 3.6894179155765343`*^9, {3.68941817367529*^9, 
   3.689418338608553*^9}, {3.68970578787713*^9, 3.689705868446164*^9}, {
   3.6897059158427067`*^9, 3.689705949949997*^9}, {3.689849016747768*^9, 
   3.6898490659361677`*^9}, {3.6898508019045267`*^9, 3.689850837179941*^9}, 
   3.690392799426714*^9, {3.6904483458369417`*^9, 3.690448380024149*^9}, {
   3.690554924735804*^9, 3.690554924947136*^9}, {3.691236536744691*^9, 
   3.691236564867426*^9}, 3.691676741522092*^9, {3.692848343996799*^9, 
   3.692848345446781*^9}, {3.7093749095959463`*^9, 
   3.7093749108782873`*^9}},ExpressionUUID->"48970f02-3bf2-4669-b0f5-\
ff953402b480"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["extract from chars", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
  3.683973766585649*^9, 3.6839737710410423`*^9}, {3.6839742986026897`*^9, 
  3.683974310684813*^9}, {3.683980216123659*^9, 3.683980224571121*^9}, {
  3.689747523366489*^9, 
  3.6897475250996103`*^9}},ExpressionUUID->"124754c2-b93e-4efb-8829-\
2bc57664e7bf"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"BackgroundGenerator", "[", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{"timingch_", ",", 
         RowBox[{"{", 
          RowBox[{"backg_", ",", "__"}], "}"}], ",", "morech___"}], "}"}], 
       ",", "evspec_", ",", "opts___Rule"}], "]"}], ",", "___"}], "]"}], ":=",
    "backg"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683980234573969*^9, 3.683980292705936*^9}, 
   3.685033569554091*^9, 3.688839620486807*^9, 
   3.709374911856461*^9},ExpressionUUID->"445f1794-8b2a-40e7-af7e-\
b043973c67ce"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"BackgroundGeneratorImplemented", "[", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{"timingch_", ",", 
         RowBox[{"{", 
          RowBox[{"backg_", ",", "reqbackgimp_", ",", "backgimp_"}], "}"}], 
         ",", "morech___"}], "}"}], ",", "evspec_", ",", "opts___Rule"}], 
      "]"}], ",", "___"}], "]"}], ":=", "backgimp"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683980234573969*^9, 3.683980292705936*^9}, 
   3.685033569554091*^9, 3.6888388840081673`*^9, {3.68883963041718*^9, 
   3.688839634766925*^9}, 
   3.709374912553989*^9},ExpressionUUID->"fbc53951-40c3-4eba-b7ae-\
2d794700d5d2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["HilbertSpaceCompatibility", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, 
   3.683973033466785*^9, {3.683973750698831*^9, 3.683973753893053*^9}, {
   3.683976432753954*^9, 3.683976436691045*^9}, 3.68397674513378*^9, {
   3.68397832756979*^9, 3.683978331696862*^9}, {3.683979633451119*^9, 
   3.6839796489973183`*^9}, 
   3.683980314039257*^9},ExpressionUUID->"2915a32b-55cd-41dc-9e44-\
ea6defbd5e9e"],

Cell[CellGroupData[{

Cell["insert into chars", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, 
   3.6897472214839287`*^9},ExpressionUUID->"67c8c818-aecb-4524-a5dd-\
cbdaa9c6da42"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"timingch_", ",", 
       RowBox[{"backgch", ":", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"backg", ":", 
           RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "__"}], "}"}]}], 
       ",", "Undefined", ",", "morech___"}], "}"}], ",", "evspec_", ",", 
     "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "hilbcomp", "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Verbose", "/.", 
            RowBox[{"{", "opts", "}"}]}], "/.", 
           RowBox[{"Options", "[", "Event", "]"}]}], ",", 
          RowBox[{"Print", "[", 
           RowBox[{
           "\"\<inserting HilbertSpaceCompatibility into Event: \>\"", ",", 
            "hilbcomp"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Event", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"False", ",", "numst"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
           "timingch", ",", "backgch", ",", "hilbcomp", ",", "morech"}], 
           "}"}], ",", "evspec", ",", "opts"}], "]"}]}], 
       "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"hilbcomp", "=", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"MatchQ", "[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"PropagationMethod", "/.", 
               RowBox[{"{", "opts", "}"}]}], ")"}], ",", "Adiabatic"}], "]"}],
            ",", "False", ",", 
           RowBox[{"HilbertSpaceCompatibility", "[", 
            RowBox[{"evspec", ",", 
             RowBox[{"BackgroundGenerator", "\[Rule]", "backg"}]}], "]"}]}], 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"!", 
         RowBox[{"MatchQ", "[", 
          RowBox[{"hilbcomp", ",", "Undefined"}], "]"}]}]}], 
       "\[IndentingNewLine]", ")"}]}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683976480502823*^9, 
   3.683976634737287*^9}, {3.6839766756355143`*^9, 3.683976688875939*^9}, {
   3.685033569558119*^9, 3.6850335695602427`*^9}, {3.6888401383764*^9, 
   3.688840152837696*^9}, {3.6888401902862053`*^9, 3.688840190524692*^9}, {
   3.689851804769758*^9, 3.689851824023171*^9}, {3.689851859300812*^9, 
   3.6898519415517282`*^9}, {3.6904483884124727`*^9, 3.690448400476754*^9}, {
   3.6915814692298107`*^9, 3.691581497544669*^9}, 3.709374917493182*^9, 
   3.725089987571086*^9},ExpressionUUID->"45da9025-4a38-455c-869d-\
77d901df6199"]
}, Closed]],

Cell[CellGroupData[{

Cell["extract from chars", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, {
   3.6839803310601482`*^9, 3.683980332240543*^9}, 
   3.6897472265199738`*^9},ExpressionUUID->"d3ed70e5-c3a4-4ae9-a84f-\
5d57ffb6b6fd"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
        "timingch_", ",", "backgch_", ",", "hilbcompat_", ",", "morech___"}], 
        "}"}], ",", "evspec_", ",", "opts___Rule"}], "]"}], ",", "___"}], 
    "]"}], ":=", "hilbcompat"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683980234573969*^9, 3.683980292705936*^9}, {
   3.683980344913636*^9, 3.683980364039876*^9}, 
   3.6850335695697117`*^9},ExpressionUUID->"66bde24b-3869-4d77-bbec-\
b190cf11a509"]
}, Closed]],

Cell[CellGroupData[{

Cell["derive HilbertSpaceCompatibility", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, {
   3.6839803310601482`*^9, 3.683980332240543*^9}, 3.6897472265199738`*^9, {
   3.690546993651972*^9, 3.690546994933474*^9}, {3.690547145924531*^9, 
   3.690547150019362*^9}},ExpressionUUID->"b31e26da-3c42-4e28-962d-\
a9010aa44191"],

Cell[CellGroupData[{

Cell["defaults", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.539265448207505*^9, 3.539265453726768*^9}, {
  3.539265561151812*^9, 
  3.539265562429594*^9}},ExpressionUUID->"f87e4302-90ca-4d92-9e05-\
4d6941a8742c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "HilbertSpaceCompatibility", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"BackgroundGenerator", "\[Rule]", "Undefined"}], "}"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.539265565903571*^9, 3.5392655749933777`*^9}, {
   3.5392656549380217`*^9, 3.539265659189908*^9}, {3.5392659693128643`*^9, 
   3.539265983957749*^9}, {3.542267490780237*^9, 3.5422675135341997`*^9}, {
   3.689851057981161*^9, 3.689851059418458*^9}, 
   3.709374923252839*^9},ExpressionUUID->"d731f18d-f25d-4bac-8b08-\
7ab24cea24f2"]
}, Closed]],

Cell[CellGroupData[{

Cell["return Undefined", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.503093847300502*^9, 3.5030938604684362`*^9}, 
   3.5147216276975393`*^9, {3.5422675661679163`*^9, 3.5422675729722633`*^9}, {
   3.543409539601869*^9, 3.543409554016547*^9}, 3.683368486193263*^9, {
   3.689851372131616*^9, 3.68985138845607*^9}, {3.690547195436379*^9, 
   3.690547213290164*^9}},ExpressionUUID->"bc7f6e5d-bd3f-4737-8544-\
0d17ad854ea6"],

Cell["\<\
reurn Undefined if any argument, or BackgroundGenerator, is undefined\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.503093847300502*^9, 3.5030938604684362`*^9}, 
   3.5147216276975393`*^9, {3.5422675661679163`*^9, 3.5422675729722633`*^9}, {
   3.543409539601869*^9, 3.543409554016547*^9}, 3.683368486193263*^9, {
   3.689851372131616*^9, 3.68985138845607*^9}, {3.690547195436379*^9, 
   3.6905471961423283`*^9}, 
   3.709374924162594*^9},ExpressionUUID->"d7fc0c42-6456-4620-8f92-\
900b3f5ad7f6"],

Cell["\<\
only take into account BackgroundGenerator if it is specified in the option \
rules.\
\>", "Text",
 CellChangeTimes->{{3.689852464113954*^9, 3.689852477566496*^9}, 
   3.709374924906275*^9},ExpressionUUID->"e36f4276-d26b-46a0-a4d3-\
afb55be56f50"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"HilbertSpaceCompatibility", "[", 
     RowBox[{"args", ":", 
      RowBox[{
       RowBox[{"Except", "[", "_Rule", "]"}], ".."}]}], "]"}], "/;", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{"Or", "@@", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"MatchQ", "[", 
          RowBox[{"#", ",", "Undefined"}], "]"}], "&"}], "/@", 
        RowBox[{"{", "args", "}"}]}], ")"}]}], ")"}]}], ":=", "Undefined"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.689852149475268*^9, 3.689852218471532*^9}, 
   3.689853226562458*^9},ExpressionUUID->"370b0b97-5025-4b9e-858e-\
80289f22f177"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{
     RowBox[{"args", ":", 
      RowBox[{
       RowBox[{"Except", "[", "_Rule", "]"}], "..."}]}], ",", "optsa___Rule", 
     ",", 
     RowBox[{"BackgroundGenerator", "\[Rule]", 
      RowBox[{"(", 
       RowBox[{"backg", ":", "Undefined"}], ")"}]}], ",", "optsb___Rule"}], 
    "]"}], ":=", "Undefined"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.689851288273192*^9, 3.6898513400780983`*^9}, {
   3.6898513945152063`*^9, 3.689851493663265*^9}, {3.6898515276305513`*^9, 
   3.6898515367786207`*^9}, {3.689852236297805*^9, 3.689852279575365*^9}, {
   3.68985233697444*^9, 3.689852405653214*^9}, {3.6898528085952787`*^9, 
   3.689852808911767*^9}, {3.689852843700831*^9, 3.6898528598788347`*^9}, 
   3.7093749256042*^9},ExpressionUUID->"2c4f25b7-e8cb-42d7-9eb8-db529b08fa17"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{
     RowBox[{"args", ":", 
      RowBox[{
       RowBox[{"Except", "[", "_Rule", "]"}], ".."}]}], ",", "opts__Rule"}], 
    "]"}], ":=", 
   RowBox[{
    RowBox[{"HilbertSpaceCompatibility", "[", "args", "]"}], "/;", 
    RowBox[{"FreeQ", "[", 
     RowBox[{
      RowBox[{"{", "opts", "}"}], ",", "BackgroundGenerator"}], "]"}]}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.689852417229432*^9, 3.689852450274897*^9}, 
   3.689853265789936*^9, 
   3.709374926301742*^9},ExpressionUUID->"9cb9db9e-9fe3-4e8b-a0a0-\
9e8c3b760e8e"]
}, Closed]],

Cell[CellGroupData[{

Cell["combine two or more arguments", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.503093847300502*^9, 3.5030938604684362`*^9}, 
   3.5147216276975393`*^9, {3.5422675661679163`*^9, 3.5422675729722633`*^9}, {
   3.543409539601869*^9, 3.543409554016547*^9}, 
   3.683368486193263*^9},ExpressionUUID->"231dce1d-75fa-4b8f-93ae-\
e60ccaa2f4d4"],

Cell[CellGroupData[{

Cell["\<\
set to Undefined if HilbertSpaceCompatibility of an element, or of \
BackgroundGenerator is Undefined\
\>", "Subsubsection",
 CellChangeTimes->{{3.689852523990015*^9, 3.689852570187973*^9}, {
   3.689852917155579*^9, 3.689852956518385*^9}, {3.690989338987417*^9, 
   3.69098933917598*^9}, 
   3.709374927030697*^9},ExpressionUUID->"1dc6aa31-5eb4-4fe1-8264-\
609258bd03a6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"HilbertSpaceCompatibility", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"args", ":", 
       RowBox[{"Repeated", "[", 
        RowBox[{
         RowBox[{"Except", "[", "_Rule", "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "Infinity"}], "}"}]}], "]"}]}], ",", 
      "\[IndentingNewLine]", "opts___Rule"}], "]"}], "/;", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{"Or", "@@", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"MatchQ", "[", 
          RowBox[{
           RowBox[{"HilbertSpaceCompatibility", "[", 
            RowBox[{"#", ",", "opts"}], "]"}], ",", "Undefined"}], "]"}], 
         "&"}], "/@", 
        RowBox[{"{", "args", "}"}]}], ")"}]}], ")"}]}], ":=", "Undefined"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.689852149475268*^9, 3.689852218471532*^9}, {
   3.689852615414262*^9, 3.689852617762125*^9}, {3.689852971301806*^9, 
   3.689852983939478*^9}, 3.689853210851042*^9, {3.689853596745799*^9, 
   3.689853629988327*^9}},ExpressionUUID->"10510408-b270-4084-9185-\
6d72b98d2b76"]
}, Closed]],

Cell[CellGroupData[{

Cell["set to True if all elements are True; False otherwise", "Subsubsection",
 CellChangeTimes->{{3.689852523990015*^9, 3.689852570187973*^9}, {
  3.689852628472856*^9, 
  3.689852658921709*^9}},ExpressionUUID->"fa874909-b4b0-4a9b-b7a1-\
e59dca37057c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{"a_", ",", 
     RowBox[{"b", ":", 
      RowBox[{
       RowBox[{"Except", "[", "_Rule", "]"}], ".."}]}], ",", "opts___Rule"}], 
    "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"And", "@@", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"TrueQ", "[", 
        RowBox[{"HilbertSpaceCompatibility", "[", 
         RowBox[{"#", ",", "opts"}], "]"}], "]"}], "&"}], "/@", 
      RowBox[{"{", 
       RowBox[{"a", ",", "b"}], "}"}]}], ")"}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.543409402644809*^9, 3.543409457810437*^9}, {
  3.543409716303382*^9, 3.543409722677971*^9}, {3.543409941480448*^9, 
  3.543409954526032*^9}, {3.6898525081366158`*^9, 3.68985251806497*^9}, {
  3.689852662508918*^9, 3.689852668642235*^9}, {3.6898529938617067`*^9, 
  3.689852999587616*^9}},ExpressionUUID->"9a9c5c55-1840-4e6f-aea4-\
e7654386d1ec"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["evolution generator specifications", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.503093847300502*^9, 3.503093896083952*^9}, {
   3.5030949750455627`*^9, 3.503094977796137*^9}, {3.506000924941127*^9, 
   3.5060009278359337`*^9}, {3.5422679204480343`*^9, 3.542267929045515*^9}, 
   3.68985120873044*^9, 
   3.690547232123403*^9},ExpressionUUID->"ec681d0c-ddb1-4b49-a2dd-\
479876b4be0d"],

Cell["with no BackgroundGenerator option", "Text",
 CellChangeTimes->{{3.69054722748566*^9, 3.690547228965711*^9}, 
   3.7093749277057133`*^9},ExpressionUUID->"a810ce88-05fd-4b77-ba06-\
716e546646f2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", "Undefined", "]"}], "=", 
   "Undefined"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.689851263785409*^9, 3.689851270335986*^9}, {
  3.689852102022154*^9, 
  3.689852105819993*^9}},ExpressionUUID->"fab23a93-df6b-4bbf-8c8f-\
63792f7f168a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", "None", "]"}], ":=", "True"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5030277474382343`*^9, 3.503027813217532*^9}, {
  3.503093709786152*^9, 3.50309374596668*^9}, {3.503093791240205*^9, 
  3.5030938122629223`*^9}, {3.5030940434049664`*^9, 3.503094046173657*^9}, {
  3.503095001830377*^9, 3.50309505246984*^9}, {3.5060008940497303`*^9, 
  3.506000896844101*^9}, {3.542879043686346*^9, 
  3.542879044280644*^9}},ExpressionUUID->"11cc094f-c783-4be2-bf0b-\
b0da411fd34a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{"_", "?", "OperatorQ"}], "]"}], ":=", "True"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5030277474382343`*^9, 3.503027813217532*^9}, {
  3.503093709786152*^9, 3.50309374596668*^9}, {3.503093791240205*^9, 
  3.5030938122629223`*^9}, {3.5030940434049664`*^9, 3.503094046173657*^9}, {
  3.503095001830377*^9, 3.50309505246984*^9}, {3.5060008940497303`*^9, 
  3.506000896844101*^9}},ExpressionUUID->"e79e2d9c-16a9-446a-b681-\
4cd03c5a48c2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"Function", "|", "PeriodicFunction", "|", "ShapeFunction"}], 
      ")"}], "[", 
     RowBox[{"__", ",", 
      RowBox[{
       RowBox[{"_", "?", "OperatorQ"}], "|", "_CommutationSuperoperator"}]}], 
     "]"}], "]"}], ":=", "True"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5030277474382343`*^9, 3.503027813217532*^9}, {
   3.503093709786152*^9, 3.50309374596668*^9}, {3.503093791240205*^9, 
   3.5030938122629223`*^9}, {3.5030940434049664`*^9, 3.503094046173657*^9}, {
   3.503095001830377*^9, 3.50309505246984*^9}, {3.5060008940497303`*^9, 
   3.506000896844101*^9}, {3.50600094510467*^9, 3.506000967696032*^9}, 
   3.5060010247248774`*^9, {3.507020509284843*^9, 3.507020543431005*^9}, 
   3.542270240367004*^9},ExpressionUUID->"6b016fe9-42da-476b-a8ac-\
3a17393e607d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{
    "_RotationSuperoperator", "|", "_UnitySuperoperator", "|", 
     "_CommutationSuperoperator"}], "]"}], ":=", "True"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5030277474382343`*^9, 3.503027813217532*^9}, {
  3.503093709786152*^9, 3.50309374596668*^9}, {3.503093791240205*^9, 
  3.5030938122629223`*^9}, {3.5030940434049664`*^9, 3.503094046173657*^9}, {
  3.503095001830377*^9, 3.50309505246984*^9}, {3.6910580679316883`*^9, 
  3.691058068513526*^9}},ExpressionUUID->"85b3df37-e9c6-499a-88ad-\
497792cb8bfe"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{"HoldPattern", "@", 
     RowBox[{"Dot", "[", 
      RowBox[{"_RotationSuperoperator", ".."}], "]"}]}], "]"}], ":=", 
   "True"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5030277474382343`*^9, 3.503027813217532*^9}, {
  3.503093709786152*^9, 3.50309374596668*^9}, {3.503093791240205*^9, 
  3.5030938122629223`*^9}, {3.5030940434049664`*^9, 3.503094046173657*^9}, {
  3.503095001830377*^9, 3.50309505246984*^9}, {3.683977661937216*^9, 
  3.683977675223523*^9}},ExpressionUUID->"7d5f4dcd-9c41-4497-b3ca-\
73cfa0446cf2"]
}, Closed]],

Cell[CellGroupData[{

Cell["EventSpecifications", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.503093847300502*^9, 3.5030938604684362`*^9}, 
   3.5147216276975393`*^9, {3.5422675661679163`*^9, 3.5422675729722633`*^9}, {
   3.54226761318013*^9, 3.54226762022583*^9}, 3.543069031750772*^9, 
   3.690547254634204*^9},ExpressionUUID->"e4c6e66c-16c1-4c1c-ae54-\
7aeed9d059af"],

Cell[CellGroupData[{

Cell["\<\
{Generator, interval} pair (BackgroundGenerator specification is needed)\
\>", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.503093847300502*^9, 3.503093896083952*^9}, {
   3.5030949750455627`*^9, 3.503094977796137*^9}, {3.506000924941127*^9, 
   3.5060009278359337`*^9}, {3.5103729718805103`*^9, 3.51037298866218*^9}, {
   3.510373302740259*^9, 3.5103733080736227`*^9}, {3.5104769421737423`*^9, 
   3.510476974236203*^9}, {3.510477226452284*^9, 3.510477236179697*^9}, 
   3.510477325698626*^9, {3.542267736389242*^9, 3.542267737125792*^9}, {
   3.542269818987047*^9, 3.5422698220548697`*^9}, {3.5434098052431917`*^9, 
   3.5434098301919403`*^9}, {3.689851162652891*^9, 3.6898511753634367`*^9}, {
   3.689851220644266*^9, 3.689851226368157*^9}, 3.709374513984824*^9, 
   3.709374928637535*^9},ExpressionUUID->"84f4e0ce-7d34-4b61-8666-\
8cfa350e77e6"],

Cell["\<\
note: for a {Generator, interval} pair, the BackgroundGenerator must act \
during the interval, so it is important for determining \
HilbertSpaceCompatibility. \
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.542269723973798*^9, 3.542269767250143*^9}, 
   3.709374514058951*^9, 
   3.7093749300226183`*^9},ExpressionUUID->"632f5eb8-e993-45c3-b170-\
c604489195f6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"HilbertSpaceCompatibility", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"evgen_", "?", "ValidGeneratorQ"}], ",", "\[Tau]_"}], "}"}], 
      ",", "opts___Rule"}], "]"}], "/;", 
    RowBox[{"!", 
     RowBox[{"TrueQ", "[", 
      RowBox[{
       RowBox[{"Chop", "[", "\[Tau]", "]"}], "\[Equal]", "0"}], "]"}]}]}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{"evgen", ",", 
     RowBox[{
      RowBox[{"BackgroundGenerator", "/.", 
       RowBox[{"{", "opts", "}"}]}], "/.", 
      RowBox[{"Options", "[", "HilbertSpaceCompatibility", "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5422681916828613`*^9, 3.5422682031801453`*^9}, {
   3.542269787773435*^9, 3.542269810285828*^9}, {3.543409888285014*^9, 
   3.543409902970767*^9}, {3.543410004277625*^9, 3.543410078571067*^9}, 
   3.7093737169715652`*^9, 
   3.709374930768091*^9},ExpressionUUID->"9bfc4320-e279-4961-8e1f-\
b92c9a6c13c6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"HilbertSpaceCompatibility", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"evgen_", "?", "ValidGeneratorQ"}], ",", "\[Tau]_"}], "}"}], 
      ",", "opts___Rule"}], "]"}], "/;", 
    RowBox[{"TrueQ", "[", 
     RowBox[{
      RowBox[{"Chop", "[", "\[Tau]", "]"}], "\[Equal]", "0"}], "]"}]}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"HilbertSpaceCompatibility", "[", "evgen", "]"}]}], ";"}]], "Input",\

 InitializationCell->True,
 CellChangeTimes->{{3.5422681916828613`*^9, 3.5422682031801453`*^9}, {
   3.542269787773435*^9, 3.542269810285828*^9}, {3.543409888285014*^9, 
   3.543409902970767*^9}, {3.543410004277625*^9, 3.543410078571067*^9}, {
   3.544599396543374*^9, 3.5445994028048553`*^9}, 
   3.709373716980886*^9},ExpressionUUID->"14608f8c-d4fe-4b56-8bb3-\
e97b5d7ddbca"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
if the event spec is a pure Generator, with additional options, simply strip \
the options\
\>", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.503093847300502*^9, 3.503093896083952*^9}, {
   3.5030949750455627`*^9, 3.503094977796137*^9}, {3.506000924941127*^9, 
   3.5060009278359337`*^9}, {3.5103729718805103`*^9, 3.51037298866218*^9}, {
   3.510373302740259*^9, 3.5103733080736227`*^9}, {3.5104769421737423`*^9, 
   3.510476974236203*^9}, {3.510477085980472*^9, 3.510477102883297*^9}, {
   3.5422678639034853`*^9, 3.542267890864607*^9}, {3.542268153841033*^9, 
   3.54226815869993*^9}, {3.542268219784878*^9, 3.5422682259464273`*^9}, {
   3.542268260402486*^9, 3.5422682685509*^9}, {3.542269829722147*^9, 
   3.542269834662375*^9}, 
   3.7093745141213913`*^9},ExpressionUUID->"a03c5aad-dac1-46e8-9c5d-\
01680975bf31"],

Cell["\<\
for an instantaneous Evolution generator, the BackgroundGenerator does not \
act, so is irrelevant to HilbertSpaceCompatibility\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.5422698378546247`*^9, 3.542269868021264*^9}, 
   3.709374932350255*^9},ExpressionUUID->"73210cf6-3012-40aa-b496-\
4f2570e8ab7c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{
     RowBox[{"evgen_", "?", "ValidGeneratorQ"}], ",", "opts__Rule"}], "]"}], ":=", 
   RowBox[{"HilbertSpaceCompatibility", "[", "evgen", "]"}]}], ";"}]], "Input",\

 InitializationCell->True,
 CellChangeTimes->{{3.5103729973779507`*^9, 3.510373070093759*^9}, {
   3.5103731880013313`*^9, 3.51037319111071*^9}, {3.5104769817607117`*^9, 
   3.510477038707361*^9}, {3.510477113088801*^9, 3.510477129136993*^9}, {
   3.510477699764401*^9, 3.5104777100420303`*^9}, 3.5422679402515287`*^9, {
   3.542268273292028*^9, 3.5422682879771833`*^9}, {3.5422698816124363`*^9, 
   3.54226988589472*^9}, 
   3.709373716989723*^9},ExpressionUUID->"23aade11-777c-491f-9d0e-\
0c6e728a0a34"]
}, Closed]],

Cell[CellGroupData[{

Cell["Superoperators", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.503093847300502*^9, 3.503093896083952*^9}, {
   3.5030949750455627`*^9, 3.503094977796137*^9}, {3.506000924941127*^9, 
   3.5060009278359337`*^9}, {3.5103729718805103`*^9, 3.51037298866218*^9}, {
   3.510373302740259*^9, 3.5103733080736227`*^9}, {3.5104769421737423`*^9, 
   3.510476974236203*^9}, {3.510477226452284*^9, 3.510477236179697*^9}, 
   3.510477325698626*^9, {3.538921891460203*^9, 3.538921921650586*^9}, {
   3.542267729037788*^9, 3.542267734725152*^9}, 3.542268092043807*^9, 
   3.5434097955032263`*^9, {3.690989538873423*^9, 3.690989541950333*^9}, 
   3.6909896409290524`*^9},ExpressionUUID->"51097dcb-b24e-4017-b575-\
61dc2c45e610"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"sop_", "?", "SuperoperatorQ"}], ",", "___"}], "}"}], ",", 
     "opts___Rule"}], "]"}], ":=", 
   RowBox[{"HilbertSpaceCompatibility", "[", "sop", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.538921940533551*^9, 3.5389219540985107`*^9}, 
   3.538922092517519*^9, {3.689851716653294*^9, 3.689851718578698*^9}, {
   3.690989554031042*^9, 
   3.690989592480506*^9}},ExpressionUUID->"bf15a96c-98de-4a08-8408-\
06b042c47411"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{
     RowBox[{
     "_RotationSuperoperator", "|", "_UnitySuperoperator", "|", 
      "_NullSuperoperator"}], ",", "opts___Rule"}], "]"}], "=", "True"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.538921940533551*^9, 3.5389219661468077`*^9}, 
   3.538922095697784*^9, {3.689851721501439*^9, 3.689851723155254*^9}, {
   3.690989599844571*^9, 
   3.6909896433853283`*^9}},ExpressionUUID->"0a5ff42e-79b9-479d-a39d-\
8f3ffc25d619"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{
     RowBox[{"_", "?", "SuperoperatorQ"}], ",", "opts___Rule"}], "]"}], ":=", 
   "False"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.538921940533551*^9, 3.5389219661468077`*^9}, 
   3.538922095697784*^9, {3.689851721501439*^9, 3.689851723155254*^9}, {
   3.690989599844571*^9, 
   3.690989668134397*^9}},ExpressionUUID->"3553cfac-df8a-411a-bf7f-\
ef26158498ad"]
}, Closed]],

Cell[CellGroupData[{

Cell["Repeat", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.503093847300502*^9, 3.503093896083952*^9}, {
   3.5030949750455627`*^9, 3.503094977796137*^9}, {3.506000924941127*^9, 
   3.5060009278359337`*^9}, {3.5103729718805103`*^9, 3.51037298866218*^9}, {
   3.510373302740259*^9, 3.5103733080736227`*^9}, {3.5104769421737423`*^9, 
   3.510476974236203*^9}, {3.510477226452284*^9, 3.510477236179697*^9}, {
   3.5104772815997066`*^9, 3.510477305511805*^9}, {3.54226763145278*^9, 
   3.5422676500489683`*^9}, {3.542267726362865*^9, 3.542267727118186*^9}, 
   3.543409776901444*^9, 3.689853846441366*^9, 
   3.6928483463423223`*^9},ExpressionUUID->"38f6f186-6ccf-412a-b1ae-\
b39910d1768a"],

Cell["\<\
The overall HilbertSpaceCompatibility of Repeat is never set to \
MixedHilbertSpaceCompatibility, since it cannot be divided.\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.547012384138212*^9, 3.54701252942557*^9}, {
   3.547012712932067*^9, 3.547012721666359*^9}, {3.547012777687043*^9, 
   3.547012800550934*^9}, {3.689853306892531*^9, 3.689853318868391*^9}, 
   3.692848347113988*^9},ExpressionUUID->"700e35c3-05aa-4bb5-a57c-\
51f0734e510a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"HilbertSpaceCompatibility", "[", 
     RowBox[{
      RowBox[{"Repeat", "[", 
       RowBox[{"evs_", ",", "n_Integer", ",", 
        RowBox[{
         RowBox[{"Except", "[", "_Rule", "]"}], "..."}], ",", 
        "repevopts___Rule"}], "]"}], ",", "opts___Rule"}], "]"}]}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"TrueQ", "[", 
    RowBox[{"HilbertSpaceCompatibility", "[", 
     RowBox[{"evs", ",", "repevopts", ",", "opts"}], "]"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.503028531813323*^9, 3.503028615228005*^9}, {
   3.503028719896583*^9, 3.503028720292967*^9}, {3.5030288437012367`*^9, 
   3.503028848658291*^9}, {3.510477246344934*^9, 3.51047725859214*^9}, {
   3.5112770415941477`*^9, 3.511277049031865*^9}, {3.5112773922568083`*^9, 
   3.511277402336863*^9}, {3.511277521852673*^9, 3.511277537988703*^9}, 
   3.5112781704789267`*^9, {3.514350106521628*^9, 3.5143501225590143`*^9}, {
   3.51472176826131*^9, 3.514721780718747*^9}, {3.54226765351816*^9, 
   3.542267682700658*^9}, {3.547012559534193*^9, 3.547012641081231*^9}, {
   3.689853311689639*^9, 3.6898533157218647`*^9}, {3.691676520207378*^9, 
   3.691676527068408*^9}, 
   3.6928483478017187`*^9},ExpressionUUID->"b8baccde-72fc-413b-83e3-\
b2caeb2aee82"]
}, Closed]],

Cell[CellGroupData[{

Cell["PrecalculatedEvent", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.503093847300502*^9, 3.503093896083952*^9}, {
   3.5030949750455627`*^9, 3.503094977796137*^9}, {3.506000924941127*^9, 
   3.5060009278359337`*^9}, {3.5103729718805103`*^9, 3.51037298866218*^9}, {
   3.510373302740259*^9, 3.5103733080736227`*^9}, {3.5104769421737423`*^9, 
   3.510476974236203*^9}, {3.510477226452284*^9, 3.510477236179697*^9}, {
   3.5104772815997066`*^9, 3.510477305511805*^9}, {3.54226763145278*^9, 
   3.5422676500489683`*^9}, {3.542267726362865*^9, 3.542267727118186*^9}, 
   3.543409776901444*^9, 3.689853846441366*^9, {3.691597751208712*^9, 
   3.691597770710338*^9}},ExpressionUUID->"e35f2831-3d97-4afb-b0e4-\
df5592781a6b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{
     RowBox[{"PrecalculatedEvent", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"tch_", ",", "evgench_", ",", "hilb_", ",", "___"}], "}"}], 
       ",", "___"}], "]"}], ",", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", "hilb"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.503028531813323*^9, 3.503028615228005*^9}, {
   3.503028719896583*^9, 3.503028720292967*^9}, {3.5030288437012367`*^9, 
   3.503028848658291*^9}, {3.510477246344934*^9, 3.51047725859214*^9}, {
   3.5112770415941477`*^9, 3.511277049031865*^9}, {3.5112773922568083`*^9, 
   3.511277402336863*^9}, {3.511277521852673*^9, 3.511277537988703*^9}, 
   3.5112781704789267`*^9, {3.514350106521628*^9, 3.5143501225590143`*^9}, {
   3.51472176826131*^9, 3.514721780718747*^9}, {3.54226765351816*^9, 
   3.542267682700658*^9}, {3.547012559534193*^9, 3.547012641081231*^9}, {
   3.689853311689639*^9, 3.6898533157218647`*^9}, {3.691597763439114*^9, 
   3.691597818510249*^9}},ExpressionUUID->"e413e7e8-5646-40cd-9281-\
323b67956357"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Event", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.503114312783538*^9, 3.503114318486227*^9}, 
   3.542267420148671*^9, 3.685033554390809*^9, 3.6850358096557293`*^9, 
   3.689007200528676*^9},ExpressionUUID->"84dc41c1-2605-448a-ab7c-\
0d1eb1354264"],

Cell[CellGroupData[{

Cell["extract if specified internally", "Subsubsection",
 CellChangeTimes->{{3.689854661426651*^9, 3.68985475033141*^9}, 
   3.6898548174348373`*^9},ExpressionUUID->"d8eea93c-e56f-443e-b2ee-\
76bd2ab08d98"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"evstatus_", ",", 
       RowBox[{"evch", ":", 
        RowBox[{"{", 
         RowBox[{"evtch_", ",", 
          RowBox[{"{", 
           RowBox[{"evbackg_", ",", "morevbackgch__"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"hilbcomp", ":", 
           RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "moreevch__"}], 
         "}"}]}], ",", "\[IndentingNewLine]", "evspecs_", ",", 
       "\[IndentingNewLine]", "evopts___Rule"}], "\[IndentingNewLine]", "]"}],
      ",", "\[IndentingNewLine]", "opts___Rule"}], "\[IndentingNewLine]", 
    "]"}], ":=", "hilbcomp"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.503114325602059*^9, 3.5031143940813093`*^9}, {
   3.5031145343735247`*^9, 3.5031145396985703`*^9}, 3.542267182485188*^9, {
   3.542267279994587*^9, 3.5422672890980053`*^9}, {3.542267356239645*^9, 
   3.542267400213476*^9}, {3.542268410683668*^9, 3.542268669894081*^9}, {
   3.542269210275975*^9, 3.542269248239641*^9}, {3.5422694108202*^9, 
   3.542269411922327*^9}, {3.6850335543995323`*^9, 3.6850335544019547`*^9}, {
   3.685035809664838*^9, 3.685035809667202*^9}, {3.685035888320031*^9, 
   3.6850358890669518`*^9}, {3.68985442870784*^9, 3.68985461041173*^9}, {
   3.689854807083908*^9, 
   3.6898548096744967`*^9}},ExpressionUUID->"9eaa52f8-a5b6-4ed4-9f12-\
e4ebd54a4953"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
set as Undefined if no backg is specified internally or externally\
\>", "Subsubsection",
 CellChangeTimes->{{3.689854661426651*^9, 3.6898546921756773`*^9}, {
  3.6898548653002977`*^9, 
  3.6898548710980043`*^9}},ExpressionUUID->"bda74eb2-b05a-4899-b959-\
6bfb7832055b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"HilbertSpaceCompatibility", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Event", "[", 
       RowBox[{"evstatus_", ",", 
        RowBox[{"evch", ":", 
         RowBox[{"{", 
          RowBox[{"evtch_", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"evbackg", ":", "Undefined"}], ",", "morevbackgch__"}], 
            "}"}], ",", "\[IndentingNewLine]", "hilbcomp_", ",", 
           "moreevch__"}], "}"}]}], ",", "\[IndentingNewLine]", "evspecs_", 
        ",", "\[IndentingNewLine]", "evopts___Rule"}], "\[IndentingNewLine]", 
       "]"}], ",", "\[IndentingNewLine]", "opts___Rule"}], 
     "\[IndentingNewLine]", "]"}], "/;", 
    RowBox[{"MatchQ", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"BackgroundGenerator", "/.", 
        RowBox[{"{", "opts", "}"}]}], "/.", 
       RowBox[{"Options", "[", "HilbertSpaceCompatibility", "]"}]}], ",", 
      "Undefined"}], "]"}]}], ":=", "Undefined"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.503114325602059*^9, 3.5031143940813093`*^9}, {
   3.5031145343735247`*^9, 3.5031145396985703`*^9}, 3.542267182485188*^9, {
   3.542267279994587*^9, 3.5422672890980053`*^9}, {3.542267356239645*^9, 
   3.542267400213476*^9}, {3.542268410683668*^9, 3.542268669894081*^9}, {
   3.542269210275975*^9, 3.542269248239641*^9}, {3.5422694108202*^9, 
   3.542269411922327*^9}, {3.6850335543995323`*^9, 3.6850335544019547`*^9}, {
   3.685035809664838*^9, 3.685035809667202*^9}, {3.685035888320031*^9, 
   3.6850358890669518`*^9}, {3.68985442870784*^9, 3.689854649348021*^9}, {
   3.6898548772678423`*^9, 3.689854900674177*^9}, 
   3.709374933445785*^9},ExpressionUUID->"ab0f9bfd-2ffc-4657-98aa-\
085cd4699116"]
}, Closed]],

Cell[CellGroupData[{

Cell["If backg is not specified internally, deduce it", "Subsubsection",
 CellChangeTimes->{{3.689854661426651*^9, 3.689854791082859*^9}, {
  3.6898549317263403`*^9, 
  3.689854933339353*^9}},ExpressionUUID->"14e1f5c9-5d68-4930-bd6d-\
3ca1b570e8cd"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"evstatus_", ",", 
       RowBox[{"evch", ":", 
        RowBox[{"{", 
         RowBox[{"evtch_", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"evbackg", ":", "Undefined"}], ",", "morevbackgch__"}], 
           "}"}], ",", "\[IndentingNewLine]", 
          RowBox[{"hilbcomp", ":", "Undefined"}], ",", "moreevch__"}], 
         "}"}]}], ",", "\[IndentingNewLine]", "evspecs_", ",", 
       "\[IndentingNewLine]", "evopts___Rule"}], "\[IndentingNewLine]", "]"}],
      ",", "\[IndentingNewLine]", "opts___Rule"}], "\[IndentingNewLine]", 
    "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "extbackg", "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{"HilbertSpaceCompatibility", "[", 
        RowBox[{"evspecs", ",", "opts"}], "]"}], "\[IndentingNewLine]", ")"}],
       "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"extbackg", "=", 
         RowBox[{
          RowBox[{"BackgroundGenerator", "/.", 
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "HilbertSpaceCompatibility", "]"}]}]}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"!", 
         RowBox[{"MatchQ", "[", 
          RowBox[{"extbackg", ",", "Undefined"}], "]"}]}]}], 
       "\[IndentingNewLine]", ")"}]}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.503114325602059*^9, 3.5031143940813093`*^9}, {
   3.5031145343735247`*^9, 3.5031145396985703`*^9}, 3.542267182485188*^9, {
   3.542267279994587*^9, 3.5422672890980053`*^9}, {3.542267356239645*^9, 
   3.542267400213476*^9}, {3.542268410683668*^9, 3.542268669894081*^9}, {
   3.542269210275975*^9, 3.542269248239641*^9}, {3.5422694108202*^9, 
   3.542269411922327*^9}, {3.6850335543995323`*^9, 3.6850335544019547`*^9}, {
   3.685035809664838*^9, 3.685035809667202*^9}, {3.685035888320031*^9, 
   3.6850358890669518`*^9}, {3.68985442870784*^9, 3.68985461041173*^9}, {
   3.6898549597424393`*^9, 3.689855037620762*^9}, 
   3.709374934368312*^9},ExpressionUUID->"edeeb30e-29b5-4113-b9e4-\
9905b25ede96"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["EventList", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.503114312783538*^9, 3.503114318486227*^9}, 
   3.542267420148671*^9, {3.543069045324604*^9, 3.543069046020339*^9}, 
   3.685033531014098*^9, 
   3.685033584867586*^9},ExpressionUUID->"3078b2c1-fd96-41e5-b797-\
26ed84381114"],

Cell[CellGroupData[{

Cell["extract if specified internally", "Subsubsection",
 CellChangeTimes->{{3.689854661426651*^9, 3.68985475033141*^9}, 
   3.6898548174348373`*^9},ExpressionUUID->"c606b5f5-c794-46d4-9733-\
86cfed498111"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{"evlstatus_", ",", 
       RowBox[{"evlch", ":", 
        RowBox[{"{", 
         RowBox[{"evltch_", ",", 
          RowBox[{"{", 
           RowBox[{"evlbackg_", ",", "morevlbackgch__"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"hilbcomp", ":", 
           RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "moreevlch__"}], 
         "}"}]}], ",", "\[IndentingNewLine]", "events_List", ",", 
       "\[IndentingNewLine]", "evlopts___Rule"}], "\[IndentingNewLine]", 
      "]"}], ",", "\[IndentingNewLine]", "opts___Rule"}], 
    "\[IndentingNewLine]", "]"}], ":=", "hilbcomp"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.503114325602059*^9, 3.5031143940813093`*^9}, {
   3.5031145343735247`*^9, 3.5031145396985703`*^9}, 3.542267182485188*^9, {
   3.542267279994587*^9, 3.5422672890980053`*^9}, {3.542267356239645*^9, 
   3.542267400213476*^9}, {3.542268410683668*^9, 3.542268669894081*^9}, {
   3.542269210275975*^9, 3.542269248239641*^9}, {3.5422694108202*^9, 
   3.542269411922327*^9}, {3.6850335543995323`*^9, 3.6850335544019547`*^9}, {
   3.685035809664838*^9, 3.685035809667202*^9}, {3.685035888320031*^9, 
   3.6850358890669518`*^9}, {3.68985442870784*^9, 3.68985461041173*^9}, {
   3.689854807083908*^9, 3.6898548096744967`*^9}, {3.6898553843635263`*^9, 
   3.689855426800116*^9}},ExpressionUUID->"1496e607-fd33-4142-adf5-\
7cf401e80641"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
set as Undefined if no backg is specified internally or externally\
\>", "Subsubsection",
 CellChangeTimes->{{3.689854661426651*^9, 3.6898546921756773`*^9}, {
  3.6898548653002977`*^9, 
  3.6898548710980043`*^9}},ExpressionUUID->"4eb1db02-a779-4196-bffd-\
c32c6020b57c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"HilbertSpaceCompatibility", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"EventList", "[", 
       RowBox[{"evlstatus_", ",", 
        RowBox[{"evlch", ":", 
         RowBox[{"{", 
          RowBox[{"evltch_", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"evlbackg", ":", "Undefined"}], ",", "morevlbackgch__"}],
             "}"}], ",", "\[IndentingNewLine]", 
           RowBox[{"hilbcomp", ":", 
            RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "moreevlch__"}],
           "}"}]}], ",", "\[IndentingNewLine]", "events_List", ",", 
        "\[IndentingNewLine]", "evlopts___Rule"}], "\[IndentingNewLine]", 
       "]"}], ",", "\[IndentingNewLine]", "opts___Rule"}], 
     "\[IndentingNewLine]", "]"}], "/;", 
    RowBox[{"MatchQ", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"BackgroundGenerator", "/.", 
        RowBox[{"{", "opts", "}"}]}], "/.", 
       RowBox[{"Options", "[", "HilbertSpaceCompatibility", "]"}]}], ",", 
      "Undefined"}], "]"}]}], ":=", "Undefined"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.503114325602059*^9, 3.5031143940813093`*^9}, {
   3.5031145343735247`*^9, 3.5031145396985703`*^9}, 3.542267182485188*^9, {
   3.542267279994587*^9, 3.5422672890980053`*^9}, {3.542267356239645*^9, 
   3.542267400213476*^9}, {3.542268410683668*^9, 3.542268669894081*^9}, {
   3.542269210275975*^9, 3.542269248239641*^9}, {3.5422694108202*^9, 
   3.542269411922327*^9}, {3.6850335543995323`*^9, 3.6850335544019547`*^9}, {
   3.685035809664838*^9, 3.685035809667202*^9}, {3.685035888320031*^9, 
   3.6850358890669518`*^9}, {3.68985442870784*^9, 3.689854649348021*^9}, {
   3.6898548772678423`*^9, 3.689854900674177*^9}, {3.689855446746731*^9, 
   3.68985545302355*^9}, 
   3.709374934987027*^9},ExpressionUUID->"c7053374-b1ba-4c60-a790-\
92d9b65aa2cd"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
If backg is not specified internally, deduce overall compatibility of \
contained events\
\>", "Subsubsection",
 CellChangeTimes->{{3.689854661426651*^9, 3.689854791082859*^9}, {
  3.6898549317263403`*^9, 3.689854933339353*^9}, {3.6898555669587107`*^9, 
  3.689855574279468*^9}},ExpressionUUID->"b86a82f3-db4b-4d91-9027-\
e4366a7da039"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{"evlstatus_", ",", 
       RowBox[{"evlch", ":", 
        RowBox[{"{", 
         RowBox[{"evltch_", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"evlbackg", ":", "Undefined"}], ",", "morevlbackgch__"}], 
           "}"}], ",", "\[IndentingNewLine]", 
          RowBox[{"hilbcomp", ":", "Undefined"}], ",", "moreevlch__"}], 
         "}"}]}], ",", "\[IndentingNewLine]", "events_List", ",", 
       "\[IndentingNewLine]", "evlopts___Rule"}], "\[IndentingNewLine]", 
      "]"}], ",", "\[IndentingNewLine]", "opts___Rule"}], 
    "\[IndentingNewLine]", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "extbackg", "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{"And", "@@", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"HilbertSpaceCompatibility", "[", 
            RowBox[{"#", ",", "opts"}], "]"}], "&"}], "/@", "events"}], 
         ")"}]}], "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"extbackg", "=", 
         RowBox[{
          RowBox[{"BackgroundGenerator", "/.", 
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "HilbertSpaceCompatibility", "]"}]}]}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"!", 
         RowBox[{"MatchQ", "[", 
          RowBox[{"extbackg", ",", "Undefined"}], "]"}]}]}], 
       "\[IndentingNewLine]", ")"}]}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.503114325602059*^9, 3.5031143940813093`*^9}, {
   3.5031145343735247`*^9, 3.5031145396985703`*^9}, 3.542267182485188*^9, {
   3.542267279994587*^9, 3.5422672890980053`*^9}, {3.542267356239645*^9, 
   3.542267400213476*^9}, {3.542268410683668*^9, 3.542268669894081*^9}, {
   3.542269210275975*^9, 3.542269248239641*^9}, {3.5422694108202*^9, 
   3.542269411922327*^9}, {3.6850335543995323`*^9, 3.6850335544019547`*^9}, {
   3.685035809664838*^9, 3.685035809667202*^9}, {3.685035888320031*^9, 
   3.6850358890669518`*^9}, {3.68985442870784*^9, 3.68985461041173*^9}, {
   3.6898549597424393`*^9, 3.689855037620762*^9}, {3.689855496529209*^9, 
   3.689855513767069*^9}, {3.6898555800832367`*^9, 3.689855595311911*^9}, 
   3.689855730382594*^9, 
   3.709374935740933*^9},ExpressionUUID->"e3554c72-a10d-4fc7-aedc-\
6500a06704d8"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["apply EventSpecifications if not recognized", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.503093847300502*^9, 3.5030938604684362`*^9}, {
   3.5147216276975393`*^9, 3.5147216680663433`*^9}, {3.514722033627378*^9, 
   3.5147220403769693`*^9}, {3.542267542423698*^9, 3.542267547514539*^9}, 
   3.6839576808417807`*^9, {3.685033531057098*^9, 3.685033554417679*^9}, 
   3.685033584905352*^9, 3.6850358096828613`*^9, 3.6850358912617598`*^9, {
   3.689855703295095*^9, 3.68985571769037*^9}, {3.6904498909303427`*^9, 
   3.69044989195798*^9}},ExpressionUUID->"94b1f768-2518-45fc-828a-\
83a1dda3e250"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"HilbertSpaceCompatibility", "[", 
     RowBox[{"spec_", ",", "opts___Rule"}], "]"}], "/;", 
    RowBox[{"MatchQ", "[", 
     RowBox[{"spec", ",", 
      RowBox[{"Except", "[", 
       RowBox[{
        RowBox[{"_", "?", "ValidGeneratorQ"}], "|", "_EventSpecifications", 
        "|", "_Event", "|", 
        RowBox[{"_", "?", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"StandardizedEventSpecificationQ", "[", 
            RowBox[{"#", ",", "opts"}], "]"}], "&"}], ")"}]}], "|", 
        "_EventList", "|", 
        RowBox[{"{", 
         RowBox[{"_Event", ".."}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
     "]"}]}], ":=", 
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{
     RowBox[{"EventSpecifications", "[", 
      RowBox[{"spec", ",", "opts"}], "]"}], ",", "opts"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.514721677414307*^9, 3.5147217264855747`*^9}, {
   3.514722057320285*^9, 3.514722077239245*^9}, {3.5147228864708242`*^9, 
   3.5147228886313*^9}, {3.5149085115138807`*^9, 3.514908553368507*^9}, {
   3.542269319412915*^9, 3.542269361500197*^9}, {3.683957680850823*^9, 
   3.683957680853038*^9}, {3.685033531066785*^9, 3.685033554429633*^9}, 
   3.685033584912129*^9, {3.685035809694848*^9, 3.6850358096977463`*^9}, {
   3.685035894071327*^9, 3.685035895696126*^9}, 3.70048500893156*^9, 
   3.7093737170076*^9},ExpressionUUID->"b328c46b-c64c-4af8-adb3-e5e0e01af664"]
}, Closed]],

Cell[CellGroupData[{

Cell["overall compatibility of many events or event specifications", \
"Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.503093847300502*^9, 3.5030938604684362`*^9}, 
   3.5147216276975393`*^9, {3.5422675661679163`*^9, 
   3.5422675729722633`*^9}},ExpressionUUID->"f03eca9d-e54d-4f30-bc66-\
efe7bee5e8ef"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{
     RowBox[{"EventSpecifications", "[", 
      RowBox[{"True", ",", "evspecs_", ",", "___Rule"}], "]"}], ",", 
     "opts___Rule"}], "]"}], ":=", 
   RowBox[{"And", "@@", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"HilbertSpaceCompatibility", "[", 
        RowBox[{"#", ",", "opts"}], "]"}], "&"}], "/@", "evspecs"}], 
     ")"}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5030277474382343`*^9, 3.503027840072901*^9}, {
   3.5030283311767063`*^9, 3.503028349646119*^9}, {3.5103729284126663`*^9, 
   3.510372945567376*^9}, {3.510478014687549*^9, 3.510478017309148*^9}, {
   3.514350044323938*^9, 3.514350070472023*^9}, 3.683957680859318*^9, {
   3.701104730807946*^9, 
   3.701104732123698*^9}},ExpressionUUID->"3acce446-f6db-49df-92d4-\
32bf3a7b82ec"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{"evlist_EventList", ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{
     RowBox[{"First", "[", "evlist", "]"}], ",", "opts"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5030277474382343`*^9, 3.503027840072901*^9}, {
   3.5030283311767063`*^9, 3.503028349646119*^9}, {3.5103729284126663`*^9, 
   3.510372945567376*^9}, {3.510478014687549*^9, 3.510478017309148*^9}, 
   3.514721743821271*^9, {3.5147218369118547`*^9, 3.51472186834522*^9}, 
   3.685033531075861*^9, 
   3.685033584917349*^9},ExpressionUUID->"009b8fc1-30d3-4205-8dbc-\
8360cdfa3a8c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{
     RowBox[{"evs", ":", 
      RowBox[{"{", 
       RowBox[{"_Event", ".."}], "}"}]}], ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"And", "@@", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"HilbertSpaceCompatibility", "[", 
        RowBox[{"#", ",", "opts"}], "]"}], "&"}], "/@", "evs"}], ")"}]}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5030277474382343`*^9, 3.503027840072901*^9}, {
   3.5030283311767063`*^9, 3.503028349646119*^9}, {3.5103729284126663`*^9, 
   3.510372945567376*^9}, {3.510478014687549*^9, 3.510478017309148*^9}, 
   3.514721743821271*^9, 3.685033554437538*^9, 3.6850358097050743`*^9, 
   3.685035897782358*^9},ExpressionUUID->"967e48e7-c6dd-4b0d-9ea4-\
2e9e5e6d9d35"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{"ev_Event", ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{
     RowBox[{"First", "[", "ev", "]"}], ",", "opts"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5030277474382343`*^9, 3.503027840072901*^9}, {
   3.5030283311767063`*^9, 3.503028349646119*^9}, {3.5103729284126663`*^9, 
   3.510372945567376*^9}, {3.510478014687549*^9, 3.510478017309148*^9}, 
   3.514721743821271*^9, {3.5147218369118547`*^9, 3.51472186834522*^9}, {
   3.5147229622282267`*^9, 3.514722969619212*^9}, 3.6850335544442883`*^9, 
   3.685035809711495*^9, 
   3.6850358994539347`*^9},ExpressionUUID->"ed861e02-56e2-4a44-a487-\
33c2850356b9"]
}, Closed]],

Cell[CellGroupData[{

Cell["everything else \[Rule] False", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.503093847300502*^9, 3.503093896083952*^9}, {
  3.503094011604957*^9, 3.5030940126116447`*^9}, {3.510373114230063*^9, 
  3.5103731167872543`*^9}, {3.5112779884034643`*^9, 3.5112779905204372`*^9}, {
  3.683368540994141*^9, 
  3.6833685419709387`*^9}},ExpressionUUID->"bbd3ea00-1a0e-4c98-af2e-\
2ad7a613a556"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", "___", "]"}], ":=", "False"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.503094018078278*^9, 3.503094023293249*^9}, {
  3.5112778545566196`*^9, 3.511277920222308*^9}, {3.511277951318204*^9, 
  3.5112779824605913`*^9}, {3.511278178996146*^9, 
  3.511278183953916*^9}},ExpressionUUID->"a9ec6884-2b06-4700-baae-\
c9a81d6e95e0"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PropagatorCalculationProcedure", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, 
   3.683973033466785*^9, {3.683973750698831*^9, 3.683973753893053*^9}, {
   3.683976432753954*^9, 3.683976436691045*^9}, 3.68397674513378*^9, {
   3.683977981950523*^9, 3.683977988834993*^9}, {3.683978334297957*^9, 
   3.683978335668144*^9}, 3.683979655379014*^9, 
   3.683980372936699*^9},ExpressionUUID->"87ced246-0a16-4cf0-97a8-\
1c43e84b9aec"],

Cell[CellGroupData[{

Cell["insert into chars", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, {
   3.689747196866548*^9, 
   3.689747198558086*^9}},ExpressionUUID->"5bfede60-911e-4a74-9f93-\
aac38dec9aa3"],

Cell[CellGroupData[{

Cell["require implemented background", "Subsubsection",
 CellChangeTimes->{{3.683976638742824*^9, 3.683976652290977*^9}, {
  3.683978158687376*^9, 
  3.683978176483617*^9}},ExpressionUUID->"92f492a0-109b-45a5-bf09-\
4e3791700615"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"timingch_", ",", 
       RowBox[{"backgch", ":", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"backg", ":", 
           RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "_", ",", 
          "True"}], "}"}]}], ",", "hilbch_", ",", "Undefined", ",", 
       "morech___"}], "}"}], ",", "evspec_", ",", "opts___Rule"}], "]"}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "timingch", ",", "backgch", ",", "hilbch", ",", "\[IndentingNewLine]", 
       RowBox[{"PropagatorCalculationProcedure", "[", "evspec", "]"}], ",", 
       "\[IndentingNewLine]", "morech"}], "}"}], ",", "\[IndentingNewLine]", 
     "evspec", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683976480502823*^9, 
   3.683976634737287*^9}, {3.6839766756355143`*^9, 3.683976688875939*^9}, {
   3.683978094400653*^9, 3.68397814152866*^9}, {3.685033569573359*^9, 
   3.685033569575466*^9}, {3.6888402889017878`*^9, 3.688840303434574*^9}, {
   3.6903923624832697`*^9, 
   3.690392364920321*^9}},ExpressionUUID->"24e8b83e-59dc-45c0-bf85-\
bf183810509d"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["extract from chars", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, {
   3.6839803863572607`*^9, 3.683980387436741*^9}, {3.689747200122188*^9, 
   3.689747201607065*^9}},ExpressionUUID->"fb78da1a-b206-4686-9a5f-\
d4908834a894"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PropagatorCalculationProcedure", "[", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
        "timingch_", ",", "backgch_", ",", "hilbcompat_", ",", "pcproc_", ",",
          "morech___"}], "}"}], ",", "evspec_", ",", "opts___Rule"}], "]"}], 
     ",", "___"}], "]"}], ":=", "pcproc"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683980234573969*^9, 3.683980292705936*^9}, {
   3.683980344913636*^9, 3.683980364039876*^9}, {3.683980398543696*^9, 
   3.683980411877936*^9}, 
   3.6850335695788794`*^9},ExpressionUUID->"93dff44f-f6a4-40b8-affc-\
98c9b596778d"]
}, Closed]],

Cell[CellGroupData[{

Cell["derive PropagatorCalculationProcedure", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.455265644416114*^9, 3.455265648200944*^9}, {
   3.464256156522676*^9, 3.4642561730892*^9}, {3.464282095283453*^9, 
   3.4642821290032167`*^9}, {3.503094263066369*^9, 3.503094264574778*^9}, {
   3.503095573470295*^9, 3.503095585674507*^9}, 3.542213044425332*^9, {
   3.689747188282693*^9, 
   3.689747212513866*^9}},ExpressionUUID->"7df60467-9831-4976-8c4a-\
c1c7414e3857"],

Cell["\<\
deduces PropagatorCalculationProcedure for events of different types, \
including the BackgroundGenerator as specified as an option\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.464254704342938*^9, 3.464254733157114*^9}, {
   3.4642821034796877`*^9, 3.464282120183125*^9}, {3.542213078611453*^9, 
   3.542213103158127*^9}, 
   3.709374936547765*^9},ExpressionUUID->"f605ff41-9b0a-4073-9bea-\
38f78ebf7b17"],

Cell[CellGroupData[{

Cell["None", "Subsubsection",
 CellChangeTimes->{{3.689746390253031*^9, 3.68974639392351*^9}, {
  3.691056326844532*^9, 
  3.691056331832768*^9}},ExpressionUUID->"744a5e4e-146c-4af0-aad6-\
6bd6c77cd239"],

Cell[BoxData[
 RowBox[{
  RowBox[{"PropagatorCalculationProcedure", "[", 
   RowBox[{"NullEventSpecificationPattern", ",", "convopts___Rule"}], "]"}], ":=",
   "None"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.691056316686*^9},ExpressionUUID->"9deac513-42d5-45c4-8b9f-ddffe9c59104"]
}, Closed]],

Cell[CellGroupData[{

Cell["IntegrateEvolutionOverTime", "Subsubsection",
 CellChangeTimes->{{3.689746390253031*^9, 
  3.6897464099140778`*^9}},ExpressionUUID->"2399840b-0340-4445-b12a-\
fbc8e8625b9b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PropagatorCalculationProcedure", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"None", "|", 
        RowBox[{"op_", "?", "OperatorQ"}]}], ",", 
       RowBox[{"timing", ":", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"tb_", "?", "NumericQ"}], ",", 
           RowBox[{"ta_", "?", "NumericQ"}]}], "}"}], "|", 
         RowBox[{"\[Tau]_", "?", "NumericQ"}]}]}], ",", "opts___Rule"}], 
      "}"}], ",", "convopts___Rule"}], "]"}], ":=", 
   "IntegrateEvolutionOverTime"}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.4642547556562862`*^9, 3.464254828687524*^9}, 
   3.464256329437847*^9, {3.46428219093368*^9, 3.4642821996909018`*^9}, {
   3.4643493802686043`*^9, 3.464349382337306*^9}, {3.464349460062024*^9, 
   3.464349499796053*^9}, {3.4643495521201077`*^9, 3.464349556405148*^9}, 
   3.4643496413851423`*^9, {3.46436132438649*^9, 3.464361328482037*^9}, {
   3.464361462917412*^9, 3.464361464315563*^9}, 3.464434342113248*^9, {
   3.503014860826033*^9, 3.5030149099909887`*^9}, {3.503014975888389*^9, 
   3.5030149760709743`*^9}, {3.503015056134593*^9, 3.503015071780902*^9}, {
   3.503015211889443*^9, 3.5030152142229443`*^9}, {3.503015848160178*^9, 
   3.503015898704132*^9}, {3.503016149364303*^9, 3.503016202705385*^9}, 
   3.50301759939469*^9, 3.503094287917411*^9, {3.503096151641757*^9, 
   3.503096155423356*^9}},ExpressionUUID->"87e88881-64d0-4904-bff5-\
40e7bcfd98ff"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PropagatorCalculationProcedure", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"func", ":", 
        RowBox[{"(", 
         RowBox[{
         "_Function", "|", "_PeriodicFunction", "|", "_ShapeFunction"}], 
         ")"}]}], ",", 
       RowBox[{"timing", ":", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"tb_", "?", "NumericQ"}], ",", 
           RowBox[{"ta_", "?", "NumericQ"}]}], "}"}], "|", 
         RowBox[{"\[Tau]_", "?", "NumericQ"}]}]}], ",", "opts___Rule"}], 
      "}"}], ",", "convopts___Rule"}], "]"}], ":=", 
   "IntegrateEvolutionOverTime"}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.4642547556562862`*^9, 3.464254828687524*^9}, 
   3.464256329437847*^9, {3.46428219093368*^9, 3.4642821996909018`*^9}, {
   3.4643493802686043`*^9, 3.464349382337306*^9}, {3.464349460062024*^9, 
   3.464349499796053*^9}, {3.4643495521201077`*^9, 3.464349556405148*^9}, 
   3.4643496413851423`*^9, {3.46436132438649*^9, 3.464361328482037*^9}, {
   3.464361462917412*^9, 3.464361464315563*^9}, 3.464434342113248*^9, {
   3.503014860826033*^9, 3.5030149099909887`*^9}, {3.503014975888389*^9, 
   3.5030149760709743`*^9}, {3.503015056134593*^9, 3.503015071780902*^9}, {
   3.503015211889443*^9, 3.5030152142229443`*^9}, {3.5030152642967253`*^9, 
   3.503015288351104*^9}, {3.503015386350095*^9, 3.5030153888425827`*^9}, {
   3.5030159202344027`*^9, 3.503015926986417*^9}, {3.503016174994515*^9, 
   3.503016206563628*^9}, 3.503017599481573*^9, 3.503094296385785*^9, {
   3.5030943416990767`*^9, 3.5030943754975033`*^9}, {3.5042599641936903`*^9, 
   3.504259965208098*^9}, {3.5070154158788643`*^9, 3.5070154184064217`*^9}, {
   3.542213224949154*^9, 
   3.542213233715067*^9}},ExpressionUUID->"bd81c6db-e6e4-49b4-b5cd-\
f10d96db3c02"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PropagatorCalculationProcedure", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"sop_", "?", "SuperoperatorQ"}], ",", 
       RowBox[{"timing", ":", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"tb_", "?", "NumericQ"}], ",", 
           RowBox[{"ta_", "?", "NumericQ"}]}], "}"}], "|", 
         RowBox[{"\[Tau]_", "?", "NumericQ"}]}]}], ",", "opts___Rule"}], 
      "}"}], ",", "convopts___Rule"}], "]"}], ":=", 
   "IntegrateEvolutionOverTime"}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.4642547556562862`*^9, 3.464254828687524*^9}, 
   3.464256329437847*^9, {3.46428219093368*^9, 3.4642821996909018`*^9}, {
   3.4643493802686043`*^9, 3.464349382337306*^9}, {3.464349460062024*^9, 
   3.464349499796053*^9}, {3.4643495521201077`*^9, 3.464349556405148*^9}, 
   3.4643496413851423`*^9, {3.46436132438649*^9, 3.464361328482037*^9}, {
   3.464361462917412*^9, 3.464361464315563*^9}, 3.464434342113248*^9, {
   3.503014860826033*^9, 3.5030149099909887`*^9}, {3.503014975888389*^9, 
   3.503015010058621*^9}, {3.5030150879029007`*^9, 3.503015089925086*^9}, {
   3.503015141014976*^9, 3.503015141380396*^9}, 3.503015221208963*^9, 
   3.503015960034131*^9, {3.5030162214033937`*^9, 3.5030162237471457`*^9}, 
   3.50301759958219*^9, 3.503025528532357*^9, 3.503094300954267*^9, {
   3.503117718447603*^9, 
   3.503117720972456*^9}},ExpressionUUID->"93890ef2-3cb4-42f1-bb8d-\
c1339a664e00"]
}, Closed]],

Cell[CellGroupData[{

Cell["InstantaneousTransformation", "Subsubsection",
 CellChangeTimes->{{3.689746390253031*^9, 
  3.689746422579988*^9}},ExpressionUUID->"fa72e5e3-df03-4dc3-8dfb-\
d09c59877056"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PropagatorCalculationProcedure", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"_", "?", "SuperoperatorQ"}], ",", "convopts___Rule"}], "]"}], ":=",
    "InstantaneousTransformation"}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.4642547556562862`*^9, 3.464254828687524*^9}, 
   3.464256329437847*^9, {3.46428219093368*^9, 3.4642821996909018`*^9}, {
   3.4643493802686043`*^9, 3.464349382337306*^9}, {3.464349460062024*^9, 
   3.464349499796053*^9}, {3.4643495521201077`*^9, 3.464349556405148*^9}, 
   3.4643496413851423`*^9, {3.46436132438649*^9, 3.464361328482037*^9}, {
   3.464361462917412*^9, 3.464361464315563*^9}, 3.464434342113248*^9, {
   3.503014860826033*^9, 3.5030149099909887`*^9}, {3.503014975888389*^9, 
   3.503015010058621*^9}, {3.5030151335269814`*^9, 3.503015133900193*^9}, 
   3.5030152185850887`*^9, {3.503015937866791*^9, 3.503015945977577*^9}, {
   3.503016186298833*^9, 3.503016211834484*^9}, {3.5030171274628553`*^9, 
   3.5030171478434668`*^9}, 3.503017599682487*^9, 3.503094310480659*^9, {
   3.503096937075532*^9, 3.503096956243425*^9}, {3.534387808873829*^9, 
   3.5343878248910933`*^9}, 
   3.534406140859586*^9},ExpressionUUID->"ec793cd5-6c4e-4d9c-a36d-\
f97d3621ae85"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PropagatorCalculationProcedure", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"_", "?", "SuperoperatorQ"}], ",", "___Rule"}], "}"}], ",", 
     "convopts___Rule"}], "]"}], ":=", "InstantaneousTransformation"}], 
  ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.4642547556562862`*^9, 3.464254828687524*^9}, 
   3.464256329437847*^9, {3.46428219093368*^9, 3.4642821996909018`*^9}, {
   3.4643493802686043`*^9, 3.464349382337306*^9}, {3.464349460062024*^9, 
   3.464349499796053*^9}, {3.4643495521201077`*^9, 3.464349556405148*^9}, 
   3.4643496413851423`*^9, {3.46436132438649*^9, 3.464361328482037*^9}, {
   3.464361462917412*^9, 3.464361464315563*^9}, 3.464434342113248*^9, {
   3.503014860826033*^9, 3.5030149099909887`*^9}, {3.503014975888389*^9, 
   3.503015010058621*^9}, {3.5030151335269814`*^9, 3.503015133900193*^9}, 
   3.5030152185850887`*^9, {3.503015937866791*^9, 3.503015945977577*^9}, {
   3.503016186298833*^9, 3.503016211834484*^9}, {3.5030171274628553`*^9, 
   3.5030171478434668`*^9}, 3.503017599682487*^9, 3.503094310480659*^9, {
   3.503096937075532*^9, 3.503096956243425*^9}, {3.534387808873829*^9, 
   3.5343878248910933`*^9}, 3.534406140859586*^9, {3.690449952806117*^9, 
   3.690449957702407*^9}},ExpressionUUID->"07c03dc9-269e-4170-b9cc-\
8365bc038933"]
}, Closed]],

Cell[CellGroupData[{

Cell["RepeatPropagator", "Subsubsection",
 CellChangeTimes->{{3.689746390253031*^9, 
  3.689746433887287*^9}},ExpressionUUID->"08762a96-338a-4b97-8b08-\
c3b5af552c6e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PropagatorCalculationProcedure", "[", 
    RowBox[{"_Repeat", ",", "convopts___Rule"}], "]"}], ":=", 
   "RepeatPropagator"}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.455331184765587*^9, 3.455331198410469*^9}, {
   3.455346771376102*^9, 3.45534677622994*^9}, {3.455633958587018*^9, 
   3.455633977642692*^9}, {3.455634015720689*^9, 3.4556340225834837`*^9}, {
   3.4556350559149942`*^9, 3.455635057807189*^9}, 3.459343601649426*^9, 
   3.464256332766728*^9, {3.4642822386776457`*^9, 3.464282244497985*^9}, {
   3.464348964076433*^9, 3.464349081886142*^9}, {3.464349507110547*^9, 
   3.464349526868887*^9}, {3.464361080036954*^9, 3.464361089752222*^9}, {
   3.464361162003334*^9, 3.464361188585981*^9}, {3.46436133574755*^9, 
   3.464361383548497*^9}, {3.4643619803961554`*^9, 3.4643619860424967`*^9}, 
   3.464441809698265*^9, 3.4644592953679533`*^9, {3.464974037994814*^9, 
   3.4649740459842653`*^9}, 3.464974212135957*^9, {3.465794178224971*^9, 
   3.465794181894285*^9}, {3.4658039821448317`*^9, 3.465803982838882*^9}, 
   3.465804270824853*^9, {3.465804875144926*^9, 3.46580489796835*^9}, 
   3.465810968168955*^9, {3.503027947844838*^9, 3.503027985185946*^9}, {
   3.50309464024751*^9, 3.503094662217595*^9}, 3.508004991732822*^9, {
   3.508339753169627*^9, 3.508339770755281*^9}, {3.514702699414304*^9, 
   3.514702706963636*^9}, {3.5149079168237*^9, 3.51490794368093*^9}, {
   3.514953478718254*^9, 3.5149534909179*^9}, {3.542213275083579*^9, 
   3.542213295282143*^9}, {3.689850627610112*^9, 3.689850629324389*^9}, 
   3.692848348678534*^9},ExpressionUUID->"3417830e-e20c-4bc5-84bf-\
dff546a63951"]
}, Closed]],

Cell[CellGroupData[{

Cell["UsePrecalculatedPropagator", "Subsubsection",
 CellChangeTimes->{{3.689746390253031*^9, 
  3.689746450049687*^9}},ExpressionUUID->"f925b26c-a9d2-478d-8357-\
5011198392ca"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PropagatorCalculationProcedure", "[", 
    RowBox[{"_PrecalculatedEvent", ",", "convopts___Rule"}], "]"}], ":=", 
   "UsePrecalculatedPropagator"}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.4642547556562862`*^9, 3.464254828687524*^9}, 
   3.464256329437847*^9, {3.46428219093368*^9, 3.4642821996909018`*^9}, {
   3.4643493802686043`*^9, 3.464349382337306*^9}, {3.464349460062024*^9, 
   3.464349499796053*^9}, {3.4643495521201077`*^9, 3.464349556405148*^9}, 
   3.4643496413851423`*^9, {3.46436132438649*^9, 3.464361328482037*^9}, {
   3.464361462917412*^9, 3.464361464315563*^9}, 3.464434342113248*^9, {
   3.503014860826033*^9, 3.5030149099909887`*^9}, {3.503014975888389*^9, 
   3.503015010058621*^9}, {3.5030151335269814`*^9, 3.503015133900193*^9}, 
   3.5030152185850887`*^9, {3.503015937866791*^9, 3.503015945977577*^9}, {
   3.503016186298833*^9, 3.503016211834484*^9}, {3.5030171274628553`*^9, 
   3.5030171478434668`*^9}, 3.503017599682487*^9, 3.503094310480659*^9, {
   3.503096937075532*^9, 3.503096956243425*^9}, {3.534387808873829*^9, 
   3.5343878248910933`*^9}, 3.534406140859586*^9, {3.538921010972638*^9, 
   3.538921019912931*^9}, {3.691437700720686*^9, 
   3.6914377131713257`*^9}},ExpressionUUID->"340c85f5-81ae-4941-80bd-\
a1cfb5c10053"]
}, Closed]],

Cell[CellGroupData[{

Cell["take PropagatorCalculationProcedure from Event options", "Subsubsection",
 CellChangeTimes->{{3.689746390253031*^9, 
  3.6897464914267387`*^9}},ExpressionUUID->"7be7ae03-295a-417b-837a-\
c3b3d22ea948"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PropagatorCalculationProcedure", "[", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Except", "[", "_Rule", "]"}], ".."}], ",", "evopts___Rule"}],
       "]"}], ",", "convopts___Rule"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"PropagatorCalculationProcedure", "/.", 
     RowBox[{"{", "evopts", "}"}]}], "/.", 
    RowBox[{"Options", "[", "Event", "]"}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5437528722597513`*^9, 3.5437529220305*^9}, {
  3.685033554772128*^9, 3.685033554774294*^9}, {3.685035810074874*^9, 
  3.685035810077793*^9}, {3.685036131157929*^9, 
  3.685036131718795*^9}},ExpressionUUID->"5d27a67b-51b3-4750-a0e7-\
b3d2efbbb6f8"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Repeat", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, 
   3.683973033466785*^9, {3.683973750698831*^9, 3.683973753893053*^9}, {
   3.683976432753954*^9, 3.683976436691045*^9}, 3.68397674513378*^9, {
   3.683977981950523*^9, 3.683977988834993*^9}, {3.683978334297957*^9, 
   3.683978335668144*^9}, 3.683979655379014*^9, 3.683980372936699*^9, {
   3.689661184405801*^9, 3.689661197923876*^9}, 3.6915769594929028`*^9, 
   3.692848349391261*^9},ExpressionUUID->"370fbc63-d71c-4f47-9d49-\
7923215d1390"],

Cell[CellGroupData[{

Cell["ExpandRepeatedEvents\[Rule]True|All", "Section",
 CellChangeTimes->{{3.689419140157094*^9, 
  3.689419192186883*^9}},ExpressionUUID->"1207523b-5192-45ad-9eb8-\
cdf42b91e037"],

Cell["implement only if the number of repeats is numeric.", "Text",
 CellChangeTimes->{{3.689419161644956*^9, 3.6894191739701366`*^9}, {
  3.6894216707808523`*^9, 3.689421679155484*^9}, {3.689421721728045*^9, 
  3.689421722629281*^9}},ExpressionUUID->"d75677bc-c7ff-4dc2-af02-\
9f8eea48d48e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"HoldPattern", "@", 
     RowBox[{"Event", "[", 
      RowBox[{
       RowBox[{"evstatus", ":", 
        RowBox[{"{", 
         RowBox[{"False", ",", "numst_"}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"evch", ":", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"timingch", ":", 
           RowBox[{"{", 
            RowBox[{"dur_", ",", "timeptch_", ",", "moretimingch__"}], 
            "}"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"evbackg_", ",", "_", ",", "_"}], "}"}], ",", "hilbch_", 
          ",", "propcalcprocchar_", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"All", "|", "True"}], ",", "True", ",", "False"}], "}"}], 
          ",", "morech___"}], "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"repev", ":", 
        RowBox[{"Repeat", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"evlist", ":", 
           RowBox[{"EventList", "[", 
            RowBox[{"evliststatus_", ",", "evlistchars_", ",", 
             RowBox[{"events", ":", 
              RowBox[{"{", 
               RowBox[{"_Event", ".."}], "}"}]}], ",", "evlistopts___Rule"}], 
            "]"}]}], ",", "\[IndentingNewLine]", "nrep_", ",", 
          "\[IndentingNewLine]", "repevopts___Rule"}], "\[IndentingNewLine]", 
         "]"}]}], ",", "\[IndentingNewLine]", "evopts___Rule"}], 
      "\[IndentingNewLine]", "]"}]}], "/;", 
    RowBox[{
     RowBox[{"NumericQ", "[", "nrep", "]"}], "&&", 
     RowBox[{"(", 
      RowBox[{"nrep", "\[GreaterEqual]", "0"}], ")"}]}]}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"newevents", ",", "firstnewevent", ",", "laternewevents"}], 
      "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Verbose", "/.", 
            RowBox[{"{", "evopts", "}"}]}], "/.", 
           RowBox[{"Options", "[", "Event", "]"}]}], ",", 
          RowBox[{
          "Print", "[", "\"\<implementing complete expansion of Repeat\>\"", 
           "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"newevents", "=", 
         RowBox[{"Flatten", "[", 
          RowBox[{
           RowBox[{"Table", "[", 
            RowBox[{"events", ",", 
             RowBox[{"{", "nrep", "}"}]}], "]"}], ",", "1"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Sequence", "@@", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"nrep", ">", "0"}], ",", "\[IndentingNewLine]", 
           RowBox[{"Join", "[", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{
              RowBox[{"Drop", "[", 
               RowBox[{"newevents", ",", 
                RowBox[{"-", "1"}]}], "]"}], "/.", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"Event", "[", 
                RowBox[{"evstatus$_", ",", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{"dur$_", ",", "moretch$___"}], "}"}], ",", 
                   "backgch$_", ",", "hilbch$_", ",", "pcp$_", ",", 
                   "exprep$_", ",", "precalchk$_", ",", "morech$___"}], "}"}],
                  ",", "evspecs$_", ",", "evopts$___Rule"}], "]"}], 
               "\[RuleDelayed]", 
               RowBox[{"Event", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"False", ",", "False"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{
                  "Undefined", ",", "backgch$", ",", "Undefined", ",", 
                   "Undefined", ",", 
                   RowBox[{"{", 
                    RowBox[{"True", ",", "True", ",", "False"}], "}"}], ",", 
                   "False", ",", "morech$"}], "}"}], ",", "evspecs$", ",", 
                 "evopts$"}], "]"}]}]}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"Last", "[", "newevents", "]"}], "}"}], "/.", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"Event", "[", 
                RowBox[{"evstatus$_", ",", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{"dur$_", ",", "moretch$___"}], "}"}], ",", 
                   "backgch$_", ",", "hilbch$_", ",", "pcp$_", ",", 
                   "exprep$_", ",", "precalchk$_", ",", "morech$___"}], "}"}],
                  ",", "evspecs$_", ",", "evopts$___Rule"}], "]"}], 
               "\[RuleDelayed]", 
               RowBox[{"Event", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"False", ",", "False"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{
                  "Undefined", ",", "backgch$", ",", "Undefined", ",", 
                   "Undefined", ",", 
                   RowBox[{"{", 
                    RowBox[{"True", ",", "True", ",", "False"}], "}"}], ",", 
                   "False", ",", "morech$"}], "}"}], ",", "evspecs$", ",", 
                 "\[IndentingNewLine]", 
                 RowBox[{"SimplifyRules", "[", "\[IndentingNewLine]", 
                  RowBox[{
                   RowBox[{"Switch", "[", 
                    RowBox[{"timeptch", ",", "\[IndentingNewLine]", 
                    RowBox[{"{", 
                    RowBox[{"_", ",", 
                    RowBox[{"Except", "[", 
                    RowBox[{"Undefined", "|", "Automatic"}], "]"}]}], "}"}], 
                    ",", 
                    RowBox[{"InitialTimePoint", "\[Rule]", 
                    RowBox[{"Last", "[", "timeptch", "]"}]}], ",", 
                    "\[IndentingNewLine]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Except", "[", 
                    RowBox[{"Undefined", "|", "Automatic"}], "]"}], ",", 
                    "_"}], "}"}], ",", 
                    RowBox[{"FinalTimePoint", "\[Rule]", 
                    RowBox[{"First", "[", "timeptch", "]"}]}], ",", 
                    "\[IndentingNewLine]", "_", ",", 
                    RowBox[{"InitialTimePoint", "\[Rule]", "Undefined"}]}], 
                    "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
                   "evopts$"}], "\[IndentingNewLine]", "]"}]}], 
                "\[IndentingNewLine]", "]"}]}]}]}], "\[IndentingNewLine]", 
            "]"}], ",", "\[IndentingNewLine]", 
           RowBox[{"{", "}"}]}], "\[IndentingNewLine]", "]"}]}]}], 
       "\[IndentingNewLine]", ")"}], "\[IndentingNewLine]", ")"}]}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.68942117919565*^9, 3.689421187794764*^9}, {
   3.689421230839958*^9, 3.689421287381941*^9}, {3.6894213187902613`*^9, 
   3.689421366433655*^9}, {3.6894214032111187`*^9, 3.689421472596354*^9}, {
   3.689421527315557*^9, 3.6894216477242937`*^9}, {3.689422103976747*^9, 
   3.689422148430278*^9}, {3.689422190669052*^9, 3.6894223257075567`*^9}, {
   3.689422453316206*^9, 3.689422524271903*^9}, {3.68942271638349*^9, 
   3.689422724779826*^9}, {3.68942279419048*^9, 3.689422831229124*^9}, {
   3.689422900711296*^9, 3.689422910412019*^9}, 3.689621891048213*^9, {
   3.689622020771947*^9, 3.689622044728767*^9}, 3.6896247989381123`*^9, {
   3.689661671285231*^9, 3.689661687467443*^9}, {3.689661731938283*^9, 
   3.689661732225088*^9}, {3.689701475851748*^9, 3.689701739154459*^9}, {
   3.6897023827009783`*^9, 3.6897026415134983`*^9}, {3.6897043375738688`*^9, 
   3.689704347122861*^9}, 3.6897054612142363`*^9, {3.690448409986876*^9, 
   3.690448413472303*^9}, {3.6904484456115*^9, 3.690448459095401*^9}, {
   3.6904590270131893`*^9, 3.6904590306498013`*^9}, {3.690459078317637*^9, 
   3.690459079082385*^9}, {3.690465346834559*^9, 3.6904653491695766`*^9}, {
   3.6904653927978477`*^9, 3.6904654212033377`*^9}, {3.690466053924374*^9, 
   3.69046606227334*^9}, {3.6910687338459*^9, 3.691068756509379*^9}, {
   3.691333647704546*^9, 3.6913337959749393`*^9}, {3.6913338791022367`*^9, 
   3.691333957054469*^9}, {3.691333989781826*^9, 3.691334092112392*^9}, {
   3.691334377071766*^9, 3.6913343952068863`*^9}, 3.6913349024185762`*^9, {
   3.691336565519606*^9, 3.691336626796359*^9}, {3.691496661636902*^9, 
   3.691496727168364*^9}, 3.6915002063688*^9, 3.691500268727724*^9, {
   3.691500346492131*^9, 3.691500362905714*^9}, {3.691500527799204*^9, 
   3.691500574665778*^9}, {3.691500631701823*^9, 3.691500657688387*^9}, {
   3.691500731118926*^9, 3.691500734292842*^9}, {3.6915015596403112`*^9, 
   3.691501596717813*^9}, {3.6915038556133623`*^9, 3.691503862762704*^9}, {
   3.691503897611308*^9, 3.691503997878357*^9}, {3.691504695327005*^9, 
   3.691504698722706*^9}, {3.6915078782997227`*^9, 3.6915078817114162`*^9}, {
   3.691507948080916*^9, 3.691507949556925*^9}, {3.691511896011722*^9, 
   3.691511897864045*^9}, {3.691579842858099*^9, 3.691579855439122*^9}, {
   3.691579928742841*^9, 3.691579943579308*^9}, {3.691580988773452*^9, 
   3.6915810295678062`*^9}, {3.691581379201681*^9, 3.691581388706237*^9}, 
   3.691581846674006*^9, 3.6915819593160963`*^9, {3.691583138889166*^9, 
   3.691583142055278*^9}, {3.691583276283039*^9, 3.691583277103498*^9}, {
   3.691583330737282*^9, 3.691583336326208*^9}, {3.691598747118277*^9, 
   3.691598788185611*^9}, {3.691598819251934*^9, 3.6915988205458603`*^9}, 
   3.691676748970252*^9, {3.692848350096483*^9, 
   3.692848350841022*^9}},ExpressionUUID->"00c2c14a-48ca-4262-8a80-\
134b3a12d177"]
}, Closed]],

Cell[CellGroupData[{

Cell["ExpandRepeatedEvents\[Rule]Automatic", "Section",
 CellChangeTimes->{{3.689419140157094*^9, 
  3.689419202185822*^9}},ExpressionUUID->"e6fc4603-d9fd-4eeb-8ea5-\
3d460bdc68ab"],

Cell["\<\
determine expansion format from PermittedTimeShift of enclosed EventList\
\>", "Text",
 CellChangeTimes->{{3.689419161644956*^9, 3.6894191739701366`*^9}, {
  3.689419205366111*^9, 3.689419235417996*^9}, {3.6897478120653677`*^9, 
  3.6897478358385677`*^9}},ExpressionUUID->"fed9e336-9baa-40c6-9085-\
5f62df62f75e"],

Cell[CellGroupData[{

Cell["force full expansion if PermittedTimeShift \[Equal] None", \
"Subsubsection",
 CellChangeTimes->{{3.6897480149202547`*^9, 
  3.6897480448151693`*^9}},ExpressionUUID->"0c75a42b-bee9-4559-b126-\
0cea0abea6ce"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"HoldPattern", "@", 
     RowBox[{"Event", "[", 
      RowBox[{
       RowBox[{"evstatus", ":", 
        RowBox[{"{", 
         RowBox[{"False", ",", "numst_"}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
        "timingch_", ",", "backgch_", ",", "hilbch_", ",", "pcpch_", ",", 
         RowBox[{"{", 
          RowBox[{"Automatic", ",", "True", ",", "False"}], "}"}], ",", 
         "morech___"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"repevs", ":", 
        RowBox[{"Repeat", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"evlist", ":", 
           RowBox[{"EventList", "[", 
            RowBox[{"evliststatus_", ",", "evlistchars_", ",", 
             RowBox[{"events", ":", 
              RowBox[{"{", 
               RowBox[{"_Event", ".."}], "}"}]}], ",", "evlistopts___Rule"}], 
            "]"}]}], ",", "\[IndentingNewLine]", "nrep_", ",", 
          "\[IndentingNewLine]", "repevopts___Rule"}], "\[IndentingNewLine]", 
         "]"}]}], ",", "\[IndentingNewLine]", "evopts___Rule"}], 
      "\[IndentingNewLine]", "]"}]}], "/;", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"PermittedTimeShift", "[", "evlist", "]"}], "\[Equal]", 
      "None"}], ")"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Verbose", "/.", 
           RowBox[{"{", "evopts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "Event", "]"}]}], ",", 
         RowBox[{
         "Print", "[", "\"\<case of PermittedTimeShift \[Equal] None\>\"", 
          "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"Event", "[", 
        RowBox[{"evstatus", ",", 
         RowBox[{"{", 
          RowBox[{
          "timingch", ",", "backgch", ",", "hilbch", ",", "pcpch", ",", 
           RowBox[{"{", 
            RowBox[{"True", ",", "True", ",", "False"}], "}"}], ",", 
           "morech"}], "}"}], ",", "repevs", ",", "evopts"}], "]"}]}], 
      "\[IndentingNewLine]", ")"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6897478565098543`*^9, 3.689748001071788*^9}, {
   3.689748051218623*^9, 3.689748051999359*^9}, {3.6898471973647013`*^9, 
   3.689847225401146*^9}, {3.6904484995313787`*^9, 3.690448525241886*^9}, {
   3.691496744456987*^9, 3.69149675503964*^9}, {3.69151191980167*^9, 
   3.6915119206712303`*^9}, 3.691583354569643*^9, 3.69167675792974*^9, 
   3.6928483515061197`*^9},ExpressionUUID->"39b8110a-d28c-4c64-8c7d-\
97f7c98c39a4"]
}, Closed]],

Cell[CellGroupData[{

Cell["no expansion acceptable if PermittedTimeShift \[Equal] All", \
"Subsubsection",
 CellChangeTimes->{{3.6897480149202547`*^9, 3.689748072637719*^9}, 
   3.690547338501878*^9, {3.6913342992066383`*^9, 
   3.69133430086924*^9}},ExpressionUUID->"63724d28-e734-472a-b41a-\
53220d709156"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"HoldPattern", "@", 
     RowBox[{"Event", "[", 
      RowBox[{
       RowBox[{"evstatus", ":", 
        RowBox[{"{", 
         RowBox[{"False", ",", "numst_"}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
        "timingch_", ",", "backgch_", ",", "hilbch_", ",", "pcpch_", ",", 
         RowBox[{"{", 
          RowBox[{"Automatic", ",", "True", ",", "False"}], "}"}], ",", 
         "morech___"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"repevs", ":", 
        RowBox[{"Repeat", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"evlist", ":", 
           RowBox[{"EventList", "[", 
            RowBox[{"evliststatus_", ",", "evlistchars_", ",", 
             RowBox[{"events", ":", 
              RowBox[{"{", 
               RowBox[{"_Event", ".."}], "}"}]}], ",", "evlistopts___Rule"}], 
            "]"}]}], ",", "\[IndentingNewLine]", "nrep_", ",", 
          "\[IndentingNewLine]", "repevopts___Rule"}], "\[IndentingNewLine]", 
         "]"}]}], ",", "\[IndentingNewLine]", "evopts___Rule"}], 
      "\[IndentingNewLine]", "]"}]}], "/;", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"PermittedTimeShift", "[", "evlist", "]"}], "\[Equal]", "All"}],
      ")"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Verbose", "/.", 
           RowBox[{"{", "evopts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "Event", "]"}]}], ",", 
         RowBox[{
         "Print", "[", "\"\<case of PermittedTimeShift \[Equal] All\>\"", 
          "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"Event", "[", 
        RowBox[{"evstatus", ",", 
         RowBox[{"{", 
          RowBox[{
          "timingch", ",", "backgch", ",", "hilbch", ",", "pcpch", ",", 
           RowBox[{"{", 
            RowBox[{"False", ",", "True", ",", "False"}], "}"}], ",", 
           "morech"}], "}"}], ",", "repevs", ",", "evopts"}], "]"}]}], 
      "\[IndentingNewLine]", ")"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6897478565098543`*^9, 3.689748001071788*^9}, {
   3.689748051218623*^9, 3.6897480926380053`*^9}, {3.689847236242959*^9, 
   3.689847245592564*^9}, {3.6904485364222183`*^9, 3.69044854581845*^9}, {
   3.6914967654799223`*^9, 3.691496772582409*^9}, {3.691511928953185*^9, 
   3.69151192966199*^9}, 3.691583362142991*^9, 3.691676764719883*^9, 
   3.692848352205337*^9},ExpressionUUID->"a8df3c22-714c-446c-8739-\
0cbaa146e851"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
no expansion acceptable if duration of repeating event is an integer multiple \
of PermittedTimeShift\
\>", "Subsubsection",
 CellChangeTimes->{{3.6897480149202547`*^9, 3.689748072637719*^9}, {
   3.689748126865788*^9, 3.689748169667617*^9}, 3.689748610345995*^9, {
   3.6897486906217833`*^9, 3.689748694666814*^9}, 3.689748802964365*^9, {
   3.689840669073797*^9, 3.689840698618966*^9}, 3.6905473334537497`*^9, {
   3.691334303898079*^9, 
   3.691334305684326*^9}},ExpressionUUID->"1926c7e6-73b5-454e-b994-\
d63a3bd113e8"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"evstatus", ":", 
       RowBox[{"{", 
        RowBox[{"False", ",", "numst_"}], "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "timingch_", ",", "backgch_", ",", "hilbch_", ",", "pcpch_", ",", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "True", ",", "False"}], "}"}], ",", 
        "morech___"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"repevs", ":", 
       RowBox[{"Repeat", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"evlist", ":", 
          RowBox[{"EventList", "[", "\[IndentingNewLine]", 
           RowBox[{"evliststatus_", ",", "\[IndentingNewLine]", 
            RowBox[{"evlistchars", ":", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"T_", ",", "_", ",", "pts_", ",", "__"}], "}"}], ",", 
               
               RowBox[{"{", 
                RowBox[{"__", ",", "True"}], "}"}], ",", "___"}], "}"}]}], 
            ",", "\[IndentingNewLine]", 
            RowBox[{"events", ":", 
             RowBox[{"{", 
              RowBox[{"_Event", ".."}], "}"}]}], ",", "\[IndentingNewLine]", 
            "evlistopts___Rule"}], "]"}]}], ",", "\[IndentingNewLine]", 
         "nrep_", ",", "\[IndentingNewLine]", "repevopts___Rule"}], 
        "\[IndentingNewLine]", "]"}]}], ",", "\[IndentingNewLine]", 
      "evopts___Rule"}], "\[IndentingNewLine]", "]"}]}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Nperiod", ",", "nperiod"}], "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Verbose", "/.", 
            RowBox[{"{", "evopts", "}"}]}], "/.", 
           RowBox[{"Options", "[", "Event", "]"}]}], ",", 
          RowBox[{
          "Print", "[", "\"\<implementing no repeat event expansion\>\"", 
           "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Event", "[", 
         RowBox[{"evstatus", ",", 
          RowBox[{"{", 
           RowBox[{
           "timingch", ",", "backgch", ",", "hilbch", ",", "pcpch", ",", 
            RowBox[{"{", 
             RowBox[{"False", ",", "True", ",", "False"}], "}"}], ",", 
            "morech"}], "}"}], ",", "repevs", ",", "evopts"}], "]"}]}], 
       "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Nperiod", "=", 
         RowBox[{"T", "/", "pts"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"nperiod", "=", 
         RowBox[{"Round", "[", "Nperiod", "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"TrueQ", "[", 
          RowBox[{
           RowBox[{"Chop", "@", 
            RowBox[{"N", "[", 
             RowBox[{"nperiod", "-", "Nperiod"}], "]"}]}], "\[Equal]", "0"}], 
          "]"}], "&&", "\[IndentingNewLine]", 
         RowBox[{"nperiod", "\[GreaterEqual]", "1"}]}]}], 
       "\[IndentingNewLine]", ")"}]}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6897478565098543`*^9, 3.689748001071788*^9}, {
   3.689748051218623*^9, 3.6897480926380053`*^9}, {3.689748195096923*^9, 
   3.6897483545314817`*^9}, {3.689748633277191*^9, 3.689748652149128*^9}, 
   3.6897487094542522`*^9, {3.689748824086748*^9, 3.6897488976091957`*^9}, {
   3.689749064172164*^9, 3.689749067228381*^9}, {3.689749110748271*^9, 
   3.689749149201342*^9}, {3.689840867237363*^9, 3.689840885259111*^9}, {
   3.6898410644649563`*^9, 3.689841163611877*^9}, {3.689841205786457*^9, 
   3.689841207265758*^9}, {3.689841237974009*^9, 3.6898412555296707`*^9}, {
   3.689856663682836*^9, 3.689856706176162*^9}, {3.689916163218072*^9, 
   3.689916173425795*^9}, 3.6903918717962637`*^9, {3.690448556275576*^9, 
   3.690448558714767*^9}, {3.6914967790630836`*^9, 3.691496790430073*^9}, {
   3.691511937601761*^9, 3.691511939000155*^9}, 3.691583368111062*^9, 
   3.691676769665537*^9, 
   3.69284835286311*^9},ExpressionUUID->"53c8b6e5-6c98-4907-ab99-\
06a9419f1408"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
implement partial expansion if PermittedTimeShift is an integer multiple of \
duration\
\>", "Subsubsection",
 CellChangeTimes->{{3.6897480149202547`*^9, 3.689748072637719*^9}, {
  3.689748126865788*^9, 3.689748169667617*^9}, {3.6897483750043497`*^9, 
  3.6897484117057037`*^9}, {3.6897489279108753`*^9, 3.689748933789508*^9}, {
  3.690448057280118*^9, 3.690448057732835*^9}, {3.691064411670573*^9, 
  3.691064412932016*^9}},ExpressionUUID->"130070a5-ccbf-49a1-a25c-\
a1bcb22b04e9"],

Cell["\<\
ensure time point information is transmitted, if it has already been \
implemented\
\>", "Text",
 CellChangeTimes->{{3.690395783412442*^9, 
  3.690395800540429*^9}},ExpressionUUID->"8baa1a55-441f-46d4-9ff7-\
f6b21a0ec6bc"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"evstatus", ":", 
       RowBox[{"{", 
        RowBox[{"False", ",", "numst_"}], "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"timingch", ":", 
         RowBox[{"{", 
          RowBox[{"dur_", ",", "timeptch_", ",", "moretimingch__"}], "}"}]}], 
        ",", "backgch_", ",", "hilbch_", ",", "pcpch_", ",", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "True", ",", "False"}], "}"}], ",", 
        "morech___"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"repevs", ":", 
       RowBox[{"Repeat", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"evlist", ":", 
          RowBox[{"EventList", "[", "\[IndentingNewLine]", 
           RowBox[{"evliststatus_", ",", "\[IndentingNewLine]", 
            RowBox[{"evlistchars", ":", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"evlisttchars", ":", 
                RowBox[{"{", 
                 RowBox[{"T_", ",", "_", ",", "pts_", ",", "__"}], "}"}]}], 
               ",", 
               RowBox[{"evlistbckgch", ":", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"evbackg", ":", 
                   RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "_", ",",
                   "True"}], "}"}]}], ",", "moreevlistchars___"}], "}"}]}], 
            ",", "\[IndentingNewLine]", 
            RowBox[{"events", ":", 
             RowBox[{"{", 
              RowBox[{"_Event", ".."}], "}"}]}], ",", "\[IndentingNewLine]", 
            "evlistopts___Rule"}], "]"}]}], ",", "\[IndentingNewLine]", 
         "nrep_", ",", "\[IndentingNewLine]", "repevopts___Rule"}], 
        "\[IndentingNewLine]", "]"}]}], ",", "\[IndentingNewLine]", 
      "evopts___Rule"}], "\[IndentingNewLine]", "]"}]}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "nrepinperiod", ",", "nperiod", ",", "nrem", ",", "remevents", ",", 
       "expandedevents"}], "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"remevents", "=", 
         RowBox[{"Flatten", "[", 
          RowBox[{
           RowBox[{"Table", "[", 
            RowBox[{"events", ",", 
             RowBox[{"{", "nrem", "}"}]}], "]"}], ",", "1"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"expandedevents", "=", 
         RowBox[{"Join", "[", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"nrem", ">", "0"}], ",", "\[IndentingNewLine]", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                RowBox[{"First", "[", "remevents", "]"}], "/.", 
                "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"Event", "[", 
                  RowBox[{"evstatus$_", ",", 
                   RowBox[{"{", 
                    RowBox[{
                    "tch$_", ",", "backgch$_", ",", "hilbch$_", ",", "pcp$_", 
                    ",", "exprep$_", ",", "precalchk$_", ",", "morech$___"}], 
                    "}"}], ",", "evspecs$_", ",", "evopts$___Rule"}], "]"}], 
                 "\[RuleDelayed]", "\[IndentingNewLine]", 
                 RowBox[{"Event", "[", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{"False", ",", "False"}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{
                    "Undefined", ",", "backgch$", ",", "Undefined", ",", 
                    "Undefined", ",", 
                    RowBox[{"{", 
                    RowBox[{"True", ",", "True", ",", "False"}], "}"}], ",", 
                    "False", ",", "morech$"}], "}"}], ",", "evspecs$", ",", 
                   "\[IndentingNewLine]", 
                   RowBox[{"SimplifyRules", "[", "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"Switch", "[", 
                    RowBox[{"timeptch", ",", "\[IndentingNewLine]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Except", "[", "Undefined", "]"}], ",", "_"}], 
                    "}"}], ",", 
                    RowBox[{"FinalTimePoint", "\[Rule]", 
                    RowBox[{"First", "[", "timeptch", "]"}]}], ",", 
                    "\[IndentingNewLine]", "_", ",", 
                    RowBox[{"FinalTimePoint", "\[Rule]", "Undefined"}]}], 
                    "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
                    "evopts$"}], "\[IndentingNewLine]", "]"}]}], "]"}]}]}], 
               ",", "\[IndentingNewLine]", 
               RowBox[{"Sequence", "@@", 
                RowBox[{"(", "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"Drop", "[", 
                   RowBox[{"remevents", ",", "1"}], "]"}], "/.", 
                  "\[IndentingNewLine]", 
                  RowBox[{
                   RowBox[{"Event", "[", 
                    RowBox[{"evstatus$_", ",", 
                    RowBox[{"{", 
                    RowBox[{
                    "tch$_", ",", "backgch$_", ",", "hilbch$_", ",", "pcp$_", 
                    ",", "exprep$_", ",", "precalchk$_", ",", "morech$___"}], 
                    "}"}], ",", "evspecs$_", ",", "evopts$___Rule"}], "]"}], 
                   "\[RuleDelayed]", "\[IndentingNewLine]", 
                   RowBox[{"Event", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"False", ",", "False"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{
                    "Undefined", ",", "backgch$", ",", "Undefined", ",", 
                    "Undefined", ",", 
                    RowBox[{"{", 
                    RowBox[{"True", ",", "True", ",", "False"}], "}"}], ",", 
                    "False", ",", "morech$"}], "}"}], ",", "evspecs$", ",", 
                    "\[IndentingNewLine]", 
                    RowBox[{"SimplifyRules", "[", "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"Switch", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"timeptch", ",", "nperiod"}], "}"}], ",", 
                    "\[IndentingNewLine]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"_", ",", 
                    RowBox[{"Except", "[", "Undefined", "]"}]}], "}"}], ",", 
                    "0"}], "}"}], ",", 
                    RowBox[{"InitialTimePoint", "\[Rule]", 
                    RowBox[{"Last", "[", "timeptch", "]"}]}], ",", 
                    "\[IndentingNewLine]", "_", ",", 
                    RowBox[{"InitialTimePoint", "\[Rule]", "Undefined"}]}], 
                    "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
                    "evopts$"}], "\[IndentingNewLine]", "]"}]}], 
                    "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
                 ")"}]}]}], "\[IndentingNewLine]", "}"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"{", "}"}]}], "\[IndentingNewLine]", "]"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"nperiod", ">", "0"}], ",", "\[IndentingNewLine]", 
             RowBox[{"{", 
              RowBox[{"Event", "[", "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"False", ",", "False"}], "}"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"{", 
                 RowBox[{
                 "Undefined", ",", "backgch", ",", "Undefined", ",", 
                  "Undefined", ",", 
                  RowBox[{"{", 
                   RowBox[{"Automatic", ",", "True", ",", "True"}], "}"}], 
                  ",", "morech"}], "}"}], ",", "\[IndentingNewLine]", 
                RowBox[{"Repeat", "[", "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"EventList", "[", 
                   RowBox[{
                    RowBox[{"{", 
                    RowBox[{"False", ",", "False"}], "}"}], ",", 
                    "\[IndentingNewLine]", 
                    RowBox[{"{", 
                    RowBox[{
                    "Undefined", ",", "evlistbckgch", ",", 
                    "moreevlistchars"}], "}"}], ",", "\[IndentingNewLine]", 
                    RowBox[{"Flatten", "[", 
                    RowBox[{
                    RowBox[{"Table", "[", "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"events", "/.", 
                    RowBox[{
                    RowBox[{"Event", "[", 
                    RowBox[{"evstatus$_", ",", 
                    RowBox[{"{", 
                    RowBox[{
                    "tch$_", ",", "backgch$_", ",", "hilbch$_", ",", "pcp$_", 
                    ",", "exprep$_", ",", "precalchk$_", ",", "morech$___"}], 
                    "}"}], ",", "evspecs$_", ",", "evopts$___Rule"}], "]"}], 
                    "\[RuleDelayed]", "\[IndentingNewLine]", 
                    RowBox[{"Event", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"False", ",", "False"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{
                    "Undefined", ",", "backgch$", ",", "Undefined", ",", 
                    "Undefined", ",", 
                    RowBox[{"{", 
                    RowBox[{"True", ",", "True", ",", "False"}], "}"}], ",", 
                    "False", ",", "morech$"}], "}"}], ",", "evspecs$", ",", 
                    "evopts$"}], "]"}]}]}], ",", "\[IndentingNewLine]", 
                    RowBox[{"{", "nrepinperiod", "}"}]}], "]"}], ",", 
                    "\[IndentingNewLine]", "1"}], "]"}], ",", 
                    "\[IndentingNewLine]", "evlistopts"}], 
                   "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
                  "nperiod", ",", "\[IndentingNewLine]", "repevopts"}], 
                 "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
                RowBox[{"SimplifyRules", "[", "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"ReleaseHold", "@", 
                   RowBox[{"Switch", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nrem", ",", "timeptch"}], "}"}], ",", 
                    "\[IndentingNewLine]", 
                    RowBox[{"{", 
                    RowBox[{"_", ",", 
                    RowBox[{"{", 
                    RowBox[{"_", ",", 
                    RowBox[{"Except", "[", "Undefined", "]"}]}], "}"}]}], 
                    "}"}], ",", 
                    RowBox[{"InitialTimePoint", "\[Rule]", 
                    RowBox[{"Last", "[", "timeptch", "]"}]}], ",", 
                    "\[IndentingNewLine]", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Except", "[", "Undefined", "]"}], ",", "_"}], 
                    "}"}]}], "}"}], ",", 
                    RowBox[{"FinalTimePoint", "\[Rule]", 
                    RowBox[{"First", "[", "timeptch", "]"}]}], ",", 
                    "\[IndentingNewLine]", "_", ",", 
                    RowBox[{"Hold", "[", 
                    RowBox[{
                    RowBox[{"FinalTimePoint", "\[Rule]", "Undefined"}], ",", 
                    RowBox[{"InitialTimePoint", "\[Rule]", "Undefined"}]}], 
                    "]"}]}], "\[IndentingNewLine]", "]"}]}], ",", 
                  "\[IndentingNewLine]", "evopts"}], "\[IndentingNewLine]", 
                 "]"}]}], "\[IndentingNewLine]", "]"}], "}"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"{", "}"}]}], "\[IndentingNewLine]", "]"}]}], 
          "\[IndentingNewLine]", "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Verbose", "/.", 
            RowBox[{"{", "evopts", "}"}]}], "/.", 
           RowBox[{"Options", "[", "Event", "]"}]}], ",", 
          RowBox[{
           RowBox[{"Print", "[", 
            RowBox[{"\"\<partial repeat event expansion: \>\"", ",", 
             RowBox[{"{", 
              RowBox[{"nrepinperiod", ",", "nperiod", ",", "nrem"}], "}"}]}], 
            "]"}], ";", "\[IndentingNewLine]", 
           RowBox[{"Print", "[", 
            RowBox[{"\"\<leading to : \>\"", ",", 
             RowBox[{"expandedevents", "/.", 
              RowBox[{"s_ShapeFunction", "\[RuleDelayed]", 
               RowBox[{"s", "\[LeftDoubleBracket]", 
                RowBox[{"{", "4", "}"}], "\[RightDoubleBracket]"}]}]}]}], 
            "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Sequence", "@@", "expandedevents"}]}], "\[IndentingNewLine]",
        ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"nrepinperiod", "=", 
         RowBox[{"Numerator", "[", 
          RowBox[{"pts", "/", "T"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"nperiod", "=", 
         RowBox[{"Floor", "[", 
          RowBox[{"nrep", "/", "nrepinperiod"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"nrem", "=", 
         RowBox[{"nrep", "-", 
          RowBox[{"nrepinperiod", "*", "nperiod"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"IntegerQ", "[", "nrepinperiod", "]"}], "&&", 
         RowBox[{"NumericQ", "[", "pts", "]"}], "&&", 
         RowBox[{"NumericQ", "[", "T", "]"}]}]}], "\[IndentingNewLine]", 
       ")"}]}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6897478565098543`*^9, 3.689748001071788*^9}, {
   3.689748051218623*^9, 3.6897480926380053`*^9}, {3.689748195096923*^9, 
   3.6897483545314817`*^9}, {3.6897484462494087`*^9, 3.6897484776038*^9}, {
   3.689748520516367*^9, 3.689748532467853*^9}, {3.6897489965259037`*^9, 
   3.68974907797939*^9}, {3.6897491581075373`*^9, 3.6897492694695787`*^9}, {
   3.689749341457389*^9, 3.689749428074143*^9}, {3.689749473315784*^9, 
   3.689749734380773*^9}, {3.689841371226198*^9, 3.68984141510308*^9}, {
   3.6898414623152*^9, 3.689841527948176*^9}, {3.6898418073936157`*^9, 
   3.689841857006259*^9}, {3.689842159692046*^9, 3.689842187296852*^9}, 
   3.689850048381095*^9, {3.689850286035409*^9, 3.689850297825889*^9}, {
   3.689850365329854*^9, 3.689850380380549*^9}, 3.689850413760113*^9, 
   3.689856454902717*^9, {3.689916659416607*^9, 3.689916751607696*^9}, {
   3.6899173535125504`*^9, 3.689917355867622*^9}, {3.690395241039772*^9, 
   3.690395241732285*^9}, {3.690395293079421*^9, 3.690395314452524*^9}, {
   3.690395407359589*^9, 3.6903954341575127`*^9}, {3.690395473840912*^9, 
   3.690395772969303*^9}, 3.690395879589654*^9, {3.6903959133951273`*^9, 
   3.690395917887487*^9}, {3.690396120740913*^9, 3.6903961218251743`*^9}, {
   3.6903963080389*^9, 3.690396309459158*^9}, 3.690396453782943*^9, {
   3.6903965807321033`*^9, 3.690396683742571*^9}, {3.6903969186549892`*^9, 
   3.690396940660364*^9}, 3.690397020338821*^9, {3.690440785213443*^9, 
   3.690441061729273*^9}, {3.690441275779832*^9, 3.690441291265567*^9}, 
   3.6904415490531883`*^9, {3.690441833723041*^9, 3.690441850372797*^9}, {
   3.6904419448974447`*^9, 3.690441946198317*^9}, {3.69044212517318*^9, 
   3.690442126425446*^9}, {3.690448584709529*^9, 3.6904485986766233`*^9}, 
   3.690466522379588*^9, {3.690467170740814*^9, 3.690467176645412*^9}, 
   3.690467209081122*^9, {3.690467514687327*^9, 3.690467556188259*^9}, {
   3.690542735960143*^9, 3.690542764715757*^9}, {3.690542930273425*^9, 
   3.690542946471183*^9}, 3.691064384953703*^9, {3.691064657965671*^9, 
   3.691064658252584*^9}, {3.6913344244430532`*^9, 3.691334425016168*^9}, {
   3.6913344714578257`*^9, 3.6913345523494053`*^9}, {3.691334611645856*^9, 
   3.691334642515545*^9}, {3.691334724036919*^9, 3.6913347395743856`*^9}, {
   3.691334769800632*^9, 3.691334776326047*^9}, {3.691334836322906*^9, 
   3.691334871591434*^9}, {3.691334912692683*^9, 3.6913349247848988`*^9}, {
   3.691334963035253*^9, 3.6913351276459217`*^9}, {3.691335298449573*^9, 
   3.691335314039068*^9}, {3.691335363902215*^9, 3.6913355155744543`*^9}, 
   3.691335564111999*^9, {3.691335621399888*^9, 3.69133562885395*^9}, {
   3.691335659894861*^9, 3.691335695705306*^9}, {3.691335765218207*^9, 
   3.691335824663499*^9}, {3.691335855636895*^9, 3.691335863748268*^9}, {
   3.691335959748569*^9, 3.691336115097887*^9}, {3.69133618405916*^9, 
   3.6913362016987*^9}, {3.691336259988962*^9, 3.691336274813877*^9}, {
   3.691336319018659*^9, 3.6913363198592854`*^9}, {3.691336388308077*^9, 
   3.691336399813286*^9}, {3.691336796767662*^9, 3.6913368058660583`*^9}, {
   3.6914967979587383`*^9, 3.6914968600594683`*^9}, {3.691496899836255*^9, 
   3.6914969003388453`*^9}, {3.691511949863233*^9, 3.691511950578478*^9}, {
   3.691582091647747*^9, 3.691582150325893*^9}, {3.6915823503183937`*^9, 
   3.6915824302246227`*^9}, {3.691582494640049*^9, 3.691582502146049*^9}, {
   3.691582549885892*^9, 3.691582558979352*^9}, {3.691582999622945*^9, 
   3.691583079728554*^9}, {3.691583112835538*^9, 3.6915831157454042`*^9}, 
   3.6915833741291943`*^9, {3.691598843557107*^9, 3.691598873663993*^9}, 
   3.691676779802663*^9, {3.692848353465947*^9, 3.6928483540989447`*^9}, {
   3.699204262331802*^9, 3.699204360624484*^9}, 3.699204427095539*^9, {
   3.699204496303486*^9, 3.699204531187188*^9}, {3.699206608001202*^9, 
   3.69920663599612*^9}, {3.6992067831172953`*^9, 3.6992067873487167`*^9}, {
   3.699207547042573*^9, 3.69920755447007*^9}, {3.699208601260343*^9, 
   3.699208624442059*^9}},ExpressionUUID->"aeeaa39b-80d2-4797-9d5e-\
2cd8dfe3adec"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
force full expansion if PermittedTimeShift is not an integer multiple of \
duration\
\>", "Subsubsection",
 CellChangeTimes->{{3.6897480149202547`*^9, 3.689748072637719*^9}, {
  3.689748126865788*^9, 3.689748169667617*^9}, {3.6897483750043497`*^9, 
  3.6897484117057037`*^9}, {3.6897489279108753`*^9, 3.689748933789508*^9}, {
  3.690448057280118*^9, 3.690448057732835*^9}, {3.691064411670573*^9, 
  3.691064445676083*^9}},ExpressionUUID->"e28600a3-1eaa-4aaa-8794-\
445c01c069d0"],

Cell["\<\
ensure time point information is transmitted, if it has already been \
implemented\
\>", "Text",
 CellChangeTimes->{{3.690395783412442*^9, 
  3.690395800540429*^9}},ExpressionUUID->"f9787b5f-5225-495b-9a02-\
9269d7a1e709"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"evstatus", ":", 
       RowBox[{"{", 
        RowBox[{"False", ",", "numst_"}], "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"ch", ":", 
       RowBox[{"{", 
        RowBox[{
        "timingch_", ",", "backgch_", ",", "hilbch_", ",", "pcpch_", ",", 
         RowBox[{"{", 
          RowBox[{"Automatic", ",", "True", ",", "False"}], "}"}], ",", 
         "precalchk_", ",", "morech___"}], "}"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"repevs", ":", 
       RowBox[{"Repeat", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"evlist", ":", 
          RowBox[{"EventList", "[", "\[IndentingNewLine]", 
           RowBox[{"evliststatus_", ",", "\[IndentingNewLine]", 
            RowBox[{"evlistchars", ":", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"evlisttchars", ":", 
                RowBox[{"{", 
                 RowBox[{"T_", ",", "_", ",", "pts_", ",", "__"}], "}"}]}], 
               ",", 
               RowBox[{"evlistbckgch", ":", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"evbackg", ":", 
                   RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "_", ",",
                   "True"}], "}"}]}], ",", "evlisthilbch_", ",", "_"}], 
              "}"}]}], ",", "\[IndentingNewLine]", 
            RowBox[{"events", ":", 
             RowBox[{"{", 
              RowBox[{"_Event", ".."}], "}"}]}], ",", "\[IndentingNewLine]", 
            "evlistopts___Rule"}], "]"}]}], ",", "\[IndentingNewLine]", 
         "nrep_", ",", "\[IndentingNewLine]", "repevopts___Rule"}], 
        "\[IndentingNewLine]", "]"}]}], ",", "\[IndentingNewLine]", 
      "evopts___Rule"}], "\[IndentingNewLine]", "]"}]}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"nrepinperiod", ",", "nperiod", ",", "nrem"}], "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Verbose", "/.", 
            RowBox[{"{", "evopts", "}"}]}], "/.", 
           RowBox[{"Options", "[", "Event", "]"}]}], ",", 
          RowBox[{"Print", "[", 
           RowBox[{"\"\<full repeat event expansion: \>\"", ",", 
            RowBox[{"{", 
             RowBox[{"nrepinperiod", ",", "nperiod", ",", "nrem"}], "}"}]}], 
           "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Event", "[", 
         RowBox[{"evstatus", ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
           "timingch", ",", "backgch", ",", "hilbch", ",", "pcpch", ",", 
            RowBox[{"{", 
             RowBox[{"All", ",", "True", ",", "False"}], "}"}], ",", "False", 
            ",", "morech"}], "}"}], ",", "\[IndentingNewLine]", "repevs", ",",
           "evopts"}], "\[IndentingNewLine]", "]"}]}], ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"nrepinperiod", "=", 
         RowBox[{"Numerator", "[", 
          RowBox[{"pts", "/", "T"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"nperiod", "=", 
         RowBox[{"Floor", "[", 
          RowBox[{"nrep", "/", "nrepinperiod"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"nrem", "=", 
         RowBox[{"nrep", "-", 
          RowBox[{"nrepinperiod", "*", "nperiod"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"!", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"IntegerQ", "[", "nrepinperiod", "]"}], "&&", 
           RowBox[{"NumericQ", "[", "pts", "]"}], "&&", 
           RowBox[{"NumericQ", "[", "T", "]"}]}], ")"}]}]}], 
       "\[IndentingNewLine]", ")"}]}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6897478565098543`*^9, 3.689748001071788*^9}, {
   3.689748051218623*^9, 3.6897480926380053`*^9}, {3.689748195096923*^9, 
   3.6897483545314817`*^9}, {3.6897484462494087`*^9, 3.6897484776038*^9}, {
   3.689748520516367*^9, 3.689748532467853*^9}, {3.6897489965259037`*^9, 
   3.68974907797939*^9}, {3.6897491581075373`*^9, 3.6897492694695787`*^9}, {
   3.689749341457389*^9, 3.689749428074143*^9}, {3.689749473315784*^9, 
   3.689749734380773*^9}, {3.689841371226198*^9, 3.68984141510308*^9}, {
   3.6898414623152*^9, 3.689841527948176*^9}, {3.6898418073936157`*^9, 
   3.689841857006259*^9}, {3.689842159692046*^9, 3.689842187296852*^9}, 
   3.689850048381095*^9, {3.689850286035409*^9, 3.689850297825889*^9}, {
   3.689850365329854*^9, 3.689850380380549*^9}, 3.689850413760113*^9, 
   3.689856454902717*^9, {3.689916659416607*^9, 3.689916751607696*^9}, {
   3.6899173535125504`*^9, 3.689917355867622*^9}, {3.690395241039772*^9, 
   3.690395241732285*^9}, {3.690395293079421*^9, 3.690395314452524*^9}, {
   3.690395407359589*^9, 3.6903954341575127`*^9}, {3.690395473840912*^9, 
   3.690395772969303*^9}, 3.690395879589654*^9, {3.6903959133951273`*^9, 
   3.690395917887487*^9}, {3.690396120740913*^9, 3.6903961218251743`*^9}, {
   3.6903963080389*^9, 3.690396309459158*^9}, 3.690396453782943*^9, {
   3.6903965807321033`*^9, 3.690396683742571*^9}, {3.6903969186549892`*^9, 
   3.690396940660364*^9}, 3.690397020338821*^9, {3.690440785213443*^9, 
   3.690441061729273*^9}, {3.690441275779832*^9, 3.690441291265567*^9}, 
   3.6904415490531883`*^9, {3.690441833723041*^9, 3.690441850372797*^9}, {
   3.6904419448974447`*^9, 3.690441946198317*^9}, {3.69044212517318*^9, 
   3.690442126425446*^9}, {3.690448584709529*^9, 3.6904485986766233`*^9}, 
   3.690466522379588*^9, {3.690467170740814*^9, 3.690467176645412*^9}, 
   3.690467209081122*^9, {3.690467514687327*^9, 3.690467556188259*^9}, {
   3.690542735960143*^9, 3.690542764715757*^9}, {3.690542930273425*^9, 
   3.690542946471183*^9}, 3.691064384953703*^9, {3.69106446204704*^9, 
   3.6910646429490356`*^9}, 3.691067281914358*^9, {3.691067331770721*^9, 
   3.691067333854403*^9}, 3.69123935013673*^9, {3.691239476957807*^9, 
   3.69123960801928*^9}, 3.6912405122201653`*^9, {3.691240641284062*^9, 
   3.69124067396318*^9}, 3.69124073731551*^9, {3.6912410525662737`*^9, 
   3.691241052912133*^9}, {3.6913370126232347`*^9, 3.691337037664125*^9}, {
   3.691337110117399*^9, 3.691337158740432*^9}, 3.691337298377512*^9, {
   3.691496917419263*^9, 3.691496927015682*^9}, {3.6915119613523893`*^9, 
   3.691511961998747*^9}, 3.69158338053426*^9, {3.6915989079546843`*^9, 
   3.6915989186838837`*^9}, 3.69167678662525*^9, 
   3.6928483547392197`*^9},ExpressionUUID->"3c9339cc-58d4-4fdb-a0a1-\
0696c8e774d8"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ExpandRepeatedEvents\[Rule]False", "Section",
 CellChangeTimes->{{3.689419140157094*^9, 3.689419202185822*^9}, {
  3.689419243129608*^9, 
  3.689419243807832*^9}},ExpressionUUID->"ca350638-f096-4041-83b7-\
2d7d2a37e377"],

Cell[CellGroupData[{

Cell["\<\
allow to proceed without expansion if there are no PermittedTimeShift \
conflicts\
\>", "Subsubsection",
 CellChangeTimes->{{3.689419161644956*^9, 3.6894191739701366`*^9}, {
  3.689419205366111*^9, 3.6894192658667107`*^9}, {3.689662460635446*^9, 
  3.68966247359625*^9}, {3.68966266261689*^9, 3.689662667342391*^9}, {
  3.6897036918341103`*^9, 3.6897037074167023`*^9}, {3.6897037626428413`*^9, 
  3.689703766543507*^9}},ExpressionUUID->"e3f46d59-e2f1-4ae8-a569-\
b736470674c5"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"evstatus", ":", 
       RowBox[{"{", 
        RowBox[{"False", ",", "numst_"}], "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"timingch_", ",", 
        RowBox[{"backgch", ":", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"evbackg", ":", 
            RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "_", ",", "_"}],
           "}"}]}], ",", "hilbch_", ",", "propcalcprocchar_", ",", 
        RowBox[{"{", 
         RowBox[{"False", ",", "True", ",", "False"}], "}"}], ",", 
        "morech___"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"repevs", ":", 
       RowBox[{"Repeat", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"evlist", ":", 
          RowBox[{"EventList", "[", 
           RowBox[{"evliststatus_", ",", "\[IndentingNewLine]", 
            RowBox[{"{", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"evlisttimech", ":", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"T", ":", 
                  RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "_", ",", 
                 
                 RowBox[{"pts", ":", 
                  RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "_"}], 
                "}"}]}], ",", "\[IndentingNewLine]", "moreevlistch__"}], 
             "}"}], ",", "\[IndentingNewLine]", 
            RowBox[{"events", ":", 
             RowBox[{"{", 
              RowBox[{"_Event", ".."}], "}"}]}], ",", "\[IndentingNewLine]", 
            "evlistopts___Rule"}], "\[IndentingNewLine]", "]"}]}], ",", 
         "\[IndentingNewLine]", "nrep_Integer", ",", "\[IndentingNewLine]", 
         "repevopts___Rule"}], "\[IndentingNewLine]", "]"}]}], ",", 
      "\[IndentingNewLine]", "evopts___Rule"}], "\[IndentingNewLine]", 
     "]"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"(", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Verbose", "/.", 
         RowBox[{"{", "evopts", "}"}]}], "/.", 
        RowBox[{"Options", "[", "Event", "]"}]}], ",", 
       RowBox[{"Print", "[", "\"\<forced no Repeat expansion\>\"", "]"}]}], 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Which", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"MatchQ", "[", 
          RowBox[{"pts", ",", "All"}], "]"}], "||", "\[IndentingNewLine]", 
         RowBox[{"TrueQ", "[", 
          RowBox[{
           RowBox[{"Chop", "@", 
            RowBox[{"N", "[", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"T", "/", "pts"}], ")"}], "-", 
              RowBox[{"Round", "[", 
               RowBox[{"T", "/", "pts"}], "]"}]}], "]"}]}], "\[Equal]", "0"}],
           "]"}]}], ")"}], ",", "\[IndentingNewLine]", "\t", 
       RowBox[{"Event", "[", 
        RowBox[{"evstatus", ",", 
         RowBox[{"{", 
          RowBox[{
          "timingch", ",", "backgch", ",", "hilbch", ",", "propcalcprocchar", 
           ",", 
           RowBox[{"{", 
            RowBox[{"False", ",", "True", ",", "True"}], "}"}], ",", 
           "morech"}], "}"}], ",", "repevs", ",", "evopts"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"MatchQ", "[", 
        RowBox[{"pts", ",", "None"}], "]"}], ",", "\[IndentingNewLine]", "\t", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Message", "[", 
          RowBox[{"Event", "::", "notperiodicrepev"}], "]"}], ";", 
         "$Failed"}], ")"}], ",", "\[IndentingNewLine]", 
       RowBox[{"!", 
        RowBox[{"TrueQ", "[", 
         RowBox[{
          RowBox[{"Chop", "@", 
           RowBox[{"N", "[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"T", "/", "pts"}], ")"}], "-", 
             RowBox[{"Round", "[", 
              RowBox[{"T", "/", "pts"}], "]"}]}], "]"}]}], "\[Equal]", "0"}], 
         "]"}]}], ",", "\[IndentingNewLine]", "\t", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Message", "[", 
          RowBox[{
           RowBox[{"Event", "::", "incompptsrepev"}], ",", "T", ",", "pts"}], 
          "]"}], ";", "$Failed"}], ")"}], ",", "\[IndentingNewLine]", "True", 
       ",", "$Failed"}], "\[IndentingNewLine]", "]"}]}], 
    "\[IndentingNewLine]", ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.68942117919565*^9, 3.689421187794764*^9}, {
   3.689421230839958*^9, 3.689421287381941*^9}, {3.6894213187902613`*^9, 
   3.689421366433655*^9}, {3.6894214032111187`*^9, 3.689421472596354*^9}, {
   3.689421527315557*^9, 3.6894216477242937`*^9}, {3.689422103976747*^9, 
   3.689422148430278*^9}, {3.689422190669052*^9, 3.6894223257075567`*^9}, {
   3.689422453316206*^9, 3.689422524271903*^9}, {3.68942271638349*^9, 
   3.689422724779826*^9}, {3.68942279419048*^9, 3.689422831229124*^9}, {
   3.689422900711296*^9, 3.689422910412019*^9}, 3.689621891048213*^9, {
   3.689622020771947*^9, 3.689622044728767*^9}, 3.6896247989381123`*^9, {
   3.689661671285231*^9, 3.689661687467443*^9}, {3.689661731938283*^9, 
   3.689661732225088*^9}, {3.689661796638577*^9, 3.689661813354232*^9}, {
   3.6896618532802877`*^9, 3.689661902798687*^9}, {3.689661933557723*^9, 
   3.68966197889191*^9}, {3.6896622400089808`*^9, 3.689662375264614*^9}, {
   3.6896625001408854`*^9, 3.689662520035637*^9}, {3.6896627068608227`*^9, 
   3.689662726469915*^9}, {3.6897030361218157`*^9, 3.689703147523931*^9}, {
   3.689703212544046*^9, 3.689703217805284*^9}, {3.6897032524130583`*^9, 
   3.689703420144416*^9}, 3.6897034551439333`*^9, {3.68970349272674*^9, 
   3.6897035364251823`*^9}, {3.6897035782033663`*^9, 3.689703583681691*^9}, {
   3.689703811665079*^9, 3.689703812398302*^9}, 3.689703844006215*^9, {
   3.68970391944004*^9, 3.689704028121447*^9}, {3.689704066718762*^9, 
   3.6897040690705557`*^9}, {3.68970430103933*^9, 3.6897043236996717`*^9}, {
   3.689704744816465*^9, 3.689704773724474*^9}, 3.689704808412684*^9, 
   3.689840201626297*^9, {3.689840268648039*^9, 3.689840325176015*^9}, {
   3.689840532712573*^9, 3.6898405356124163`*^9}, 3.689840575364771*^9, {
   3.690393690881765*^9, 3.69039369155466*^9}, {3.69039381776068*^9, 
   3.6903938420603228`*^9}, {3.690448607938883*^9, 3.6904486104886503`*^9}, {
   3.691337324657593*^9, 3.6913373844608088`*^9}, {3.691337506460071*^9, 
   3.6913375912275667`*^9}, {3.691337685304119*^9, 3.691337754770955*^9}, {
   3.691337786071513*^9, 3.691337790096745*^9}, {3.691337857871752*^9, 
   3.691337866860683*^9}, {3.691496942465845*^9, 3.691496954095612*^9}, 
   3.6916767924722443`*^9, 3.692848355340458*^9, 
   3.692849705826954*^9},ExpressionUUID->"beaa37a6-c02e-44bc-b7eb-\
d872c6877f6f"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PrecalculatedEvent", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, 
   3.683973033466785*^9, {3.683973750698831*^9, 3.683973753893053*^9}, {
   3.683976432753954*^9, 3.683976436691045*^9}, 3.68397674513378*^9, {
   3.683977981950523*^9, 3.683977988834993*^9}, {3.683978334297957*^9, 
   3.683978335668144*^9}, 3.683979655379014*^9, 3.683980372936699*^9, {
   3.689661184405801*^9, 3.689661197923876*^9}, {3.691435476615394*^9, 
   3.6914354802537823`*^9}, 
   3.691436746942408*^9},ExpressionUUID->"c36bc7ad-fd89-43fc-8818-\
27fb930b5c1d"],

Cell[CellGroupData[{

Cell["initialize checks", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, 
   3.6897472214839287`*^9, {3.691483618399654*^9, 
   3.691483640463098*^9}},ExpressionUUID->"5bd0506e-1315-4ec3-81d3-\
19da9f654422"],

Cell[CellGroupData[{

Cell["case of PrecalculatedEvent", "Subsubsection",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, 
   3.6897472214839287`*^9, {3.691483618399654*^9, 3.691483640463098*^9}, {
   3.691483726782041*^9, 3.691483727765574*^9}, {3.691483862644187*^9, 
   3.691483867169935*^9}, {3.691484708571612*^9, 
   3.691484716571047*^9}},ExpressionUUID->"cbfbf55f-943d-481f-adfe-\
dc4f65b08b0c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"st", ":", 
      RowBox[{"{", 
       RowBox[{"False", ",", "_"}], "}"}]}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"firstch", ":", 
        RowBox[{"Repeated", "[", 
         RowBox[{"_", ",", 
          RowBox[{"{", "5", "}"}]}], "]"}]}], ",", 
       RowBox[{"Undefined", "|", "False"}], ",", "morech___"}], "}"}], ",", 
     "precalev_PrecalculatedEvent", ",", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{"st", ",", 
     RowBox[{"{", 
      RowBox[{"firstch", ",", 
       RowBox[{"{", 
        RowBox[{"True", ",", "False", ",", "False", ",", "False"}], "}"}], 
       ",", "morech"}], "}"}], ",", "precalev", ",", "opts"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683976480502823*^9, 
   3.683976634737287*^9}, {3.6839766756355143`*^9, 3.683976688875939*^9}, {
   3.685033569558119*^9, 3.6850335695602427`*^9}, {3.6888401383764*^9, 
   3.688840152837696*^9}, {3.6888401902862053`*^9, 3.688840190524692*^9}, {
   3.689851804769758*^9, 3.689851824023171*^9}, {3.689851859300812*^9, 
   3.6898519415517282`*^9}, {3.6904483884124727`*^9, 3.690448400476754*^9}, {
   3.691483480590191*^9, 3.691483600640427*^9}, {3.691483752078665*^9, 
   3.6914837562185917`*^9}, {3.691484460619452*^9, 3.691484461608562*^9}, {
   3.691484730535697*^9, 3.691484766117159*^9}, {3.691599059671054*^9, 
   3.691599060670405*^9}},ExpressionUUID->"520bee49-c13b-456b-b2df-\
89819f0eebac"]
}, Closed]],

Cell[CellGroupData[{

Cell["not a PrecalculatedEvent", "Subsubsection",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, 
   3.6897472214839287`*^9, {3.691483618399654*^9, 3.691483640463098*^9}, {
   3.691483726782041*^9, 3.691483727765574*^9}, {3.691483862644187*^9, 
   3.691483867169935*^9}},ExpressionUUID->"aca168a8-9b6b-40e9-8574-\
578106bcafbd"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"st", ":", 
      RowBox[{"{", 
       RowBox[{"False", ",", "_"}], "}"}]}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"firstch", ":", 
        RowBox[{"Repeated", "[", 
         RowBox[{"_", ",", 
          RowBox[{"{", "5", "}"}]}], "]"}]}], ",", 
       RowBox[{"Except", "[", 
        RowBox[{"True", "|", 
         RowBox[{"{", 
          RowBox[{"True", ".."}], "}"}]}], "]"}], ",", "morech___"}], "}"}], 
     ",", 
     RowBox[{"evspec", ":", 
      RowBox[{"Except", "[", "_PrecalculatedEvent", "]"}]}], ",", 
     "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{"st", ",", 
     RowBox[{"{", 
      RowBox[{"firstch", ",", "True", ",", "morech"}], "}"}], ",", "evspec", 
     ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683976480502823*^9, 
   3.683976634737287*^9}, {3.6839766756355143`*^9, 3.683976688875939*^9}, {
   3.685033569558119*^9, 3.6850335695602427`*^9}, {3.6888401383764*^9, 
   3.688840152837696*^9}, {3.6888401902862053`*^9, 3.688840190524692*^9}, {
   3.689851804769758*^9, 3.689851824023171*^9}, {3.689851859300812*^9, 
   3.6898519415517282`*^9}, {3.6904483884124727`*^9, 3.690448400476754*^9}, {
   3.691483480590191*^9, 3.691483600640427*^9}, {3.691483752078665*^9, 
   3.6914837562185917`*^9}, {3.69148387823921*^9, 
   3.691483926618992*^9}},ExpressionUUID->"ec564a1d-dd20-4a06-82ad-\
155c25b23825"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["perform checks", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, 
   3.6897472214839287`*^9, {3.691483618399654*^9, 3.691483640463098*^9}, {
   3.691483726782041*^9, 
   3.691483727765574*^9}},ExpressionUUID->"876dbd1d-0d17-464a-903f-\
e9366029d6e4"],

Cell[CellGroupData[{

Cell["trap background conflict", "Subsubsection",
 CellChangeTimes->{{3.691435972800692*^9, 3.691435979302993*^9}, {
   3.691436012308934*^9, 3.69143601754146*^9}, 3.6914838017387953`*^9, {
   3.6914842290706873`*^9, 
   3.691484229764233*^9}},ExpressionUUID->"3e2a0fc0-782c-4182-af34-\
3aff4133e9a5"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"evstatus", ":", 
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}]}], ",", "\[IndentingNewLine]", 
     
     RowBox[{"{", 
      RowBox[{"evtiming_", ",", 
       RowBox[{"backgch", ":", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"evbackg", ":", 
           RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "_", ",", "_"}], 
         "}"}]}], ",", "hilbch_", ",", "pcp_", ",", "exprep_", ",", 
       RowBox[{"{", 
        RowBox[{"True", ",", "False", ",", "False", ",", "False"}], "}"}], 
       ",", "evmorech___"}], "}"}], ",", "\[IndentingNewLine]", 
     "precalev_PrecalculatedEvent", ",", "\[IndentingNewLine]", 
     "evopts___Rule"}], "\[IndentingNewLine]", "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"If", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"GeneratorMatchQ", "[", 
      RowBox[{"evbackg", ",", 
       RowBox[{"BackgroundGenerator", "[", "precalev", "]"}]}], "]"}], ",", 
     "\[IndentingNewLine]", "\t", 
     RowBox[{"Event", "[", 
      RowBox[{"evstatus", ",", "\[IndentingNewLine]", "\t", 
       RowBox[{"{", 
        RowBox[{
        "evtiming", ",", "backgch", ",", "hilbch", ",", "pcp", ",", "exprep", 
         ",", 
         RowBox[{"{", 
          RowBox[{"True", ",", "True", ",", "False", ",", "False"}], "}"}], 
         ",", "evmorech"}], "}"}], ",", "\[IndentingNewLine]", "\t", 
       "precalev", ",", "evopts"}], "]"}], ",", "\[IndentingNewLine]", "\t", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Message", "[", 
        RowBox[{"PrecalculatedEvent", "::", "precalbackgconflict"}], "]"}], 
       ";", "$Failed"}], ")"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691436028108746*^9, 3.691436033562479*^9}, {
   3.691436070565824*^9, 3.6914362320789223`*^9}, {3.691436323793571*^9, 
   3.6914363427411957`*^9}, 3.6914364363588667`*^9, {3.6914367471056004`*^9, 
   3.691436747110643*^9}, {3.691437081567905*^9, 3.691437122519652*^9}, 
   3.6914374379093933`*^9, {3.691483983233308*^9, 3.691484014049601*^9}, {
   3.69148404669042*^9, 3.691484204638831*^9}, {3.6914845621454773`*^9, 
   3.69148456809439*^9}, {3.691492011787732*^9, 3.6914920147367697`*^9}, {
   3.691498451874126*^9, 3.691498468102113*^9}, 3.709374300990754*^9, 
   3.7093749373045063`*^9},ExpressionUUID->"0be90947-de82-481c-97f6-\
5da7c12804c0"]
}, Closed]],

Cell[CellGroupData[{

Cell["trap timing conflict", "Subsubsection",
 CellChangeTimes->{{3.691435972800692*^9, 3.691435979302993*^9}, {
   3.691436012308934*^9, 3.69143601754146*^9}, 3.6914838017387953`*^9, {
   3.6914842290706873`*^9, 
   3.691484247347835*^9}},ExpressionUUID->"27449e83-298d-45dd-a531-\
cf782926b10a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"evstatus", ":", 
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}]}], ",", "\[IndentingNewLine]", 
     
     RowBox[{"{", 
      RowBox[{
       RowBox[{"evtch", ":", 
        RowBox[{"{", 
         RowBox[{"_", ",", "_", ",", "_", ",", 
          RowBox[{"evstandintv", ":", 
           RowBox[{"Except", "[", "Undefined", "]"}]}]}], "}"}]}], ",", 
       "evbgch_", ",", "evhilbch_", ",", "evpcp_", ",", "evexprep_", ",", 
       RowBox[{"{", 
        RowBox[{"True", ",", "True", ",", "False", ",", "False"}], "}"}], ",",
        "evmorech___"}], "}"}], ",", "\[IndentingNewLine]", 
     "precalev_PrecalculatedEvent", ",", "\[IndentingNewLine]", 
     "evopts___Rule"}], "\[IndentingNewLine]", "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"If", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"MatchQ", "[", 
      RowBox[{"evstandintv", ",", 
       RowBox[{"StandardizedInterval", "@", "precalev"}]}], "]"}], ",", 
     "\[IndentingNewLine]", "\t", 
     RowBox[{"Event", "[", 
      RowBox[{"evstatus", ",", "\[IndentingNewLine]", "\t", 
       RowBox[{"{", 
        RowBox[{
        "evtch", ",", "evbgch", ",", "evhilbch", ",", "evpcp", ",", 
         "evexprep", ",", 
         RowBox[{"{", 
          RowBox[{"True", ",", "True", ",", "True", ",", "False"}], "}"}], 
         ",", "evmorech"}], "}"}], ",", "\[IndentingNewLine]", "\t", 
       "precalev", ",", "evopts"}], "]"}], ",", "\[IndentingNewLine]", "\t", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Message", "[", 
        RowBox[{"PrecalculatedEvent", "::", "precaltimingconflict"}], "]"}], 
       ";", "$Failed"}], ")"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691436028108746*^9, 3.691436033562479*^9}, {
   3.691436070565824*^9, 3.6914362320789223`*^9}, {3.691436323793571*^9, 
   3.6914363427411957`*^9}, 3.6914364363588667`*^9, {3.6914367471056004`*^9, 
   3.691436747110643*^9}, {3.691437081567905*^9, 3.691437122519652*^9}, 
   3.6914374379093933`*^9, {3.691483983233308*^9, 3.691484014049601*^9}, {
   3.69148404669042*^9, 3.691484204638831*^9}, {3.691484260432482*^9, 
   3.691484376431663*^9}, {3.691484574425083*^9, 3.69148458041291*^9}, {
   3.691491200385171*^9, 3.691491225407876*^9}, {3.691491268006949*^9, 
   3.691491272480036*^9}, {3.691498482808833*^9, 
   3.691498504936245*^9}},ExpressionUUID->"3f33bd6a-69e0-43fd-af7b-\
df0245667971"]
}, Closed]],

Cell[CellGroupData[{

Cell["inherit PrecalculatedEvent characteristics", "Subsubsection",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, 
   3.6897472214839287`*^9, {3.691483618399654*^9, 3.691483640463098*^9}, {
   3.691483726782041*^9, 3.691483727765574*^9}, {3.691484434897243*^9, 
   3.691484451791679*^9}},ExpressionUUID->"536b0544-49e7-439d-b9ac-\
ae9679a0917d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"evstatus", ":", 
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}]}], ",", "\[IndentingNewLine]", 
     
     RowBox[{"{", 
      RowBox[{
      "evtch_", ",", "evbgch_", ",", "evhilbch_", ",", "evpcp_", ",", 
       "evexprep_", ",", 
       RowBox[{"{", 
        RowBox[{"True", ",", "True", ",", "True", ",", "False"}], "}"}], ",", 
       "evmorech___"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"precalev", ":", 
      RowBox[{"PrecalculatedEvent", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
         "precalevtch_", ",", "precalevbgch_", ",", "precalhilbch_", ",", 
          "precalevexprep_", ",", "precalmorech___"}], "}"}], ",", 
        "\[IndentingNewLine]", "propagator_", ",", "precalevopts___Rule"}], 
       "]"}]}], ",", "\[IndentingNewLine]", "evopts___Rule"}], 
    "\[IndentingNewLine]", "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{"evstatus", ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "evtch", ",", "precalevbgch", ",", "precalhilbch", ",", "evpcp", ",", 
       "precalevexprep", ",", 
       RowBox[{"{", 
        RowBox[{"True", ",", "True", ",", "True", ",", "True"}], "}"}], ",", 
       "evmorech"}], "}"}], ",", "\[IndentingNewLine]", "precalev", ",", 
     "\[IndentingNewLine]", 
     RowBox[{"SimplifyRules", "[", 
      RowBox[{"precalevopts", ",", "evopts", ",", "evopts"}], "]"}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691435547128901*^9, 3.691435682254319*^9}, {
   3.6914357422937717`*^9, 3.6914357483170233`*^9}, {3.6914357877095623`*^9, 
   3.691435938089678*^9}, 3.691436001586972*^9, {3.691436701387446*^9, 
   3.6914367136410007`*^9}, 3.6914367465309877`*^9, {3.691437170811988*^9, 
   3.691437352856876*^9}, {3.691437516350451*^9, 3.691437548335384*^9}, {
   3.6914911438607397`*^9, 3.691491184231147*^9}, {3.6914912887351103`*^9, 
   3.691491501857704*^9}},ExpressionUUID->"7564f890-0dfb-4787-8a3a-\
b78a81cb9d1e"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["set check status to True", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, 
   3.6897472214839287`*^9, {3.691483618399654*^9, 3.691483657830803*^9}, {
   3.691484502058107*^9, 
   3.6914845126413927`*^9}},ExpressionUUID->"00bc18a0-e4f3-41e0-baab-\
d91f883560f0"],

Cell[CellGroupData[{

Cell["all checks done", "Subsubsection",
 CellChangeTimes->{{3.69148451591156*^9, 
  3.691484519045335*^9}},ExpressionUUID->"4ca8d489-acc9-427e-ae8c-\
8246c4679753"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "[", 
    RowBox[{"st_", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"firstch", ":", 
        RowBox[{"Repeated", "[", 
         RowBox[{"_", ",", 
          RowBox[{"{", "5", "}"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"True", ".."}], "}"}], ",", "morech___"}], "}"}], ",", 
     "evspec_", ",", "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Event", "[", 
    RowBox[{"st", ",", 
     RowBox[{"{", 
      RowBox[{"firstch", ",", "True", ",", "morech"}], "}"}], ",", "evspec", 
     ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683976480502823*^9, 
   3.683976634737287*^9}, {3.6839766756355143`*^9, 3.683976688875939*^9}, {
   3.685033569558119*^9, 3.6850335695602427`*^9}, {3.6888401383764*^9, 
   3.688840152837696*^9}, {3.6888401902862053`*^9, 3.688840190524692*^9}, {
   3.689851804769758*^9, 3.689851824023171*^9}, {3.689851859300812*^9, 
   3.6898519415517282`*^9}, {3.6904483884124727`*^9, 3.690448400476754*^9}, {
   3.691483480590191*^9, 3.691483600640427*^9}, {3.691483660585376*^9, 
   3.6914836680871773`*^9}, {3.691483770990797*^9, 3.691483781292886*^9}, 
   3.691492075552844*^9},ExpressionUUID->"0a8b3dd5-58bf-415a-8ef4-\
33baa79259c7"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["debugging routines", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, 
   3.683973033466785*^9, {3.683973750698831*^9, 3.683973753893053*^9}, {
   3.683976432753954*^9, 3.683976436691045*^9}, 3.68397674513378*^9, {
   3.683977981950523*^9, 3.683977988834993*^9}, {3.683978334297957*^9, 
   3.683978335668144*^9}, 3.683979655379014*^9, 3.683980372936699*^9, {
   3.689661184405801*^9, 3.689661197923876*^9}, {3.691435476615394*^9, 
   3.6914354802537823`*^9}, 3.691436746942408*^9, {3.691578910998966*^9, 
   3.691578914207658*^9}},ExpressionUUID->"291ddf99-211c-40fd-a9d0-\
cf4db0a12080"],

Cell[CellGroupData[{

Cell["trap characteristics of incorrect length", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683959026765203*^9, 3.683959030893498*^9}, {
   3.683959139611781*^9, 3.683959149162888*^9}, {3.683964513177393*^9, 
   3.683964514887501*^9}, 3.6912374833924713`*^9, {3.6915780626169567`*^9, 
   3.6915780839942636`*^9}, {3.691578813830924*^9, 
   3.6915788149875097`*^9}},ExpressionUUID->"966addab-8666-490f-b5c8-\
795bb93a02c3"],

Cell["this is useful for debugging purposes", "Text",
 CellChangeTimes->{{3.6915782465740957`*^9, 
  3.691578252208765*^9}},ExpressionUUID->"4832414d-83e4-48bc-857b-\
1acf509447c1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Event", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "_"}], "}"}], ",", "chars_List", ",", 
      "more___"}], "]"}], "/;", 
    RowBox[{
     RowBox[{"Length", "[", "chars", "]"}], "!=", 
     RowBox[{"CharacteristicsDimension", "[", "Event", "]"}]}]}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Message", "[", 
      RowBox[{"Event", "::", "badcharlen"}], "]"}], ";", "$Failed"}], ")"}]}],
   ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691578093687908*^9, 3.6915781633320017`*^9}, 
   3.691578217861908*^9, {3.691578295840374*^9, 3.6915782993988333`*^9}, {
   3.6915786580120153`*^9, 3.691578682543268*^9}, 
   3.691578771272049*^9},ExpressionUUID->"eaf5cb67-760f-493d-85fd-\
de7cd9bc68a4"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["messages", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, 
   3.683973033466785*^9, {3.683973490977272*^9, 
   3.683973492079618*^9}},ExpressionUUID->"d6f8eb87-9c1c-4c6b-924e-\
f1893eefb39f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "::", "badtimepts"}], "=", 
   "\"\<Unable to assign initial or final time points of an event.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683973498852951*^9, 3.6839735139731617`*^9}, {
   3.683973599098839*^9, 3.683973602179441*^9}, 
   3.685033569582511*^9},ExpressionUUID->"5fe693a2-53c1-478d-88cb-\
0fee19ac3c9a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "::", "tconflict"}], "=", 
   "\"\<A timing conflict has been generated.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683973498852951*^9, 3.6839735139731617`*^9}, {
   3.683973599098839*^9, 3.683973602179441*^9}, {3.683980705947625*^9, 
   3.683980714326729*^9}, 
   3.685033569586253*^9},ExpressionUUID->"18b6deb8-7b45-4d37-8453-\
a19806b00258"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "::", "negdur"}], "=", 
   "\"\<A negative event duration has been encountered.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6839809044651737`*^9, 3.683980915602433*^9}, 
   3.685033569589961*^9},ExpressionUUID->"38d4f7a0-c96d-4741-bea4-\
000260f41ad7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "::", "backgconflict"}], "=", 
   "\"\<A conflict in evolution backgrounds has been encountered.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.684040045259514*^9, 3.684040105528532*^9}, {
   3.685033554329681*^9, 3.6850335695934896`*^9}, 3.685035810688266*^9, 
   3.68900719846987*^9, {3.689745961713004*^9, 
   3.689745979301751*^9}},ExpressionUUID->"2d0ea6fd-60bf-4969-9158-\
7a939f0c6504"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "::", "repevbackgconflict"}], "=", 
   "\"\<The evolution background in the Repeat object differs from that in \
the enclosing Event.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.684040045259514*^9, 3.684040105528532*^9}, {
   3.685033554329681*^9, 3.6850335695934896`*^9}, 3.685035810688266*^9, 
   3.68900719846987*^9, {3.6890089000588913`*^9, 3.6890089327606077`*^9}, 
   3.692849708612228*^9},ExpressionUUID->"0bd9e42a-8b87-44ba-b712-\
4fad80b49220"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "::", "notperiodicrepev"}], "=", 
   "\"\<The Repeat cannot be implemented without expansion, since the event \
itself or the BackgroundGenerator are not invariant to \
time-translation.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6913376018488398`*^9, 3.691337666130106*^9}, {
   3.6913378527171373`*^9, 3.6913378533105383`*^9}, 3.6928497096682177`*^9, 
   3.709374938024435*^9},ExpressionUUID->"8cf70e51-d948-41a9-b589-\
397a53db1021"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "::", "incompptsrepev"}], "=", 
   "\"\<The Repeat cannot be implemented without expansion, since the \
duration `1` of the repeating event is incompatible with the allowed \
time-translation interval `2` of the event, taking into account the \
BackgroundGenerator.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.684040045259514*^9, 3.684040105528532*^9}, {
   3.685033554329681*^9, 3.6850335695934896`*^9}, 3.685035810688266*^9, 
   3.68900719846987*^9, {3.6890089000588913`*^9, 3.6890089327606077`*^9}, {
   3.689662387066208*^9, 3.6896624284461946`*^9}, {3.690554829395829*^9, 
   3.690554834596367*^9}, {3.69133739602807*^9, 3.691337490481583*^9}, {
   3.691337760467606*^9, 3.691337831676543*^9}, {3.691337862583665*^9, 
   3.691337863143277*^9}, 3.6928497104970093`*^9, 
   3.7093749386993647`*^9},ExpressionUUID->"d10bc928-ef1c-4c0d-be74-\
f380693faa0c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Event", "::", "badcharlen"}], "=", 
   "\"\<The Event characteristics do not have the correct length.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691578179634967*^9, 
  3.6915782083048773`*^9}},ExpressionUUID->"84869682-9295-4a8e-a43e-\
d3d73d3df710"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["EventList", "Subtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455265644416114*^9, 3.455265648200944*^9}, {
   3.464256156522676*^9, 3.4642561730892*^9}, {3.464282095283453*^9, 
   3.4642821290032167`*^9}, {3.503094263066369*^9, 3.503094264574778*^9}, {
   3.503095573470295*^9, 3.503095585674507*^9}, {3.504260287465301*^9, 
   3.504260304480891*^9}, 3.504261075915663*^9, 3.514538950441782*^9, {
   3.541442633712829*^9, 3.541442660772046*^9}, {3.541485412813717*^9, 
   3.541485415659666*^9}, 3.54149325313505*^9, {3.683128937510503*^9, 
   3.683128938174704*^9}, {3.683958413924868*^9, 3.683958431756687*^9}, {
   3.683981398158553*^9, 3.6839813991814127`*^9}, {3.683998958276259*^9, 
   3.6839989653111134`*^9}, {3.6839992231386623`*^9, 3.683999224024452*^9}, 
   3.685033539456119*^9, {3.685033905139474*^9, 3.685033906927793*^9}, 
   3.689747054878276*^9},ExpressionUUID->"0cc2e383-fb5e-4e41-9703-\
00c5085a1420"],

Cell[CellGroupData[{

Cell["defaults", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, {
   3.683973033466785*^9, 3.683973044514587*^9}, {3.6842152838892107`*^9, 
   3.684215286048623*^9}},ExpressionUUID->"7e30ba0c-4c16-4901-a2e2-\
fd3e8d13d0b6"],

Cell[CellGroupData[{

Cell["default options", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683958534614377*^9, 
   3.683958557622012*^9}},ExpressionUUID->"31a4a36f-e372-424f-a3e2-\
08e5e7141ef0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "EventList", "]"}], "=", "\[IndentingNewLine]", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"InitialTimePoint", "\[Rule]", "Automatic"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FinalTimePoint", "\[Rule]", "Automatic"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"BackgroundGenerator", "\[Rule]", "Undefined"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"ExpandRepeatedEvents", "\[Rule]", "Automatic"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Verbose", "\[Rule]", "False"}]}], "\[IndentingNewLine]", 
    "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.683958565949041*^9, {3.6839731149650908`*^9, 3.683973130952525*^9}, {
   3.683973897643121*^9, 3.683973898836083*^9}, {3.683977863168428*^9, 
   3.683977866811652*^9}, 3.683981650615322*^9, {3.683981748697813*^9, 
   3.683981749304667*^9}, 3.683981815370844*^9, {3.683999557015005*^9, 
   3.683999557678842*^9}, 3.6850335394863377`*^9, {3.689411552144205*^9, 
   3.689411559515828*^9}, {3.6894255585873938`*^9, 3.689425559949933*^9}, {
   3.690448711896319*^9, 3.6904487155422983`*^9}, 
   3.709374939779333*^9},ExpressionUUID->"aee02f02-5b53-4644-ab7a-\
b654cc534e9d"]
}, Closed]],

Cell[CellGroupData[{

Cell["$Failed", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683958534614377*^9, 3.683958551028832*^9}, {
   3.68395858577393*^9, 3.6839585874144278`*^9}, 3.683959406338385*^9, {
   3.683963242982894*^9, 3.6839632443419228`*^9}, {3.683964445776389*^9, 
   3.683964455294318*^9}, {3.683983901941537*^9, 
   3.6839839032008667`*^9}},ExpressionUUID->"311fc07b-fe46-4fb3-ade5-\
f7b336bc7ae6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventList", "[", "args___", "]"}], "/;", 
    RowBox[{"!", 
     RowBox[{"FreeQ", "[", 
      RowBox[{
       RowBox[{"{", "args", "}"}], ",", "$Failed"}], "]"}]}]}], ":=", 
   "$Failed"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683983914149064*^9, 3.683983952764398*^9}, 
   3.685033539491296*^9},ExpressionUUID->"094e1e02-0289-4da5-a60f-\
055dc4830da3"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["status & characteristics", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, {
   3.683973033466785*^9, 3.683973044514587*^9}, 
   3.684000480954048*^9},ExpressionUUID->"8ce5ff9a-4cf7-4f33-a7b4-\
7fe112366fde"],

Cell[CellGroupData[{

Cell["EventList status & char syntax", "Section",
 CellChangeTimes->{{3.6839769345684147`*^9, 3.683976958869174*^9}, {
   3.6851721013267317`*^9, 3.6851721044519997`*^9}, {3.685194518004075*^9, 
   3.6851945584473248`*^9}, 3.685196786924673*^9, {3.689410168208012*^9, 
   3.689410178325425*^9}},ExpressionUUID->"f8802b46-cbc4-4f32-a262-\
d41cb4d07695"],

Cell[CellGroupData[{

Cell["status:{<ready>,<numeric>}", "Subsection",
 CellChangeTimes->{{3.689409839714465*^9, 
  3.689409859202746*^9}},ExpressionUUID->"de508d1d-4a05-44a7-9657-\
9a6bc51e2ba3"],

Cell["\<\
<ready>=True if ready for computation.
<numeric>=True if the event descriptors are purely numeric.\
\>", "Text",
 CellChangeTimes->{{3.689409869650942*^9, 
  3.6894098789918537`*^9}},ExpressionUUID->"3b2003de-aeb1-45fd-9b62-\
311f80dd2add"]
}, Closed]],

Cell[CellGroupData[{

Cell["characteristics:{timing,evbackg,hilbertcompat,expreps}", "Subsection",
 CellChangeTimes->{{3.689409839714465*^9, 3.689409859202746*^9}, 
   3.689409906443359*^9, {3.6894102102018147`*^9, 
   3.689410213614195*^9}},ExpressionUUID->"57efa8cc-0834-4e1a-8d22-\
6df17fc6f3ee"],

Cell[CellGroupData[{

Cell["<timing> = {duration, {tb,ta}, pts, {tb0,ta0}}", "Subsubsection",
 CellChangeTimes->{{3.689409839714465*^9, 3.6894098405450172`*^9}, {
  3.689409926890026*^9, 
  3.689409928752881*^9}},ExpressionUUID->"dd9bc39b-e1a0-4705-8da6-\
d0399a093825"],

Cell["\<\
{tb,ta} are the final and initial times in the sequence of events;
pts is the permitted time shift (pts) according to the type of event and the \
evolution background;
{tb0,ta0} are the standardized final and initial times, taking into account \
the pts. \
\>", "Text",
 CellChangeTimes->{{3.689409937366763*^9, 
  3.689409954352346*^9}},ExpressionUUID->"3b2eed51-dc02-4829-9136-\
5e3bc32ef617"]
}, Closed]],

Cell[CellGroupData[{

Cell["<evbackg> = {evbackg,<reqbackgimp>,<backgimp>}\t\t", "Subsubsection",
 CellChangeTimes->{{3.689409839714465*^9, 3.6894098405450172`*^9}, {
  3.689409973737048*^9, 
  3.689409983579653*^9}},ExpressionUUID->"0ccdc951-5a40-483a-bd4d-\
235273f57d05"],

Cell["\<\
where evbackg is the evolution background
<reqbackgimp>=True requests implementation of the background
<backgimp> = True when the backg has been incorporated into the event.\
\>", "Text",
 CellChangeTimes->{{3.6894099872283*^9, 
  3.689409995597994*^9}},ExpressionUUID->"e73fd3b9-10f8-478e-84f4-\
bb98d35fedb8"]
}, Closed]],

Cell[CellGroupData[{

Cell["<hilbert>", "Subsubsection",
 CellChangeTimes->{{3.689409839714465*^9, 3.6894098405450172`*^9}, 
   3.68941000604853*^9},ExpressionUUID->"f6775406-bc14-486a-8142-\
a1935eaf4049"],

Cell["<hilbertcompat> = True|False|MixedHilbertSpaceCompatibility", "Text",
 CellChangeTimes->{3.689410013197605*^9, 
  3.689410236830711*^9},ExpressionUUID->"809b57ba-bb33-4505-9a7a-\
1908fecbbaca"]
}, Closed]],

Cell[CellGroupData[{

Cell["<exprep>={type,request,implemented}", "Subsubsection",
 CellChangeTimes->{{3.689409839714465*^9, 3.6894098405450172`*^9}, {
  3.689410048942927*^9, 3.689410051061969*^9}, {3.689410603637149*^9, 
  3.689410608688835*^9}},ExpressionUUID->"553a7f1d-39e7-4de9-a937-\
19f932856dd7"],

Cell["\<\
type = True|All|Automatic
exprequest = True (requests expansion)
expimp=True (indicates implementation)\
\>", "Text",
 CellChangeTimes->{{3.689410614538101*^9, 
  3.689410668509727*^9}},ExpressionUUID->"4f946dd7-cb9e-4b14-a38c-\
f86da2ad8f1b"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["status", "Section",
 CellChangeTimes->{{3.6839769345684147`*^9, 3.683976958869174*^9}, {
   3.6851950645150757`*^9, 3.6851950710944757`*^9}, 
   3.691578949794083*^9},ExpressionUUID->"ea9d7201-78ec-42f4-871c-\
33b44fa66574"],

Cell[CellGroupData[{

Cell["fill in False status if omitted", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683958727456455*^9, 3.6839587370191107`*^9}, {
  3.6839587793697453`*^9, 
  3.683958781370214*^9}},ExpressionUUID->"9580abd3-0eec-452b-9f5f-\
9d1bd3482569"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"events", ":", 
      RowBox[{"Except", "[", "_Rule", "]"}]}], ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"EventList", "[", 
    RowBox[{"False", ",", "events", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683958754729197*^9, 3.683958774518546*^9}, {
  3.683958838874489*^9, 3.683958841777224*^9}, {3.683959513848783*^9, 
  3.68395951454422*^9}, {3.683981827400544*^9, 3.683981844352995*^9}, {
  3.685033539495494*^9, 
  3.685033539497529*^9}},ExpressionUUID->"c9aaed10-69e4-4648-b723-\
71feabc1023f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{"False", ",", 
     RowBox[{"more", ":", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Except", "[", "_Rule", "]"}], "..."}], ")"}]}], ",", 
     "opts___Rule"}], "]"}], ":=", 
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "False"}], "}"}], ",", "more", ",", "opts"}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6839586027648153`*^9, 3.6839586045846977`*^9}, {
   3.683958667580065*^9, 3.683958711018367*^9}, 3.683959521095401*^9, {
   3.683981848608028*^9, 3.683981852892179*^9}, {3.685033539501849*^9, 
   3.685033539503861*^9}},ExpressionUUID->"a63ffe0f-d79d-42fe-bc1e-\
ab5e7d6f2569"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["characteristics", "Section",
 CellChangeTimes->{{3.6839769345684147`*^9, 3.683976958869174*^9}, {
   3.6851950645150757`*^9, 3.685195081500847*^9}, 
   3.6915789525819407`*^9},ExpressionUUID->"9afe5f75-837f-4879-b3c2-\
3aee82351253"],

Cell[CellGroupData[{

Cell["set up undefined characteristics", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683958727456455*^9, 3.6839587370191107`*^9}, {
  3.6839587793697453`*^9, 3.683958818498616*^9}, {3.683959106962901*^9, 
  3.683959108074684*^9}},ExpressionUUID->"7ba85771-e44f-4383-b6e3-\
575f3e46511f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CharacteristicsDimension", "[", "EventList", "]"}], "=", "4"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.685195090505018*^9, 3.685195107861063*^9}, {
  3.685195143030073*^9, 
  3.685195143377007*^9}},ExpressionUUID->"907c6d9f-4e02-4746-8873-\
f324aff0a642"],

Cell[BoxData[
 RowBox[{
  RowBox[{"EventList", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"False", ",", "False"}], "}"}], ",", 
    RowBox[{"events", ":", 
     RowBox[{"Except", "[", "_Rule", "]"}]}], ",", "opts___Rule"}], "]"}], ":=", 
  RowBox[{"EventList", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"False", ",", "False"}], "}"}], ",", "Undefined", ",", "events", 
    ",", "opts"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.685195170130768*^9, 
  3.6851951800661907`*^9}},ExpressionUUID->"8c568151-9af8-4bd6-9081-\
f3d42b34cff4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "False"}], "}"}], ",", "Undefined", ",", 
     RowBox[{"events", ":", 
      RowBox[{"Except", "[", "_Rule", "]"}]}], ",", "opts___Rule"}], "]"}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "False"}], "}"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{"Undefined", ",", 
       RowBox[{"{", 
        RowBox[{"CharacteristicsDimension", "[", "EventList", "]"}], "}"}]}], 
      "]"}], ",", "events", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683958923673643*^9, 3.683958957650661*^9}, 
   3.68395953006183*^9, {3.6839818603988533`*^9, 3.683981871917111*^9}, {
   3.683981920171609*^9, 3.683981920507165*^9}, {3.685033539509947*^9, 
   3.6850335395129957`*^9}, {3.685195150827013*^9, 
   3.685195186106042*^9}},ExpressionUUID->"0061c691-8dff-4e0c-bd77-\
aa00ccecd465"]
}, Closed]],

Cell[CellGroupData[{

Cell["expand undefined timing", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683959026765203*^9, 3.683959030893498*^9}, {
  3.683964505178977*^9, 
  3.683964506044327*^9}},ExpressionUUID->"ba140227-df19-4252-a8cd-\
34b944fffb9f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{"status_", ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Undefined", ",", "charsb___"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"events", ":", 
      RowBox[{"Except", "[", "_Rule", "]"}]}], ",", "opts___Rule"}], "]"}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"EventList", "[", 
    RowBox[{"status", ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
        "Undefined", ",", "Undefined", ",", "Undefined", ",", "Undefined"}], 
        "}"}], ",", "charsb"}], "}"}], ",", "\[IndentingNewLine]", "events", 
     ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683958754729197*^9, 3.683958774518546*^9}, {
   3.683958829583066*^9, 3.683958907629072*^9}, {3.683958964954804*^9, 
   3.683958973570923*^9}, {3.6839590069879436`*^9, 3.68395902262541*^9}, {
   3.68395907781216*^9, 3.683959098863797*^9}, {3.683959173590906*^9, 
   3.683959178632263*^9}, 3.683959535236364*^9, {3.68398192947958*^9, 
   3.6839819394916563`*^9}, 3.683983260496846*^9, {3.685033539518495*^9, 
   3.685033539520924*^9}},ExpressionUUID->"29376cdc-fa2a-4161-b6ba-\
b84f624d75b5"]
}, Closed]],

Cell[CellGroupData[{

Cell["expand undefined BackgroundGenerator", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683959026765203*^9, 3.683959030893498*^9}, {
   3.683959139611781*^9, 3.683959149162888*^9}, {3.683964513177393*^9, 
   3.683964514887501*^9}, 
   3.709374940551311*^9},ExpressionUUID->"50f59122-d174-4244-8547-\
4c22ea64e648"],

Cell[BoxData[
 RowBox[{
  RowBox[{"EventList", "[", 
   RowBox[{"status_", ",", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{"charsa_", ",", "Undefined", ",", "charsb___"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"events", ":", 
     RowBox[{"Except", "[", "_Rule", "]"}]}], ",", "opts___Rule"}], "]"}], ":=",
   "\[IndentingNewLine]", 
  RowBox[{"EventList", "[", 
   RowBox[{"status", ",", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{"charsa", ",", 
      RowBox[{"{", 
       RowBox[{"Undefined", ",", "Undefined", ",", "Undefined"}], "}"}], ",", 
      "charsb"}], "}"}], ",", "\[IndentingNewLine]", "events", ",", "opts"}], 
   "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683959160009802*^9, 3.683959170708824*^9}, {
   3.6839592323540983`*^9, 3.6839592401844482`*^9}, 3.683959539589954*^9, {
   3.683981963657967*^9, 3.683981984731845*^9}, {3.6850335395253477`*^9, 
   3.685033539527384*^9}, {3.688838252226807*^9, 
   3.688838254288083*^9}},ExpressionUUID->"50914bbe-4011-4d73-9e73-\
15b2f2549708"]
}, Closed]],

Cell[CellGroupData[{

Cell["expand undefined Repeat expansion", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683959026765203*^9, 3.683959030893498*^9}, {
   3.683959139611781*^9, 3.683959149162888*^9}, {3.683964513177393*^9, 
   3.683964514887501*^9}, {3.685195210732675*^9, 3.685195216255212*^9}, 
   3.6905549504701223`*^9, 
   3.692848356183332*^9},ExpressionUUID->"f14be8ed-402d-4f03-b9ae-\
20247a48f313"],

Cell[BoxData[
 RowBox[{
  RowBox[{"EventList", "[", 
   RowBox[{"status_", ",", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"charsa", ":", 
       RowBox[{"Repeated", "[", 
        RowBox[{"_", ",", 
         RowBox[{"{", "3", "}"}]}], "]"}]}], ",", "Undefined", ",", 
      "charsb___"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"events", ":", 
     RowBox[{"Except", "[", "_Rule", "]"}]}], ",", "opts___Rule"}], "]"}], ":=",
   "\[IndentingNewLine]", 
  RowBox[{"EventList", "[", 
   RowBox[{"status", ",", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{"charsa", ",", 
      RowBox[{"{", 
       RowBox[{"Undefined", ",", " ", "Undefined", ",", "Undefined"}], "}"}], 
      ",", "charsb"}], "}"}], ",", "\[IndentingNewLine]", "events", ",", 
    "opts"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683959160009802*^9, 3.683959170708824*^9}, {
   3.6839592323540983`*^9, 3.6839592401844482`*^9}, 3.683959539589954*^9, {
   3.683981963657967*^9, 3.683981984731845*^9}, {3.6850335395253477`*^9, 
   3.685033539527384*^9}, {3.68519524124466*^9, 3.6851952630012207`*^9}, {
   3.6894114124983397`*^9, 
   3.6894114141423397`*^9}},ExpressionUUID->"67c07e1b-149f-40ee-b5c4-\
0bc33b43a9a7"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ready status", "Section",
 CellChangeTimes->{{3.6839769345684147`*^9, 
  3.6839769370653973`*^9}},ExpressionUUID->"37325bf9-417c-47b9-86b3-\
8ed564696f9e"],

Cell[CellGroupData[{

Cell["\<\
set ready status to True if no Undefined characteristics left, the \
BackgroundGenerator has been implemented, and all ExpandRepeatedEvents \
instructions have been carried out. \
\>", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683958727456455*^9, 3.6839587370191107`*^9}, {
   3.6839587793697453`*^9, 3.683958781370214*^9}, {3.683963257225411*^9, 
   3.683963281087583*^9}, {3.685197967212616*^9, 3.685197971778261*^9}, 
   3.690450153369383*^9, {3.690985408230434*^9, 3.690985450925138*^9}, 
   3.690986337396821*^9, 
   3.7093749413308277`*^9},ExpressionUUID->"0f239192-8601-4416-b44c-\
b94c7270a669"],

Cell["\<\
also strip out any implemented BackgroundGenerator and ExpandRepeatedEvents \
instructions which have already been implemented\
\>", "Text",
 CellChangeTimes->{{3.69098618783429*^9, 3.690986211241982*^9}, {
   3.690986288924268*^9, 3.6909862951934967`*^9}, {3.690986366802268*^9, 
   3.690986368135908*^9}, 
   3.709374942239315*^9},ExpressionUUID->"3c6dbb7c-1cbf-41e4-a75d-\
67364374d78a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventList", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "False"}], "}"}], ",", "chars_List", ",", 
      RowBox[{"more", ":", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Except", "[", "_Rule", "]"}], "..."}], ")"}]}], ",", 
      "opts___Rule"}], "]"}], "/;", 
    RowBox[{
     RowBox[{"FreeQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"chars", ",", "more"}], "}"}], ",", "Undefined"}], "]"}], "&&", 
     RowBox[{"MatchQ", "[", 
      RowBox[{"chars", ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", 
         RowBox[{"{", 
          RowBox[{"__", ",", "True"}], "}"}], ",", "_", ",", 
         RowBox[{"{", 
          RowBox[{"__", ",", "True"}], "}"}], ",", "___"}], "}"}]}], 
      "]"}]}]}], ":=", 
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"True", ",", "False"}], "}"}], ",", "chars", ",", "more", ",", 
     RowBox[{"Sequence", "@@", 
      RowBox[{"FilterRules", "[", 
       RowBox[{
        RowBox[{"{", "opts", "}"}], ",", 
        RowBox[{"Except", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"BackgroundGenerator", "|", "ExpandRepeatedEvents"}], 
           ")"}], "\[Rule]", "_"}], "]"}]}], "\[IndentingNewLine]", "]"}]}]}],
     "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6839586027648153`*^9, 3.6839586045846977`*^9}, {
   3.683958667580065*^9, 3.683958711018367*^9}, 3.683959521095401*^9, {
   3.68396328628866*^9, 3.68396333632965*^9}, {3.68398203580695*^9, 
   3.68398203902606*^9}, {3.6850335395317183`*^9, 3.685033539533765*^9}, {
   3.685197975031987*^9, 3.6851979927600737`*^9}, {3.690393484790226*^9, 
   3.690393485203662*^9}, {3.6904026546749983`*^9, 3.6904026557814407`*^9}, {
   3.690402714194106*^9, 3.690402730606637*^9}, 3.690450163749612*^9, {
   3.690985454767144*^9, 3.690985485067655*^9}, 3.6909859043253098`*^9, {
   3.690986319355239*^9, 3.690986322106587*^9}, 
   3.709374943008534*^9},ExpressionUUID->"9889d07c-6457-4d9e-a3eb-\
86579fbcfbc4"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["NumericalComputationReadyQ", "Section",
 CellChangeTimes->{{3.6839769345684147`*^9, 3.6839769370653973`*^9}, {
   3.6839769827104883`*^9, 3.6839769835869827`*^9}, 
   3.683982641875346*^9},ExpressionUUID->"4c184bd5-8098-4e10-b4ee-\
e12e4ca6207d"],

Cell[CellGroupData[{

Cell["insert", "Subsubsection",
 CellChangeTimes->{{3.683982649959613*^9, 
  3.68398265260308*^9}},ExpressionUUID->"b7efa55f-e5d0-451d-a07e-\
1870341fbfed"],

Cell["\<\
set numeric status to True if all contained events have True numeric status\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.683958727456455*^9, 3.6839587370191107`*^9}, {
  3.6839587793697453`*^9, 3.683958781370214*^9}, {3.683963257225411*^9, 
  3.683963281087583*^9}, {3.683963372862555*^9, 3.683963411440543*^9}, {
  3.683964593734514*^9, 3.683964602800449*^9}, {3.683982136140542*^9, 
  3.683982156411853*^9}},ExpressionUUID->"c49512d1-df7e-4743-ae21-\
e79013799bef"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventList", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"True", ",", "False"}], "}"}], ",", "chars_List", ",", 
      "events_List", ",", "opts___Rule"}], "]"}], "/;", "\[IndentingNewLine]", 
    RowBox[{"TrueQ", "[", 
     RowBox[{"And", "@@", 
      RowBox[{"(", 
       RowBox[{"NumericalComputationReadyQ", "/@", "events"}], ")"}]}], 
     "]"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"True", ",", "True"}], "}"}], ",", "chars", ",", "events", ",", 
     "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.68396418471384*^9, 3.683964279298118*^9}, {
  3.6839649380405807`*^9, 3.683964939726199*^9}, {3.683982090937641*^9, 
  3.683982093601562*^9}, {3.683982162152418*^9, 3.6839823345003138`*^9}, {
  3.68398271563235*^9, 3.683982735284978*^9}, {3.683982861542894*^9, 
  3.683982894426715*^9}, {3.6850335395379972`*^9, 3.6850335395402412`*^9}, {
  3.685198126932831*^9, 3.6851981271662817`*^9}, {3.690438567888126*^9, 
  3.690438570284131*^9}},ExpressionUUID->"bc9ef455-f7c2-46da-b372-\
24f952a5e4fa"]
}, Closed]],

Cell[CellGroupData[{

Cell["extract", "Subsubsection",
 CellChangeTimes->{{3.683982649959613*^9, 
  3.68398266461884*^9}},ExpressionUUID->"2bbd4557-424d-4ba1-ae97-\
e850bd99d175"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NumericalComputationReadyQ", "[", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"False", ",", "num_"}], "}"}], ",", 
       RowBox[{"more", ":", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Except", "[", "_Rule", "]"}], "..."}], ")"}]}], ",", 
       "opts___Rule"}], "]"}], ",", "___Rule"}], "]"}], ":=", "False"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683982538783044*^9, 3.6839825780899067`*^9}, {
   3.6839826160631943`*^9, 3.683982620853909*^9}, 3.6839826877779427`*^9, 
   3.685033539544807*^9},ExpressionUUID->"6a6dc4c4-417c-4a32-8e50-\
7cbb52524dc9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NumericalComputationReadyQ", "[", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"True", ",", "num_"}], "}"}], ",", 
       RowBox[{"more", ":", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Except", "[", "_Rule", "]"}], "..."}], ")"}]}], ",", 
       "opts___Rule"}], "]"}], ",", "___Rule"}], "]"}], ":=", "num"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683982538783044*^9, 3.6839825780899067`*^9}, 
   3.6839826898567057`*^9, 
   3.685033539548975*^9},ExpressionUUID->"dbcce04f-876c-4e98-ab09-\
f8ed66187a09"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["apply to arguments", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, 
   3.683973033466785*^9, {3.683995717350603*^9, 3.683995720671194*^9}, {
   3.683997563601553*^9, 3.683997573622693*^9}, {3.683999046791951*^9, 
   3.6839990726296883`*^9}, {3.685034289222179*^9, 
   3.685034289499123*^9}},ExpressionUUID->"cbbb79a8-3c4d-440e-8561-\
cc0c82a43cee"],

Cell[CellGroupData[{

Cell["\<\
single raw (i.e. non-interpreted) argument: apply EventSpecifications\
\>", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.514738781326264*^9, 3.514738794407814*^9}, {
   3.545052507651114*^9, 3.545052523135462*^9}, 3.545052645598901*^9, {
   3.683728597721991*^9, 3.683728601679985*^9}, 3.683824888026587*^9, 
   3.683825236493738*^9, {3.6845007268874474`*^9, 3.6845007275152473`*^9}, {
   3.68469204776381*^9, 3.684692052603023*^9}, {3.685167949084659*^9, 
   3.68516795322388*^9}, {3.685169347033834*^9, 3.685169352628289*^9}, {
   3.6912361863455257`*^9, 3.691236187084941*^9}, {3.699208270839369*^9, 
   3.699208276354423*^9}},ExpressionUUID->"8fa8b7a9-d64f-4677-9262-\
7218bdb1f1ef"],

Cell["\<\
apply EventSpecifications but leave out any timing or evolution background \
information. These will be deployed in the level of EventList.\
\>", "Text",
 CellChangeTimes->{{3.6851686047676687`*^9, 3.6851686228911743`*^9}, {
  3.6851686925318327`*^9, 
  3.685168712746539*^9}},ExpressionUUID->"58879946-947a-4f4c-97e7-\
e8f6dd01ed65"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "False"}], "}"}], ",", "chars_", ",", 
     RowBox[{"arg", ":", 
      RowBox[{"Except", "[", 
       RowBox[{"_EventSpecifications", "|", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"_Event", "|", "_EventList"}], ")"}], ".."}], "}"}]}], 
       "]"}]}], ",", "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Verbose", "/.", 
         RowBox[{"{", "opts", "}"}]}], "/.", 
        RowBox[{"Options", "[", "EventList", "]"}]}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<applying EventList to raw argument: \>\"", ",", 
         RowBox[{
          RowBox[{"Expand", "[", "arg", "]"}], "/.", 
          RowBox[{"s_ShapeFunction", "\[RuleDelayed]", 
           RowBox[{"s", "\[LeftDoubleBracket]", 
            RowBox[{"{", "4", "}"}], "\[RightDoubleBracket]"}]}]}]}], "]"}]}],
       "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"EventList", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"False", ",", "False"}], "}"}], ",", "chars", ",", 
       "\[IndentingNewLine]", 
       RowBox[{"EventSpecifications", "[", 
        RowBox[{"arg", ",", "\[IndentingNewLine]", 
         RowBox[{"SimplifyRules", "[", "\[IndentingNewLine]", 
          RowBox[{"Sequence", "@@", 
           RowBox[{"Cases", "[", 
            RowBox[{
             RowBox[{"{", "opts", "}"}], ",", 
             RowBox[{"Except", "[", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                "InitialTimePoint", "|", "FinalTimePoint", "|", 
                 "BackgroundGenerator", "|", "Chronology"}], ")"}], "\[Rule]",
                "_"}], "]"}]}], "\[IndentingNewLine]", "]"}]}], 
          "\[IndentingNewLine]", "]"}]}], "]"}], ",", "\[IndentingNewLine]", 
       "opts"}], "\[IndentingNewLine]", "]"}]}], ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5144539068637877`*^9, 3.514453917622925*^9}, 
   3.5144562170712967`*^9, {3.683367787008257*^9, 3.683367806267187*^9}, 
   3.6839576812160807`*^9, {3.6846920070224867`*^9, 3.684692031666548*^9}, {
   3.685167883339087*^9, 3.685167922748328*^9}, {3.685167964937786*^9, 
   3.685167979538583*^9}, 3.685168183292129*^9, {3.685168224839028*^9, 
   3.685168231159745*^9}, {3.685168565607396*^9, 3.685168593033745*^9}, {
   3.690443714271566*^9, 3.690443739331502*^9}, {3.690443849081726*^9, 
   3.690443850918497*^9}, {3.690445836426656*^9, 3.690445839735813*^9}, {
   3.690445907512248*^9, 3.6904459207591248`*^9}, {3.690448757137682*^9, 
   3.6904487733708467`*^9}, 3.690983271036454*^9, {3.691232462145403*^9, 
   3.691232464865383*^9}, {3.691510210998925*^9, 3.691510216339655*^9}, {
   3.691583604615444*^9, 3.691583610626399*^9}, 3.69920467623925*^9, 
   3.709374943773864*^9},ExpressionUUID->"61b8a805-1e52-4277-af9d-\
53743b074a09"]
}, Closed]],

Cell[CellGroupData[{

Cell["EventSpecifications: generate list of Event objects", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683958534614377*^9, 3.683958551028832*^9}, {
   3.68395858577393*^9, 3.6839585874144278`*^9}, 3.683959406338385*^9, {
   3.683963242982894*^9, 3.6839632443419228`*^9}, {3.683964445776389*^9, 
   3.683964455294318*^9}, {3.683982931173691*^9, 3.68398295774756*^9}, {
   3.683983044466885*^9, 3.6839830502378283`*^9}, 3.683983403805419*^9, {
   3.6839838750085382`*^9, 3.683983875895556*^9}, {3.683999232794664*^9, 
   3.683999276614259*^9}, {3.685167755953803*^9, 3.6851677584494877`*^9}, {
   3.685169357090248*^9, 
   3.685169364796047*^9}},ExpressionUUID->"9434eb18-6306-4a03-8bc3-\
552f6021c694"],

Cell["\<\
pass any EventSpecification options into each contained event\
\>", "Text",
 CellChangeTimes->{{3.684214179386592*^9, 
  3.684214226480644*^9}},ExpressionUUID->"47ce46ff-1548-4a34-90cc-\
376cd655cf45"],

Cell["\<\
Leave out any timing or evolution background information. These will be \
deployed on the level of EventList.\
\>", "Text",
 CellChangeTimes->{{3.6851686047676687`*^9, 3.6851686228911743`*^9}, {
  3.6851686925318327`*^9, 3.685168735586772*^9}, {3.685190651561165*^9, 
  3.685190651817696*^9}},ExpressionUUID->"14da2ea3-0f50-4620-b460-\
67e48c1a3262"],

Cell["\<\
The Chronology information is handled at the level of EventSpecifications and \
is also stripped out\
\>", "Text",
 CellChangeTimes->{{3.6909832865148153`*^9, 3.690983317792346*^9}, {
  3.6909837765126877`*^9, 
  3.69098378185559*^9}},ExpressionUUID->"92484261-964d-4fa1-8d55-\
956b4b962dca"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "False"}], "}"}], ",", "chars_", ",", 
     RowBox[{"EventSpecifications", "[", 
      RowBox[{"True", ",", "evspecs_List", ",", "evspecopts___Rule"}], "]"}], 
     ",", "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"(", "\[IndentingNewLine]", 
    RowBox[{"EventList", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "False"}], "}"}], ",", "\[IndentingNewLine]", 
      "chars", ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"Event", "[", 
         RowBox[{"#", ",", "\[IndentingNewLine]", 
          RowBox[{"SimplifyRules", "[", "\[IndentingNewLine]", 
           RowBox[{"Sequence", "@@", 
            RowBox[{"Cases", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"evspecopts", ",", "opts"}], "}"}], ",", 
              RowBox[{"Except", "[", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                 "InitialTimePoint", "|", "FinalTimePoint", "|", 
                  "BackgroundGenerator", "|", "Chronology"}], ")"}], 
                "\[Rule]", "_"}], "]"}]}], "\[IndentingNewLine]", "]"}]}], 
           "\[IndentingNewLine]", "]"}]}], "]"}], "&"}], "/@", "evspecs"}], 
      ",", "\[IndentingNewLine]", 
      RowBox[{"SimplifyRules", "[", "\[IndentingNewLine]", 
       RowBox[{"Sequence", "@@", 
        RowBox[{"Cases", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"evspecopts", ",", "opts"}], "}"}], ",", 
          RowBox[{"Except", "[", 
           RowBox[{"Chronology", "\[Rule]", "_"}], "]"}]}], 
         "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}]}], 
     "\[IndentingNewLine]", "]"}], ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683983345176896*^9, 3.68398339313787*^9}, {
   3.6839992897574987`*^9, 3.6839993107078323`*^9}, 3.683999350679184*^9, {
   3.683999386116839*^9, 3.683999391351132*^9}, {3.6839994877530613`*^9, 
   3.683999510287216*^9}, {3.6842142051347303`*^9, 3.684214210629221*^9}, {
   3.684214242863926*^9, 3.6842142926131277`*^9}, {3.684214554084909*^9, 
   3.68421464827101*^9}, {3.684214787557776*^9, 3.684214803451076*^9}, {
   3.6850335395659637`*^9, 3.685033539568087*^9}, 3.685033569604122*^9, {
   3.690983239587098*^9, 3.690983243519144*^9}, {3.690983346050132*^9, 
   3.690983453203998*^9}, {3.6909836840350122`*^9, 3.690983747292481*^9}, {
   3.691510345561494*^9, 3.6915103735815697`*^9}, 3.6915833935344954`*^9, 
   3.709374944459588*^9},ExpressionUUID->"1b0e8082-25c4-4cb1-84b6-\
92dee5462f79"]
}, Closed]],

Cell[CellGroupData[{

Cell["absorb contained EventLists", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.514738781326264*^9, 3.514738794407814*^9}, {
   3.545052507651114*^9, 3.545052523135462*^9}, 3.545052645598901*^9, {
   3.683728597721991*^9, 3.683728601679985*^9}, 3.683824888026587*^9, 
   3.683825236493738*^9, {3.6845007268874474`*^9, 3.6845007275152473`*^9}, {
   3.68469204776381*^9, 3.684692052603023*^9}, {3.685167949084659*^9, 
   3.68516795322388*^9}, {3.685169347033834*^9, 3.685169352628289*^9}, {
   3.690446016138824*^9, 3.690446032737879*^9}, 3.690447239954175*^9, {
   3.690545905548209*^9, 
   3.690545907662828*^9}},ExpressionUUID->"c1e4a6f3-76d3-4011-897f-\
10e735b2f4cf"],

Cell["this can arise upon some expansions of Repeat", "Text",
 CellChangeTimes->{{3.690446036729986*^9, 3.6904460487928743`*^9}, 
   3.6928483572318707`*^9},ExpressionUUID->"3016d654-7efa-4531-84c8-\
9a5b2bc37b0c"],

Cell[CellGroupData[{

Cell["\<\
case where BackgroundGenerator has already been implemented in the contained \
EventLists\
\>", "Subsubsection",
 CellChangeTimes->{{3.690545393834996*^9, 3.6905454098549013`*^9}, {
   3.690545622226955*^9, 3.690545630256796*^9}, {3.690545871517002*^9, 
   3.690545920397769*^9}, 
   3.709374945347967*^9},ExpressionUUID->"80dbd4d1-c940-4b2e-b7a2-\
1bd6c0138b0d"],

Cell["\<\
ensure that all Repeat expansions have been done on the contained EventLists \
and that the BackgroundGenerator information of the contained EventLists is \
uniform. \
\>", "Text",
 CellChangeTimes->{{3.690446056168482*^9, 3.6904461132413683`*^9}, 
   3.690447257234901*^9, {3.690447342282113*^9, 3.6904473431997023`*^9}, {
   3.690543125058085*^9, 3.690543137351739*^9}, {3.690543213088643*^9, 
   3.690543213864381*^9}, {3.690545109353692*^9, 3.690545120117093*^9}, {
   3.690545192085585*^9, 3.6905452415743628`*^9}, {3.690545372833227*^9, 
   3.690545415468775*^9}, {3.690545648518647*^9, 3.690545701773889*^9}, 
   3.69284971229735*^9, 
   3.709374946188669*^9},ExpressionUUID->"d221f472-32a9-4803-861a-\
1f16b441b8ca"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{"evlistst_", ",", 
     RowBox[{"evlistch", ":", 
      RowBox[{"{", 
       RowBox[{"evlisttch_", ",", 
        RowBox[{"evlistbackgch", ":", 
         RowBox[{"{", 
          RowBox[{"backg_", ",", "backgimplreq_", ",", "backgimpl_"}], 
          "}"}]}], ",", "morech___"}], "}"}]}], ",", 
     RowBox[{"args", ":", 
      RowBox[{"{", 
       RowBox[{"___", ",", "_EventList", ",", "___"}], "}"}]}], ",", 
     "evlistopts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "eventlists", "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Verbose", "/.", 
            RowBox[{"{", "evlistopts", "}"}]}], "/.", 
           RowBox[{"Options", "[", "EventList", "]"}]}], ",", 
          RowBox[{"Print", "[", 
           RowBox[{
           "\"\<absorbing EventList contained in another EventList (case of \
uniform implemented BackgroundGenerator): \>\"", ",", 
            RowBox[{"Expand", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"backg", ",", "backgimplreq", ",", "backgimpl"}], 
               "}"}], "/.", 
              RowBox[{"s_ShapeFunction", "\[RuleDelayed]", 
               RowBox[{"s", "\[LeftDoubleBracket]", 
                RowBox[{"{", "4", "}"}], "\[RightDoubleBracket]"}]}]}], 
             "]"}]}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"EventList", "[", 
         RowBox[{"evlistst", ",", 
          RowBox[{"{", 
           RowBox[{"evlisttch", ",", 
            RowBox[{"{", 
             RowBox[{"backg", ",", "True", ",", "True"}], "}"}], ",", 
            "morech"}], "}"}], ",", "\[IndentingNewLine]", 
          RowBox[{"Replace", "[", 
           RowBox[{"args", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"EventList", "[", 
              RowBox[{"_", ",", "_", ",", 
               RowBox[{"evs", ":", 
                RowBox[{"{", 
                 RowBox[{"_Event", ".."}], "}"}]}], ",", "___"}], "]"}], 
             "\[RuleDelayed]", 
             RowBox[{"Sequence", "@@", "evs"}]}], ",", "\[IndentingNewLine]", 
            "1"}], "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
          "evlistopts"}], "\[IndentingNewLine]", "]"}]}], 
       "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"eventlists", "=", 
         RowBox[{"Cases", "[", 
          RowBox[{"args", ",", "_EventList"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"And", "@@", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"MatchQ", "[", 
             RowBox[{"#", ",", 
              RowBox[{"EventList", "[", 
               RowBox[{"_", ",", 
                RowBox[{"{", 
                 RowBox[{"_", ",", 
                  RowBox[{"{", 
                   RowBox[{"backg", ",", "_", ",", "True"}], "}"}], ",", "__",
                   ",", 
                  RowBox[{"True", "|", 
                   RowBox[{"{", 
                    RowBox[{"__", ",", "True"}], "}"}]}]}], "}"}], ",", 
                "___"}], "]"}]}], "]"}], "&"}], "/@", "eventlists"}], 
          ")"}]}]}], "\[IndentingNewLine]", ")"}]}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.690446124137162*^9, 3.690446233522373*^9}, {
   3.6904462939608707`*^9, 3.690446635378927*^9}, {3.6904466877841387`*^9, 
   3.690446769056583*^9}, {3.690446799099853*^9, 3.6904468605172358`*^9}, {
   3.69044726688463*^9, 3.690447272370254*^9}, {3.690447307197813*^9, 
   3.690447322948901*^9}, {3.690447357709384*^9, 3.6904473945539618`*^9}, {
   3.690447568756743*^9, 3.69044759462567*^9}, {3.69044878964844*^9, 
   3.690448796614476*^9}, 3.690545125521051*^9, {3.690545730588673*^9, 
   3.6905457850818987`*^9}, {3.6905459445228024`*^9, 3.690546014068275*^9}, {
   3.690546080986349*^9, 3.6905460814222317`*^9}, {3.691233181470536*^9, 
   3.6912332274271297`*^9}, {3.69123330964539*^9, 3.691233330725675*^9}, {
   3.6912333614752274`*^9, 3.6912333643973207`*^9}, 3.699204694495532*^9, 
   3.709374947137738*^9},ExpressionUUID->"0933c3ab-f4c0-434a-a3a7-\
4f6b351171dc"],

Cell["absorb a single contained EventList", "Text",
 CellChangeTimes->{{3.6910690540872602`*^9, 
  3.6910690630765667`*^9}},ExpressionUUID->"345d21f9-72c0-4b2a-82e3-\
8c573e52c5b7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{"evlistst_", ",", "evlistch_", ",", 
     RowBox[{"{", 
      RowBox[{"eventlist_EventList", ",", "___Rule"}], "}"}], ",", 
     "evlistopts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Verbose", "/.", 
         RowBox[{"{", "evlistopts", "}"}]}], "/.", 
        RowBox[{"Options", "[", "EventList", "]"}]}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<absorbing a single EventList into another: \>\"", ",", 
         RowBox[{"Expand", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"{", 
             RowBox[{"evlistch", ",", "eventlist"}], "}"}], "}"}], "/.", 
           RowBox[{"s_ShapeFunction", "\[RuleDelayed]", 
            RowBox[{"s", "\[LeftDoubleBracket]", 
             RowBox[{"{", "4", "}"}], "\[RightDoubleBracket]"}]}]}], "]"}]}], 
        "]"}]}], "]"}], ";", "\[IndentingNewLine]", "eventlist"}], 
    "\[IndentingNewLine]", ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.690446124137162*^9, 3.690446233522373*^9}, {
   3.6904462939608707`*^9, 3.690446635378927*^9}, {3.6904466877841387`*^9, 
   3.690446769056583*^9}, {3.690446799099853*^9, 3.6904468605172358`*^9}, {
   3.69044726688463*^9, 3.690447272370254*^9}, {3.690447307197813*^9, 
   3.690447322948901*^9}, {3.690447357709384*^9, 3.6904473945539618`*^9}, {
   3.690447568756743*^9, 3.69044759462567*^9}, {3.69044878964844*^9, 
   3.690448796614476*^9}, 3.690545125521051*^9, {3.690545730588673*^9, 
   3.6905457850818987`*^9}, {3.6905459445228024`*^9, 3.690546014068275*^9}, {
   3.690546080986349*^9, 3.6905460814222317`*^9}, {3.691068956058672*^9, 
   3.691068975361786*^9}, {3.691069068665367*^9, 3.691069110921434*^9}, {
   3.691069214268504*^9, 3.69106922310461*^9}, {3.69123406052879*^9, 
   3.6912341364435863`*^9}, 
   3.6992047040685873`*^9},ExpressionUUID->"1fc8cc6a-cb1e-4fc4-94ee-\
e35408d28ada"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
case where BackgroundGenerator has not been implemented in the contained \
EventLists\
\>", "Subsubsection",
 CellChangeTimes->{{3.690545393834996*^9, 3.6905454098549013`*^9}, {
   3.690545622226955*^9, 3.690545630256796*^9}, {3.690545871517002*^9, 
   3.690545920397769*^9}, {3.690546035037993*^9, 3.6905460354756737`*^9}, 
   3.709374947934339*^9},ExpressionUUID->"85e13c3d-c35c-4e2e-9e7e-\
aebfb108a313"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{"evlistst_", ",", 
     RowBox[{"evlistch", ":", 
      RowBox[{"{", 
       RowBox[{"evlisttch_", ",", 
        RowBox[{"evlistbackgch", ":", 
         RowBox[{"{", 
          RowBox[{"backg_", ",", "backgimplreq_", ",", "backgimpl_"}], 
          "}"}]}], ",", "morech___"}], "}"}]}], ",", 
     RowBox[{"args", ":", 
      RowBox[{"{", 
       RowBox[{"___", ",", "_EventList", ",", "___"}], "}"}]}], ",", 
     "evlistopts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "eventlists", "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Verbose", "/.", 
            RowBox[{"{", "evlistopts", "}"}]}], "/.", 
           RowBox[{"Options", "[", "EventList", "]"}]}], ",", 
          RowBox[{"Print", "[", 
           RowBox[{
           "\"\<absorbing EventList contained in another EventList (case of \
non-implemented BackgroundGenerator): \>\"", ",", " ", 
            RowBox[{"Expand", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"backg", ",", "backgimplreq", ",", "backgimpl"}], 
               "}"}], "/.", 
              RowBox[{"s_ShapeFunction", "\[RuleDelayed]", 
               RowBox[{"s", "\[LeftDoubleBracket]", 
                RowBox[{"{", "4", "}"}], "\[RightDoubleBracket]"}]}]}], 
             "]"}]}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"EventList", "[", 
         RowBox[{"evlistst", ",", 
          RowBox[{"{", 
           RowBox[{"evlisttch", ",", "evlistbackgch", ",", "morech"}], "}"}], 
          ",", "\[IndentingNewLine]", 
          RowBox[{"Replace", "[", 
           RowBox[{"args", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"EventList", "[", 
              RowBox[{"_", ",", "_", ",", 
               RowBox[{"evs", ":", 
                RowBox[{"{", 
                 RowBox[{"_Event", ".."}], "}"}]}], ",", "___"}], "]"}], 
             "\[RuleDelayed]", 
             RowBox[{"Sequence", "@@", "evs"}]}], ",", "\[IndentingNewLine]", 
            "1"}], "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
          "evlistopts"}], "\[IndentingNewLine]", "]"}]}], 
       "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"eventlists", "=", 
         RowBox[{"Cases", "[", 
          RowBox[{"args", ",", "_EventList"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"And", "@@", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"MatchQ", "[", 
             RowBox[{"#", ",", 
              RowBox[{"EventList", "[", 
               RowBox[{"_", ",", 
                RowBox[{"{", 
                 RowBox[{"_", ",", 
                  RowBox[{"{", 
                   RowBox[{"_", ",", "_", ",", 
                    RowBox[{"False", "|", "Undefined"}]}], "}"}], ",", "__", 
                  ",", 
                  RowBox[{"True", "|", 
                   RowBox[{"{", 
                    RowBox[{"__", ",", "True"}], "}"}]}]}], "}"}], ",", 
                "___"}], "]"}]}], "]"}], "&"}], "/@", "eventlists"}], 
          ")"}]}]}], "\[IndentingNewLine]", ")"}]}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.690446124137162*^9, 3.690446233522373*^9}, {
   3.6904462939608707`*^9, 3.690446635378927*^9}, {3.6904466877841387`*^9, 
   3.690446769056583*^9}, {3.690446799099853*^9, 3.6904468605172358`*^9}, {
   3.69044726688463*^9, 3.690447272370254*^9}, {3.690447307197813*^9, 
   3.690447322948901*^9}, {3.690447357709384*^9, 3.6904473945539618`*^9}, {
   3.690447568756743*^9, 3.69044759462567*^9}, {3.69044878964844*^9, 
   3.690448796614476*^9}, 3.690545125521051*^9, {3.690545730588673*^9, 
   3.6905457850818987`*^9}, {3.6905459445228024`*^9, 3.69054607516712*^9}, {
   3.691233253201291*^9, 3.691233261334598*^9}, {3.691233352688559*^9, 
   3.691233369441098*^9}, 3.699204711428273*^9, 
   3.709374948735203*^9},ExpressionUUID->"efbddada-95cf-4cd6-b93c-\
10283d1eea5a"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["convert empty event list to null Event", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683958534614377*^9, 3.683958551028832*^9}, {
   3.68395858577393*^9, 3.6839585874144278`*^9}, 3.683959406338385*^9, {
   3.683963242982894*^9, 3.6839632443419228`*^9}, {3.683964445776389*^9, 
   3.683964455294318*^9}, {3.683982931173691*^9, 3.68398295774756*^9}, {
   3.683983044466885*^9, 3.6839830502378283`*^9}, 3.683983403805419*^9, {
   3.6839838750085382`*^9, 3.683983875895556*^9}, {3.683999232794664*^9, 
   3.683999276614259*^9}, {3.685167755953803*^9, 3.6851677584494877`*^9}, {
   3.685169357090248*^9, 3.685169364796047*^9}, {3.689423026107966*^9, 
   3.689423028474004*^9}, {3.689423109016385*^9, 
   3.6894231157915583`*^9}},ExpressionUUID->"d7b31436-2529-4afe-abbc-\
73e2ae08a046"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{"status_", ",", "chars_", ",", 
     RowBox[{"{", "}"}], ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"EventList", "[", 
    RowBox[{"status", ",", "chars", ",", 
     RowBox[{"{", 
      RowBox[{"None", ",", "0"}], "}"}], ",", "opts"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.689423049908082*^9, 
  3.6894230828084173`*^9}},ExpressionUUID->"c9bb266f-d141-4299-bfb7-\
a706c37c49bf"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["timing", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, 
   3.683973033466785*^9},ExpressionUUID->"9506b8f2-d011-41cb-879f-\
0648bd8689ad"],

Cell[CellGroupData[{

Cell["duration", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 
  3.683972657724471*^9}},ExpressionUUID->"d9acc479-969d-4102-99ae-\
71d3cca0dc7d"],

Cell[CellGroupData[{

Cell["insert", "Subsubsection",
 CellChangeTimes->{{3.683979726739832*^9, 
  3.6839797551918573`*^9}},ExpressionUUID->"555a7783-9304-4e53-8bd9-\
647236a33075"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Undefined", ",", "moretimingch___"}], "}"}], ",", 
       "morech___"}], "}"}], ",", "events_List", ",", "opts___Rule"}], "]"}], 
   ":=", "\[IndentingNewLine]", 
   RowBox[{"(", 
    RowBox[{"EventList", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "numst"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Plus", "@@", 
           RowBox[{"(", 
            RowBox[{"EventDuration", "/@", "events"}], ")"}]}], ",", 
          "moretimingch"}], "}"}], ",", "morech"}], "}"}], ",", "events", ",",
       "opts"}], "]"}], ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.683980767477813*^9, 3.683980892077633*^9}, {3.6839835771899776`*^9, 
   3.683983598545579*^9}, {3.683983641039324*^9, 3.683983668572143*^9}, {
   3.685033539572385*^9, 3.685033539574477*^9}, {3.691510510851687*^9, 
   3.691510535863675*^9}, {3.691583405989126*^9, 3.691583407738287*^9}, 
   3.7117164232682204`*^9},ExpressionUUID->"5c68a882-48a8-43a2-97db-\
5f5043e5f370"]
}, Closed]],

Cell[CellGroupData[{

Cell["extract", "Subsubsection",
 CellChangeTimes->{{3.683979726739832*^9, 
  3.6839797455615473`*^9}},ExpressionUUID->"4ae39db0-7e99-43fc-8c8b-\
edaf59d01898"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventDuration", "[", 
    RowBox[{"EventList", "[", 
     RowBox[{"status_", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"dur_", ",", "moretimingch___"}], "}"}], ",", "morech___"}], 
       "}"}], ",", "evspec_", ",", "opts___Rule"}], "]"}], "]"}], ":=", 
   "dur"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683979669007251*^9, 3.683979709514778*^9}, {
   3.6839800680331697`*^9, 3.6839800796436453`*^9}, 3.6839836750186234`*^9, 
   3.6850335395786943`*^9, 
   3.7117164232911625`*^9},ExpressionUUID->"c5ebbe6c-6bbb-466f-b1b0-\
57f0372b4877"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["InitialTimePoint", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
  3.683972979124955*^9, 3.683972983330543*^9}, {3.683983989832539*^9, 
  3.683984004603756*^9}},ExpressionUUID->"22ab5e41-8e65-497c-8722-\
42baa799d6d0"],

Cell[CellGroupData[{

Cell["insert", "Subsubsection",
 CellChangeTimes->{{3.683979764654413*^9, 
  3.683979767388132*^9}},ExpressionUUID->"cf5daa2a-3f7e-40bf-b9db-\
f36bab98a6a9"],

Cell["\<\
insert InitialTimePoint and calculate all subsequent event intervals\
\>", "Text",
 CellChangeTimes->{{3.683984011523447*^9, 
  3.683984046906507*^9}},ExpressionUUID->"15a72b31-1757-4922-8167-\
4d75c001900a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"dur_", ",", "Undefined", ",", "moretimingch___"}], "}"}], 
       ",", "morech___"}], "}"}], ",", 
     RowBox[{"events", ":", 
      RowBox[{"{", 
       RowBox[{"_Event", "..."}], "}"}]}], ",", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ta", ",", "tb", ",", "tavals", ",", "modevents"}], "}"}], ",", 
     
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"tavals", "=", 
         RowBox[{"Most", "[", 
          RowBox[{"FoldList", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{"#1", "+", "#2"}], ")"}], "&"}], ",", "ta", ",", 
            RowBox[{"EventDuration", "/@", 
             RowBox[{"Reverse", "@", "events"}]}]}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"modevents", "=", 
         RowBox[{"Reverse", "@", 
          RowBox[{"MapThread", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{"#1", "/.", 
               RowBox[{
                RowBox[{"Event", "[", 
                 RowBox[{
                  RowBox[{"evspecs", ":", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Except", "[", "_Rule", "]"}], ".."}], ")"}]}], 
                  ",", "evspecopts___Rule"}], "]"}], "\[RuleDelayed]", 
                RowBox[{"Event", "[", 
                 RowBox[{"evspecs", ",", 
                  RowBox[{"SimplifyRules", "[", 
                   RowBox[{
                    RowBox[{"InitialTimePoint", "\[Rule]", "#2"}], ",", 
                    RowBox[{"FinalTimePoint", "\[Rule]", "Automatic"}], ",", 
                    "evspecopts"}], "]"}]}], "]"}]}]}], "\[IndentingNewLine]",
               ")"}], "&"}], ",", "\[IndentingNewLine]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Reverse", "@", "events"}], ",", "tavals"}], "}"}]}], 
           "\[IndentingNewLine]", "]"}]}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"EventList", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"False", ",", "numst"}], "}"}], ",", 
          RowBox[{"{", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"dur", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"FinalTimePoint", "[", 
                 RowBox[{"First", "@", "modevents"}], "]"}], ",", "ta"}], 
               "}"}], ",", "moretimingch"}], "}"}], ",", "morech"}], "}"}], 
          ",", "\[IndentingNewLine]", "modevents", ",", 
          RowBox[{"Sequence", "@@", 
           RowBox[{"Cases", "[", 
            RowBox[{
             RowBox[{"{", "opts", "}"}], ",", 
             RowBox[{"Except", "[", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"FinalTimePoint", "|", "InitialTimePoint"}], ")"}], 
               "\[Rule]", "_"}], "]"}]}], "]"}]}]}], "]"}]}], 
       "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"tb", ",", "ta"}], "}"}], "=", 
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{"FinalTimePoint", ",", "InitialTimePoint"}], "}"}], "/.", 
           
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "EventList", "]"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"MatchQ", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"tb", ",", "ta"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Automatic", "|", "Undefined"}], ",", 
            RowBox[{"Except", "[", 
             RowBox[{"Automatic", "|", "Undefined"}], "]"}]}], "}"}]}], 
         "]"}]}], "\[IndentingNewLine]", ")"}]}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839730673310833`*^9, 3.683973101981674*^9}, {3.683973149371945*^9, 
   3.6839734156761*^9}, {3.683973451960537*^9, 3.683973475698845*^9}, {
   3.6839736462618427`*^9, 3.683973679564999*^9}, {3.68397413593371*^9, 
   3.683974154427085*^9}, {3.6839804964580173`*^9, 3.683980530614625*^9}, {
   3.683984164369474*^9, 3.683984178295782*^9}, {3.683984226867147*^9, 
   3.683984259073943*^9}, {3.683984318412486*^9, 3.683984411262455*^9}, {
   3.683984553922329*^9, 3.683984594795735*^9}, {3.683984657396819*^9, 
   3.683984888406836*^9}, {3.683984940094124*^9, 3.683985001420158*^9}, {
   3.6839851076626663`*^9, 3.683985111814556*^9}, {3.683985227066613*^9, 
   3.683985254788437*^9}, {3.683985305923213*^9, 3.683985312694499*^9}, 
   3.6839853441331177`*^9, {3.683985480077479*^9, 3.683985610546199*^9}, {
   3.683985853237884*^9, 3.683985853820777*^9}, 3.683986123201687*^9, {
   3.685033539583531*^9, 3.685033539591022*^9}, {3.685033569609044*^9, 
   3.685033569614585*^9}, {3.689425284738627*^9, 3.689425294679872*^9}, 
   3.689624912208839*^9, {3.6915105468435993`*^9, 3.6915105868792677`*^9}, {
   3.6915804190927687`*^9, 3.691580435656695*^9}, {3.6915806666035757`*^9, 
   3.6915806684635353`*^9}, {3.69158341461417*^9, 3.69158342069838*^9}, 
   3.711716423298194*^9},ExpressionUUID->"79986373-2677-4506-8983-\
1fb471427dfe"]
}, Closed]],

Cell[CellGroupData[{

Cell["extract", "Subsubsection",
 CellChangeTimes->{{3.683979764654413*^9, 
  3.683979774036868*^9}},ExpressionUUID->"535cd45c-7da4-4c44-b1e0-\
86412b59d275"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"InitialTimePoint", "[", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"dur_", ",", "Undefined", ",", "moretimingch___"}], "}"}], 
         ",", "morech___"}], "}"}], ",", "events_List", ",", "opts___Rule"}], 
      "]"}], ",", "___Rule"}], "]"}], ":=", "Undefined"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683979781879389*^9, 3.683979821255773*^9}, {
   3.6839840698205442`*^9, 3.683984075416164*^9}, {3.683999833127915*^9, 
   3.68399986646821*^9}, 
   3.6850335395970087`*^9},ExpressionUUID->"523cc1dc-9f3c-4fda-af66-\
24cb20ff2e45"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"InitialTimePoint", "[", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"dur_", ",", 
           RowBox[{"{", 
            RowBox[{"tb_", ",", "ta_"}], "}"}], ",", "moretimingch___"}], 
          "}"}], ",", "morech___"}], "}"}], ",", "events_List", ",", 
       "opts___Rule"}], "]"}], ",", "___Rule"}], "]"}], ":=", "ta"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683979781879389*^9, 3.683979821255773*^9}, {
   3.6839840698205442`*^9, 3.683984075416164*^9}, {3.683999835364059*^9, 
   3.6839998429831333`*^9}, 
   3.685033539602604*^9},ExpressionUUID->"7904ee9b-6dcb-4288-a634-\
d9d6ceda4b3d"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FinalTimePoint", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
  3.683972979124955*^9, 3.683972983330543*^9}, {3.683983989832539*^9, 
  3.683984004603756*^9}, {3.683984085958849*^9, 
  3.683984086666649*^9}},ExpressionUUID->"52484f4b-2586-4e7d-af35-\
ec00bf925438"],

Cell[CellGroupData[{

Cell["insert", "Subsubsection",
 CellChangeTimes->{{3.683979764654413*^9, 
  3.683979767388132*^9}},ExpressionUUID->"0d8d6514-a5ba-4af5-8015-\
2dc9ec0f92fa"],

Cell["\<\
insert FinalTimePoint and insert all previous event intervals into the events\
\
\>", "Text",
 CellChangeTimes->{{3.683984011523447*^9, 3.683984046906507*^9}, {
  3.683984114281218*^9, 3.683984118270114*^9}, {3.683984304674389*^9, 
  3.683984314100296*^9}},ExpressionUUID->"8827f450-56cf-4055-8d76-\
9b812cd1914d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"dur_", ",", "Undefined", ",", "moretimingch___"}], "}"}], 
       ",", "morech___"}], "}"}], ",", 
     RowBox[{"events", ":", 
      RowBox[{"{", 
       RowBox[{"_Event", "..."}], "}"}]}], ",", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ta", ",", "tb", ",", "tbvals", ",", "modevents"}], "}"}], ",", 
     
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"tbvals", "=", 
         RowBox[{"Most", "[", 
          RowBox[{"FoldList", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{"#1", "-", "#2"}], ")"}], "&"}], ",", "tb", ",", 
            RowBox[{"EventDuration", "/@", "events"}]}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"modevents", "=", 
         RowBox[{"MapThread", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"#1", "/.", 
              RowBox[{
               RowBox[{"Event", "[", 
                RowBox[{
                 RowBox[{"evspecs", ":", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"Except", "[", "_Rule", "]"}], ".."}], ")"}]}], 
                 ",", "evspecopts___Rule"}], "]"}], "\[RuleDelayed]", 
               RowBox[{"Event", "[", 
                RowBox[{"evspecs", ",", 
                 RowBox[{"SimplifyRules", "[", 
                  RowBox[{
                   RowBox[{"FinalTimePoint", "\[Rule]", "#2"}], ",", 
                   RowBox[{"InitialTimePoint", "\[Rule]", "Automatic"}], ",", 
                   "evspecopts"}], "]"}]}], "]"}]}]}], "\[IndentingNewLine]", 
             ")"}], "&"}], ",", "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{"events", ",", "tbvals"}], "}"}]}], "\[IndentingNewLine]",
           "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"EventList", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"False", ",", "numst"}], "}"}], ",", 
          RowBox[{"{", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"dur", ",", 
              RowBox[{"{", 
               RowBox[{"tb", ",", 
                RowBox[{"InitialTimePoint", "[", 
                 RowBox[{"Last", "@", "modevents"}], "]"}]}], "}"}], ",", 
              "moretimingch"}], "}"}], ",", "morech"}], "}"}], ",", 
          "\[IndentingNewLine]", "modevents", ",", 
          RowBox[{"Sequence", "@@", 
           RowBox[{"Cases", "[", 
            RowBox[{
             RowBox[{"{", "opts", "}"}], ",", 
             RowBox[{"Except", "[", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"FinalTimePoint", "|", "InitialTimePoint"}], ")"}], 
               "\[Rule]", "_"}], "]"}]}], "]"}]}]}], "]"}]}], 
       "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"tb", ",", "ta"}], "}"}], "=", 
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{"FinalTimePoint", ",", "InitialTimePoint"}], "}"}], "/.", 
           
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "EventList", "]"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"MatchQ", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"tb", ",", "ta"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Except", "[", 
             RowBox[{"Automatic", "|", "Undefined"}], "]"}], ",", 
            RowBox[{"Automatic", "|", "Undefined"}]}], "}"}]}], "]"}]}], 
       "\[IndentingNewLine]", ")"}]}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839730673310833`*^9, 3.683973101981674*^9}, {3.683973149371945*^9, 
   3.6839734156761*^9}, {3.683973451960537*^9, 3.683973475698845*^9}, {
   3.6839736462618427`*^9, 3.683973679564999*^9}, {3.68397413593371*^9, 
   3.683974154427085*^9}, {3.6839804964580173`*^9, 3.683980530614625*^9}, {
   3.683984164369474*^9, 3.683984178295782*^9}, {3.683984226867147*^9, 
   3.683984259073943*^9}, {3.683984318412486*^9, 3.683984411262455*^9}, {
   3.683984553922329*^9, 3.683984594795735*^9}, {3.683984657396819*^9, 
   3.683984888406836*^9}, {3.683984940094124*^9, 3.683985001420158*^9}, {
   3.6839851076626663`*^9, 3.683985111814556*^9}, {3.683985227066613*^9, 
   3.683985254788437*^9}, {3.683985305923213*^9, 3.683985312694499*^9}, 
   3.6839853441331177`*^9, {3.6850335396075068`*^9, 3.685033539613379*^9}, {
   3.685033569619466*^9, 3.685033569624937*^9}, {3.689425300746578*^9, 
   3.689425305976383*^9}, 3.689624918432572*^9, 
   3.711716423314827*^9},ExpressionUUID->"b01abfc3-af02-43f9-bcab-\
a6bfcc2908c3"]
}, Closed]],

Cell[CellGroupData[{

Cell["extract", "Subsubsection",
 CellChangeTimes->{{3.683979764654413*^9, 
  3.683979774036868*^9}},ExpressionUUID->"709fddac-9c08-4f6b-8b91-\
71305e288a87"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FinalTimePoint", "[", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"dur_", ",", "Undefined", ",", "moretimingch___"}], "}"}], 
         ",", "morech___"}], "}"}], ",", "events_List", ",", "opts___Rule"}], 
      "]"}], ",", "___Rule"}], "]"}], ":=", "Undefined"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683979781879389*^9, 3.683979821255773*^9}, {
   3.6839840698205442`*^9, 3.683984105119051*^9}, {3.68399981064543*^9, 
   3.683999816062976*^9}, 
   3.6850335396176023`*^9},ExpressionUUID->"a7ad67e0-3e2b-4673-958d-\
d8b1e1223951"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FinalTimePoint", "[", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"dur_", ",", 
           RowBox[{"{", 
            RowBox[{"tb_", ",", "ta_"}], "}"}], ",", "moretimingch___"}], 
          "}"}], ",", "morech___"}], "}"}], ",", "events_List", ",", 
       "opts___Rule"}], "]"}], ",", "___Rule"}], "]"}], ":=", "tb"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683979781879389*^9, 3.683979821255773*^9}, {
   3.6839840698205442`*^9, 3.683984105119051*^9}, 
   3.685033539623046*^9},ExpressionUUID->"1c8ca20c-83ae-45bf-8b28-\
b4aa222b1ea3"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["trap time point conflicts", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
  3.683972979124955*^9, 3.683972983330543*^9}, {3.683983989832539*^9, 
  3.683984004603756*^9}, {3.683984085958849*^9, 3.683984086666649*^9}, {
  3.683985420049254*^9, 
  3.683985458290825*^9}},ExpressionUUID->"93a08745-3a42-4f82-9abd-\
8e78af36f664"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"dur_", ",", 
         RowBox[{"{", 
          RowBox[{"tb_", ",", "ta_"}], "}"}], ",", "moretimingch___"}], "}"}],
        ",", "morech___"}], "}"}], ",", "events_", ",", "opts___Rule"}], 
    "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"newtb", ",", "newta"}], "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Message", "[", 
         RowBox[{"EventList", "::", "tconflict"}], "]"}], ";", "$Failed"}], 
       "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"newtb", ",", "newta"}], "}"}], "=", 
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{"FinalTimePoint", ",", "InitialTimePoint"}], "}"}], "/.", 
           
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "EventList", "]"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"TrueQ", "[", 
          RowBox[{
           RowBox[{"Chop", "[", 
            RowBox[{"newtb", "-", "tb"}], "]"}], "\[NotEqual]", "0"}], "]"}], 
         "||", 
         RowBox[{"TrueQ", "[", 
          RowBox[{
           RowBox[{"Chop", "[", 
            RowBox[{"newta", "-", "ta"}], "]"}], "\[NotEqual]", "0"}], 
          "]"}]}]}], "\[IndentingNewLine]", ")"}]}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6839805406968193`*^9, 3.683980682667598*^9}, {
   3.6839850361782*^9, 3.6839850462672367`*^9}, 3.6839862301513853`*^9, 
   3.683986354157617*^9, 3.683986435620257*^9, {3.68503353962991*^9, 
   3.6850335396350718`*^9}},ExpressionUUID->"262f99ea-a4e9-4646-bb58-\
6c6078cecc69"]
}, Closed]],

Cell[CellGroupData[{

Cell["permitted time shift", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683972979124955*^9, 3.683972983330543*^9}, {3.683975121606881*^9, 
   3.683975124294643*^9}, {3.683986600465363*^9, 3.683986611117754*^9}, {
   3.683986726263802*^9, 3.68398672786756*^9}, 
   3.684000383682273*^9},ExpressionUUID->"2f9ae897-6433-4e52-b270-\
be4027042cd0"],

Cell[CellGroupData[{

Cell["insert", "Subsubsection",
 CellChangeTimes->{{3.683979764654413*^9, 
  3.683979767388132*^9}},ExpressionUUID->"5f239cca-c413-4b58-88b5-\
e802931ffa10"],

Cell["\<\
implement only if BackgroundGenerator has been defined and implemented in all \
elements\
\>", "Text",
 CellChangeTimes->{{3.683975126804194*^9, 3.6839751371992807`*^9}, {
   3.689624556647335*^9, 3.689624559284821*^9}, {3.6896246058127813`*^9, 
   3.689624612129286*^9}, 
   3.709374949511231*^9},ExpressionUUID->"2d25d461-c39d-4a72-b1a9-\
2ebe26c82384"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
        "dur_", ",", "timepts_", ",", "Undefined", ",", "moretimingch___"}], 
        "}"}], ",", 
       RowBox[{"backgch", ":", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Except", "[", "Undefined", "]"}], ",", "_", ",", "True"}], 
         "}"}]}], ",", "morech___"}], "}"}], ",", "events_", ",", 
     "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "pts", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"pts", "=", 
        RowBox[{"PermittedTimeShift", "[", 
         RowBox[{"Sequence", "@@", "events"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"EventList", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"False", ",", "numst"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"dur", ",", "timepts", ",", "pts", ",", "moretimingch"}], 
            "}"}], ",", "backgch", ",", "morech"}], "}"}], ",", "events", ",",
          "opts"}], "]"}]}], "\[IndentingNewLine]", ")"}]}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839730673310833`*^9, 3.683973101981674*^9}, {3.683973149371945*^9, 
   3.6839734156761*^9}, {3.683973451960537*^9, 3.683973475698845*^9}, {
   3.6839736462618427`*^9, 3.683973679564999*^9}, {3.68397413593371*^9, 
   3.683974154427085*^9}, {3.683975165827835*^9, 3.683975303283432*^9}, {
   3.683975518769175*^9, 3.683975550989229*^9}, {3.683975611081381*^9, 
   3.683975614758295*^9}, {3.683975718424601*^9, 3.6839757265040693`*^9}, {
   3.68398632843747*^9, 3.683986347570067*^9}, {3.683986378927763*^9, 
   3.6839863958492317`*^9}, 3.6839864262374277`*^9, {3.685033539639579*^9, 
   3.685033539641858*^9}, {3.68942545604743*^9, 3.6894254581975813`*^9}, {
   3.6896245955973253`*^9, 3.689624598362671*^9}, {3.689624628925419*^9, 
   3.689624662313889*^9}, {3.6915106841260843`*^9, 3.691510693331688*^9}, 
   3.691512377128529*^9, 
   3.691583425325103*^9},ExpressionUUID->"b3290a6e-d380-4cb5-a790-\
a2b39eda8ab9"]
}, Closed]],

Cell[CellGroupData[{

Cell["extract", "Subsubsection",
 CellChangeTimes->{{3.683979764654413*^9, 
  3.683979774036868*^9}},ExpressionUUID->"3b925f2a-5aa4-4722-8111-\
42de70a1640e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
          "dur_", ",", "timepts_", ",", "pts_", ",", "moretimingch___"}], 
          "}"}], ",", "morech___"}], "}"}], ",", "events_", ",", 
       "opts___Rule"}], "]"}], ",", "___Rule"}], "]"}], ":=", "pts"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683979781879389*^9, 3.683979821255773*^9}, {
   3.683979901994973*^9, 3.683979916587043*^9}, {3.68398010211156*^9, 
   3.683980110356081*^9}, 3.68398017786053*^9, {3.683986411141746*^9, 
   3.683986423761941*^9}, 
   3.685033539646431*^9},ExpressionUUID->"5942f005-c53e-4ed6-94d5-\
133cac6aa8f1"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["StandardizedInterval", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683972979124955*^9, 3.683972983330543*^9}, {3.683975121606881*^9, 
   3.683975124294643*^9}, {3.68397585606467*^9, 3.683975860766287*^9}, 
   3.683979956583666*^9, {3.683986711525063*^9, 3.6839867206771593`*^9}, 
   3.684000387425634*^9},ExpressionUUID->"8e8d754c-d5f5-40a0-a6ed-\
35440fc437e6"],

Cell["\<\
implement only if time points and permitted time shift have been defined\
\>", "Text",
 CellChangeTimes->{{3.683975126804194*^9, 3.6839751371992807`*^9}, {
  3.68397587060152*^9, 
  3.6839758797351437`*^9}},ExpressionUUID->"f30a31b6-e4bc-41d0-94c2-\
cba8edc26001"],

Cell[CellGroupData[{

Cell["insert", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.54306105604321*^9, 3.5430610664577312`*^9}, {
   3.543062494985586*^9, 3.5430625010874157`*^9}, 3.683976182937875*^9, {
   3.6839799398099117`*^9, 
   3.68397997971033*^9}},ExpressionUUID->"0090270d-8ca4-442d-80c4-\
ef2bd2dc4f46"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"st", ":", 
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}]}], ",", 
     RowBox[{"ch", ":", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"dur_", ",", 
          RowBox[{"{", 
           RowBox[{"tb_", ",", "ta_"}], "}"}], ",", 
          RowBox[{"pts", ":", 
           RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "Undefined"}], 
         "}"}], ",", "morech___"}], "}"}]}], ",", "events_", ",", 
     "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"dur", ",", 
         RowBox[{"{", 
          RowBox[{"tb", ",", "ta"}], "}"}], ",", "pts", ",", 
         RowBox[{"StandardizedInterval", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"tb", ",", "ta"}], "}"}], ",", "pts"}], "]"}]}], "}"}], 
       ",", "morech"}], "}"}], ",", "\[IndentingNewLine]", "events", ",", 
     "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839730673310833`*^9, 3.683973101981674*^9}, {3.683973149371945*^9, 
   3.6839734156761*^9}, {3.683973451960537*^9, 3.683973475698845*^9}, {
   3.6839736462618427`*^9, 3.683973679564999*^9}, {3.68397413593371*^9, 
   3.683974154427085*^9}, {3.683975165827835*^9, 3.683975303283432*^9}, {
   3.683975518769175*^9, 3.683975550989229*^9}, {3.683975611081381*^9, 
   3.683975614758295*^9}, {3.683975718424601*^9, 3.6839757265040693`*^9}, {
   3.683975889885955*^9, 3.683975944990078*^9}, {3.683976054866651*^9, 
   3.683976077613914*^9}, {3.68397614157769*^9, 3.683976179005468*^9}, {
   3.683976227116561*^9, 3.683976238578314*^9}, 3.683979983856182*^9, 
   3.683980125738283*^9, {3.6839866410027246`*^9, 3.683986652005805*^9}, {
   3.6839866887943077`*^9, 3.683986693625091*^9}, {3.685033539650992*^9, 
   3.685033539655219*^9}, {3.691510703740266*^9, 3.6915107349711943`*^9}, {
   3.691511158244149*^9, 3.69151117070573*^9}, 3.691511211872939*^9, {
   3.69151140993958*^9, 3.691511440904084*^9}, {3.691511555767158*^9, 
   3.6915115560032463`*^9}, {3.6915117236882887`*^9, 3.691511730269465*^9}, {
   3.6915834309238253`*^9, 
   3.691583442232554*^9}},ExpressionUUID->"b202ee03-4a74-451d-b692-\
3fe45c3df0e2"]
}, Closed]],

Cell[CellGroupData[{

Cell["extract", "Subsubsection",
 CellChangeTimes->{{3.683979764654413*^9, 
  3.683979774036868*^9}},ExpressionUUID->"41917f39-26c4-4d57-b3a1-\
d1b443e458ad"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizedInterval", "[", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
          "dur_", ",", "timepts_", ",", "pts_", ",", "standintv_", ",", 
           "moretimingch___"}], "}"}], ",", "morech___"}], "}"}], ",", 
       "events_", ",", "opts___Rule"}], "]"}], ",", "___Rule"}], "]"}], ":=", 
   "standintv"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683979781879389*^9, 3.683979821255773*^9}, {
   3.683979901994973*^9, 3.683979916587043*^9}, 3.683980041030056*^9, {
   3.683980132621498*^9, 3.683980192916423*^9}, {3.6839866847908583`*^9, 
   3.683986701073138*^9}, 
   3.685033539661071*^9},ExpressionUUID->"e8d72d2f-6bba-4e6a-ac7a-\
daea7b0d9fd2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["timing corrections & resets", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683972979124955*^9, 3.683972983330543*^9}, {3.683975121606881*^9, 
   3.683975124294643*^9}, {3.68397585606467*^9, 3.683975860766287*^9}, 
   3.683979956583666*^9, {3.683986711525063*^9, 3.6839867206771593`*^9}, 
   3.684000387425634*^9, {3.6905432844824944`*^9, 3.690543292165162*^9}, {
   3.6905436733695173`*^9, 
   3.690543679149097*^9}},ExpressionUUID->"9cd6f064-3444-414d-aa97-\
044a16c045eb"],

Cell[CellGroupData[{

Cell["\<\
reset timing of contained events if needed, and if the enclosing EventList \
contains the initial and final time points\
\>", "Subsubsection",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683972979124955*^9, 3.683972983330543*^9}, {3.683975121606881*^9, 
   3.683975124294643*^9}, {3.68397585606467*^9, 3.683975860766287*^9}, 
   3.683979956583666*^9, {3.683986711525063*^9, 3.6839867206771593`*^9}, 
   3.684000387425634*^9, {3.6905432844824944`*^9, 3.690543292165162*^9}, {
   3.690543686014332*^9, 3.690543695405426*^9}, {3.6905441701323643`*^9, 
   3.69054418142447*^9}},ExpressionUUID->"3086fcf8-446b-43ab-8943-\
db3111050656"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventList", "[", 
     RowBox[{
      RowBox[{"st", ":", 
       RowBox[{"{", 
        RowBox[{"False", ",", "numst_"}], "}"}]}], ",", 
      RowBox[{"ch", ":", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"dur_", ",", 
           RowBox[{"{", 
            RowBox[{"tb_", ",", 
             RowBox[{"ta", ":", 
              RowBox[{"Except", "[", "Undefined", "]"}]}]}], "}"}], ",", 
           "moretch___"}], "}"}], ",", "morech___"}], "}"}]}], ",", 
      RowBox[{"events", ":", 
       RowBox[{"{", 
        RowBox[{"_Event", ".."}], "}"}]}], ",", "opts___Rule"}], "]"}], "/;", 
    
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"InitialTimePoint", "/@", "events"}], ",", 
        RowBox[{"FinalTimePoint", "/@", "events"}]}], "]"}], ",", 
      "Undefined"}], "]"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "newevlist", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{"EventList", "[", 
       RowBox[{"st", ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"dur", ",", "Undefined", ",", "moretch"}], "}"}], ",", 
          "morech"}], "}"}], ",", "\[IndentingNewLine]", "events", ",", 
        "\[IndentingNewLine]", 
        RowBox[{"SimplifyRules", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"InitialTimePoint", "\[Rule]", "ta"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"FinalTimePoint", "\[Rule]", "Automatic"}], ",", 
          "\[IndentingNewLine]", "opts"}], "\[IndentingNewLine]", "]"}]}], 
       "\[IndentingNewLine]", "]"}], "\[IndentingNewLine]", ")"}]}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.690543329247077*^9, 3.690543650585127*^9}, {
   3.690543702843224*^9, 3.690543707128552*^9}, {3.6905437844426394`*^9, 
   3.69054380459171*^9}, 3.690544013224084*^9, {3.6905440474160347`*^9, 
   3.690544096239839*^9}, {3.6914997484427643`*^9, 3.691499827092299*^9}, {
   3.691499970122604*^9, 3.691499970543294*^9}, {3.6915033060170383`*^9, 
   3.691503323325264*^9}, {3.6915108170486097`*^9, 3.691510818806533*^9}, {
   3.691580226418738*^9, 3.691580230023841*^9}, {3.691580313624485*^9, 
   3.691580317669293*^9}, 
   3.691583446939707*^9},ExpressionUUID->"741c5058-6f04-4572-b154-\
dac8989d01e2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventList", "[", 
     RowBox[{
      RowBox[{"st", ":", 
       RowBox[{"{", 
        RowBox[{"False", ",", "numst_"}], "}"}]}], ",", 
      RowBox[{"ch", ":", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"dur_", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"tb", ":", 
              RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "ta_"}], 
            "}"}], ",", "moretch___"}], "}"}], ",", "morech___"}], "}"}]}], 
      ",", 
      RowBox[{"events", ":", 
       RowBox[{"{", 
        RowBox[{"_Event", ".."}], "}"}]}], ",", "opts___Rule"}], "]"}], "/;", 
    
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"InitialTimePoint", "/@", "events"}], ",", 
        RowBox[{"FinalTimePoint", "/@", "events"}]}], "]"}], ",", 
      "Undefined"}], "]"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "newevlist", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{"EventList", "[", 
       RowBox[{"st", ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"dur", ",", "Undefined", ",", "moretch"}], "}"}], ",", 
          "morech"}], "}"}], ",", "\[IndentingNewLine]", "events", ",", 
        "\[IndentingNewLine]", 
        RowBox[{"SimplifyRules", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"InitialTimePoint", "\[Rule]", "Automatic"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"FinalTimePoint", "\[Rule]", "tb"}], ",", 
          "\[IndentingNewLine]", "opts"}], "\[IndentingNewLine]", "]"}]}], 
       "\[IndentingNewLine]", "]"}], "\[IndentingNewLine]", ")"}]}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.690543329247077*^9, 3.690543650585127*^9}, {
   3.690543702843224*^9, 3.690543707128552*^9}, {3.6905437844426394`*^9, 
   3.69054380459171*^9}, 3.690544013224084*^9, {3.6905440474160347`*^9, 
   3.690544096239839*^9}, {3.690544135638132*^9, 3.690544150085905*^9}, {
   3.6914998395343933`*^9, 3.691499874773198*^9}, {3.69149996593025*^9, 
   3.691499966470104*^9}, {3.6915033274159317`*^9, 3.691503335238008*^9}, {
   3.6915108254170923`*^9, 3.691510826942357*^9}, {3.6915803256210814`*^9, 
   3.6915803401894608`*^9}, 
   3.691583450921419*^9},ExpressionUUID->"4e4067c1-f9f2-423b-b9f9-\
f922e3926c66"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["BackgroundGenerator", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, 
   3.683973033466785*^9, {3.683973750698831*^9, 3.683973753893053*^9}, {
   3.683978343050639*^9, 3.683978346277608*^9}, 3.683979658068478*^9, 
   3.683980204513444*^9, 
   3.709374952840026*^9},ExpressionUUID->"41479b7b-5028-479e-941f-\
be2b54ee4eec"],

Cell[CellGroupData[{

Cell["insert", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 
   3.6839802072697153`*^9},ExpressionUUID->"b200f867-a15d-4de7-857d-\
9eb138ac5b26"],

Cell[CellGroupData[{

Cell["\<\
insert BackgroundGenerator into characteristics when available\
\>", "Subsubsection",
 CellChangeTimes->{{3.683995882762225*^9, 3.683995894432136*^9}, 
   3.685033554348171*^9, 3.685035809619052*^9, 3.68517040347085*^9, {
   3.68528096909787*^9, 3.685280986297069*^9}, {3.6852817709278584`*^9, 
   3.685281780938259*^9}, {3.689010473744309*^9, 3.689010480565526*^9}, {
   3.689424052588152*^9, 3.689424057345055*^9}, 
   3.709374954796999*^9},ExpressionUUID->"28e8245d-20ca-4e6e-9591-\
78aa962f115a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventList", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"timingch_", ",", 
        RowBox[{"{", 
         RowBox[{"Undefined", ",", "_", ",", "_"}], "}"}], ",", "morech___"}],
        "}"}], ",", 
      RowBox[{"events", ":", 
       RowBox[{"{", 
        RowBox[{"_Event", "..."}], "}"}]}], ",", "opts___Rule"}], "]"}], ":=",
     "\[IndentingNewLine]", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", "evbackg", "}"}], ",", 
      RowBox[{
       RowBox[{"(", "\[IndentingNewLine]", 
        RowBox[{"EventList", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"False", ",", "numst"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"timingch", ",", 
            RowBox[{"{", 
             RowBox[{"evbackg", ",", "False", ",", "False"}], "}"}], ",", 
            "morech"}], "}"}], ",", "\[IndentingNewLine]", "events", ",", 
          "\[IndentingNewLine]", 
          RowBox[{"Sequence", "@@", 
           RowBox[{"Cases", "[", 
            RowBox[{
             RowBox[{"{", "opts", "}"}], ",", 
             RowBox[{"Except", "[", 
              RowBox[{"BackgroundGenerator", "\[Rule]", "_"}], "]"}]}], 
            "]"}]}]}], "\[IndentingNewLine]", "]"}], "\[IndentingNewLine]", 
        ")"}], "/;", 
       RowBox[{"(", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"evbackg", "=", 
          RowBox[{
           RowBox[{"BackgroundGenerator", "/.", 
            RowBox[{"{", "opts", "}"}]}], "/.", 
           RowBox[{"Options", "[", "EventList", "]"}]}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"!", 
          RowBox[{"MatchQ", "[", 
           RowBox[{"evbackg", ",", "Undefined"}], "]"}]}]}], 
        "\[IndentingNewLine]", ")"}]}]}], "]"}]}], ";"}], " "}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683995823935512*^9, 
   3.683995850951077*^9}, {3.683995953724373*^9, 3.683996164873061*^9}, {
   3.685033539666007*^9, 3.685033569634625*^9}, {3.685280366071978*^9, 
   3.685280366834993*^9}, {3.6852817871736593`*^9, 3.685281800750771*^9}, {
   3.688838497339658*^9, 3.688838506712433*^9}, {3.6894128694128323`*^9, 
   3.6894129028978863`*^9}, {3.689413747412732*^9, 3.689413750792139*^9}, {
   3.6894239116930428`*^9, 3.689423920339079*^9}, {3.689423998759162*^9, 
   3.689423999659368*^9}, {3.689424255440865*^9, 3.689424273230249*^9}, {
   3.689622376907209*^9, 3.689622382919942*^9}, 3.689622490166404*^9, {
   3.6896226691261463`*^9, 3.689622672107609*^9}, 3.689623400434723*^9, 
   3.6896247989097853`*^9, 3.6896248691547203`*^9, {3.6915108890225163`*^9, 
   3.691510922924698*^9}, 3.691583453928644*^9, {3.7093749561488256`*^9, 
   3.709374956913786*^9}},ExpressionUUID->"62e9de03-03ec-43f5-850b-\
0da54b3a785e"]
}, Closed]],

Cell[CellGroupData[{

Cell["request implementation of BackgroundGenerator", "Subsubsection",
 CellChangeTimes->{{3.683995882762225*^9, 3.683995894432136*^9}, 
   3.685033554348171*^9, 3.685035809619052*^9, 3.68517040347085*^9, {
   3.68528096909787*^9, 3.685280986297069*^9}, {3.6852817709278584`*^9, 
   3.685281780938259*^9}, {3.689010473744309*^9, 3.689010480565526*^9}, {
   3.6894240418036222`*^9, 3.689424061596233*^9}, 
   3.709374957856393*^9},ExpressionUUID->"4c480338-e8bb-455c-92dc-\
d19fdf1570a2"],

Cell["\<\
only request implementation when all Repeat expansions have been done\
\>", "Text",
 CellChangeTimes->{{3.691512597980013*^9, 3.691512620123664*^9}, 
   3.6928497139323463`*^9},ExpressionUUID->"0fac3ef4-22fa-4cb2-a3a3-\
1983231a413a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"timingch_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"evbackg", ":", 
          RowBox[{"Except", "[", "Undefined", "]"}]}], ",", 
         RowBox[{"False", "|", "Undefined"}], ",", "_"}], "}"}], ",", "hilb_",
        ",", 
       RowBox[{"repevexpch", ":", 
        RowBox[{"True", "|", 
         RowBox[{"{", 
          RowBox[{"__", ",", "True"}], "}"}]}]}], ",", "morech___"}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"events", ":", 
      RowBox[{"{", 
       RowBox[{"_Event", "..."}], "}"}]}], ",", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"(", "\[IndentingNewLine]", 
    RowBox[{"EventList", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "numst"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"timingch", ",", 
        RowBox[{"{", 
         RowBox[{"evbackg", ",", "True", ",", "False"}], "}"}], ",", "hilb", 
        ",", "repevexpch", ",", "morech"}], "}"}], ",", "\[IndentingNewLine]",
       "events", ",", "\[IndentingNewLine]", 
      RowBox[{"Sequence", "@@", 
       RowBox[{"Cases", "[", 
        RowBox[{
         RowBox[{"{", "opts", "}"}], ",", 
         RowBox[{"Except", "[", 
          RowBox[{"BackgroundGenerator", "\[Rule]", "_"}], "]"}]}], "]"}]}]}],
      "\[IndentingNewLine]", "]"}], "\[IndentingNewLine]", ")"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683995823935512*^9, 
   3.683995850951077*^9}, {3.683995953724373*^9, 3.683996164873061*^9}, {
   3.685033539666007*^9, 3.685033569634625*^9}, {3.685280366071978*^9, 
   3.685280366834993*^9}, {3.6852817871736593`*^9, 3.685281800750771*^9}, {
   3.688838497339658*^9, 3.688838506712433*^9}, {3.6894128694128323`*^9, 
   3.6894129028978863`*^9}, {3.689413747412732*^9, 3.689413750792139*^9}, {
   3.6894239116930428`*^9, 3.689423920339079*^9}, {3.689423998759162*^9, 
   3.689423999659368*^9}, {3.689424073747998*^9, 3.689424127926732*^9}, {
   3.6894242860733747`*^9, 3.689424286757609*^9}, 3.689622740133307*^9, {
   3.689623392515909*^9, 3.6896233967919397`*^9}, {3.689623503467298*^9, 
   3.689623530864276*^9}, {3.689623653887183*^9, 3.689623659420879*^9}, {
   3.689624798915576*^9, 3.6896248126496887`*^9}, {3.689624874514057*^9, 
   3.6896248772308283`*^9}, {3.691510928964157*^9, 3.691510954217742*^9}, {
   3.691512640975528*^9, 3.691512661202503*^9}, 3.6915127117655773`*^9, {
   3.69157779618892*^9, 3.691577813962153*^9}, 3.691583457241589*^9, 
   3.7093749586462517`*^9},ExpressionUUID->"3446677d-4d34-440e-a47d-\
4c03ed4ac6a5"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["pass down to contained events", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, {
   3.690394392305965*^9, 
   3.6903943968153*^9}},ExpressionUUID->"98b08f3b-736f-403c-a6bf-\
fb2fd9ba8f01"],

Cell[CellGroupData[{

Cell["\<\
pass BackgroundGenerator and implementation request to each contained Event\
\>", "Subsubsection",
 CellChangeTimes->{{3.685280837839164*^9, 3.6852808438799467`*^9}, {
   3.6852809298678293`*^9, 3.685280950759725*^9}, {3.685281000699381*^9, 
   3.685281001395069*^9}, {3.685281817558907*^9, 3.685281821247967*^9}, {
   3.689010550333774*^9, 3.6890105531149178`*^9}, {3.689413051179784*^9, 
   3.689413063155238*^9}, 3.689424064303047*^9, 
   3.7093749594103413`*^9},ExpressionUUID->"0df3cbf3-2229-4b35-96c9-\
19c791305b25"],

Cell["\<\
ensure that BackgroundGenerator implementation is not requested for an \
already implemented Event\
\>", "Text",
 CellChangeTimes->{{3.691233721739171*^9, 3.691233771496656*^9}, 
   3.709374960310235*^9},ExpressionUUID->"0559e3de-36ef-45f4-8f0d-\
867d774c419d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"st", ":", 
      RowBox[{"{", 
       RowBox[{"False", ",", "_"}], "}"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"timingch_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"backg", ":", 
          RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "backgimplreq_", 
         ",", "False"}], "}"}], ",", "morech___"}], "}"}], ",", 
     RowBox[{"events", ":", 
      RowBox[{"{", 
       RowBox[{"_Event", "..."}], "}"}]}], ",", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Verbose", "/.", 
          RowBox[{"{", "opts", "}"}]}], "/.", 
         RowBox[{"Options", "[", "EventList", "]"}]}], ",", 
        RowBox[{"Print", "[", 
         RowBox[{
         "\"\<passing BackgroundGenerator down to events inside EventList: \>\
\"", ",", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"backg", ",", "backgimplreq", ",", "False"}], "}"}], 
             ",", 
             RowBox[{"Expand", "@", "events"}]}], "}"}], "/.", 
           RowBox[{"s_ShapeFunction", "\[RuleDelayed]", 
            RowBox[{"s", "\[LeftDoubleBracket]", 
             RowBox[{"{", "4", "}"}], "\[RightDoubleBracket]"}]}]}]}], 
         "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"EventList", "[", 
       RowBox[{"st", ",", 
        RowBox[{"{", 
         RowBox[{"timingch", ",", 
          RowBox[{"{", 
           RowBox[{"backg", ",", "backgimplreq", ",", "False"}], "}"}], ",", 
          "morech"}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"Replace", "[", "\[IndentingNewLine]", 
         RowBox[{"events", ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Event", "[", 
             RowBox[{"evst_", ",", 
              RowBox[{"evch", ":", 
               RowBox[{"{", 
                RowBox[{"evtimingch_", ",", 
                 RowBox[{"evbackgch", ":", 
                  RowBox[{"Except", "[", 
                   RowBox[{"{", 
                    RowBox[{"__", ",", "True"}], "}"}], "]"}]}], ",", 
                 "evmorech___"}], "}"}]}], ",", "\[IndentingNewLine]", 
              "evspecs_", ",", "evopts___Rule"}], "]"}], "\[RuleDelayed]", 
            RowBox[{"Event", "[", 
             RowBox[{"evst", ",", 
              RowBox[{"{", 
               RowBox[{"evtimingch", ",", 
                RowBox[{"{", 
                 RowBox[{"backg", ",", "backgimplreq", ",", "False"}], "}"}], 
                ",", "evmorech"}], "}"}], ",", "evspecs", ",", "evopts"}], 
             "]"}]}], "}"}], ",", "\[IndentingNewLine]", "1"}], 
         "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", "opts"}], 
       "]"}]}], "\[IndentingNewLine]", ")"}], "/;", 
    RowBox[{"Or", "@@", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"MatchQ", "[", 
         RowBox[{"#", ",", "\[IndentingNewLine]", 
          RowBox[{"HoldPattern", "@", 
           RowBox[{"Event", "[", 
            RowBox[{"evst_", ",", 
             RowBox[{"evch", ":", 
              RowBox[{"{", 
               RowBox[{"_", ",", 
                RowBox[{"{", 
                 RowBox[{"_", ",", 
                  RowBox[{"False", "|", "Undefined"}], ",", "_"}], "}"}], ",",
                 "___"}], "}"}]}], ",", "___"}], "]"}]}]}], 
         "\[IndentingNewLine]", "]"}], "&"}], "/@", "events"}], ")"}]}]}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.685280501378611*^9, 3.6852805784636593`*^9}, {
   3.685280634164528*^9, 3.685280646147335*^9}, {3.6852806811010036`*^9, 
   3.685280819906938*^9}, {3.6852808518239107`*^9, 3.685280920517477*^9}, 
   3.685281104934045*^9, {3.6852812440437107`*^9, 3.685281273463298*^9}, {
   3.6852818392737303`*^9, 3.6852819105325823`*^9}, {3.688838515690022*^9, 
   3.688838528401292*^9}, {3.6890103458130302`*^9, 3.6890103479388866`*^9}, {
   3.6890105227911177`*^9, 3.689010525005052*^9}, {3.689412838864313*^9, 
   3.6894128565649137`*^9}, {3.689412982579432*^9, 3.689413009332736*^9}, {
   3.689413147100787*^9, 3.689413148129038*^9}, {3.689424404094623*^9, 
   3.689424439777028*^9}, {3.689622934437344*^9, 3.689623072917139*^9}, 
   3.6896231737791767`*^9, {3.6896233628199987`*^9, 3.689623384089663*^9}, 
   3.689624798924426*^9, {3.6896248847621183`*^9, 3.689624890364489*^9}, {
   3.690440011743452*^9, 3.690440038111909*^9}, {3.690440116439343*^9, 
   3.690440133067438*^9}, {3.690448813156196*^9, 3.6904488301058607`*^9}, {
   3.691232855598935*^9, 3.691232891791669*^9}, 3.6912329946752157`*^9, {
   3.691233694740004*^9, 3.691233711399494*^9}, {3.691233909146716*^9, 
   3.691233924772256*^9}, {3.6915109672228003`*^9, 3.691510968753133*^9}, 
   3.691512719520978*^9, 3.6915128901682653`*^9, {3.691577593810226*^9, 
   3.691577609600284*^9}, {3.691583460329104*^9, 3.691583493892016*^9}, 
   3.699204720775388*^9, 
   3.709374960941844*^9},ExpressionUUID->"e6a490d3-aec9-43ee-99cd-\
d2e3a82c13c5"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["pass up from contained events", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, {
   3.690394392305965*^9, 
   3.690394411552886*^9}},ExpressionUUID->"1a999fc8-8494-4ccf-ba75-\
650a1abdf950"],

Cell[CellGroupData[{

Cell["\<\
set backg of EventList if already present in all contained events\
\>", "Subsubsection",
 CellChangeTimes->{{3.685280837839164*^9, 3.6852808438799467`*^9}, {
  3.6852809298678293`*^9, 3.685280943377266*^9}, {3.6852810053889723`*^9, 
  3.6852810152103167`*^9}, {3.685281916820855*^9, 3.685281919001607*^9}, {
  3.6888385853930607`*^9, 3.68883864498806*^9}, {3.68901039942449*^9, 
  3.689010410184332*^9}, {3.690394424899064*^9, 3.6903944382307453`*^9}, {
  3.6903945030633097`*^9, 3.6903945043180428`*^9}, {3.69039482795173*^9, 
  3.690394830355267*^9}},ExpressionUUID->"e221540d-01f5-4dea-b3d1-\
5a827ff20062"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"timingch_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"evlistbackg", ":", "Undefined"}], ",", "_", ",", "_"}], 
        "}"}], ",", "morech___"}], "}"}], ",", 
     RowBox[{"events", ":", 
      RowBox[{"{", 
       RowBox[{"_Event", "..."}], "}"}]}], ",", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "backgs", "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{"If", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"GeneratorMatchQ", "@@", "backgs"}], ")"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"EventList", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"False", ",", "numst"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"timingch", ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"First", "@", "backgs"}], ",", "False", ",", "False"}],
               "}"}], ",", "morech"}], "}"}], ",", "\[IndentingNewLine]", 
           "events", ",", "opts"}], "\[IndentingNewLine]", "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Message", "[", 
            RowBox[{"EventList", "::", "backgmismatch"}], "]"}], ";", 
           "$Failed"}], ")"}]}], "\[IndentingNewLine]", "]"}], 
       "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"backgs", "=", 
         RowBox[{"BackgroundGenerator", "/@", "events"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Or", "@@", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"!", 
             RowBox[{"MatchQ", "[", 
              RowBox[{"#", ",", "Undefined"}], "]"}]}], "&"}], "/@", 
           "backgs"}], ")"}]}]}], "\[IndentingNewLine]", ")"}]}]}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683996479128291*^9, 3.683996666134377*^9}, 
   3.683996770293785*^9, {3.6839970807786922`*^9, 3.683997081864418*^9}, {
   3.683997194636444*^9, 3.683997316888473*^9}, {3.683997365424621*^9, 
   3.6839973713975277`*^9}, {3.683997710258274*^9, 3.683997730113914*^9}, {
   3.6839978979904003`*^9, 3.683997899292357*^9}, {3.683998002682864*^9, 
   3.6839980044324427`*^9}, {3.685033539679089*^9, 3.685033569641567*^9}, {
   3.688838550752159*^9, 3.6888385746978827`*^9}, {3.68883861141486*^9, 
   3.688838618309884*^9}, {3.6890103902993383`*^9, 3.689010390969832*^9}, 
   3.6890105920437403`*^9, {3.6894243091853523`*^9, 3.6894243098858347`*^9}, {
   3.6903945179355087`*^9, 3.690394528350492*^9}, {3.6903945596938868`*^9, 
   3.6903946145952*^9}, {3.690394650496812*^9, 3.69039471383661*^9}, {
   3.690394752508424*^9, 3.690394821918231*^9}, 3.691493136291236*^9, {
   3.691493242637908*^9, 3.6914933696623163`*^9}, {3.691493470947983*^9, 
   3.69149348356992*^9}, 3.709374300999695*^9, 
   3.709374961627717*^9},ExpressionUUID->"d9489179-b936-4016-a4aa-\
58fcb8c6fcd7"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
set backg implemented flag if it has been implemented in all Events\
\>", "Subsubsection",
 CellChangeTimes->{{3.685280837839164*^9, 3.6852808438799467`*^9}, {
  3.6852809298678293`*^9, 3.685280943377266*^9}, {3.6852810053889723`*^9, 
  3.6852810152103167`*^9}, {3.685281916820855*^9, 3.685281919001607*^9}, {
  3.6888385853930607`*^9, 3.68883864498806*^9}, {3.68901039942449*^9, 
  3.689010410184332*^9}},ExpressionUUID->"8acd7fea-c211-4d2f-8d71-\
7bc8c1dd8350"],

Cell["also check that all backgrounds are consistent", "Text",
 CellChangeTimes->{{3.6914940016400633`*^9, 
  3.691494012493244*^9}},ExpressionUUID->"b87266d5-9756-4125-b80a-\
869702d42754"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"timingch_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"evlistbackg", ":", 
          RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "_", ",", 
         RowBox[{"False", "|", "Undefined"}]}], "}"}], ",", "morech___"}], 
      "}"}], ",", 
     RowBox[{"events", ":", 
      RowBox[{"{", 
       RowBox[{"_Event", "..."}], "}"}]}], ",", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "evbackgs", "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{"EventList", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"False", ",", "numst"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"timingch", ",", 
           RowBox[{"{", 
            RowBox[{"evlistbackg", ",", "True", ",", "True"}], "}"}], ",", 
           "morech"}], "}"}], ",", "\[IndentingNewLine]", "events", ",", 
         "opts"}], "\[IndentingNewLine]", "]"}], "\[IndentingNewLine]", ")"}],
       "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"evbackgs", "=", 
         RowBox[{"BackgroundGenerator", "/@", "events"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"FreeQ", "[", 
          RowBox[{"evbackgs", ",", "Undefined"}], "]"}], "&&", 
         "\[IndentingNewLine]", 
         RowBox[{"GeneratorMatchQ", "@@", 
          RowBox[{"Join", "[", 
           RowBox[{"evbackgs", ",", 
            RowBox[{"{", "evlistbackg", "}"}]}], "]"}]}]}]}], 
       "\[IndentingNewLine]", ")"}]}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683996479128291*^9, 3.683996666134377*^9}, 
   3.683996770293785*^9, {3.6839970807786922`*^9, 3.683997081864418*^9}, {
   3.683997194636444*^9, 3.683997316888473*^9}, {3.683997365424621*^9, 
   3.6839973713975277`*^9}, {3.683997710258274*^9, 3.683997730113914*^9}, {
   3.6839978979904003`*^9, 3.683997899292357*^9}, {3.683998002682864*^9, 
   3.6839980044324427`*^9}, {3.685033539679089*^9, 3.685033569641567*^9}, {
   3.688838550752159*^9, 3.6888385746978827`*^9}, {3.68883861141486*^9, 
   3.688838618309884*^9}, {3.6890103902993383`*^9, 3.689010390969832*^9}, 
   3.6890105920437403`*^9, {3.6894243091853523`*^9, 3.6894243098858347`*^9}, {
   3.691236317647048*^9, 3.69123632795212*^9}, {3.691493698604121*^9, 
   3.691493813985094*^9}, {3.6914938471110783`*^9, 3.691493852443811*^9}, {
   3.691493892452911*^9, 3.6914939827271767`*^9}, 3.7093743010077972`*^9, 
   3.709374962369961*^9},ExpressionUUID->"5888274e-276b-4a1d-bb1a-\
457d55ca7eab"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["extract", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
  3.683973766585649*^9, 3.6839737710410423`*^9}, {3.6839742986026897`*^9, 
  3.683974310684813*^9}, {3.683980216123659*^9, 
  3.683980224571121*^9}},ExpressionUUID->"aeb75dcc-3cfe-4633-ad4c-\
e3c27c3702a0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"BackgroundGenerator", "[", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{"timingch_", ",", 
         RowBox[{"{", 
          RowBox[{"backg_", ",", "__"}], "}"}], ",", "morech___"}], "}"}], 
       ",", "events_", ",", "opts___Rule"}], "]"}], ",", "___"}], "]"}], ":=",
    "backg"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683980234573969*^9, 3.683980292705936*^9}, {
   3.683996695241515*^9, 3.6839967030015783`*^9}, 3.685033539686366*^9, 
   3.689009226494648*^9, 
   3.709374963067141*^9},ExpressionUUID->"c2180c5d-c52b-4a7c-93da-\
fbc8523d42c7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"BackgroundGeneratorImplemented", "[", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{"timingch_", ",", 
         RowBox[{"{", 
          RowBox[{"backg_", ",", "requbackgimp_", ",", "backgimp_"}], "}"}], 
         ",", "morech___"}], "}"}], ",", "events_", ",", "opts___Rule"}], 
      "]"}], ",", "___"}], "]"}], ":=", 
   RowBox[{"TrueQ", "[", "backgimp", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683980234573969*^9, 3.683980292705936*^9}, 
   3.685033569554091*^9, 3.6888388840081673`*^9, {3.688838920399446*^9, 
   3.688838960803563*^9}, {3.69106148449118*^9, 3.6910614897518063`*^9}, 
   3.709374963821039*^9},ExpressionUUID->"e08f034b-cba8-4065-b297-\
6d303da20f6f"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["HilbertSpaceCompatibility", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, 
   3.683973033466785*^9, {3.683973750698831*^9, 3.683973753893053*^9}, {
   3.683976432753954*^9, 3.683976436691045*^9}, 3.68397674513378*^9, {
   3.68397832756979*^9, 3.683978331696862*^9}, {3.683979633451119*^9, 
   3.6839796489973183`*^9}, 
   3.683980314039257*^9},ExpressionUUID->"f2fe6078-e3fd-46dd-8c95-\
816603381768"],

Cell[CellGroupData[{

Cell["insert", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 
   3.6839802072697153`*^9},ExpressionUUID->"99967d54-6f77-474d-bedb-\
8c4e667c5c06"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventList", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"timingch_", ",", 
        RowBox[{"backgch", ":", 
         RowBox[{"{", 
          RowBox[{"backg_", ",", "backgimpreq_", ",", "backgimp_"}], "}"}]}], 
        ",", "Undefined", ",", "morech___"}], "}"}], ",", "events_List", ",", 
      "opts___Rule"}], "]"}], "/;", 
    RowBox[{"MatchQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"backg", ",", "backgimp"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Except", "[", "Undefined", "]"}], ",", "True"}], "}"}]}], 
     "]"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "hilbcomp", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"hilbcomp", "=", 
        RowBox[{"Switch", "[", 
         RowBox[{
          RowBox[{"HilbertSpaceCompatibility", "/@", "events"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"True", ".."}], "}"}], ",", "True", ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"False", ".."}], "}"}], ",", "False", ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"True", "|", "False"}], ")"}], ".."}], "}"}], ",", 
          "MixedHilbertSpaceCompatibility", ",", "\[IndentingNewLine]", "_", 
          ",", "$Failed"}], "\[IndentingNewLine]", "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"EventList", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"False", ",", "numst"}], "}"}], ",", "\[IndentingNewLine]", 
         
         RowBox[{"{", 
          RowBox[{
          "timingch", ",", "backgch", ",", "hilbcomp", ",", "morech"}], "}"}],
          ",", "\[IndentingNewLine]", "events", ",", "opts"}], "]"}]}], 
      "\[IndentingNewLine]", ")"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683976480502823*^9, 
   3.683976634737287*^9}, {3.6839766756355143`*^9, 3.683976688875939*^9}, 
   3.683998087149127*^9, {3.68399813170348*^9, 3.68399829794249*^9}, {
   3.685033539691053*^9, 3.685033539693742*^9}, {3.689622396241652*^9, 
   3.689622407414858*^9}, {3.6896232652176113`*^9, 3.689623271014195*^9}, {
   3.689624896836812*^9, 3.6896248986654778`*^9}, {3.691581291194838*^9, 
   3.691581311407995*^9}, {3.6915815553124332`*^9, 3.691581659835607*^9}, 
   3.6915835083355713`*^9},ExpressionUUID->"21354e6e-a805-47f6-84c6-\
57acc606e07d"]
}, Closed]],

Cell[CellGroupData[{

Cell["extract", "Section",
 CellChangeTimes->{{3.683972649262375*^9, 3.683972657724471*^9}, {
   3.683973766585649*^9, 3.6839737710410423`*^9}, 3.6839802072697153`*^9, {
   3.6839803310601482`*^9, 
   3.683980332240543*^9}},ExpressionUUID->"f290e655-452e-4de1-b109-\
11adaa2be48a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
        "timingch_", ",", "backgch_", ",", "hilbcompat_", ",", "morech___"}], 
        "}"}], ",", "events_", ",", "opts___Rule"}], "]"}], ",", "___"}], 
    "]"}], ":=", "hilbcompat"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683980234573969*^9, 3.683980292705936*^9}, {
   3.683980344913636*^9, 3.683980364039876*^9}, {3.683998373865924*^9, 
   3.6839983818296833`*^9}, 
   3.685033539698225*^9},ExpressionUUID->"6934fac6-aa82-4880-aea4-\
59bfb03b64ba"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["expand Repeat", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, 
   3.683973033466785*^9, {3.683973750698831*^9, 3.683973753893053*^9}, {
   3.683976432753954*^9, 3.683976436691045*^9}, 3.68397674513378*^9, {
   3.68397832756979*^9, 3.683978331696862*^9}, {3.683979633451119*^9, 
   3.6839796489973183`*^9}, 3.683980314039257*^9, {3.685195358390863*^9, 
   3.685195362675988*^9}, {3.685195771587109*^9, 3.685195776906199*^9}, 
   3.685196178721356*^9, {3.689662134744946*^9, 3.6896621371420393`*^9}, 
   3.6904421044040728`*^9, 
   3.692848358174252*^9},ExpressionUUID->"69c51c16-70f5-4cb5-8546-\
34ec58036ce6"],

Cell[CellGroupData[{

Cell["set expansion type", "Section",
 CellChangeTimes->{{3.685195397090225*^9, 3.685195408272977*^9}, {
   3.6851955281105633`*^9, 3.685195529549337*^9}, 
   3.6894115427587633`*^9},ExpressionUUID->"c02521c6-49dc-4152-bb21-\
4219903a77e0"],

Cell[CellGroupData[{

Cell["using options to EventList", "Subsubsection",
 CellChangeTimes->{{3.691505120426874*^9, 
  3.691505128402646*^9}},ExpressionUUID->"26d38f2c-8669-4620-ac6a-\
8c6950677e44"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"chars", ":", 
        RowBox[{"Repeated", "[", 
         RowBox[{"_", ",", 
          RowBox[{"{", "3", "}"}]}], "]"}]}], ",", 
       RowBox[{"Undefined", "|", 
        RowBox[{"{", 
         RowBox[{"Undefined", ",", "morerepevch__"}], "}"}]}], ",", 
       "morech___"}], "}"}], ",", 
     RowBox[{"events", ":", 
      RowBox[{"{", 
       RowBox[{"_Event", "..."}], "}"}]}], ",", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"(", "\[IndentingNewLine]", 
    RowBox[{"EventList", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "numst"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"chars", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"ExpandRepeatedEvents", "/.", 
            RowBox[{"{", "opts", "}"}]}], "/.", 
           RowBox[{"Options", "[", "EventList", "]"}]}], ",", "False", ",", 
          "False"}], "}"}], ",", "morech"}], "}"}], ",", "events", ",", 
      "opts"}], "]"}], "\[IndentingNewLine]", ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.689411578020068*^9, 3.68941163662068*^9}, {
  3.691504953116613*^9, 3.691504953897336*^9}, {3.691509795428233*^9, 
  3.6915098399286947`*^9}, {3.6915099043557653`*^9, 3.691509905854949*^9}, {
  3.6915100105685043`*^9, 3.691510016937169*^9}, {3.691583513392887*^9, 
  3.691583516473976*^9}},ExpressionUUID->"b46c921a-acf2-4668-a5b6-\
a7320178af5e"]
}, Closed]],

Cell[CellGroupData[{

Cell["using an ExpandRepeatedEvents call", "Subsubsection",
 CellChangeTimes->{{3.685195397090225*^9, 3.685195408272977*^9}, {
  3.6851955281105633`*^9, 3.685195529549337*^9}, {3.689411846398017*^9, 
  3.689411852378552*^9}, {3.6915051062061*^9, 
  3.6915051481459293`*^9}},ExpressionUUID->"49a7a2be-0fe1-4d7b-81a2-\
aad0192dd05a"],

Cell["\<\
Forced specifications supersede any previously set expansion type\
\>", "Text",
 CellChangeTimes->{{3.689411080782607*^9, 3.689411098002344*^9}, {
  3.689411752367354*^9, 
  3.689411755933936*^9}},ExpressionUUID->"7e454695-cb47-462b-8635-\
4666a4feaea9"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ExpandRepeatedEvents", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"False", ",", "numst_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"chars", ":", 
          RowBox[{"Repeated", "[", 
           RowBox[{"_", ",", 
            RowBox[{"{", "3", "}"}]}], "]"}]}], ",", 
         RowBox[{"Undefined", "|", 
          RowBox[{"{", 
           RowBox[{"Undefined", ".."}], "}"}]}], ",", "morech___"}], "}"}], 
       ",", 
       RowBox[{"events", ":", 
        RowBox[{"{", 
         RowBox[{"_Event", "..."}], "}"}]}], ",", "opts___Rule"}], "]"}], ",",
      "\[IndentingNewLine]", "exprepevopts___Rule"}], "\[IndentingNewLine]", 
    "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"chars", ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"ExpandRepeatedEvents", "/.", 
           RowBox[{"{", 
            RowBox[{"exprepevopts", ",", "opts"}], "}"}]}], "/.", 
          RowBox[{"Options", "[", "EventList", "]"}]}], ",", "True", ",", 
         "False"}], "}"}], ",", "morech"}], "}"}], ",", "\[IndentingNewLine]",
      "events", ",", "\[IndentingNewLine]", 
     RowBox[{"Sequence", "@@", 
      RowBox[{"FilterRules", "[", 
       RowBox[{
        RowBox[{"{", "opts", "}"}], ",", 
        RowBox[{"Except", "[", "ExpandRepeatedEvents", "]"}]}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683976480502823*^9, 
   3.683976634737287*^9}, {3.6839766756355143`*^9, 3.683976688875939*^9}, {
   3.683978094400653*^9, 3.68397814152866*^9}, {3.685033569573359*^9, 
   3.685033569575466*^9}, {3.6851719103554573`*^9, 3.685171975503281*^9}, {
   3.685172339369125*^9, 3.685172341020883*^9}, {3.685172489973385*^9, 
   3.685172520225918*^9}, {3.685196491944193*^9, 3.685196589021195*^9}, {
   3.685196691557366*^9, 3.685196692151346*^9}, {3.685197084908311*^9, 
   3.6851970882027617`*^9}, 3.685197417676444*^9, {3.689411067867564*^9, 
   3.6894110770160418`*^9}, {3.689411108371003*^9, 3.6894111806565323`*^9}, {
   3.6894112139978113`*^9, 3.689411253044654*^9}, {3.68941172256238*^9, 
   3.689411735062286*^9}, {3.689411866460848*^9, 3.689411915677025*^9}, 
   3.689411952487459*^9, {3.689412103147284*^9, 3.689412187654455*^9}, 
   3.691504890959405*^9, {3.691504973739896*^9, 3.691504976698399*^9}, {
   3.691505019403335*^9, 3.691505041463005*^9}, {3.691505249744025*^9, 
   3.691505267886866*^9}},ExpressionUUID->"e452591d-9802-4264-ab6f-\
19442c716c99"],

Cell[BoxData[
 TemplateBox[{
  "EventList","badcharlen",
   "\"The EventList characteristics do not have the correct length.\"",2,724,
   4,18076369408218967604,"Local"},
  "MessageTemplate"]], "Message", "MSG",
 CellChangeTimes->{
  3.6916769102406397`*^9},ExpressionUUID->"bd600057-46c4-4808-86c5-\
ee589a15b5d6"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ExpandRepeatedEvents", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"False", ",", "numst_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"chars", ":", 
          RowBox[{"Repeated", "[", 
           RowBox[{"_", ",", 
            RowBox[{"{", "3", "}"}]}], "]"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"expreptype_", ",", "_", ",", "False"}], "}"}], ",", 
         "morech___"}], "}"}], ",", 
       RowBox[{"events", ":", 
        RowBox[{"{", 
         RowBox[{"_Event", "..."}], "}"}]}], ",", "opts___Rule"}], "]"}], ",",
      "\[IndentingNewLine]", "exprepevopts___Rule"}], "\[IndentingNewLine]", 
    "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"chars", ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"ExpandRepeatedEvents", "/.", 
           RowBox[{"{", 
            RowBox[{"exprepevopts", ",", "opts", ",", 
             RowBox[{"ExpandRepeatedEvents", "\[Rule]", "expreptype"}]}], 
            "}"}]}], "/.", 
          RowBox[{"Options", "[", "EventList", "]"}]}], ",", "True", ",", 
         "False"}], "}"}], ",", "morech"}], "}"}], ",", "\[IndentingNewLine]",
      "events", ",", "\[IndentingNewLine]", 
     RowBox[{"Sequence", "@@", 
      RowBox[{"FilterRules", "[", 
       RowBox[{
        RowBox[{"{", "opts", "}"}], ",", 
        RowBox[{"Except", "[", "ExpandRepeatedEvents", "]"}]}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683976480502823*^9, 
   3.683976634737287*^9}, {3.6839766756355143`*^9, 3.683976688875939*^9}, {
   3.683978094400653*^9, 3.68397814152866*^9}, {3.685033569573359*^9, 
   3.685033569575466*^9}, {3.6851719103554573`*^9, 3.685171975503281*^9}, {
   3.685172339369125*^9, 3.685172341020883*^9}, {3.685172489973385*^9, 
   3.685172520225918*^9}, {3.685196491944193*^9, 3.685196589021195*^9}, {
   3.685196691557366*^9, 3.685196692151346*^9}, {3.685197084908311*^9, 
   3.6851970882027617`*^9}, 3.685197417676444*^9, {3.689411067867564*^9, 
   3.6894110770160418`*^9}, {3.689411108371003*^9, 3.6894111806565323`*^9}, {
   3.6894112139978113`*^9, 3.689411253044654*^9}, {3.68941172256238*^9, 
   3.689411735062286*^9}, {3.689411866460848*^9, 3.689411915677025*^9}, 
   3.689411952487459*^9, {3.689412103147284*^9, 3.689412187654455*^9}, 
   3.691504890959405*^9, {3.691504973739896*^9, 3.691504976698399*^9}, {
   3.691505019403335*^9, 3.691505041463005*^9}, {3.6915052734632683`*^9, 
   3.691505286912215*^9}},ExpressionUUID->"2dd5c432-4421-485c-9feb-\
1b595b4ed946"],

Cell[BoxData[
 TemplateBox[{
  "EventList","badcharlen",
   "\"The EventList characteristics do not have the correct length.\"",2,725,
   5,18076369408218967604,"Local"},
  "MessageTemplate"]], "Message", "MSG",
 CellChangeTimes->{
  3.691676910292667*^9},ExpressionUUID->"e942d7e1-3e48-44d1-b23d-\
27e76802eef0"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ExpandRepeatedEvents", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"evlist", ":", 
      RowBox[{"EventList", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"False", ",", "numst_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"chars", ":", 
           RowBox[{"Repeated", "[", 
            RowBox[{"_", ",", 
             RowBox[{"{", "3", "}"}]}], "]"}]}], ",", "True", ",", 
          "morech___"}], "}"}], ",", 
        RowBox[{"events", ":", 
         RowBox[{"{", 
          RowBox[{"_Event", "..."}], "}"}]}], ",", "opts___Rule"}], "]"}]}], 
     ",", "\[IndentingNewLine]", "exprepevopts___Rule"}], 
    "\[IndentingNewLine]", "]"}], ":=", "evlist"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683972665614608*^9, 3.6839726803028727`*^9}, {
   3.683972732473475*^9, 3.683972824335319*^9}, 3.683972865064205*^9, {
   3.6839738086511183`*^9, 3.683973880704813*^9}, {3.6839739112705708`*^9, 
   3.6839739946006117`*^9}, 3.683974090771921*^9, {3.683976480502823*^9, 
   3.683976634737287*^9}, {3.6839766756355143`*^9, 3.683976688875939*^9}, {
   3.683978094400653*^9, 3.68397814152866*^9}, {3.685033569573359*^9, 
   3.685033569575466*^9}, {3.6851719103554573`*^9, 3.685171975503281*^9}, {
   3.685172339369125*^9, 3.685172341020883*^9}, {3.685172489973385*^9, 
   3.685172520225918*^9}, {3.685196491944193*^9, 3.685196589021195*^9}, {
   3.685196691557366*^9, 3.685196692151346*^9}, {3.685197084908311*^9, 
   3.6851970882027617`*^9}, 3.685197417676444*^9, {3.689411067867564*^9, 
   3.6894110770160418`*^9}, {3.689411108371003*^9, 3.6894111806565323`*^9}, {
   3.6894112139978113`*^9, 3.689411253044654*^9}, {3.68941172256238*^9, 
   3.689411735062286*^9}, {3.689411866460848*^9, 3.689411915677025*^9}, 
   3.689411952487459*^9, {3.689412103147284*^9, 3.689412187654455*^9}, 
   3.691504890959405*^9, {3.691504973739896*^9, 3.691504976698399*^9}, {
   3.691505019403335*^9, 3.691505041463005*^9}, {3.6915052734632683`*^9, 
   3.691505286912215*^9}, {3.6915066658059177`*^9, 
   3.6915066773532877`*^9}},ExpressionUUID->"6410e8e3-79b3-4540-a640-\
4ab17658ba02"],

Cell[BoxData[
 TemplateBox[{
  "EventList","badcharlen",
   "\"The EventList characteristics do not have the correct length.\"",2,726,
   6,18076369408218967604,"Local"},
  "MessageTemplate"]], "Message", "MSG",
 CellChangeTimes->{
  3.691676910342478*^9},ExpressionUUID->"b0d7f722-821f-484b-ade7-\
8f0bc8aba2ff"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["request expansion", "Section",
 CellChangeTimes->{{3.685195397090225*^9, 3.685195408272977*^9}, {
  3.6851955281105633`*^9, 3.685195529549337*^9}, {3.685197541347682*^9, 
  3.685197552338084*^9}, {3.6851977505906773`*^9, 3.685197763436391*^9}, {
  3.6852084898708973`*^9, 3.6852085507879133`*^9}, {3.685210320263929*^9, 
  3.6852103342026978`*^9}, {3.691505168353607*^9, 
  3.69150517216094*^9}},ExpressionUUID->"262509f2-ee28-464f-ab0d-\
1bb79655b8e6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"st", ":", 
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}]}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"chars", ":", 
        RowBox[{"Repeated", "[", 
         RowBox[{"_", ",", 
          RowBox[{"{", "3", "}"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"expreptype", ":", 
          RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "False", ",", 
         "_"}], "}"}], ",", "morech___"}], "}"}], ",", 
     RowBox[{"events", ":", 
      RowBox[{"{", 
       RowBox[{"_Event", "..."}], "}"}]}], ",", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"(", "\[IndentingNewLine]", 
    RowBox[{"EventList", "[", 
     RowBox[{"st", ",", 
      RowBox[{"{", 
       RowBox[{"chars", ",", 
        RowBox[{"{", 
         RowBox[{"expreptype", ",", "True", ",", "False"}], "}"}], ",", 
        "morech"}], "}"}], ",", "events", ",", "opts"}], "]"}], 
    "\[IndentingNewLine]", ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691505198113583*^9, 3.691505213545041*^9}, {
  3.691505311527794*^9, 3.69150535290662*^9}, {3.6915053836604433`*^9, 
  3.691505389183483*^9}, {3.691509981325*^9, 3.691510031842041*^9}, {
  3.691583520110886*^9, 
  3.691583521711458*^9}},ExpressionUUID->"9aadaac9-04fa-4468-8091-\
619fa73ffde5"]
}, Closed]],

Cell[CellGroupData[{

Cell["apply expansion to every Repeat object", "Section",
 CellChangeTimes->{{3.685195397090225*^9, 3.685195408272977*^9}, {
   3.6851955281105633`*^9, 3.685195529549337*^9}, {3.685197541347682*^9, 
   3.685197552338084*^9}, {3.6851977505906773`*^9, 3.685197763436391*^9}, {
   3.6852084898708973`*^9, 3.6852085507879133`*^9}, {3.685210320263929*^9, 
   3.6852103342026978`*^9}, 
   3.69284835905886*^9},ExpressionUUID->"fda7e369-6533-417a-abf5-\
5b7f503ecd04"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventList", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"cha", ":", 
         RowBox[{"Repeated", "[", 
          RowBox[{"_", ",", 
           RowBox[{"{", "3", "}"}]}], "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"expreptype", ":", 
           RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "True", ",", 
          "False"}], "}"}], ",", "morech___"}], "}"}], ",", 
      RowBox[{"events", ":", 
       RowBox[{"{", 
        RowBox[{"_Event", "..."}], "}"}]}], ",", "opts___Rule"}], "]"}], ":=",
     "\[IndentingNewLine]", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{"EventList", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"False", ",", "numst"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"cha", ",", 
         RowBox[{"{", 
          RowBox[{"True", ",", "True"}], "}"}], ",", "morech"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"ExpandRepeatedEvents", "[", 
          RowBox[{"#", ",", "\[IndentingNewLine]", 
           RowBox[{"SimplifyRules", "[", 
            RowBox[{"opts", ",", 
             RowBox[{"ExpandRepeatedEvents", "\[Rule]", "expreptype"}]}], 
            "]"}]}], "\[IndentingNewLine]", "]"}], "&"}], "/@", "events"}], 
       ",", "\[IndentingNewLine]", "opts"}], "\[IndentingNewLine]", "]"}], 
     ")"}]}], ";"}], " "}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6851975645232983`*^9, 3.685197740295822*^9}, 
   3.685197772765085*^9, {3.6852085864638042`*^9, 3.685208812725114*^9}, {
   3.68520884353718*^9, 3.685208845471023*^9}, {3.685208895643512*^9, 
   3.6852091286926413`*^9}, 3.685209158799426*^9, {3.685209208755945*^9, 
   3.685209255653184*^9}, {3.685210263468933*^9, 3.6852102667543507`*^9}, {
   3.68525428046314*^9, 3.68525428541059*^9}, {3.685254342823876*^9, 
   3.685254346724115*^9}, {3.6852545480258217`*^9, 3.6852545551337013`*^9}, {
   3.689412021659944*^9, 3.689412077129919*^9}, 3.691504620912498*^9, {
   3.6915053728149557`*^9, 3.691505379786401*^9}, 3.691505479587533*^9, {
   3.69150927386318*^9, 3.691509299851153*^9}, {3.691510039588078*^9, 
   3.691510053680727*^9}, {3.691583526016219*^9, 
   3.691583527438161*^9}},ExpressionUUID->"b0939484-9370-4cc8-acd3-\
a549b7a130da"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
set expansion status to True if all Event expansions have been done\
\>", "Section",
 CellChangeTimes->{{3.685195397090225*^9, 3.685195408272977*^9}, {
  3.6851955281105633`*^9, 3.685195529549337*^9}, {3.685197541347682*^9, 
  3.685197552338084*^9}, {3.6851977505906773`*^9, 
  3.685197763436391*^9}},ExpressionUUID->"7326574c-4b01-4f9e-9960-\
9dba8c556cf7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventList", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "numst_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"cha", ":", 
         RowBox[{"Repeated", "[", 
          RowBox[{"_", ",", 
           RowBox[{"{", "3", "}"}]}], "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"expreptype_", ",", "True", ",", "False"}], "}"}], ",", 
        "morech___"}], "}"}], ",", 
      RowBox[{"events", ":", 
       RowBox[{"{", 
        RowBox[{"_Event", "..."}], "}"}]}], ",", "opts___Rule"}], "]"}], "/;", 
    RowBox[{"And", "@@", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"MatchQ", "[", 
         RowBox[{"#", ",", "\[IndentingNewLine]", 
          RowBox[{"Event", "[", 
           RowBox[{"_", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Repeated", "[", 
               RowBox[{"_", ",", 
                RowBox[{"{", "4", "}"}]}], "]"}], ",", 
              RowBox[{"True", "|", 
               RowBox[{"{", 
                RowBox[{"_", ",", "True", ",", "True"}], "}"}]}], ",", 
              "___"}], "}"}], ",", "_", ",", "___Rule"}], "]"}]}], "]"}], 
        "&"}], "/@", "events"}], ")"}]}]}], ":=", 
   RowBox[{"EventList", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"False", ",", "numst"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"cha", ",", 
       RowBox[{"{", 
        RowBox[{"expreptype", ",", "True", ",", "True"}], "}"}], ",", 
       "morech"}], "}"}], ",", "events", ",", "opts"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6851975645232983`*^9, 3.685197740295822*^9}, 
   3.685197772765085*^9, {3.6894122250631657`*^9, 3.689412253419739*^9}, {
   3.691068440460731*^9, 
   3.69106844882705*^9}},ExpressionUUID->"88fb44f2-ba8b-4526-8867-\
5c6c71bc3e55"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["debugging routines", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, 
   3.683973033466785*^9, {3.683973750698831*^9, 3.683973753893053*^9}, {
   3.683976432753954*^9, 3.683976436691045*^9}, 3.68397674513378*^9, {
   3.683977981950523*^9, 3.683977988834993*^9}, {3.683978334297957*^9, 
   3.683978335668144*^9}, 3.683979655379014*^9, 3.683980372936699*^9, {
   3.689661184405801*^9, 3.689661197923876*^9}, {3.691435476615394*^9, 
   3.6914354802537823`*^9}, 3.691436746942408*^9, {3.691578910998966*^9, 
   3.691578914207658*^9}},ExpressionUUID->"34ef9abf-a527-44bd-a833-\
12a9fac43ef2"],

Cell[CellGroupData[{

Cell["trap characteristics of incorrect length", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.683959026765203*^9, 3.683959030893498*^9}, {
   3.683959139611781*^9, 3.683959149162888*^9}, {3.683964513177393*^9, 
   3.683964514887501*^9}, 3.6912374833924713`*^9, {3.6915780626169567`*^9, 
   3.6915780839942636`*^9}, {3.691578813830924*^9, 
   3.6915788149875097`*^9}},ExpressionUUID->"5102c673-e515-437d-ac70-\
6e13fd16c530"],

Cell["this is useful for debugging purposes", "Text",
 CellChangeTimes->{{3.6915782465740957`*^9, 
  3.691578252208765*^9}},ExpressionUUID->"d2718061-7712-4c8b-b134-\
cf506f3f088c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventList", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "_"}], "}"}], ",", "chars_List", ",", 
      "more___"}], "]"}], "/;", 
    RowBox[{
     RowBox[{"Length", "[", "chars", "]"}], "!=", 
     RowBox[{"CharacteristicsDimension", "[", "EventList", "]"}]}]}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Message", "[", 
      RowBox[{"EventList", "::", "badcharlen"}], "]"}], ";", "$Failed"}], 
    ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691578093687908*^9, 3.6915781633320017`*^9}, 
   3.691578217861908*^9, {3.691578295840374*^9, 3.6915782993988333`*^9}, {
   3.6915786580120153`*^9, 3.691578682543268*^9}, 3.691578771272049*^9, {
   3.691578835069619*^9, 
   3.691578843971352*^9}},ExpressionUUID->"4e92e5e9-50eb-4480-acfc-\
791ece0f42d7"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["messages", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, 
   3.683973033466785*^9, {3.683973490977272*^9, 
   3.683973492079618*^9}},ExpressionUUID->"6c9e8735-4f4d-4168-b63e-\
e720b4051335"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "::", "badtimepts"}], "=", 
   "\"\<Unable to assign initial or final time points of an event \
list.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683973498852951*^9, 3.6839735139731617`*^9}, {
   3.683973599098839*^9, 3.683973602179441*^9}, {3.683985081204343*^9, 
   3.683985091919984*^9}, 
   3.6850335397025146`*^9},ExpressionUUID->"39317f67-488b-4c70-ba3c-\
e35f306dd260"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "::", "tconflict"}], "=", 
   "\"\<A timing conflict has been generated.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683973498852951*^9, 3.6839735139731617`*^9}, {
   3.683973599098839*^9, 3.683973602179441*^9}, {3.683980705947625*^9, 
   3.683980714326729*^9}, 3.6839850822917843`*^9, 
   3.6850335397067547`*^9},ExpressionUUID->"6f426e21-c800-4846-901b-\
93c5d91c86d2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "::", "backgmismatch"}], "=", 
   "\"\<An EventList contains events with different values for \
BackgroundGenerator.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.690394849268731*^9, 3.690394894772564*^9}, 
   3.709374964901126*^9},ExpressionUUID->"3fd09abd-5d06-484a-8b14-\
d2ae0118d0b1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "::", "cantabsorb"}], "=", 
   "\"\<A mismatch has arisen when trying to absorb an EventList by a \
containing EventList.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.690546193316502*^9, 
  3.690546269558056*^9}},ExpressionUUID->"e2da17c2-0c58-41d6-81a6-\
284f159469a2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventList", "::", "badcharlen"}], "=", 
   "\"\<The EventList characteristics do not have the correct length.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691578179634967*^9, 3.6915782083048773`*^9}, {
  3.6915788646378736`*^9, 
  3.6915788661210546`*^9}},ExpressionUUID->"11cf7bd0-47a4-4cb5-9197-\
fe11a746b21b"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Repeat", "Subtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, {
   3.6839784568941393`*^9, 3.6839784594924717`*^9}, {3.683981558381579*^9, 
   3.683981559470546*^9}, {3.683995601394485*^9, 3.683995601780772*^9}, {
   3.684862236116166*^9, 3.6848622381161547`*^9}, 3.684946401348587*^9, 
   3.692848359851941*^9},ExpressionUUID->"7b2bc00f-d54c-47e5-b739-\
8733ae828ea0"],

Cell[CellGroupData[{

Cell["general", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, {
   3.683973033466785*^9, 3.683973044514587*^9}, {3.6842152838892107`*^9, 
   3.684215286048623*^9}, 3.6915975386670732`*^9, 
   3.691677351716166*^9},ExpressionUUID->"ddc0a198-1bcd-4dfd-9513-\
24ea2e8245ff"],

Cell[CellGroupData[{

Cell["obsolete RepeatEvent", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.459362895307612*^9, 3.4593628998113203`*^9}, {
  3.459365281739501*^9, 3.4593652848141613`*^9}, {3.464332329647097*^9, 
  3.464332333435628*^9}, {3.464348405558358*^9, 3.464348434155554*^9}, {
  3.464974399057695*^9, 3.464974400864071*^9}, {3.684946497137876*^9, 
  3.684946500900744*^9}},ExpressionUUID->"7cf69058-ce00-47e1-863f-\
f2d5ed522fee"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RepeatEvent", "[", "args___", "]"}], ":=", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Message", "[", 
      RowBox[{"RepeatEvent", "::", "usage"}], "]"}], ";", "$Failed"}], 
    ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.68494650302813*^9, 
  3.684946549718858*^9}},ExpressionUUID->"461e7175-2c1c-468d-9302-\
c26232b5640d"]
}, Closed]],

Cell[CellGroupData[{

Cell["defaults", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.459362895307612*^9, 3.4593628998113203`*^9}, {
  3.459365281739501*^9, 3.4593652848141613`*^9}, {3.464332329647097*^9, 
  3.464332333435628*^9}, {3.464348405558358*^9, 3.464348434155554*^9}, {
  3.464974399057695*^9, 
  3.464974400864071*^9}},ExpressionUUID->"ebd52414-4355-40d2-9bc0-\
2e2184ad7d7c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "Repeat", "]"}], "=", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Chronology", "\[RuleDelayed]", "$Chronology"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"ExpandRepeatedEvents", "\[Rule]", "Automatic"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Expand", "\[Rule]", "False"}]}], "\[IndentingNewLine]", "}"}]}],
   ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.464974402231822*^9, 3.464974415442891*^9}, {
   3.465625036909164*^9, 3.465625042590254*^9}, 3.465625080470196*^9, {
   3.465657740406492*^9, 3.4656577834309607`*^9}, 3.465730076429566*^9, {
   3.4657347387759457`*^9, 3.4657347467634783`*^9}, 3.465790322619904*^9, {
   3.465793549521761*^9, 3.465793551122835*^9}, {3.4657936477659388`*^9, 
   3.4657936619337597`*^9}, {3.465793985441434*^9, 3.465793990842483*^9}, {
   3.5031013631921043`*^9, 3.503101363566592*^9}, 3.504268593601696*^9, {
   3.508308162779997*^9, 3.508308165079607*^9}, 3.508313228283949*^9, {
   3.508313438227887*^9, 3.508313443113968*^9}, 3.508313625229313*^9, {
   3.508313672882084*^9, 3.508313676320614*^9}, 3.508339025662491*^9, {
   3.514456471826111*^9, 3.514456482775215*^9}, 3.5147992176760683`*^9, 
   3.54505051714954*^9, {3.683783044841251*^9, 3.683783049590928*^9}, {
   3.684862419791086*^9, 3.684862420068823*^9}, 3.684946401360272*^9, {
   3.6894131747321653`*^9, 3.689413184986969*^9}, 
   3.692848361968247*^9},ExpressionUUID->"545f8faa-5f84-4519-8ed5-\
2b4370853580"]
}, Closed]],

Cell[CellGroupData[{

Cell["output format", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5522104160865507`*^9, 3.5522104203335*^9}, {
   3.5522104538300877`*^9, 3.552210457491542*^9}, {3.6898560312710752`*^9, 
   3.689856034868783*^9}, 
   3.691675820402131*^9},ExpressionUUID->"227953d7-89c0-4895-aae1-\
be2da0545511"],

Cell["abbreviate unless Expand is applied", "Text",
 CellChangeTimes->{{3.684039624335204*^9, 
  3.6840396302626457`*^9}},ExpressionUUID->"4c395e5b-afe2-4ba5-8efe-\
ddfd299cac8b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Format", "[", 
    RowBox[{
     RowBox[{"HoldPattern", "@", 
      RowBox[{"Repeat", "[", 
       RowBox[{"evspecs_", ",", "n_", ",", "repopts___Rule"}], "]"}]}], "/;", 
     
     RowBox[{"!", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Expand", "/.", 
         RowBox[{"{", "repopts", "}"}]}], "/.", 
        RowBox[{"Options", "[", "Repeat", "]"}]}], ")"}]}]}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"RowBox", "[", 
     RowBox[{"{", "\[IndentingNewLine]", 
      RowBox[{"\"\<Repeat[<<..>>,\>\"", "<>", 
       RowBox[{"ToString", "[", "n", "]"}], "<>", "\"\<]\>\""}], 
      "\[IndentingNewLine]", "}"}], "]"}], "//", "DisplayForm"}]}], 
  ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.456506526794569*^9, 3.456506591362196*^9}, {
   3.456506807868026*^9, 3.456506808025032*^9}, {3.456506916908573*^9, 
   3.4565069187852793`*^9}, {3.4565070618491907`*^9, 3.456507073361253*^9}, {
   3.456507104811401*^9, 3.456507132515758*^9}, {3.456507184889974*^9, 
   3.456507193088976*^9}, {3.456507308088336*^9, 3.456507322174239*^9}, {
   3.456507881744112*^9, 3.45650788247845*^9}, {3.45934179309993*^9, 
   3.459341855795616*^9}, {3.459341927566702*^9, 3.459342010299419*^9}, {
   3.459408401457662*^9, 3.4594084060043497`*^9}, {3.459409171653989*^9, 
   3.459409242047933*^9}, {3.459411835080905*^9, 3.4594118370779*^9}, 
   3.4594277811731243`*^9, {3.459427811773448*^9, 3.459427814043268*^9}, {
   3.543516067337283*^9, 3.54351607152363*^9}, {3.5435161166946297`*^9, 
   3.543516150245316*^9}, {3.543944472977648*^9, 3.5439444878167267`*^9}, {
   3.5522103412226353`*^9, 3.552210366421303*^9}, {3.564575196019809*^9, 
   3.56457526757395*^9}, {3.564575331857535*^9, 3.5645753321442413`*^9}, 
   3.5645753719636707`*^9, {3.616751552311964*^9, 3.6167515604540777`*^9}, {
   3.684037961099319*^9, 3.6840379967438803`*^9}, {3.684038319532517*^9, 
   3.6840383256214447`*^9}, {3.684038578653755*^9, 3.684038584867961*^9}, {
   3.6840386857529707`*^9, 3.684038688486998*^9}, {3.684862381790451*^9, 
   3.6848624522451563`*^9}, 3.684862606909531*^9, {3.684946401367641*^9, 
   3.684946401374755*^9}, {3.689856024328103*^9, 3.689856026829156*^9}, {
   3.691668191206423*^9, 3.691668193722704*^9}, {3.69167616177232*^9, 
   3.691676164858625*^9}, {3.691677358761304*^9, 3.691677362312907*^9}, {
   3.691677425785713*^9, 3.691677428974777*^9}, {3.692848363089472*^9, 
   3.6928483646684713`*^9}},ExpressionUUID->"c51a58d6-d314-4dd6-ab2b-\
dd0963e76a80"]
}, Closed]],

Cell[CellGroupData[{

Cell["Expand", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5522104160865507`*^9, 3.5522104203335*^9}, {
  3.5522104538300877`*^9, 3.552210457491542*^9}, {3.684038007663995*^9, 
  3.6840380089830313`*^9}},ExpressionUUID->"571acdfc-0623-4e73-9059-\
b882f98c9957"],

Cell[BoxData[
 RowBox[{
  RowBox[{"UserLevelDependentDefinition", "[", 
   RowBox[{"Expand", ",", "Infinity", ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"Expand", "[", "a_", "]"}], "/;", 
      RowBox[{"!", 
       RowBox[{"FreeQ", "[", 
        RowBox[{"a", ",", "_Repeat"}], "]"}]}]}], ":=", 
     RowBox[{"a", "/.", 
      RowBox[{"{", 
       RowBox[{"repev_Repeat", ":>", 
        RowBox[{"Expand", "[", "repev", "]"}]}], "}"}]}]}]}], 
   "\[IndentingNewLine]", "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6831189892754908`*^9, 3.683119052512022*^9}, {
  3.68311930129748*^9, 3.683119310848708*^9}, {3.684039119191174*^9, 
  3.684039184058563*^9}, {3.684862472330338*^9, 3.6848624903762417`*^9}, {
  3.684946401380826*^9, 3.684946401383492*^9}, {3.69284836547711*^9, 
  3.69284836815832*^9}, {3.745400304652981*^9, 3.745400308245989*^9}, {
  3.745413126025454*^9, 3.745413128523651*^9}, {3.745413211731243*^9, 
  3.745413213671924*^9}},ExpressionUUID->"7cdd9689-4411-4a16-9f0c-\
635cf1d88254"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Repeat", "/:", 
   RowBox[{"Expand", "[", 
    RowBox[{"HoldPattern", "@", 
     RowBox[{"Repeat", "[", 
      RowBox[{"args___", ",", "opts___Rule"}], "]"}]}], "]"}], ":=", 
   RowBox[{"Repeat", "[", 
    RowBox[{"args", ",", 
     RowBox[{"Evaluate", "@", 
      RowBox[{"SimplifyRules", "[", 
       RowBox[{
        RowBox[{"Expand", "\[Rule]", "True"}], ",", "opts"}], "]"}]}]}], 
    "]"}]}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.456506526794569*^9, 3.456506591362196*^9}, {
   3.456506807868026*^9, 3.456506808025032*^9}, {3.456506916908573*^9, 
   3.4565069187852793`*^9}, {3.4565070618491907`*^9, 3.456507073361253*^9}, {
   3.456507104811401*^9, 3.456507132515758*^9}, {3.456507184889974*^9, 
   3.456507193088976*^9}, {3.456507308088336*^9, 3.456507322174239*^9}, {
   3.456507881744112*^9, 3.45650788247845*^9}, {3.45934179309993*^9, 
   3.459341855795616*^9}, {3.459341927566702*^9, 3.459342010299419*^9}, {
   3.459408401457662*^9, 3.4594084060043497`*^9}, {3.459409171653989*^9, 
   3.459409242047933*^9}, {3.459411835080905*^9, 3.4594118370779*^9}, 
   3.4594277811731243`*^9, {3.459427811773448*^9, 3.459427814043268*^9}, {
   3.543516067337283*^9, 3.54351607152363*^9}, {3.5435161166946297`*^9, 
   3.543516150245316*^9}, {3.543944472977648*^9, 3.5439444878167267`*^9}, {
   3.5522103412226353`*^9, 3.552210366421303*^9}, {3.564575196019809*^9, 
   3.56457526757395*^9}, {3.564575331857535*^9, 3.5645753321442413`*^9}, 
   3.5645753719636707`*^9, {3.616751552311964*^9, 3.6167515604540777`*^9}, {
   3.684037961099319*^9, 3.684038061733614*^9}, {3.684038502504335*^9, 
   3.684038504165616*^9}, {3.6848624996809187`*^9, 3.6848625081199617`*^9}, {
   3.684946401392948*^9, 3.684946401397406*^9}, {3.691661786432395*^9, 
   3.691661792398685*^9}, {3.692848369108287*^9, 
   3.692848370375268*^9}},ExpressionUUID->"1aa9b8a0-0bb3-4ab7-96d7-\
62ffbbfb65aa"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Repeat", "/:", 
   RowBox[{"ExpandAll", "[", 
    RowBox[{"HoldPattern", "@", 
     RowBox[{"Repeat", "[", 
      RowBox[{"args___", ",", "opts___Rule"}], "]"}]}], "]"}], ":=", 
   RowBox[{"Repeat", "[", 
    RowBox[{"args", ",", 
     RowBox[{"Evaluate", "@", 
      RowBox[{"SimplifyRules", "[", 
       RowBox[{
        RowBox[{"Expand", "\[Rule]", "True"}], ",", "opts"}], "]"}]}]}], 
    "]"}]}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.456506526794569*^9, 3.456506591362196*^9}, {
   3.456506807868026*^9, 3.456506808025032*^9}, {3.456506916908573*^9, 
   3.4565069187852793`*^9}, {3.4565070618491907`*^9, 3.456507073361253*^9}, {
   3.456507104811401*^9, 3.456507132515758*^9}, {3.456507184889974*^9, 
   3.456507193088976*^9}, {3.456507308088336*^9, 3.456507322174239*^9}, {
   3.456507881744112*^9, 3.45650788247845*^9}, {3.45934179309993*^9, 
   3.459341855795616*^9}, {3.459341927566702*^9, 3.459342010299419*^9}, {
   3.459408401457662*^9, 3.4594084060043497`*^9}, {3.459409171653989*^9, 
   3.459409242047933*^9}, {3.459411835080905*^9, 3.4594118370779*^9}, 
   3.4594277811731243`*^9, {3.459427811773448*^9, 3.459427814043268*^9}, {
   3.543516067337283*^9, 3.54351607152363*^9}, {3.5435161166946297`*^9, 
   3.543516150245316*^9}, {3.543944472977648*^9, 3.5439444878167267`*^9}, {
   3.5522103412226353`*^9, 3.552210366421303*^9}, {3.564575196019809*^9, 
   3.56457526757395*^9}, {3.564575331857535*^9, 3.5645753321442413`*^9}, 
   3.5645753719636707`*^9, {3.616751552311964*^9, 3.6167515604540777`*^9}, {
   3.684037961099319*^9, 3.684038061733614*^9}, {3.684038502504335*^9, 
   3.684038504165616*^9}, {3.6848624996809187`*^9, 3.6848625081199617`*^9}, {
   3.684946401392948*^9, 3.684946401397406*^9}, {3.691661786432395*^9, 
   3.691661800246146*^9}, {3.692848370928138*^9, 
   3.6928483721819344`*^9}},ExpressionUUID->"8ba48ece-63e0-4234-a908-\
6e26966ad5f3"]
}, Open  ]],

Cell[CellGroupData[{

Cell["$Failed", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, {
   3.683973033466785*^9, 3.683973044514587*^9}, {3.6842152838892107`*^9, 
   3.684215286048623*^9}, {3.6915975386670732`*^9, 3.691597584905129*^9}, 
   3.691676086451666*^9, {3.6916774389583273`*^9, 
   3.691677440309701*^9}},ExpressionUUID->"43076762-126f-4899-8438-\
61147831df63"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Repeat", "[", "args___", "]"}], "/;", 
    RowBox[{"!", 
     RowBox[{"FreeQ", "[", 
      RowBox[{
       RowBox[{"{", "args", "}"}], ",", "$Failed"}], "]"}]}]}], ":=", 
   "$Failed"}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.464974402231822*^9, 3.464974415442891*^9}, {
   3.465625036909164*^9, 3.465625042590254*^9}, 3.465625080470196*^9, {
   3.465657740406492*^9, 3.4656577834309607`*^9}, 3.465730076429566*^9, {
   3.4657347387759457`*^9, 3.4657347467634783`*^9}, 3.465790322619904*^9, {
   3.465793549521761*^9, 3.465793551122835*^9}, {3.4657936477659388`*^9, 
   3.4657936619337597`*^9}, {3.465793985441434*^9, 3.465793990842483*^9}, {
   3.5031013631921043`*^9, 3.503101363566592*^9}, 3.504268593601696*^9, {
   3.508308162779997*^9, 3.508308165079607*^9}, 3.508313228283949*^9, {
   3.508313438227887*^9, 3.508313443113968*^9}, 3.508313625229313*^9, {
   3.508313672882084*^9, 3.508313676320614*^9}, 3.508339025662491*^9, {
   3.514456471826111*^9, 3.514456482775215*^9}, 3.5147992176760683`*^9, 
   3.54505051714954*^9, {3.68339391660454*^9, 3.6833939388108063`*^9}, 
   3.684946401404299*^9, {3.691668919144882*^9, 3.691668947276263*^9}, 
   3.6916759936093817`*^9, 3.691676097506621*^9, 
   3.69284837294279*^9},ExpressionUUID->"9b8d6190-1fef-4630-aa39-\
5b40c6e5aca2"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["repetition number", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, {
   3.683973033466785*^9, 3.683973044514587*^9}, {3.6842152838892107`*^9, 
   3.684215286048623*^9}, {3.6915975386670732`*^9, 
   3.6915975562023363`*^9}},ExpressionUUID->"9a124edb-f669-42cb-8fbe-\
4dea0e749d0b"],

Cell[CellGroupData[{

Cell["Automatic", "Subsection",
 CellChangeTimes->{{3.689706704346487*^9, 
  3.689706720200883*^9}},ExpressionUUID->"f7fe34a8-2c90-40de-8a33-\
52aae41fe4d1"],

Cell["used in some special syntaxes eg Signal1D", "Text",
 CellChangeTimes->{{3.689706722665574*^9, 
  3.689706734704185*^9}},ExpressionUUID->"b412286b-52ec-47c9-814d-\
8f2f26041812"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Repeat", "[", 
    RowBox[{"evspecs_", ",", "repopts___Rule"}], "]"}], ":=", 
   RowBox[{"Repeat", "[", 
    RowBox[{"evspecs", ",", "Automatic", ",", "repopts"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.662921293515086*^9, 3.6629213079852037`*^9}, {
  3.684946401412397*^9, 3.6849464014148083`*^9}, {3.6928483735928698`*^9, 
  3.69284837417702*^9}},ExpressionUUID->"8c715c68-521d-4ed3-936b-\
704d233173a0"]
}, Closed]],

Cell[CellGroupData[{

Cell["trap errors", "Subsection",
 CellChangeTimes->{{3.689706704346487*^9, 
  3.689706706369281*^9}},ExpressionUUID->"9571cc32-e0d4-4d13-a9be-\
9a2d1eaad7cf"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Repeat", "[", 
     RowBox[{"evspecs_", ",", "n_", ",", "repopts___Rule"}], "]"}], "/;", 
    RowBox[{"TrueQ", "[", 
     RowBox[{"n", "<", "0"}], "]"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Message", "[", 
      RowBox[{"Repeat", "::", "negn"}], "]"}], ";", "$Failed"}], ")"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5436863215582733`*^9, 3.543686326918457*^9}, {
  3.543686357591632*^9, 3.5436864151150217`*^9}, {3.543734997873802*^9, 
  3.5437350942128897`*^9}, {3.662922112722767*^9, 3.66292211456081*^9}, {
  3.684862838819385*^9, 3.684862839812572*^9}, {3.6848628710612926`*^9, 
  3.684862876069398*^9}, {3.6849464014213953`*^9, 3.684946401424958*^9}, {
  3.6928483748186083`*^9, 
  3.692848375437029*^9}},ExpressionUUID->"097fe774-92c7-4b73-b3b8-\
386e67bc078a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Repeat", "[", 
     RowBox[{"evspecs_", ",", "n_", ",", "repopts___Rule"}], "]"}], "/;", 
    RowBox[{"TrueQ", "[", 
     RowBox[{"!", 
      RowBox[{"Element", "[", 
       RowBox[{"n", ",", "Integers"}], "]"}]}], "]"}]}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Message", "[", 
      RowBox[{"Repeat", "::", "nonintegn"}], "]"}], ";", "$Failed"}], ")"}]}],
   ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5436863215582733`*^9, 3.543686326918457*^9}, {
  3.543686357591632*^9, 3.5436864151150217`*^9}, {3.543734997873802*^9, 
  3.5437350942128897`*^9}, {3.662922112722767*^9, 3.66292211456081*^9}, {
  3.684862838819385*^9, 3.684862839812572*^9}, {3.6848628710612926`*^9, 
  3.684862913921294*^9}, {3.68486299376875*^9, 3.684863026525079*^9}, {
  3.68486311922681*^9, 3.6848631241791077`*^9}, {3.684946401431263*^9, 
  3.6849464014339237`*^9}, {3.692848376053814*^9, 
  3.692848376597805*^9}},ExpressionUUID->"765552b9-97f1-4c95-8c80-\
e3e87c4170e4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Repeat", "[", 
     RowBox[{"evspecs_", ",", "n_", ",", "repopts___Rule"}], "]"}], "/;", 
    RowBox[{"TrueQ", "[", 
     RowBox[{"!", 
      RowBox[{"Element", "[", 
       RowBox[{"n", ",", "Integers"}], "]"}]}], "]"}]}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Message", "[", 
      RowBox[{"Repeat", "::", "nonintegn"}], "]"}], ";", "$Failed"}], ")"}]}],
   ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5436863215582733`*^9, 3.543686326918457*^9}, {
  3.543686357591632*^9, 3.5436864151150217`*^9}, {3.543734997873802*^9, 
  3.5437350942128897`*^9}, {3.662922112722767*^9, 3.66292211456081*^9}, {
  3.684862838819385*^9, 3.684862839812572*^9}, {3.6848628710612926`*^9, 
  3.684862913921294*^9}, {3.68486299376875*^9, 3.684863026525079*^9}, {
  3.68486311922681*^9, 3.6848631241791077`*^9}, {3.684946401431263*^9, 
  3.6849464014339237`*^9}, {3.692848377159004*^9, 
  3.692848377677845*^9}},ExpressionUUID->"63f0cbfc-6dc7-481a-b9b2-\
285b6a18cb51"]
}, Closed]],

Cell[CellGroupData[{

Cell["single repetition", "Subsection",
 CellChangeTimes->{{3.689706704346487*^9, 3.689706706369281*^9}, {
  3.689706754737256*^9, 
  3.6897067623194323`*^9}},ExpressionUUID->"49d7eee2-8387-4567-81f1-\
e3ffd217218b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Repeat", "[", 
    RowBox[{
     RowBox[{"evspecs", ":", "EventSpecificationPattern"}], ",", "1", ",", 
     "repopts___Rule"}], "]"}], ":=", 
   RowBox[{"Event", "[", 
    RowBox[{"evspecs", ",", "repopts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.689744590342255*^9, 3.689744599686234*^9}, {
   3.689745216540147*^9, 3.689745218468296*^9}, 3.69284837822254*^9, {
   3.7092929289282627`*^9, 3.7092929308437*^9}, {3.709293015856823*^9, 
   3.709293016171566*^9}, {3.709293071550367*^9, 3.709293128955158*^9}, {
   3.7197457736352777`*^9, 
   3.719745774633079*^9}},ExpressionUUID->"883d7907-b89f-4fee-8b68-\
14d8ea18034a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Repeat", "[", 
    RowBox[{
     RowBox[{"evspecs", ":", 
      RowBox[{"{", 
       RowBox[{"EventSpecificationPattern", "..."}], "}"}]}], ",", "1", ",", 
     "repopts___Rule"}], "]"}], ":=", 
   RowBox[{
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"#", ",", "repopts"}], "]"}], "&"}], "/@", "evspecs"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.689744590342255*^9, 3.689744599686234*^9}, {
   3.689745216540147*^9, 3.689745218468296*^9}, 3.69284837822254*^9, {
   3.7092929289282627`*^9, 3.7092929308437*^9}, {3.709293015856823*^9, 
   3.709293016171566*^9}, {3.709293071550367*^9, 3.709293128955158*^9}, {
   3.7197457736352777`*^9, 
   3.7197458278714*^9}},ExpressionUUID->"bb351f6a-8c95-4fd4-8a7c-\
0e0690f216f6"]
}, Closed]],

Cell[CellGroupData[{

Cell["null repetitions", "Subsection",
 CellChangeTimes->{{3.689706704346487*^9, 3.689706706369281*^9}, {
  3.689706754737256*^9, 3.689706785551189*^9}, {3.689744525748165*^9, 
  3.689744528864057*^9}},ExpressionUUID->"14552be0-3250-4615-a432-\
a72cd283941e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Repeat", "[", 
    RowBox[{"evspecs_", ",", "0", ",", "repopts___Rule"}], "]"}], ":=", 
   RowBox[{"Event", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"None", ",", "0"}], "}"}], ",", "repopts"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.689744554826995*^9, 3.689744572198524*^9}, {
   3.689745226599001*^9, 3.689745227004971*^9}, 
   3.692848378838284*^9},ExpressionUUID->"a264721b-a51f-4fc8-b409-\
9bf0671e729d"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["implement EventList", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, {
   3.683973033466785*^9, 3.683973044514587*^9}, {3.6842152838892107`*^9, 
   3.684215286048623*^9}, {3.6915975386670732`*^9, 
   3.691597572233128*^9}},ExpressionUUID->"3063970c-14a2-4b9c-9ca8-\
7388d12b45f4"],

Cell["\<\
leave the BackgroundGenerator undefined. It will be imposed when EventList is \
applied to the Repeat object.\
\>", "Text",
 CellChangeTimes->{{3.685171146938402*^9, 3.685171174917099*^9}, 
   3.6928497242227*^9, 
   3.709374965845477*^9},ExpressionUUID->"1b2f9b7c-644c-4439-a170-\
6f14368754ee"],

Cell["The definition below is superseded in Events`PulseSequence", "Text",
 CellChangeTimes->{{3.692850292185247*^9, 
  3.692850305888897*^9}},ExpressionUUID->"b6728fc0-5d32-482a-b93d-\
0d9bed1b971e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Repeat", "[", 
    RowBox[{
     RowBox[{"evspecs", ":", 
      RowBox[{"Except", "[", "_EventList", "]"}]}], ",", 
     RowBox[{"n", ":", 
      RowBox[{"Except", "[", "Automatic", "]"}]}], ",", "repopts___Rule"}], 
    "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Repeat", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{"evspecs", ",", "repopts"}], "]"}], ",", "\[IndentingNewLine]", 
     "n", ",", "\[IndentingNewLine]", "repopts"}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6851690271342583`*^9, 3.6851690381594343`*^9}, {
  3.6851692051471777`*^9, 3.685169254872654*^9}, {3.685169881364065*^9, 
  3.6851699111505537`*^9}, {3.685169990792889*^9, 3.6851699910494947`*^9}, {
  3.691055875481851*^9, 3.69105588352931*^9}, {3.691431773363123*^9, 
  3.691431778016273*^9}, {3.692848379461667*^9, 
  3.692848380037489*^9}},ExpressionUUID->"5ccd78a8-f54c-4dc5-ad64-\
9f765e396a4d"]
}, Closed]],

Cell[CellGroupData[{

Cell["extraction of parameters", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, {
   3.683973033466785*^9, 3.683973044514587*^9}, {3.6842152838892107`*^9, 
   3.684215286048623*^9}, {3.6915975386670732`*^9, 
   3.691597584905129*^9}},ExpressionUUID->"e93aeac6-c5ac-4f70-afe6-\
a49d97872969"],

Cell[CellGroupData[{

Cell["EventDuration", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.459362895307612*^9, 3.4593628998113203`*^9}, {
   3.459365281739501*^9, 3.4593652848141613`*^9}, {3.464332329647097*^9, 
   3.464332333435628*^9}, {3.464348405558358*^9, 3.464348421707734*^9}, 
   3.464348752112537*^9, {3.464355708897279*^9, 3.4643557200139017`*^9}, {
   3.465620456020699*^9, 3.465620461919022*^9}, {3.465620515903688*^9, 
   3.4656205269721327`*^9}, {3.5083056353133507`*^9, 3.50830563894232*^9}, {
   3.508315731194721*^9, 3.508315740352083*^9}, {3.508338933020782*^9, 
   3.508338953707696*^9}, {3.5083390471241007`*^9, 3.5083390534421864`*^9}, 
   3.508342873955846*^9, {3.5099638041435843`*^9, 3.5099638110847*^9}, 
   3.509979894532662*^9, {3.509980105665625*^9, 3.5099801101518507`*^9}, {
   3.514112082613894*^9, 3.5141120878270693`*^9}, {3.5148779616426477`*^9, 
   3.514877970866297*^9}, {3.683394319052099*^9, 3.683394328488966*^9}, 
   3.6837323724449673`*^9, 3.683957673656495*^9, {3.685169018752771*^9, 
   3.685169019325304*^9}, 3.6851692500349083`*^9, {3.6851711365334187`*^9, 
   3.685171140603956*^9}, {3.689007424549198*^9, 3.6890074285643177`*^9}, {
   3.691678891038115*^9, 3.691678895373827*^9}, 
   3.711716423588085*^9},ExpressionUUID->"d917158c-0e8b-44c6-99d4-\
7f93c85a64e5"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"EventDuration", "[", 
     RowBox[{
      RowBox[{"Repeat", "[", 
       RowBox[{"evlist_EventList", ",", "nrep_", ",", "repevopts___Rule"}], 
       "]"}], ",", "opts___Rule"}], "]"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"nrep", "\[Times]", 
    RowBox[{"EventDuration", "[", 
     RowBox[{"evlist", ",", "opts"}], "]"}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.689007431624962*^9, 3.689007485033271*^9}, {
   3.6890075488970947`*^9, 3.6890075641260443`*^9}, {3.689011760781726*^9, 
   3.68901177867319*^9}, {3.6890118364739532`*^9, 3.689011938675509*^9}, {
   3.6890119786929626`*^9, 3.689011978923691*^9}, {3.689012032770831*^9, 
   3.689012077305427*^9}, 3.6894140653270187`*^9, {3.6916762770497723`*^9, 
   3.69167628055194*^9}, {3.6916763496147346`*^9, 3.691676351358768*^9}, {
   3.691678899722348*^9, 3.6916789348850718`*^9}, 3.6928483807115583`*^9, {
   3.71171642361656*^9, 
   3.711716423618065*^9}},ExpressionUUID->"ab75e57a-bae8-4831-8ed6-\
cfae334838d8"]
}, Closed]],

Cell[CellGroupData[{

Cell["PermittedTimeShift", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.459362895307612*^9, 3.4593628998113203`*^9}, {
   3.459365281739501*^9, 3.4593652848141613`*^9}, {3.464332329647097*^9, 
   3.464332333435628*^9}, {3.464348405558358*^9, 3.464348421707734*^9}, 
   3.464348752112537*^9, {3.464355708897279*^9, 3.4643557200139017`*^9}, {
   3.465620456020699*^9, 3.465620461919022*^9}, {3.465620515903688*^9, 
   3.4656205269721327`*^9}, {3.5083056353133507`*^9, 3.50830563894232*^9}, {
   3.508315731194721*^9, 3.508315740352083*^9}, {3.508338933020782*^9, 
   3.508338953707696*^9}, {3.5083390471241007`*^9, 3.5083390534421864`*^9}, 
   3.508342873955846*^9, {3.5099638041435843`*^9, 3.5099638110847*^9}, 
   3.509979894532662*^9, {3.509980105665625*^9, 3.5099801101518507`*^9}, {
   3.514112082613894*^9, 3.5141120878270693`*^9}, {3.5148779616426477`*^9, 
   3.514877970866297*^9}, {3.683394319052099*^9, 3.683394328488966*^9}, 
   3.6837323724449673`*^9, 3.683957673656495*^9, {3.685169018752771*^9, 
   3.685169019325304*^9}, 3.6851692500349083`*^9, {3.6851711365334187`*^9, 
   3.685171140603956*^9}, {3.689007424549198*^9, 
   3.6890074285643177`*^9}},ExpressionUUID->"77a0778a-308f-40af-ba7e-\
e4d83c0f670b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"PermittedTimeShift", "[", 
     RowBox[{
      RowBox[{"Repeat", "[", 
       RowBox[{"evlist_EventList", ",", "nrep_", ",", "repevopts___Rule"}], 
       "]"}], ",", "ptsopts___Rule"}], "]"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"PermittedTimeShift", "[", "evlist", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.689007431624962*^9, 3.689007485033271*^9}, {
   3.6890075488970947`*^9, 3.6890075641260443`*^9}, {3.689011760781726*^9, 
   3.68901177867319*^9}, {3.6890118364739532`*^9, 3.689011938675509*^9}, {
   3.6890119786929626`*^9, 3.689011978923691*^9}, {3.689012032770831*^9, 
   3.689012077305427*^9}, 3.6894140653270187`*^9, {3.6916762770497723`*^9, 
   3.69167628055194*^9}, {3.6916763496147346`*^9, 3.691676351358768*^9}, 
   3.6928483813377953`*^9},ExpressionUUID->"5081d83b-7f3b-4a1a-950e-\
f455d01cef7b"]
}, Closed]],

Cell[CellGroupData[{

Cell["BackgroundGenerator", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.459362895307612*^9, 3.4593628998113203`*^9}, {
   3.459365281739501*^9, 3.4593652848141613`*^9}, {3.464332329647097*^9, 
   3.464332333435628*^9}, {3.464348405558358*^9, 3.464348421707734*^9}, 
   3.464348752112537*^9, {3.464355708897279*^9, 3.4643557200139017`*^9}, {
   3.465620456020699*^9, 3.465620461919022*^9}, {3.465620515903688*^9, 
   3.4656205269721327`*^9}, {3.5083056353133507`*^9, 3.50830563894232*^9}, {
   3.508315731194721*^9, 3.508315740352083*^9}, {3.508338933020782*^9, 
   3.508338953707696*^9}, {3.5083390471241007`*^9, 3.5083390534421864`*^9}, 
   3.508342873955846*^9, {3.5099638041435843`*^9, 3.5099638110847*^9}, 
   3.509979894532662*^9, {3.509980105665625*^9, 3.5099801101518507`*^9}, {
   3.514112082613894*^9, 3.5141120878270693`*^9}, {3.5148779616426477`*^9, 
   3.514877970866297*^9}, {3.683394319052099*^9, 3.683394328488966*^9}, 
   3.6837323724449673`*^9, 3.683957673656495*^9, {3.685169018752771*^9, 
   3.685169019325304*^9}, 3.6851692500349083`*^9, {3.6851711365334187`*^9, 
   3.685171140603956*^9}, {3.689007424549198*^9, 3.6890074285643177`*^9}, {
   3.6894139719525557`*^9, 3.68941397579846*^9}, 
   3.70937496945641*^9},ExpressionUUID->"901db580-8d88-4bf9-8355-\
cf2adf4cafab"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"BackgroundGenerator", "[", 
     RowBox[{
      RowBox[{"Repeat", "[", 
       RowBox[{"evlist_EventList", ",", "nrep_", ",", "repevopts___Rule"}], 
       "]"}], ",", "opts___Rule"}], "]"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"BackgroundGenerator", "[", "evlist", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6894139890449877`*^9, 3.689414017762512*^9}, 
   3.691676287849043*^9, {3.69167634279184*^9, 3.691676344470945*^9}, 
   3.692848382003581*^9, {3.709374970199687*^9, 
   3.709374971122203*^9}},ExpressionUUID->"96a0bd5f-854f-4920-be43-\
b2403fad0bbb"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["messages", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, {
   3.683973033466785*^9, 3.683973044514587*^9}, {3.6842152838892107`*^9, 
   3.684215286048623*^9}, {3.6915975386670732`*^9, 
   3.691597608647683*^9}},ExpressionUUID->"7d3927ad-d297-47f0-9a44-\
7976c9165132"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Repeat", "::", "negn"}], "=", 
   "\"\<The repeat number cannot be negative.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.684862843917007*^9, 3.684862861171624*^9}, 
   3.684946401454391*^9, 
   3.692848382572369*^9},ExpressionUUID->"925b6db4-647f-4b66-9104-\
209d32650a59"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Repeat", "::", "nonintegn"}], "=", 
   "\"\<The repeat number must be an integer.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6848630323012114`*^9, 3.6848630422601137`*^9}, 
   3.6849464014630003`*^9, 
   3.6928483833514423`*^9},ExpressionUUID->"944f84d5-13d3-4cd0-a877-\
0872b12fdd0c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Repeat", "::", "experr"}], "=", 
   "\"\<Error in expanding Repeat object.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.685212464733775*^9, 3.685212476021357*^9}, {
  3.692848384075362*^9, 
  3.692848385236144*^9}},ExpressionUUID->"e3ae4f7e-2386-468d-aa1a-\
899bf860b20b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Repeat", "::", "badargs"}], "=", 
   "\"\<Inappropriate arguments for Repeat.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.69166862209543*^9, 3.691668643478252*^9}, {
  3.6928483859545*^9, 
  3.692848386634582*^9}},ExpressionUUID->"8d612bfe-89a2-403b-9b17-\
0dd1a16bbe14"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["PrecalculatedEvent", "Subtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, {
   3.6839784568941393`*^9, 3.6839784594924717`*^9}, {3.683981558381579*^9, 
   3.683981559470546*^9}, {3.683995601394485*^9, 3.683995601780772*^9}, {
   3.684862236116166*^9, 3.6848622381161547`*^9}, 3.684946401348587*^9, {
   3.691431250085331*^9, 3.691431252226363*^9}, 3.691431527895235*^9, {
   3.691434792107875*^9, 3.69143479413208*^9}, {3.691434836387704*^9, 
   3.691434861449085*^9}, 
   3.691436746595337*^9},ExpressionUUID->"8dc269e2-25dd-416b-a8bf-\
595aec765b35"],

Cell["most coding is inside Event", "Text",
 CellChangeTimes->{{3.691437966048209*^9, 
  3.6914379803029213`*^9}},ExpressionUUID->"2b7fa00f-9003-4e6b-affe-\
c6cab56e64fd"],

Cell[CellGroupData[{

Cell["general", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, {
   3.683973033466785*^9, 3.683973044514587*^9}, {3.6842152838892107`*^9, 
   3.684215286048623*^9}, {3.6915975386670732`*^9, 3.691597584905129*^9}, {
   3.691679184793501*^9, 
   3.691679185823969*^9}},ExpressionUUID->"8b2da559-2e72-4738-ba63-\
ea4dccb1423f"],

Cell[CellGroupData[{

Cell["defaults", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5522104160865507`*^9, 3.5522104203335*^9}, {
   3.5522104538300877`*^9, 3.552210457491542*^9}, {3.6898560312710752`*^9, 
   3.689856034868783*^9}, 3.691675820402131*^9, {3.691679425452197*^9, 
   3.691679426817836*^9}},ExpressionUUID->"96e8d8ef-8433-4251-8ab0-\
f59555d5629b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "PrecalculatedEvent", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"Expand", "\[Rule]", "False"}], "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691679430986268*^9, 
  3.6916794355555677`*^9}},ExpressionUUID->"49e19798-3aae-4f8d-bd91-\
307d8252019a"]
}, Closed]],

Cell[CellGroupData[{

Cell["output format", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5522104160865507`*^9, 3.5522104203335*^9}, {
   3.5522104538300877`*^9, 3.552210457491542*^9}, {3.6898560312710752`*^9, 
   3.689856034868783*^9}, 
   3.691675820402131*^9},ExpressionUUID->"127ef8a1-aff2-4786-9541-\
1403de7dbcc4"],

Cell["\<\
abbreviate unless Expand is applied (* this does not work, for unclear \
reasons *)\
\>", "Text",
 CellChangeTimes->{{3.684039624335204*^9, 3.6840396302626457`*^9}, {
  3.69168037825109*^9, 
  3.691680388685857*^9}},ExpressionUUID->"bafd6600-9203-4725-91bd-\
2dd2a7949f6e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Format", "[", 
    RowBox[{
     RowBox[{"precalev", ":", 
      RowBox[{"PrecalculatedEvent", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Except", "[", "_Rule", "]"}], ".."}], ",", 
        "precalopts___Rule"}], "]"}]}], "/;", 
     RowBox[{"!", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Expand", "/.", 
         RowBox[{"{", "precalopts", "}"}]}], "/.", 
        RowBox[{"Options", "[", "PrecalculatedEvent", "]"}]}], ")"}]}]}], 
    "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"RowBox", "[", 
     RowBox[{"{", 
      RowBox[{"\"\<PrecalculatedEvent[\>\"", ",", 
       RowBox[{"ToBoxes", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"InitialTimePoint", "[", "precalev", "]"}], ",", 
          RowBox[{"FinalTimePoint", "[", "precalev", "]"}]}], "}"}], "]"}], 
       ",", "\[IndentingNewLine]", "\"\<, <<..>>]\>\""}], 
      "\[IndentingNewLine]", "}"}], "]"}], "//", "DisplayForm"}]}], 
  ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.456506526794569*^9, 3.456506591362196*^9}, {
   3.456506807868026*^9, 3.456506808025032*^9}, {3.456506916908573*^9, 
   3.4565069187852793`*^9}, {3.4565070618491907`*^9, 3.456507073361253*^9}, {
   3.456507104811401*^9, 3.456507132515758*^9}, {3.456507184889974*^9, 
   3.456507193088976*^9}, {3.456507308088336*^9, 3.456507322174239*^9}, {
   3.456507881744112*^9, 3.45650788247845*^9}, {3.45934179309993*^9, 
   3.459341855795616*^9}, {3.459341927566702*^9, 3.459342010299419*^9}, {
   3.459408401457662*^9, 3.4594084060043497`*^9}, {3.459409171653989*^9, 
   3.459409242047933*^9}, {3.459411835080905*^9, 3.4594118370779*^9}, 
   3.4594277811731243`*^9, {3.459427811773448*^9, 3.459427814043268*^9}, {
   3.543516067337283*^9, 3.54351607152363*^9}, {3.5435161166946297`*^9, 
   3.543516150245316*^9}, {3.543944472977648*^9, 3.5439444878167267`*^9}, {
   3.5522103412226353`*^9, 3.552210366421303*^9}, {3.564575196019809*^9, 
   3.56457526757395*^9}, {3.564575331857535*^9, 3.5645753321442413`*^9}, 
   3.5645753719636707`*^9, {3.616751552311964*^9, 3.6167515604540777`*^9}, {
   3.684037961099319*^9, 3.6840379967438803`*^9}, {3.684038319532517*^9, 
   3.6840383256214447`*^9}, {3.684038578653755*^9, 3.684038584867961*^9}, {
   3.6840386857529707`*^9, 3.684038688486998*^9}, {3.684862381790451*^9, 
   3.6848624522451563`*^9}, 3.684862606909531*^9, {3.684946401367641*^9, 
   3.684946401374755*^9}, {3.689856024328103*^9, 3.689856026829156*^9}, {
   3.691668191206423*^9, 3.691668193722704*^9}, {3.69167616177232*^9, 
   3.691676164858625*^9}, {3.691677358761304*^9, 3.691677362312907*^9}, {
   3.691677425785713*^9, 3.691677428974777*^9}, {3.691679209869575*^9, 
   3.6916793491430197`*^9}, 3.691679407992011*^9, 3.691679451246915*^9, {
   3.6916795535661077`*^9, 3.691679587603421*^9}, {3.691679704343998*^9, 
   3.6916797045005302`*^9}, {3.691680052696569*^9, 3.691680057253463*^9}, {
   3.7190568826796494`*^9, 
   3.7190569144042053`*^9}},ExpressionUUID->"20209e1b-4ee4-4011-b9b7-\
884a5b356286"]
}, Closed]],

Cell[CellGroupData[{

Cell["Expand", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5522104160865507`*^9, 3.5522104203335*^9}, {
   3.5522104538300877`*^9, 3.552210457491542*^9}, {3.684038007663995*^9, 
   3.6840380089830313`*^9}, {3.691680396114151*^9, 3.691680404687253*^9}, 
   3.745413362271452*^9},ExpressionUUID->"4f5afd21-a67a-4306-9450-\
9f2e2be47769"],

Cell[BoxData[
 RowBox[{
  RowBox[{"UserLevelDependentDefinition", "[", 
   RowBox[{"Expand", ",", "Infinity", ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"Expand", "[", "a_", "]"}], "/;", 
      RowBox[{"!", 
       RowBox[{"FreeQ", "[", 
        RowBox[{"a", ",", "_PrecalculatedEvent"}], "]"}]}]}], ":=", 
     RowBox[{"a", "/.", 
      RowBox[{"{", 
       RowBox[{"precalev_PrecalculatedEvent", ":>", 
        RowBox[{"Expand", "[", "precalev", "]"}]}], "}"}]}]}]}], 
   "\[IndentingNewLine]", "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6831189892754908`*^9, 3.683119052512022*^9}, {
  3.68311930129748*^9, 3.683119310848708*^9}, {3.684039119191174*^9, 
  3.684039184058563*^9}, {3.684862472330338*^9, 3.6848624903762417`*^9}, {
  3.684946401380826*^9, 3.684946401383492*^9}, {3.6916793783701897`*^9, 
  3.691679392513247*^9}, {3.745400315801402*^9, 3.745400319403317*^9}, {
  3.745413134290156*^9, 3.745413137023583*^9}, {3.745413254274107*^9, 
  3.74541325638335*^9}},ExpressionUUID->"7b5f6d8b-6e3c-44b3-9ccd-\
2fef1994e423"],

Cell["code below causes problems and has been disabled", "Text",
 CellChangeTimes->{{3.745413350719241*^9, 
  3.7454133592190523`*^9}},ExpressionUUID->"3cd3e40c-2523-49e8-b65b-\
166402522e1c"],

Cell[BoxData[
 RowBox[{"(*", 
  RowBox[{"UserLevelDependentDefinition", "[", 
   RowBox[{"Expand", ",", "Infinity", ",", 
    RowBox[{
     RowBox[{"Expand", "[", 
      RowBox[{"HoldPattern", "@", 
       RowBox[{"PrecalculatedEvent", "[", 
        RowBox[{
         RowBox[{"args", ":", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Except", "[", "_Rule", "]"}], ".."}], ")"}]}], ",", 
         "opts___Rule"}], "]"}]}], "]"}], ":=", 
     RowBox[{"PrecalculatedEvent", "[", 
      RowBox[{"args", ",", 
       RowBox[{"Evaluate", "@", 
        RowBox[{"SimplifyRules", "[", 
         RowBox[{
          RowBox[{"Expand", "\[Rule]", "True"}], ",", "opts"}], "]"}]}]}], 
      "]"}]}]}], "\[IndentingNewLine]", "]"}], "*)"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.456506526794569*^9, 3.456506591362196*^9}, {
   3.456506807868026*^9, 3.456506808025032*^9}, {3.456506916908573*^9, 
   3.4565069187852793`*^9}, {3.4565070618491907`*^9, 3.456507073361253*^9}, {
   3.456507104811401*^9, 3.456507132515758*^9}, {3.456507184889974*^9, 
   3.456507193088976*^9}, {3.456507308088336*^9, 3.456507322174239*^9}, {
   3.456507881744112*^9, 3.45650788247845*^9}, {3.45934179309993*^9, 
   3.459341855795616*^9}, {3.459341927566702*^9, 3.459342010299419*^9}, {
   3.459408401457662*^9, 3.4594084060043497`*^9}, {3.459409171653989*^9, 
   3.459409242047933*^9}, {3.459411835080905*^9, 3.4594118370779*^9}, 
   3.4594277811731243`*^9, {3.459427811773448*^9, 3.459427814043268*^9}, {
   3.543516067337283*^9, 3.54351607152363*^9}, {3.5435161166946297`*^9, 
   3.543516150245316*^9}, {3.543944472977648*^9, 3.5439444878167267`*^9}, {
   3.5522103412226353`*^9, 3.552210366421303*^9}, {3.564575196019809*^9, 
   3.56457526757395*^9}, {3.564575331857535*^9, 3.5645753321442413`*^9}, 
   3.5645753719636707`*^9, {3.616751552311964*^9, 3.6167515604540777`*^9}, {
   3.684037961099319*^9, 3.684038061733614*^9}, {3.684038502504335*^9, 
   3.684038504165616*^9}, {3.6848624996809187`*^9, 3.6848625081199617`*^9}, {
   3.684946401392948*^9, 3.684946401397406*^9}, {3.691661786432395*^9, 
   3.691661792398685*^9}, {3.6916796483838873`*^9, 3.691679653912397*^9}, {
   3.691679823838313*^9, 3.691679835828553*^9}, {3.6916801094776707`*^9, 
   3.691680118602654*^9}, {3.745400324510696*^9, 3.745400327591958*^9}, {
   3.745413141225718*^9, 3.7454131434229927`*^9}, {3.7454132881740007`*^9, 
   3.7454132902723427`*^9}, {3.745413337478088*^9, 
   3.745413340119128*^9}},ExpressionUUID->"27474e0b-361e-4c86-8009-\
bb3aba1c55a5"],

Cell[BoxData[
 RowBox[{"(*", 
  RowBox[{
   RowBox[{"UserLevelDependentDefinition", "[", 
    RowBox[{"ExpandAll", ",", "Infinity", ",", 
     RowBox[{
      RowBox[{"ExpandAll", "[", 
       RowBox[{"HoldPattern", "@", 
        RowBox[{"PrecalculatedEvent", "[", 
         RowBox[{
          RowBox[{"args", ":", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Except", "[", "_Rule", "]"}], ".."}], ")"}]}], ",", 
          "opts___Rule"}], "]"}]}], "]"}], ":=", 
      RowBox[{"PrecalculatedEvent", "[", 
       RowBox[{"args", ",", 
        RowBox[{"Evaluate", "@", 
         RowBox[{"SimplifyRules", "[", 
          RowBox[{
           RowBox[{"Expand", "\[Rule]", "True"}], ",", "opts"}], "]"}]}]}], 
       "]"}]}]}], "\[IndentingNewLine]", "]"}], ";"}], "*)"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.456506526794569*^9, 3.456506591362196*^9}, {
   3.456506807868026*^9, 3.456506808025032*^9}, {3.456506916908573*^9, 
   3.4565069187852793`*^9}, {3.4565070618491907`*^9, 3.456507073361253*^9}, {
   3.456507104811401*^9, 3.456507132515758*^9}, {3.456507184889974*^9, 
   3.456507193088976*^9}, {3.456507308088336*^9, 3.456507322174239*^9}, {
   3.456507881744112*^9, 3.45650788247845*^9}, {3.45934179309993*^9, 
   3.459341855795616*^9}, {3.459341927566702*^9, 3.459342010299419*^9}, {
   3.459408401457662*^9, 3.4594084060043497`*^9}, {3.459409171653989*^9, 
   3.459409242047933*^9}, {3.459411835080905*^9, 3.4594118370779*^9}, 
   3.4594277811731243`*^9, {3.459427811773448*^9, 3.459427814043268*^9}, {
   3.543516067337283*^9, 3.54351607152363*^9}, {3.5435161166946297`*^9, 
   3.543516150245316*^9}, {3.543944472977648*^9, 3.5439444878167267`*^9}, {
   3.5522103412226353`*^9, 3.552210366421303*^9}, {3.564575196019809*^9, 
   3.56457526757395*^9}, {3.564575331857535*^9, 3.5645753321442413`*^9}, 
   3.5645753719636707`*^9, {3.616751552311964*^9, 3.6167515604540777`*^9}, {
   3.684037961099319*^9, 3.684038061733614*^9}, {3.684038502504335*^9, 
   3.684038504165616*^9}, {3.6848624996809187`*^9, 3.6848625081199617`*^9}, {
   3.684946401392948*^9, 3.684946401397406*^9}, {3.691661786432395*^9, 
   3.691661800246146*^9}, {3.69167965624837*^9, 3.691679659368911*^9}, {
   3.691679843072289*^9, 3.691679850898615*^9}, 3.691680123805328*^9, {
   3.745400333695812*^9, 3.74540033779422*^9}, {3.745413147290618*^9, 
   3.7454131493282413`*^9}},ExpressionUUID->"83fb9d30-a678-4c32-890c-\
854bd591767e"]
}, Closed]],

Cell[CellGroupData[{

Cell["$Failed", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.459362895307612*^9, 3.4593628998113203`*^9}, {
  3.459365281739501*^9, 3.4593652848141613`*^9}, {3.464332329647097*^9, 
  3.464332333435628*^9}, {3.464348405558358*^9, 3.464348434155554*^9}, {
  3.464974399057695*^9, 3.464974400864071*^9}, {3.683393911051723*^9, 
  3.68339391248285*^9}},ExpressionUUID->"dbbf3a36-b433-45ea-b8ec-\
cf793f1fbcb5"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"PrecalculatedEvent", "[", "args___", "]"}], "/;", 
    RowBox[{"!", 
     RowBox[{"FreeQ", "[", 
      RowBox[{
       RowBox[{"{", "args", "}"}], ",", "$Failed"}], "]"}]}]}], ":=", 
   "$Failed"}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.464974402231822*^9, 3.464974415442891*^9}, {
   3.465625036909164*^9, 3.465625042590254*^9}, 3.465625080470196*^9, {
   3.465657740406492*^9, 3.4656577834309607`*^9}, 3.465730076429566*^9, {
   3.4657347387759457`*^9, 3.4657347467634783`*^9}, 3.465790322619904*^9, {
   3.465793549521761*^9, 3.465793551122835*^9}, {3.4657936477659388`*^9, 
   3.4657936619337597`*^9}, {3.465793985441434*^9, 3.465793990842483*^9}, {
   3.5031013631921043`*^9, 3.503101363566592*^9}, 3.504268593601696*^9, {
   3.508308162779997*^9, 3.508308165079607*^9}, 3.508313228283949*^9, {
   3.508313438227887*^9, 3.508313443113968*^9}, 3.508313625229313*^9, {
   3.508313672882084*^9, 3.508313676320614*^9}, 3.508339025662491*^9, {
   3.514456471826111*^9, 3.514456482775215*^9}, 3.5147992176760683`*^9, 
   3.54505051714954*^9, {3.68339391660454*^9, 3.6833939388108063`*^9}, 
   3.684946401404299*^9, {3.6914348200897303`*^9, 3.691434864480197*^9}, 
   3.6914367466039867`*^9},ExpressionUUID->"d34759d0-a137-4558-b2a1-\
20e3547f857c"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["extraction of parameters", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, {
   3.683973033466785*^9, 3.683973044514587*^9}, {3.6842152838892107`*^9, 
   3.684215286048623*^9}, {3.6915975386670732`*^9, 
   3.691597584905129*^9}},ExpressionUUID->"7e7c566e-41ed-4a7b-a413-\
a4c02e77b932"],

Cell[CellGroupData[{

Cell["EventDuration", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.459362895307612*^9, 3.4593628998113203`*^9}, {
   3.459365281739501*^9, 3.4593652848141613`*^9}, {3.464332329647097*^9, 
   3.464332333435628*^9}, {3.464348405558358*^9, 3.464348421707734*^9}, 
   3.464348752112537*^9, {3.464355708897279*^9, 3.4643557200139017`*^9}, {
   3.465620456020699*^9, 3.465620461919022*^9}, {3.465620515903688*^9, 
   3.4656205269721327`*^9}, {3.5083056353133507`*^9, 3.50830563894232*^9}, {
   3.508315731194721*^9, 3.508315740352083*^9}, {3.508338933020782*^9, 
   3.508338953707696*^9}, {3.5083390471241007`*^9, 3.5083390534421864`*^9}, 
   3.508342873955846*^9, {3.5099638041435843`*^9, 3.5099638110847*^9}, 
   3.509979894532662*^9, {3.509980105665625*^9, 3.5099801101518507`*^9}, {
   3.514112082613894*^9, 3.5141120878270693`*^9}, {3.5148779616426477`*^9, 
   3.514877970866297*^9}, {3.683394319052099*^9, 3.683394328488966*^9}, 
   3.6837323724449673`*^9, 3.683957673656495*^9, {3.685169018752771*^9, 
   3.685169019325304*^9}, 3.6851692500349083`*^9, {3.6851711365334187`*^9, 
   3.685171140603956*^9}, {3.689007424549198*^9, 3.6890074285643177`*^9}, {
   3.691678891038115*^9, 3.691678895373827*^9}, 
   3.711716423887985*^9},ExpressionUUID->"1179fb47-5978-432a-8cf8-\
373f479d2b28"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventDuration", "[", 
    RowBox[{
     RowBox[{"PrecalculatedEvent", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"dur_", ",", "___"}], "}"}], ",", "___"}], "}"}], ",", 
       "___"}], "]"}], ",", "___Rule"}], "]"}], ":=", "dur"}], ";"}]], "Input",\

 InitializationCell->True,
 CellChangeTimes->{{3.691497961587887*^9, 3.6914980790290613`*^9}, {
   3.691498221738246*^9, 3.691498223910906*^9}, 
   3.71171642390838*^9},ExpressionUUID->"5a707190-fa62-4b24-81e0-\
0c85ce9b938a"]
}, Closed]],

Cell[CellGroupData[{

Cell["InitialTimePoint", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.459362895307612*^9, 3.4593628998113203`*^9}, {
   3.459365281739501*^9, 3.4593652848141613`*^9}, {3.464332329647097*^9, 
   3.464332333435628*^9}, {3.464348405558358*^9, 3.464348421707734*^9}, 
   3.464348752112537*^9, {3.464355708897279*^9, 3.4643557200139017`*^9}, {
   3.465620456020699*^9, 3.465620461919022*^9}, {3.465620515903688*^9, 
   3.4656205269721327`*^9}, {3.5083056353133507`*^9, 3.50830563894232*^9}, {
   3.508315731194721*^9, 3.508315740352083*^9}, {3.508338933020782*^9, 
   3.508338953707696*^9}, {3.5083390471241007`*^9, 3.5083390534421864`*^9}, 
   3.508342873955846*^9, {3.5099638041435843`*^9, 3.5099638110847*^9}, 
   3.509979894532662*^9, {3.509980105665625*^9, 3.5099801101518507`*^9}, {
   3.514112082613894*^9, 3.5141120878270693`*^9}, {3.5148779616426477`*^9, 
   3.514877970866297*^9}, {3.683394319052099*^9, 3.683394328488966*^9}, 
   3.6837323724449673`*^9, 3.683957673656495*^9, {3.685169018752771*^9, 
   3.685169019325304*^9}, 3.6851692500349083`*^9, {3.6851711365334187`*^9, 
   3.685171140603956*^9}, {3.689007424549198*^9, 3.6890074285643177`*^9}, {
   3.691678891038115*^9, 3.691678895373827*^9}, 
   3.691679120471092*^9},ExpressionUUID->"787fa760-80b3-4c58-8165-\
e2bbdee3f8a1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"InitialTimePoint", "[", 
    RowBox[{
     RowBox[{"PrecalculatedEvent", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"_", ",", 
           RowBox[{"{", 
            RowBox[{"_", ",", "ta_"}], "}"}], ",", "___"}], "}"}], ",", 
         "___"}], "}"}], ",", "___"}], "]"}], ",", "___Rule"}], "]"}], ":=", 
   "ta"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691497961587887*^9, 3.6914980994438562`*^9}, {
  3.691498229578041*^9, 
  3.691498237430956*^9}},ExpressionUUID->"27fb3e26-0901-4544-ad54-\
3a37a6df61ae"]
}, Closed]],

Cell[CellGroupData[{

Cell["FinalTimePoint", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.459362895307612*^9, 3.4593628998113203`*^9}, {
   3.459365281739501*^9, 3.4593652848141613`*^9}, {3.464332329647097*^9, 
   3.464332333435628*^9}, {3.464348405558358*^9, 3.464348421707734*^9}, 
   3.464348752112537*^9, {3.464355708897279*^9, 3.4643557200139017`*^9}, {
   3.465620456020699*^9, 3.465620461919022*^9}, {3.465620515903688*^9, 
   3.4656205269721327`*^9}, {3.5083056353133507`*^9, 3.50830563894232*^9}, {
   3.508315731194721*^9, 3.508315740352083*^9}, {3.508338933020782*^9, 
   3.508338953707696*^9}, {3.5083390471241007`*^9, 3.5083390534421864`*^9}, 
   3.508342873955846*^9, {3.5099638041435843`*^9, 3.5099638110847*^9}, 
   3.509979894532662*^9, {3.509980105665625*^9, 3.5099801101518507`*^9}, {
   3.514112082613894*^9, 3.5141120878270693`*^9}, {3.5148779616426477`*^9, 
   3.514877970866297*^9}, {3.683394319052099*^9, 3.683394328488966*^9}, 
   3.6837323724449673`*^9, 3.683957673656495*^9, {3.685169018752771*^9, 
   3.685169019325304*^9}, 3.6851692500349083`*^9, {3.6851711365334187`*^9, 
   3.685171140603956*^9}, {3.689007424549198*^9, 3.6890074285643177`*^9}, {
   3.691678891038115*^9, 3.691678895373827*^9}, {3.691679120471092*^9, 
   3.691679131301882*^9}},ExpressionUUID->"7ad9d9d8-7a23-432f-a712-\
41e0c96ede00"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FinalTimePoint", "[", 
    RowBox[{
     RowBox[{"PrecalculatedEvent", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"_", ",", 
           RowBox[{"{", 
            RowBox[{"tb_", ",", "_"}], "}"}], ",", "___"}], "}"}], ",", 
         "___"}], "}"}], ",", "___"}], "]"}], ",", "___Rule"}], "]"}], ":=", 
   "tb"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691497961587887*^9, 3.691498119410935*^9}, {
  3.691498232367311*^9, 
  3.691498235279567*^9}},ExpressionUUID->"33dbdee1-8c7d-4f77-9a37-\
4782cc1ab2f9"]
}, Closed]],

Cell[CellGroupData[{

Cell["PermittedTimeShift", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.459362895307612*^9, 3.4593628998113203`*^9}, {
   3.459365281739501*^9, 3.4593652848141613`*^9}, {3.464332329647097*^9, 
   3.464332333435628*^9}, {3.464348405558358*^9, 3.464348421707734*^9}, 
   3.464348752112537*^9, {3.464355708897279*^9, 3.4643557200139017`*^9}, {
   3.465620456020699*^9, 3.465620461919022*^9}, {3.465620515903688*^9, 
   3.4656205269721327`*^9}, {3.5083056353133507`*^9, 3.50830563894232*^9}, {
   3.508315731194721*^9, 3.508315740352083*^9}, {3.508338933020782*^9, 
   3.508338953707696*^9}, {3.5083390471241007`*^9, 3.5083390534421864`*^9}, 
   3.508342873955846*^9, {3.5099638041435843`*^9, 3.5099638110847*^9}, 
   3.509979894532662*^9, {3.509980105665625*^9, 3.5099801101518507`*^9}, {
   3.514112082613894*^9, 3.5141120878270693`*^9}, {3.5148779616426477`*^9, 
   3.514877970866297*^9}, {3.683394319052099*^9, 3.683394328488966*^9}, 
   3.6837323724449673`*^9, 3.683957673656495*^9, {3.685169018752771*^9, 
   3.685169019325304*^9}, 3.6851692500349083`*^9, {3.6851711365334187`*^9, 
   3.685171140603956*^9}, {3.689007424549198*^9, 3.6890074285643177`*^9}, {
   3.691678891038115*^9, 3.691678895373827*^9}, {3.691679120471092*^9, 
   3.691679145272184*^9}},ExpressionUUID->"818788cc-6a46-422a-bd1a-\
5872f800b63e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{
     RowBox[{"PrecalculatedEvent", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"_", ",", "_", ",", "pts_", ",", "___"}], "}"}], ",", 
         "___"}], "}"}], ",", "___"}], "]"}], ",", "___Rule"}], "]"}], ":=", 
   "pts"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691497961587887*^9, 3.691498139400633*^9}, 
   3.6914982409281073`*^9},ExpressionUUID->"7e7e6bd2-2f37-42d9-bfbf-\
86df143c72f0"]
}, Closed]],

Cell[CellGroupData[{

Cell["StandardizedInterval", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.459362895307612*^9, 3.4593628998113203`*^9}, {
   3.459365281739501*^9, 3.4593652848141613`*^9}, {3.464332329647097*^9, 
   3.464332333435628*^9}, {3.464348405558358*^9, 3.464348421707734*^9}, 
   3.464348752112537*^9, {3.464355708897279*^9, 3.4643557200139017`*^9}, {
   3.465620456020699*^9, 3.465620461919022*^9}, {3.465620515903688*^9, 
   3.4656205269721327`*^9}, {3.5083056353133507`*^9, 3.50830563894232*^9}, {
   3.508315731194721*^9, 3.508315740352083*^9}, {3.508338933020782*^9, 
   3.508338953707696*^9}, {3.5083390471241007`*^9, 3.5083390534421864`*^9}, 
   3.508342873955846*^9, {3.5099638041435843`*^9, 3.5099638110847*^9}, 
   3.509979894532662*^9, {3.509980105665625*^9, 3.5099801101518507`*^9}, {
   3.514112082613894*^9, 3.5141120878270693`*^9}, {3.5148779616426477`*^9, 
   3.514877970866297*^9}, {3.683394319052099*^9, 3.683394328488966*^9}, 
   3.6837323724449673`*^9, 3.683957673656495*^9, {3.685169018752771*^9, 
   3.685169019325304*^9}, 3.6851692500349083`*^9, {3.6851711365334187`*^9, 
   3.685171140603956*^9}, {3.689007424549198*^9, 3.6890074285643177`*^9}, {
   3.691678891038115*^9, 3.691678895373827*^9}, {3.691679120471092*^9, 
   3.691679152541789*^9}},ExpressionUUID->"44d3abc0-1a85-4f9f-8ef8-\
d88dee91b5b7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizedInterval", "[", 
    RowBox[{
     RowBox[{"PrecalculatedEvent", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"_", ",", "_", ",", "_", ",", "stdintv_", ",", "___"}], 
          "}"}], ",", "___"}], "}"}], ",", "___"}], "]"}], ",", "___Rule"}], 
    "]"}], ":=", "stdintv"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691497961587887*^9, 3.691498165902439*^9}, 
   3.691498243153412*^9},ExpressionUUID->"2585f3e6-dd39-462e-8f5b-\
7c2cc1a11b11"]
}, Closed]],

Cell[CellGroupData[{

Cell["BackgroundGenerator", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.459362895307612*^9, 3.4593628998113203`*^9}, {
   3.459365281739501*^9, 3.4593652848141613`*^9}, {3.464332329647097*^9, 
   3.464332333435628*^9}, {3.464348405558358*^9, 3.464348421707734*^9}, 
   3.464348752112537*^9, {3.464355708897279*^9, 3.4643557200139017`*^9}, {
   3.465620456020699*^9, 3.465620461919022*^9}, {3.465620515903688*^9, 
   3.4656205269721327`*^9}, {3.5083056353133507`*^9, 3.50830563894232*^9}, {
   3.508315731194721*^9, 3.508315740352083*^9}, {3.508338933020782*^9, 
   3.508338953707696*^9}, {3.5083390471241007`*^9, 3.5083390534421864`*^9}, 
   3.508342873955846*^9, {3.5099638041435843`*^9, 3.5099638110847*^9}, 
   3.509979894532662*^9, {3.509980105665625*^9, 3.5099801101518507`*^9}, {
   3.514112082613894*^9, 3.5141120878270693`*^9}, {3.5148779616426477`*^9, 
   3.514877970866297*^9}, {3.683394319052099*^9, 3.683394328488966*^9}, 
   3.6837323724449673`*^9, 3.683957673656495*^9, {3.685169018752771*^9, 
   3.685169019325304*^9}, 3.6851692500349083`*^9, {3.6851711365334187`*^9, 
   3.685171140603956*^9}, {3.689007424549198*^9, 3.6890074285643177`*^9}, {
   3.691678891038115*^9, 3.691678895373827*^9}, {3.691679120471092*^9, 
   3.691679161750483*^9}, 
   3.709374973026155*^9},ExpressionUUID->"b96d7703-f02c-4531-81d2-\
e9e46aec1562"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"BackgroundGenerator", "[", 
    RowBox[{
     RowBox[{"PrecalculatedEvent", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"tch_", ",", 
         RowBox[{"{", 
          RowBox[{"backg_", ",", "__"}], "}"}], ",", "___"}], "}"}], ",", 
       "___"}], "]"}], ",", "___Rule"}], "]"}], ":=", "backg"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691497961587887*^9, 3.6914980247908287`*^9}, 
   3.691498245086835*^9, 
   3.709374973743685*^9},ExpressionUUID->"a9e04766-10f8-4ab6-b421-\
76388dae603f"]
}, Closed]],

Cell[CellGroupData[{

Cell["HilbertSpaceCompatibility", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.459362895307612*^9, 3.4593628998113203`*^9}, {
   3.459365281739501*^9, 3.4593652848141613`*^9}, {3.464332329647097*^9, 
   3.464332333435628*^9}, {3.464348405558358*^9, 3.464348421707734*^9}, 
   3.464348752112537*^9, {3.464355708897279*^9, 3.4643557200139017`*^9}, {
   3.465620456020699*^9, 3.465620461919022*^9}, {3.465620515903688*^9, 
   3.4656205269721327`*^9}, {3.5083056353133507`*^9, 3.50830563894232*^9}, {
   3.508315731194721*^9, 3.508315740352083*^9}, {3.508338933020782*^9, 
   3.508338953707696*^9}, {3.5083390471241007`*^9, 3.5083390534421864`*^9}, 
   3.508342873955846*^9, {3.5099638041435843`*^9, 3.5099638110847*^9}, 
   3.509979894532662*^9, {3.509980105665625*^9, 3.5099801101518507`*^9}, {
   3.514112082613894*^9, 3.5141120878270693`*^9}, {3.5148779616426477`*^9, 
   3.514877970866297*^9}, {3.683394319052099*^9, 3.683394328488966*^9}, 
   3.6837323724449673`*^9, 3.683957673656495*^9, {3.685169018752771*^9, 
   3.685169019325304*^9}, 3.6851692500349083`*^9, {3.6851711365334187`*^9, 
   3.685171140603956*^9}, {3.689007424549198*^9, 3.6890074285643177`*^9}, {
   3.691678891038115*^9, 3.691678895373827*^9}, {3.691679120471092*^9, 
   3.691679169308956*^9}},ExpressionUUID->"4e7b485c-f80e-4130-8bfa-\
3d966556787c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HilbertSpaceCompatibility", "[", 
    RowBox[{
     RowBox[{"PrecalculatedEvent", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"tch_", ",", "backgch_", ",", "hilbch_", ",", "___"}], "}"}], 
       ",", "___"}], "]"}], ",", "___Rule"}], "]"}], ":=", "hilbch"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691497961587887*^9, 3.6914980247908287`*^9}, {
   3.6914981832406273`*^9, 3.691498199749201*^9}, 
   3.691498246864455*^9},ExpressionUUID->"bb1c7ba5-7527-4b6e-8c37-\
86b0d7d4c62d"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["messages", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, {
   3.683973033466785*^9, 3.683973044514587*^9}, {3.6842152838892107`*^9, 
   3.684215286048623*^9}, {3.6915975386670732`*^9, 3.691597584905129*^9}, 
   3.6916790960544043`*^9},ExpressionUUID->"33f7429c-ad9f-48a0-9634-\
a2bd8b1ebf2a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PrecalculatedEvent", "::", "precalbackgconflict"}], "=", 
   "\"\<A PrecalculatedEvent may only be deployed with the same \
BackgroundGenerator as when it was generated.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6914364613261147`*^9, 3.691436499047127*^9}, {
   3.691436746611574*^9, 3.691436746615328*^9}, {3.6914376412397947`*^9, 
   3.691437666672476*^9}, 
   3.7093749750488234`*^9},ExpressionUUID->"c1b87ac2-063a-4699-a39c-\
e2fd1e4863dc"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PrecalculatedEvent", "::", "precaltimingconflict"}], "=", 
   "\"\<The timing of the PrecalculatedEvent is incompatible with its current \
use.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6914364613261147`*^9, 3.691436666731823*^9}, {
  3.691436746622964*^9, 3.691436746627527*^9}, {3.6914376382471943`*^9, 
  3.691437638755343*^9}},ExpressionUUID->"9266d04d-2b55-40a4-a9dd-\
9d56b1ca7b11"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["computational status", "Subtitle",
 InitializationCell->True,
 CellChangeTimes->{
  3.454575871518672*^9, {3.455454799573387*^9, 3.455454807249361*^9}, {
   3.4639354155697937`*^9, 3.463935420440484*^9}, 3.4639358563258047`*^9, {
   3.4642544189569263`*^9, 3.464254422443118*^9}, {3.464327897371964*^9, 
   3.464327899208976*^9}, 3.464350113189783*^9, {3.5414885543485613`*^9, 
   3.541488570856545*^9}, 3.569514863691033*^9, {3.6831287382470083`*^9, 
   3.6831287738920813`*^9}, {3.6832657719805393`*^9, 3.68326577626611*^9}, {
   3.683265876779852*^9, 3.683265881964329*^9}, {3.68326601584654*^9, 
   3.683266016317121*^9}, {3.683368681532878*^9, 3.68336871152321*^9}, {
   3.683369462677738*^9, 3.6833694693018627`*^9}, 3.689744796718789*^9, {
   3.690449777565691*^9, 3.6904497860243587`*^9}, 3.690555650328928*^9},
 Background->RGBColor[
  1, 1, 0],ExpressionUUID->"6368a012-da4d-48f0-b5ab-49611f086313"],

Cell["these routines may be used on Event or EventList", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.683368461116847*^9, 3.6833684714436483`*^9}, {
   3.6850335309837313`*^9, 3.685033554369426*^9}, 3.685033584850724*^9, 
   3.6850358096357393`*^9, 
   3.685035846826277*^9},ExpressionUUID->"d3b93a9b-5355-4c01-98d9-\
6e3608262357"],

Cell[CellGroupData[{

Cell["NumericalComputationReadyQ ", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, {
   3.683982398078658*^9, 
   3.683982407022208*^9}},ExpressionUUID->"8560a142-d444-43dd-ada6-\
36b54135ece5"],

Cell["coding is within individual routines", "Text",
 CellChangeTimes->{{3.6839956157177963`*^9, 
  3.683995630979714*^9}},ExpressionUUID->"725e92f8-041b-4e12-b965-\
23a66cc17d42"]
}, Closed]],

Cell[CellGroupData[{

Cell["UnassignedSymbols ", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9},
 Background->RGBColor[
  1, 1, 0],ExpressionUUID->"d90527f6-7bba-4c53-899f-e4f351749428"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "UnassignedSymbols", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"PermittedSymbols", "\[Rule]", 
     RowBox[{"{", "}"}]}], "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683963833240959*^9, 
  3.683963849955926*^9}},ExpressionUUID->"486c2945-9b2f-42e9-9976-\
8470425b3f76"],

Cell[CellGroupData[{

Cell["convert non-list arguments", "Subsubsection",
 CellChangeTimes->{{3.6839651717504263`*^9, 3.683965226304183*^9}, {
  3.697876393615052*^9, 
  3.69787639792244*^9}},ExpressionUUID->"710606ff-bd58-4c26-80e6-\
49086a6e3852"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"UnassignedSymbols", "[", 
    RowBox[{
     RowBox[{"x", ":", 
      RowBox[{"Except", "[", "_List", "]"}]}], ",", "more___", ",", 
     "opts___Rule"}], "]"}], ":=", 
   RowBox[{"UnassignedSymbols", "[", 
    RowBox[{
     RowBox[{"{", "x", "}"}], ",", "more", ",", "opts"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.697876402738982*^9, 
  3.697876452061948*^9}},ExpressionUUID->"16f40556-85e8-4d3e-ab5c-\
2262639d15ed"]
}, Closed]],

Cell[CellGroupData[{

Cell["only detect problematic symbols in the Global context", "Subsubsection",
 CellChangeTimes->{{3.6839651717504263`*^9, 
  3.683965226304183*^9}},ExpressionUUID->"dbc7dea9-8aba-47b9-979f-\
8d21351c94fd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"UnassignedSymbols", "[", 
   RowBox[{"arg_", ",", "permittedsymbols_List", ",", "opts___Rule"}], "]"}], 
  ":=", "\[IndentingNewLine]", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"symbols1", ",", "suspectsymbols1", ",", "suspectsymbols2"}], 
     "}"}], ",", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"symbols1", "=", 
       RowBox[{"Union", "@", 
        RowBox[{"Cases", "[", 
         RowBox[{
          RowBox[{"Cases", "[", 
           RowBox[{
            RowBox[{"ReleaseHold", "[", 
             RowBox[{"arg", "/.", 
              RowBox[{"{", 
               RowBox[{"sa_SparseArray", "\[RuleDelayed]", 
                RowBox[{"Normal", "[", "sa", "]"}]}], "}"}]}], "]"}], ",", 
            "_Symbol", ",", "Infinity"}], "]"}], ",", "_Symbol"}], "]"}]}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "the", " ", "curious", " ", "syntax", " ", "above", " ", "ensures", 
        " ", "that", " ", "held", " ", "symbols", " ", "which", " ", "have", 
        " ", "set", " ", "numerical", " ", "arguments", " ", "are", " ", 
        "not", " ", "selected"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"suspectsymbols1", "=", 
       RowBox[{"Select", "[", 
        RowBox[{"symbols1", ",", 
         RowBox[{
          RowBox[{"MatchQ", "[", 
           RowBox[{
            RowBox[{"Context", "[", "#", "]"}], ",", "\"\<Global`\>\""}], 
           "]"}], "&"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"suspectsymbols2", "=", 
       RowBox[{"Select", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"FixedPoint", "[", 
            RowBox[{"Head", ",", "#"}], "]"}], "&"}], "/@", 
          RowBox[{"Union", "@", 
           RowBox[{"Cases", "[", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"ReleaseHold", "[", 
              RowBox[{"arg", "/.", 
               RowBox[{"{", 
                RowBox[{"sa_SparseArray", ":>", 
                 RowBox[{"Normal", "[", "sa", "]"}]}], "}"}]}], "]"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"_", "?", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"!", 
                 RowBox[{"MatchQ", "[", 
                  RowBox[{
                   RowBox[{"Head", "[", "#", "]"}], ",", 
                   RowBox[{
                   "Symbol", "|", "Integer", "|", "Real", "|", "String"}]}], 
                  "]"}]}], "&"}], ")"}]}], ",", "\[IndentingNewLine]", 
             "Infinity"}], "\[IndentingNewLine]", "]"}]}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"MatchQ", "[", 
           RowBox[{
            RowBox[{"Context", "@", "#"}], ",", "\"\<Global`\>\""}], "]"}], 
          "&"}]}], "\[IndentingNewLine]", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Union", "@", 
       RowBox[{"Select", "[", 
        RowBox[{
         RowBox[{"Flatten", "[", 
          RowBox[{"{", 
           RowBox[{"suspectsymbols1", ",", "suspectsymbols2"}], "}"}], "]"}], 
         ",", 
         RowBox[{
          RowBox[{"!", 
           RowBox[{"MemberQ", "[", 
            RowBox[{"permittedsymbols", ",", "#"}], "]"}]}], "&"}]}], 
        "]"}]}]}], "\[IndentingNewLine]", ")"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.683963865383606*^9, {3.683963899855727*^9, 3.6839639024078703`*^9}, {
   3.688837235804742*^9, 3.688837261106283*^9}, {3.697876964442994*^9, 
   3.697877029342849*^9}, {3.7172444609951167`*^9, 
   3.717244475476841*^9}},ExpressionUUID->"616a8968-1c1e-4afa-a683-\
5273534be7d5"]
}, Open  ]],

Cell[CellGroupData[{

Cell["permit any LocalGeneratorSymbols", "Subsubsection",
 CellChangeTimes->{{3.6839651717504263`*^9, 3.683965186667962*^9}, {
   3.6842121803851852`*^9, 3.684212189349101*^9}, {3.684212259456705*^9, 
   3.68421226409914*^9}, {3.7443552328659983`*^9, 3.7443552389731092`*^9}, 
   3.744357524226961*^9, {3.744359910235984*^9, 3.7443599181334763`*^9}, {
   3.744359973039589*^9, 3.7443599753329487`*^9}},
 Background->RGBColor[
  1, 1, 0],ExpressionUUID->"acec8ad6-8dbe-4af1-86d9-9476b7ba2e5c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"UnassignedSymbols", "[", 
    RowBox[{"arg_", ",", "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "permsymb", "}"}], ",", 
     RowBox[{
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{"UnassignedSymbols", "[", 
        RowBox[{"arg", ",", "\[IndentingNewLine]", 
         RowBox[{"SimplifyRules", "[", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"PermittedSymbols", "\[Rule]", 
            RowBox[{"{", "permsymb", "}"}]}], ",", "\[IndentingNewLine]", 
           "opts"}], "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", 
        "]"}], "\[IndentingNewLine]", ")"}], "/;", 
      RowBox[{"(", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"permsymb", "=", 
         RowBox[{
          RowBox[{"PermittedSymbols", "/.", 
           RowBox[{"{", "opts", "}"}]}], "/.", 
          RowBox[{"Options", "[", "UnassignedSymbols", "]"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"!", 
         RowBox[{"MatchQ", "[", 
          RowBox[{"permsymb", ",", "_List"}], "]"}]}]}], 
       "\[IndentingNewLine]", ")"}]}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6839639903452797`*^9, 3.683964083378866*^9}, 
   3.683964125975326*^9},ExpressionUUID->"682eb89c-1527-450a-b505-\
f508fdae849d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"UnassignedSymbols", "[", 
    RowBox[{"arg_", ",", "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "permsymb", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"UnassignedSymbols", "[", "\[IndentingNewLine]", 
        RowBox[{"arg", ",", "\[IndentingNewLine]", 
         RowBox[{"Join", "[", 
          RowBox[{
           RowBox[{"LocalGeneratorSymbols", "[", "arg", "]"}], ",", 
           "permsymb"}], "]"}]}], "\[IndentingNewLine]", "]"}], "/;", 
       RowBox[{"(", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"permsymb", "=", 
          RowBox[{
           RowBox[{"PermittedSymbols", "/.", 
            RowBox[{"{", "opts", "}"}]}], "/.", 
           RowBox[{"Options", "[", "UnassignedSymbols", "]"}]}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"MatchQ", "[", 
          RowBox[{"permsymb", ",", "_List"}], "]"}]}], "\[IndentingNewLine]", 
        ")"}]}], "\[IndentingNewLine]", ")"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683963914716785*^9, 3.683963973355898*^9}, {
   3.6839641114859*^9, 3.68396413716013*^9}, {3.683964326508052*^9, 
   3.6839643436184587`*^9}, {3.6839646951040363`*^9, 3.683964698668799*^9}, {
   3.684212023721937*^9, 3.684212030107594*^9}, {3.684212129736362*^9, 
   3.6842121956156673`*^9}, {3.684212252490361*^9, 3.684212256281784*^9}, {
   3.744355619711594*^9, 3.744355620662942*^9}, 3.744356176663598*^9, 
   3.744356794582279*^9, 3.7443575293328114`*^9, {3.744359847840963*^9, 
   3.744359877034031*^9}, {3.7443651925304317`*^9, 
   3.7443651994766893`*^9}},ExpressionUUID->"6548e9a3-ca3d-4ebb-aa7c-\
ae9c6543f3f9"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ReportSuspectSymbols ", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455119111984703*^9, 3.455119113726385*^9}, {
   3.456338125870762*^9, 3.456338128092794*^9}, {3.464426854630291*^9, 
   3.464426855428462*^9}, {3.541442774600806*^9, 3.5414427905242567`*^9}, 
   3.541488256748209*^9, 3.6833678572992487`*^9, {3.683957788670847*^9, 
   3.683957789873822*^9}, {3.683959370434451*^9, 3.683959372582471*^9}, {
   3.683963462002846*^9, 3.683963467650033*^9}, 3.683963828516594*^9, 
   3.690972679490494*^9},ExpressionUUID->"0945ee8c-d102-420c-9da4-\
8e67987e7341"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ReportSuspectSymbols", "[", 
    RowBox[{"args_", ",", "opts___Rule"}], "]"}], ":=", "\n", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "suspsymbols", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"suspsymbols", "=", 
        RowBox[{"UnassignedSymbols", "[", "args", "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Switch", "[", "\[IndentingNewLine]", 
        RowBox[{"suspsymbols", ",", "\[IndentingNewLine]", 
         RowBox[{"{", "}"}], ",", 
         RowBox[{"Message", "[", 
          RowBox[{"ReportSuspectSymbols", "::", "nosuspsymbol"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", "_", "}"}], ",", 
         RowBox[{"Message", "[", 
          RowBox[{
           RowBox[{"ReportSuspectSymbols", "::", "suspsymbol"}], ",", 
           RowBox[{"First", "@", "suspsymbols"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", "__", "}"}], ",", 
         RowBox[{"Message", "[", 
          RowBox[{
           RowBox[{"ReportSuspectSymbols", "::", "suspsymbols"}], ",", 
           "suspsymbols"}], "]"}], ",", "\[IndentingNewLine]", "_", ",", 
         "Null"}], "]"}], ";"}], "\[IndentingNewLine]", ")"}]}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.690972844635788*^9},ExpressionUUID->"c3eec483-5aae-4152-990d-\
2f29a2a69f72"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ReportSuspectSymbols", "::", "nosuspsymbol"}], "=", 
   "\"\<A non-numeric symbol has not been detected.\>\""}], ";"}]], "Code",
 CellChangeTimes->{{3.690972748384226*^9, 
  3.690972761774702*^9}},ExpressionUUID->"b59f3ec2-5b40-4ec3-ba8d-\
41ca6d11aa11"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ReportSuspectSymbols", "::", "suspsymbol"}], "=", 
   "\"\<The following symbol does not have a numerical value: `1`\>\""}], 
  ";"}]], "Code",
 CellChangeTimes->{{3.690972748384226*^9, 3.690972798212776*^9}, 
   3.691060538442034*^9},ExpressionUUID->"28738460-b069-4be5-8bdd-\
2237705e4a8e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ReportSuspectSymbols", "::", "suspsymbols"}], "=", 
   "\"\<The following symbols do not have a numerical value: `1`\>\""}], 
  ";"}]], "Code",
 CellChangeTimes->{{3.690972748384226*^9, 3.690972761774702*^9}, {
   3.690972805313455*^9, 3.69097281289093*^9}, 
   3.691060542122423*^9},ExpressionUUID->"3c782de1-8846-4e2d-b687-\
548fa8dbb797"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["timing routines", "Subtitle",
 InitializationCell->True,
 CellChangeTimes->{
  3.454575871518672*^9, {3.455454799573387*^9, 3.455454807249361*^9}, {
   3.4639354155697937`*^9, 3.463935420440484*^9}, 3.4639358563258047`*^9, {
   3.4642544189569263`*^9, 3.464254422443118*^9}, {3.464327897371964*^9, 
   3.464327899208976*^9}, 3.464350113189783*^9, {3.5414885543485613`*^9, 
   3.541488570856545*^9}, 3.569514863691033*^9, {3.6831287382470083`*^9, 
   3.6831287738920813`*^9}, {3.6832657719805393`*^9, 3.68326577626611*^9}, {
   3.683265876779852*^9, 3.683265881964329*^9}, {3.68326601584654*^9, 
   3.683266016317121*^9}, {3.683368681532878*^9, 3.68336871152321*^9}, {
   3.683369462677738*^9, 3.6833694693018627`*^9}, 3.689744796718789*^9, 
   3.690449545350794*^9},ExpressionUUID->"67931949-6efc-425c-890f-\
4c123445922c"],

Cell[CellGroupData[{

Cell["EventDuration", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{
  3.454575871518672*^9, {3.455454799573387*^9, 3.455454807249361*^9}, {
   3.4639354155697937`*^9, 3.463935420440484*^9}, 3.4639358563258047`*^9, {
   3.4642544189569263`*^9, 3.464254422443118*^9}, {3.464327897371964*^9, 
   3.464327899208976*^9}, 3.464350113189783*^9, {3.5414885543485613`*^9, 
   3.541488570856545*^9}, 3.569514863691033*^9, {3.6831287382470083`*^9, 
   3.6831287738920813`*^9}, 
   3.711716424177856*^9},ExpressionUUID->"b3676f75-b2bd-47ea-a7cb-\
8253c08a66dc"],

Cell["\<\
includes an option for fast evaluation without validity checks\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.683787836598414*^9, 
  3.683787866713168*^9}},ExpressionUUID->"99e8912c-0596-481b-b722-\
990dd65e9545"],

Cell[CellGroupData[{

Cell["defaults", "Section",
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.430456142359412*^9, 3.430456146975505*^9}, 
   3.507987350544482*^9, {3.683265920824624*^9, 
   3.683265922072591*^9}},ExpressionUUID->"3f183259-7815-4d14-9ada-\
88e90ab91a66"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "EventDuration", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"Check", "\[Rule]", "True"}], "}"}]}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.46279631759762*^9, 3.462796327632659*^9}, {
   3.462796499979474*^9, 3.462796500513019*^9}, {3.4639405693833437`*^9, 
   3.463940573195265*^9}, 3.4648620535854683`*^9, {3.465814564759466*^9, 
   3.465814565148946*^9}, {3.465814951086645*^9, 3.465814954331346*^9}, {
   3.486719403502572*^9, 3.486719404123908*^9}, {3.4973642921097937`*^9, 
   3.497364297163127*^9}, {3.497371897761854*^9, 3.497371906743175*^9}, {
   3.49737232158766*^9, 3.4973723346500463`*^9}, 3.497572542270018*^9, {
   3.507967964539795*^9, 3.5079679692385*^9}, {3.507987360899263*^9, 
   3.50798737104079*^9}, {3.513610381378356*^9, 3.5136103895811357`*^9}, {
   3.513610587662859*^9, 3.513610597164446*^9}, {3.513613408284902*^9, 
   3.513613408634112*^9}, 3.514799182569167*^9, {3.515039742807293*^9, 
   3.5150397476597137`*^9}, {3.515042680120165*^9, 3.515042694597378*^9}, {
   3.5150429590936117`*^9, 3.515042964619792*^9}, {3.51504322905488*^9, 
   3.5150432315719423`*^9}, 3.534398461097693*^9, 3.545050528526733*^9, 
   3.6809554772994223`*^9, {3.683785008025079*^9, 3.6837850086825323`*^9}, {
   3.683786894819295*^9, 3.68378689988735*^9}, 
   3.7117164242088714`*^9},ExpressionUUID->"9797ceec-0c7c-4150-875d-\
295045551ceb"]
}, Closed]],

Cell[CellGroupData[{

Cell["$Failed", "Section",
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.430456142359412*^9, 3.430456146975505*^9}, {
  3.507987346927556*^9, 3.507987347413859*^9}, {3.683785174599112*^9, 
  3.6837851758222733`*^9}},ExpressionUUID->"4178f8f6-b32c-4b7c-a50f-\
611f876ff7b9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EventDuration", "[", "args___", "]"}], "/;", 
    RowBox[{"!", 
     RowBox[{"FreeQ", "[", 
      RowBox[{
       RowBox[{"{", "args", "}"}], ",", "$Failed"}], "]"}]}]}], ":=", 
   "$Failed"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6837851797087812`*^9, 3.6837851944345427`*^9}, 
   3.7117164242148876`*^9},ExpressionUUID->"627f94c6-f084-4e52-b861-\
c86a5eb32d65"]
}, Closed]],

Cell[CellGroupData[{

Cell["main", "Section",
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.430456142359412*^9, 3.430456146975505*^9}, {
  3.507987346927556*^9, 
  3.507987347413859*^9}},ExpressionUUID->"c66375d2-c363-4de2-803e-\
a2f5be268cb3"],

Cell[CellGroupData[{

Cell["standardize if needed", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.502341753419042*^9, 3.502341758834941*^9}, {
  3.5023418337005*^9, 3.5023418366509123`*^9}, {3.5079888632015877`*^9, 
  3.507988866775111*^9}, {3.508003426508595*^9, 3.508003430889907*^9}, {
  3.508005781252936*^9, 3.508005795184581*^9}, {3.508176761564308*^9, 
  3.5081767635387077`*^9}, {3.6837850416045637`*^9, 3.683785046480949*^9}, {
  3.689014030079598*^9, 
  3.689014038011759*^9}},ExpressionUUID->"b2856ef6-41a1-4d1c-9320-\
3ef4e2229a9a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventDuration", "[", 
    RowBox[{
     RowBox[{"ev", ":", 
      RowBox[{"Except", "[", 
       RowBox[{
       "_Event", "|", "_EventList", "|", "_EventSpecifications", "|", 
        RowBox[{"_", "?", "StandardizedEventSpecificationQ"}]}], "]"}]}], ",",
      "opts___Rule"}], "]"}], ":=", 
   RowBox[{"EventDuration", "[", 
    RowBox[{
     RowBox[{"EventSpecifications", "[", "ev", "]"}], ",", "opts"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5023361117189417`*^9, 3.5023361238505087`*^9}, {
   3.502336160186226*^9, 3.502336169290873*^9}, {3.502336295916547*^9, 
   3.502336297636364*^9}, {3.5023416783618517`*^9, 3.502341692526786*^9}, {
   3.502342128607431*^9, 3.502342128780467*^9}, {3.5079870187914658`*^9, 
   3.507987019415697*^9}, 3.5079876422660513`*^9, {3.5079877033662977`*^9, 
   3.50798770572405*^9}, 3.507988885636655*^9, {3.5080034387755957`*^9, 
   3.5080034521251163`*^9}, 3.508003688954329*^9, {3.508003728753807*^9, 
   3.5080037309129753`*^9}, {3.508176767004498*^9, 3.508176774677539*^9}, 
   3.5150441973108673`*^9, {3.6809554883694344`*^9, 3.680955489190909*^9}, {
   3.683785070217565*^9, 3.6837851676312113`*^9}, {3.683785423289983*^9, 
   3.6837854291282797`*^9}, {3.683957680943907*^9, 3.6839576809459133`*^9}, 
   3.684943707614956*^9, {3.684943832451621*^9, 3.6849438336808977`*^9}, {
   3.684943870832203*^9, 3.684943876301632*^9}, {3.685033531214429*^9, 
   3.6850335545495577`*^9}, 3.685033585011944*^9, 3.685035809826807*^9, 
   3.6850359191250143`*^9, {3.68901401418373*^9, 3.689014043094489*^9}, 
   3.7004850089658003`*^9, {3.7117164242178955`*^9, 
   3.7117164242178955`*^9}},ExpressionUUID->"a091509e-9e93-434f-8397-\
2eb5f346542d"]
}, Closed]],

Cell[CellGroupData[{

Cell["EventSpecifications", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.502341753419042*^9, 3.502341758834941*^9}, {
   3.5023418337005*^9, 3.5023418366509123`*^9}, {3.5079888632015877`*^9, 
   3.507988866775111*^9}, {3.508003426508595*^9, 3.508003430889907*^9}, {
   3.508005781252936*^9, 3.508005795184581*^9}, {3.514119336728142*^9, 
   3.514119348170376*^9}, 3.6837855959239492`*^9, 
   3.68395768095975*^9},ExpressionUUID->"64c52543-c4e0-48dc-bdd3-\
bdf164fab0b0"],

Cell["disable checks on elements if already done", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.683787110148237*^9, 
  3.6837871168832684`*^9}},ExpressionUUID->"4d9c79cc-5fac-4995-9aec-\
5a13f5a782c2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventDuration", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"EventSpecifications", "[", 
      RowBox[{"status_", ",", "eventspecs_List", ",", "___Rule"}], "]"}], ",",
      "\[IndentingNewLine]", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Plus", "@@", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"EventDuration", "[", 
        RowBox[{"#", ",", "\[IndentingNewLine]", 
         RowBox[{"Sequence", "@@", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"MatchQ", "[", 
             RowBox[{"status", ",", 
              RowBox[{"True", "|", 
               RowBox[{"{", 
                RowBox[{"True", ",", "__"}], "}"}]}]}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"SimplifyRules", "[", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"Check", "\[Rule]", "False"}], ",", "opts"}], "}"}], 
             "]"}], ",", "\[IndentingNewLine]", 
            RowBox[{"{", "opts", "}"}]}], "\[IndentingNewLine]", "]"}]}]}], 
        "\[IndentingNewLine]", "]"}], "&"}], "/@", "eventspecs"}], 
     "\[IndentingNewLine]", ")"}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5023361117189417`*^9, 3.5023361238505087`*^9}, {
   3.502336160186226*^9, 3.502336169290873*^9}, {3.502336295916547*^9, 
   3.502336297636364*^9}, {3.5023416783618517`*^9, 3.502341692526786*^9}, {
   3.502342128607431*^9, 3.502342128780467*^9}, {3.5079870187914658`*^9, 
   3.507987019415697*^9}, 3.5079876422660513`*^9, {3.5079877033662977`*^9, 
   3.50798770572405*^9}, 3.507988885636655*^9, {3.5080034387755957`*^9, 
   3.5080034521251163`*^9}, 3.508003688954329*^9, {3.508003728753807*^9, 
   3.5080037309129753`*^9}, {3.5086802243402567`*^9, 3.508680227409891*^9}, {
   3.5141193648516893`*^9, 3.514119383726309*^9}, {3.514119423618466*^9, 
   3.514119431626519*^9}, 3.515043389426641*^9, {3.680955490439563*^9, 
   3.6809554922173147`*^9}, {3.683785220866531*^9, 3.683785229367594*^9}, {
   3.683786927183011*^9, 3.683787099627425*^9}, 3.683957680968295*^9, {
   3.68422687711938*^9, 3.684226948656701*^9}, {3.7117164242178955`*^9, 
   3.7117164242178955`*^9}},ExpressionUUID->"24ffc0ab-9926-4280-8847-\
f8bc888c898d"]
}, Closed]],

Cell[CellGroupData[{

Cell["individual event specifications", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.502341753419042*^9, 3.502341758834941*^9}, {
  3.50798883349017*^9, 3.507988842087309*^9}, {3.508003471787965*^9, 
  3.508003479129231*^9}, {3.514119413539898*^9, 3.514119416003693*^9}, {
  3.683785529456365*^9, 3.6837855346539927`*^9}, {3.68378559913269*^9, 
  3.683785601506434*^9}},ExpressionUUID->"162cd0b7-ee31-407b-8319-\
a431c77d3adf"],

Cell[CellGroupData[{

Cell["{evspec, options} event", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.6837871219397707`*^9, 3.6837871230908337`*^9}, {
   3.68423470628473*^9, 3.6842347260824223`*^9}, 
   3.68423476637177*^9},ExpressionUUID->"8affda2f-45af-41cb-9862-\
3c111078bb4f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventDuration", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"ev_", "?", "StandardizedEventSpecificationQ"}], ",", 
       "___Rule"}], "}"}], ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"EventDuration", "[", 
    RowBox[{"ev", ",", "opts"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.684234686421068*^9, 3.6842346917303133`*^9}, {
   3.684234735225362*^9, 3.6842347584419117`*^9}, 3.700485008972683*^9, {
   3.7117164242178955`*^9, 
   3.7117164242345295`*^9}},ExpressionUUID->"2cffb4db-8182-4c4d-bc66-\
d96ad17ec7c8"]
}, Closed]],

Cell[CellGroupData[{

Cell["instantaneous superoperator event", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.6837871219397707`*^9, 3.6837871230908337`*^9}, {
  3.689014249554886*^9, 
  3.689014256709054*^9}},ExpressionUUID->"a0be3577-ca6c-4c5c-81a5-\
cdaef8cbaae9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventDuration", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"_", "?", "SuperoperatorQ"}], ",", "___Rule"}], "}"}], ",", 
     "opts___Rule"}], "]"}], "=", "0"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.689014222124704*^9, {3.689014261282982*^9, 3.689014266193529*^9}, 
   3.7117164242400446`*^9},ExpressionUUID->"0d110f14-ff65-4f96-bbe0-\
7f81c5301858"]
}, Closed]],

Cell[CellGroupData[{

Cell["finite generator event", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.6837871219397707`*^9, 3.6837871230908337`*^9}, {
  3.689014249554886*^9, 
  3.689014288478888*^9}},ExpressionUUID->"ff16dbc2-0b76-452f-b87e-\
9f3910e81ce6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventDuration", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"_", "?", "ValidGeneratorQ"}], ",", 
       RowBox[{"\[Tau]", ":", "DurationPattern"}], ",", "___Rule"}], "}"}], 
     ",", "opts___Rule"}], "]"}], ":=", "\[Tau]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.689014222124704*^9, {3.689014261282982*^9, 3.689014266193529*^9}, {
   3.689014300636682*^9, 3.6890143095789623`*^9}, {3.689014340356399*^9, 
   3.689014342029023*^9}, 3.7093737171707983`*^9, 
   3.711716424245057*^9},ExpressionUUID->"549fcf96-0e8e-489a-8d3e-\
10369956418e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventDuration", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"_", "?", "ValidGeneratorQ"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"tb", ":", "TimePointPattern"}], ",", 
         RowBox[{"ta", ":", "TimePointPattern"}]}], "}"}], ",", "___Rule"}], 
      "}"}], ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"Abs", "[", 
    RowBox[{"tb", "-", "ta"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.689014222124704*^9, {3.689014261282982*^9, 3.689014266193529*^9}, {
   3.689014300636682*^9, 3.689014336758933*^9}, 3.7093737171790743`*^9, 
   3.7117164242500706`*^9},ExpressionUUID->"2306c364-874f-47eb-bdbf-\
111b823706c5"]
}, Closed]],

Cell[CellGroupData[{

Cell["Repeat", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.6837871219397707`*^9, 3.6837871230908337`*^9}, {
   3.689014249554886*^9, 3.689014288478888*^9}, {3.692258462730351*^9, 
   3.6922584650587397`*^9}, 
   3.692848387893922*^9},ExpressionUUID->"93a1ce48-d58a-4b14-99fb-\
db8207e6546b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventDuration", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Repeat", "[", 
        RowBox[{"events_", ",", 
         RowBox[{"nrep", ":", "RepeatNumberPattern"}], ",", "___Rule"}], 
        "]"}], ",", "___Rule"}], "}"}], ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"EventDuration", "[", "events", "]"}], "*", "nrep"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.689014222124704*^9, {3.689014261282982*^9, 3.689014266193529*^9}, {
   3.689014300636682*^9, 3.689014336758933*^9}, {3.689014382919262*^9, 
   3.689014406058462*^9}, 3.691676802159667*^9, 3.692258638274746*^9, 
   3.692848388681304*^9, {3.711716424254583*^9, 
   3.7117164242555847`*^9}},ExpressionUUID->"337be608-f23c-46d4-abdc-\
c78716fc0112"]
}, Closed]],

Cell[CellGroupData[{

Cell["PrecalculatedEvent", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.6837871219397707`*^9, 3.6837871230908337`*^9}, {
  3.689014249554886*^9, 3.689014288478888*^9}, {3.692258462730351*^9, 
  3.692258473352157*^9}},ExpressionUUID->"2abf8c37-7087-41ea-af36-\
2e52e55a9777"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventDuration", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"PrecalculatedEvent", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"T_", ",", "___"}], "}"}], ",", "___"}], "}"}], ",", 
         "___"}], "]"}], ",", "___Rule"}], "}"}], ",", "opts___Rule"}], "]"}],
    ":=", "T"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.691435282734775*^9, 3.691435338856955*^9}, 
   3.691436746633855*^9, 
   3.711716424260598*^9},ExpressionUUID->"10e50f1c-80bb-44eb-8005-\
cb644e04d129"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["EventList", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.502341753419042*^9, 3.502341797354877*^9}, 
   3.50798780072894*^9, {3.683785541910282*^9, 3.6837855505166693`*^9}, 
   3.685033531236208*^9, 
   3.6850335850289383`*^9},ExpressionUUID->"3c7384f4-16a0-4300-b99e-\
ffd128c5f562"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"EventDuration", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"EventList", "[", "\[IndentingNewLine]", 
       RowBox[{"status_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"dur_", ",", "moretiming__"}], "}"}], ",", "___"}], "}"}], 
        ",", "more__", ",", "\[IndentingNewLine]", "evlistopts___Rule"}], 
       "]"}], ",", "\[IndentingNewLine]", "opts___Rule"}], 
     "\[IndentingNewLine]", "]"}]}], ":=", "dur"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.502338731425508*^9, 3.502338742463173*^9}, 
   3.507987025913437*^9, {3.5079877573197603`*^9, 3.507987760722664*^9}, {
   3.51470151592933*^9, 3.514701517862228*^9}, {3.514701549777547*^9, 
   3.514701600793933*^9}, {3.515043847076516*^9, 3.51504385127275*^9}, 
   3.680955499900985*^9, {3.685033531245572*^9, 3.685033554558469*^9}, 
   3.685033585035565*^9, 3.685035809833426*^9, 3.685035920285946*^9, {
   3.689013340161708*^9, 3.68901342556952*^9}, 
   3.7117164242666144`*^9},ExpressionUUID->"75440357-5eca-42cc-a929-\
2d8d0a344576"]
}, Closed]],

Cell[CellGroupData[{

Cell["Event", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.502341753419042*^9, 3.502341797354877*^9}, 
   3.50798780072894*^9, {3.683785541910282*^9, 3.683785560412821*^9}, 
   3.685033554574367*^9, 3.685035809852543*^9, 
   3.685035921289281*^9},ExpressionUUID->"3c3bb6a6-1b36-44f5-911c-\
47f00cd3b0c7"],

Cell[CellGroupData[{

Cell["individual Events", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.502341753419042*^9, 3.502341758834941*^9}, {
  3.5023418337005*^9, 3.5023418366509123`*^9}, {3.5079888632015877`*^9, 
  3.507988866775111*^9}, {3.508003426508595*^9, 3.508003430889907*^9}, {
  3.508005781252936*^9, 3.508005795184581*^9}, {3.514119336728142*^9, 
  3.514119348170376*^9}, {3.5141195193073874`*^9, 
  3.514119523841762*^9}},ExpressionUUID->"3581047a-07d2-4d4a-b480-\
320f6cbe0c11"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"EventDuration", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Event", "[", "\[IndentingNewLine]", 
       RowBox[{"status_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"dur_", ",", "moretiming__"}], "}"}], ",", "___"}], "}"}], 
        ",", "more__", ",", "\[IndentingNewLine]", "evopts___Rule"}], "]"}], 
      ",", "\[IndentingNewLine]", "opts___Rule"}], "\[IndentingNewLine]", 
     "]"}]}], ":=", "dur"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.502338731425508*^9, 3.502338742463173*^9}, 
   3.507987025913437*^9, {3.5079877573197603`*^9, 3.507987760722664*^9}, {
   3.51470151592933*^9, 3.514701517862228*^9}, {3.514701549777547*^9, 
   3.514701600793933*^9}, {3.515043847076516*^9, 3.51504385127275*^9}, 
   3.680955499900985*^9, {3.685033531245572*^9, 3.685033554558469*^9}, 
   3.685033585035565*^9, 3.685035809833426*^9, 3.685035920285946*^9, {
   3.689013340161708*^9, 3.689013462394614*^9}, 
   3.7117164242716274`*^9},ExpressionUUID->"8dbe2cc2-0fa0-4120-8c54-\
1fdae311601e"]
}, Closed]],

Cell[CellGroupData[{

Cell["list of Events", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.502341753419042*^9, 3.502341758834941*^9}, {
  3.5023418337005*^9, 3.5023418366509123`*^9}, {3.5079888632015877`*^9, 
  3.507988866775111*^9}, {3.508003426508595*^9, 3.508003430889907*^9}, {
  3.508005781252936*^9, 3.508005795184581*^9}, {3.514119336728142*^9, 
  3.514119348170376*^9}, {3.5141195193073874`*^9, 3.514119523841762*^9}, {
  3.5147016692920427`*^9, 
  3.514701670242362*^9}},ExpressionUUID->"7e41eb06-bd9a-4e19-9d15-\
c0a76ef224f2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"EventDuration", "[", 
     RowBox[{
      RowBox[{"events", ":", 
       RowBox[{"{", 
        RowBox[{"_Event", ".."}], "}"}]}], ",", "opts___Rule"}], "]"}]}], ":=", 
   RowBox[{"Plus", "@@", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"EventDuration", "[", 
        RowBox[{
         RowBox[{"Evaluate", "[", "#", "]"}], ",", "opts"}], "]"}], "&"}], "/@",
       "events"}], ")"}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5023387009600058`*^9, 3.502338706936269*^9}, 
   3.5079870251531143`*^9, 3.5079877510065193`*^9, 3.5079877832207193`*^9, {
   3.514119582859661*^9, 3.5141195861153393`*^9}, {3.514119635270705*^9, 
   3.5141196385024652`*^9}, {3.514701687168003*^9, 3.514701738363311*^9}, 
   3.515043865094349*^9, {3.680955501532445*^9, 3.680955502860099*^9}, 
   3.685033554591218*^9, 3.6850358098712397`*^9, 3.68503592332312*^9, {
   3.7117164242761393`*^9, 
   3.711716424277643*^9}},ExpressionUUID->"be9ce7cd-87ef-4209-abda-\
15fd41bae6bb"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Superseded use of Duration", "Section",
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.4483607649589663`*^9, 3.448360776920339*^9}, {
   3.455107952027554*^9, 3.455107958728705*^9}, {3.46279631431157*^9, 
   3.462796315357738*^9}, 3.515041734449781*^9, 3.515041840223567*^9, {
   3.680285159316864*^9, 3.680285194378611*^9}, {3.680285325949678*^9, 
   3.680285333469342*^9}},ExpressionUUID->"40836cbb-1856-4e3d-bbd0-\
af89e448c4fa"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$VersionNumber", "<", "11"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Duration", "[", "args___", "]"}], ":=", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Message", "[", 
        RowBox[{"Duration", "::", "usage"}], "]"}], ";", "$Failed"}], 
      ")"}]}]}], "\[IndentingNewLine]", "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.680285208991441*^9, 
  3.680285264570121*^9}},ExpressionUUID->"88623d3d-cdab-4e39-818c-\
8b8e0a7f0296"],

Cell["\<\
for higher Mathematica versions, the symbol Duration is built-in and \
protected, so a different approach has been taken (see \
General`ModifyBuiltIn`).\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.680285283669406*^9, 3.680285321130127*^9}, {
  3.680285453187461*^9, 3.680285481136793*^9}, {3.6832659635209427`*^9, 
  3.683265972395612*^9}},ExpressionUUID->"541c85ec-15bd-4f80-9ef6-\
96b9ad8ac11b"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["PermittedTimeShift", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{{3.455265644416114*^9, 3.455265648200944*^9}, {
   3.464256156522676*^9, 3.4642561730892*^9}, {3.464282095283453*^9, 
   3.4642821290032167`*^9}, {3.503094263066369*^9, 3.503094264574778*^9}, {
   3.503095573470295*^9, 3.503095585674507*^9}, {3.504260287465301*^9, 
   3.504260304480891*^9}, 3.504261075915663*^9, 3.514538950441782*^9, {
   3.541442633712829*^9, 3.541442660772046*^9}, {3.541485412813717*^9, 
   3.541485415659666*^9}},ExpressionUUID->"977312a9-60e9-4516-8649-\
8d2783b42138"],

Cell["\<\
deduces PermittedTimeShift for an Generator, an Event, an \
EventSpecifications, or an EventList, when combined with a specified \
BackgroundGenerator\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.541485463956483*^9, 3.5414854886818037`*^9}, {
   3.541485519717235*^9, 3.541485537012985*^9}, {3.541531534244877*^9, 
   3.5415315674867907`*^9}, 3.683957680879586*^9, {3.68503353110078*^9, 
   3.685033554461671*^9}, 3.685033584934362*^9, 3.685035809730633*^9, 
   3.6850359009583263`*^9, 3.709374514193307*^9, 
   3.709374975970294*^9},ExpressionUUID->"c7ac9279-fa90-4f71-b111-\
3674a5cdab4f"],

Cell[CellGroupData[{

Cell["defaults", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.541488931187296*^9, 3.541488939247919*^9}, {
  3.541489738046503*^9, 3.541489740091172*^9}, {3.541493839856646*^9, 
  3.541493841030037*^9}},ExpressionUUID->"0526a318-a8ff-49cf-9a4a-\
951897fc0cda"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "PermittedTimeShift", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"BackgroundGenerator", "\[Rule]", "None"}], "}"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5414938429129343`*^9, 3.54149385809694*^9}, 
   3.709374976871111*^9},ExpressionUUID->"bd9f9b79-45c0-4877-bb2e-\
d80f09759c02"]
}, Closed]],

Cell[CellGroupData[{

Cell["single evolution generator or event spec", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.541488931187296*^9, 3.541488939247919*^9}, {
  3.541489738046503*^9, 3.541489740091172*^9}, {3.541494020477665*^9, 
  3.5414940414256573`*^9}, {3.541494154122068*^9, 3.5414941634782057`*^9}, {
  3.5414942885866137`*^9, 
  3.541494291150825*^9}},ExpressionUUID->"3d501e7f-56fd-4872-a4e9-\
148c327bf341"],

Cell[CellGroupData[{

Cell["\<\
ignore the BackgroundGenerator if it\[CloseCurlyQuote]s an instantaneous \
superoperator\
\>", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.464696964645693*^9, 3.464696967220211*^9}, {
   3.541485708101438*^9, 3.5414857120002613`*^9}, {3.541485819010337*^9, 
   3.541485819813015*^9}, 3.541485954967263*^9, {3.541493976533239*^9, 
   3.5414939813790903`*^9}, {3.541494027657962*^9, 3.541494029354062*^9}, {
   3.5414941503451*^9, 3.541494150638833*^9}, {3.54149425418596*^9, 
   3.541494256071916*^9}, {3.5414944579582663`*^9, 3.541494460497624*^9}, {
   3.543064929666276*^9, 3.543064956816551*^9}, 
   3.709374977679942*^9},ExpressionUUID->"2949361a-2132-4497-8a9c-\
75ed1524907d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{
     RowBox[{"evgen", ":", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"_", "?", "SuperoperatorQ"}], "|", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"_", "?", "SuperoperatorQ"}], ",", "___opts"}], "}"}]}], 
       ")"}]}], ",", "opts__Rule"}], "]"}], ":=", 
   RowBox[{"PermittedTimeShift", "[", "evgen", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5414940506377583`*^9, 3.541494057067884*^9}, {
  3.541494179991297*^9, 3.5414942465851803`*^9}, {3.54306496149931*^9, 
  3.543065000869474*^9}, {3.5430650601005993`*^9, 
  3.543065071535277*^9}},ExpressionUUID->"5722aca9-2145-4e1f-baa9-\
62b4d6516d29"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
single evolution generator or event spec specifying an BackgroundGenerator\
\>", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.464696964645693*^9, 3.464696967220211*^9}, {
   3.541485708101438*^9, 3.5414857120002613`*^9}, {3.541485819010337*^9, 
   3.541485819813015*^9}, 3.541485954967263*^9, {3.541493976533239*^9, 
   3.5414939813790903`*^9}, {3.541494027657962*^9, 3.541494029354062*^9}, {
   3.5414941503451*^9, 3.541494150638833*^9}, {3.54149425418596*^9, 
   3.541494256071916*^9}, {3.5414944579582663`*^9, 3.541494460497624*^9}, 
   3.7093749783430653`*^9},ExpressionUUID->"d16eddfb-d5ae-46de-b45d-\
042f0e65d5c8"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{
     RowBox[{"evgen_", "?", "ValidGeneratorQ"}], ",", "opts__Rule"}], "]"}], ":=", 
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"evgen", ",", 
     RowBox[{
      RowBox[{"BackgroundGenerator", "/.", 
       RowBox[{"{", "opts", "}"}]}], "/.", 
      RowBox[{"Options", "[", "PermittedTimeShift", "]"}]}]}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5414940506377583`*^9, 3.541494057067884*^9}, {
   3.541494179991297*^9, 3.5414942465851803`*^9}, 3.7093737171946163`*^9, 
   3.709374978952654*^9},ExpressionUUID->"939585e0-5dfb-4156-b065-\
013b5ea607c2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{
     RowBox[{"evspec_", "?", "StandardizedEventSpecificationQ"}], ",", 
     "opts__Rule"}], "]"}], ":=", 
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"evspec", ",", 
     RowBox[{
      RowBox[{"BackgroundGenerator", "/.", 
       RowBox[{"{", "opts", "}"}]}], "/.", 
      RowBox[{"Options", "[", "PermittedTimeShift", "]"}]}]}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5414940506377583`*^9, 3.541494057067884*^9}, {
   3.541494179991297*^9, 3.541494277309804*^9}, 3.700485008979225*^9, 
   3.7093749795937347`*^9},ExpressionUUID->"b7d8b13b-09b6-47f4-b4d7-\
7b43f8fdd9e1"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
single evolution generator or event spec with no BackgroundGenerator specified\
\>", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.464696964645693*^9, 3.464696967220211*^9}, {
   3.541485708101438*^9, 3.5414857120002613`*^9}, {3.541485819010337*^9, 
   3.541485819813015*^9}, 3.541485954967263*^9, {3.541493976533239*^9, 
   3.5414939813790903`*^9}, {3.541494027657962*^9, 3.541494029354062*^9}, {
   3.541494442134089*^9, 3.541494451785626*^9}, 
   3.7093749802893343`*^9},ExpressionUUID->"780c94ef-5a0c-49db-a06c-\
7bc5b98ef380"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", "None", "]"}], "=", "All"}], 
  ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.464427898906756*^9, 3.464427936313549*^9}, {
   3.464428858915824*^9, 3.464428864146068*^9}, {3.464448406760846*^9, 
   3.464448411328624*^9}, {3.464696764849861*^9, 3.464696832819385*^9}, {
   3.4646972448146887`*^9, 3.464697245117278*^9}, 3.4649735205912523`*^9, {
   3.4657301789063587`*^9, 3.4657302347584047`*^9}, {3.465730327929268*^9, 
   3.465730329316383*^9}, {3.465790964549438*^9, 3.4657909718140306`*^9}, 
   3.465811475548271*^9, 3.497572608520342*^9, {3.5414855496346073`*^9, 
   3.541485562887759*^9}},ExpressionUUID->"0faa37e6-f587-4f60-a0d8-\
1fa5b95f5cc2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", "0", "]"}], "=", "All"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.541485593107395*^9, 
  3.541485597345999*^9}},ExpressionUUID->"0cbe11bc-2f59-47ca-9428-\
5f5f6a9fe45c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"op_", "?", "OperatorQ"}], "]"}], "=", "All"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.54148561129249*^9, 
  3.541485613699483*^9}},ExpressionUUID->"68ebfb1e-9166-47d5-ab6a-\
3172bc6d4c36"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"sop_", "?", "SuperoperatorQ"}], "]"}], "=", "All"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5414856297647038`*^9, 
  3.541485649638817*^9}},ExpressionUUID->"234467c0-f2bf-4409-8cac-\
dda4caa50c6c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"sop_", "?", "SuperoperatorQ"}], ",", "evspecopts___Rule"}], 
     "}"}], "]"}], "=", "All"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.541485656544569*^9, 
  3.541485659696007*^9}},ExpressionUUID->"ef3afb71-0cbf-4aea-b396-\
d202bbb45388"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"evgen", ":", 
       RowBox[{"(", 
        RowBox[{"None", "|", 
         RowBox[{"_", "?", "OperatorQ"}], "|", 
         RowBox[{"_", "?", "SuperoperatorQ"}]}], ")"}]}], ",", "timing___", 
      ",", "evspecopts___Rule"}], "}"}], "]"}], "=", "All"}], ";"}]], "Input",\

 InitializationCell->True,
 CellChangeTimes->{
  3.541485675277357*^9},ExpressionUUID->"2316ba5a-3dd4-4228-ac2e-\
2613c7b70e80"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"evgen", ":", 
       RowBox[{"ShapeFunction", "[", 
        RowBox[{"t_", ",", 
         RowBox[{"\[Tau]xpairs_", "?", "MatrixQ"}], ",", "T_", ",", "func_"}],
         "]"}]}], ",", "timing___", ",", "evspecopts___Rule"}], "}"}], "]"}], 
   ":=", "T"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.541485692521317*^9, 
  3.541485721790427*^9}},ExpressionUUID->"1be17f4c-4c90-4a8d-99d1-\
625187654879"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"{", 
     RowBox[{
     "_PrecalculatedPropagator", ",", "timing___", ",", "evspecopts___Rule"}],
      "}"}], "]"}], "=", "None"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5414858282418013`*^9, 
  3.5414858391123123`*^9}},ExpressionUUID->"da794ed6-5aaf-4b63-8c70-\
9564b81b32b7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"ShapeFunction", "[", 
     RowBox[{"t_", ",", 
      RowBox[{"\[Tau]xpairs_", "?", "MatrixQ"}], ",", "T_", ",", "func_"}], 
     "]"}], "]"}], ":=", "T"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.541485692521317*^9, 
  3.541485743975286*^9}},ExpressionUUID->"18fdeaa8-684c-413b-8504-\
c3855e887d50"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"evgen", ":", 
     RowBox[{"(", 
      RowBox[{"_Function", "|", "_PeriodicFunction"}], ")"}]}], "]"}], ":=", 
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"StandardizeGenerator", "[", 
     RowBox[{"evgen", ",", 
      RowBox[{"ImplementMatrixRepresentation", "\[Rule]", "False"}]}], "]"}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.541485692521317*^9, 3.541485774147853*^9}, {
   3.54149390386456*^9, 3.541493907309269*^9}, 
   3.709374087370162*^9},ExpressionUUID->"4285af76-ea8b-4cc6-b0d5-\
196f184a7eb7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"evgen", ":", 
       RowBox[{"(", 
        RowBox[{"_Function", "|", "_PeriodicFunction"}], ")"}]}], ",", 
      "timing___", ",", "evspecopts___Rule"}], "}"}], "]"}], ":=", 
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"StandardizeGenerator", "[", 
     RowBox[{"evgen", ",", 
      RowBox[{"ImplementMatrixRepresentation", "\[Rule]", "False"}]}], "]"}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.541485692521317*^9, 3.541485791597631*^9}, {
   3.5414939133190317`*^9, 3.5414939137427063`*^9}, 
   3.709374087377082*^9},ExpressionUUID->"e8dfbb42-196b-483b-a96c-\
5a17a95a3537"]
}, Closed]],

Cell[CellGroupData[{

Cell["Repeat", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.464696964645693*^9, 3.464696967220211*^9}, {
   3.541485708101438*^9, 3.5414857120002613`*^9}, {3.541485819010337*^9, 
   3.541485819813015*^9}, 3.541485954967263*^9, 3.541487964856385*^9, {
   3.541493939151435*^9, 3.541493943468109*^9}, 3.5415314868063517`*^9, {
   3.6916593853823957`*^9, 3.691659386779435*^9}, 
   3.6928483894937487`*^9},ExpressionUUID->"16be6bbd-acf7-4b46-9652-\
a064e95a5dd5"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"PermittedTimeShift", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Repeat", "[", 
       RowBox[{"evlist_EventList", ",", "nrep_", ",", "repopts___Rule"}], 
       "]"}], ",", "\[IndentingNewLine]", "opts___Rule"}], 
     "\[IndentingNewLine]", "]"}]}], ":=", 
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"evlist", ",", 
     RowBox[{"SimplifyRules", "[", 
      RowBox[{"repopts", ",", "opts"}], "]"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.541485886825441*^9, 3.5414858973581257`*^9}, 
   3.541489076372674*^9, {3.5414892113501*^9, 3.5414892183865*^9}, {
   3.541489303610671*^9, 3.541489321120784*^9}, {3.5415311783138647`*^9, 
   3.541531192116064*^9}, {3.543063474442956*^9, 3.543063476433413*^9}, 
   3.6839576808927317`*^9, 3.685252476958606*^9, {3.691659392142626*^9, 
   3.69165944626455*^9}, 3.691676809623363*^9, 
   3.6928483901761312`*^9},ExpressionUUID->"398ba91a-60a5-414f-a3b4-\
5c9eb6f63bd9"]
}, Closed]],

Cell[CellGroupData[{

Cell["PrecalculatedEvent", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.464696964645693*^9, 3.464696967220211*^9}, {
   3.541485708101438*^9, 3.5414857120002613`*^9}, {3.541485819010337*^9, 
   3.541485819813015*^9}, 3.541485954967263*^9, 3.541487964856385*^9, {
   3.541493939151435*^9, 3.541493943468109*^9}, 3.5415314868063517`*^9, {
   3.69148286598289*^9, 
   3.6914828691735897`*^9}},ExpressionUUID->"11ca9c4a-ecf8-4df7-adde-\
5b10c12b2325"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"PrecalculatedEvent", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"_", ",", "_", ",", "pts_", ",", "_"}], "}"}], ",", "___"}],
         "}"}], ",", "___"}], "]"}], ",", "\[IndentingNewLine]", 
     "opts___Rule"}], "\[IndentingNewLine]", "]"}], ":=", "pts"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.541485886825441*^9, 3.5414858973581257`*^9}, 
   3.541489076372674*^9, {3.5414892113501*^9, 3.5414892183865*^9}, {
   3.541489303610671*^9, 3.541489321120784*^9}, {3.5415311783138647`*^9, 
   3.541531192116064*^9}, {3.541957352226674*^9, 3.54195736047513*^9}, {
   3.543063480611916*^9, 3.543063482633106*^9}, 3.685033531117152*^9, 
   3.6850335849414787`*^9, 3.685252476964303*^9, {3.691482881953265*^9, 
   3.69148292968475*^9}},ExpressionUUID->"0cd6d657-1379-4c2a-8ed8-\
b73d2a86fc53"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["EventSpecifications", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.541488931187296*^9, 3.541488939247919*^9}, {
   3.541489340741947*^9, 3.541489346139209*^9}, {3.541494370052877*^9, 
   3.541494374240769*^9}, {3.541530638712957*^9, 3.541530642555469*^9}, 
   3.683957680907324*^9},ExpressionUUID->"6dfbcaf5-4c5d-4f0b-8051-\
a9b9e28e554b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{
     RowBox[{"HoldPattern", "@", 
      RowBox[{"EventSpecifications", "[", 
       RowBox[{"status_", ",", "evspecs_List", ",", "evspecsopts___Rule"}], 
       "]"}]}], ",", "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{
     RowBox[{"Sequence", "@@", "evspecs"}], ",", "opts"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.541489484073989*^9, 3.541489494038859*^9}, 
   3.5414895299048367`*^9, {3.541529930007936*^9, 3.541529945036165*^9}, {
   3.541530415032969*^9, 3.541530489331279*^9}, {3.5415305346513233`*^9, 
   3.541530574382234*^9}, {3.541531303930119*^9, 3.5415313169942713`*^9}, 
   3.683957680914937*^9, {3.6851918626528788`*^9, 
   3.685191866903213*^9}},ExpressionUUID->"be8cc496-ed62-4a7a-a214-\
cb8e02483c21"]
}, Closed]],

Cell[CellGroupData[{

Cell["single Event", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.541489664506324*^9, 3.541489671249476*^9}, 
   3.5414943901465397`*^9, 3.685033554479939*^9, 3.685035809749316*^9, 
   3.685035901992165*^9},ExpressionUUID->"64d8f673-9c53-4e27-a99b-\
117999f7a190"],

Cell[CellGroupData[{

Cell["\<\
if the BackgroundGenerator is already implemented, use it (and ignore the \
request)\
\>", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.541494479923983*^9, 3.541494488969893*^9}, {
   3.541494577105967*^9, 3.541494590710291*^9}, 
   3.709374980910009*^9},ExpressionUUID->"dd57af5b-9b47-4c9b-9b60-\
c1184626687c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"dur_", ",", "_", ",", 
           RowBox[{"pts", ":", 
            RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "___"}], "}"}], 
         ",", "___"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"evbackg_", ",", "_", ",", "True"}], "}"}], ",", 
       "evopts___Rule"}], "]"}], ",", "opts___Rule"}], "\[IndentingNewLine]", 
    "]"}], ":=", "pts"}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.455457891544944*^9, 3.4554578930481462`*^9}, {
  3.455461685165592*^9, 3.455461697642029*^9}, {3.464448511569353*^9, 
  3.464448515275141*^9}, {3.54148955723843*^9, 3.541489560507629*^9}, {
  3.541493421549021*^9, 3.541493587605392*^9}, {3.541493621453246*^9, 
  3.5414936251250143`*^9}, {3.54306350947436*^9, 3.543063511792615*^9}, {
  3.685033554489943*^9, 3.6850335544925213`*^9}, {3.685035809758225*^9, 
  3.6850358097604933`*^9}, {3.685035902913826*^9, 3.685035903887549*^9}, {
  3.691659517962522*^9, 
  3.691659600842404*^9}},ExpressionUUID->"fe1e3e60-c589-4349-97f9-\
ac9718bd7818"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
if the BackgroundGenerator is not yet implemented, combine with the implicit \
BackgroundGenerator\
\>", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.541494479923983*^9, 3.541494521490118*^9}, {
  3.541494566845183*^9, 3.541494567306343*^9}, {3.691659741820298*^9, 
  3.691659744281492*^9}, {3.7093749816113453`*^9, 
  3.709374982260015*^9}},ExpressionUUID->"35b4ed62-8519-4c39-bc40-\
7c96a8fe3585"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"dur_", ",", "_", ",", "pts_", ",", "___"}], "}"}], ",", 
         "___"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"evbackg", ":", 
          RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "_", ",", 
         "False"}], "}"}], ",", "evopts___Rule"}], "]"}], ",", 
     "opts___Rule"}], "\[IndentingNewLine]", "]"}], ":=", 
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"pts", ",", 
     RowBox[{"PermittedTimeShift", "[", "evbackg", "]"}]}], "]"}]}], 
  ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.455457891544944*^9, 3.4554578930481462`*^9}, {
  3.455461685165592*^9, 3.455461697642029*^9}, {3.464448511569353*^9, 
  3.464448515275141*^9}, {3.54148955723843*^9, 3.541489560507629*^9}, {
  3.541493421549021*^9, 3.541493587605392*^9}, {3.541493621453246*^9, 
  3.5414936251250143`*^9}, {3.54306350947436*^9, 3.543063511792615*^9}, {
  3.685033554489943*^9, 3.6850335544925213`*^9}, {3.685035809758225*^9, 
  3.6850358097604933`*^9}, {3.685035902913826*^9, 3.685035903887549*^9}, {
  3.691659517962522*^9, 3.691659669224107*^9}, {3.691659706986476*^9, 
  3.6916597093653927`*^9}},ExpressionUUID->"a4aace2d-6bbb-49bf-9c00-\
98fb18d4125b"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
if the internal BackgroundGenerator is not yet defined, combine with the \
requested options\
\>", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.541494479923983*^9, 3.541494521490118*^9}, {
   3.541494566845183*^9, 3.541494567306343*^9}, {3.691659741820298*^9, 
   3.6916597655512342`*^9}, {3.6916598579507513`*^9, 3.691659860741043*^9}, 
   3.7093749829562273`*^9},ExpressionUUID->"fc2e5acc-66b4-4ea0-86b1-\
f7ce981f759a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"status_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"dur_", ",", "_", ",", "pts_", ",", "___"}], "}"}], ",", 
         "___"}], "}"}], ",", 
       RowBox[{"Undefined", "|", 
        RowBox[{"{", 
         RowBox[{"Undefined", ",", "___"}], "}"}]}], ",", "evopts___Rule"}], 
      "]"}], ",", "opts___Rule"}], "\[IndentingNewLine]", "]"}], ":=", 
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"pts", ",", "opts"}], "]"}]}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.455457891544944*^9, 3.4554578930481462`*^9}, {
  3.455461685165592*^9, 3.455461697642029*^9}, {3.464448511569353*^9, 
  3.464448515275141*^9}, {3.54148955723843*^9, 3.541489560507629*^9}, {
  3.541493421549021*^9, 3.541493587605392*^9}, {3.541493621453246*^9, 
  3.5414936251250143`*^9}, {3.54306350947436*^9, 3.543063511792615*^9}, {
  3.685033554489943*^9, 3.6850335544925213`*^9}, {3.685035809758225*^9, 
  3.6850358097604933`*^9}, {3.685035902913826*^9, 3.685035903887549*^9}, {
  3.691659517962522*^9, 3.691659669224107*^9}, {3.691659706986476*^9, 
  3.6916597093653927`*^9}, {3.691659769354992*^9, 
  3.6916597852554703`*^9}},ExpressionUUID->"7ca1f8e6-dce9-4abb-b275-\
00a1b18dc3d4"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["list of Events", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.541489664506324*^9, 3.541489671249476*^9}, 
   3.5414943901465397`*^9, {3.543063198196208*^9, 
   3.5430632015467854`*^9}},ExpressionUUID->"7d1a1e64-6f7e-4768-9dfb-\
eabce37cce77"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{
     RowBox[{"evlist", ":", 
      RowBox[{"{", 
       RowBox[{"_Event", ".."}], "}"}]}], ",", "opts___Rule"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"CommonPeriod", "@@", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"PermittedTimeShift", "[", 
        RowBox[{"#", ",", "opts"}], "]"}], "&"}], "/@", "evlist"}], ")"}]}]}],
   ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.543063206573311*^9, 3.543063277781221*^9}, 
   3.685033554509384*^9, 3.685035809779405*^9, 
   3.6850359071353197`*^9},ExpressionUUID->"cafd0ec7-f4f2-4384-b420-\
8b79aca15aa3"]
}, Closed]],

Cell[CellGroupData[{

Cell["EventList", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.541488931187296*^9, 3.541488939247919*^9}, {
   3.541489340741947*^9, 3.541489346139209*^9}, 3.541489511195188*^9, {
   3.541489564835805*^9, 3.541489566120036*^9}, 3.54148966838201*^9, 
   3.541494393688336*^9, {3.541494622967675*^9, 3.541494623222911*^9}, 
   3.5415311478019743`*^9, 3.685033531138105*^9, 
   3.6850335849566593`*^9},ExpressionUUID->"bf1b7288-25b1-4515-910e-\
dc370138a193"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"HoldPattern", "@", 
      RowBox[{"EventList", "[", 
       RowBox[{
        RowBox[{"events", ":", 
         RowBox[{"{", 
          RowBox[{"_Event", "..."}], "}"}]}], ",", "timing_", ",", 
        "evlistopts___Rule"}], "]"}]}], ",", "\[IndentingNewLine]", 
     "opts___Rule"}], "\[IndentingNewLine]", "]"}], ":=", 
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{
     RowBox[{"Sequence", "@@", "events"}], ",", "evlistopts", ",", "opts"}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.541530910997768*^9, 3.541531021644535*^9}, {
   3.5430635402903833`*^9, 3.543063540696951*^9}, {3.685033531147162*^9, 
   3.685033554514894*^9}, 3.6850335849633617`*^9, 3.6850358097864037`*^9, 
   3.685035908735074*^9},ExpressionUUID->"e70f85ea-57e1-46ce-9342-\
877384f1f0be"]
}, Closed]],

Cell[CellGroupData[{

Cell["combinations of two or more items", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.464696964645693*^9, 3.464696967220211*^9}, {
   3.541485708101438*^9, 3.5414857120002613`*^9}, {3.541485819010337*^9, 
   3.541485819813015*^9}, 3.541485954967263*^9, {3.541486036555203*^9, 
   3.541486044345131*^9}, {3.541486338197731*^9, 3.541486355360798*^9}, {
   3.541489117674075*^9, 3.54148912880729*^9}, {3.541489552203813*^9, 
   3.541489552886943*^9}},ExpressionUUID->"481dab05-8054-4e28-805a-\
e1b693ae1755"],

Cell[CellGroupData[{

Cell["combine PermittedTimeShifts for two items", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.464696964645693*^9, 3.464696967220211*^9}, {
   3.541485708101438*^9, 3.5414857120002613`*^9}, {3.541485819010337*^9, 
   3.541485819813015*^9}, 3.541485954967263*^9, {3.541486036555203*^9, 
   3.541486044345131*^9}, {3.541486338197731*^9, 3.541486355360798*^9}, {
   3.5414891818823967`*^9, 3.541489182646041*^9}, {3.541494776816221*^9, 
   3.5414947856335707`*^9}, 
   3.541530138909349*^9},ExpressionUUID->"d8122eb3-bbfb-4fb4-adb1-\
1ba4ed5fabe7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"evgen1_", ",", 
     RowBox[{"evgen2", ":", 
      RowBox[{"Except", "[", "_Rule", "]"}]}], ",", "opts___Rule"}], "]"}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"pts1", ",", "pts2"}], "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"pts1", "=", 
        RowBox[{"PermittedTimeShift", "[", 
         RowBox[{"evgen1", ",", "opts"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"pts2", "=", 
        RowBox[{"PermittedTimeShift", "[", 
         RowBox[{"evgen2", ",", "opts"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"If", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"MatchQ", "[", 
          RowBox[{"pts1", ",", 
           RowBox[{"All", "|", "None", "|", 
            RowBox[{"DurationPattern", "&&", 
             RowBox[{"MatchQ", "[", 
              RowBox[{"pts2", ",", 
               RowBox[{"All", "|", "None", "|", "DurationPattern"}]}], 
              "]"}]}]}]}], "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"CommonPeriod", "[", 
          RowBox[{"pts1", ",", "pts2"}], "]"}], ",", "\[IndentingNewLine]", 
         "None"}], "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", 
      ")"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.541486092899879*^9, 3.5414861955183973`*^9}, {
  3.5414863583948593`*^9, 3.541486436265202*^9}, {3.541489134097487*^9, 
  3.5414891390947847`*^9}, {3.5415300390651407`*^9, 3.541530085370845*^9}, {
  3.54153024544018*^9, 3.541530249510441*^9}, {3.541530330516695*^9, 
  3.541530334850881*^9}, {3.683435506975374*^9, 3.683435516226836*^9}, {
  3.6834438438104353`*^9, 3.683443853745816*^9}, {3.683444219656008*^9, 
  3.683444232592033*^9}},ExpressionUUID->"9ccb910f-1cec-41be-8e41-\
2e1920314d35"]
}, Closed]],

Cell[CellGroupData[{

Cell["combine PermittedTimeShifts for more than two items", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.464696964645693*^9, 3.464696967220211*^9}, {
   3.541485708101438*^9, 3.5414857120002613`*^9}, {3.541485819010337*^9, 
   3.541485819813015*^9}, 3.541485954967263*^9, {3.541486036555203*^9, 
   3.541486044345131*^9}, {3.541486338197731*^9, 3.541486355360798*^9}, {
   3.5414864451447067`*^9, 3.541486446710559*^9}, {3.54148918939371*^9, 
   3.5414891901496563`*^9}},ExpressionUUID->"8146c864-2817-4f11-8f62-\
881e499f9c60"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PermittedTimeShift", "[", 
    RowBox[{"evgen1_", ",", "evgen2_", ",", 
     RowBox[{"evgens3", ":", 
      RowBox[{
       RowBox[{"Except", "[", "_Rule", "]"}], ".."}]}], ",", "opts___Rule"}], 
    "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"CommonPeriod", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"PermittedTimeShift", "[", 
      RowBox[{"evgen1", ",", "evgen2", ",", "opts"}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"PermittedTimeShift", "[", 
      RowBox[{"evgens3", ",", "opts"}], "]"}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.541487462802616*^9, 3.5414874990564213`*^9}, {
  3.541487880511442*^9, 3.541487892780326*^9}, {3.541489148406221*^9, 
  3.541489154170896*^9}, {3.541530342969556*^9, 
  3.5415303542231493`*^9}},ExpressionUUID->"169f0d59-7fd4-4c1c-8dfd-\
e0cee6eacf6b"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["time intervals", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{
  3.454575871518672*^9, {3.455454799573387*^9, 3.455454807249361*^9}, {
   3.4639354155697937`*^9, 3.463935420440484*^9}, 3.4639358563258047`*^9, {
   3.4642544189569263`*^9, 3.464254422443118*^9}, {3.464327897371964*^9, 
   3.464327899208976*^9}, 3.464350113189783*^9, {3.5414885543485613`*^9, 
   3.541488570856545*^9}, 3.569514863691033*^9, {3.6831287382470083`*^9, 
   3.6831287738920813`*^9}, {3.689746730469932*^9, 3.6897467439661407`*^9}, {
   3.689746791371194*^9, 
   3.689746793456909*^9}},ExpressionUUID->"75c11cf9-1cf1-40a1-8521-\
1e5b1bae82bd"],

Cell[CellGroupData[{

Cell["EventTimeIntervals", "Section",
 InitializationCell->True,
 CellChangeTimes->{
  3.454575871518672*^9, {3.455454799573387*^9, 3.455454807249361*^9}, {
   3.4639354155697937`*^9, 3.463935420440484*^9}, 3.4639358563258047`*^9, {
   3.4642544189569263`*^9, 3.464254422443118*^9}, {3.464327897371964*^9, 
   3.464327899208976*^9}, 3.464350113189783*^9, {3.5414885543485613`*^9, 
   3.541488570856545*^9}, 3.569514863691033*^9, {3.6831287382470083`*^9, 
   3.6831287738920813`*^9}, 
   3.689746730469932*^9},ExpressionUUID->"4b86c2a8-db82-4580-bc17-\
669235942ba6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventTimeIntervals", "[", 
    RowBox[{"{", "}"}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}]}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.484745608684359*^9, 3.484745615875738*^9}, {
   3.514725964865637*^9, 3.5147259978165007`*^9}, 
   3.683128128572983*^9},ExpressionUUID->"966bb394-95ad-478f-9046-\
4f8f21e50eab"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventTimeIntervals", "[", 
    RowBox[{"EventList", "[", 
     RowBox[{"st_", ",", "ch_", ",", 
      RowBox[{"events", ":", 
       RowBox[{"{", 
        RowBox[{"_Event", ".."}], "}"}]}], ",", "___"}], "]"}], "]"}], ":=", 
   RowBox[{"EventTimeIntervals", "[", "events", "]"}]}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.455634362302541*^9, 3.4556344908849277`*^9}, {
   3.456228114490411*^9, 3.4562281244193153`*^9}, {3.4562281700937357`*^9, 
   3.456228213308015*^9}, {3.514725925239687*^9, 3.5147259456150007`*^9}, {
   3.683128129237055*^9, 3.683128129962894*^9}, 3.685033531258835*^9, 
   3.685033585041151*^9, {3.691658801952183*^9, 
   3.6916588474301853`*^9}},ExpressionUUID->"f34bd3ed-58fe-4e0f-b999-\
8a7548507927"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventTimeIntervals", "[", 
    RowBox[{"events", ":", 
     RowBox[{"{", 
      RowBox[{"_Event", "..."}], "}"}]}], "]"}], ":=", 
   RowBox[{"Join", "@@", 
    RowBox[{"(", 
     RowBox[{"EventTimeIntervals", "/@", "events"}], ")"}]}]}], 
  ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.455634362302541*^9, 3.4556344908849277`*^9}, {
   3.456228114490411*^9, 3.4562281244193153`*^9}, {3.4562281700937357`*^9, 
   3.456228213308015*^9}, {3.683128130605122*^9, 3.6831281312388573`*^9}, 
   3.685033554598406*^9, 3.685035809878522*^9, 3.685035924333398*^9, {
   3.691658856933045*^9, 
   3.691658858723557*^9}},ExpressionUUID->"2509fd3f-86ef-4c54-a83b-\
8c9a3d8d64e3"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventTimeIntervals", "[", 
    RowBox[{"Event", "[", 
     RowBox[{"st_", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"_", ",", 
          RowBox[{"{", 
           RowBox[{"tb_", ",", "ta_"}], "}"}], ",", "__"}], "}"}], ",", 
        "__"}], "}"}], ",", "___"}], "]"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{"{", 
     RowBox[{"tb", ",", "ta"}], "}"}], "}"}]}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.455634362302541*^9, 3.455634438790359*^9}, {
   3.456228088384156*^9, 3.4562280990377398`*^9}, 3.459439806249247*^9, 
   3.683128131953973*^9, 3.685033554605245*^9, 3.685035809885461*^9, 
   3.6850359250439034`*^9, {3.691658706406373*^9, 
   3.691658752548327*^9}},ExpressionUUID->"bc73b927-fbec-4882-89c3-\
c26f56849834"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"EventTimeIntervals", "[", 
    RowBox[{"Event", "[", 
     RowBox[{"st_", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"_", ",", "Undefined", ",", "__"}], "}"}], ",", "__"}], 
       "}"}], ",", "___"}], "]"}], "]"}], ":=", "Undefined"}], ";"}]], "Input",\

 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.455634362302541*^9, 3.455634438790359*^9}, {
   3.456228088384156*^9, 3.4562280990377398`*^9}, 3.459439806249247*^9, 
   3.683128131953973*^9, 3.685033554605245*^9, 3.685035809885461*^9, 
   3.6850359250439034`*^9, {3.691658706406373*^9, 
   3.691658776453668*^9}},ExpressionUUID->"5bf66812-8cd4-4dc6-9c91-\
e6ff8fa624c4"]
}, Closed]],

Cell[CellGroupData[{

Cell["TotalEventTimeInterval", "Section",
 InitializationCell->True,
 CellChangeTimes->{
  3.454575871518672*^9, {3.455454799573387*^9, 3.455454807249361*^9}, {
   3.4639354155697937`*^9, 3.463935420440484*^9}, 3.4639358563258047`*^9, {
   3.4642544189569263`*^9, 3.464254422443118*^9}, {3.464256108963293*^9, 
   3.464256112185646*^9}, {3.4642778249100723`*^9, 3.464277825961912*^9}, {
   3.464362999993634*^9, 3.4643630103773413`*^9}, {3.4973771791245623`*^9, 
   3.497377184059304*^9}, 3.507987014371545*^9, 3.514723801421639*^9, 
   3.683128215451289*^9},ExpressionUUID->"e38c5dab-50fd-4b7e-abc7-\
c9bc5f97a1d3"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TotalEventTimeInterval", "[", 
    RowBox[{
     RowBox[{"events", ":", 
      RowBox[{"Except", "[", 
       RowBox[{"_EventList", "|", 
        RowBox[{"{", 
         RowBox[{"_Event", ".."}], "}"}], "|", "_Event"}], "]"}]}], ",", 
     "opts___Rule"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"TotalEventTimeInterval", "[", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{"events", ",", 
       RowBox[{"ExpandRepeatedEvents", "\[Rule]", "False"}], ",", 
       RowBox[{"ImplementBackgroundGenerator", "\[Rule]", "False"}], ",", 
       "opts"}], "]"}], ",", "opts"}], "]"}]}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.455107966909136*^9, 3.455108073525446*^9}, {
   3.45511468950836*^9, 3.455114746935917*^9}, 3.455258897181007*^9, {
   3.455271628107841*^9, 3.455271677971651*^9}, 3.45527182667826*^9, {
   3.4552744641864967`*^9, 3.455274466298558*^9}, {3.455274556422624*^9, 
   3.4552745586884604`*^9}, {3.45527501689292*^9, 3.455275022038907*^9}, 
   3.455275086439406*^9, {3.4553574664410133`*^9, 3.4553574781656218`*^9}, {
   3.456322606803937*^9, 3.4563226174627132`*^9}, {3.456322712349271*^9, 
   3.4563227155044537`*^9}, {3.456331257436535*^9, 3.456331277846345*^9}, {
   3.459414169942325*^9, 3.459414174470708*^9}, 3.464328072726432*^9, {
   3.4658188998001623`*^9, 3.465818914069458*^9}, 3.497572510063349*^9, 
   3.514457067608636*^9, {3.514723657736459*^9, 3.5147237163724546`*^9}, 
   3.51472405574021*^9, 3.514724156790996*^9, {3.6831282163286743`*^9, 
   3.683128217782111*^9}, {3.685033531269141*^9, 3.685033554612843*^9}, {
   3.685033585046344*^9, 3.6850335850484953`*^9}, {3.685035809892622*^9, 
   3.685035809894911*^9}, {3.685035925962578*^9, 3.685035926879464*^9}, 
   3.691658927268417*^9, {3.69166187764018*^9, 3.691661878966905*^9}, 
   3.7093749835200377`*^9},ExpressionUUID->"3b466775-1f49-40bd-affb-\
2d2b54efe230"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TotalEventTimeInterval", "[", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{"st_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"_", ",", 
           RowBox[{"{", 
            RowBox[{"tb_", ",", "ta_"}], "}"}], ",", "__"}], "}"}], ",", 
         "__"}], "}"}], ",", "___"}], "]"}], ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{"tb", ",", "ta"}], "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.514723723162425*^9, 3.514723753402665*^9}, 
   3.514724053993935*^9, {3.542391096480006*^9, 3.54239110217594*^9}, 
   3.683128218886902*^9, 3.685033531278735*^9, 3.685033585053747*^9, {
   3.691658908538019*^9, 
   3.6916589303774233`*^9}},ExpressionUUID->"baf06206-4095-4cda-b191-\
339e0e92c34c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TotalEventTimeInterval", "[", 
    RowBox[{
     RowBox[{"EventList", "[", 
      RowBox[{"st_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"_", ",", "Undefined", ",", "__"}], "}"}], ",", "__"}], 
        "}"}], ",", "___"}], "]"}], ",", "opts___Rule"}], "]"}], ":=", 
   "Undefined"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.514723723162425*^9, 3.514723753402665*^9}, 
   3.514724053993935*^9, {3.542391096480006*^9, 3.54239110217594*^9}, 
   3.683128218886902*^9, 3.685033531278735*^9, 3.685033585053747*^9, {
   3.691658908538019*^9, 3.6916589303774233`*^9}, {3.6916590298824463`*^9, 
   3.691659035225265*^9}},ExpressionUUID->"e196eb0e-0c42-4428-a500-\
7321ae7e186f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TotalEventTimeInterval", "[", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"st_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"_", ",", 
           RowBox[{"{", 
            RowBox[{"tb_", ",", "ta_"}], "}"}], ",", "__"}], "}"}], ",", 
         "__"}], "}"}], ",", "___"}], "]"}], ",", "opts___Rule"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{"tb", ",", "ta"}], "}"}]}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.484907724976082*^9, 3.484907742727107*^9}, {
   3.514723756956663*^9, 3.514723767714876*^9}, {3.6831282196837587`*^9, 
   3.683128220285668*^9}, 3.685033554619486*^9, 3.685035809901897*^9, 
   3.68503592799127*^9, 3.691658932864141*^9, {3.691658991881138*^9, 
   3.69165899806653*^9}},ExpressionUUID->"559631e1-7e52-4be4-be8b-\
11a3677fcb56"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TotalEventTimeInterval", "[", 
    RowBox[{
     RowBox[{"Event", "[", 
      RowBox[{"st_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"_", ",", "Undefined", ",", "__"}], "}"}], ",", "__"}], 
        "}"}], ",", "___"}], "]"}], ",", "opts___Rule"}], "]"}], ":=", 
   "Undefined"}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.484907724976082*^9, 3.484907742727107*^9}, {
   3.514723756956663*^9, 3.514723767714876*^9}, {3.6831282196837587`*^9, 
   3.683128220285668*^9}, 3.685033554619486*^9, 3.685035809901897*^9, 
   3.68503592799127*^9, 3.691658932864141*^9, {3.691658991881138*^9, 
   3.69165899806653*^9}, {3.691659061290205*^9, 
   3.691659063426436*^9}},ExpressionUUID->"65ed33ee-8889-4ead-97da-\
ce62d806fa60"]
}, Closed]],

Cell[CellGroupData[{

Cell["StandardizeInterval", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.455265644416114*^9, 3.455265648200944*^9}, {
   3.464256156522676*^9, 3.4642561730892*^9}, {3.464282095283453*^9, 
   3.4642821290032167`*^9}, {3.503094263066369*^9, 3.503094264574778*^9}, {
   3.503095573470295*^9, 3.503095585674507*^9}, {3.504260287465301*^9, 
   3.504260304480891*^9}, 3.504261075915663*^9, 3.514538950441782*^9, 
   3.543059791360136*^9},ExpressionUUID->"f55d1b40-96ff-4fbf-a552-\
aa4b2c09ce61"],

Cell[CellGroupData[{

Cell["StandardizeInterval[{tb,ta},permittedtimeshift]", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.54306105604321*^9, 3.5430610664577312`*^9}, {
  3.543062494985586*^9, 
  3.5430625010874157`*^9}},ExpressionUUID->"17c89699-36c2-43ba-96a4-\
43e247f6f8df"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeInterval", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"tb_", ",", "ta_"}], "}"}], ",", "None"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{"tb", ",", "ta"}], "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.543061071317506*^9, 
  3.543061134256523*^9}},ExpressionUUID->"07714143-5c12-4320-b5ce-\
dd92aac9d167"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeInterval", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"tb_", ",", "ta_"}], "}"}], ",", "All"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"ta", "-", "tb"}]}], "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.543061071317506*^9, 
  3.543061120356283*^9}},ExpressionUUID->"b43246b8-20f9-4cd6-baf4-\
800538164ca9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"StandardizeInterval", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"tb_", ",", "ta_"}], "}"}], ",", 
      RowBox[{"T_", "?", "NumericQ"}]}], "]"}], "/;", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Chop", "[", "T", "]"}], "\[NotEqual]", "0"}], ")"}]}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "\[CapitalDelta]", "}"}], ",", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[CapitalDelta]", "=", 
        RowBox[{"T", "\[Times]", 
         RowBox[{"Floor", "[", 
          RowBox[{"tb", "/", "T"}], "]"}]}]}], ";", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"tb", "-", "\[CapitalDelta]"}], ",", 
         RowBox[{"ta", "-", "\[CapitalDelta]"}]}], "}"}]}], ")"}]}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.543061071317506*^9, 3.543061120356283*^9}, {
  3.543061174816175*^9, 3.543061180727231*^9}, {3.543061241219453*^9, 
  3.5430613232894373`*^9}, {3.543061418406011*^9, 
  3.543061423220615*^9}},ExpressionUUID->"e0a06437-b489-49bb-8d6e-\
02bb3c6121a8"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"StandardizeInterval", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"tb_", ",", "ta_"}], "}"}], ",", 
      RowBox[{"T_", "?", "NumericQ"}]}], "]"}], "/;", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Chop", "[", "T", "]"}], "==", "0"}], ")"}]}], ":=", 
   RowBox[{"StandardizeInterval", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"tb", ",", "ta"}], "}"}], ",", "All"}], "]"}]}], ";"}]], "Input",\

 InitializationCell->True,
 CellChangeTimes->{{3.543061071317506*^9, 3.543061120356283*^9}, {
  3.543061174816175*^9, 3.543061180727231*^9}, {3.543061241219453*^9, 
  3.5430613232894373`*^9}, {3.543061418406011*^9, 
  3.543061473043766*^9}},ExpressionUUID->"e8e9eb42-f270-4fca-af2d-\
2111c50de307"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeInterval", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"tb_", ",", "ta_"}], "}"}], ",", 
     RowBox[{"T", ":", "DurationPattern"}]}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "\[CapitalDelta]", "}"}], ",", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[CapitalDelta]", "=", 
        RowBox[{"T", "\[Times]", 
         RowBox[{"Floor", "[", 
          RowBox[{"tb", "/", "T"}], "]"}]}]}], ";", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"tb", "-", "\[CapitalDelta]"}], ",", 
         RowBox[{"ta", "-", "\[CapitalDelta]"}]}], "}"}]}], ")"}]}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.543061071317506*^9, 3.543061120356283*^9}, {
  3.543061174816175*^9, 3.543061180727231*^9}, {3.543061241219453*^9, 
  3.5430613232894373`*^9}, {3.543061418406011*^9, 3.543061423220615*^9}, {
  3.683435058940647*^9, 3.683435064995463*^9}, {3.6834522212415037`*^9, 
  3.683452222308874*^9}},ExpressionUUID->"3c95a25e-8e6b-4cb6-8169-\
141df663c7df"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeInterval", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"tb_", ",", "ta_"}], "}"}], ",", "_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{"tb", ",", "ta"}], "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.543061518442151*^9, 
  3.543061524328844*^9}},ExpressionUUID->"ae609166-f063-4f02-9b8b-\
a8b4ab63e703"]
}, Closed]],

Cell[CellGroupData[{

Cell["no action on Event if timing is already standardized", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.514122644206769*^9, 3.514122653477813*^9}, {
   3.543062507215115*^9, 3.5430625099593*^9}, 3.685033554789976*^9, 
   3.6850358100953817`*^9, 
   3.685036132371221*^9},ExpressionUUID->"86c01827-bf8a-49c4-a05f-\
59a514d6085a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeInterval", "[", 
    RowBox[{"ev", ":", 
     RowBox[{"Event", "[", 
      RowBox[{"st_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"_", ",", 
           RowBox[{"{", 
            RowBox[{"tb_", ",", "ta_"}], "}"}], ",", "pts_", ",", 
           RowBox[{"{", 
            RowBox[{"tbs_", ",", "tas_"}], "}"}], ",", "___"}], "}"}], ",", 
         "__"}], "}"}], ",", "___"}], "]"}]}], "]"}], ":=", "ev"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.514122024149823*^9, 3.51412208367838*^9}, 
   3.51453894951484*^9, {3.5145389957681427`*^9, 3.514539019510439*^9}, {
   3.5145395456809177`*^9, 3.5145395498944473`*^9}, {3.514539745301546*^9, 
   3.514539781308655*^9}, {3.5145398659233723`*^9, 3.5145398677937737`*^9}, 
   3.54305976019777*^9, 3.543059791424034*^9, 3.683452263554749*^9, {
   3.685033554798958*^9, 3.685033554805683*^9}, {3.685035810105023*^9, 
   3.6850358101105003`*^9}, {3.6850361329443617`*^9, 3.6850361341385*^9}, {
   3.691659100117853*^9, 
   3.691659163847227*^9}},ExpressionUUID->"ad9e3faa-90cc-498e-9b14-\
7c1b3f51f251"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeInterval", "[", 
    RowBox[{"ev", ":", 
     RowBox[{"Event", "[", 
      RowBox[{"st_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"_", ",", 
           RowBox[{"{", 
            RowBox[{"tb_", ",", "ta_"}], "}"}], ",", 
           RowBox[{"pts", ":", 
            RowBox[{"Except", "[", "Undefined", "]"}]}], ",", "Undefined", 
           ",", "___"}], "}"}], ",", "__"}], "}"}], ",", "___"}], "]"}]}], 
    "]"}], ":=", 
   RowBox[{"StandardizeInterval", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"tb", ",", "ta"}], "}"}], ",", "pts"}], "]"}]}], ";"}]], "Input",\

 InitializationCell->True,
 CellChangeTimes->{{3.514122024149823*^9, 3.51412208367838*^9}, 
   3.51453894951484*^9, {3.5145389957681427`*^9, 3.514539019510439*^9}, {
   3.5145395456809177`*^9, 3.5145395498944473`*^9}, {3.514539745301546*^9, 
   3.514539781308655*^9}, {3.5145398659233723`*^9, 3.5145398677937737`*^9}, 
   3.54305976019777*^9, 3.543059791424034*^9, 3.683452263554749*^9, {
   3.685033554798958*^9, 3.685033554805683*^9}, {3.685035810105023*^9, 
   3.6850358101105003`*^9}, {3.6850361329443617`*^9, 3.6850361341385*^9}, {
   3.691659100117853*^9, 
   3.691659209288787*^9}},ExpressionUUID->"d4eb02d8-4a6d-4e1c-a8aa-\
7ddadd1802e3"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeInterval", "[", 
    RowBox[{"ev", ":", 
     RowBox[{"Event", "[", 
      RowBox[{"st_", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"_", ",", "_", ",", "Undefined", ",", "___"}], "}"}], ",", 
         "__"}], "}"}], ",", "___"}], "]"}]}], "]"}], ":=", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Message", "[", 
      RowBox[{"StandardizeInterval", "::", "undefptshft"}], "]"}], ";", 
     "$Failed"}], ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.514122024149823*^9, 3.51412208367838*^9}, 
   3.51453894951484*^9, {3.5145389957681427`*^9, 3.514539019510439*^9}, {
   3.5145395456809177`*^9, 3.5145395498944473`*^9}, {3.514539745301546*^9, 
   3.514539781308655*^9}, {3.5145398659233723`*^9, 3.5145398677937737`*^9}, 
   3.54305976019777*^9, 3.543059791424034*^9, 3.683452263554749*^9, {
   3.685033554798958*^9, 3.685033554805683*^9}, {3.685035810105023*^9, 
   3.6850358101105003`*^9}, {3.6850361329443617`*^9, 3.6850361341385*^9}, {
   3.691659100117853*^9, 
   3.6916592456177263`*^9}},ExpressionUUID->"5ed23c59-290e-477f-a331-\
b37aa58fcf2b"]
}, Closed]],

Cell[CellGroupData[{

Cell["messages", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.514122644206769*^9, 3.514122683567586*^9}, {
  3.5430615457614107`*^9, 3.543061551118721*^9}, {3.543062577095928*^9, 
  3.54306257826268*^9}},ExpressionUUID->"e7ef73d4-9033-4494-a306-\
d2ee3d14b582"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StandardizeInterval", "::", "undefptshft"}], "=", 
   "\"\<cannot standardize interval if PermittedTimeShift is Undefined\>\""}],
   ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.543062583664956*^9, 3.543062601746624*^9}, {
  3.691659273608025*^9, 
  3.691659274664295*^9}},ExpressionUUID->"53d243c2-80e7-41a1-b0db-\
3c30b6c2077e"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["start and end time points", "Subsubtitle",
 InitializationCell->True,
 CellChangeTimes->{
  3.454575871518672*^9, {3.455454799573387*^9, 3.455454807249361*^9}, {
   3.4639354155697937`*^9, 3.463935420440484*^9}, 3.4639358563258047`*^9, {
   3.4642544189569263`*^9, 3.464254422443118*^9}, {3.464327897371964*^9, 
   3.464327899208976*^9}, 3.464350113189783*^9, {3.5414885543485613`*^9, 
   3.541488570856545*^9}, 3.569514863691033*^9, {3.6831287382470083`*^9, 
   3.6831287738920813`*^9}, {3.689746730469932*^9, 
   3.689746772154896*^9}},ExpressionUUID->"2d8d66f0-9c70-4df8-af1a-\
ec344df501fd"],

Cell[CellGroupData[{

Cell["EventStartTimePoint", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.455454136981319*^9, 3.455454190582034*^9}, {
   3.455454472943882*^9, 3.455454474174431*^9}, {3.455455572039624*^9, 
   3.4554555804898577`*^9}, 3.455456514657077*^9, 3.455456988966947*^9, 
   3.455457877088749*^9, {3.455458472404565*^9, 3.45545847594317*^9}, {
   3.455634306583088*^9, 3.455634315476242*^9}, {3.456228041618371*^9, 
   3.456228065560835*^9}, {3.464332395796246*^9, 3.4643323961969957`*^9}, {
   3.4646057352918577`*^9, 
   3.464605771018136*^9}},ExpressionUUID->"cc2bfcb4-7ead-4e15-8368-\
d4624d727ff6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"EventStartTimePoint", "[", "events_", "]"}]}], ":=", 
   RowBox[{"Last", "@", 
    RowBox[{"Flatten", "@", 
     RowBox[{"EventTimeIntervals", "[", "events", "]"}]}]}]}], ";"}]], "Input",\

 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.455634362302541*^9, 3.455634438790359*^9}, {
   3.456228088384156*^9, 3.4562280990377398`*^9}, 3.459439806249247*^9, {
   3.464605775426915*^9, 3.464605808750239*^9}, 
   3.683128132684412*^9},ExpressionUUID->"5c989e41-2e1f-4142-85f8-\
c68ac08ab85f"]
}, Closed]],

Cell[CellGroupData[{

Cell["EventEndTimePoint", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.455454136981319*^9, 3.455454190582034*^9}, {
   3.455454472943882*^9, 3.455454474174431*^9}, {3.455455572039624*^9, 
   3.4554555804898577`*^9}, 3.455456514657077*^9, 3.455456988966947*^9, 
   3.455457877088749*^9, {3.455458472404565*^9, 3.45545847594317*^9}, {
   3.455634306583088*^9, 3.455634315476242*^9}, {3.456228041618371*^9, 
   3.456228065560835*^9}, {3.464332395796246*^9, 3.4643323961969957`*^9}, {
   3.4646057352918577`*^9, 3.464605771018136*^9}, {3.464605819725277*^9, 
   3.464605820219603*^9}},ExpressionUUID->"d62d9ff7-6f3e-45f6-a62c-\
5fb1f3f44c7a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"HoldPattern", "@", 
    RowBox[{"EventEndTimePoint", "[", "events_", "]"}]}], ":=", 
   RowBox[{"First", "@", 
    RowBox[{"Flatten", "@", 
     RowBox[{"EventTimeIntervals", "[", "events", "]"}]}]}]}], ";"}]], "Input",\

 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.455634362302541*^9, 3.455634438790359*^9}, {
   3.456228088384156*^9, 3.4562280990377398`*^9}, 3.459439806249247*^9, {
   3.464605775426915*^9, 3.464605827022264*^9}, 
   3.6831281341306353`*^9},ExpressionUUID->"c5145f5e-9e42-43e3-a577-\
8811765d44c9"]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["BackgroundGenerator", "Subtitle",
 InitializationCell->True,
 CellChangeTimes->{
  3.454575871518672*^9, {3.455454799573387*^9, 3.455454807249361*^9}, {
   3.4639354155697937`*^9, 3.463935420440484*^9}, 3.4639358563258047`*^9, {
   3.4642544189569263`*^9, 3.464254422443118*^9}, {3.464327897371964*^9, 
   3.464327899208976*^9}, 3.464350113189783*^9, {3.5414885543485613`*^9, 
   3.541488570856545*^9}, 3.569514863691033*^9, {3.6831287382470083`*^9, 
   3.6831287738920813`*^9}, {3.6832657719805393`*^9, 3.68326577626611*^9}, {
   3.683265876779852*^9, 3.683265881964329*^9}, {3.68326601584654*^9, 
   3.683266016317121*^9}, {3.683368681532878*^9, 3.68336871152321*^9}, {
   3.683369462677738*^9, 3.6833694693018627`*^9}, 3.689744796718789*^9, 
   3.690449545350794*^9, {3.709374703995901*^9, 3.709374710463539*^9}, 
   3.7093747690588017`*^9},ExpressionUUID->"68c89773-19ed-4105-a3d6-\
09cc2af71744"],

Cell[CellGroupData[{

Cell["EvolutionBackground (obsolete)", "Section",
 InitializationCell->True,
 CellChangeTimes->{
  3.454575871518672*^9, {3.455454799573387*^9, 3.455454807249361*^9}, {
   3.4639354155697937`*^9, 3.463935420440484*^9}, 3.4639358563258047`*^9, {
   3.4642544189569263`*^9, 3.464254422443118*^9}, {3.464327897371964*^9, 
   3.464327899208976*^9}, 3.464350113189783*^9, {3.5414885543485613`*^9, 
   3.541488570856545*^9}, 3.569514863691033*^9, {3.6831287382470083`*^9, 
   3.6831287738920813`*^9}, {3.6832657719805393`*^9, 3.68326577626611*^9}, {
   3.683265876779852*^9, 3.683265881964329*^9}, {3.68326601584654*^9, 
   3.683266016317121*^9}, {3.683368681532878*^9, 3.68336871152321*^9}, {
   3.683369462677738*^9, 3.6833694693018627`*^9}, 3.689744796718789*^9, 
   3.690449545350794*^9, {3.709374703995901*^9, 3.709374710463539*^9}, 
   3.7093749842279778`*^9, 
   3.7093750959796553`*^9},ExpressionUUID->"74878d79-6e3a-4b53-945a-\
b68110f3dce8"],

Cell[BoxData[
 RowBox[{
  RowBox[{"EvolutionBackground", ":=", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Message", "[", 
      RowBox[{"EvolutionBackground", "::", "usage"}], "]"}], ";", 
     "BackgroundGenerator"}], ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.709373270765923*^9, 3.7093733190299263`*^9}, {
  3.7093733836744223`*^9, 3.709373385889963*^9}, {3.7093736332296247`*^9, 
  3.709373638145521*^9}, {3.7093747301783953`*^9, 3.709374752246993*^9}, {
  3.709374984892518*^9, 3.709374986186365*^9}, {3.709375087032302*^9, 
  3.709375093061885*^9}},ExpressionUUID->"64875045-42a8-4420-a5d2-\
cfbe3fc147f2"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["end", "Subtitle",
 InitializationCell->True,
 CellChangeTimes->{
  3.454575871518672*^9, {3.455454799573387*^9, 3.455454807249361*^9}, {
   3.4639354155697937`*^9, 3.463935420440484*^9}, 3.4639358563258047`*^9, {
   3.4642544189569263`*^9, 3.464254422443118*^9}, {3.464327897371964*^9, 
   3.464327899208976*^9}, 3.464350113189783*^9, {3.5414885543485613`*^9, 
   3.541488570856545*^9}, 3.569514863691033*^9, {3.6831287382470083`*^9, 
   3.683128785303934*^9}, 
   3.683367749735805*^9},ExpressionUUID->"c41bd980-f903-41f9-ad31-\
7c6dd4d741e8"],

Cell["\<\
End[];
EndPackage[];\
\>", "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.51238794480797*^9, 3.512387952205147*^9}, 
   3.513231730353488*^9},
 AspectRatioFixed->
  True,ExpressionUUID->"a4a6ec76-7991-4111-a11c-99ebd9b44914"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["history", "Title",
 PageWidth->WindowWidth,
 Evaluatable->False,
 CellChangeTimes->{
  3.682593334636269*^9, {3.69926171587668*^9, 3.69926172886936*^9}, {
   3.69926187929764*^9, 3.6992618809832497`*^9}},
 AspectRatioFixed->
  True,ExpressionUUID->"0e02bf6d-1670-49cf-976b-e1907a5b86ff"],

Cell[CellGroupData[{

Cell["1703406 changes incorporated into v3.0.0b12", "Subsection",
 CellChangeTimes->{{3.6992617377127457`*^9, 3.699261750424902*^9}, 
   3.699261872727805*^9, {3.699261961052656*^9, 3.699261967080655*^9}, {
   3.699267263610918*^9, 3.699267264970158*^9}, 3.6995424563123937`*^9, {
   3.699545398748596*^9, 3.6995453990235744`*^9}, {3.699598420679057*^9, 
   3.6995984207889843`*^9}, {3.699623620808276*^9, 3.699623620919536*^9}, {
   3.699684596084985*^9, 3.699684596623074*^9}, {3.699685713789143*^9, 
   3.699685784922206*^9}, {3.6999571421771107`*^9, 3.6999571579260406`*^9}, {
   3.700484912077015*^9, 
   3.700484926388489*^9}},ExpressionUUID->"5af9302f-bcce-45c3-a52e-\
4594cef79698"],

Cell["\<\
StandardizedEventSpecificationQ changed to StandardizedEventSpecificationQ, \
reflecting its actual functionality; new routine \
StandardizedEventSpecificationQ is more general (detecting non-standardized, \
but valid, event specifications)\
\>", "Text",
 CellChangeTimes->{{3.7004849291008253`*^9, 
  3.7004850090876913`*^9}},ExpressionUUID->"671b5c3c-f6f2-4a9e-805e-\
6a9a33e0af47"],

Cell["HilbertSpaceCompatibility applied to EventSpecifications", "Text",
 CellChangeTimes->{{3.7011047466040583`*^9, 
  3.701104757707468*^9}},ExpressionUUID->"9a5f65ae-7c0f-4292-8e09-\
a4e200993996"],

Cell["\<\
loading of SpinSuperoperators removed. SpinSuperoperators has been absorbed \
in QM`Liouville\
\>", "Text",
 CellChangeTimes->{{3.701151390974813*^9, 
  3.701151409840228*^9}},ExpressionUUID->"4aa1b96b-26a2-4929-bb6c-\
bd5c779ef5bd"]
}, Open  ]]
}, Closed]]
},
AutoGeneratedPackage->Automatic,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{808, 751},
WindowMargins->{{Automatic, 11}, {Automatic, 0}},
ShowCellLabel->True,
FrontEndVersion->"11.3 for Mac OS X x86 (32-bit, 64-bit Kernel) (March 5, \
2018)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[StyleDefinitions -> "MHL-StyleSheet.nb"]], 
   Cell[
    StyleData["Subsection"]], 
   Cell[
    StyleData["Section"]], 
   Cell[
    StyleData["Subchapter"], FontSize -> 24]}, Visible -> False, 
  FrontEndVersion -> 
  "11.3 for Mac OS X x86 (32-bit, 64-bit Kernel) (March 5, 2018)", 
  StyleDefinitions -> "PrivateStylesheetFormatting.nb"],
PrivateNotebookOptions -> {"ColorPalette" -> {RGBColor, 128}},
RenderingOptions -> {"ObjectDithering" -> True},
RenderingOptions -> {"RasterDithering" -> False},
CharacterEncoding -> "MacintoshAutomaticEncoding"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "EventSpecificationList"->{
  Cell[97001, 2335, 517, 10, 41, "Subsubtitle",ExpressionUUID->"dbb7611f-0ccc-457f-8ba5-770b92c44939",
   InitializationCell->True,
   CellTags->{"PrecalculationRules - working", "EventSpecificationList"}]},
 "PrecalculationRules - working"->{
  Cell[97001, 2335, 517, 10, 41, "Subsubtitle",ExpressionUUID->"dbb7611f-0ccc-457f-8ba5-770b92c44939",
   InitializationCell->True,
   CellTags->{"PrecalculationRules - working", "EventSpecificationList"}]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"EventSpecificationList", 857549, 20781},
 {"PrecalculationRules - working", 857793, 20785}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 392, 9, 114, "Title",ExpressionUUID->"48318cb7-511a-408b-9448-ce078472d7aa",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[984, 35, 150, 3, 36, "Section",ExpressionUUID->"6a54542e-86b7-49cc-9af2-7a878fc8402d"],
Cell[1137, 40, 456, 10, 68, "Text",ExpressionUUID->"b0a9a083-7a31-4160-bfde-14ef45ccf5ce",
 InitializationCell->True],
Cell[1596, 52, 1193, 25, 220, "Text",ExpressionUUID->"d28ad154-feec-4b44-86d1-3b399d96329a",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[2826, 82, 341, 8, 57, "Subsubtitle",ExpressionUUID->"6e44c6d6-3fbc-4265-9751-fef4e485686e",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[3170, 92, 1486, 28, 320, "Input",ExpressionUUID->"2bfa497a-f373-405e-a723-bfef431dc644",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[4693, 125, 278, 7, 41, "Subsubtitle",ExpressionUUID->"c3e96f5f-7777-4da0-89db-ac69d48ce737",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[4996, 136, 860, 15, 36, "Section",ExpressionUUID->"e445d6e0-5503-40e6-b249-bafeb479c090",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[5881, 155, 493, 10, 45, "Subsubsection",ExpressionUUID->"03f89ecf-f14d-437b-b70c-d97a5299587c",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[6377, 167, 1479, 24, 406, "Input",ExpressionUUID->"6aca0684-d5bc-4463-ad5a-84fe60cce813",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[7893, 196, 266, 6, 37, "Subsubsection",ExpressionUUID->"68cb664e-c697-42ca-a77a-6d47306ccebd",
 InitializationCell->True],
Cell[8162, 204, 582, 13, 91, "Input",ExpressionUUID->"8bd1d0fb-705e-4801-bc5f-9858bcdf40e8",
 InitializationCell->True],
Cell[8747, 219, 602, 13, 91, "Input",ExpressionUUID->"d55af982-7217-4a53-94d5-92babb0c412c",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[9386, 237, 205, 4, 37, "Subsubsection",ExpressionUUID->"1c8bc702-3c1f-4171-911c-6efe6a83bc80",
 InitializationCell->True],
Cell[9594, 243, 496, 11, 91, "Input",ExpressionUUID->"1fd19918-44aa-42f5-85c6-7e028d914af0",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[10127, 259, 918, 14, 37, "Subsubsection",ExpressionUUID->"6e74398f-ff04-426f-a898-091cebb410ec",
 InitializationCell->True],
Cell[11048, 275, 423, 10, 91, "Input",ExpressionUUID->"cacf5db1-b9ce-4850-8a7f-b9ffb27078b9",
 InitializationCell->True],
Cell[11474, 287, 447, 10, 91, "Input",ExpressionUUID->"743078f6-9f64-42c8-8ab3-a08ca0d05c0e",
 InitializationCell->True],
Cell[11924, 299, 501, 11, 112, "Input",ExpressionUUID->"cc2f3737-acbd-4c0f-90cd-9683b47b64f9",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[12462, 315, 885, 13, 37, "Subsubsection",ExpressionUUID->"67c7b615-5fbc-45cb-9664-ab9879c37a4b",
 InitializationCell->True],
Cell[13350, 330, 941, 17, 194, "Input",ExpressionUUID->"569cb565-ba3a-4db0-98d1-5bbe5b16f430",
 InitializationCell->True],
Cell[14294, 349, 414, 9, 89, "Input",ExpressionUUID->"541593df-e626-4dc3-a2c0-fcdbbab71986",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[14745, 363, 991, 15, 45, "Subsubsection",ExpressionUUID->"3ea83b6a-8e4b-4d25-b6de-579fde0da160",
 InitializationCell->True],
Cell[15739, 380, 545, 12, 112, "Input",ExpressionUUID->"a881c8d3-a9bb-4a06-ac7a-b3d3592383a4",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[16333, 398, 763, 14, 36, "Section",ExpressionUUID->"7fbc7ad7-d9e9-441d-b4ea-569f15f739e4",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[17121, 416, 481, 10, 39, "Subsubsection",ExpressionUUID->"10ab5f22-2a2e-4d36-87d6-3be4871f8707",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[17605, 428, 2165, 34, 85, "Input",ExpressionUUID->"06fbb407-9ad2-4789-b9a9-7e6bdc8090d6",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[19807, 467, 231, 4, 49, "Subsubsection",ExpressionUUID->"09603298-2ea4-4233-a5ed-39e60a7c2e83",
 InitializationCell->True],
Cell[20041, 473, 424, 10, 66, "Input",ExpressionUUID->"ab3e33e1-7e6e-47d9-bbf7-65977f03f0a1",
 InitializationCell->True],
Cell[20468, 485, 494, 10, 85, "Input",ExpressionUUID->"0a42c1ad-3176-4426-bd09-a3d6e2a7675f",
 InitializationCell->True],
Cell[20965, 497, 448, 11, 85, "Input",ExpressionUUID->"85df9acf-d875-452a-8986-b4cf23b8d616",
 InitializationCell->True],
Cell[21416, 510, 397, 10, 85, "Input",ExpressionUUID->"a42c8a19-f580-47e3-9fb6-91815ca1a2fb",
 InitializationCell->True],
Cell[21816, 522, 468, 10, 85, "Input",ExpressionUUID->"4add872f-c24a-484c-bdd6-80b4eccaace0",
 InitializationCell->True],
Cell[22287, 534, 395, 10, 85, "Input",ExpressionUUID->"41ab7692-f980-44f2-b952-698c5b283ede",
 InitializationCell->True],
Cell[22685, 546, 1091, 20, 161, "Input",ExpressionUUID->"3d291e98-fded-4198-9a7a-8ee914c2f6e6",
 InitializationCell->True],
Cell[23779, 568, 1131, 20, 142, "Input",ExpressionUUID->"ef655e92-7d05-442d-94b7-6d8905ac8acb",
 InitializationCell->True],
Cell[24913, 590, 1207, 21, 256, "Input",ExpressionUUID->"7879936f-cbf6-4ff0-9b80-1eeb573da296",
 InitializationCell->True],
Cell[26123, 613, 1512, 25, 201, "Input",ExpressionUUID->"5d791b3a-cad8-486d-b329-04f00dbddc87",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[27684, 644, 794, 14, 36, "Section",ExpressionUUID->"030a298a-2990-463c-9492-f393613422e0",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[28503, 662, 535, 11, 35, "Subsubsection",ExpressionUUID->"bfad492b-a589-4c49-b139-395b976c9780",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[29041, 675, 2300, 37, 91, "Input",ExpressionUUID->"2bce2125-94bc-423b-8e48-4fa7fa59d091",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[31378, 717, 577, 11, 35, "Subsubsection",ExpressionUUID->"0ef4b960-4c9d-4f1d-9b88-df93e4e56422",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[31958, 730, 773, 14, 133, "Input",ExpressionUUID->"3eb9acf3-b789-44e7-a33d-e63cf7e6ba2d",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[32780, 750, 896, 16, 36, "Section",ExpressionUUID->"5d954f93-85ce-49d4-8580-0a3b289e5626",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[33701, 770, 804, 14, 35, "Subsubsection",ExpressionUUID->"cf18bcb9-0e15-424e-b107-95796338b06f",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[34508, 786, 581, 13, 70, "Input",ExpressionUUID->"af1ccd96-b7a1-4a6d-b23e-7a95a1a42787",
 InitializationCell->True],
Cell[35092, 801, 509, 12, 112, "Input",ExpressionUUID->"8e160aa2-ca5a-4c59-9908-4033b92207c4",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[35638, 818, 410, 9, 35, "Subsubsection",ExpressionUUID->"60590028-251a-4fc0-8fb4-7cf747ca95d3",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[36051, 829, 441, 11, 91, "Input",ExpressionUUID->"934133a6-961f-40fd-b756-50aef022b537",
 InitializationCell->True],
Cell[36495, 842, 436, 11, 91, "Input",ExpressionUUID->"8f6d1f67-0038-42a9-af82-c436af20b6c9",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[36968, 858, 813, 14, 35, "Subsubsection",ExpressionUUID->"125f548f-0951-467a-b22b-183cbf02b032",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[37784, 874, 639, 13, 91, "Input",ExpressionUUID->"ef2adbc7-4afb-49a4-b3b0-525ff89fbfa7",
 InitializationCell->True],
Cell[38426, 889, 692, 15, 91, "Input",ExpressionUUID->"871dbf5c-8027-4a94-88b9-97477a3dea87",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[39155, 909, 264, 4, 35, "Subsubsection",ExpressionUUID->"e6ac0f0c-a803-4459-8f8e-355106282c0a"],
Cell[39422, 915, 679, 13, 133, "Input",ExpressionUUID->"7939091f-ae20-4d0a-806c-ac0e5e5c940d",
 InitializationCell->True],
Cell[40104, 930, 452, 10, 70, "Input",ExpressionUUID->"04fdc510-047d-4968-9bff-0dcb87ef2865",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[40605, 946, 1050, 18, 36, "Section",ExpressionUUID->"fa172ec7-49b1-44c0-aed5-ac2de417cd85",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[41658, 966, 2613, 40, 112, "Input",ExpressionUUID->"24ee6785-5bca-4824-958b-ed322f3f878c",
 InitializationCell->True],
Cell[44274, 1008, 2844, 43, 175, "Input",ExpressionUUID->"f37de882-975d-4636-a103-43a7c20782ad",
 InitializationCell->True],
Cell[47121, 1053, 338, 8, 48, "Input",ExpressionUUID->"3e63b788-842e-4e12-a2b8-f86cd3a29f7f",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[47496, 1066, 852, 15, 36, "Section",ExpressionUUID->"c6817669-6518-4a17-9cf5-eb83a84c33b0",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[48373, 1085, 293, 5, 34, "Subsubsection",ExpressionUUID->"3dfaa3fc-98f4-4163-8335-363fc8db2868",
 InitializationCell->True],
Cell[48669, 1092, 1168, 20, 218, "Input",ExpressionUUID->"920b749e-5813-4523-a5de-295d891e8308",
 InitializationCell->True],
Cell[49840, 1114, 397, 10, 46, "Input",ExpressionUUID->"e1578a28-487b-46ed-a2c5-24054f1a6555",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[50274, 1129, 291, 5, 28, "Subsubsection",ExpressionUUID->"2d67b4f4-b862-49e3-b499-31dabf85a5fe",
 InitializationCell->True],
Cell[50568, 1136, 504, 11, 91, "Input",ExpressionUUID->"79640fce-7560-48c2-aad8-baf35eaa025a",
 InitializationCell->True],
Cell[51075, 1149, 544, 11, 91, "Input",ExpressionUUID->"82f47344-a58b-4e53-89ca-98e1566a9c31",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[51668, 1166, 978, 17, 36, "Section",ExpressionUUID->"8f78acff-3c5e-4b00-96c3-3b209aa9e9db",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[52671, 1187, 144, 3, 45, "Subsubsection",ExpressionUUID->"998ccb7b-858e-4ae2-ab85-8646f212a022"],
Cell[52818, 1192, 460, 10, 52, "Code",ExpressionUUID->"3955ab39-ed9c-4d9e-9619-361c1611edbd"],
Cell[53281, 1204, 397, 10, 52, "Code",ExpressionUUID->"6b44094e-03fb-4b54-8805-a21e6a960e03"]
}, Closed]],
Cell[CellGroupData[{
Cell[53715, 1219, 177, 3, 37, "Subsubsection",ExpressionUUID->"61b91b07-b58e-4e9f-90a0-abe4509a4dc4"],
Cell[53895, 1224, 452, 9, 52, "Code",ExpressionUUID->"7d7079dc-f068-4ca7-81fc-d909da5a1228"]
}, Closed]],
Cell[CellGroupData[{
Cell[54384, 1238, 199, 4, 37, "Subsubsection",ExpressionUUID->"79708f8f-f7f0-47c4-a548-206c75be708b"],
Cell[54586, 1244, 336, 9, 70, "Input",ExpressionUUID->"6396e5b2-c2a5-476a-b1d9-c9f6dd213bb9",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[54971, 1259, 1165, 19, 36, "Section",ExpressionUUID->"ab364176-e77a-401a-8f1a-a06ec8d0b739",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[56139, 1280, 622, 12, 112, "Input",ExpressionUUID->"f9073b9e-2041-4d79-84ba-6f4193c36546",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[56810, 1298, 524, 9, 69, "Subtitle",ExpressionUUID->"f6a4639d-21b9-47cf-80c2-6ab689b66ada",
 InitializationCell->True],
Cell[57337, 1309, 299, 7, 48, "Input",ExpressionUUID->"386fe2da-14a9-4ee0-81fc-8eece48e28b6",
 InitializationCell->True],
Cell[57639, 1318, 201, 5, 48, "Input",ExpressionUUID->"b663f77e-9118-45ef-856b-5369c5612cdc",
 InitializationCell->True,
 InitializationGroup->True]
}, Closed]],
Cell[CellGroupData[{
Cell[57877, 1328, 540, 9, 69, "Subtitle",ExpressionUUID->"7d849820-cf44-4104-a28e-adcd3126e45a",
 InitializationCell->True],
Cell[58420, 1339, 555, 10, 58, "Text",ExpressionUUID->"0e610048-9225-46f6-b7ef-b6331670b097",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[59000, 1353, 911, 14, 41, "Subsubtitle",ExpressionUUID->"63a7b40a-a51f-4805-bd10-b9328fae2fc0",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[59936, 1371, 322, 5, 39, "Section",ExpressionUUID->"3aaf33e7-1cd7-4380-a8df-7bbbd604f0b5",
 InitializationCell->True],
Cell[60261, 1378, 356, 8, 49, "Text",ExpressionUUID->"c087b7ff-50f2-4778-8e9c-7204b11a8b90",
 InitializationCell->True],
Cell[60620, 1388, 966, 24, 92, "Input",ExpressionUUID->"6125cabb-d8c1-4147-b945-dec5292c9d2a",
 InitializationCell->True],
Cell[61589, 1414, 248, 6, 48, "Input",ExpressionUUID->"a7787159-f280-4cd7-b1a5-4ad8e3dec3a4",
 InitializationCell->True],
Cell[61840, 1422, 455, 11, 50, "Input",ExpressionUUID->"df8a71e7-e105-456d-a7a5-e03a7681a616",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[62332, 1438, 334, 6, 39, "Section",ExpressionUUID->"d2619672-7b04-48d9-b73e-75ffc3560767",
 InitializationCell->True],
Cell[62669, 1446, 409, 8, 49, "Text",ExpressionUUID->"dceda209-3c06-4c5d-99af-083eda26b69f",
 InitializationCell->True],
Cell[63081, 1456, 915, 23, 92, "Input",ExpressionUUID->"d6011324-76e4-40d6-b47e-a79f6f8b0d15",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[64033, 1484, 469, 8, 39, "Section",ExpressionUUID->"0353233e-e4fc-4e8a-9777-249f5f7974c9",
 InitializationCell->True],
Cell[64505, 1494, 450, 9, 49, "Text",ExpressionUUID->"d1858c31-fd1d-4982-aa1e-e752fff32312"],
Cell[CellGroupData[{
Cell[64980, 1507, 407, 7, 49, "Subsection",ExpressionUUID->"5ff53a0e-a196-416e-ab94-a89579ed3b81",
 InitializationCell->True],
Cell[65390, 1516, 996, 22, 112, "Input",ExpressionUUID->"aff8381b-b213-4d5b-b3a7-0ec547716bb4",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[66423, 1543, 656, 10, 41, "Subsection",ExpressionUUID->"b781b4f1-2a9e-459f-b352-9656a54bf14b",
 InitializationCell->True],
Cell[67082, 1555, 1714, 39, 154, "Input",ExpressionUUID->"574d6238-21ba-4567-b107-f2464046372d",
 InitializationCell->True],
Cell[68799, 1596, 690, 14, 48, "Input",ExpressionUUID->"4ef724b0-808d-4ead-8dd0-22cf4c6c78d1",
 InitializationCell->True],
Cell[69492, 1612, 360, 9, 71, "Input",ExpressionUUID->"3a3fbe7f-5351-4817-987a-e6ea08cddeb1",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[69889, 1626, 510, 8, 41, "Subsection",ExpressionUUID->"66610759-a816-4914-887b-9d5575fe512d",
 InitializationCell->True],
Cell[70402, 1636, 834, 15, 91, "Input",ExpressionUUID->"c090f14c-eb9b-4c56-9692-16db01f0c87f",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[71273, 1656, 521, 8, 41, "Subsection",ExpressionUUID->"b47969f6-d612-4149-b402-24cd4ef087af",
 InitializationCell->True],
Cell[71797, 1666, 396, 9, 71, "Input",ExpressionUUID->"97aebe76-d776-49c3-82e6-eb2dbfa5e15e",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[72230, 1680, 387, 7, 41, "Subsection",ExpressionUUID->"e3db0bf6-9fda-42a2-9ecf-ddf5119cdd54",
 InitializationCell->True],
Cell[72620, 1689, 252, 4, 30, "Text",ExpressionUUID->"c07ee9d2-9cf5-4676-a1d6-5e2ee6b2a686"],
Cell[72875, 1695, 1215, 27, 87, "Input",ExpressionUUID->"eedf6f9f-2920-483f-82ec-882144e54fe5",
 InitializationCell->True],
Cell[74093, 1724, 1135, 25, 143, "Input",ExpressionUUID->"12a9ed6f-81c8-4422-97ec-262082987ba1",
 InitializationCell->True],
Cell[75231, 1751, 847, 19, 106, "Input",ExpressionUUID->"9bf0c0f4-27e0-4fd5-9180-2a020700ef14",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[76139, 1777, 405, 7, 41, "Subsubtitle",ExpressionUUID->"22b8ed96-6222-4b25-b829-a388785f63b4",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[76569, 1788, 172, 3, 44, "Subsection",ExpressionUUID->"47748f9d-b4cd-4b50-a8d3-edd0c9374add"],
Cell[76744, 1793, 402, 10, 48, "Input",ExpressionUUID->"409b65e6-8cee-4368-9f81-c4a690516fbb",
 InitializationCell->True],
Cell[77149, 1805, 367, 10, 48, "Input",ExpressionUUID->"e8344ead-015b-4fbc-9d7d-7b549bde50a2",
 InitializationCell->True],
Cell[77519, 1817, 319, 9, 48, "Input",ExpressionUUID->"5323e98c-8d56-4062-9d16-07af069252a8",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[77875, 1831, 155, 3, 36, "Subsection",ExpressionUUID->"e0a1b0ed-e9b4-4681-b226-4a2c192b634f"],
Cell[78033, 1836, 479, 10, 49, "Text",ExpressionUUID->"db23e459-df29-422c-a36f-d581ea5520e9",
 InitializationCell->True],
Cell[78515, 1848, 280, 7, 30, "Text",ExpressionUUID->"9410e384-6f37-4b31-8fc6-ebfa221a0ffc"],
Cell[78798, 1857, 351, 9, 48, "Input",ExpressionUUID->"c1ca660e-662b-4a15-a5c6-8b8be3acffb6",
 InitializationCell->True],
Cell[79152, 1868, 614, 13, 48, "Input",ExpressionUUID->"82328dc2-a3f4-4177-b38e-a6de810c1c03",
 InitializationCell->True],
Cell[79769, 1883, 463, 11, 70, "Input",ExpressionUUID->"682d521d-e10b-4ded-937b-51c5b108d8d9",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[80281, 1900, 300, 5, 41, "Subsubtitle",ExpressionUUID->"911fa07b-7c21-4b76-a897-7ce41702d784",
 InitializationCell->True],
Cell[80584, 1907, 402, 8, 49, "Text",ExpressionUUID->"080ce6fd-4dc8-4190-8095-3f4c1f0de93e",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[81011, 1919, 396, 7, 39, "Subsubsection",ExpressionUUID->"7b8318fd-8632-4e3a-b367-ed3275616988",
 InitializationCell->True],
Cell[81410, 1928, 451, 11, 46, "Input",ExpressionUUID->"000fd58c-126a-4815-a71f-592ebe2c5715",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[81898, 1944, 303, 5, 33, "Subsubsection",ExpressionUUID->"e986dd95-e9f6-4e65-9942-84f53d334880",
 InitializationCell->True],
Cell[82204, 1951, 885, 20, 70, "Input",ExpressionUUID->"ea71c5a8-455a-410c-8e7e-adc4e207c305",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[83126, 1976, 370, 6, 33, "Subsubsection",ExpressionUUID->"c58d0339-022c-4bf7-a39e-ba4056b6e9b1",
 InitializationCell->True],
Cell[83499, 1984, 715, 19, 66, "Input",ExpressionUUID->"02bd5712-b208-48f2-a1c8-ff042fa58299",
 InitializationCell->True],
Cell[84217, 2005, 786, 20, 66, "Input",ExpressionUUID->"7588e4e8-9494-4407-b3eb-1d3091a26074",
 InitializationCell->True],
Cell[85006, 2027, 876, 20, 66, "Input",ExpressionUUID->"7d5e20d3-8d1e-4dd4-84dc-21847c5e90a0",
 InitializationCell->True],
Cell[85885, 2049, 1012, 23, 104, "Input",ExpressionUUID->"11f12166-e8ce-4421-8351-86a87575da9c",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[86934, 2077, 362, 6, 33, "Subsubsection",ExpressionUUID->"ab86cb5a-0955-4f9d-aefd-433e7f07e705",
 InitializationCell->True],
Cell[87299, 2085, 1620, 37, 161, "Input",ExpressionUUID->"5f2b2eda-f769-49c3-848a-dd24ee5a94c0",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[88956, 2127, 296, 5, 33, "Subsubsection",ExpressionUUID->"da22ab44-083b-44fd-9671-58d18c88b3bb",
 InitializationCell->True],
Cell[89255, 2134, 583, 12, 66, "Input",ExpressionUUID->"b61b344f-30e9-457f-959f-11af106f7235",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[89875, 2151, 354, 6, 33, "Subsubsection",ExpressionUUID->"3b989af0-03d4-46ae-8aac-59baf2fa2b7c",
 InitializationCell->True],
Cell[90232, 2159, 1165, 27, 143, "Input",ExpressionUUID->"3d3c9733-029d-4d7f-8fd7-d059dc8fa91e",
 InitializationCell->True],
Cell[91400, 2188, 1224, 30, 181, "Input",ExpressionUUID->"576509f0-7605-4e01-9a69-93e1b063ecd7",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[92661, 2223, 371, 6, 33, "Subsubsection",ExpressionUUID->"9c07ff43-fc9b-4b05-b0aa-536e69c3cfbe",
 InitializationCell->True],
Cell[93035, 2231, 1237, 32, 166, "Input",ExpressionUUID->"c4a5d5f9-61f8-4684-a76d-28af8988f0f5",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[94309, 2268, 363, 6, 33, "Subsubsection",ExpressionUUID->"07f4fe60-fd5e-467d-815d-152e819671b6",
 InitializationCell->True],
Cell[94675, 2276, 397, 10, 70, "Input",ExpressionUUID->"38b86f92-c12a-43cb-9b4d-105bbf78349c",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[95121, 2292, 937, 14, 41, "Subsubtitle",ExpressionUUID->"3f39e788-032a-401a-84be-b2232fcef1e0",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[96083, 2310, 497, 8, 44, "Subsection",ExpressionUUID->"fb6143f8-1eba-485b-9431-790712021bc9",
 InitializationCell->True],
Cell[96583, 2320, 369, 9, 48, "Input",ExpressionUUID->"11021d44-8264-48ac-8ae3-3e206905bf31",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[97001, 2335, 517, 10, 41, "Subsubtitle",ExpressionUUID->"dbb7611f-0ccc-457f-8ba5-770b92c44939",
 InitializationCell->True,
 CellTags->{"PrecalculationRules - working", "EventSpecificationList"}],
Cell[97521, 2347, 321, 7, 30, "Text",ExpressionUUID->"f96203a3-1df3-40b1-a8da-9b58da91dd29",
 InitializationCell->True],
Cell[97845, 2356, 414, 8, 30, "Text",ExpressionUUID->"070d1636-9898-45f1-a9c4-411ddcbe41ce",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[98284, 2368, 371, 7, 39, "Section",ExpressionUUID->"f3febb1c-5cc4-4b05-b3e5-462296ba04d5",
 InitializationCell->True],
Cell[98658, 2377, 1078, 21, 201, "Text",ExpressionUUID->"762b9976-b349-489d-ab70-7aa340952d8e",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[99773, 2403, 382, 7, 39, "Section",ExpressionUUID->"be139c20-8f97-4d97-9667-d60d7469852b",
 InitializationCell->True],
Cell[100158, 2412, 1327, 24, 91, "Input",ExpressionUUID->"5da2111a-512d-46a4-bb47-80ae5116612d",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[101522, 2441, 421, 7, 39, "Section",ExpressionUUID->"8e152ec3-5602-4c9c-8342-b745d44a7839",
 InitializationCell->True],
Cell[101946, 2450, 530, 14, 48, "Input",ExpressionUUID->"84b95d0f-3a13-407b-8a4f-bab4a2471a51",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[102513, 2469, 399, 7, 39, "Section",ExpressionUUID->"721d808a-977a-4851-bfb1-24e7d2aa1ae5",
 InitializationCell->True],
Cell[102915, 2478, 455, 9, 49, "Text",ExpressionUUID->"56f6bd58-803c-4596-915c-68cea3616c22",
 InitializationCell->True],
Cell[103373, 2489, 335, 9, 48, "Input",ExpressionUUID->"28b41ab0-e0c4-466a-b344-abac20e72fb7",
 InitializationCell->True],
Cell[103711, 2500, 690, 15, 70, "Input",ExpressionUUID->"9939a4af-9086-4a00-891b-e756f2b9643b",
 InitializationCell->True],
Cell[104404, 2517, 771, 17, 91, "Input",ExpressionUUID->"8188266a-a7d6-4499-8052-a95b60fa72ea",
 InitializationCell->True],
Cell[105178, 2536, 524, 14, 70, "Input",ExpressionUUID->"5a086a83-3041-40a6-a133-fd26effab7d2",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[105739, 2555, 459, 8, 39, "Section",ExpressionUUID->"59cfcc90-ebc0-4315-9a13-219a37fac72a",
 InitializationCell->True],
Cell[106201, 2565, 628, 15, 70, "Input",ExpressionUUID->"723cc71d-4ef3-47a1-8262-759681c41642",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[106866, 2585, 835, 13, 39, "Section",ExpressionUUID->"847bc678-d3f7-4742-9a43-d3d882091872",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[107726, 2602, 302, 5, 44, "Subsection",ExpressionUUID->"ef0e3636-2b30-4d3c-af2b-4681bb506c36",
 InitializationCell->True],
Cell[108031, 2609, 355, 8, 68, "Text",ExpressionUUID->"b54a845c-9f58-450b-a2a6-0e9867f2f861"],
Cell[108389, 2619, 1085, 24, 133, "Input",ExpressionUUID->"00a469f0-35cb-47fe-813e-38688d73d2a8",
 InitializationCell->True],
Cell[109477, 2645, 1093, 25, 133, "Input",ExpressionUUID->"2eb07680-cf8e-481e-8bf3-ba253a1476ab",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[110607, 2675, 336, 6, 36, "Subsection",ExpressionUUID->"68e2fb64-f5ac-428e-900d-67404a713477",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[110968, 2685, 295, 5, 35, "Subsubsection",ExpressionUUID->"24f65e0e-b1da-457f-96e1-d0e34067d5ba",
 InitializationCell->True],
Cell[111266, 2692, 1232, 30, 133, "Input",ExpressionUUID->"e1d42cbe-3546-4d25-8ce3-cb008303ba2b",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[112535, 2727, 687, 10, 29, "Subsubsection",ExpressionUUID->"eea80239-f3a9-4064-aa1d-185def147793",
 InitializationCell->True],
Cell[113225, 2739, 1481, 37, 238, "Input",ExpressionUUID->"0ba4d5fd-331d-438f-95f8-7c722afed365",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[114743, 2781, 221, 4, 29, "Subsubsection",ExpressionUUID->"7ee47225-10e5-41ca-aed6-0ef8a882af84",
 InitializationCell->True],
Cell[114967, 2787, 1175, 25, 133, "Input",ExpressionUUID->"3ac83fcf-5896-4a75-b094-b745cd50a84a",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[116179, 2817, 474, 9, 29, "Subsubsection",ExpressionUUID->"7b8cbd62-f49a-496e-9cb1-bfdf532af150",
 InitializationCell->True],
Cell[116656, 2828, 1271, 28, 175, "Input",ExpressionUUID->"115d1bfc-5a9d-4c54-a3a0-75c28a81be2d",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[117964, 2861, 298, 5, 29, "Subsubsection",ExpressionUUID->"dc6da514-2585-4627-9bd9-e4ade4fdb216",
 InitializationCell->True],
Cell[118265, 2868, 4476, 94, 553, "Input",ExpressionUUID->"3f2c46ef-5de3-40fd-ae10-37db61902700",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[122778, 2967, 305, 5, 35, "Subsubsection",ExpressionUUID->"e776e1b7-ba64-49cf-a647-895b7325e9b0",
 InitializationCell->True],
Cell[123086, 2974, 343, 5, 30, "Text",ExpressionUUID->"d4f14bd9-761d-4b26-bc57-c80bfdb03ef4"],
Cell[123432, 2981, 5100, 111, 574, "Input",ExpressionUUID->"164ff284-a8bc-4870-adde-73f64a38a096",
 InitializationCell->True],
Cell[128535, 3094, 340, 7, 30, "Text",ExpressionUUID->"95f71a18-b3e3-4676-a23e-46e2d43bb360"],
Cell[128878, 3103, 1013, 23, 112, "Input",ExpressionUUID->"b9c1ccce-18be-40c3-a8bb-a4b4d5cbcb16",
 InitializationCell->True],
Cell[129894, 3128, 1132, 25, 133, "Input",ExpressionUUID->"341e46ac-faf6-4fbc-8986-403e6614d165",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[131063, 3158, 640, 10, 35, "Subsubsection",ExpressionUUID->"8949da97-649c-4d76-b714-07026c6e6774",
 InitializationCell->True],
Cell[131706, 3170, 1258, 32, 196, "Input",ExpressionUUID->"e54b1fb6-d545-4871-bdb8-e81fd52c5ca3",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[133001, 3207, 725, 12, 29, "Subsubsection",ExpressionUUID->"36b7f580-d8fa-4684-97b6-011bf9c27d08",
 InitializationCell->True],
Cell[133729, 3221, 392, 7, 49, "Text",ExpressionUUID->"ea62fddc-817a-432e-90d0-139c07b2baa1"],
Cell[134124, 3230, 4337, 94, 616, "Input",ExpressionUUID->"373b4955-7a29-4842-b64e-a3791ffb1899",
 InitializationCell->True],
Cell[138464, 3326, 646, 11, 49, "Text",ExpressionUUID->"3c5dd2b0-cd24-471b-a97c-a55a294d0e90"],
Cell[139113, 3339, 4844, 101, 658, "Input",ExpressionUUID->"c10af689-4121-453c-8095-061cf5221930",
 InitializationCell->True],
Cell[143960, 3442, 4760, 99, 658, "Input",ExpressionUUID->"f69a9378-830c-47d0-8472-7043344d3d20",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[148757, 3546, 645, 10, 35, "Subsubsection",ExpressionUUID->"64be09ce-ae9d-44bf-a330-a50e319133e8",
 InitializationCell->True],
Cell[149405, 3558, 339, 7, 49, "Text",ExpressionUUID->"613f7e19-cf83-461b-8ba2-beb31339a254"],
Cell[149747, 3567, 414, 8, 49, "Text",ExpressionUUID->"337f35ca-1ba6-4b7d-bd86-2775315f00de"],
Cell[150164, 3577, 231, 6, 30, "Text",ExpressionUUID->"7e2e4c63-5446-4282-9b66-452e74b8cd31"],
Cell[150398, 3585, 7303, 154, 952, "Input",ExpressionUUID->"fb32791d-6168-4cea-b5e3-756296b39812",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[157762, 3746, 393, 7, 39, "Section",ExpressionUUID->"228fb6c7-4440-4481-bb8d-911a9cdf2603",
 InitializationCell->True],
Cell[158158, 3755, 545, 11, 87, "Text",ExpressionUUID->"51f8096e-94b3-44de-af43-2e5bbc17c133",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[158728, 3770, 306, 5, 35, "Subsubsection",ExpressionUUID->"9c0a021e-55c2-4be3-81ac-320a7a144a24",
 InitializationCell->True],
Cell[159037, 3777, 749, 19, 70, "Input",ExpressionUUID->"ed20430b-9728-41f6-8a2e-abdd54d005ee",
 InitializationCell->True],
Cell[159789, 3798, 793, 20, 70, "Input",ExpressionUUID->"4b0836b0-0892-4045-94a7-4ff92aecba20",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[160619, 3823, 217, 4, 29, "Subsubsection",ExpressionUUID->"8287af4e-1776-40b9-8883-8a3082a6d238",
 InitializationCell->True],
Cell[160839, 3829, 953, 25, 91, "Input",ExpressionUUID->"08fdc227-fc89-4f7a-883a-4315185ee65b",
 InitializationCell->True],
Cell[161795, 3856, 4068, 87, 364, "Input",ExpressionUUID->"233afebb-c3a5-4e72-89bf-bcdc28078f85",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[165912, 3949, 874, 13, 39, "Section",ExpressionUUID->"02de2996-4bcc-4328-8e02-434a063f67ae",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[166811, 3966, 345, 6, 44, "Subsection",ExpressionUUID->"fda99d7e-eb4d-4716-833f-481cee6b7eba",
 InitializationCell->True],
Cell[167159, 3974, 1002, 23, 112, "Input",ExpressionUUID->"74451bd8-f5e4-4db4-ad4b-ec25eec8fc12",
 InitializationCell->True],
Cell[168164, 3999, 1047, 24, 112, "Input",ExpressionUUID->"2a537976-e29a-4484-9582-9c1a5140c38c",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[169248, 4028, 392, 7, 36, "Subsection",ExpressionUUID->"06e49fc0-f7c9-431c-9d48-b6a4aa2bfccc",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[169665, 4039, 293, 5, 35, "Subsubsection",ExpressionUUID->"de92505d-d136-414d-a764-d97828becc98",
 InitializationCell->True],
Cell[169961, 4046, 1028, 24, 133, "Input",ExpressionUUID->"b2f41a8d-3cc5-4e46-abf7-ca2b48c62e5a",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[171026, 4075, 279, 5, 29, "Subsubsection",ExpressionUUID->"6b928a63-8bd0-40b3-8643-482957d9398b",
 InitializationCell->True],
Cell[171308, 4082, 2485, 59, 238, "Input",ExpressionUUID->"a7de85dc-3056-465c-bb97-6a397741ff0a",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[173830, 4146, 285, 5, 35, "Subsubsection",ExpressionUUID->"4f893a2b-2fe0-4423-aa96-72d6b734e01c",
 InitializationCell->True],
Cell[174118, 4153, 1212, 29, 133, "Input",ExpressionUUID->"68ddfc19-6db0-4f64-87d7-348fa925f08e",
 InitializationCell->True],
Cell[175333, 4184, 1507, 36, 154, "Input",ExpressionUUID->"a9912b59-ed3e-484b-9fcc-776e0f5c2609",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[176901, 4227, 1298, 19, 39, "Section",ExpressionUUID->"fe7a76f9-4979-44b5-8fb7-fa719f7e1b55",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[178224, 4250, 986, 15, 44, "Subsection",ExpressionUUID->"e6b2e57a-6bea-4955-8f83-5dbe434f41ac",
 InitializationCell->True],
Cell[179213, 4267, 2438, 45, 91, "Input",ExpressionUUID->"cbbece38-b9bc-4a95-b6c2-18251d93e804",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[181688, 4317, 1100, 16, 44, "Subsection",ExpressionUUID->"5de6a9c0-24b4-4a4c-8ea2-1e6a6425951c",
 InitializationCell->True],
Cell[182791, 4335, 2523, 47, 91, "Input",ExpressionUUID->"e8c4d971-8480-4e6b-b322-08a64019337a",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[185351, 4387, 987, 15, 44, "Subsection",ExpressionUUID->"8a7c02e4-c1e8-489c-8bab-858a37612f29",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[186363, 4406, 303, 5, 35, "Subsubsection",ExpressionUUID->"49e1425d-bd92-4fe5-802e-ad2479805247",
 InitializationCell->True],
Cell[186669, 4413, 366, 8, 30, "Text",ExpressionUUID->"6d291b02-1714-4dfc-aa84-ea9f063a38c4",
 InitializationCell->True],
Cell[187038, 4423, 1123, 26, 112, "Input",ExpressionUUID->"a4cb3036-4d46-4ebf-8036-9d4478250d05",
 InitializationCell->True],
Cell[188164, 4451, 1162, 27, 112, "Input",ExpressionUUID->"3fd54889-51a0-49c8-9953-2bb282c8cef4",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[189363, 4483, 505, 8, 35, "Subsubsection",ExpressionUUID->"591d0724-128b-41bf-a15a-5ae9b4b7aa89",
 InitializationCell->True],
Cell[189871, 4493, 1463, 33, 175, "Input",ExpressionUUID->"6f53eea6-3c23-4b90-9ced-ad884d8ac1c9",
 InitializationCell->True],
Cell[191337, 4528, 339, 7, 30, "Text",ExpressionUUID->"1cb96aef-8fac-4e16-8eaa-5e094e7fdcbe"],
Cell[191679, 4537, 7558, 149, 595, "Input",ExpressionUUID->"a6c87318-0a6b-4468-b7ca-3e23b366e259",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[199298, 4693, 444, 7, 39, "Section",ExpressionUUID->"5011e495-cd25-4819-8ee2-ed923e1fbaa2",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[199767, 4704, 350, 6, 35, "Subsubsection",ExpressionUUID->"c6b436d8-5917-48b6-b299-c13449903b93",
 InitializationCell->True],
Cell[200120, 4712, 672, 19, 70, "Input",ExpressionUUID->"e8588fd7-3537-4d82-9a67-dc324e9a4803",
 InitializationCell->True],
Cell[200795, 4733, 719, 19, 70, "Input",ExpressionUUID->"26d274cb-33c2-40bb-b3c6-15e9c4fb2033",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[201551, 4757, 353, 6, 35, "Subsubsection",ExpressionUUID->"cd7ab4f7-e7a9-49e6-aeb6-839412530044",
 InitializationCell->True],
Cell[201907, 4765, 777, 21, 70, "Input",ExpressionUUID->"8e0325f8-d08a-4b6a-840b-ff9e2c5d33e7",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[202721, 4791, 411, 7, 35, "Subsubsection",ExpressionUUID->"058bf942-bb10-483f-b1bd-e4ba51b98161",
 InitializationCell->True],
Cell[203135, 4800, 877, 24, 70, "Input",ExpressionUUID->"f07c5966-e353-479c-a096-9572905f6f7a",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[204061, 4830, 182, 4, 39, "Section",ExpressionUUID->"6e86f818-eff8-439c-9a1a-64d240c24f61",
 InitializationCell->True],
Cell[204246, 4836, 546, 11, 48, "Input",ExpressionUUID->"e387baec-8106-4e86-86a2-960b9fc387e6",
 InitializationCell->True],
Cell[204795, 4849, 512, 11, 48, "Input",ExpressionUUID->"6f2bda63-d220-4815-b248-b7179997ba6d",
 InitializationCell->True],
Cell[205310, 4862, 446, 10, 112, "Input",ExpressionUUID->"a4ffcff0-e53d-4975-9706-ee25c205d721",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[205805, 4878, 668, 10, 41, "Subsubtitle",ExpressionUUID->"964bc46f-cc50-42d0-a1fc-78c5c0e42b3b",
 InitializationCell->True],
Cell[206476, 4890, 1250, 20, 150, "Text",ExpressionUUID->"970e0f98-ce33-44cc-89a7-0bc84bf6ead3",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[207751, 4914, 742, 11, 36, "Section",ExpressionUUID->"3a12f4b1-4796-446d-ba34-56bacbc4f761",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[208518, 4929, 385, 7, 45, "Subsubsection",ExpressionUUID->"990c6fcd-41de-499e-af64-7e14ed62448a",
 InitializationCell->True],
Cell[208906, 4938, 389, 10, 46, "Input",ExpressionUUID->"282d42cf-2a2e-4774-aaef-661c0b92d58c",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[209332, 4953, 378, 6, 45, "Subsubsection",ExpressionUUID->"e3a06712-80af-4ad8-a169-9698ce3465f6",
 InitializationCell->True],
Cell[209713, 4961, 1190, 27, 123, "Input",ExpressionUUID->"4c98df68-0fd1-4f21-affe-dd830d65e57e",
 InitializationCell->True],
Cell[210906, 4990, 1005, 26, 142, "Input",ExpressionUUID->"3bade84b-ba43-4231-93f1-3a0db18192e5",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[211948, 5021, 292, 5, 34, "Subsubsection",ExpressionUUID->"8798c4ea-4a16-4661-ba58-84c250af199f",
 InitializationCell->True],
Cell[212243, 5028, 1154, 30, 142, "Input",ExpressionUUID->"008d9a65-d056-4e63-bcf9-4c4878549412",
 InitializationCell->True],
Cell[213400, 5060, 943, 23, 104, "Input",ExpressionUUID->"fe4ee9f1-2312-467d-92da-9bf5a2ae58c0",
 InitializationCell->True],
Cell[214346, 5085, 966, 26, 123, "Input",ExpressionUUID->"35a8131a-15e0-4dc2-b5a3-8a1a964d3716",
 InitializationCell->True],
Cell[215315, 5113, 674, 17, 104, "Input",ExpressionUUID->"c544bf46-0e19-436c-a9ed-c1dd94e2a5e5",
 InitializationCell->True],
Cell[215992, 5132, 1229, 26, 105, "Input",ExpressionUUID->"f702515d-bc37-44c9-aafc-42a1b41d69ad",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[217258, 5163, 302, 5, 34, "Subsubsection",ExpressionUUID->"65cdf9b7-6d03-4960-882a-17a6199288b4",
 InitializationCell->True],
Cell[217563, 5170, 1722, 32, 142, "Input",ExpressionUUID->"35f8ac68-0967-472e-9c45-01dce1e98010",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[219322, 5207, 330, 6, 34, "Subsubsection",ExpressionUUID->"88a2ac90-f4b2-4dfb-90d1-55fa0d832488",
 InitializationCell->True],
Cell[219655, 5215, 1989, 39, 199, "Input",ExpressionUUID->"023025f0-7347-47d5-be10-874a7e4db287",
 InitializationCell->True],
Cell[221647, 5256, 2590, 53, 124, "Input",ExpressionUUID->"dd709984-4bec-40af-bf8d-9800bb44c1a2",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[224274, 5314, 283, 5, 34, "Subsubsection",ExpressionUUID->"a67e0d2d-c7ea-4eb0-ac71-123a2f713aa6",
 InitializationCell->True],
Cell[224560, 5321, 371, 9, 66, "Input",ExpressionUUID->"e2dd347c-383d-433d-aca6-c96bb4748ab5",
 InitializationCell->True],
Cell[224934, 5332, 505, 11, 85, "Input",ExpressionUUID->"8f8b2339-999a-488b-af3f-6ec7823143a2",
 InitializationCell->True],
Cell[225442, 5345, 394, 9, 66, "Input",ExpressionUUID->"9dcca2d3-0d07-4c8e-bf4d-e2ee82a920b2",
 InitializationCell->True]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[225885, 5360, 686, 11, 33, "Section",ExpressionUUID->"9b23ef1f-2800-4790-8c2f-e4d97d210764",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[226596, 5375, 714, 11, 43, "Subsection",ExpressionUUID->"b0711809-9960-4d3f-9bbf-b6ef35884f3e",
 InitializationCell->True],
Cell[227313, 5388, 281, 7, 30, "Text",ExpressionUUID->"9cf13029-e13c-44c4-94bc-1b94fa7c8a6d",
 InitializationCell->True],
Cell[227597, 5397, 2705, 62, 161, "Input",ExpressionUUID->"f2e4de2b-6453-48cd-8953-2a5ae917a9e9",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[230339, 5464, 660, 10, 43, "Subsection",ExpressionUUID->"d80351dc-2d20-450e-9832-f4ba500ab416",
 InitializationCell->True],
Cell[231002, 5476, 466, 9, 49, "Text",ExpressionUUID->"cb3a2dbb-28ba-4d20-9ee1-97db550f21c3",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[231493, 5489, 303, 5, 34, "Subsubsection",ExpressionUUID->"ff6d49f6-7f8f-4484-ab65-c8512cb75cf9",
 InitializationCell->True],
Cell[231799, 5496, 1605, 33, 67, "Input",ExpressionUUID->"a923afc9-3fb0-4cac-8d4f-1f24c15cf596",
 InitializationCell->True],
Cell[233407, 5531, 1306, 25, 66, "Input",ExpressionUUID->"7a10f97d-bf72-4b5a-875b-a2b78502cae8",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[234750, 5561, 269, 5, 34, "Subsubsection",ExpressionUUID->"92a4180c-8e0c-4dad-90fe-309118a4de81",
 InitializationCell->True],
Cell[235022, 5568, 1309, 28, 46, "Input",ExpressionUUID->"68aa72fe-93cd-4278-acf1-9b207a342273",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[236368, 5601, 390, 6, 34, "Subsubsection",ExpressionUUID->"371aa48e-d8d1-4666-bc63-d371889077f2",
 InitializationCell->True],
Cell[236761, 5609, 6047, 102, 180, "Input",ExpressionUUID->"88da2a19-22f7-4431-9ffb-c54ad7612103",
 InitializationCell->True]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[242857, 5717, 261, 5, 43, "Subsection",ExpressionUUID->"18f54331-80d4-4a90-99cc-53a94e2fd46e",
 InitializationCell->True],
Cell[243121, 5724, 418, 9, 49, "Text",ExpressionUUID->"08028e46-1fd3-44c4-950e-80dd9c2d6581",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[243564, 5737, 239, 4, 34, "Subsubsection",ExpressionUUID->"ee52a645-8bec-48a9-b09d-50db3ab5bd06",
 InitializationCell->True],
Cell[243806, 5743, 518, 13, 46, "Input",ExpressionUUID->"9f497d1b-ee12-4d96-beb7-4825dd79951d",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[244361, 5761, 399, 9, 34, "Subsubsection",ExpressionUUID->"157d0ce4-7210-43e5-88d7-d0506adc0320",
 InitializationCell->True],
Cell[244763, 5772, 1073, 25, 66, "Input",ExpressionUUID->"ff7f2d11-7e6b-4238-80f1-7aa92c772f7e",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[245873, 5802, 421, 7, 34, "Subsubsection",ExpressionUUID->"cc88bf87-bc59-40ba-a257-87ab84ab7b35",
 InitializationCell->True],
Cell[246297, 5811, 2202, 49, 142, "Input",ExpressionUUID->"cff19ebd-5777-46be-b0bb-3a1c2b840a6a",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[248536, 5865, 360, 7, 34, "Subsubsection",ExpressionUUID->"1e5996d2-0234-4839-a20b-fcc439b4e5d9",
 InitializationCell->True],
Cell[248899, 5874, 2738, 59, 218, "Input",ExpressionUUID->"2deea053-9e2d-4c5c-8e4c-17b5ecdb5a17",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[251674, 5938, 329, 8, 34, "Subsubsection",ExpressionUUID->"289c026b-5efa-472f-9cd7-fe1da20f95c0",
 InitializationCell->True],
Cell[252006, 5948, 3412, 73, 256, "Input",ExpressionUUID->"b05268a8-fffe-4ef5-968d-c1cf0af70a0e",
 InitializationCell->True]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[255467, 6027, 358, 6, 43, "Subsection",ExpressionUUID->"30f8b561-1f11-4c72-8574-d668f4933010",
 InitializationCell->True],
Cell[255828, 6035, 364, 8, 49, "Text",ExpressionUUID->"307e2706-387d-4317-9bf7-02998b01d5af",
 InitializationCell->True],
Cell[256195, 6045, 1117, 30, 67, "Input",ExpressionUUID->"a8caa22d-4ec8-4e44-9436-e4dff0ef1966",
 InitializationCell->True],
Cell[257315, 6077, 3305, 83, 237, "Input",ExpressionUUID->"f07724aa-bca3-43b4-a546-a985a70d1cf9",
 InitializationCell->True]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[260693, 6168, 812, 13, 69, "Subtitle",ExpressionUUID->"54e0fb25-289c-4485-8feb-1f22e137e576",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[261530, 6185, 649, 10, 41, "Subsubtitle",ExpressionUUID->"4a9cf47d-8688-4a3c-8e72-15e3abba5a96",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[262204, 6199, 482, 8, 44, "Section",ExpressionUUID->"8b04d86b-bb63-4155-9eba-e6652af56ad3",
 InitializationCell->True],
Cell[262689, 6209, 1909, 37, 320, "Input",ExpressionUUID->"3c8dfdde-594c-478d-b0b1-c41faff9f258",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[264635, 6251, 697, 11, 36, "Section",ExpressionUUID->"10ea614f-cedf-4eab-ad86-decea377b265",
 InitializationCell->True],
Cell[265335, 6264, 442, 13, 48, "Input",ExpressionUUID->"04026fe6-ef9b-4b4a-9bb3-e01843376fb0",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[265826, 6283, 635, 9, 41, "Subsubtitle",ExpressionUUID->"99a900ab-6db8-4294-b4e0-1cb6dec9b4b8",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[266486, 6296, 348, 5, 36, "Section",ExpressionUUID->"5cdb659b-0fba-4510-ac99-c904f4a6082c"],
Cell[CellGroupData[{
Cell[266859, 6305, 174, 3, 44, "Subsection",ExpressionUUID->"81321411-4ad4-4bca-a707-ba693aa2f13f"],
Cell[267036, 6310, 250, 6, 49, "Text",ExpressionUUID->"e33b998e-f34e-45b3-893a-66267fbe4a1a"]
}, Closed]],
Cell[CellGroupData[{
Cell[267323, 6321, 327, 6, 36, "Subsection",ExpressionUUID->"f5411a0c-7fbc-4027-8b4e-d5bfaaee2334"],
Cell[CellGroupData[{
Cell[267675, 6331, 248, 4, 35, "Subsubsection",ExpressionUUID->"37e0e147-143e-47e2-a0c6-06876ba000e7"],
Cell[267926, 6337, 405, 9, 68, "Text",ExpressionUUID->"a4e1119a-487f-4acf-bb4c-0f8e3710cfc4"]
}, Closed]],
Cell[CellGroupData[{
Cell[268368, 6351, 252, 4, 29, "Subsubsection",ExpressionUUID->"42d3edda-fc5a-4919-8914-e5de2c866cb1"],
Cell[268623, 6357, 321, 7, 85, "Text",ExpressionUUID->"09a97470-0a10-45ae-83c3-845383b538eb"]
}, Closed]],
Cell[CellGroupData[{
Cell[268981, 6369, 184, 3, 29, "Subsubsection",ExpressionUUID->"e5688cc5-e6f2-411d-8123-0d4ae798d203"],
Cell[269168, 6374, 169, 3, 30, "Text",ExpressionUUID->"0884613f-2fc3-4187-95c7-477e449726f9"]
}, Closed]],
Cell[CellGroupData[{
Cell[269374, 6382, 190, 3, 35, "Subsubsection",ExpressionUUID->"f7227176-75bc-47d4-a39d-0803059a3dfc"],
Cell[269567, 6387, 220, 5, 37, "Text",ExpressionUUID->"8781db84-dbf7-46dc-a3a4-70af809d6268"]
}, Closed]],
Cell[CellGroupData[{
Cell[269824, 6397, 283, 4, 29, "Subsubsection",ExpressionUUID->"758d80c1-4292-4874-aaa0-9c1e64278a1e"],
Cell[270110, 6403, 253, 7, 68, "Text",ExpressionUUID->"190b9038-8e9f-4664-8061-bb211196bc6a"]
}, Closed]],
Cell[CellGroupData[{
Cell[270400, 6415, 333, 5, 29, "Subsubsection",ExpressionUUID->"1ac074c2-1b2f-4de3-be09-ef9885fa4e49"],
Cell[270736, 6422, 277, 7, 49, "Text",ExpressionUUID->"cd17452c-41df-4bda-a3c5-56393d971b53"]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[271074, 6436, 230, 4, 36, "Section",ExpressionUUID->"a8fb6b7a-c9a4-4052-9a6e-e1a8a3c829fd"],
Cell[CellGroupData[{
Cell[271329, 6444, 262, 5, 35, "Subsubsection",ExpressionUUID->"69cc3e14-733a-446f-97d2-a3f8e24a1609",
 InitializationCell->True],
Cell[271594, 6451, 586, 14, 48, "Input",ExpressionUUID->"c97a54ec-c2a7-4a45-b828-1e07a9660938",
 InitializationCell->True],
Cell[272183, 6467, 794, 21, 70, "Input",ExpressionUUID->"0f093802-91b2-4b7e-b4fc-5c0649ae1938",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[273026, 6494, 237, 4, 36, "Section",ExpressionUUID->"ffdb3ea2-f96a-4c04-8138-43f15eda8bb7"],
Cell[273266, 6500, 404, 10, 48, "Input",ExpressionUUID->"0a989bf7-73c3-4e45-a7d5-9c98e69980f1",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[273695, 6514, 309, 5, 35, "Subsubsection",ExpressionUUID->"f698207f-f2ab-4ffa-a82b-1ec90e80846b",
 InitializationCell->True],
Cell[274007, 6521, 593, 17, 70, "Input",ExpressionUUID->"d1a27a5c-479a-40a5-be59-a4cca9153de1",
 InitializationCell->True],
Cell[274603, 6540, 1014, 25, 91, "Input",ExpressionUUID->"da4c5cdb-14bc-489a-bea7-2b9c987f92d2",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[275654, 6570, 250, 5, 29, "Subsubsection",ExpressionUUID->"2e982b5e-3140-44fb-ba99-92705fafdd22",
 InitializationCell->True],
Cell[275907, 6577, 1178, 27, 154, "Input",ExpressionUUID->"a77aae09-bfc2-4753-afdd-3c7a23ef98cb",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[277122, 6609, 361, 6, 29, "Subsubsection",ExpressionUUID->"4d93bce9-fcf3-473a-ac6b-54b7f276c83c",
 InitializationCell->True],
Cell[277486, 6617, 1070, 25, 154, "Input",ExpressionUUID->"da67b12c-f884-458d-b318-77b330244fab",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[278605, 6648, 161, 3, 36, "Section",ExpressionUUID->"d611554f-3770-4589-905e-38b19b1cfd03"],
Cell[CellGroupData[{
Cell[278791, 6655, 449, 7, 35, "Subsubsection",ExpressionUUID->"1ef4b491-adae-448a-81f5-d6ac1cf05e52",
 InitializationCell->True],
Cell[279243, 6664, 331, 7, 49, "Text",ExpressionUUID->"7a985d63-07cd-4c27-a08e-a09e1a43db4d"],
Cell[279577, 6673, 394, 8, 49, "Text",ExpressionUUID->"2705352d-bbd1-4937-94cb-52204361a98c"],
Cell[279974, 6683, 2337, 57, 196, "Input",ExpressionUUID->"ed2379ec-fd89-4aa3-aae9-2c0989bd643b",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[282360, 6746, 254, 4, 36, "Section",ExpressionUUID->"dc9258ee-f71e-4c2b-ab84-cfb33b4da93c"],
Cell[CellGroupData[{
Cell[282639, 6754, 157, 3, 45, "Subsubsection",ExpressionUUID->"1cb946b5-0262-4d82-a777-9281e2e42ead"],
Cell[282799, 6759, 438, 9, 35, "Text",ExpressionUUID->"4bef32a1-bb6d-43ca-ac46-d3a6e28be19a",
 InitializationCell->True],
Cell[283240, 6770, 1033, 29, 89, "Input",ExpressionUUID->"00e7f44c-d898-4508-837d-25e8848e55a4",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[284310, 6804, 158, 3, 45, "Subsubsection",ExpressionUUID->"ab9a2857-0b8c-42f2-b237-9b3810caaa8d"],
Cell[284471, 6809, 678, 19, 70, "Input",ExpressionUUID->"68d66e55-9195-4e6c-a2b3-e1a0fdbdd7f7",
 InitializationCell->True],
Cell[285152, 6830, 620, 18, 70, "Input",ExpressionUUID->"eaedc20c-720c-4b32-8098-3082d109bde4",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[285821, 6854, 299, 5, 36, "Section",ExpressionUUID->"f49a92a7-6e3b-4c06-af02-cfdcd8c80aa3"],
Cell[CellGroupData[{
Cell[286145, 6863, 834, 12, 44, "Subsection",ExpressionUUID->"2c465202-b08f-4dc7-b80a-5b8a7299201a"],
Cell[286982, 6877, 1910, 43, 133, "Input",ExpressionUUID->"02f91f8e-c33b-45e8-824e-2743aced3c16",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[288929, 6925, 968, 15, 36, "Subsection",ExpressionUUID->"99cb97de-a0cb-4be1-a36d-a3360f50ac98"],
Cell[289900, 6942, 264, 6, 30, "Text",ExpressionUUID->"be78e486-b752-46c8-b493-47dbe179e671"],
Cell[CellGroupData[{
Cell[290189, 6952, 2734, 59, 259, "Input",ExpressionUUID->"bd9f4ef1-9259-4da8-bcf8-28f9e6bec5bb",
 InitializationCell->True],
Cell[292926, 7013, 305, 8, 24, "Message",ExpressionUUID->"b0e1c270-5d5f-4f6b-981e-1d335ab8af09"]
}, Closed]],
Cell[CellGroupData[{
Cell[293268, 7026, 2737, 59, 274, "Input",ExpressionUUID->"de84e5d2-062e-432d-8700-ce18cffdb89a",
 InitializationCell->True],
Cell[296008, 7087, 305, 8, 24, "Message",ExpressionUUID->"f670e2e0-4ca8-45e9-9b1a-2788c2c60a36"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[296362, 7101, 978, 14, 36, "Subsection",ExpressionUUID->"48a053e0-fd14-42e9-9c1f-9eb745b13f33"],
Cell[297343, 7117, 2115, 41, 133, "Input",ExpressionUUID->"ea1630e2-9ca5-4187-b4c5-aeea64e72dd7",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[299495, 7163, 720, 10, 36, "Subsection",ExpressionUUID->"7f03823b-efef-4df3-87f1-69690226bbcf"],
Cell[CellGroupData[{
Cell[300240, 7177, 702, 10, 35, "Subsubsection",ExpressionUUID->"ad408c22-087c-4808-92cb-d661048dd80c"],
Cell[300945, 7189, 2279, 50, 133, "Input",ExpressionUUID->"50a3278a-fba0-4219-9ed6-6187f2f11cd7",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[303261, 7244, 671, 10, 35, "Subsubsection",ExpressionUUID->"5dab55db-ebc2-43b9-bc93-cce62a4d8c76"],
Cell[303935, 7256, 1717, 38, 91, "Input",ExpressionUUID->"1216421f-6d55-42ec-87dc-020208c2fdb8",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[305725, 7302, 869, 13, 41, "Subsubtitle",ExpressionUUID->"f4277938-1f08-4c32-b0cd-995689f85af3",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[306619, 7319, 272, 5, 34, "Section",ExpressionUUID->"64d68854-25eb-40e9-9af5-c9d4d78df540"],
Cell[CellGroupData[{
Cell[306916, 7328, 349, 5, 44, "Subsection",ExpressionUUID->"4b3a67fd-1ccd-4237-ab45-b3b6958f213f"],
Cell[CellGroupData[{
Cell[307290, 7337, 357, 6, 35, "Subsubsection",ExpressionUUID->"a49e27fe-f213-4a39-aa9b-49cd7f4cab47"],
Cell[307650, 7345, 1068, 26, 154, "Input",ExpressionUUID->"d0c6a572-2e0e-4a8c-b550-c03ae72111e3",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[308755, 7376, 351, 5, 35, "Subsubsection",ExpressionUUID->"a5b2d11c-f0fe-4f3e-8aae-9c4899a03017"],
Cell[309109, 7383, 1000, 24, 133, "Input",ExpressionUUID->"f395fdb9-d164-495d-bac2-64f898bccf80",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[310146, 7412, 371, 6, 35, "Subsubsection",ExpressionUUID->"43a47e7e-1b85-429a-ad32-7e50ab9ad471"],
Cell[310520, 7420, 1110, 24, 91, "Input",ExpressionUUID->"3c273add-aad8-4fa1-be95-ace77cc6bac2",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[311679, 7450, 379, 6, 36, "Subsection",ExpressionUUID->"5b9fdac1-8adf-46d0-8b93-01d3ea4352e2"],
Cell[312061, 7458, 254, 6, 30, "Text",ExpressionUUID->"0b594fbf-027f-400c-9b06-9c0f2f46982e"],
Cell[312318, 7466, 1098, 27, 154, "Input",ExpressionUUID->"b288d4a3-b08c-4f1f-af87-36a55f54bce5",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[313453, 7498, 312, 5, 36, "Subsection",ExpressionUUID->"348bb81e-e17b-4b72-9fd6-7d8a5b1ae7d5"],
Cell[313768, 7505, 641, 16, 70, "Input",ExpressionUUID->"f7dc1fba-b2b0-4641-b744-bc486f2f49e5",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[314446, 7526, 346, 5, 44, "Subsection",ExpressionUUID->"0f308417-516f-47c4-9251-442dda40b82c"],
Cell[314795, 7533, 678, 16, 70, "Input",ExpressionUUID->"caf2ab4e-2c2d-4aec-91ce-66bc75a29be0",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[315522, 7555, 243, 5, 34, "Section",ExpressionUUID->"8bb77fe6-641b-4d17-9186-adb037f0e056"],
Cell[315768, 7562, 204, 3, 30, "Text",ExpressionUUID->"2fe800e6-c223-4508-90c7-499606fd4366"],
Cell[315975, 7567, 4442, 103, 616, "Input",ExpressionUUID->"26ccd987-e6d3-482a-8461-880975604ded",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[320454, 7675, 293, 5, 34, "Section",ExpressionUUID->"d7565882-126a-47c9-a447-3eb185cf9a23"],
Cell[320750, 7682, 218, 4, 30, "Text",ExpressionUUID->"56971b9e-27ed-4371-a5ef-6dd7ddc8322d"],
Cell[320971, 7688, 594, 13, 48, "Input",ExpressionUUID->"add70586-8f2e-44d0-9130-bfd8293f7623",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[321614, 7707, 571, 9, 41, "Subsubtitle",ExpressionUUID->"120563db-c9e5-41ee-b8bb-bd6373bf0abb",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[322210, 7720, 153, 3, 39, "Section",ExpressionUUID->"030f5f76-13c1-49bd-89c1-0e78faa382be"],
Cell[CellGroupData[{
Cell[322388, 7727, 159, 3, 39, "Subsubsection",ExpressionUUID->"85b7b8e8-cc5f-44b0-8d3d-5e9a19fd223a"],
Cell[322550, 7732, 2519, 62, 229, "Input",ExpressionUUID->"c517dd5c-1b65-4dca-9d53-d826cc34eade",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[325106, 7799, 160, 3, 33, "Subsubsection",ExpressionUUID->"30002d7b-00f4-4bdb-b46c-3803a3f57cb0"],
Cell[325269, 7804, 620, 16, 70, "Input",ExpressionUUID->"0b3a77c8-f6a5-48e6-b405-a5783882faff",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[325938, 7826, 223, 4, 39, "Section",ExpressionUUID->"82c5d2f9-f04a-4a1f-aadb-845f1a4506d1"],
Cell[CellGroupData[{
Cell[326186, 7834, 215, 4, 35, "Subsubsection",ExpressionUUID->"9cc803c5-6806-49a7-bff9-ad90e3949695"],
Cell[326404, 7840, 1978, 55, 217, "Input",ExpressionUUID->"17cea919-6fc7-4934-b54b-4aa1d4f023a4",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[328419, 7900, 157, 3, 29, "Subsubsection",ExpressionUUID->"97bb7cff-cdf1-44dd-9ec9-cb8f98b543b6"],
Cell[328579, 7905, 4792, 114, 448, "Input",ExpressionUUID->"0fb825d9-1dec-4c4d-a7b5-7d73a052de44",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[333408, 8024, 284, 5, 35, "Subsubsection",ExpressionUUID->"7b437d82-78cc-47bf-819d-e20264f71606"],
Cell[333695, 8031, 420, 8, 49, "Text",ExpressionUUID->"c39e1f14-1d61-4366-aa04-661579391a26"],
Cell[334118, 8041, 4372, 92, 637, "Input",ExpressionUUID->"16bd16fd-e2d4-4955-bee3-f7be4afbc8ad",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[338527, 8138, 158, 3, 35, "Subsubsection",ExpressionUUID->"8a898fe8-63ee-4b70-bf4e-7301e545bad8"],
Cell[338688, 8143, 587, 16, 70, "Input",ExpressionUUID->"31459d09-ca3b-488c-9165-63af4ef4b69e",
 InitializationCell->True],
Cell[339278, 8161, 668, 19, 70, "Input",ExpressionUUID->"20be9f17-df00-4a2f-8ea4-cc06b8f4799a",
 InitializationCell->True],
Cell[339949, 8182, 588, 16, 70, "Input",ExpressionUUID->"0c6bc07a-b229-4392-9e00-faaaa8a6d6ce",
 InitializationCell->True],
Cell[340540, 8200, 665, 19, 70, "Input",ExpressionUUID->"e750d540-dafe-402d-bd0e-17a905f461d3",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[341254, 8225, 260, 4, 39, "Section",ExpressionUUID->"dcb44ddc-951c-47ba-a225-b7fac540df71"],
Cell[CellGroupData[{
Cell[341539, 8233, 157, 3, 35, "Subsubsection",ExpressionUUID->"2c1f9e90-4e02-4cde-b306-5c90dc682bb8"],
Cell[341699, 8238, 457, 9, 30, "Text",ExpressionUUID->"6906c4ac-5c09-4d43-a124-a93817927e80"],
Cell[342159, 8249, 3115, 73, 238, "Input",ExpressionUUID->"ab69c563-b95a-47a6-a420-d519b026ff6e",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[345311, 8327, 158, 3, 35, "Subsubsection",ExpressionUUID->"0f7a2118-4bbb-43f1-954e-0e9831b62920"],
Cell[345472, 8332, 753, 20, 91, "Input",ExpressionUUID->"eeb9a307-90c5-4f7c-a18d-2b66d7a5a08d",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[346274, 8358, 333, 5, 39, "Section",ExpressionUUID->"64855f77-290a-4be0-b77f-0768adbdbced"],
Cell[346610, 8365, 274, 6, 30, "Text",ExpressionUUID->"64ce4b43-76b2-423c-9e3a-2538bb52e979"],
Cell[CellGroupData[{
Cell[346909, 8375, 311, 6, 35, "Subsubsection",ExpressionUUID->"b4a13f02-c554-4fb4-b3a0-c62da0202a86",
 InitializationCell->True],
Cell[347223, 8383, 2672, 61, 175, "Input",ExpressionUUID->"507ceb13-7c16-4c10-b0cd-3cf220965689",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[349932, 8449, 158, 3, 35, "Subsubsection",ExpressionUUID->"81ceceae-d0e4-4ed8-81ab-f85f010e14ea"],
Cell[350093, 8454, 783, 20, 91, "Input",ExpressionUUID->"d9c69c5c-e330-48df-b247-fff98b2fa088",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[350913, 8479, 334, 6, 35, "Subsubsection",ExpressionUUID->"9c51666b-9365-4db8-b2bb-fbc54b188ae0",
 InitializationCell->True],
Cell[351250, 8487, 478, 13, 48, "Input",ExpressionUUID->"ad037959-4cbc-4894-911f-b0cc05ac1533",
 InitializationCell->True],
Cell[351731, 8502, 479, 14, 48, "Input",ExpressionUUID->"767ce42b-c83e-4b18-9c20-d276ec41a71e",
 InitializationCell->True],
Cell[352213, 8518, 1186, 33, 91, "Input",ExpressionUUID->"cbd5c911-ce59-457c-96a4-ab05c0bedade",
 InitializationCell->True],
Cell[353402, 8553, 877, 23, 70, "Input",ExpressionUUID->"946abef3-f939-4a22-bc3f-e1ce34892f97",
 InitializationCell->True],
Cell[354282, 8578, 1166, 30, 91, "Input",ExpressionUUID->"c66a516a-416c-47ee-88da-1c255efbdc3b",
 InitializationCell->True],
Cell[355451, 8610, 453, 13, 48, "Input",ExpressionUUID->"ac603640-ffe7-4b5b-bda9-42d7ec28f2eb",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[355953, 8629, 391, 6, 39, "Section",ExpressionUUID->"1dae09ac-723e-4040-87fc-496135ef2641"],
Cell[CellGroupData[{
Cell[356369, 8639, 214, 3, 44, "Subsection",ExpressionUUID->"a286657e-f168-4dda-997e-08ffe7bfa022"],
Cell[356586, 8644, 219, 5, 30, "Text",ExpressionUUID->"553b7770-3246-4337-9b78-feb25a3de1aa"],
Cell[356808, 8651, 428, 8, 49, "Text",ExpressionUUID->"8e16a0a4-251a-4b26-b3d0-05971b0863ad"],
Cell[357239, 8661, 4154, 93, 700, "Input",ExpressionUUID->"a27549e3-4fcb-426b-b604-683f9de25aaf",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[361454, 8761, 754, 12, 41, "Subsubtitle",ExpressionUUID->"de734165-1451-4b23-a41f-c50d41f83a5f",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[362233, 8777, 289, 5, 33, "Section",ExpressionUUID->"1bf014b3-da8a-421c-9de3-03b31936147d"],
Cell[CellGroupData[{
Cell[362547, 8786, 230, 4, 43, "Subsection",ExpressionUUID->"c10b2b01-9a13-4b86-a398-0310eb0bead8"],
Cell[362780, 8792, 2668, 63, 238, "Input",ExpressionUUID->"afa1450d-d61b-474d-a7ad-bff8d54702bd",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[365485, 8860, 187, 3, 35, "Subsection",ExpressionUUID->"cc2443ad-2b30-44b8-b137-fcee8ea06032"],
Cell[365675, 8865, 2335, 55, 181, "Input",ExpressionUUID->"937c9353-1dc4-4b3d-9bd8-2a0c5e3daea8",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[368059, 8926, 344, 5, 33, "Section",ExpressionUUID->"0e8c40d5-15a3-4985-a778-ae61b23793c8"],
Cell[CellGroupData[{
Cell[368428, 8935, 362, 6, 44, "Subsection",ExpressionUUID->"aa71010e-7f39-4e89-b0f8-d7b9a16a6dce"],
Cell[CellGroupData[{
Cell[368815, 8945, 271, 4, 35, "Subsubsection",ExpressionUUID->"8eb321da-76c4-4629-909b-1c133b7358b6"],
Cell[369089, 8951, 293, 4, 30, "Text",ExpressionUUID->"015c49db-6bf1-4f97-91f5-529de04544e8"],
Cell[369385, 8957, 2236, 52, 196, "Input",ExpressionUUID->"f22e3173-545e-49d4-9885-1fa79e02e385",
 InitializationCell->True],
Cell[371624, 9011, 314, 5, 30, "Text",ExpressionUUID->"9a521704-4bd4-4e4e-87ae-74e7c68ea5cb"],
Cell[371941, 9018, 1901, 45, 154, "Input",ExpressionUUID->"d00d55bd-65a8-4fa2-a9c9-ce71ade8c6a3",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[373879, 9068, 379, 6, 35, "Subsubsection",ExpressionUUID->"119e5f4f-f410-4041-8e38-af170eefdb15"],
Cell[374261, 9076, 293, 4, 30, "Text",ExpressionUUID->"77de03cc-fcde-46d1-979b-cb7553f8feb2"],
Cell[374557, 9082, 2450, 59, 175, "Input",ExpressionUUID->"102d11b2-e672-4fdc-9c92-8fa590dd751a",
 InitializationCell->True],
Cell[377010, 9143, 255, 4, 30, "Text",ExpressionUUID->"90d5d7e1-7e7d-4621-bb46-4fbcc94be68e"],
Cell[377268, 9149, 3533, 82, 280, "Input",ExpressionUUID->"4a4ab5e7-3a7d-42f4-bbd6-91bce5e904d6",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[380850, 9237, 594, 9, 44, "Subsection",ExpressionUUID->"abb66f6d-7fe9-4440-9ce2-016ece8a70b3"],
Cell[CellGroupData[{
Cell[381469, 9250, 240, 4, 35, "Subsubsection",ExpressionUUID->"9a274f8d-8820-43ea-b188-4744149690a4"],
Cell[381712, 9256, 675, 12, 49, "Text",ExpressionUUID->"57977b8e-4af3-4f24-b542-d38ec03acd99"],
Cell[382390, 9270, 4388, 97, 364, "Input",ExpressionUUID->"c637d09a-434b-4d2c-b858-a2bbf6b0d6b9",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[386815, 9372, 322, 5, 35, "Subsubsection",ExpressionUUID->"add1664d-2443-4340-9999-c37d31cadd74"],
Cell[387140, 9379, 556, 10, 49, "Text",ExpressionUUID->"b682c1cf-26c7-44f8-bb67-4d72a3f23a4d"],
Cell[387699, 9391, 3772, 81, 343, "Input",ExpressionUUID->"7bf78a7a-4c78-440b-8891-ffecbaf16753",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[391508, 9477, 307, 5, 35, "Subsubsection",ExpressionUUID->"a6aabd53-ac4c-4705-8d9c-e8252026de53"],
Cell[391818, 9484, 537, 9, 30, "Text",ExpressionUUID->"9390a7b3-a4f4-4116-96b1-c9e7b2e9d1ad"],
Cell[392358, 9495, 4534, 99, 406, "Input",ExpressionUUID->"4d18b33e-208b-4a9f-8277-285e7df305fd",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[396929, 9599, 309, 6, 35, "Subsubsection",ExpressionUUID->"df700886-8f6a-4443-aabc-b1403ac6c1b3"],
Cell[397241, 9607, 620, 11, 30, "Text",ExpressionUUID->"bd11ba15-2d06-4261-a529-319bde2c4ae9"],
Cell[397864, 9620, 4519, 98, 406, "Input",ExpressionUUID->"48970f02-3bf2-4669-b0f5-ff953402b480",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[402444, 9725, 362, 6, 33, "Section",ExpressionUUID->"124754c2-b93e-4efb-8829-2bc57664e7bf"],
Cell[402809, 9733, 636, 17, 70, "Input",ExpressionUUID->"445f1794-8b2a-40e7-af7e-b043973c67ce",
 InitializationCell->True],
Cell[403448, 9752, 734, 18, 91, "Input",ExpressionUUID->"fbc53951-40c3-4eba-b7ae-2d794700d5d2",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[404231, 9776, 834, 13, 41, "Subsubtitle",ExpressionUUID->"2915a32b-55cd-41dc-9e44-ea6defbd5e9e",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[405090, 9793, 263, 4, 47, "Section",ExpressionUUID->"67c8c818-aecb-4524-a5dd-cbdaa9c6da42"],
Cell[405356, 9799, 3090, 73, 256, "Input",ExpressionUUID->"45da9025-4a38-455c-869d-77d901df6199",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[408483, 9877, 316, 5, 39, "Section",ExpressionUUID->"d3ed70e5-c3a4-4ae9-a84f-5d57ffb6b6fd"],
Cell[408802, 9884, 622, 16, 91, "Input",ExpressionUUID->"66bde24b-3869-4d77-bbec-b190cf11a509",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[409461, 9905, 426, 6, 39, "Section",ExpressionUUID->"b31e26da-3c42-4e28-962d-a9010aa44191"],
Cell[CellGroupData[{
Cell[409912, 9915, 232, 5, 49, "Subsection",ExpressionUUID->"f87e4302-90ca-4d92-9e05-4d6941a8742c",
 InitializationCell->True],
Cell[410147, 9922, 590, 13, 48, "Input",ExpressionUUID->"d731f18d-f25d-4bac-8b08-7ab24cea24f2",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[410774, 9940, 439, 7, 41, "Subsection",ExpressionUUID->"bc7f6e5d-bd3f-4737-8544-0d17ad854ea6",
 InitializationCell->True],
Cell[411216, 9949, 522, 10, 30, "Text",ExpressionUUID->"d7fc0c42-6456-4620-8f92-900b3f5ad7f6",
 InitializationCell->True],
Cell[411741, 9961, 257, 6, 30, "Text",ExpressionUUID->"e36f4276-d26b-46a0-a4d3-afb55be56f50"],
Cell[412001, 9969, 695, 20, 70, "Input",ExpressionUUID->"370b0b97-5025-4b9e-858e-80289f22f177",
 InitializationCell->True],
Cell[412699, 9991, 894, 19, 70, "Input",ExpressionUUID->"2c4f25b7-e8cb-42d7-9eb8-db529b08fa17",
 InitializationCell->True],
Cell[413596, 10012, 649, 19, 70, "Input",ExpressionUUID->"9cb9db9e-9fe3-4e8b-a0a0-9e8c3b760e8e",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[414282, 10036, 357, 6, 41, "Subsection",ExpressionUUID->"231dce1d-75fa-4b8f-93ae-e60ccaa2f4d4",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[414664, 10046, 382, 8, 59, "Subsubsection",ExpressionUUID->"1dc6aa31-5eb4-4fe1-8264-609258bd03a6"],
Cell[415049, 10056, 1154, 30, 133, "Input",ExpressionUUID->"10510408-b270-4084-9185-6d72b98d2b76",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[416240, 10091, 253, 4, 29, "Subsubsection",ExpressionUUID->"fa874909-b4b0-4a9b-b7a1-e59dca37057c"],
Cell[416496, 10097, 965, 24, 70, "Input",ExpressionUUID->"9a9c5c55-1840-4e6f-aea4-e7654386d1ec",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[417510, 10127, 409, 7, 41, "Subsection",ExpressionUUID->"ec681d0c-ddb1-4b49-a2dd-479876b4be0d",
 InitializationCell->True],
Cell[417922, 10136, 200, 3, 30, "Text",ExpressionUUID->"a810ce88-05fd-4b77-ba06-716e546646f2"],
Cell[418125, 10141, 342, 9, 48, "Input",ExpressionUUID->"fab23a93-df6b-4bbf-8c8f-63792f7f168a",
 InitializationCell->True],
Cell[418470, 10152, 577, 12, 48, "Input",ExpressionUUID->"11cc094f-c783-4be2-bf0b-b0da411fd34a",
 InitializationCell->True],
Cell[419050, 10166, 555, 11, 48, "Input",ExpressionUUID->"e79e2d9c-16a9-446a-b681-4cd03c5a48c2",
 InitializationCell->True],
Cell[419608, 10179, 925, 20, 71, "Input",ExpressionUUID->"6b016fe9-42da-476b-a8ac-3a17393e607d",
 InitializationCell->True],
Cell[420536, 10201, 631, 13, 70, "Input",ExpressionUUID->"85b3df37-e9c6-499a-88ad-497792cb8bfe",
 InitializationCell->True],
Cell[421170, 10216, 636, 14, 48, "Input",ExpressionUUID->"7d5f4dcd-9c41-4497-b3ca-73cfa0446cf2",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[421843, 10235, 367, 6, 41, "Subsection",ExpressionUUID->"e4c6e66c-16c1-4c1c-ae54-7aeed9d059af",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[422235, 10245, 880, 14, 35, "Subsubsection",ExpressionUUID->"84f4e0ce-7d34-4b61-8666-8cfa350e77e6",
 InitializationCell->True],
Cell[423118, 10261, 388, 9, 49, "Text",ExpressionUUID->"632f5eb8-e993-45c3-b170-c604489195f6",
 InitializationCell->True],
Cell[423509, 10272, 1086, 28, 133, "Input",ExpressionUUID->"9bfc4320-e279-4961-8e1f-b92c9a6c13c6",
 InitializationCell->True],
Cell[424598, 10302, 870, 22, 91, "Input",ExpressionUUID->"14608f8c-d4fe-4b56-8bb3-e97b5d7ddbca",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[425505, 10329, 849, 15, 1787, "Subsubsection",ExpressionUUID->"a03c5aad-dac1-46e8-9c5d-01680975bf31",
 InitializationCell->True],
Cell[426357, 10346, 329, 7, 49, "Text",ExpressionUUID->"73210cf6-3012-40aa-b496-4f2570e8ab7c",
 InitializationCell->True],
Cell[426689, 10355, 765, 16, 70, "Input",ExpressionUUID->"23aade11-777c-491f-9d0e-0c6e728a0a34",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[427491, 10376, 736, 11, 35, "Subsubsection",ExpressionUUID->"51097dcb-b24e-4017-b575-61dc2c45e610",
 InitializationCell->True],
Cell[428230, 10389, 594, 15, 70, "Input",ExpressionUUID->"bf15a96c-98de-4a08-8408-06b042c47411",
 InitializationCell->True],
Cell[428827, 10406, 546, 14, 70, "Input",ExpressionUUID->"0a5ff42e-79b9-479d-a39d-8f3ffc25d619",
 InitializationCell->True],
Cell[429376, 10422, 481, 12, 48, "Input",ExpressionUUID->"3553cfac-df8a-411a-bf7f-ef26158498ad",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[429894, 10439, 707, 11, 29, "Subsubsection",ExpressionUUID->"38f6f186-6ccf-412a-b1ae-b39910d1768a",
 InitializationCell->True],
Cell[430604, 10452, 470, 9, 49, "Text",ExpressionUUID->"700e35c3-05aa-4bb5-a57c-51f0734e510a",
 InitializationCell->True],
Cell[431077, 10463, 1345, 28, 91, "Input",ExpressionUUID->"b8baccde-72fc-413b-83e3-b2caeb2aee82",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[432459, 10496, 741, 11, 29, "Subsubsection",ExpressionUUID->"e35f2831-3d97-4afb-b0e4-df5592781a6b",
 InitializationCell->True],
Cell[433203, 10509, 1133, 22, 91, "Input",ExpressionUUID->"e413e7e8-5646-40cd-9281-323b67956357",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[434385, 10537, 275, 5, 41, "Subsection",ExpressionUUID->"84dc41c1-2605-448a-ab7c-0d1eb1354264",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[434685, 10546, 206, 3, 39, "Subsubsection",ExpressionUUID->"d8eea93c-e56f-443e-b2ee-76bd2ab08d98"],
Cell[434894, 10551, 1494, 30, 196, "Input",ExpressionUUID->"9eaa52f8-a5b6-4ed4-9f12-e4ebd54a4953",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[436425, 10586, 280, 6, 33, "Subsubsection",ExpressionUUID->"bda74eb2-b05a-4899-b959-6bfb7832055b"],
Cell[436708, 10594, 1767, 37, 217, "Input",ExpressionUUID->"ab0f9bfd-2ffc-4657-98aa-085cd4699116",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[438512, 10636, 249, 4, 33, "Subsubsection",ExpressionUUID->"14e1f5c9-5d68-4930-bd6d-3ca1b570e8cd"],
Cell[438764, 10642, 2271, 50, 301, "Input",ExpressionUUID->"edeeb30e-29b5-4113-b9e4-9905b25ede96",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[441084, 10698, 305, 6, 41, "Subsection",ExpressionUUID->"3078b2c1-fd96-41e5-b797-26ed84381114",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[441414, 10708, 206, 3, 35, "Subsubsection",ExpressionUUID->"c606b5f5-c794-46d4-9733-86cfed498111"],
Cell[441623, 10713, 1557, 30, 196, "Input",ExpressionUUID->"1496e607-fd33-4142-adf5-7cf401e80641",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[443217, 10748, 280, 6, 29, "Subsubsection",ExpressionUUID->"4eb1db02-a779-4196-bffd-c32c6020b57c"],
Cell[443500, 10756, 1911, 40, 217, "Input",ExpressionUUID->"c7053374-b1ba-4c60-a790-92d9b65aa2cd",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[445448, 10801, 346, 7, 35, "Subsubsection",ExpressionUUID->"b86a82f3-db4b-4d91-9027-e4366a7da039"],
Cell[445797, 10810, 2537, 56, 301, "Input",ExpressionUUID->"e3554c72-a10d-4fc7-aedc-6500a06704d8",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[448383, 10872, 617, 9, 41, "Subsection",ExpressionUUID->"94b1f768-2518-45fc-828a-83a1dda3e250",
 InitializationCell->True],
Cell[449003, 10883, 1506, 34, 133, "Input",ExpressionUUID->"b328c46b-c64c-4af8-adb3-e5e0e01af664",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[450546, 10922, 318, 6, 41, "Subsection",ExpressionUUID->"f03eca9d-e54d-4f30-bc66-efe7bee5e8ef",
 InitializationCell->True],
Cell[450867, 10930, 894, 22, 70, "Input",ExpressionUUID->"3acce446-f6db-49df-92d4-32bf3a7b82ec",
 InitializationCell->True],
Cell[451764, 10954, 722, 16, 70, "Input",ExpressionUUID->"009b8fc1-30d3-4205-8dbc-8360cdfa3a8c",
 InitializationCell->True],
Cell[452489, 10972, 835, 21, 70, "Input",ExpressionUUID->"967e48e7-c6dd-4b0d-9ea4-2e9e5e6d9d35",
 InitializationCell->True],
Cell[453327, 10995, 788, 17, 70, "Input",ExpressionUUID->"ed861e02-56e2-4a44-a487-33c2850356b9",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[454152, 11017, 407, 7, 41, "Subsection",ExpressionUUID->"bbd3ea00-1a0e-4c98-af2e-2ad7a613a556",
 InitializationCell->True],
Cell[454562, 11026, 431, 10, 48, "Input",ExpressionUUID->"a9ec6884-2b06-4700-baae-c9a81d6e95e0",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[455054, 11043, 860, 13, 41, "Subsubtitle",ExpressionUUID->"87ced246-0a16-4cf0-97a8-1c43e84b9aec",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[455939, 11060, 289, 5, 34, "Section",ExpressionUUID->"5bfede60-911e-4a74-9f93-aac38dec9aa3"],
Cell[CellGroupData[{
Cell[456253, 11069, 230, 4, 35, "Subsubsection",ExpressionUUID->"92f492a0-109b-45a5-bf09-4e3791700615"],
Cell[456486, 11075, 1660, 37, 175, "Input",ExpressionUUID->"24e8b83e-59dc-45c0-bf85-bf183810509d",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[458195, 11118, 338, 5, 34, "Section",ExpressionUUID->"fb78da1a-b206-4686-9a5f-d4908834a894"],
Cell[458536, 11125, 691, 17, 91, "Input",ExpressionUUID->"93dff44f-f6a4-40b8-affc-98c9b596778d",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[459264, 11147, 478, 8, 34, "Section",ExpressionUUID->"7df60467-9831-4976-8c4a-c1c7414e3857",
 InitializationCell->True],
Cell[459745, 11157, 433, 9, 49, "Text",ExpressionUUID->"f605ff41-9b0a-4073-9bea-38f78ebf7b17",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[460203, 11170, 203, 4, 35, "Subsubsection",ExpressionUUID->"744a5e4e-146c-4af0-aad6-6bd6c77cd239"],
Cell[460409, 11176, 303, 7, 70, "Input",ExpressionUUID->"9deac513-42d5-45c4-8b9f-ddffe9c59104",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[460749, 11188, 179, 3, 29, "Subsubsection",ExpressionUUID->"2399840b-0340-4445-b12a-fbc8e8625b9b"],
Cell[460931, 11193, 1532, 32, 91, "Input",ExpressionUUID->"87e88881-64d0-4904-bff5-40e7bcfd98ff",
 InitializationCell->True],
Cell[462466, 11227, 1882, 39, 112, "Input",ExpressionUUID->"bd81c6db-e6e4-49b4-b5cd-f10d96db3c02",
 InitializationCell->True],
Cell[464351, 11268, 1532, 32, 91, "Input",ExpressionUUID->"93890ef2-3cb4-42f1-bb8d-c1339a664e00",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[465920, 11305, 178, 3, 29, "Subsubsection",ExpressionUUID->"fa72e5e3-df03-4dc3-8dfb-d09c59877056"],
Cell[466101, 11310, 1302, 23, 70, "Input",ExpressionUUID->"ec793cd5-6c4e-4d9c-a36d-f97d3621ae85",
 InitializationCell->True],
Cell[467406, 11335, 1414, 26, 70, "Input",ExpressionUUID->"07c03dc9-269e-4170-b9cc-8365bc038933",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[468857, 11366, 167, 3, 29, "Subsubsection",ExpressionUUID->"08762a96-338a-4b97-8b08-c3b5af552c6e"],
Cell[469027, 11371, 1704, 28, 48, "Input",ExpressionUUID->"3417830e-e20c-4bc5-84bf-dff546a63951",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[470768, 11404, 177, 3, 35, "Subsubsection",ExpressionUUID->"f925b26c-a9d2-478d-8357-5011198392ca"],
Cell[470948, 11409, 1355, 23, 70, "Input",ExpressionUUID->"340c85f5-81ae-4941-80bd-a1cfb5c10053",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[472340, 11437, 207, 3, 29, "Subsubsection",ExpressionUUID->"7be7ae03-295a-417b-837a-c3b3d22ea948"],
Cell[472550, 11442, 761, 19, 70, "Input",ExpressionUUID->"5d27a67b-51b3-4750-a0e7-b3d2efbbb6f8",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[473372, 11468, 932, 14, 41, "Subsubtitle",ExpressionUUID->"370fbc63-d71c-4f47-9d49-7923215d1390",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[474329, 11486, 180, 3, 34, "Section",ExpressionUUID->"1207523b-5192-45ad-9eb8-cdf42b91e037"],
Cell[474512, 11491, 292, 4, 30, "Text",ExpressionUUID->"d75677bc-c7ff-4dc2-af02-9f8eea48d48e"],
Cell[474807, 11497, 9667, 195, 910, "Input",ExpressionUUID->"00c2c14a-48ca-4262-8a80-134b3a12d177",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[484511, 11697, 181, 3, 34, "Section",ExpressionUUID->"e6fc4603-d9fd-4eeb-8ea5-3d460bdc68ab"],
Cell[484695, 11702, 323, 6, 30, "Text",ExpressionUUID->"fed9e336-9baa-40c6-9085-5f62df62f75e"],
Cell[CellGroupData[{
Cell[485043, 11712, 213, 4, 35, "Subsubsection",ExpressionUUID->"0c75a42b-bee9-4559-b126-0cea0abea6ce"],
Cell[485259, 11718, 2697, 64, 364, "Input",ExpressionUUID->"39b8110a-d28c-4c64-8c7d-97f7c98c39a4",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[487993, 11787, 287, 5, 29, "Subsubsection",ExpressionUUID->"63724d28-e734-472a-b41a-53220d709156"],
Cell[488283, 11794, 2694, 64, 364, "Input",ExpressionUUID->"a8df3c22-714c-446c-8739-0cbaa146e851",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[491014, 11863, 533, 10, 53, "Subsubsection",ExpressionUUID->"1926c7e6-73b5-454e-b994-d63a3bd113e8"],
Cell[491550, 11875, 4163, 92, 511, "Input",ExpressionUUID->"53c8b6e5-6c98-4907-ab99-06a9419f1408",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[495750, 11972, 492, 9, 29, "Subsubsection",ExpressionUUID->"130070a5-ccbf-49a1-a25c-a1bcb22b04e9"],
Cell[496245, 11983, 232, 6, 30, "Text",ExpressionUUID->"8baa1a55-441f-46d4-9ff7-f6b21a0ec6bc"],
Cell[496480, 11991, 17975, 350, 1960, "Input",ExpressionUUID->"aeeaa39b-80d2-4797-9d5e-2cd8dfe3adec",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[514492, 12346, 489, 9, 29, "Subsubsection",ExpressionUUID->"e28600a3-1eaa-4aaa-8794-445c01c069d0"],
Cell[514984, 12357, 232, 6, 30, "Text",ExpressionUUID->"f9787b5f-5225-495b-9a02-9269d7a1e709"],
Cell[515219, 12365, 6678, 132, 616, "Input",ExpressionUUID->"3c9339cc-58d4-4fdb-a0a1-0696c8e774d8",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[521946, 12503, 226, 4, 34, "Section",ExpressionUUID->"ca350638-f096-4041-83b7-2d7d2a37e377"],
Cell[CellGroupData[{
Cell[522197, 12511, 487, 9, 35, "Subsubsection",ExpressionUUID->"e3f46d59-e2f1-4ae8-a569-b736470674c5"],
Cell[522687, 12522, 6772, 141, 658, "Input",ExpressionUUID->"beaa37a6-c02e-44bc-b7eb-d872c6877f6f",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[529520, 12670, 972, 15, 41, "Subsubtitle",ExpressionUUID->"c36bc7ad-fd89-43fc-8818-27fb930b5c1d",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[530517, 12689, 313, 5, 33, "Section",ExpressionUUID->"5bd0506e-1315-4ec3-81d3-19da9f654422"],
Cell[CellGroupData[{
Cell[530855, 12698, 474, 7, 35, "Subsubsection",ExpressionUUID->"cbfbf55f-943d-481f-adfe-dc4f65b08b0c"],
Cell[531332, 12707, 1810, 39, 91, "Input",ExpressionUUID->"520bee49-c13b-456b-b2df-89819f0eebac",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[533179, 12751, 422, 6, 35, "Subsubsection",ExpressionUUID->"aca168a8-9b6b-40e9-8574-578106bcafbd"],
Cell[533604, 12759, 1769, 40, 91, "Input",ExpressionUUID->"ec564a1d-dd20-4a06-82ad-155c25b23825",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[535422, 12805, 360, 6, 33, "Section",ExpressionUUID->"876dbd1d-0d17-464a-903f-e9366029d6e4"],
Cell[CellGroupData[{
Cell[535807, 12815, 301, 5, 34, "Subsubsection",ExpressionUUID->"3e2a0fc0-782c-4182-af34-3aff4133e9a5"],
Cell[536111, 12822, 2467, 55, 275, "Input",ExpressionUUID->"0be90947-de82-481c-97f6-5da7c12804c0",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[538615, 12882, 297, 5, 28, "Subsubsection",ExpressionUUID->"27449e83-298d-45dd-a531-cf782926b10a"],
Cell[538915, 12889, 2519, 56, 322, "Input",ExpressionUUID->"3f33bd6a-69e0-43fd-af7b-df0245667971",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[541471, 12950, 440, 6, 28, "Subsubsection",ExpressionUUID->"536b0544-49e7-439d-b9ac-ae9679a0917d"],
Cell[541914, 12958, 2138, 47, 322, "Input",ExpressionUUID->"7564f890-0dfb-4787-8a3a-b78a81cb9d1e",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[544101, 13011, 372, 6, 33, "Section",ExpressionUUID->"00bc18a0-e4f3-41e0-baab-d91f883560f0"],
Cell[CellGroupData[{
Cell[544498, 13021, 165, 3, 35, "Subsubsection",ExpressionUUID->"4ca8d489-acc9-427e-ae8c-8246c4679753"],
Cell[544666, 13026, 1548, 32, 91, "Input",ExpressionUUID->"0a8b3dd5-58bf-415a-8ef4-33baa79259c7",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[546275, 13065, 1018, 15, 41, "Subsubtitle",ExpressionUUID->"291ddf99-211c-40fd-a9d0-cf4db0a12080",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[547318, 13084, 445, 7, 39, "Subsubsection",ExpressionUUID->"966addab-8666-490f-b5c8-795bb93a02c3",
 InitializationCell->True],
Cell[547766, 13093, 181, 3, 30, "Text",ExpressionUUID->"4832414d-83e4-48bc-857b-1acf509447c1"],
Cell[547950, 13098, 835, 23, 90, "Input",ExpressionUUID->"eaf5cb67-760f-493d-85fd-de7cd9bc68a4",
 InitializationCell->True]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[548834, 13127, 623, 10, 41, "Subsubtitle",ExpressionUUID->"d6f8eb87-9c1c-4c6b-924e-f1893eefb39f",
 InitializationCell->True],
Cell[549460, 13139, 405, 10, 48, "Input",ExpressionUUID->"5fe693a2-53c1-478d-88cb-0fee19ac3c9a",
 InitializationCell->True],
Cell[549868, 13151, 430, 10, 48, "Input",ExpressionUUID->"18b6deb8-7b45-4d37-8453-a19806b00258",
 InitializationCell->True],
Cell[550301, 13163, 340, 9, 48, "Input",ExpressionUUID->"38d4f7a0-c96d-4741-bea4-000260f41ad7",
 InitializationCell->True],
Cell[550644, 13174, 478, 11, 48, "Input",ExpressionUUID->"2d0ea6fd-60bf-4969-9158-7a939f0c6504",
 InitializationCell->True],
Cell[551125, 13187, 538, 11, 91, "Input",ExpressionUUID->"0bd9e42a-8b87-44ba-b712-4fad80b49220",
 InitializationCell->True],
Cell[551666, 13200, 522, 11, 91, "Input",ExpressionUUID->"8cf70e51-d948-41a9-b589-397a53db1021",
 InitializationCell->True],
Cell[552191, 13213, 943, 17, 133, "Input",ExpressionUUID->"d10bc928-ef1c-4c0d-be74-f380693faa0c",
 InitializationCell->True],
Cell[553137, 13232, 331, 9, 48, "Input",ExpressionUUID->"84869682-9295-4a8e-a43e-d3d73d3df710",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[553517, 13247, 936, 14, 69, "Subtitle",ExpressionUUID->"0cc2e383-fb5e-4e41-9703-00c5085a1420",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[554478, 13265, 649, 10, 37, "Subsubtitle",ExpressionUUID->"7e30ba0c-4c16-4901-a2e2-fd3e8d13d0b6",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[555152, 13279, 482, 8, 34, "Section",ExpressionUUID->"31a4a36f-e372-424f-a3e2-08e5e7141ef0",
 InitializationCell->True],
Cell[555637, 13289, 1267, 26, 196, "Input",ExpressionUUID->"aee02f02-5b53-4644-ab7a-b654cc534e9d",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[556941, 13320, 697, 11, 34, "Section",ExpressionUUID->"311fc07b-fe46-4fb3-ade5-f7b336bc7ae6",
 InitializationCell->True],
Cell[557641, 13333, 442, 13, 48, "Input",ExpressionUUID->"094e1e02-0289-4da5-a60f-055dc4830da3",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[558132, 13352, 639, 10, 37, "Subsubtitle",ExpressionUUID->"8ce5ff9a-4cf7-4f33-a7b4-7fe112366fde",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[558796, 13366, 352, 5, 34, "Section",ExpressionUUID->"f8802b46-cbc4-4f32-a262-d41cb4d07695"],
Cell[CellGroupData[{
Cell[559173, 13375, 174, 3, 44, "Subsection",ExpressionUUID->"de508d1d-4a05-44a7-9657-9a6bc51e2ba3"],
Cell[559350, 13380, 250, 6, 49, "Text",ExpressionUUID->"3b2003de-aeb1-45fd-9b62-311f80dd2add"]
}, Closed]],
Cell[CellGroupData[{
Cell[559637, 13391, 277, 4, 36, "Subsection",ExpressionUUID->"57efa8cc-0834-4e1a-8d22-6df17fc6f3ee"],
Cell[CellGroupData[{
Cell[559939, 13399, 248, 4, 35, "Subsubsection",ExpressionUUID->"dd9bc39b-e1a0-4705-8da6-d0399a093825"],
Cell[560190, 13405, 405, 9, 85, "Text",ExpressionUUID->"3b2eed51-dc02-4829-9136-5e3bc32ef617"]
}, Closed]],
Cell[CellGroupData[{
Cell[560632, 13419, 252, 4, 29, "Subsubsection",ExpressionUUID->"0ccdc951-5a40-483a-bd4d-235273f57d05"],
Cell[560887, 13425, 321, 7, 85, "Text",ExpressionUUID->"e73fd3b9-10f8-478e-84f4-bb98d35fedb8"]
}, Closed]],
Cell[CellGroupData[{
Cell[561245, 13437, 184, 3, 29, "Subsubsection",ExpressionUUID->"f6775406-bc14-486a-8142-a1935eaf4049"],
Cell[561432, 13442, 199, 3, 37, "Text",ExpressionUUID->"809b57ba-bb33-4505-9a7a-1908fecbbaca"]
}, Closed]],
Cell[CellGroupData[{
Cell[561668, 13450, 283, 4, 29, "Subsubsection",ExpressionUUID->"553a7f1d-39e7-4de9-a937-19f932856dd7"],
Cell[561954, 13456, 253, 7, 85, "Text",ExpressionUUID->"4f946dd7-cb9e-4b14-a38c-f86da2ad8f1b"]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[562268, 13470, 230, 4, 34, "Section",ExpressionUUID->"ea9d7201-78ec-42f4-871c-33b44fa66574"],
Cell[CellGroupData[{
Cell[562523, 13478, 262, 5, 35, "Subsubsection",ExpressionUUID->"9580abd3-0eec-452b-9f5f-9d1bd3482569",
 InitializationCell->True],
Cell[562788, 13485, 641, 15, 48, "Input",ExpressionUUID->"c9aaed10-69e4-4648-b723-71feabc1023f",
 InitializationCell->True],
Cell[563432, 13502, 754, 20, 70, "Input",ExpressionUUID->"a63ffe0f-d79d-42fe-bc1e-ab5e7d6f2569",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[564235, 13528, 239, 4, 34, "Section",ExpressionUUID->"9afe5f75-837f-4879-b3c2-3aee82351253"],
Cell[CellGroupData[{
Cell[564499, 13536, 309, 5, 35, "Subsubsection",ExpressionUUID->"7ba85771-e44f-4383-b6e3-575f3e46511f",
 InitializationCell->True],
Cell[564811, 13543, 332, 9, 48, "Input",ExpressionUUID->"907c6d9f-4e02-4746-8873-f324aff0a642",
 InitializationCell->True],
Cell[565146, 13554, 574, 16, 70, "Input",ExpressionUUID->"8c568151-9af8-4bd6-9081-f3d42b34cff4",
 InitializationCell->True],
Cell[565723, 13572, 1024, 25, 91, "Input",ExpressionUUID->"0061c691-8dff-4e0c-bd77-aa00ccecd465",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[566784, 13602, 250, 5, 29, "Subsubsection",ExpressionUUID->"ba140227-df19-4252-a8cd-34b944fffb9f",
 InitializationCell->True],
Cell[567037, 13609, 1259, 28, 154, "Input",ExpressionUUID->"29376cdc-fa2a-4161-b6ba-b84f624d75b5",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[568333, 13642, 337, 6, 29, "Subsubsection",ExpressionUUID->"50f59122-d174-4244-8547-4c22ea64e648",
 InitializationCell->True],
Cell[568673, 13650, 1064, 24, 154, "Input",ExpressionUUID->"50914bbe-4011-4d73-9e73-15b2f2549708",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[569774, 13679, 408, 7, 29, "Subsubsection",ExpressionUUID->"f14be8ed-402d-4f03-b9ae-20247a48f313",
 InitializationCell->True],
Cell[570185, 13688, 1252, 29, 154, "Input",ExpressionUUID->"67c07e1b-149f-40ee-b5c4-0bc33b43a9a7",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[571486, 13723, 161, 3, 34, "Section",ExpressionUUID->"37325bf9-417c-47b9-86b3-8ed564696f9e"],
Cell[CellGroupData[{
Cell[571672, 13730, 637, 12, 83, "Subsubsection",ExpressionUUID->"0f239192-8601-4416-b44c-b94c7270a669",
 InitializationCell->True],
Cell[572312, 13744, 400, 8, 49, "Text",ExpressionUUID->"3c6dbb7c-1cbf-41e4-a75d-67364374d78a"],
Cell[572715, 13754, 2114, 52, 196, "Input",ExpressionUUID->"9889d07c-6457-4d9e-a3eb-86579fbcfbc4",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[574878, 13812, 252, 4, 34, "Section",ExpressionUUID->"4c184bd5-8098-4e10-b4ee-e12e4ca6207d"],
Cell[CellGroupData[{
Cell[575155, 13820, 156, 3, 35, "Subsubsection",ExpressionUUID->"b7efa55f-e5d0-451d-a07e-1870341fbfed"],
Cell[575314, 13825, 495, 9, 30, "Text",ExpressionUUID->"c49512d1-df7e-4743-ae21-e79013799bef",
 InitializationCell->True],
Cell[575812, 13836, 1179, 27, 91, "Input",ExpressionUUID->"bc9ef455-f7c2-46da-b372-24f952a5e4fa",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[577028, 13868, 157, 3, 35, "Subsubsection",ExpressionUUID->"2bbd4557-424d-4ba1-ae97-e850bd99d175"],
Cell[577188, 13873, 704, 19, 70, "Input",ExpressionUUID->"6a6dc4c4-417c-4a32-8e50-7cbb52524dc9",
 InitializationCell->True],
Cell[577895, 13894, 653, 19, 70, "Input",ExpressionUUID->"dbcce04f-876c-4e98-ab09-f8ed66187a09",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[578609, 13920, 781, 12, 37, "Subsubtitle",ExpressionUUID->"cbbb79a8-3c4d-440e-8561-cc0c82a43cee",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[579415, 13936, 712, 12, 34, "Section",ExpressionUUID->"8fa8b7a9-d64f-4677-9262-7218bdb1f1ef",
 InitializationCell->True],
Cell[580130, 13950, 344, 7, 49, "Text",ExpressionUUID->"58879946-947a-4f4c-97e7-e8f6dd01ed65"],
Cell[580477, 13959, 3029, 66, 343, "Input",ExpressionUUID->"61b8a805-1e52-4277-af9d-53743b074a09",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[583543, 14030, 1012, 15, 34, "Section",ExpressionUUID->"9434eb18-6306-4a03-8bc3-552f6021c694",
 InitializationCell->True],
Cell[584558, 14047, 211, 5, 30, "Text",ExpressionUUID->"47ce46ff-1548-4a34-90cc-376cd655cf45"],
Cell[584772, 14054, 360, 7, 30, "Text",ExpressionUUID->"14da2ea3-0f50-4620-b460-67e48c1a3262"],
Cell[585135, 14063, 302, 7, 30, "Text",ExpressionUUID->"92484261-964d-4fa1-8d55-956b4b962dca"],
Cell[585440, 14072, 2684, 57, 385, "Input",ExpressionUUID->"1b0e8082-25c4-4cb1-84b6-92dee5462f79",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[588161, 14134, 686, 11, 34, "Section",ExpressionUUID->"c1e4a6f3-76d3-4011-897f-10e735b2f4cf",
 InitializationCell->True],
Cell[588850, 14147, 214, 3, 30, "Text",ExpressionUUID->"3016d654-7efa-4531-84c8-9a5b2bc37b0c"],
Cell[CellGroupData[{
Cell[589089, 14154, 373, 8, 59, "Subsubsection",ExpressionUUID->"80dbd4d1-c940-4b2e-b7a2-1bd6c0138b0d"],
Cell[589465, 14164, 734, 13, 49, "Text",ExpressionUUID->"d221f472-32a9-4803-861a-1f16b441b8ca"],
Cell[590202, 14179, 4377, 99, 511, "Input",ExpressionUUID->"0933c3ab-f4c0-434a-a3a7-4f6b351171dc",
 InitializationCell->True],
Cell[594582, 14280, 181, 3, 30, "Text",ExpressionUUID->"345d21f9-72c0-4b2a-82e3-8c573e52c5b7"],
Cell[594766, 14285, 2033, 42, 175, "Input",ExpressionUUID->"1fc8cc6a-cb1e-4fc4-94ee-e35408d28ada",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[596836, 14332, 417, 8, 59, "Subsubsection",ExpressionUUID->"85e13c3d-c35c-4e2e-9e7e-aebfb108a313"],
Cell[597256, 14342, 4243, 97, 511, "Input",ExpressionUUID->"efbddada-95cf-4cd6-b93c-10283d1eea5a",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[601548, 14445, 1097, 16, 34, "Section",ExpressionUUID->"d7b31436-2529-4afe-abbc-73e2ae08a046",
 InitializationCell->True],
Cell[602648, 14463, 500, 14, 70, "Input",ExpressionUUID->"c9bb266f-d141-4299-bfb7-a706c37c49bf",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[603197, 14483, 571, 9, 37, "Subsubtitle",ExpressionUUID->"9506b8f2-d011-41cb-879f-0648bd8689ad",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[603793, 14496, 153, 3, 34, "Section",ExpressionUUID->"d9acc479-969d-4102-99ae-71d3cca0dc7d"],
Cell[CellGroupData[{
Cell[603971, 14503, 159, 3, 35, "Subsubsection",ExpressionUUID->"555a7783-9304-4e53-8bd9-647236a33075"],
Cell[604133, 14508, 1398, 35, 112, "Input",ExpressionUUID->"5c68a882-48a8-43a2-97db-5f5043e5f370",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[605568, 14548, 160, 3, 35, "Subsubsection",ExpressionUUID->"4ae39db0-7e99-43fc-8c8b-edaf59d01898"],
Cell[605731, 14553, 652, 17, 70, "Input",ExpressionUUID->"c5ebbe6c-6bbb-466f-b1b0-57f0372b4877",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[606432, 14576, 256, 4, 34, "Section",ExpressionUUID->"22ab5e41-8e65-497c-8722-42baa799d6d0"],
Cell[CellGroupData[{
Cell[606713, 14584, 157, 3, 35, "Subsubsection",ExpressionUUID->"cf5daa2a-3f7e-40bf-b9db-f36bab98a6a9"],
Cell[606873, 14589, 218, 5, 30, "Text",ExpressionUUID->"15a72b31-1757-4922-8167-4d75c001900a"],
Cell[607094, 14596, 5625, 129, 469, "Input",ExpressionUUID->"79986373-2677-4506-8983-1fb471427dfe",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[612756, 14730, 158, 3, 35, "Subsubsection",ExpressionUUID->"535cd45c-7da4-4c44-b1e0-86412b59d275"],
Cell[612917, 14735, 720, 18, 70, "Input",ExpressionUUID->"523cc1dc-9f3c-4fda-af66-24cb20ff2e45",
 InitializationCell->True],
Cell[613640, 14755, 780, 21, 91, "Input",ExpressionUUID->"7904ee9b-6dcb-4288-a634-d9d6ceda4b3d",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[614469, 14782, 303, 5, 34, "Section",ExpressionUUID->"52484f4b-2586-4e7d-af35-ec00bf925438"],
Cell[CellGroupData[{
Cell[614797, 14791, 157, 3, 35, "Subsubsection",ExpressionUUID->"0d8d6514-a5ba-4af5-8015-2dc9ec0f92fa"],
Cell[614957, 14796, 324, 7, 30, "Text",ExpressionUUID->"8827f450-56cf-4055-8d76-9b812cd1914d"],
Cell[615284, 14805, 5163, 122, 448, "Input",ExpressionUUID->"b01abfc3-af02-43f9-bcab-a6bfcc2908c3",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[620484, 14932, 158, 3, 35, "Subsubsection",ExpressionUUID->"709fddac-9c08-4f6b-8b91-71305e288a87"],
Cell[620645, 14937, 718, 18, 70, "Input",ExpressionUUID->"a7ad67e0-3e2b-4673-958d-d8b1e1223951",
 InitializationCell->True],
Cell[621366, 14957, 726, 20, 70, "Input",ExpressionUUID->"1c8ca20c-83ae-45bf-8b28-b4aa222b1ea3",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[622141, 14983, 363, 6, 34, "Section",ExpressionUUID->"93a08745-3a42-4f82-9abd-8e78af36f664"],
Cell[622507, 14991, 2027, 55, 238, "Input",ExpressionUUID->"262f99ea-a4e9-4646-bb58-6c6078cecc69",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[624571, 15051, 383, 6, 34, "Section",ExpressionUUID->"2f9ae897-6433-4e52-b270-be4027042cd0"],
Cell[CellGroupData[{
Cell[624979, 15061, 157, 3, 35, "Subsubsection",ExpressionUUID->"5f239cca-c413-4b58-88b5-e802931ffa10"],
Cell[625139, 15066, 365, 8, 30, "Text",ExpressionUUID->"2d25d461-c39d-4a72-b1a9-2ebe26c82384"],
Cell[625507, 15076, 2431, 55, 175, "Input",ExpressionUUID->"b3290a6e-d380-4cb5-a790-a2b39eda8ab9",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[627975, 15136, 158, 3, 35, "Subsubsection",ExpressionUUID->"3b925f2a-5aa4-4722-8111-42de70a1640e"],
Cell[628136, 15141, 807, 21, 91, "Input",ExpressionUUID->"5942f005-c53e-4ed6-94d5-133cac6aa8f1",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[628992, 15168, 407, 6, 34, "Section",ExpressionUUID->"8e8d754c-d5f5-40a0-a6ed-35440fc437e6"],
Cell[629402, 15176, 274, 6, 30, "Text",ExpressionUUID->"f30a31b6-e4bc-41d0-94c2-cba8edc26001"],
Cell[CellGroupData[{
Cell[629701, 15186, 311, 6, 35, "Subsubsection",ExpressionUUID->"0090270d-8ca4-442d-80c4-ef2bd2dc4f46",
 InitializationCell->True],
Cell[630015, 15194, 2575, 55, 154, "Input",ExpressionUUID->"b202ee03-4a74-451d-b692-3fe45c3df0e2",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[632627, 15254, 158, 3, 35, "Subsubsection",ExpressionUUID->"41917f39-26c4-4d57-b3a1-d1b443e458ad"],
Cell[632788, 15259, 840, 21, 91, "Input",ExpressionUUID->"e8d72d2f-6bba-4e6a-ac7a-daea7b0d9fd2",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[633677, 15286, 518, 8, 34, "Section",ExpressionUUID->"9cd6f064-3444-414d-aa97-044a16c045eb"],
Cell[CellGroupData[{
Cell[634220, 15298, 669, 11, 59, "Subsubsection",ExpressionUUID->"3086fcf8-446b-43ab-8943-db3111050656"],
Cell[634892, 15311, 2478, 60, 343, "Input",ExpressionUUID->"741c5058-6f04-4572-b154-dac8989d01e2",
 InitializationCell->True],
Cell[637373, 15373, 2489, 61, 343, "Input",ExpressionUUID->"4e4067c1-f9f2-423b-b9f9-f922e3926c66",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[639923, 15441, 754, 12, 37, "Subsubtitle",ExpressionUUID->"41479b7b-5028-479e-941f-be2b54ee4eec",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[640702, 15457, 228, 4, 33, "Section",ExpressionUUID->"b200f867-a15d-4de7-857d-9eb138ac5b26"],
Cell[CellGroupData[{
Cell[640955, 15465, 510, 9, 35, "Subsubsection",ExpressionUUID->"28e8245d-20ca-4e6e-9591-78aa962f115a"],
Cell[641468, 15476, 3119, 69, 259, "Input",ExpressionUUID->"62e9de03-03ec-43f5-850b-0da54b3a785e",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[644624, 15550, 487, 7, 35, "Subsubsection",ExpressionUUID->"4c480338-e8bb-455c-92dc-d19fdf1570a2"],
Cell[645114, 15559, 244, 5, 30, "Text",ExpressionUUID->"0fac3ef4-22fa-4cb2-a3a3-1983231a413a"],
Cell[645361, 15566, 2980, 63, 259, "Input",ExpressionUUID->"3446677d-4d34-440e-a47d-4c03ed4ac6a5",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[648390, 15635, 299, 5, 33, "Section",ExpressionUUID->"98b08f3b-736f-403c-a6bf-fb2fd9ba8f01"],
Cell[CellGroupData[{
Cell[648714, 15644, 533, 9, 35, "Subsubsection",ExpressionUUID->"0df3cbf3-2229-4b35-96c9-19c791305b25"],
Cell[649250, 15655, 271, 6, 30, "Text",ExpressionUUID->"0559e3de-36ef-45f4-8f0d-867d774c419d"],
Cell[649524, 15663, 5192, 115, 490, "Input",ExpressionUUID->"e6a490d3-aec9-43ee-99cd-d2e3a82c13c5",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[654765, 15784, 301, 5, 33, "Section",ExpressionUUID->"1a999fc8-8494-4ccf-ba75-650a1abdf950"],
Cell[CellGroupData[{
Cell[655091, 15793, 620, 10, 34, "Subsubsection",ExpressionUUID->"e221540d-01f5-4dea-b3d1-5a827ff20062"],
Cell[655714, 15805, 3240, 75, 294, "Input",ExpressionUUID->"d9489179-b936-4016-a4aa-58fcb8c6fcd7",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[658991, 15885, 473, 8, 28, "Subsubsection",ExpressionUUID->"8acd7fea-c211-4d2f-8d71-7bc8c1dd8350"],
Cell[659467, 15895, 190, 3, 30, "Text",ExpressionUUID->"b87266d5-9756-4125-b80a-869702d42754"],
Cell[659660, 15900, 2798, 64, 257, "Input",ExpressionUUID->"5888274e-276b-4a1d-bb1a-457d55ca7eab",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[662507, 15970, 300, 5, 33, "Section",ExpressionUUID->"aeb75dcc-3cfe-4633-ad4c-e3c27c3702a0"],
Cell[662810, 15977, 692, 18, 70, "Input",ExpressionUUID->"c2180c5d-c52b-4a7c-93da-fbc8523d42c7",
 InitializationCell->True],
Cell[663505, 15997, 820, 19, 91, "Input",ExpressionUUID->"e08f034b-cba8-4065-b297-6d303da20f6f",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[664374, 16022, 834, 13, 37, "Subsubtitle",ExpressionUUID->"f2fe6078-e3fd-46dd-8c95-816603381768",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[665233, 16039, 228, 4, 34, "Section",ExpressionUUID->"99967d54-6f77-474d-bedb-8c4e667c5c06"],
Cell[665464, 16045, 2897, 68, 322, "Input",ExpressionUUID->"21354e6e-a805-47f6-84c6-57acc606e07d",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[668398, 16118, 281, 5, 34, "Section",ExpressionUUID->"f290e655-452e-4de1-b109-11adaa2be48a"],
Cell[668682, 16125, 676, 17, 91, "Input",ExpressionUUID->"6934fac6-aa82-4880-aea4-59bfb03b64ba",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[669407, 16148, 1042, 16, 37, "Subsubtitle",ExpressionUUID->"69c51c16-70f5-4cb5-8546-34ec58036ce6",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[670474, 16168, 240, 4, 34, "Section",ExpressionUUID->"c02521c6-49dc-4152-bb21-4219903a77e0"],
Cell[CellGroupData[{
Cell[670739, 16176, 177, 3, 35, "Subsubsection",ExpressionUUID->"26d38f2c-8669-4620-ac6a-8c6950677e44"],
Cell[670919, 16181, 1627, 42, 196, "Input",ExpressionUUID->"b46c921a-acf2-4668-a5b6-a7320178af5e",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[672583, 16228, 331, 5, 29, "Subsubsection",ExpressionUUID->"49a7a2be-0fe1-4d7b-81a2-aad0192dd05a"],
Cell[672917, 16235, 264, 6, 30, "Text",ExpressionUUID->"7e454695-cb47-462b-8635-4666a4feaea9"],
Cell[CellGroupData[{
Cell[673206, 16245, 3041, 64, 280, "Input",ExpressionUUID->"e452591d-9802-4264-ab6f-19442c716c99",
 InitializationCell->True],
Cell[676250, 16311, 315, 8, 24, "Message",ExpressionUUID->"bd600057-46c4-4808-86c5-ee589a15b5d6"]
}, Closed]],
Cell[CellGroupData[{
Cell[676602, 16324, 3116, 65, 295, "Input",ExpressionUUID->"2dd5c432-4421-485c-9feb-1b595b4ed946",
 InitializationCell->True],
Cell[679721, 16391, 313, 8, 24, "Message",ExpressionUUID->"e942d7e1-3e48-44d1-b23d-27e76802eef0"]
}, Closed]],
Cell[CellGroupData[{
Cell[680071, 16404, 2189, 42, 127, "Input",ExpressionUUID->"6410e8e3-79b3-4540-a640-4ab17658ba02",
 InitializationCell->True],
Cell[682263, 16448, 313, 8, 24, "Message",ExpressionUUID->"b0d7f722-821f-484b-ade7-8f0bc8aba2ff"]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[682637, 16463, 459, 7, 34, "Section",ExpressionUUID->"262509f2-ee28-464f-ab0d-1bb79655b8e6"],
Cell[683099, 16472, 1420, 38, 154, "Input",ExpressionUUID->"9aadaac9-04fa-4468-8091-619fa73ffde5",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[684556, 16515, 461, 7, 34, "Section",ExpressionUUID->"fda7e369-6533-417a-abf5-5b7f503ecd04"],
Cell[685020, 16524, 2429, 56, 238, "Input",ExpressionUUID->"b0939484-9370-4cc8-acd3-a549b7a130da",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[687486, 16585, 368, 7, 34, "Section",ExpressionUUID->"7326574c-4b01-4f9e-9960-9dba8c556cf7"],
Cell[687857, 16594, 1889, 53, 133, "Input",ExpressionUUID->"88fb44f2-ba8b-4526-8867-5c6c71bc3e55",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[689795, 16653, 1018, 15, 37, "Subsubtitle",ExpressionUUID->"34ef9abf-a527-44bd-a833-12a9fac43ef2",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[690838, 16672, 445, 7, 35, "Subsubsection",ExpressionUUID->"5102c673-e515-437d-ac70-6e13fd16c530",
 InitializationCell->True],
Cell[691286, 16681, 181, 3, 30, "Text",ExpressionUUID->"d2718061-7712-4c8b-b134-cf506f3f088c"],
Cell[691470, 16686, 899, 24, 112, "Input",ExpressionUUID->"4e92e5e9-50eb-4480-acfc-791ece0f42d7",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[692418, 16716, 623, 10, 37, "Subsubtitle",ExpressionUUID->"6c9e8735-4f4d-4168-b63e-e720b4051335",
 InitializationCell->True],
Cell[693044, 16728, 465, 11, 70, "Input",ExpressionUUID->"39317f67-488b-4c70-ba3c-e35f306dd260",
 InitializationCell->True],
Cell[693512, 16741, 460, 10, 48, "Input",ExpressionUUID->"6f426e21-c800-4846-901b-93c5d91c86d2",
 InitializationCell->True],
Cell[693975, 16753, 376, 9, 70, "Input",ExpressionUUID->"3fd09abd-5d06-484a-8b14-d2ae0118d0b1",
 InitializationCell->True],
Cell[694354, 16764, 358, 9, 91, "Input",ExpressionUUID->"e2da17c2-0c58-41d6-81a6-284f159469a2",
 InitializationCell->True],
Cell[694715, 16775, 392, 10, 70, "Input",ExpressionUUID->"11cf7bd0-47a4-4cb5-9197-fe11a746b21b",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[695156, 16791, 792, 12, 69, "Subtitle",ExpressionUUID->"7b2bc00f-d54c-47e5-b739-8733ae828ea0",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[695973, 16807, 698, 11, 41, "Subsubtitle",ExpressionUUID->"ddc0a198-1bcd-4dfd-9513-24ea2e8245ff",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[696696, 16822, 435, 7, 36, "Section",ExpressionUUID->"7cf69058-ce00-47e1-863f-f2d5ed522fee",
 InitializationCell->True],
Cell[697134, 16831, 404, 12, 48, "Input",ExpressionUUID->"461e7175-2c1c-468d-9302-c26232b5640d",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[697575, 16848, 377, 7, 36, "Section",ExpressionUUID->"ebd52414-4355-40d2-9bc0-2e2184ad7d7c",
 InitializationCell->True],
Cell[697955, 16857, 1563, 29, 133, "Input",ExpressionUUID->"545f8faa-5f84-4519-8ed5-2b4370853580",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[699555, 16891, 312, 6, 36, "Section",ExpressionUUID->"227953d7-89c0-4895-aae1-be2da0545511",
 InitializationCell->True],
Cell[699870, 16899, 179, 3, 30, "Text",ExpressionUUID->"4c395e5b-afe2-4ba5-8efe-ddfd299cac8b"],
Cell[700052, 16904, 2574, 49, 133, "Input",ExpressionUUID->"c51a58d6-d314-4dd6-ab2b-dd0963e76a80",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[702663, 16958, 277, 5, 36, "Section",ExpressionUUID->"571acdfc-0623-4e73-9059-b882f98c9957",
 InitializationCell->True],
Cell[702943, 16965, 1054, 23, 89, "Input",ExpressionUUID->"7cdd9689-4411-4a16-9f0c-635cf1d88254",
 InitializationCell->True],
Cell[704000, 16990, 1959, 36, 68, "Input",ExpressionUUID->"1aa9b8a0-0bb3-4ab7-96d7-62ffbbfb65aa",
 InitializationCell->True],
Cell[705962, 17028, 1964, 36, 68, "Input",ExpressionUUID->"8ba48ece-63e0-4234-a908-6e26966ad5f3",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[707963, 17069, 770, 12, 36, "Section",ExpressionUUID->"43076762-126f-4899-8438-61147831df63",
 InitializationCell->True],
Cell[708736, 17083, 1393, 27, 48, "Input",ExpressionUUID->"9b8d6190-1fef-4630-aa39-5b40c6e5aca2",
 InitializationCell->True]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[710178, 17116, 712, 11, 41, "Subsubtitle",ExpressionUUID->"9a124edb-f669-42cb-8fbe-4dea0e749d0b",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[710915, 17131, 157, 3, 54, "Subsection",ExpressionUUID->"f7fe34a8-2c90-40de-8a33-52aae41fe4d1"],
Cell[711075, 17136, 183, 3, 30, "Text",ExpressionUUID->"b412286b-52ec-47c9-814d-8f2f26041812"],
Cell[711261, 17141, 492, 12, 48, "Input",ExpressionUUID->"8c715c68-521d-4ed3-936b-704d233173a0",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[711790, 17158, 159, 3, 38, "Subsection",ExpressionUUID->"9571cc32-e0d4-4d13-a9be-9a2d1eaad7cf"],
Cell[711952, 17163, 900, 21, 91, "Input",ExpressionUUID->"097fe774-92c7-4b73-b3b8-386e67bc078a",
 InitializationCell->True],
Cell[712855, 17186, 1070, 25, 91, "Input",ExpressionUUID->"765552b9-97f1-4c95-8c80-e3e87c4170e4",
 InitializationCell->True],
Cell[713928, 17213, 1070, 25, 91, "Input",ExpressionUUID->"63f0cbfc-6dc7-481a-b9b2-285b6a18cb51",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[715035, 17243, 216, 4, 38, "Subsection",ExpressionUUID->"49d7eee2-8387-4567-81f1-e3ffd217218b"],
Cell[715254, 17249, 705, 16, 70, "Input",ExpressionUUID->"883d7907-b89f-4fee-8b68-14d8ea18034a",
 InitializationCell->True],
Cell[715962, 17267, 807, 21, 70, "Input",ExpressionUUID->"bb351f6a-8c95-4fd4-8a7c-0e0690f216f6",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[716806, 17293, 259, 4, 38, "Subsection",ExpressionUUID->"14552be0-3250-4615-a432-a72cd283941e"],
Cell[717068, 17299, 506, 14, 48, "Input",ExpressionUUID->"a264721b-a51f-4fc8-b409-9bf0671e729d",
 InitializationCell->True]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[717623, 17319, 712, 11, 41, "Subsubtitle",ExpressionUUID->"3063970c-14a2-4b9c-9ca8-7388d12b45f4",
 InitializationCell->True],
Cell[718338, 17332, 306, 7, 30, "Text",ExpressionUUID->"1b2f9b7c-644c-4439-a170-6f14368754ee"],
Cell[718647, 17341, 200, 3, 30, "Text",ExpressionUUID->"b6728fc0-5d32-482a-b93d-0d9bed1b971e"],
Cell[718850, 17346, 1033, 23, 154, "Input",ExpressionUUID->"5ccd78a8-f54c-4dc5-ad64-9f765e396a4d",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[719920, 17374, 717, 11, 41, "Subsubtitle",ExpressionUUID->"e93aeac6-c5ac-4f70-afe6-a49d97872969",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[720662, 17389, 1301, 19, 34, "Section",ExpressionUUID->"d917158c-0e8b-44c6-99d4-7f93c85a64e5",
 InitializationCell->True],
Cell[721966, 17410, 1085, 22, 91, "Input",ExpressionUUID->"ab75e57a-bae8-4831-8ed6-cfae334838d8",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[723088, 17437, 1234, 18, 34, "Section",ExpressionUUID->"77a0778a-308f-40af-ba7e-e4d83c0f670b",
 InitializationCell->True],
Cell[724325, 17457, 933, 18, 91, "Input",ExpressionUUID->"5081d83b-7f3b-4a1a-950e-f455d01cef7b",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[725295, 17480, 1307, 19, 34, "Section",ExpressionUUID->"901db580-8d88-4bf9-8355-cf2adf4cafab",
 InitializationCell->True],
Cell[726605, 17501, 677, 15, 91, "Input",ExpressionUUID->"96a0bd5f-854f-4920-be43-b2403fad0bbb",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[727331, 17522, 701, 11, 41, "Subsubtitle",ExpressionUUID->"7d3927ad-d297-47f0-9a44-7976c9165132",
 InitializationCell->True],
Cell[728035, 17535, 350, 9, 48, "Input",ExpressionUUID->"925b6db4-647f-4b66-9104-209d32650a59",
 InitializationCell->True],
Cell[728388, 17546, 363, 9, 48, "Input",ExpressionUUID->"944f84d5-13d3-4cd0-a877-0872b12fdd0c",
 InitializationCell->True],
Cell[728754, 17557, 348, 9, 48, "Input",ExpressionUUID->"e3ae4f7e-2386-468d-aa1a-899bf860b20b",
 InitializationCell->True],
Cell[729105, 17568, 348, 9, 48, "Input",ExpressionUUID->"8d612bfe-89a2-403b-9b17-0dd1a16bbe14",
 InitializationCell->True]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[729502, 17583, 975, 15, 69, "Subtitle",ExpressionUUID->"8dc269e2-25dd-416b-a8bf-595aec765b35",
 InitializationCell->True],
Cell[730480, 17600, 171, 3, 35, "Text",ExpressionUUID->"2b7fa00f-9003-4e6b-affe-c6cab56e64fd"],
Cell[CellGroupData[{
Cell[730676, 17607, 750, 12, 41, "Subsubtitle",ExpressionUUID->"8b2da559-2e72-4738-ba63-ea4dccb1423f",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[731451, 17623, 353, 6, 36, "Section",ExpressionUUID->"96e8d8ef-8433-4251-8ab0-f59555d5629b",
 InitializationCell->True],
Cell[731807, 17631, 338, 9, 48, "Input",ExpressionUUID->"49e19798-3aae-4f8d-bd91-307d8252019a",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[732182, 17645, 312, 6, 36, "Section",ExpressionUUID->"127ef8a1-aff2-4786-9541-1403de7dbcc4",
 InitializationCell->True],
Cell[732497, 17653, 283, 7, 30, "Text",ExpressionUUID->"bafd6600-9203-4725-91bd-2dd2a7949f6e"],
Cell[732783, 17662, 3099, 60, 154, "Input",ExpressionUUID->"20209e1b-4ee4-4011-b9b7-884a5b356286",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[735919, 17727, 351, 6, 36, "Section",ExpressionUUID->"4f5afd21-a67a-4306-9450-9f2e2be47769",
 InitializationCell->True],
Cell[736273, 17735, 1087, 23, 110, "Input",ExpressionUUID->"7b5f6d8b-6e3c-44b3-9ccd-2fef1994e423",
 InitializationCell->True],
Cell[737363, 17760, 192, 3, 35, "Text",ExpressionUUID->"3cd3e40c-2523-49e8-b65b-166402522e1c"],
Cell[737558, 17765, 2567, 47, 110, "Input",ExpressionUUID->"27474e0b-361e-4c86-8009-bb3aba1c55a5",
 InitializationCell->True],
Cell[740128, 17814, 2476, 46, 110, "Input",ExpressionUUID->"83fb9d30-a678-4c32-890c-854bd591767e",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[742641, 17865, 421, 7, 36, "Section",ExpressionUUID->"dbbf3a36-b433-45ea-b8ec-cf793f1fbcb5",
 InitializationCell->True],
Cell[743065, 17874, 1360, 26, 48, "Input",ExpressionUUID->"d34759d0-a137-4558-b2a1-20e3547f857c",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[744474, 17906, 717, 11, 41, "Subsubtitle",ExpressionUUID->"7e7c566e-41ed-4a7b-a413-a4c02e77b932",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[745216, 17921, 1301, 19, 34, "Section",ExpressionUUID->"1179fb47-5978-432a-8cf8-373f479d2b28",
 InitializationCell->True],
Cell[746520, 17942, 581, 17, 48, "Input",ExpressionUUID->"5a707190-fa62-4b24-81e0-0c85ce9b938a",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[747138, 17964, 1304, 19, 34, "Section",ExpressionUUID->"787fa760-80b3-4c58-8165-e2bbdee3f8a1",
 InitializationCell->True],
Cell[748445, 17985, 638, 19, 48, "Input",ExpressionUUID->"27fb3e26-0901-4544-ad54-3a37a6df61ae",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[749120, 18009, 1326, 19, 34, "Section",ExpressionUUID->"7ad9d9d8-7a23-432f-a712-41e0c96ede00",
 InitializationCell->True],
Cell[750449, 18030, 634, 19, 48, "Input",ExpressionUUID->"33dbdee1-8c7d-4f77-9a37-4782cc1ab2f9",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[751120, 18054, 1330, 19, 34, "Section",ExpressionUUID->"818788cc-6a46-422a-bd1a-5872f800b63e",
 InitializationCell->True],
Cell[752453, 18075, 561, 16, 48, "Input",ExpressionUUID->"7e7e6bd2-2f37-42d9-bfbf-86df143c72f0",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[753051, 18096, 1332, 19, 34, "Section",ExpressionUUID->"44d3abc0-1a85-4f9f-8ef8-d88dee91b5b7",
 InitializationCell->True],
Cell[754386, 18117, 581, 16, 70, "Input",ExpressionUUID->"2585f3e6-dd39-462e-8f5b-7c2cc1a11b11",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[755004, 18138, 1357, 20, 34, "Section",ExpressionUUID->"b96d7703-f02c-4531-81d2-e9e46aec1562",
 InitializationCell->True],
Cell[756364, 18160, 581, 17, 70, "Input",ExpressionUUID->"a9e04766-10f8-4ab6-b421-76388dae603f",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[756982, 18182, 1337, 19, 34, "Section",ExpressionUUID->"4e7b485c-f80e-4130-8bfa-3d966556787c",
 InitializationCell->True],
Cell[758322, 18203, 570, 15, 70, "Input",ExpressionUUID->"bb1c7ba5-7527-4b6e-8c37-86b0d7d4c62d",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[758941, 18224, 725, 11, 41, "Subsubtitle",ExpressionUUID->"33f7429c-ad9f-48a0-9634-a2bd8b1ebf2a",
 InitializationCell->True],
Cell[759669, 18237, 523, 11, 91, "Input",ExpressionUUID->"c1b87ac2-063a-4699-a39c-e2fd1e4863dc",
 InitializationCell->True],
Cell[760195, 18250, 467, 10, 70, "Input",ExpressionUUID->"9266d04d-2b55-40a4-a9dd-9d56b1ca7b11",
 InitializationCell->True]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[760711, 18266, 915, 14, 69, "Subtitle",ExpressionUUID->"6368a012-da4d-48f0-b5ab-49611f086313",
 InitializationCell->True],
Cell[761629, 18282, 344, 6, 35, "Text",ExpressionUUID->"d3b93a9b-5355-4c01-98d9-6e3608262357",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[761998, 18292, 620, 10, 41, "Subsubtitle",ExpressionUUID->"8560a142-d444-43dd-ada6-36b54135ece5",
 InitializationCell->True],
Cell[762621, 18304, 180, 3, 30, "Text",ExpressionUUID->"725e92f8-041b-4e12-b965-23a66cc17d42"]
}, Closed]],
Cell[CellGroupData[{
Cell[762838, 18312, 590, 9, 57, "Subsubtitle",ExpressionUUID->"d90527f6-7bba-4c53-899f-e4f351749428",
 InitializationCell->True],
Cell[763431, 18323, 362, 10, 46, "Input",ExpressionUUID->"486c2945-9b2f-42e9-9976-8470425b3f76",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[763818, 18337, 227, 4, 45, "Subsubsection",ExpressionUUID->"710606ff-bd58-4c26-80e6-49086a6e3852"],
Cell[764048, 18343, 505, 15, 68, "Input",ExpressionUUID->"16f40556-85e8-4d3e-ab5c-2262639d15ed",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[764590, 18363, 206, 3, 37, "Subsubsection",ExpressionUUID->"dbc7dea9-8aba-47b9-979f-8d21351c94fd"],
Cell[764799, 18368, 3703, 90, 425, "Input",ExpressionUUID->"616a8968-1c1e-4afa-a683-5273534be7d5",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[768539, 18463, 492, 7, 61, "Subsubsection",ExpressionUUID->"acec8ad6-8dbe-4af1-86d9-9476b7ba2e5c"],
Cell[769034, 18472, 1392, 34, 278, "Input",ExpressionUUID->"682eb89c-1527-450a-b505-f508fdae849d",
 InitializationCell->True],
Cell[770429, 18508, 1776, 38, 236, "Input",ExpressionUUID->"6548e9a3-ca3d-4ebb-aa7c-ae9c6543f3f9",
 InitializationCell->True]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[772254, 18552, 586, 9, 41, "Subsubtitle",ExpressionUUID->"0945ee8c-d102-420c-9da4-8e67987e7341",
 InitializationCell->True],
Cell[772843, 18563, 1433, 35, 236, "Input",ExpressionUUID->"c3eec483-5aae-4152-990d-2f29a2a69f72",
 InitializationCell->True],
Cell[774279, 18600, 301, 7, 52, "Code",ExpressionUUID->"b59f3ec2-5b40-4ec3-ba8d-41ca6d11aa11"],
Cell[774583, 18609, 339, 8, 52, "Code",ExpressionUUID->"28738460-b069-4be5-8bdd-2237705e4a8e"],
Cell[774925, 18619, 388, 9, 52, "Code",ExpressionUUID->"3c782de1-8846-4e2d-b687-548fa8dbb797"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[775362, 18634, 829, 13, 69, "Subtitle",ExpressionUUID->"67931949-6efc-425c-890f-4c123445922c",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[776216, 18651, 565, 10, 41, "Subsubtitle",ExpressionUUID->"b3676f75-b2bd-47ea-a7cb-8253c08a66dc",
 InitializationCell->True],
Cell[776784, 18663, 239, 6, 35, "Text",ExpressionUUID->"99e8912c-0596-481b-b722-990dd65e9545",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[777048, 18673, 281, 6, 36, "Section",ExpressionUUID->"3f183259-7815-4d14-9ada-88e90ab91a66",
 InitializationCell->True,
 InitializationGroup->True],
Cell[777332, 18681, 1458, 25, 48, "Input",ExpressionUUID->"9797ceec-0c7c-4150-875d-295045551ceb",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[778827, 18711, 304, 6, 36, "Section",ExpressionUUID->"4178f8f6-b32c-4b7c-a50f-611f876ff7b9",
 InitializationCell->True,
 InitializationGroup->True],
Cell[779134, 18719, 452, 13, 48, "Input",ExpressionUUID->"627f94c6-f084-4e52-b861-c86a5eb32d65",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[779623, 18737, 253, 6, 36, "Section",ExpressionUUID->"c66375d2-c363-4de2-803e-a2f5be268cb3",
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[779901, 18747, 539, 9, 44, "Subsection",ExpressionUUID->"b2856ef6-41a1-4d1c-9320-3ef4e2229a9a",
 InitializationCell->True],
Cell[780443, 18758, 1762, 33, 70, "Input",ExpressionUUID->"a091509e-9e93-434f-8397-2eb5f346542d",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[782242, 18796, 488, 8, 36, "Subsection",ExpressionUUID->"64c52543-c4e0-48dc-bdd3-bdf164fab0b0",
 InitializationCell->True],
Cell[782733, 18806, 213, 4, 30, "Text",ExpressionUUID->"4d9c79cc-5fac-4995-9aec-5a13f5a782c2",
 InitializationCell->True],
Cell[782949, 18812, 2319, 47, 259, "Input",ExpressionUUID->"24ffc0ab-9926-4280-8847-f8bc888c898d",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[785305, 18864, 445, 7, 36, "Subsection",ExpressionUUID->"162cd0b7-ee31-407b-8319-a431c77d3adf",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[785775, 18875, 278, 5, 35, "Subsubsection",ExpressionUUID->"8affda2f-45af-41cb-9862-3c111078bb4f",
 InitializationCell->True],
Cell[786056, 18882, 625, 16, 48, "Input",ExpressionUUID->"2cffb4db-8182-4c4d-bc66-d96ad17ec7c8",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[786718, 18903, 264, 5, 29, "Subsubsection",ExpressionUUID->"a0be3577-ca6c-4c5c-81a5-cdaef8cbaae9",
 InitializationCell->True],
Cell[786985, 18910, 452, 13, 48, "Input",ExpressionUUID->"0d110f14-ff65-4f96-bbe0-7f81c5301858",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[787474, 18928, 253, 5, 29, "Subsubsection",ExpressionUUID->"ff16dbc2-0b76-452f-b87e-9f3910e81ce6",
 InitializationCell->True],
Cell[787730, 18935, 640, 16, 48, "Input",ExpressionUUID->"549fcf96-0e8e-489a-8d3e-10369956418e",
 InitializationCell->True],
Cell[788373, 18953, 746, 20, 70, "Input",ExpressionUUID->"2306c364-874f-47eb-bdbf-111b823706c5",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[789156, 18978, 313, 6, 29, "Subsubsection",ExpressionUUID->"93a1ce48-d58a-4b14-99fb-db8207e6546b",
 InitializationCell->True],
Cell[789472, 18986, 826, 21, 70, "Input",ExpressionUUID->"337be608-f23c-46d4-abdc-c78716fc0112",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[790335, 19012, 295, 5, 35, "Subsubsection",ExpressionUUID->"2abf8c37-7087-41ea-af36-2e52e55a9777",
 InitializationCell->True],
Cell[790633, 19019, 631, 19, 48, "Input",ExpressionUUID->"10e50f1c-80bb-44eb-8005-cb644e04d129",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[791313, 19044, 308, 6, 44, "Subsection",ExpressionUUID->"3c7384f4-16a0-4300-b99e-ffd128c5f562",
 InitializationCell->True],
Cell[791624, 19052, 1160, 24, 154, "Input",ExpressionUUID->"75440357-5eca-42cc-a929-2d8d0a344576",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[792821, 19081, 322, 6, 36, "Subsection",ExpressionUUID->"3c3bb6a6-1b36-44f5-911c-47f00cd3b0c7",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[793168, 19091, 487, 8, 35, "Subsubsection",ExpressionUUID->"3581047a-07d2-4d4a-b480-320f6cbe0c11",
 InitializationCell->True],
Cell[793658, 19101, 1152, 24, 154, "Input",ExpressionUUID->"8dbe2cc2-0fa0-4120-8c54-1fdae311601e",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[794847, 19130, 535, 9, 29, "Subsubsection",ExpressionUUID->"7e41eb06-bd9a-4e19-9d15-c0a76ef224f2",
 InitializationCell->True],
Cell[795385, 19141, 1080, 26, 70, "Input",ExpressionUUID->"be9ce7cd-87ef-4209-abda-15fd41bae6bb",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[796526, 19174, 468, 8, 36, "Section",ExpressionUUID->"40836cbb-1856-4e3d-bbd0-af89e448c4fa",
 InitializationCell->True,
 InitializationGroup->True],
Cell[796997, 19184, 551, 15, 89, "Input",ExpressionUUID->"88623d3d-cdab-4e39-818c-8b8e0a7f0296",
 InitializationCell->True],
Cell[797551, 19201, 426, 9, 58, "Text",ExpressionUUID->"541c85ec-15bd-4f80-9ef6-96b9ad8ac11b",
 InitializationCell->True]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[798026, 19216, 581, 9, 41, "Subsubtitle",ExpressionUUID->"977312a9-60e9-4516-8649-8d2783b42138",
 InitializationCell->True],
Cell[798610, 19227, 619, 12, 49, "Text",ExpressionUUID->"c7ac9279-fa90-4f71-b111-3674a5cdab4f",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[799254, 19243, 275, 5, 34, "Section",ExpressionUUID->"0526a318-a8ff-49cf-9a4a-951897fc0cda",
 InitializationCell->True],
Cell[799532, 19250, 375, 10, 48, "Input",ExpressionUUID->"bd9f9b79-45c0-4877-bb2e-d80f09759c02",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[799944, 19265, 411, 7, 34, "Section",ExpressionUUID->"3d501e7f-56fd-4872-a4e9-148c327bf341",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[800380, 19276, 714, 13, 35, "Subsubsection",ExpressionUUID->"2949361a-2132-4497-8a9c-75ed1524907d",
 InitializationCell->True],
Cell[801097, 19291, 744, 19, 70, "Input",ExpressionUUID->"5722aca9-2145-4e1f-baa9-62b4d6516d29",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[801878, 19315, 652, 11, 29, "Subsubsection",ExpressionUUID->"d16eddfb-d5ae-46de-b45d-042f0e65d5c8",
 InitializationCell->True],
Cell[802533, 19328, 684, 17, 91, "Input",ExpressionUUID->"939585e0-5dfb-4156-b065-013b5ea607c2",
 InitializationCell->True],
Cell[803220, 19347, 706, 18, 91, "Input",ExpressionUUID->"b7d8b13b-09b6-47f4-b4d7-7b43f8fdd9e1",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[803963, 19370, 561, 10, 35, "Subsubsection",ExpressionUUID->"780c94ef-5a0c-49db-a06c-7bc5b98ef380",
 InitializationCell->True],
Cell[804527, 19382, 770, 15, 48, "Input",ExpressionUUID->"0faa37e6-f587-4f60-a0d8-1fa5b95f5cc2",
 InitializationCell->True],
Cell[805300, 19399, 271, 8, 48, "Input",ExpressionUUID->"0cbe11bc-2f59-47ca-9428-5f5f6a9fe45c",
 InitializationCell->True],
Cell[805574, 19409, 302, 8, 48, "Input",ExpressionUUID->"68ebfb1e-9166-47d5-ab6a-3172bc6d4c36",
 InitializationCell->True],
Cell[805879, 19419, 314, 9, 48, "Input",ExpressionUUID->"234467c0-f2bf-4409-8cac-dda4caa50c6c",
 InitializationCell->True],
Cell[806196, 19430, 384, 11, 48, "Input",ExpressionUUID->"ef3afb71-0cbf-4aea-b396-d202bbb45388",
 InitializationCell->True],
Cell[806583, 19443, 520, 16, 70, "Input",ExpressionUUID->"2316ba5a-3dd4-4228-ac2e-2613c7b70e80",
 InitializationCell->True],
Cell[807106, 19461, 544, 15, 70, "Input",ExpressionUUID->"1be17f4c-4c90-4a8d-99d1-625187654879",
 InitializationCell->True],
Cell[807653, 19478, 393, 11, 48, "Input",ExpressionUUID->"da794ed6-5aaf-4b63-8c70-9564b81b32b7",
 InitializationCell->True],
Cell[808049, 19491, 412, 11, 48, "Input",ExpressionUUID->"18fdeaa8-684c-413b-8504-c3855e887d50",
 InitializationCell->True],
Cell[808464, 19504, 640, 16, 91, "Input",ExpressionUUID->"4285af76-ea8b-4cc6-b0d5-196f184a7eb7",
 InitializationCell->True],
Cell[809107, 19522, 743, 19, 112, "Input",ExpressionUUID->"e8dfbb42-196b-483b-a96c-5a17a95a3537",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[809887, 19546, 481, 8, 29, "Subsubsection",ExpressionUUID->"16be6bbd-acf7-4b46-9652-a064e95a5dd5",
 InitializationCell->True],
Cell[810371, 19556, 1047, 22, 112, "Input",ExpressionUUID->"398ba91a-60a5-414f-a3b4-5c9eb6f63bd9",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[811455, 19583, 468, 8, 35, "Subsubsection",ExpressionUUID->"11ca9c4a-ecf8-4df7-adde-5b10c12b2325",
 InitializationCell->True],
Cell[811926, 19593, 996, 22, 112, "Input",ExpressionUUID->"0cd6d657-1379-4c2a-8ed8-b73d2a86fc53",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[812971, 19621, 360, 6, 34, "Section",ExpressionUUID->"6dfbcaf5-4c5d-4f0b-8051-a9b9e28e554b",
 InitializationCell->True],
Cell[813334, 19629, 907, 20, 112, "Input",ExpressionUUID->"be8cc496-ed62-4a7a-a214-cb8e02483c21",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[814278, 19654, 279, 5, 34, "Section",ExpressionUUID->"64d8f673-9c53-4e27-a99b-117999f7a190",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[814582, 19663, 343, 8, 35, "Subsubsection",ExpressionUUID->"dd57af5b-9b47-4c9b-9b60-c1184626687c",
 InitializationCell->True],
Cell[814928, 19673, 1292, 29, 112, "Input",ExpressionUUID->"fe1e3e60-c589-4349-97f9-ac9718bd7818",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[816257, 19707, 430, 9, 53, "Subsubsection",ExpressionUUID->"35b4ed62-8519-4c39-bc40-7c96a8fe3585",
 InitializationCell->True],
Cell[816690, 19718, 1468, 33, 112, "Input",ExpressionUUID->"a4aace2d-6bbb-49bf-9c00-98fb18d4125b",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[818195, 19756, 453, 9, 53, "Subsubsection",ExpressionUUID->"fc2e5acc-66b4-4ea0-86b1-f7ce981f759a",
 InitializationCell->True],
Cell[818651, 19767, 1405, 30, 112, "Input",ExpressionUUID->"7ca1f8e6-dce9-4abb-b275-00a1b18dc3d4",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[820105, 19803, 263, 5, 34, "Section",ExpressionUUID->"7d1a1e64-6f7e-4768-9dfb-eabce37cce77",
 InitializationCell->True],
Cell[820371, 19810, 689, 20, 70, "Input",ExpressionUUID->"cafd0ec7-f4f2-4384-b420-8b79aca15aa3",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[821097, 19835, 471, 8, 34, "Section",ExpressionUUID->"bf1b7288-25b1-4515-910e-dc370138a193",
 InitializationCell->True],
Cell[821571, 19845, 929, 22, 112, "Input",ExpressionUUID->"e70f85ea-57e1-46ce-9342-877384f1f0be",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[822537, 19872, 521, 8, 34, "Section",ExpressionUUID->"481dab05-8054-4e28-805a-e1b693ae1755",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[823083, 19884, 566, 9, 35, "Subsubsection",ExpressionUUID->"d8122eb3-bbfb-4fb4-adb1-1ba4ed5fabe7",
 InitializationCell->True],
Cell[823652, 19895, 1921, 42, 259, "Input",ExpressionUUID->"9ccb910f-1cec-41be-8e41-2e1920314d35",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[825610, 19942, 549, 8, 35, "Subsubsection",ExpressionUUID->"8146c864-2817-4f11-8f62-881e499f9c60",
 InitializationCell->True],
Cell[826162, 19952, 934, 22, 133, "Input",ExpressionUUID->"169f0d59-7fd4-4c1c-8dfd-e0cee6eacf6b",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[827157, 19981, 642, 11, 41, "Subsubtitle",ExpressionUUID->"75c11cf9-1cf1-40a1-8521-1e5b1bae82bd",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[827824, 19996, 566, 10, 34, "Section",ExpressionUUID->"4b86c2a8-db82-4580-bc17-669235942ba6",
 InitializationCell->True],
Cell[828393, 20008, 435, 12, 48, "Input",ExpressionUUID->"966bb394-95ad-478f-9046-4f8f21e50eab",
 InitializationCell->True],
Cell[828831, 20022, 824, 18, 70, "Input",ExpressionUUID->"f34bd3ed-58fe-4e0f-b999-8a7548507927",
 InitializationCell->True],
Cell[829658, 20042, 757, 19, 48, "Input",ExpressionUUID->"2509fd3f-86ef-4c54-a83b-8c9a3d8d64e3",
 InitializationCell->True],
Cell[830418, 20063, 864, 23, 48, "Input",ExpressionUUID->"bc73b927-fbec-4882-89c3-c26f56849834",
 InitializationCell->True],
Cell[831285, 20088, 737, 19, 48, "Input",ExpressionUUID->"5bf66812-8cd4-4dc6-9c91-e6ff8fa624c4",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[832059, 20112, 616, 10, 34, "Section",ExpressionUUID->"e38c5dab-50fd-4b7e-abc7-c9bc5f97a1d3",
 InitializationCell->True],
Cell[832678, 20124, 1968, 37, 112, "Input",ExpressionUUID->"3b466775-1f49-40bd-affb-2d2b54efe230",
 InitializationCell->True],
Cell[834649, 20163, 840, 22, 70, "Input",ExpressionUUID->"baf06206-4095-4cda-b191-339e0e92c34c",
 InitializationCell->True],
Cell[835492, 20187, 787, 19, 70, "Input",ExpressionUUID->"e196eb0e-0c42-4428-a500-7321ae7e186f",
 InitializationCell->True],
Cell[836282, 20208, 906, 23, 70, "Input",ExpressionUUID->"559631e1-7e52-4be4-be8b-11a3677fcb56",
 InitializationCell->True],
Cell[837191, 20233, 855, 21, 70, "Input",ExpressionUUID->"65ed33ee-8889-4ead-97da-ce62d806fa60",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[838083, 20259, 504, 8, 34, "Section",ExpressionUUID->"f55d1b40-96ff-4fbf-a552-aa4b2c09ce61",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[838612, 20271, 277, 5, 35, "Subsubsection",ExpressionUUID->"17c89699-36c2-43ba-96a4-43e247f6f8df",
 InitializationCell->True],
Cell[838892, 20278, 404, 12, 48, "Input",ExpressionUUID->"07714143-5c12-4320-b5ce-dd92aac9d167",
 InitializationCell->True],
Cell[839299, 20292, 429, 13, 48, "Input",ExpressionUUID->"b43246b8-20f9-4cd6-baf4-800538164ca9",
 InitializationCell->True],
Cell[839731, 20307, 1134, 32, 91, "Input",ExpressionUUID->"e0a06437-b489-49bb-8d6e-02bb3c6121a8",
 InitializationCell->True],
Cell[840868, 20341, 778, 22, 70, "Input",ExpressionUUID->"e8e9eb42-f270-4fca-af2d-2111c50de307",
 InitializationCell->True],
Cell[841649, 20365, 1111, 29, 91, "Input",ExpressionUUID->"3c95a25e-8e6b-4cb6-8169-141df663c7df",
 InitializationCell->True],
Cell[842763, 20396, 401, 12, 48, "Input",ExpressionUUID->"ae609166-f063-4f02-9b8b-a8b4ab63e703",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[843201, 20413, 351, 6, 35, "Subsubsection",ExpressionUUID->"86c01827-bf8a-49c4-a05f-59a514d6085a",
 InitializationCell->True],
Cell[843555, 20421, 1172, 27, 70, "Input",ExpressionUUID->"ad9e3faa-90cc-498e-9b14-7c1b3f51f251",
 InitializationCell->True],
Cell[844730, 20450, 1325, 32, 91, "Input",ExpressionUUID->"d4eb02d8-4a6d-4e1c-a8aa-7ddadd1802e3",
 InitializationCell->True],
Cell[846058, 20484, 1179, 27, 91, "Input",ExpressionUUID->"5ed23c59-290e-477f-a331-b37aa58fcf2b",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[847274, 20516, 282, 5, 35, "Subsubsection",ExpressionUUID->"e7ef73d4-9033-4494-a306-d2ee3d14b582",
 InitializationCell->True],
Cell[847559, 20523, 398, 10, 70, "Input",ExpressionUUID->"53d243c2-80e7-41a1-b0db-3c30b6c2077e",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[848018, 20540, 601, 10, 41, "Subsubtitle",ExpressionUUID->"2d8d66f0-9c70-4df8-af1a-ec344df501fd",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[848644, 20554, 609, 10, 34, "Section",ExpressionUUID->"cc2bfcb4-7ead-4e15-8368-d4624d727ff6",
 InitializationCell->True],
Cell[849256, 20566, 594, 15, 70, "Input",ExpressionUUID->"5c989e41-2e1f-4142-85f8-c68ac08ab85f",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[849887, 20586, 653, 10, 34, "Section",ExpressionUUID->"d62d9ff7-6f3e-45f6-a62c-5fb1f3f44c7a",
 InitializationCell->True],
Cell[850543, 20598, 595, 15, 48, "Input",ExpressionUUID->"c5145f5e-9e42-43e3-a577-8811765d44c9",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[851199, 20620, 907, 14, 69, "Subtitle",ExpressionUUID->"68c89773-19ed-4105-a3d6-09cc2af71744",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[852131, 20638, 945, 15, 36, "Section",ExpressionUUID->"74878d79-6e3a-4b53-945a-b68110f3dce8",
 InitializationCell->True],
Cell[853079, 20655, 646, 14, 70, "Input",ExpressionUUID->"64875045-42a8-4420-a5d2-cfbe3fc147f2",
 InitializationCell->True]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[853774, 20675, 550, 10, 69, "Subtitle",ExpressionUUID->"c41bd980-f903-41f9-ad31-7c6dd4d741e8",
 InitializationCell->True],
Cell[854327, 20687, 299, 10, 66, "Input",ExpressionUUID->"a4a6ec76-7991-4111-a11c-99ebd9b44914",
 InitializationCell->True,
 InitializationGroup->True]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[854675, 20703, 293, 7, 98, "Title",ExpressionUUID->"0e02bf6d-1670-49cf-976b-e1907a5b86ff",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[854993, 20714, 690, 10, 44, "Subsection",ExpressionUUID->"5af9302f-bcce-45c3-a52e-4594cef79698"],
Cell[855686, 20726, 394, 8, 68, "Text",ExpressionUUID->"671b5c3c-f6f2-4a9e-805e-6a9a33e0af47"],
Cell[856083, 20736, 200, 3, 30, "Text",ExpressionUUID->"9a5f65ae-7c0f-4292-8e09-a4e200993996"],
Cell[856286, 20741, 243, 6, 30, "Text",ExpressionUUID->"4aa1b96b-26a2-4929-bb6c-bd5c779ef5bd"]
}, Open  ]]
}, Closed]]
}
]
*)

