(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     53586,       1469]
NotebookOptionsPosition[     48834,       1302]
NotebookOutlinePosition[     49229,       1319]
CellTagsIndexPosition[     49186,       1316]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Interactions`Interactions`", "Title",
 InitializationCell->True,
 CellChangeTimes->{{3.574846182639838*^9, 3.5748461881813183`*^9}, {
   3.6825936968926888`*^9, 3.6825936978525543`*^9}, {3.682612768669744*^9, 
   3.6826127689298697`*^9}, 3.691678395488236*^9, 3.692256322901679*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"BeginPackage", "[", 
   RowBox[{"\"\<Interactions`Interactions`\>\"", ",", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<Global`\>\"", ",", "\[IndentingNewLine]", "\"\<SpinDynamica`\>\"", 
      ",", "\[IndentingNewLine]", "\"\<General`Miscellaneous`\>\"", ",", 
      "\[IndentingNewLine]", "\"\<General`PhysicalConstants`\>\""}], 
     "\[IndentingNewLine]", "}"}]}], "\[IndentingNewLine]", "]"}], 
  ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{
  3.4482891836043177`*^9, {3.564598196767482*^9, 3.564598197018395*^9}, {
   3.56459902366787*^9, 3.564599024607004*^9}, 3.596883741048459*^9, {
   3.596885578899312*^9, 3.596885579236474*^9}, {3.68259369334947*^9, 
   3.6825936942136927`*^9}, {3.68259471075417*^9, 3.682594732293944*^9}, {
   3.682597213608466*^9, 3.682597214926736*^9}, {3.682612771099472*^9, 
   3.682612771475587*^9}, {3.683110311627295*^9, 3.68311031410594*^9}, 
   3.691678704814987*^9, 3.6922563253165197`*^9},
 AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["exports", "Subsubtitle",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.574846275000592*^9, 3.574846295983759*^9}},
 AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Isotope", "Subsection",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.409834749153043*^9, 3.409834750106721*^9}, {
   3.574844315921792*^9, 3.574844317327209*^9}, 3.5748462750021772`*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Isotope", "::", "usage"}], " ", "=", " ", 
   "\"\<Isotope[A] finds the isotope with mass number A. A warning is issued \
if there are duplicate mass numbers.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574846162136796*^9, 3.574846170184763*^9}, 
   3.5748462750032597`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["NuclearSpinQuantumNumber", "Subsection",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.409834749153043*^9, 3.409834750106721*^9}, 
   3.574846275003632*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NuclearSpinQuantumNumber", "::", "usage"}], " ", "=", " ", 
   "\"\<NuclearSpinQuantumNumber[isotope] gives the ground-state nuclear spin \
quantum number of the given isotope, specified using either its mass number, \
or as an abbreviation, for example \\\"13C\\\".\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574845208885941*^9, 3.574845249654599*^9}, {
   3.574845394556673*^9, 3.5748454556303*^9}, {3.574845488079295*^9, 
   3.574845489195837*^9}, 3.574846275004181*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["MagnetogyricRatio", "Subsection",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{3.574846275004548*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MagnetogyricRatio", "::", "usage"}], " ", "=", " ", 
   "\"\<MagnetogyricRatio[isotope] gives the magnetogyric ratio of the \
specified isotope, in units of rad\[NonBreakingSpace]\!\(\*SuperscriptBox[\(s\
\), \(-1\)]\)\[NonBreakingSpace]\!\(\*SuperscriptBox[\(T\), \(-1\)]\). The \
isotope is specified either by its mass number, or as an abbreviation, for \
example \\\"13C\\\".\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574845208885941*^9, 3.574845249654599*^9}, {
   3.5748454715280247`*^9, 3.5748454973209248`*^9}, 3.574846275005172*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["NaturalAbundance", "Subsection",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{3.574846275004548*^9, 3.5748464247703857`*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NaturalAbundance", "::", "usage"}], " ", "=", " ", 
   "\"\<NaturalAbundance[isotope] gives the natural abundance of the \
specified isotope. The isotope is specified either by its mass number, or as \
an abbreviation, for example \\\"13C\\\".\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574845208885941*^9, 3.574845249654599*^9}, {
   3.5748454715280247`*^9, 3.5748454973209248`*^9}, 3.574846275005172*^9, {
   3.57484642672023*^9, 3.574846438149652*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["GyromagneticRatio", "Subsection",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{3.409834534118566*^9, 3.5748462750055532`*^9},
 AspectRatioFixed->True],

Cell["GyromagneticRatio::usage = \"same as MagnetogyricRatio\";", "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.409834535361435*^9, 3.409834552787219*^9}, 
   3.57484627500591*^9},
 AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["LarmorFrequency", "Subsection",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{3.57484627500627*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"LarmorFrequency", "::", "usage"}], " ", "=", " ", 
   "\"\<LarmorFrequency[isotope,\!\(\*SubscriptBox[\(B\), \(0\)]\)] gives the \
Larmor frequency of the specified isotope in the field \!\(\*SubscriptBox[\(B\
\), \(0\)]\) [specified in Tesla], in units of \
rad\[NonBreakingSpace]\!\(\*SuperscriptBox[\(s\), \(-1\)]\). The isotope is \
specified either by its mass number, or as an abbreviation, for example \
\\\"13C\\\".\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574845208885941*^9, 3.5748453012209806`*^9}, {
   3.574845507793983*^9, 3.574845512876615*^9}, 3.574846275006874*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["DirectDipolarCoupling", "Subsection",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{3.574846275007186*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DirectDipolarCoupling", "::", "usage"}], " ", "=", " ", 
   "\"\<DirectDipolarCoupling[{isotope1,isotope2},r] gives the dipolar \
coupling (in rad\[NonBreakingSpace]\!\(\*SuperscriptBox[\(s\), \(-1\)]\)) \
between the nuclei of two isotopes separated by r (in meters). \
DirectDipolarCoupling[isotope1,r] assumes two identical isotopes. Each \
isotope is specified either by its mass number, or as an abbreviation, for \
example \\\"13C\\\".\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574845315587294*^9, 3.574845371672052*^9}, {
   3.5748455257542*^9, 3.574845529134532*^9}, 3.574846275007721*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["main", "Subsubtitle",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.574846292735124*^9, 3.574846293156372*^9}},
 AspectRatioFixed->True],

Cell["Begin[\"`Private`\"];", "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Isotope", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5747642001488857`*^9, 3.574764223301285*^9}, 
   3.574765542974821*^9}],

Cell[CellGroupData[{

Cell["IsotopicData", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.5747642001488857`*^9, 3.574764207196454*^9}, 
   3.574765600443101*^9, {3.5747656651472473`*^9, 3.5747656821671658`*^9}, 
   3.574844583008374*^9}],

Cell["Values compiled by Soumya Singha Roy", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.574844931574904*^9, 3.574844939395381*^9}},
 Background->RGBColor[1, 1, 0]],

Cell["correction to \[Gamma] for 1H, 113In and 115In in v2.7.1b1", "Text",
 CellChangeTimes->{{3.582610632705473*^9, 3.5826106478649054`*^9}, {
  3.582865694091352*^9, 3.582865715815958*^9}},
 Background->RGBColor[0.87, 0.94, 1]],

Cell[BoxData[
 RowBox[{
  RowBox[{"IsotopicData", "=", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
       ",", 
       RowBox[{"#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
       ",", 
       RowBox[{
        RowBox[{"#", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}], 
        "\[Times]", 
        SuperscriptBox["10", "7"]}], ",", 
       RowBox[{"#", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
       ",", 
       RowBox[{
        RowBox[{"#", "\[LeftDoubleBracket]", "5", "\[RightDoubleBracket]"}], 
        "/", "100"}]}], "}"}], "&"}], "/@", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "\"\<1H\>\"", ",", "26.7522128", ",", 
        RowBox[{"1", "/", "2"}], ",", "99.9885"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "2", ",", "\"\<2H\>\"", ",", "4.10662791", ",", "1", ",", "0.0115"}], 
       "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"3", ",", "\"\<3H\>\"", ",", "28.5349779", ",", 
        RowBox[{"1", "/", "2"}], ",", "0"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"3", ",", "\"\<3He\>\"", ",", 
        RowBox[{"-", "20.3801587"}], ",", 
        RowBox[{"1", "/", "2"}], ",", "0.000137"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "6", ",", "\"\<6Li\>\"", ",", "3.9371709", ",", "1", ",", "7.59"}], 
       "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"7", ",", "\"\<7Li\>\"", ",", "10.3977013", ",", 
        RowBox[{"3", "/", "2"}], ",", "92.41"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"9", ",", "\"\<9Be\>\"", ",", 
        RowBox[{"-", "3.759666"}], ",", 
        RowBox[{"3", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "10", ",", "\"\<10B\>\"", ",", "2.8746786", ",", "3", ",", "19.9"}], 
       "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"11", ",", "\"\<11B\>\"", ",", "8.5847044", ",", 
        RowBox[{"3", "/", "2"}], ",", "80.1"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"13", ",", "\"\<13C\>\"", ",", "6.728284", ",", 
        RowBox[{"1", "/", "2"}], ",", "1.07"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "14", ",", "\"\<14N\>\"", ",", "1.9337792", ",", "1", ",", "99.632"}], 
       "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"15", ",", "\"\<15N\>\"", ",", 
        RowBox[{"-", "2.71261804"}], ",", 
        RowBox[{"1", "/", "2"}], ",", "0.368"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"17", ",", "\"\<17O\>\"", ",", 
        RowBox[{"-", "3.62808"}], ",", 
        RowBox[{"5", "/", "2"}], ",", "0.038"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"19", ",", "\"\<19F\>\"", ",", "25.18148", ",", 
        RowBox[{"1", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"21", ",", "\"\<21Ne\>\"", ",", 
        RowBox[{"-", "2.11308"}], ",", 
        RowBox[{"3", "/", "2"}], ",", "0.27"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"23", ",", "\"\<23Na\>\"", ",", "7.0808493", ",", 
        RowBox[{"3", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"25", ",", "\"\<25Mg\>\"", ",", 
        RowBox[{"-", "1.63887"}], ",", 
        RowBox[{"5", "/", "2"}], ",", "10.00"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"27", ",", "\"\<27Al\>\"", ",", "6.9762715", ",", 
        RowBox[{"5", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"29", ",", "\"\<29Si\>\"", ",", 
        RowBox[{"-", "5.3190"}], ",", 
        RowBox[{"1", "/", "2"}], ",", "4.6832"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"31", ",", "\"\<31P\>\"", ",", "10.8394", ",", 
        RowBox[{"1", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"33", ",", "\"\<33S\>\"", ",", "2.055685", ",", 
        RowBox[{"3", "/", "2"}], ",", "0.76"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"35", ",", "\"\<35Cl\>\"", ",", "2.624198", ",", 
        RowBox[{"3", "/", "2"}], ",", "75.77"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"37", ",", "\"\<37Cl\>\"", ",", "2.184368", ",", 
        RowBox[{"3", "/", "2"}], ",", "24.23"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"39", ",", "\"\<39K\>\"", ",", "1.2500608", ",", 
        RowBox[{"3", "/", "2"}], ",", "93.2581"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"40", ",", "\"\<40K\>\"", ",", 
        RowBox[{"-", "1.5542854"}], ",", "4", ",", "0.0117"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"41", ",", "\"\<41K\>\"", ",", "0.68606808", ",", 
        RowBox[{"3", "/", "2"}], ",", "6.7302"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"43", ",", "\"\<43Ca\>\"", ",", 
        RowBox[{"-", "1.803069"}], ",", 
        RowBox[{"7", "/", "2"}], ",", "0.135"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"45", ",", "\"\<45Sc\>\"", ",", "6.5087973", ",", 
        RowBox[{"7", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"47", ",", "\"\<47Ti\>\"", ",", 
        RowBox[{"-", "1.5105"}], ",", 
        RowBox[{"5", "/", "2"}], ",", "7.44"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"49", ",", "\"\<49Ti\>\"", ",", 
        RowBox[{"-", "1.51095"}], ",", 
        RowBox[{"7", "/", "2"}], ",", "5.41"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "50", ",", "\"\<50V\>\"", ",", "2.6706490", ",", "6", ",", "0.250"}], 
       "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"51", ",", "\"\<51V\>\"", ",", "7.0455117", ",", 
        RowBox[{"7", "/", "2"}], ",", "99.750"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"53", ",", "\"\<53Cr\>\"", ",", 
        RowBox[{"-", "1.5152"}], ",", 
        RowBox[{"3", "/", "2"}], ",", "9.501"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"55", ",", "\"\<55Mn\>\"", ",", "6.6452546", ",", 
        RowBox[{"5", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"57", ",", "\"\<57Fe\>\"", ",", "0.8680624", ",", 
        RowBox[{"1", "/", "2"}], ",", "2.2"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"59", ",", "\"\<59Co\>\"", ",", "6.332", ",", 
        RowBox[{"7", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"61", ",", "\"\<61Ni\>\"", ",", 
        RowBox[{"-", "2.3948"}], ",", 
        RowBox[{"3", "/", "2"}], ",", "1.140"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"63", ",", "\"\<63Cu\>\"", ",", "7.1117890", ",", 
        RowBox[{"3", "/", "2"}], ",", "69.17"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"65", ",", "\"\<65Cu\>\"", ",", "7.60435", ",", 
        RowBox[{"3", "/", "2"}], ",", "30.83"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"67", ",", "\"\<67Zn\>\"", ",", "1.676688", ",", 
        RowBox[{"5", "/", "2"}], ",", "4.1"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"69", ",", "\"\<69Ga\>\"", ",", "6.438855", ",", 
        RowBox[{"3", "/", "2"}], ",", "60.108"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"71", ",", "\"\<71Ga\>\"", ",", "8.181171", ",", 
        RowBox[{"3", "/", "2"}], ",", "39.892"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"73", ",", "\"\<73Ge\>\"", ",", 
        RowBox[{"-", "0.9360303"}], ",", 
        RowBox[{"9", "/", "2"}], ",", "7.73"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"75", ",", "\"\<75As\>\"", ",", "4.596163", ",", 
        RowBox[{"3", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"77", ",", "\"\<77Se\>\"", ",", "5.1253857", ",", 
        RowBox[{"1", "/", "2"}], ",", "7.63"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"79", ",", "\"\<79Br\>\"", ",", "6.725616", ",", 
        RowBox[{"3", "/", "2"}], ",", "50.69"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"81", ",", "\"\<81Br\>\"", ",", "7.249776", ",", 
        RowBox[{"3", "/", "2"}], ",", "49.31"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"83", ",", "\"\<83Kr\>\"", ",", 
        RowBox[{"-", "1.03310"}], ",", 
        RowBox[{"9", "/", "2"}], ",", "11.5"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"85", ",", "\"\<85Rb\>\"", ",", "2.5927050", ",", 
        RowBox[{"5", "/", "2"}], ",", "72.165"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"87", ",", "\"\<87Rb\>\"", ",", "8.786400", ",", 
        RowBox[{"3", "/", "2"}], ",", "27.835"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"87", ",", "\"\<87Sr\>\"", ",", 
        RowBox[{"-", "1.1639376"}], ",", 
        RowBox[{"9", "/", "2"}], ",", "7.00"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"89", ",", "\"\<89Y\>\"", ",", 
        RowBox[{"-", "1.3162791"}], ",", 
        RowBox[{"1", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"91", ",", "\"\<91Zr\>\"", ",", 
        RowBox[{"-", "2.49743"}], ",", 
        RowBox[{"5", "/", "2"}], ",", "11.22"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"93", ",", "\"\<93Nb\>\"", ",", "6.5674", ",", 
        RowBox[{"9", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"95", ",", "\"\<95Mo\>\"", ",", 
        RowBox[{"-", "1.751"}], ",", 
        RowBox[{"5", "/", "2"}], ",", "15.92"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"97", ",", "\"\<97Mo\>\"", ",", 
        RowBox[{"-", "1.788"}], ",", 
        RowBox[{"5", "/", "2"}], ",", "9.55"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"99", ",", "\"\<99Tc\>\"", ",", "6.046", ",", 
        RowBox[{"9", "/", "2"}], ",", "0"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"99", ",", "\"\<99Ru\>\"", ",", 
        RowBox[{"-", "1.229"}], ",", 
        RowBox[{"5", "/", "2"}], ",", "12.7"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"101", ",", "\"\<101Ru\>\"", ",", 
        RowBox[{"-", "1.377"}], ",", 
        RowBox[{"5", "/", "2"}], ",", "17"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"103", ",", "\"\<103Rh\>\"", ",", 
        RowBox[{"-", "0.8468"}], ",", 
        RowBox[{"1", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"105", ",", "\"\<105Pd\>\"", ",", 
        RowBox[{"-", "1.23"}], ",", 
        RowBox[{"5", "/", "2"}], ",", "22.33"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"107", ",", "\"\<107Ag\>\"", ",", 
        RowBox[{"-", "1.0889181"}], ",", 
        RowBox[{"1", "/", "2"}], ",", "51.839"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"109", ",", "\"\<109Ag\>\"", ",", 
        RowBox[{"-", "1.2518634"}], ",", 
        RowBox[{"1", "/", "2"}], ",", "48.161"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"111", ",", "\"\<111Cd\>\"", ",", 
        RowBox[{"-", "5.6983131"}], ",", 
        RowBox[{"1", "/", "2"}], ",", "12.8"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"113", ",", "\"\<113Cd\>\"", ",", 
        RowBox[{"-", "5.9609153"}], ",", 
        RowBox[{"1", "/", "2"}], ",", "12.22"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"113", ",", "\"\<113In\>\"", ",", " ", "5.8845", ",", 
        RowBox[{"9", "/", "2"}], ",", "4.3"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"115", ",", "\"\<115In\>\"", ",", "5.8972", ",", 
        RowBox[{"9", "/", "2"}], ",", "95.7"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"115", ",", "\"\<115Sn\>\"", ",", 
        RowBox[{"-", "8.8013"}], ",", 
        RowBox[{"1", "/", "2"}], ",", "0.34"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"117", ",", "\"\<117Sn\>\"", ",", 
        RowBox[{"-", "9.58879"}], ",", 
        RowBox[{"1", "/", "2"}], ",", "7.68"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"119", ",", "\"\<119Sn\>\"", ",", 
        RowBox[{"-", "10.0317"}], ",", 
        RowBox[{"1", "/", "2"}], ",", "8.59"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"121", ",", "\"\<121Sb\>\"", ",", "6.4435", ",", 
        RowBox[{"5", "/", "2"}], ",", "57.36"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"123", ",", "\"\<123Sb\>\"", ",", "3.4892", ",", 
        RowBox[{"7", "/", "2"}], ",", "42.64"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"123", ",", "\"\<123Te\>\"", ",", 
        RowBox[{"-", "7.059098"}], ",", 
        RowBox[{"1", "/", "2"}], ",", "0.908"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"125", ",", "\"\<125Te\>\"", ",", 
        RowBox[{"-", "8.5108404"}], ",", 
        RowBox[{"1", "/", "2"}], ",", "7.139"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"127", ",", "\"\<127I\>\"", ",", "5.389573", ",", 
        RowBox[{"5", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"129", ",", "\"\<129Xe\>\"", ",", 
        RowBox[{"-", "7.452103"}], ",", 
        RowBox[{"1", "/", "2"}], ",", "26.4"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"131", ",", "\"\<131Xe\>\"", ",", "2.209076", ",", 
        RowBox[{"3", "/", "2"}], ",", "21.2"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"133", ",", "\"\<133Cs\>\"", ",", "3.5332539", ",", 
        RowBox[{"7", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"135", ",", "\"\<135Ba\>\"", ",", "2.65750", ",", 
        RowBox[{"3", "/", "2"}], ",", "6.592"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"137", ",", "\"\<137Ba\>\"", ",", "2.99295", ",", 
        RowBox[{"3", "/", "2"}], ",", "11.23"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "138", ",", "\"\<138La\>\"", ",", "3.557239", ",", "5", ",", 
        "0.0902"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"139", ",", "\"\<139La\>\"", ",", "3.8083318", ",", 
        RowBox[{"7", "/", "2"}], ",", "99.9098"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"141", ",", "\"\<141Pr\>\"", ",", "8.1907", ",", 
        RowBox[{"5", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"143", ",", "\"\<143Nd\>\"", ",", 
        RowBox[{"-", "1.457"}], ",", 
        RowBox[{"7", "/", "2"}], ",", "12.18"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"145", ",", "\"\<145Nd\>\"", ",", 
        RowBox[{"-", "0.898"}], ",", 
        RowBox[{"7", "/", "2"}], ",", "8.30"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"147", ",", "\"\<147Pm\>\"", ",", "3.613", ",", 
        RowBox[{"7", "/", "2"}], ",", "0"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"147", ",", "\"\<147Sm\>\"", ",", 
        RowBox[{"-", "1.115"}], ",", 
        RowBox[{"7", "/", "2"}], ",", "15.0"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"149", ",", "\"\<149Sm\>\"", ",", 
        RowBox[{"-", "0.9192"}], ",", 
        RowBox[{"7", "/", "2"}], ",", "13.8"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"151", ",", "\"\<151Eu\>\"", ",", "6.6510", ",", 
        RowBox[{"5", "/", "2"}], ",", "47.8"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"153", ",", "\"\<153Eu\>\"", ",", "2.9369", ",", 
        RowBox[{"5", "/", "2"}], ",", "52.2"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"155", ",", "\"\<155Gd\>\"", ",", 
        RowBox[{"-", "0.82132"}], ",", 
        RowBox[{"3", "/", "2"}], ",", "14.8"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"157", ",", "\"\<157Gd\>\"", ",", 
        RowBox[{"-", "1.0769"}], ",", 
        RowBox[{"3", "/", "2"}], ",", "15.65"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"159", ",", "\"\<159Tb\>\"", ",", "6.431", ",", 
        RowBox[{"3", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"161", ",", "\"\<161Dy\>\"", ",", 
        RowBox[{"-", "0.9201"}], ",", 
        RowBox[{"5", "/", "2"}], ",", "18.9"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"163", ",", "\"\<163Dy\>\"", ",", "1.289", ",", 
        RowBox[{"5", "/", "2"}], ",", "24.9"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"165", ",", "\"\<165Ho\>\"", ",", "5.710", ",", 
        RowBox[{"7", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"167", ",", "\"\<167Er\>\"", ",", 
        RowBox[{"-", "0.77157"}], ",", 
        RowBox[{"7", "/", "2"}], ",", "22.95"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"169", ",", "\"\<169Tm\>\"", ",", 
        RowBox[{"-", "2.218"}], ",", 
        RowBox[{"1", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"171", ",", "\"\<171Yb\>\"", ",", "4.7288", ",", 
        RowBox[{"1", "/", "2"}], ",", "14.3"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"173", ",", "\"\<173Yb\>\"", ",", 
        RowBox[{"-", "1.3025"}], ",", 
        RowBox[{"5", "/", "2"}], ",", "16.12"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"175", ",", "\"\<175Lu\>\"", ",", "3.0552", ",", 
        RowBox[{"7", "/", "2"}], ",", "97.41"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "176", ",", "\"\<176Lu\>\"", ",", "2.1684", ",", "7", ",", "2.59"}], 
       "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"177", ",", "\"\<177Hf\>\"", ",", "1.086", ",", 
        RowBox[{"7", "/", "2"}], ",", "18.606"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"179", ",", "\"\<179Hf\>\"", ",", 
        RowBox[{"-", "0.6821"}], ",", 
        RowBox[{"9", "/", "2"}], ",", "13.629"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"181", ",", "\"\<181Ta\>\"", ",", "3.2438", ",", 
        RowBox[{"7", "/", "2"}], ",", "99.988"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"183", ",", "\"\<183W\>\"", ",", "1.1282403", ",", 
        RowBox[{"1", "/", "2"}], ",", "14.3"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"185", ",", "\"\<185Re\>\"", ",", "6.1057", ",", 
        RowBox[{"5", "/", "2"}], ",", "37.40"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"187", ",", "\"\<187Re\>\"", ",", "6.1682", ",", 
        RowBox[{"5", "/", "2"}], ",", "62.60"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"187", ",", "\"\<187Os\>\"", ",", "0.6192895", ",", 
        RowBox[{"1", "/", "2"}], ",", "1.6"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"189", ",", "\"\<189Os\>\"", ",", "2.10713", ",", 
        RowBox[{"3", "/", "2"}], ",", "16.1"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"191", ",", "\"\<191Ir\>\"", ",", "0.4812", ",", 
        RowBox[{"3", "/", "2"}], ",", "37.3"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"193", ",", "\"\<193Ir\>\"", ",", "0.5227", ",", 
        RowBox[{"3", "/", "2"}], ",", "62.7"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"195", ",", "\"\<195Pt\>\"", ",", "5.8385", ",", 
        RowBox[{"1", "/", "2"}], ",", "33.8"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"197", ",", "\"\<197Au\>\"", ",", "0.473060", ",", 
        RowBox[{"3", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"199", ",", "\"\<199Hg\>\"", ",", "4.8457916", ",", 
        RowBox[{"1", "/", "2"}], ",", "16.87"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"201", ",", "\"\<201Hg\>\"", ",", 
        RowBox[{"-", "1.788769"}], ",", 
        RowBox[{"3", "/", "2"}], ",", "13.18"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"203", ",", "\"\<203Tl\>\"", ",", "15.5393338", ",", 
        RowBox[{"1", "/", "2"}], ",", "29.524"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"205", ",", "\"\<205Tl\>\"", ",", "15.6921808", ",", 
        RowBox[{"1", "/", "2"}], ",", "70.476"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"207", ",", "\"\<207Pb\>\"", ",", "5.58046", ",", 
        RowBox[{"1", "/", "2"}], ",", "22.1"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"209", ",", "\"\<209Bi\>\"", ",", "4.3750", ",", 
        RowBox[{"9", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"209", ",", "\"\<209Po\>\"", ",", "7.4", ",", 
        RowBox[{"1", "/", "2"}], ",", "0"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"227", ",", "\"\<227Ac\>\"", ",", "3.5", ",", 
        RowBox[{"3", "/", "2"}], ",", "0"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"229", ",", "\"\<229Th\>\"", ",", "0.40", ",", 
        RowBox[{"5", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"231", ",", "\"\<231Pa\>\"", ",", "3.21", ",", 
        RowBox[{"3", "/", "2"}], ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"235", ",", "\"\<235U\>\"", ",", 
        RowBox[{"-", "0.52"}], ",", 
        RowBox[{"7", "/", "2"}], ",", "0.7200"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"237", ",", "\"\<237Np\>\"", ",", "3.1", ",", 
        RowBox[{"5", "/", "2"}], ",", "0"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"239", ",", "\"\<239Pu\>\"", ",", "0.972", ",", 
        RowBox[{"1", "/", "2"}], ",", "0"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"241", ",", "\"\<241Am\>\"", ",", "1.54", ",", 
        RowBox[{"5", "/", "2"}], ",", "0"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"243", ",", "\"\<243Am\>\"", ",", "1.54", ",", 
        RowBox[{"5", "/", "2"}], ",", "0"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"247", ",", "\"\<247Cm\>\"", ",", "0.20", ",", 
        RowBox[{"9", "/", "2"}], ",", "0"}], "}"}]}], "\[IndentingNewLine]", 
     "}"}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574764127818143*^9, 3.574764146560858*^9}, 
   3.574765600512808*^9, {3.5747656652082243`*^9, 3.574765682226581*^9}, 
   3.574844583061285*^9, {3.5748466491769457`*^9, 3.5748467229989347`*^9}, 
   3.582610613777966*^9, 3.582865677256068*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Isotope", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.5747642001488857`*^9, 3.574764207196454*^9}, 
   3.574765600443101*^9, {3.5747656651472473`*^9, 3.5747656821671658`*^9}, {
   3.574844567371648*^9, 3.574844568585204*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Isotope", "[", "A_Integer", "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "candidates", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"candidates", "=", 
        RowBox[{"Cases", "[", 
         RowBox[{"IsotopicData", ",", 
          RowBox[{"{", 
           RowBox[{"A", ",", "___"}], "}"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Switch", "[", 
        RowBox[{
         RowBox[{"Length", "[", "candidates", "]"}], ",", 
         "\[IndentingNewLine]", "0", ",", "\[IndentingNewLine]", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Message", "[", 
            RowBox[{
             RowBox[{"Isotope", "::", "none"}], ",", "A"}], "]"}], ";", 
           "$Failed"}], ")"}], ",", "\[IndentingNewLine]", "1", ",", 
         RowBox[{"candidates", "\[LeftDoubleBracket]", 
          RowBox[{"1", ",", "2"}], "\[RightDoubleBracket]"}], ",", 
         "\[IndentingNewLine]", "_", ",", "\[IndentingNewLine]", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Message", "[", 
            RowBox[{
             RowBox[{"Isotope", "::", "dup"}], ",", "A", ",", 
             RowBox[{
              RowBox[{
               RowBox[{"(", 
                RowBox[{"\"\<\\\"\>\"", "<>", 
                 RowBox[{
                 "#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
                 "<>", "\"\<\\\"\>\""}], ")"}], "&"}], "/@", "candidates"}]}],
             "]"}], ";", 
           RowBox[{
            RowBox[{
             RowBox[{"Part", "[", 
              RowBox[{"#", ",", "2"}], "]"}], "&"}], "/@", "candidates"}]}], 
          ")"}]}], "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", 
      ")"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5747642262896013`*^9, 3.574764369218368*^9}, {
   3.5747646985663147`*^9, 3.5747648187276487`*^9}, {3.574764900061468*^9, 
   3.574764925519422*^9}, {3.574765546963183*^9, 3.574765562385747*^9}, 
   3.574765600552322*^9, {3.5747656650221863`*^9, 3.574765682030464*^9}, 
   3.574844387142437*^9, 3.574844583064514*^9, {3.575981486370075*^9, 
   3.575981490551104*^9}, {3.575981636971472*^9, 3.575981639953856*^9}, {
   3.575981686697935*^9, 3.5759817136173162`*^9}, {3.575981880441394*^9, 
   3.575981884053192*^9}, {3.5759819194073057`*^9, 3.575981922668648*^9}, {
   3.575981958439103*^9, 3.575981963547402*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Isotope", "[", "isotope_String", "]"}], ":=", "isotope"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574844377633974*^9, 3.574844383104603*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Isotope", "::", "none"}], "=", 
   "\"\<No isotope found with mass number `1`\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574764730015553*^9, 3.574764743201384*^9}, 
   3.574765563149557*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Isotope", "::", "dup"}], "=", 
   "\"\<The mass number `1` is ambiguous. Specify one the following isotopes \
explicitly: `2`\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574764831821876*^9, 3.57476487523945*^9}, 
   3.57476556361351*^9, {3.5759814989322557`*^9, 3.575981532791316*^9}, {
   3.5759815773793163`*^9, 3.5759816729997063`*^9}, {3.575981718309223*^9, 
   3.575981720685902*^9}, 3.5759818406785393`*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["GyromagneticRatio", "Section",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.4098345871341047`*^9, 3.409834595998296*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"GyromagneticRatio", "[", 
    RowBox[{"$Failed", "|", "_List"}], "]"}], "=", "$Failed"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5747650628034153`*^9, 3.574765069336191*^9}, 
   3.5759818175360622`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"GyromagneticRatio", "[", "A_Integer", "]"}], ":=", 
   RowBox[{"GyromagneticRatio", "[", 
    RowBox[{"Isotope", "[", "A", "]"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.57476417977684*^9, 3.574764187008339*^9}, {
   3.5747650550311003`*^9, 3.574765078651126*^9}, 3.574765568321855*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"GyromagneticRatio", "[", "isotope_String", "]"}], ":=", 
   RowBox[{"(", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"GyromagneticRatio", "[", "isotope", "]"}], "=", 
     RowBox[{
      RowBox[{"Cases", "[", 
       RowBox[{"IsotopicData", ",", 
        RowBox[{"{", 
         RowBox[{"_", ",", "isotope", ",", "___"}], "}"}]}], "]"}], 
      "\[LeftDoubleBracket]", 
      RowBox[{"1", ",", "3"}], "\[RightDoubleBracket]"}]}], 
    "\[IndentingNewLine]", ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574765122247364*^9, 3.574765127399419*^9}, {
   3.574765157915448*^9, 3.574765203699493*^9}, 3.574765600555113*^9, {
   3.574765665039723*^9, 3.57476568203405*^9}, 3.574844583091332*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["MagnetogyricRatio", "Section",
 PageWidth->WindowWidth,
 InitializationCell->True,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"MagnetogyricRatio", "[", "n___", "]"}], ":=", 
  RowBox[{"GyromagneticRatio", "[", "n", "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574765091396553*^9, 3.574765108604973*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["NuclearSpinQuantumNumber", "Section",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{3.409834799997674*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NuclearSpinQuantumNumber", "[", 
    RowBox[{"$Failed", "|", "_List"}], "]"}], "=", "$Failed"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5747650628034153`*^9, 3.574765069336191*^9}, 
   3.574765233916112*^9, 3.5759818127240133`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NuclearSpinQuantumNumber", "[", "A_Integer", "]"}], ":=", 
   RowBox[{"NuclearSpinQuantumNumber", "[", 
    RowBox[{"Isotope", "[", "A", "]"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.57476417977684*^9, 3.574764187008339*^9}, {
   3.5747650550311003`*^9, 3.574765078651126*^9}, {3.574765236364189*^9, 
   3.57476523762814*^9}, 3.574765568343471*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NuclearSpinQuantumNumber", "[", "isotope_String", "]"}], ":=", 
   RowBox[{"(", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"NuclearSpinQuantumNumber", "[", "isotope", "]"}], "=", 
     RowBox[{
      RowBox[{"Cases", "[", 
       RowBox[{"IsotopicData", ",", 
        RowBox[{"{", 
         RowBox[{"_", ",", "isotope", ",", "___"}], "}"}]}], "]"}], 
      "\[LeftDoubleBracket]", 
      RowBox[{"1", ",", "4"}], "\[RightDoubleBracket]"}]}], 
    "\[IndentingNewLine]", ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574765122247364*^9, 3.574765127399419*^9}, {
   3.574765157915448*^9, 3.574765203699493*^9}, {3.5747652403164577`*^9, 
   3.574765249726768*^9}, 3.574765600590596*^9, {3.5747656650416193`*^9, 
   3.574765682036717*^9}, 3.574844583146666*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["NaturalAbundance", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.574764158258441*^9, 3.574764176234552*^9}, {
  3.574765220547174*^9, 3.574765261156146*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NaturalAbundance", "[", 
    RowBox[{"$Failed", "|", "_List"}], "]"}], "=", "$Failed"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5747650628034153`*^9, 3.574765069336191*^9}, 
   3.574765233916112*^9, 3.574765264806384*^9, 3.575981807488811*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NaturalAbundance", "[", "A_Integer", "]"}], ":=", 
   RowBox[{"NaturalAbundance", "[", 
    RowBox[{"Isotope", "[", "A", "]"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.57476417977684*^9, 3.574764187008339*^9}, {
   3.5747650550311003`*^9, 3.574765078651126*^9}, {3.574765236364189*^9, 
   3.574765267909399*^9}, 3.5747655683591013`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NaturalAbundance", "[", "isotope_String", "]"}], ":=", 
   RowBox[{"(", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"NaturalAbundance", "[", "isotope", "]"}], "=", 
     RowBox[{
      RowBox[{"Cases", "[", 
       RowBox[{"IsotopicData", ",", 
        RowBox[{"{", 
         RowBox[{"_", ",", "isotope", ",", "___"}], "}"}]}], "]"}], 
      "\[LeftDoubleBracket]", 
      RowBox[{"1", ",", "5"}], "\[RightDoubleBracket]"}]}], 
    "\[IndentingNewLine]", ")"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574765122247364*^9, 3.574765127399419*^9}, {
   3.574765157915448*^9, 3.574765203699493*^9}, {3.5747652403164577`*^9, 
   3.574765274032291*^9}, {3.574765308258567*^9, 3.574765308810158*^9}, 
   3.574765600598522*^9, {3.574765665043458*^9, 3.574765682039383*^9}, 
   3.574844583197673*^9, {3.574846739464098*^9, 3.574846740076367*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["LarmorFrequency", "Section",
 PageWidth->WindowWidth,
 InitializationCell->True,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"LarmorFrequency", "[", 
    RowBox[{
     RowBox[{"$Failed", "|", "_List"}], ",", "___"}], "]"}], ":=", 
   "$Failed"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574844663031719*^9, 3.574844671358199*^9}, 
   3.575981802411833*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"LarmorFrequency", "[", 
    RowBox[{"A_Integer", ",", "Bzero_"}], "]"}], ":=", "\[IndentingNewLine]", 
   
   RowBox[{"LarmorFrequency", "[", 
    RowBox[{
     RowBox[{"Isotope", "[", "A", "]"}], ",", "Bzero"}], "]"}]}], 
  ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.574844648426804*^9, 3.574844657672716*^9}, {
  3.574846878711328*^9, 3.574846901412388*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"LarmorFrequency", "[", 
    RowBox[{"isotope_String", ",", "Bzero_"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"MagnetogyricRatio", "[", "isotope", "]"}]}], "*", "Bzero"}]}], 
  ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 CellChangeTimes->{{3.574844648426804*^9, 3.574844657672716*^9}, {
  3.574846906124483*^9, 3.5748469070208273`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["DirectDipolarCoupling", "Section",
 PageWidth->WindowWidth,
 InitializationCell->True,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DirectDipolarCoupling", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"$Failed", "|", "_List"}], ",", "_"}], "}"}], ",", "___"}], 
    "]"}], ":=", "$Failed"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574844689671392*^9, 3.5748447050770473`*^9}, {
  3.575981789374339*^9, 3.57598179091525*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DirectDipolarCoupling", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"_", ",", 
       RowBox[{"$Failed", "|", "_List"}]}], "}"}], ",", "___"}], "]"}], ":=", 
   "$Failed"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574844689671392*^9, 3.574844713813635*^9}, 
   3.5748465416391287`*^9, {3.575981795910624*^9, 3.575981797731183*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DirectDipolarCoupling", "[", 
    RowBox[{
     RowBox[{"iso1", ":", 
      RowBox[{"Except", "[", "_List", "]"}]}], ",", "r_"}], "]"}], " ", ":=", 
   
   RowBox[{"DirectDipolarCoupling", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"iso1", ",", "iso1"}], "}"}], ",", "r"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574844768308004*^9, 3.574844804731141*^9}, 
   3.5748465709919357`*^9, {3.57484718509527*^9, 3.5748472008829613`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DirectDipolarCoupling", "[", 
    RowBox[{"iso1_", ",", "iso2_", ",", "r_"}], "]"}], " ", ":=", 
   RowBox[{"DirectDipolarCoupling", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"iso1", ",", "iso2"}], "}"}], ",", "r"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574844768308004*^9, 3.5748448065953827`*^9}, {
  3.5748472130380373`*^9, 3.574847214914184*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DirectDipolarCoupling", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"A1_Integer", ",", "iso2_"}], "}"}], ",", "r_"}], "]"}], " ", ":=", 
   RowBox[{"DirectDipolarCoupling", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Isotope", "[", "A1", "]"}], ",", "iso2"}], "}"}], ",", "r"}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574844768308004*^9, 3.5748448065953827`*^9}, {
  3.574846947805442*^9, 3.574846959219696*^9}, {3.574847224755772*^9, 
  3.574847227147352*^9}, {3.574847318729373*^9, 3.57484732045293*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DirectDipolarCoupling", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"iso1_", ",", "A2_Integer"}], "}"}], ",", "r_"}], "]"}], " ", ":=", 
   RowBox[{"DirectDipolarCoupling", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"iso1", ",", 
       RowBox[{"Isotope", "[", "A2", "]"}]}], "}"}], ",", "r"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574844768308004*^9, 3.5748448065953827`*^9}, {
  3.574846947805442*^9, 3.574846991602766*^9}, {3.574847230206105*^9, 
  3.57484723281124*^9}, {3.57484732301355*^9, 3.5748473244049*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DirectDipolarCoupling", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"iso1_String", ",", "iso2_String"}], "}"}], ",", "r_"}], "]"}], 
   " ", ":=", 
   RowBox[{
    RowBox[{"N", "[", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{"\[Mu]0", "/", 
         RowBox[{"(", 
          RowBox[{"4", "\[Pi]"}], ")"}]}], ")"}]}], "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"MagnetogyricRatio", "[", "iso1", "]"}], "*", 
          RowBox[{"MagnetogyricRatio", "[", "iso2", "]"}], "*", "\[HBar]"}], 
         ")"}], "/", 
        RowBox[{"(", 
         RowBox[{"r", "^", "3"}], ")"}]}], ")"}]}], "]"}], "/.", 
    "PhysicalConstantValues"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.574844746324307*^9, 3.574844759976709*^9}, {
  3.574846996575931*^9, 3.574847000963078*^9}, {3.575612938343614*^9, 
  3.5756129392601423`*^9}, {3.596886204846698*^9, 3.59688622740038*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["end", "Subsubtitle",
 PageWidth->WindowWidth,
 InitializationCell->True,
 AspectRatioFixed->True],

Cell["\<\
End[];
EndPackage[];\
\>", "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 AspectRatioFixed->True]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->Automatic,
WindowSize->{740, 867},
WindowMargins->{{Automatic, 321}, {Automatic, 0}},
FrontEndVersion->"11.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (July 28, \
2016)",
StyleDefinitions->"MHL-StyleSheet2.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 288, 4, 92, "Title",
 InitializationCell->True],
Cell[870, 28, 1056, 21, 175, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[1951, 53, 173, 4, 39, "Subsubtitle",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[2149, 61, 246, 5, 44, "Subsection",
 InitializationCell->True],
Cell[2398, 68, 351, 8, 63, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[2786, 81, 215, 5, 36, "Subsection",
 InitializationCell->True],
Cell[3004, 88, 551, 10, 80, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[3592, 103, 158, 4, 36, "Subsection",
 InitializationCell->True],
Cell[3753, 109, 620, 11, 99, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[4410, 125, 181, 4, 36, "Subsection",
 InitializationCell->True],
Cell[4594, 131, 533, 10, 80, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[5164, 146, 182, 4, 36, "Subsection",
 InitializationCell->True],
Cell[5349, 152, 242, 5, 29, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[5628, 162, 155, 4, 36, "Subsection",
 InitializationCell->True],
Cell[5786, 168, 662, 12, 99, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[6485, 185, 162, 4, 36, "Subsection",
 InitializationCell->True],
Cell[6650, 191, 676, 12, 133, "Input",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[7375, 209, 170, 4, 39, "Subsubtitle",
 InitializationCell->True],
Cell[7548, 215, 115, 3, 48, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[7688, 222, 147, 3, 34, "Section",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[7860, 229, 234, 4, 35, "Subsubsection",
 InitializationCell->True],
Cell[8097, 235, 177, 3, 46, "Text",
 InitializationCell->True],
Cell[8277, 240, 229, 3, 46, "Text"],
Cell[8509, 245, 24935, 592, 2279, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[33481, 842, 253, 4, 35, "Subsubsection",
 InitializationCell->True],
Cell[33737, 848, 2506, 56, 216, "Input",
 InitializationCell->True],
Cell[36246, 906, 218, 6, 28, "Input",
 InitializationCell->True],
Cell[36467, 914, 268, 7, 28, "Input",
 InitializationCell->True],
Cell[36738, 923, 490, 10, 63, "Input",
 InitializationCell->True]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[37277, 939, 181, 4, 34, "Section",
 InitializationCell->True],
Cell[37461, 945, 279, 8, 31, "Input",
 InitializationCell->True],
Cell[37743, 955, 368, 8, 28, "Input",
 InitializationCell->True],
Cell[38114, 965, 771, 18, 63, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[38922, 988, 113, 3, 34, "Section",
 InitializationCell->True],
Cell[39038, 993, 234, 5, 28, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[39309, 1003, 162, 4, 34, "Section",
 InitializationCell->True],
Cell[39474, 1009, 308, 8, 31, "Input",
 InitializationCell->True],
Cell[39785, 1019, 431, 9, 28, "Input",
 InitializationCell->True],
Cell[40219, 1030, 840, 19, 80, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[41096, 1054, 177, 3, 34, "Section",
 InitializationCell->True],
Cell[41276, 1059, 320, 8, 31, "Input",
 InitializationCell->True],
Cell[41599, 1069, 418, 9, 28, "Input",
 InitializationCell->True],
Cell[42020, 1080, 918, 20, 63, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[42975, 1105, 111, 3, 34, "Section",
 InitializationCell->True],
Cell[43089, 1110, 303, 9, 31, "Input",
 InitializationCell->True],
Cell[43395, 1121, 455, 13, 46, "Input",
 InitializationCell->True],
Cell[43853, 1136, 454, 13, 46, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[44344, 1154, 117, 3, 34, "Section",
 InitializationCell->True],
Cell[44464, 1159, 389, 11, 50, "Input",
 InitializationCell->True],
Cell[44856, 1172, 412, 11, 50, "Input",
 InitializationCell->True],
Cell[45271, 1185, 526, 15, 70, "Input",
 InitializationCell->True],
Cell[45800, 1202, 448, 12, 70, "Input",
 InitializationCell->True],
Cell[46251, 1216, 627, 16, 70, "Input",
 InitializationCell->True],
Cell[46881, 1234, 622, 16, 70, "Input",
 InitializationCell->True],
Cell[47506, 1252, 1023, 29, 91, "Input",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[48578, 1287, 103, 3, 39, "Subsubtitle",
 InitializationCell->True],
Cell[48684, 1292, 122, 6, 66, "Input",
 InitializationCell->True]
}, Open  ]]
}, Open  ]]
}
]
*)

