(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     18539,        612]
NotebookOptionsPosition[     14623,        467]
NotebookOutlinePosition[     15313,        492]
CellTagsIndexPosition[     15270,        489]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Rotations`Quaternion`", "Title",
 PageWidth->WindowWidth,
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"BeginPackage", "[", 
   RowBox[{"\"\<Rotations`Quaternion`\>\"", ",", 
    RowBox[{"{", 
     RowBox[{
     "\"\<Global`\>\"", ",", "\"\<SpinDynamica`\>\"", ",", 
      "\"\<Rotations`Polar`\>\"", ",", "\"\<Rotations`Euler`\>\"", ",", 
      "\"\<General`General3D`\>\"", ",", "\"\<General`Miscellaneous`\>\""}], 
     "}"}]}], "]"}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.4098313382181873`*^9, {3.448289230234581*^9, 3.448289230623312*^9}, {
   3.564598286011815*^9, 3.564598286417039*^9}, {3.5645991001369658`*^9, 
   3.56459910060577*^9}, 3.683134432472591*^9},
 AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[BoxData["Exports"], "Subsubtitle",
 PageWidth->WindowWidth,
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[BoxData["Quaternions"], "Section",
 PageWidth->WindowWidth,
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->False],

Cell[CellGroupData[{

Cell[BoxData["PolarToQuaternion"], "Subsubsection",
 PageWidth->WindowWidth,
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PolarToQuaternion", "::", "\"\<usage\>\""}], "=", 
   "\"\<PolarToQuaternion[{\[Xi],\[Theta],\[Phi]}] converts a PolarRotation \
to a unit Quaternion of the form {cos(\[Xi]/2), \!\(\*SubscriptBox[\(n\), \(x\
\)]\)sin(\[Xi]/2), \!\(\*SubscriptBox[\(n\), \(y\)]\)sin(\[Xi]/2), \
\!\(\*SubscriptBox[\(n\), \(z\)]\)sin(\[Xi]/2)} where \!\(\*
StyleBox[\"n\",
FontWeight->\"Plain\"]\) is the rotation axis.\>\""}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.409831469949232*^9, 3.409831505379078*^9}, {
  3.409831595478774*^9, 3.409831595925449*^9}, {3.473576794108927*^9, 
  3.473576932389448*^9}},
 AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["QuaternionToPolar"], "Subsubsection",
 PageWidth->WindowWidth,
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuaternionToPolar", "::", "\"\<usage\>\""}], "=", 
   "\"\<QuaternionToPolar[{q1,q2,q3,q4}] converts a unit Quaternion to polar \
rotation angles {\[Xi],\[Theta],\[Phi]}. The quaternion is defined \
{cos(\[Xi]/2), \!\(\*SubscriptBox[\(n\), \(x\)]\)sin(\[Xi]/2), \
\!\(\*SubscriptBox[\(n\), \(y\)]\)sin(\[Xi]/2), \!\(\*SubscriptBox[\(n\), \(z\
\)]\)sin(\[Xi]/2)} where \!\(\*
StyleBox[\"n\",
FontWeight->\"Plain\"]\) is the rotation axis.\>\""}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.4098314741968603`*^9, 3.409831477554788*^9}, {
  3.4098315124983177`*^9, 3.409831558182248*^9}, {3.473576950392168*^9, 
  3.473576978852556*^9}},
 AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["QuaternionToEuler"], "Subsubsection",
 PageWidth->WindowWidth,
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuaternionToEuler", "::", "\"\<usage\>\""}], "=", 
   "\"\<QuaternionToEuler[{q1,q2,q3,q4}] converts a unit Quaternion to Euler \
angles {\[Alpha],\[Beta],\[Gamma]}. The quaternion is defined {cos(\[Xi]/2), \
\!\(\*SubscriptBox[\(n\), \(x\)]\)sin(\[Xi]/2), \!\(\*SubscriptBox[\(n\), \(y\
\)]\)sin(\[Xi]/2), \!\(\*SubscriptBox[\(n\), \(z\)]\)sin(\[Xi]/2)} where \!\(\*
StyleBox[\"n\",
FontWeight->\"Plain\"]\) is the rotation axis.\>\""}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.409831481223611*^9, 3.409831481584525*^9}, 
   3.409831529917913*^9, {3.409831562893943*^9, 3.4098315723006773`*^9}, {
   3.473576983849924*^9, 3.4735769906436453`*^9}},
 AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["QuaternionProduct"], "Subsubsection",
 PageWidth->WindowWidth,
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuaternionProduct", "::", "\"\<usage\>\""}], "=", 
   "\"\<QuaternionProduct[q2,q1] calculates products of quaternion \
representations of rotations.\>\""}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.409831580731275*^9, 3.409831590835524*^9}, {
  3.437733571310391*^9, 3.437733572721249*^9}, {3.473576995792325*^9, 
  3.4735769962960033`*^9}},
 AspectRatioFixed->True]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["main"], "Subsubtitle",
 PageWidth->WindowWidth,
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"Begin", "[", "\"\<`Private`\>\"", "]"}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[BoxData["Quaternions"], "Section",
 PageWidth->WindowWidth,
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->False],

Cell[CellGroupData[{

Cell[BoxData["PolarToQuaternion"], "Subsection",
 PageWidth->WindowWidth,
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PolarToQuaternion", "[", 
    RowBox[{"{", 
     RowBox[{"\[Xi]_", ",", "\[Theta]_", ",", "\[Phi]_"}], "}"}], "]"}], ":=",
    "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "nvec", "}"}], ",", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"nvec", "=", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"Sin", "[", "\[Theta]", "]"}], 
           RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
          RowBox[{
           RowBox[{"Sin", "[", "\[Theta]", "]"}], 
           RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
          RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Flatten", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Cos", "[", 
           RowBox[{"\[Xi]", "/", "2"}], "]"}], ",", 
          RowBox[{
           RowBox[{"Sin", "[", 
            RowBox[{"\[Xi]", "/", "2"}], "]"}], "nvec"}]}], "}"}], "]"}]}], 
      "\[IndentingNewLine]", ")"}]}], "]"}]}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.4098316529375553`*^9, 3.4098316628871193`*^9}, {
  3.4735770099044523`*^9, 3.473577030237793*^9}, {3.4735770618639307`*^9, 
  3.473577174174262*^9}, {3.473577370144238*^9, 3.4735773875368443`*^9}},
 AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["QuaternionProduct"], "Subsection",
 PageWidth->WindowWidth,
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"QuaternionProduct", "[", "q_", "]"}], ":=", "q"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.409831723749902*^9, 3.409831731883574*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuaternionProduct", "[", 
    RowBox[{"q2_", ",", "q1_"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"v1", ",", "v2"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"v1", "=", 
       RowBox[{"q1", "\[LeftDoubleBracket]", 
        RowBox[{"{", 
         RowBox[{"2", ",", "3", ",", "4"}], "}"}], 
        "\[RightDoubleBracket]"}]}], ";", 
      RowBox[{"v2", "=", 
       RowBox[{"q2", "\[LeftDoubleBracket]", 
        RowBox[{"{", 
         RowBox[{"2", ",", "3", ",", "4"}], "}"}], 
        "\[RightDoubleBracket]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Normalize", "[", "\[IndentingNewLine]", 
       RowBox[{"Flatten", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
            "q1", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], " ", 
            RowBox[{
            "q2", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], 
           "-", 
           RowBox[{"v1", ".", "v2"}]}], ",", 
          RowBox[{
           RowBox[{"v1", " ", 
            RowBox[{
            "q2", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], 
           "+", 
           RowBox[{
            RowBox[{
            "q1", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], " ",
             "v2"}], "-", 
           RowBox[{"v1", "\[Cross]", "v2"}]}]}], "}"}], "]"}], 
       "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.409831737961763*^9, 3.409831833684124*^9}, {
   3.409831960321128*^9, 3.4098319621309547`*^9}, 3.473577184209469*^9, {
   3.4735776218870983`*^9, 3.47357763557631*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuaternionProduct", "[", 
    RowBox[{"qn__", ",", "q2_", ",", "q1_"}], "]"}], ":=", 
   RowBox[{"QuaternionProduct", "[", 
    RowBox[{"qn", ",", 
     RowBox[{"QuaternionProduct", "[", 
      RowBox[{"q2", ",", "q1"}], "]"}]}], "]"}]}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.40983185814152*^9, 3.4098318794930983`*^9}, {
  3.409831965924315*^9, 3.409831967639208*^9}},
 AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["QuaternionToPolar"], "Subsection",
 PageWidth->WindowWidth,
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuaternionToPolar", "[", 
    RowBox[{"{", 
     RowBox[{"q1_", ",", "q2_", ",", "q3_", ",", "q4_"}], "}"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "\[Xi]", ",", "\[Theta]", ",", "\[Phi]", ",", "c", ",", "s", ",", "cth",
        ",", "sth"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"c", "=", "q1"}], ";", 
      RowBox[{"s", "=", 
       SqrtBox[
        RowBox[{
         SuperscriptBox["q2", "2"], "+", 
         SuperscriptBox["q3", "2"], "+", 
         SuperscriptBox["q4", "2"]}]]}], ";", "\[IndentingNewLine]", 
      RowBox[{"\[Xi]", "=", 
       RowBox[{"2", " ", 
        RowBox[{"ArcTan", "[", 
         RowBox[{"c", ",", "s"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"\[Theta]", "=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"q2", "\[Equal]", "q3", "\[Equal]", "q4", "\[Equal]", "0"}], 
         ",", "\[IndentingNewLine]", 
         RowBox[{"2", " ", 
          RowBox[{"ArcCos", "[", "q1", "]"}]}], ",", "\[IndentingNewLine]", 
         RowBox[{"ArcTan", "[", 
          RowBox[{"q4", ",", 
           SqrtBox[
            RowBox[{
             SuperscriptBox["q2", "2"], "+", 
             SuperscriptBox["q3", "2"]}]]}], "]"}]}], "\[IndentingNewLine]", 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"\[Phi]", "=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"q2", "\[Equal]", "q3", "\[Equal]", "0"}], ",", 
         "\[IndentingNewLine]", "0", ",", "\[IndentingNewLine]", 
         RowBox[{"ArcTan", "[", 
          RowBox[{"q2", ",", "q3"}], "]"}]}], "\[IndentingNewLine]", "]"}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"\[Xi]", ",", "\[Theta]", ",", "\[Phi]"}], "}"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.4098322361497793`*^9, 3.409832248986352*^9}, {
   3.4735772067591667`*^9, 3.473577277039583*^9}, 3.473577349922995*^9, {
   3.473577416719573*^9, 3.4735774835625353`*^9}, {3.473577773933062*^9, 
   3.473577775330233*^9}},
 AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["QuaternionToEuler"], "Subsection",
 PageWidth->WindowWidth,
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"QuaternionToEuler", "[", 
   RowBox[{"{", 
    RowBox[{"q1_", ",", "q2_", ",", "q3_", ",", "q4_"}], "}"}], "]"}], ":=", 
  RowBox[{"AxesToEuler", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"ex", ",", "ey", ",", "ez"}], "}"}], ",", 
    RowBox[{"RotatePolar", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"ex", ",", "ey", ",", "ez"}], "}"}], ",", 
      RowBox[{"QuaternionToPolar", "[", 
       RowBox[{"{", 
        RowBox[{"q1", ",", "q2", ",", "q3", ",", "q4"}], "}"}], "]"}]}], 
     "]"}]}], "]"}]}]], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.409832359012103*^9, 3.409832368336837*^9}, {
  3.409832405342527*^9, 3.4098324179346952`*^9}, {3.473578032951195*^9, 
  3.473578036117283*^9}},
 AspectRatioFixed->True]
}, Closed]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["end"], "Subsubtitle",
 PageWidth->WindowWidth,
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->True],

Cell[BoxData[{
 RowBox[{
  RowBox[{"End", "[", "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"EndPackage", "[", "]"}], ";"}]}], "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->True]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->Automatic,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{842, 911},
WindowMargins->{{14, Automatic}, {Automatic, 0}},
ShowCellLabel->True,
ShowCellTags->False,
FrontEndVersion->"11.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (July 28, \
2016)",
StyleDefinitions->"mhl-default.nb",
PrivateNotebookOptions -> {"ColorPalette" -> {RGBColor, 128}},
RenderingOptions -> {"ObjectDithering" -> True},
RenderingOptions -> {"RasterDithering" -> False},
CharacterEncoding -> "MacintoshAutomaticEncoding"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 164, 5, 92, "Title",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[734, 29, 707, 17, 91, "Input",
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[1466, 50, 165, 5, 34, "Subsubtitle",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[1656, 59, 166, 5, 32, "Section",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[1847, 68, 177, 5, 25, "Subsubsection",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[2027, 75, 741, 16, 61, "Input",
 InitializationCell->True,
 InitializationGroup->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[2805, 96, 177, 5, 25, "Subsubsection",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[2985, 103, 782, 17, 77, "Input",
 InitializationCell->True,
 InitializationGroup->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[3804, 125, 177, 5, 25, "Subsubsection",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[3984, 132, 799, 16, 77, "Input",
 InitializationCell->True,
 InitializationGroup->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[4820, 153, 177, 5, 25, "Subsubsection",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[5000, 160, 492, 12, 43, "Input",
 InitializationCell->True,
 InitializationGroup->True]
}, Open  ]]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[5553, 179, 162, 5, 34, "Subsubtitle",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[5718, 186, 197, 6, 48, "Input",
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[5940, 196, 166, 5, 32, "Section",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[6131, 205, 174, 5, 46, "Subsection",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[6308, 212, 1424, 38, 88, "Input",
 InitializationCell->True,
 InitializationGroup->True]
}, Closed]],
Cell[CellGroupData[{
Cell[7769, 255, 174, 5, 38, "Subsection",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[7946, 262, 266, 7, 27, "Input",
 InitializationCell->True,
 InitializationGroup->True],
Cell[8215, 271, 1879, 50, 118, "Input",
 InitializationCell->True,
 InitializationGroup->True],
Cell[10097, 323, 523, 14, 27, "Input",
 InitializationCell->True,
 InitializationGroup->True]
}, Closed]],
Cell[CellGroupData[{
Cell[10657, 342, 174, 5, 38, "Subsection",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[10834, 349, 2249, 57, 294, "Input",
 InitializationCell->True,
 InitializationGroup->True]
}, Closed]],
Cell[CellGroupData[{
Cell[13120, 411, 174, 5, 38, "Subsection",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[13297, 418, 837, 23, 43, "Input",
 InitializationCell->True,
 InitializationGroup->True]
}, Closed]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[14195, 448, 161, 5, 34, "Subsubtitle",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[14359, 455, 236, 8, 70, "Input",
 InitializationCell->True,
 InitializationGroup->True]
}, Open  ]]
}, Open  ]]
}
]
*)

