(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     42115,       1063]
NotebookOptionsPosition[     33228,        853]
NotebookOutlinePosition[     33900,        877]
CellTagsIndexPosition[     33857,        874]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["SpinDynamica`", "Title",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.4526855795508213`*^9, 3.452685580972721*^9}, {
  3.564596805625153*^9, 3.5645968086614637`*^9}},
 AspectRatioFixed->True,
 Background->RGBColor[
  1, 0.9, 0.8],ExpressionUUID->"bfba5351-15bb-46f5-8501-a7ef998f6700"],

Cell[BoxData[
 RowBox[{
  RowBox[{"BeginPackage", "[", 
   RowBox[{"\"\<SpinDynamica`\>\"", ",", 
    RowBox[{"{", 
     RowBox[{"\"\<Global`\>\"", ",", "\"\<SubKernels`\>\""}], "}"}]}], "]"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.564596993278902*^9, 3.564597038361494*^9}, {
   3.5645977451468143`*^9, 3.564597745332718*^9}, 3.564598046999627*^9, {
   3.564810443505661*^9, 3.5648104544496*^9}, {3.564811342898856*^9, 
   3.564811344327629*^9}, {3.570807605947206*^9, 3.570807629934265*^9}, {
   3.5748458070717173`*^9, 3.574845811338969*^9}, {3.574846126118534*^9, 
   3.574846128599468*^9}, {3.596883672895265*^9, 3.596883673181073*^9}, {
   3.6424250049846897`*^9, 3.642425005302615*^9}, {3.666352404368473*^9, 
   3.6663524048068223`*^9}, {3.666352492954552*^9, 
   3.6663524933719854`*^9}},ExpressionUUID->"0446ccae-7620-4702-9694-\
4a5afa6eb9b2"],

Cell[CellGroupData[{

Cell["parallel code explanation", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.680959148082035*^9, 
  3.680959159961363*^9}},ExpressionUUID->"b2dea3df-5cb9-4f12-835a-\
0cf0c965b14b"],

Cell["\<\
explanations:

1) In standard setup the subkernels do not read the init.m file and do
not distribute $Path to subkernels, because subkernels could run on
remote machines where packages are found in different locations.
Hence, normally the user needs to define the proper $Path for the
subkernels manually.

2) However, as soon as any kernel related command is executed, the
Package \[OpenCurlyDoubleQuote]SubKernels`\[CloseCurlyDoubleQuote] is loaded \
that provides a variable \[OpenCurlyDoubleQuote]stdargs\[CloseCurlyDoubleQuote]
that can be changed so that subkernels do read the init.m file.\
\>", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.570885909441362*^9, 3.570885921762833*^9}, 
   3.57088595830311*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["exports", "Subsubtitle",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.5645976645027018`*^9, 3.5645976646736937`*^9}},
 AspectRatioFixed->
  True,ExpressionUUID->"44db492e-b0a7-4f06-809c-38153c2d1b76"],

Cell[CellGroupData[{

Cell["Chronology and $Chronology", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.513610238724885*^9, 3.5136102409150677`*^9}, {
   3.5755559238100967`*^9, 3.575555930691103*^9}, {3.6424203587861853`*^9, 
   3.642420367216194*^9}, 3.6571115456380987`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Chronology", "::", "usage"}], "=", 
   "\"\<Chronology is an option for many routines in SpinDynamica, which \
controls the temporal order of events in all event sequences used by the \
routines. The value of Chronology must be \\\"LeftToRight\\\" or \
\\\"RightToLeft\\\". The default value is set by $Chronology.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5114262873763437`*^9, 3.5114263253503933`*^9}, {
   3.511429076592346*^9, 3.5114290995119877`*^9}, {3.513610254929747*^9, 
   3.513610333494897*^9}, {3.513610366936887*^9, 3.513610368414483*^9}, {
   3.5136131685919733`*^9, 3.5136132005967913`*^9}, 3.51361323812261*^9, {
   3.534398189837288*^9, 3.534398202089366*^9}, {3.545050244004449*^9, 
   3.545050294446129*^9}, {3.5645975412251673`*^9, 3.564597551886457*^9}, {
   3.657111496888939*^9, 3.657111497094153*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"$Chronology", "::", "usage"}], "=", 
   "\"\<$Chronology is the default setting for Chronology. The value of \
$Chronology must be \\\"LeftToRight\\\" or \\\"RightToLeft\\\".\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5114262873763437`*^9, 3.5114263253503933`*^9}, {
   3.511429076592346*^9, 3.5114290995119877`*^9}, {3.513610254929747*^9, 
   3.513610333494897*^9}, {3.513610366936887*^9, 3.513610368414483*^9}, {
   3.5136131685919733`*^9, 3.5136132005967913`*^9}, 3.51361323812261*^9, {
   3.534398189837288*^9, 3.534398202089366*^9}, {3.545050230308173*^9, 
   3.545050235976706*^9}, {3.5450503365921183`*^9, 3.545050362022235*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["$SpinDynamicaVersion", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.513610238724885*^9, 3.5136102409150677`*^9}, 
   3.564597860619755*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"$SpinDynamicaVersion", "::", "usage"}], "=", 
   "\"\<$SpinDynamicaVersion is the active version of SpinDynamica\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5114262873763437`*^9, 3.5114263253503933`*^9}, {
   3.511429076592346*^9, 3.5114290995119877`*^9}, {3.513610254929747*^9, 
   3.513610333494897*^9}, {3.513610366936887*^9, 3.513610368414483*^9}, {
   3.5136131685919733`*^9, 3.5136132005967913`*^9}, 3.51361323812261*^9, {
   3.534398189837288*^9, 3.534398202089366*^9}, {3.545050230308173*^9, 
   3.545050235976706*^9}, {3.5450503365921183`*^9, 3.545050362022235*^9}, {
   3.564597836498204*^9, 3.564597853599237*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["MinimumMathematicaVersionNumber", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.513610238724885*^9, 3.5136102409150677`*^9}, 
   3.564597860619755*^9, 3.564597949549645*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MinimumMathematicaVersionNumber", "::", "usage"}], "=", 
   "\"\<MinimumMathematicaVersionNumber is the minimum version of Mathematica \
which is compatible with the current version of SpinDynamica.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5114262873763437`*^9, 3.5114263253503933`*^9}, {
   3.511429076592346*^9, 3.5114290995119877`*^9}, {3.513610254929747*^9, 
   3.513610333494897*^9}, {3.513610366936887*^9, 3.513610368414483*^9}, {
   3.5136131685919733`*^9, 3.5136132005967913`*^9}, 3.51361323812261*^9, {
   3.534398189837288*^9, 3.534398202089366*^9}, {3.545050230308173*^9, 
   3.545050235976706*^9}, {3.5450503365921183`*^9, 3.545050362022235*^9}, {
   3.564597836498204*^9, 3.564597853599237*^9}, {3.5645979567771893`*^9, 
   3.564597983475747*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Sorted and Checked", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.513610238724885*^9, 3.5136102409150677`*^9}, {
   3.5755559238100967`*^9, 3.575555930691103*^9}, {3.642419818578085*^9, 
   3.642419824633174*^9}, {3.6424203386689043`*^9, 3.64242034641089*^9}, 
   3.6571115304159403`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Sorted", "::", "usage"}], "=", "\"\<\!\(\*
StyleBox[\"Sorted\",
FontSlant->\"Italic\"]\) is an option for various routines in \!\(\*
StyleBox[\"SpinDynamica\",
FontSlant->\"Italic\"]\), which indicates whether the arguments to a function \
have been placed in canonical order or not.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5114262873763437`*^9, 3.5114263253503933`*^9}, {
   3.511429076592346*^9, 3.5114290995119877`*^9}, {3.513610254929747*^9, 
   3.513610333494897*^9}, {3.513610366936887*^9, 3.513610368414483*^9}, {
   3.5136131685919733`*^9, 3.5136132005967913`*^9}, 3.51361323812261*^9, {
   3.534398189837288*^9, 3.534398202089366*^9}, {3.545050244004449*^9, 
   3.545050294446129*^9}, {3.5645975412251673`*^9, 3.564597551886457*^9}, {
   3.6424198278200483`*^9, 3.642419855411022*^9}, {3.6571115017767878`*^9, 
   3.657111510933713*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Checked", "::", "usage"}], "=", "\"\<\!\(\*
StyleBox[\"Checked\",
FontSlant->\"Italic\"]\) is an option for various routines in SpinDynamica, \
which indicates whether the arguments to a function have been checked for \
validity or not.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5114262873763437`*^9, 3.5114263253503933`*^9}, {
   3.511429076592346*^9, 3.5114290995119877`*^9}, {3.513610254929747*^9, 
   3.513610333494897*^9}, {3.513610366936887*^9, 3.513610368414483*^9}, {
   3.5136131685919733`*^9, 3.5136132005967913`*^9}, 3.51361323812261*^9, {
   3.534398189837288*^9, 3.534398202089366*^9}, {3.545050244004449*^9, 
   3.545050294446129*^9}, {3.5645975412251673`*^9, 3.564597551886457*^9}, {
   3.6424198278200483`*^9, 3.64241987588011*^9}, {3.6571115059353228`*^9, 
   3.657111515917378*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["SpinDynamicaSymbols", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.513610238724885*^9, 3.5136102409150677`*^9}, {
   3.5755559238100967`*^9, 3.575555930691103*^9}, {3.642419818578085*^9, 
   3.642419824633174*^9}, {3.6424203386689043`*^9, 3.64242034641089*^9}, 
   3.6571115304159403`*^9, {3.683132987889833*^9, 3.683132991577269*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SpinDynamicaSymbols", "::", "usage"}], "=", "\"\<\!\(\*
StyleBox[\"SpinDynamicaSymbols\",
FontSlant->\"Italic\"]\)\!\(\*
StyleBox[\"[\",
FontSlant->\"Italic\"]\)\!\(\*
StyleBox[\"]\",
FontSlant->\"Italic\"]\) generates a clickable list of all \!\(\*
StyleBox[\"SpinDynamica\",
FontSlant->\"Italic\"]\) symbols.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683132995459807*^9, 3.6831330202232103`*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["start", "Subsubtitle",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->
  True,ExpressionUUID->"84bd21d8-8de3-4979-ae35-e6d0737a8af3"],

Cell[BoxData[
 RowBox[{"SubKernels`Protected`stdargs", "=", 
  "\"\< -subkernel\>\""}]], "Input",
 InitializationCell->True],

Cell["Begin[\"`Private`\"];", "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["code", "Subsubtitle",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.5645978796420193`*^9, {3.564597919957129*^9, 3.564597920460723*^9}},
 AspectRatioFixed->
  True,ExpressionUUID->"d2eab483-156d-4dcc-b0ca-2e255fe90590"],

Cell[CellGroupData[{

Cell["load version numbers and check compatibility", "Section",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.5645978796420193`*^9, {3.5645979282359943`*^9, 3.564597932447899*^9}, {
   3.56459803018865*^9, 3.564598041225412*^9}},
 AspectRatioFixed->
  True,ExpressionUUID->"11a6a6f8-5489-47b7-ba8d-8c43353197cd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$SpinDynamicaVersion", ":=", 
   RowBox[{"First", "@", 
    RowBox[{"ReadList", "[", 
     RowBox[{"\"\<VersionInfo.txt\>\"", ",", "String"}], "]"}]}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.513146287209375*^9, 3.513146295198801*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MinimumMathematicaVersionNumber", "=", "8.0"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.549275912424939*^9, 3.549275938501244*^9}, 
   3.5492760659790487`*^9, 3.54927643324198*^9}],

Cell["Only give version messages when top Kernel is launched", "Text",
 InitializationCell->True,
 CellChangeTimes->{{3.570345311674345*^9, 3.570345352792267*^9}, 
   3.5703455776887827`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"$KernelID", "\[Equal]", "0"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Print", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"FrameBox", "@", 
        RowBox[{"ToBoxes", "[", 
         RowBox[{"$SpinDynamicaVersion", "<>", "\"\< loaded\>\""}], "]"}]}], "//",
        "DisplayForm"}], "\[IndentingNewLine]", "]"}], ";"}]}], 
   "\[IndentingNewLine]", "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.409326743611483*^9, 3.5131463075765533`*^9, {3.5257581828378057`*^9, 
   3.52575818742441*^9}, 3.525758233236266*^9, 3.562215020816587*^9, 
   3.56221545634509*^9, 3.5645980227509203`*^9, {3.570345281665061*^9, 
   3.570345307085703*^9}, {3.570345534805572*^9, 3.570345568090646*^9}, {
   3.5703457673455048`*^9, 3.570345789686943*^9}, {3.570806063914793*^9, 
   3.5708060640027103`*^9}, {3.570806624066278*^9, 3.570806644486931*^9}, {
   3.5708067312557163`*^9, 3.5708067680869713`*^9}, {3.5708839591299562`*^9, 
   3.570883969542485*^9}, 3.570884065656498*^9, {3.680282572261313*^9, 
   3.680282690207348*^9}, 3.6802833414408484`*^9, {3.6802833741552887`*^9, 
   3.68028339386687*^9}, {3.68028390613767*^9, 3.6802839080541353`*^9}, {
   3.680284088939148*^9, 3.680284111302635*^9}, {3.6802847972335978`*^9, 
   3.6802848516940193`*^9}, {3.680285629147375*^9, 3.6802856301691847`*^9}, {
   3.680327519737262*^9, 3.68032753668639*^9}, {3.680327680807661*^9, 
   3.680327688876852*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"$KernelID", "\[Equal]", "0"}], ")"}], "&&", 
     "\[IndentingNewLine]", 
     RowBox[{"(", 
      RowBox[{"$VersionNumber", "<", "MinimumMathematicaVersionNumber"}], 
      ")"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"(", 
     RowBox[{"StylePrint", "[", "\[IndentingNewLine]", 
      RowBox[{"DisplayForm", "@", 
       RowBox[{"ButtonBox", "[", "\[IndentingNewLine]", 
        RowBox[{
        "\"\<Warning: This version of SpinDynamica requires Mathematica \
version \>\"", "<>", 
         RowBox[{"ToString", "[", "MinimumMathematicaVersionNumber", "]"}], 
         "<>", "\"\< or higher. Problems may be encountered.\>\""}], 
        "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}], 
     "\[IndentingNewLine]", ")"}]}], "\[IndentingNewLine]", "]"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.549275843185669*^9, 3.549275892559023*^9}, {
   3.549275949642577*^9, 3.5492760536498957`*^9}, {3.549276142217573*^9, 
   3.549276169649551*^9}, {3.549276327487298*^9, 3.5492763620855837`*^9}, {
   3.5492763985530033`*^9, 3.549276410137793*^9}, 3.564598025224148*^9, {
   3.5703453696500463`*^9, 3.5703453833569117`*^9}, {3.570806066618204*^9, 
   3.570806066681786*^9}, 3.570806656975739*^9, {3.5708067761924562`*^9, 
   3.570806780455459*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["load PhysicalConstants package if less than version 9", "Section",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.5645978796420193`*^9, {3.5645979282359943`*^9, 3.564597932447899*^9}, {
   3.56459803018865*^9, 3.564598041225412*^9}, {3.5968836069169693`*^9, 
   3.596883614219503*^9}, {3.5968836892934513`*^9, 3.5968836916517887`*^9}},
 AspectRatioFixed->
  True,ExpressionUUID->"e700934f-3ed4-4348-be08-33d885b5e0e8"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"$KernelID", "\[Equal]", "0"}], ")"}], "&&", 
     "\[IndentingNewLine]", 
     RowBox[{"(", 
      RowBox[{"$VersionNumber", "<", "9"}], ")"}]}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"Needs", "[", "\"\<PhysicalConstants`\>\"", "]"}]}], 
   "\[IndentingNewLine]", "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.513146287209375*^9, 3.513146295198801*^9}, {
  3.596883628919396*^9, 3.5968836642292643`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["SpinDynamicaSymbols", "Section",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.5645978796420193`*^9, {3.5645979282359943`*^9, 3.564597932447899*^9}, {
   3.56459803018865*^9, 3.564598041225412*^9}, {3.5968836069169693`*^9, 
   3.596883614219503*^9}, {3.5968836892934513`*^9, 3.5968836916517887`*^9}, {
   3.683133058972336*^9, 3.683133061629549*^9}},
 AspectRatioFixed->
  True,ExpressionUUID->"7a23df56-b9f8-4586-861c-83c04541b8ed"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinDynamicaSymbols", "[", "opts___Rule", "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Information", "[", "\"\<SpinDynamica`*\>\"", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<SD`Propagation`*\>\"", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<SD`Approximations`*\>\"", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<Signals`Signal`*\>\"", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<QM`Hilbert`*\>\"", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<QM`Liouville`*\>\"", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<QM`SpinOperators`*\>\"", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<Graphics`Visualization`*\>\"", "]"}], 
    ";", 
    RowBox[{"Information", "[", "\"\<General`Matrices`*\>\"", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<Rotations`Euler`*\>\"", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<Rotations`Wigner`*\>\"", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{
    "Information", "[", "\"\<Ensemble`OrientationalSampling`*\>\"", "]"}], 
    ";", "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<Interactions`Interactions`*\>\"", "]"}], 
    ";", "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<SD`Generators`*\>\"", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<Events`Events`*\>\"", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<General`PhysicalConstants`*\>\"", "]"}], 
    ";", "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<General`ModifyBuiltIn`*\>\"", "]"}], ";",
     "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<General`Miscellaneous`*\>\"", "]"}], ";",
     "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<Rotations`Polar`*\>\"", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<Rotations`Quaternion`*\>\"", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Information", "[", "\"\<Signals`Lineshapes`*\>\"", "]"}], ";"}], 
   "\[IndentingNewLine]", ")"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.683133070023656*^9, 3.68313331109302*^9}, {
   3.683133372201885*^9, 3.6831334858262243`*^9}, {3.683133546960778*^9, 
   3.6831335826558723`*^9}, {3.683133627974943*^9, 3.683133631124217*^9}, {
   3.683134084248189*^9, 3.683134138116818*^9}, 3.683134356508*^9, 
   3.683134830874592*^9, {3.683191961511545*^9, 3.683191967852162*^9}, {
   3.691410723922687*^9, 3.691410724125162*^9}, {3.7185335471053977`*^9, 
   3.718533556659727*^9}, {3.7185337050769987`*^9, 3.7185337442286243`*^9}, {
   3.7185337756585245`*^9, 3.718533785079561*^9}, {3.718533829212143*^9, 
   3.7185338634940124`*^9}},ExpressionUUID->"57a6db79-31fe-4f99-a46d-\
b2cf8a61258f"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["end", "Subsubtitle",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 AspectRatioFixed->
  True,ExpressionUUID->"a9cad4bf-a94a-4f8b-8b4c-16dcfb729256"],

Cell["\<\
End[];
EndPackage[];\
\>", "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.51238794480797*^9, 3.512387952205147*^9}, 
   3.513231730353488*^9},
 AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["SpinDynamica",
  FontSlant->"Italic"],
 " package declarations"
}], "Subsubtitle",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.5645976645027018`*^9, 3.5645976646736937`*^9}, {
   3.5648104711436033`*^9, 3.5648104771020412`*^9}, {3.564810754570716*^9, 
   3.56481075807093*^9}, 3.6819870758637857`*^9},
 AspectRatioFixed->
  True,ExpressionUUID->"772565f3-37e0-4341-b262-41d78aca0ade"],

Cell[CellGroupData[{

Cell["Ensemble`Orientations`", "Section",
 InitializationCell->True,
 CellChangeTimes->{
  3.5648105210875874`*^9, {3.691678525194685*^9, 3.691678528272519*^9}, 
   3.6916785746229897`*^9},ExpressionUUID->"e754c78f-db78-44fc-8f76-\
569da9e16df9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"DeclarePackage", "[", "\[IndentingNewLine]", 
   RowBox[{
   "\"\<Ensemble`OrientationalSampling`\>\"", ",", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<OrientationalSamplingScheme\>\"", ",", "\[IndentingNewLine]", 
      "\"\<DefineOrientationalSamplingScheme\>\"", ",", 
      "\[IndentingNewLine]", "\"\<Orientations\>\"", ",", 
      "\[IndentingNewLine]", "\"\<OrientationsAndWeights\>\"", ",", 
      "\[IndentingNewLine]", "\"\<OrientationWeights\>\""}], 
     "\[IndentingNewLine]", "}"}]}], "\[IndentingNewLine]", "]"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.564810497049201*^9, 3.5648105938046494`*^9}, 
   3.691678534701375*^9, 3.69167857208888*^9, {3.7038385105849967`*^9, 
   3.7038385133255687`*^9}},ExpressionUUID->"3fabd052-3956-45a0-9435-\
74681bf57955"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "load ",
 StyleBox["SpinDynamica",
  FontSlant->"Italic"],
 " packages & set defaults"
}], "Subsubtitle",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.605545641519261*^9, {3.642424883993432*^9, 3.642424892001383*^9}, {
   3.64242495670002*^9, 3.642424957234228*^9}, {3.683119716225142*^9, 
   3.683119730216501*^9}},
 AspectRatioFixed->True,
 Background->RGBColor[
  1, 0.9, 0.8],ExpressionUUID->"8c5ad6a2-1925-41c2-abb7-a5c77e9b5fdd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Needs", "/@", " ", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
    "\"\<General`UserLevel`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<QM`Hilbert`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<QM`SpinOperators`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<QM`Liouville`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<SD`Generators`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<SD`Approximations`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<SD`Propagation`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<Events`Events`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<Events`PulseSequence`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<Interactions`Interactions`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<Interactions`Hamiltonians`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<Interactions`Relaxation`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<Simulations`Polarization`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<Simulations`Simulations`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<Signals`Signal`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<Signals`Lineshapes`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<Graphics`Visualization`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<Rotations`Polar`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<Rotations`Quaternion`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<Tensors`CG`\>\"", ",", "\[IndentingNewLine]", 
     "\"\<Tensors`IST`\>\""}], "\[IndentingNewLine]", "}"}]}], ";"}]], "Input",\

 InitializationCell->True,
 CellChangeTimes->{{3.6424249661544323`*^9, 3.642425012898428*^9}, {
   3.681042323475066*^9, 3.681042344970253*^9}, {3.6811024359655848`*^9, 
   3.6811024417225313`*^9}, {3.682593239615035*^9, 3.682593251182222*^9}, {
   3.682593763353941*^9, 3.68259382343357*^9}, {3.6825938546011257`*^9, 
   3.682593900952001*^9}, {3.682595304813249*^9, 3.682595310424222*^9}, {
   3.682595633723222*^9, 3.6825956351019773`*^9}, {3.682596845949067*^9, 
   3.682596854837201*^9}, {3.682598375212502*^9, 3.682598376210919*^9}, {
   3.682599626325069*^9, 3.6825996308433037`*^9}, {3.682612832904244*^9, 
   3.682612840673251*^9}, {3.683108941850814*^9, 3.6831089853845873`*^9}, 
   3.683110272867401*^9, {3.683118461208426*^9, 3.683118506838763*^9}, {
   3.683127105045773*^9, 3.6831271081565657`*^9}, {3.68313364519655*^9, 
   3.683133666361293*^9}, {3.683134027937656*^9, 3.683134031957807*^9}, {
   3.6831347818771353`*^9, 3.683134791273467*^9}, {3.683214935488085*^9, 
   3.683214937166902*^9}, {3.691410744829871*^9, 3.691410745011601*^9}, {
   3.691678276253324*^9, 3.691678293115243*^9}, {3.691833163894046*^9, 
   3.6918331699663677`*^9}, {3.692255794796267*^9, 3.69225579748146*^9}, {
   3.692256032016363*^9, 3.692256101823667*^9}, {3.6922561802771606`*^9, 
   3.692256180564754*^9}, {3.69486422668926*^9, 3.694864241053936*^9}, 
   3.701151299544896*^9, {3.7453911031852283`*^9, 3.7453911297356*^9}, 
   3.7453963592289*^9},ExpressionUUID->"84057188-abbf-4436-b22f-b807162f0305"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Chronology", "=", "\"\<LeftToRight\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.545050393596538*^9, 3.5450503995208893`*^9}, {
  3.545050732985859*^9, 
  3.545050735366548*^9}},ExpressionUUID->"8c5fe231-201a-49f7-bca8-\
adc6f71f694b"],

Cell["$RecursionLimit=1000;", "Input",
 PageWidth->WindowWidth,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.51238794480797*^9, 3.512387952205147*^9}, {
  3.513231730353488*^9, 3.513231750805922*^9}},
 AspectRatioFixed->
  True,ExpressionUUID->"6cefb556-7f4d-42c1-9464-ddbae2dba5bf"]
}, Closed]],

Cell[CellGroupData[{

Cell["set initial user level", "Subsubtitle",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{
  3.605545641519261*^9, {3.642424883993432*^9, 3.642424892001383*^9}, {
   3.64242495670002*^9, 3.642424957234228*^9}, {3.683119739784093*^9, 
   3.683119746791615*^9}, {3.6914107679000807`*^9, 3.691410768346019*^9}, {
   3.745392032842732*^9, 3.745392042549773*^9}, {3.745401467786045*^9, 
   3.745401472092243*^9}},
 AspectRatioFixed->True,
 Background->RGBColor[
  1, 0.9, 0.8],ExpressionUUID->"2f7a2376-445c-48e4-ad42-d1f2444fe203"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$KernelID", "\[Equal]", "0"}], ",", 
    RowBox[{"Message", "[", 
     RowBox[{
      RowBox[{"SetUserLevel", "::", "ini"}], ",", "2", ",", 
      "MaximumUserLevel"}], "]"}]}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SetUserLevel", "[", "2", "]"}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.7453911352990837`*^9, 3.745391140188941*^9}, {
   3.745392013560307*^9, 3.745392019199547*^9}, 3.74540146514354*^9, {
   3.7454832938101797`*^9, 3.745483294345168*^9}, {3.745483366794368*^9, 
   3.7454833779929657`*^9}, {3.7454834487428923`*^9, 3.74548345844473*^9}, {
   3.745484130428938*^9, 3.745484130868285*^9}, {3.7455099803891907`*^9, 
   3.745509986605363*^9}},ExpressionUUID->"21c0930b-5a5b-4017-8071-\
7b00b7810a45"]
}, Open  ]],

Cell[CellGroupData[{

Cell["global settings", "Subsubtitle",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True,
 CellChangeTimes->{{3.5645976645027018`*^9, 3.5645976646736937`*^9}, {
  3.5648104711436033`*^9, 3.5648104771020412`*^9}, {3.564810754570716*^9, 
  3.564810766831402*^9}},
 AspectRatioFixed->
  True,ExpressionUUID->"7f2956d9-9e0d-4079-abd4-bdde2e424e27"],

Cell[CellGroupData[{

Cell["Plot", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.564810858303357*^9, 3.564810858957135*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Plot", ",", " ", 
    RowBox[{"LabelStyle", "\[Rule]", 
     RowBox[{"Directive", "[", 
      RowBox[{
       RowBox[{"FontFamily", "\[Rule]", "\"\<Helvetica\>\""}], ",", 
       RowBox[{"FontSize", "\[Rule]", "14"}]}], "]"}]}], ",", 
    RowBox[{"Frame", "\[Rule]", "True"}], ",", 
    RowBox[{"Axes", "\[Rule]", "None"}]}], "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5333026111723824`*^9, 3.533302635260756*^9}, {
  3.564810880873742*^9, 3.564810883889326*^9}, {3.56615485409627*^9, 
  3.566154854408133*^9}, {3.681986872813925*^9, 3.68198687577988*^9}, {
  3.6819934175867233`*^9, 3.681993428817223*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["ListPlot", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.564810858303357*^9, 3.5648108775421762`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SetOptions", "[", 
    RowBox[{"ListPlot", ",", " ", 
     RowBox[{"LabelStyle", "\[Rule]", 
      RowBox[{"Directive", "[", 
       RowBox[{
        RowBox[{"FontFamily", "\[Rule]", "\"\<Helvetica\>\""}], ",", 
        RowBox[{"FontSize", "\[Rule]", "14"}]}], "]"}]}], ",", 
     RowBox[{"Frame", "\[Rule]", "True"}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
     RowBox[{"Axes", "\[Rule]", "None"}]}], "]"}], ";"}], " "}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5333026111723824`*^9, 3.533302635260756*^9}, {
  3.564810887204104*^9, 3.564810906802703*^9}, {3.56615484653797*^9, 
  3.566154849127849*^9}, {3.681986973026626*^9, 3.681986975672236*^9}, {
  3.681993432313157*^9, 3.6819934379943743`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["ParametricPlot3D", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.564810858303357*^9, 3.5648108775421762`*^9}, 
   3.564811085000263*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SetOptions", "[", 
    RowBox[{"ParametricPlot3D", ",", " ", 
     RowBox[{"LabelStyle", "\[Rule]", 
      RowBox[{"Directive", "[", 
       RowBox[{
        RowBox[{"FontFamily", "\[Rule]", "\"\<Helvetica\>\""}], ",", 
        RowBox[{"FontSize", "\[Rule]", "14"}]}], "]"}]}]}], "]"}], ";"}], 
  " "}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5333026111723824`*^9, 3.533302635260756*^9}, {
  3.564810887204104*^9, 3.564810906802703*^9}, {3.564811086870064*^9, 
  3.56481109665447*^9}, {3.681986979778263*^9, 3.681986982912253*^9}, {
  3.681993441128584*^9, 3.6819934461286716`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Graphics", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.564810858303357*^9, 3.5648108716857567`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SetOptions", "[", 
    RowBox[{"Graphics", ",", " ", 
     RowBox[{"LabelStyle", "\[Rule]", 
      RowBox[{"Directive", "[", 
       RowBox[{
        RowBox[{"FontFamily", "\[Rule]", "\"\<Helvetica\>\""}], ",", 
        RowBox[{"FontSize", "\[Rule]", "14"}]}], "]"}]}]}], "]"}], ";"}], 
  " "}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.533302642990561*^9, 3.533302644236795*^9}, {
  3.6819869648825483`*^9, 3.681986968223421*^9}, {3.6819934485043364`*^9, 
  3.681993453393477*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Graphics3D", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.564810858303357*^9, 3.5648108716857567`*^9}, 
   3.564811072083*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Graphics3D", ",", " ", 
    RowBox[{"LabelStyle", "\[Rule]", 
     RowBox[{"Directive", "[", 
      RowBox[{
       RowBox[{"FontFamily", "\[Rule]", "\"\<Helvetica\>\""}], ",", 
       RowBox[{"FontSize", "\[Rule]", "14"}]}], "]"}]}]}], "]"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.533302642990561*^9, 3.533302644236795*^9}, {
  3.564811052212891*^9, 3.564811052571498*^9}, {3.681986988377512*^9, 
  3.681986991527479*^9}, {3.681993455679913*^9, 3.6819934612084093`*^9}}]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["history", "Title",
 PageWidth->WindowWidth,
 Evaluatable->False,
 CellChangeTimes->{
  3.682593334636269*^9, {3.69926171587668*^9, 3.69926172886936*^9}, {
   3.69926187929764*^9, 3.6992618809832497`*^9}},
 AspectRatioFixed->
  True,ExpressionUUID->"1f3f3a6b-c0dd-4a55-bd2f-ce62fd19fd57"],

Cell[CellGroupData[{

Cell["170515 changes incorporated into v3.0.0b15", "Subsection",
 CellChangeTimes->{{3.6992617377127457`*^9, 3.699261750424902*^9}, 
   3.699261872727805*^9, {3.699261961052656*^9, 3.699261967080655*^9}, {
   3.699267263610918*^9, 3.699267264970158*^9}, 3.6995424563123937`*^9, {
   3.699542530452357*^9, 3.6995425326806383`*^9}, {3.699545410099059*^9, 
   3.6995454103978543`*^9}, {3.699684558842494*^9, 3.699684559329362*^9}, {
   3.703838582991973*^9, 3.703838598864158*^9}},
 ExpressionUUID -> "82f0a69c-884f-4599-9470-fddb7c7b370a"],

Cell["\<\
Adjusted package name:

Ensemble`Orientations` \[Rule] Ensemble`OrientationalSampling` 

\
\>", "Text",
 CellChangeTimes->{{3.6992793268113956`*^9, 3.699279350177552*^9}, {
  3.6995424688031*^9, 3.6995424844515934`*^9}, {3.6995425465206327`*^9, 
  3.699542682270033*^9}, {3.699543154054568*^9, 3.699543189960311*^9}, {
  3.7038386073349867`*^9, 
  3.7038386298962917`*^9}},ExpressionUUID->"9cad1038-0834-42b9-abc4-\
8103802f881d"]
}, Open  ]]
}, Closed]]
},
AutoGeneratedPackage->Automatic,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{808, 751},
WindowMargins->{{Automatic, 12}, {Automatic, 0}},
ShowCellLabel->True,
FrontEndVersion->"11.3 for Mac OS X x86 (32-bit, 64-bit Kernel) (March 5, \
2018)",
StyleDefinitions->"MHL-StyleSheet.nb",
PrivateNotebookOptions -> {"ColorPalette" -> {RGBColor, 128}},
RenderingOptions -> {"ObjectDithering" -> True},
RenderingOptions -> {"RasterDithering" -> False},
CharacterEncoding -> "MacintoshAutomaticEncoding"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 344, 8, 114, "Title",ExpressionUUID->"bfba5351-15bb-46f5-8501-a7ef998f6700",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[914, 32, 885, 17, 46, "Input",ExpressionUUID->"0446ccae-7620-4702-9694-4a5afa6eb9b2",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[1824, 53, 197, 4, 36, "Section",ExpressionUUID->"b2dea3df-5cb9-4f12-835a-0cf0c965b14b",
 InitializationCell->True],
Cell[2024, 59, 738, 16, 220, "Text",ExpressionUUID->"af4148d1-c63a-4d2d-b0d0-f24b5f7fc90d",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[2799, 80, 259, 6, 41, "Subsubtitle",ExpressionUUID->"44db492e-b0a7-4f06-809c-38153c2d1b76",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[3083, 90, 268, 4, 34, "Section",ExpressionUUID->"e8372736-5e73-473e-8117-65298176ed41",
 InitializationCell->True],
Cell[3354, 96, 907, 16, 175, "Input",ExpressionUUID->"ed394047-f643-4de3-a102-04a89011aec4",
 InitializationCell->True],
Cell[4264, 114, 717, 13, 112, "Input",ExpressionUUID->"650cf367-b571-4c6e-8c7a-19579685da11",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[5018, 132, 160, 3, 34, "Section",ExpressionUUID->"f6378496-319d-4bb2-8bf0-530ec9a3c9b2",
 InitializationCell->True],
Cell[5181, 137, 705, 13, 31, "Input",ExpressionUUID->"39c2d9bb-e18e-464d-860e-f6b63680a288",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[5923, 155, 193, 3, 34, "Section",ExpressionUUID->"e8d4d0bd-8c11-4932-82c0-167a83511327",
 InitializationCell->True],
Cell[6119, 160, 843, 15, 63, "Input",ExpressionUUID->"6a783583-d015-43f2-9f72-473906a4d2e5",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[6999, 180, 309, 5, 34, "Section",ExpressionUUID->"0cc9513d-9286-4041-84cb-1519adf6c044",
 InitializationCell->True],
Cell[7311, 187, 927, 17, 112, "Input",ExpressionUUID->"09d89e16-d53f-4c62-a66b-e358a9bf1dc1",
 InitializationCell->True],
Cell[8241, 206, 879, 16, 112, "Input",ExpressionUUID->"152573a3-3a44-4a69-b2be-c9769c153c09",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[9157, 227, 356, 5, 34, "Section",ExpressionUUID->"3ffb0b2b-138a-4382-8ce8-67e8e5933902",
 InitializationCell->True],
Cell[9516, 234, 477, 13, 70, "Input",ExpressionUUID->"326eb151-1440-47a5-a527-f7da7f162fff",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[10042, 253, 187, 5, 41, "Subsubtitle",ExpressionUUID->"84bd21d8-8de3-4979-ae35-e6d0737a8af3",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[10232, 260, 124, 3, 48, "Input",ExpressionUUID->"ff1ab5e8-e2dd-4b0b-b14b-3e6be3e5e90a",
 InitializationCell->True],
Cell[10359, 265, 143, 4, 48, "Input",ExpressionUUID->"2de4f0a7-f174-4e52-b175-e42bdf206ca6",
 InitializationCell->True,
 InitializationGroup->True]
}, Closed]],
Cell[CellGroupData[{
Cell[10539, 274, 279, 7, 41, "Subsubtitle",ExpressionUUID->"d2eab483-156d-4dcc-b0ca-2e255fe90590",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[10843, 285, 366, 8, 36, "Section",ExpressionUUID->"11a6a6f8-5489-47b7-ba8d-8c43353197cd",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[11212, 295, 301, 8, 70, "Input",ExpressionUUID->"b41de17b-356f-4134-9d44-180506ac0053",
 InitializationCell->True],
Cell[11516, 305, 241, 5, 48, "Input",ExpressionUUID->"e6add847-b7c4-4db7-a989-79f30edb9362",
 InitializationCell->True],
Cell[11760, 312, 191, 3, 30, "Text",ExpressionUUID->"5d517e96-4f36-4bf6-a044-b3ce5b956532",
 InitializationCell->True],
Cell[11954, 317, 1533, 28, 175, "Input",ExpressionUUID->"0b2d29e6-81bd-428e-b3da-8258f5c891f1",
 InitializationCell->True],
Cell[13490, 347, 1418, 30, 301, "Input",ExpressionUUID->"e09989e1-25ed-44a7-9c4f-102f25f8f86f",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[14945, 382, 477, 9, 36, "Section",ExpressionUUID->"e700934f-3ed4-4348-be08-33d885b5e0e8",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[15425, 393, 567, 15, 133, "Input",ExpressionUUID->"3098e9d7-4569-4491-8660-e6af2aa23ebb",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[16029, 413, 493, 10, 36, "Section",ExpressionUUID->"7a23df56-b9f8-4586-861c-83c04541b8ed",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[16525, 425, 3030, 58, 509, "Input",ExpressionUUID->"57a6db79-31fe-4f99-a46d-b2cf8a61258f",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[19604, 489, 185, 5, 41, "Subsubtitle",ExpressionUUID->"a9cad4bf-a94a-4f8b-8b4c-16dcfb729256",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[19792, 496, 241, 9, 66, "Input",ExpressionUUID->"6423d8f8-261c-443a-95ea-565a611fb13d",
 InitializationCell->True,
 InitializationGroup->True]
}, Closed]],
Cell[CellGroupData[{
Cell[20070, 510, 465, 12, 41, "Subsubtitle",ExpressionUUID->"772565f3-37e0-4341-b262-41d78aca0ade",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[20560, 526, 246, 5, 36, "Section",ExpressionUUID->"e754c78f-db78-44fc-8f76-569da9e16df9",
 InitializationCell->True],
Cell[20809, 533, 859, 18, 215, "Input",ExpressionUUID->"3fabd052-3956-45a0-9435-74681bf57955",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[21717, 557, 507, 15, 57, "Subsubtitle",ExpressionUUID->"8c5ad6a2-1925-41c2-abb7-a5c77e9b5fdd",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[22227, 574, 2981, 47, 509, "Input",ExpressionUUID->"84057188-abbf-4436-b22f-b807162f0305",
 InitializationCell->True],
Cell[25211, 623, 299, 7, 46, "Input",ExpressionUUID->"8c5fe231-201a-49f7-bca8-adc6f71f694b",
 InitializationCell->True],
Cell[25513, 632, 315, 7, 46, "Input",ExpressionUUID->"6cefb556-7f4d-42c1-9464-ddbae2dba5bf",
 InitializationCell->True,
 InitializationGroup->True]
}, Closed]],
Cell[CellGroupData[{
Cell[25865, 644, 578, 12, 57, "Subsubtitle",ExpressionUUID->"2f7a2376-445c-48e4-ad42-d1f2444fe203",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[26446, 658, 834, 18, 68, "Input",ExpressionUUID->"21c0930b-5a5b-4017-8071-7b00b7810a45",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[27317, 681, 369, 8, 41, "Subsubtitle",ExpressionUUID->"7f2956d9-9e0d-4079-abd4-bdde2e424e27",
 Evaluatable->False,
 InitializationCell->True,
 InitializationGroup->True],
Cell[CellGroupData[{
Cell[27711, 693, 116, 2, 36, "Section",ExpressionUUID->"a40e6bde-00e4-4add-bdeb-7b8b8fc15682",
 InitializationCell->True],
Cell[27830, 697, 703, 15, 75, "Input",ExpressionUUID->"c1e06872-60b2-4b6b-b152-1900fc7b8724",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[28570, 717, 122, 2, 36, "Section",ExpressionUUID->"9906c046-c98e-4b5e-bb99-4f9eb9c34b27",
 InitializationCell->True],
Cell[28695, 721, 837, 18, 70, "Input",ExpressionUUID->"8bf074d1-588b-4ee6-9797-81086343ce84",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[29569, 744, 156, 3, 36, "Section",ExpressionUUID->"b3f7260d-bec3-4f72-9548-337dcacd4174",
 InitializationCell->True],
Cell[29728, 749, 649, 15, 70, "Input",ExpressionUUID->"cf91993c-7e4e-4a9c-aeea-4bfb7d10f62b",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[30414, 769, 122, 2, 36, "Section",ExpressionUUID->"bc217198-ee15-452b-9e81-223e7c0ab75e",
 InitializationCell->True],
Cell[30539, 773, 547, 14, 48, "Input",ExpressionUUID->"c1d064b4-4c2d-4fc3-93c8-30e2e5b42201",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[31123, 792, 147, 3, 36, "Section",ExpressionUUID->"bda324ea-61b3-494a-b340-95ed580f871d",
 InitializationCell->True],
Cell[31273, 797, 568, 13, 48, "Input",ExpressionUUID->"d6980e48-ef42-4e9d-84f6-c215cf164edd",
 InitializationCell->True]
}, Closed]]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[31902, 817, 293, 7, 98, "Title",ExpressionUUID->"1f3f3a6b-c0dd-4a55-bd2f-ce62fd19fd57",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[32220, 828, 537, 7, 54, "Subsection",ExpressionUUID->"82f0a69c-884f-4599-9470-fddb7c7b370a"],
Cell[32760, 837, 440, 12, 127, "Text",ExpressionUUID->"9cad1038-0834-42b9-abc4-8103802f881d"]
}, Open  ]]
}, Closed]]
}
]
*)

